#ifndef HEARTBEATPARMS_HXX
#define HEARTBEATPARMS_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const HeartbeatParmsVersion = "$Id$";


#include "Sptr.hxx"
#include "Data.hxx"


/// Prototype for the Provisioning Server Callback
void updateHeartbeatParms( const string& data, const string& fileName, bool );


/**
 *  This class defines an object containing global heartbeat parameters   
 */

class HeartbeatParms
{
    public:
        /**
	 * Returns the only heartbeatParms or create it if it doesn't already exist.
	 * @return Reference of the singleton
	 */
        static HeartbeatParms& instance();
        /// Destructor
        virtual ~HeartbeatParms();
        /**
	 * Get Multicast Address
	 * @param  address  the global multicast ip/port used by the system.
	 */
        void getMulticastAddr( Data & address);
        /**
	 * Get Multicast Host
	 * @param  host  the global multicast ip used by the system.
	 */
        void getMulticastHost( Data & host);
        /**
	 * Get Multicast Port
	 * @return  the global multicast port used by the system.
	 */
        int getMulticastPort() const;
        /**
	 * Get Maximum Missed Heartbeats
	 * @return  the global max missed heartbearts used by the system.
	 */
        int getMaxMissedHeartbeats() const;
        /**
	 * Get Heartbeat Interval
	 * @return  the global heartbeat interval used by the system.
	 */
        int getHeartbeatInterval() const;
        /**
	 * Set Multicast Port
	 * @param  multicastPort  the multicast port to be used by the system.
	 */
        void setMulticastPort( const int multicastPort);
        /**
	 * Set Maximum Missed Heartbeats
	 * @param  maxMissedHeartbeats  the maximum missed heartbeats tolerated before a
	 *                              server is considered dead.
	 */
        void setMaxMissedHeartbeats( const int maxMissedHeartbeats);
        /**
	 * Set Heartbeat Interval
	 * @param  heartbeatInterval  the heartbeat interval used to schedule heartbeat
	 *                            transmission and to schedule housekeeping.
	 */
        void setHeartbeatInterval( const int heartbeatInterval);
        /**
	 * Display Heartbeat Parameters stored in this object.
	 */
        void displayHeartbeatParms();

protected:
        /// Constructor
        HeartbeatParms();
        /// This is the single instance of HeartbeatParms
        static HeartbeatParms* theHeartbeatParms;

private:
        /// its Multicast Address
        Data itsMulticastAddr;
        /// its multicast host
        Data itsMulticastHost;
        /// its multicast port
        int itsMulticastPort;
        /// its max# of missed heartbeats allowed
        int itsMaxMissedHeartbeats;
        /// its heartbeat inteval in milliseconds
        int itsHeartbeatInterval;

        /// Copy Constructor
        HeartbeatParms(const HeartbeatParms& rhs);
        /// Overload operator assignment =
        HeartbeatParms& operator=(const HeartbeatParms& rhs);

};

#endif  // HEARTBEATPARMS_HXX
