#ifndef HEARTBEATTXTHREAD_HXX
#define HEARTBEATTXTHREAD_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const HeartbeatTxThread_hxx_Version = "$Id$";


#include "Sptr.hxx"
#include "ThreadIf.hxx"
#include "UdpStack.hxx"


/**
 *  HeartbeatTxThread is responsible for sending a regular hearbeat to
 *  a multicast address. Note that the responsibility of the heartbeat
 *  is to provide a hardware reliability check for the monitor. It is not
 *  designed as a software reliability check since whatever inputs that 
 *  crashed the first program will probably just crash the backup server as
 *  well so there is not much point in trying to resolve software bugs 
 *  via heartbeat.
 */

class HeartbeatTxThread: public ThreadIf
{
public:
        /**
	 * Constructor - Create a heartbeat transmitter thread.
         * The heartbeat is sent via the udp stack. 
         * @param port the local port identifier.
         */
        HeartbeatTxThread( const int port );
        /**
	 * Constructor
	 * @param  port  The local port
	 * @param  delayIntervalInMs  Delay in milli-seconds between sending heartbeats
	 * @param  multicastIP  Address to send heartbeats to
	 * @param  multicastPort  Port on which to transmit heartbeats
	 */
        HeartbeatTxThread( const int port,
                           int delayIntervalInMs,
                           const char * multicastIP,
                           u_int16_t multicastPort );

        /// Destroy the heartbeat thread. It will attempt to shutdown the thread.
        virtual ~HeartbeatTxThread();

        /// get Thread priority
        int getThreadPriority() const
        {
            return ThreadIf::itsThread.getPriority();
        }
        /**
	 * Obtain the transmit UDP stack
	 * @return  The local UDP stack for transmitting
	 */
        Sptr < UdpStack > getTxUdpStack() const;

protected:
        /// The new thread executes this method.
        virtual void thread();

private:
        /// The multicast group we are sending the heatbeat to.
        NetworkAddress multicastGroupAddr;

        /// The delay interval.
        int delayInMillisec;

        /// The transmitter.
        Sptr < UdpStack > itsTxUdpStack;

        /// The local port identifier.
        u_int16_t itsPort;
};

#endif // HEARTBEATTXTHREAD_HXX
