#ifndef HEARTBEAT_UTIL_H
#define HEARTBEAT_UTIL_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const HeartbeatUtil_h_Version =
    "$Id: HeartbeatUtil.h,v 1.17 2001/10/30 22:33:14 bko Exp $";

#include "HeartbeatCommon.h"


/**
 * Convert the enumerated server type into a readable representation.
 * @param  type  Server type to be converted
 * @return  The text version of the server type
 */
inline char* convertToString (const ServerType type)
{
    switch (type)
    {
        case SERVER_Unknown:
        return ("Unknown");
        break;
        case SERVER_PS:
        return ("PS");
        break;
        case SERVER_RS:
        return ("RS");
        break;
        case SERVER_MS:
        return ("MS");
        break;
        case SERVER_FS:
        return ("FS");
        break;
        case SERVER_JS:
        return ("JS");
        break;
        case SERVER_POS:
        return ("POS");
        break;
        case SERVER_CDR:
        return ("CDR");
        break;
        case SERVER_CA:
        return ("CA");
        break;
        case SERVER_UAVM:
        return ("UAVM");
        break;
        default:
        return ("Unrecognized");
        break;
    }
}

/**
 * Convert the server status enumerated type into a readable form
 * @param  status  Enumerated type for a server's status
 * @return  The status in a readable form
 */
inline char* convertToString (const ServerStatus status)
{
    switch (status)
    {
        case Inactive:
        return ("Inactive");
        break;
        case Active:
        return ("Active");
        break;
        case ActivePending:
        return ("ActivePending");
        break;
        case UnknownStatus:
        return ("UnknownStatus");
        break;
        default:
        return (0);
        break;
    }
}

#endif // HEARTBEAT_UTIL_H
