
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const HouseKeepingThread_cxx_Version =
    "$Id: HouseKeepingThread.cxx,v 1.10 2001/02/27 05:59:23 icahoon Exp $";

#include "cpLog.h"
#include "HeartbeatParms.hxx"
#include "HouseKeepingThread.hxx"
#include "HeartbeatCommon.h"
#include "VMutex.h"
#include "VCondition.h"

/**
 * Constructor - Define HouseKeepingThread
 * @param serverContainer the container of all servers
 */
HouseKeepingThread::HouseKeepingThread( BaseServerContainer& serverContainer )
        : ThreadIf(), itsServerContainer(serverContainer) 
{

    delayInMillisec = HeartbeatParms::instance().getHeartbeatInterval();
}

/**
 * Constructor 
 * @param  serverContainer  The Server Container 
 * @param  delayIntervalInMs  Delay in milli-seconds between sending heartbeats
 */
HouseKeepingThread::HouseKeepingThread(
    BaseServerContainer& serverContainer,
    int delayIntervalInMs
    )
        : ThreadIf (),
	  itsServerContainer(serverContainer),
          delayInMillisec(delayIntervalInMs)
{}

/**
 * Destructor
 */
HouseKeepingThread::~HouseKeepingThread()
{}

/**
 * Do Housekeeping continuously (with the given delay in between)
 * for as long as this thread is active.
 */
void
HouseKeepingThread::thread()
{
    cpLog( LOG_DEBUG, "Start HouseKeeping thread" );

    unsigned long delay = delayInMillisec * 1000;

    while (isShutdown() != true )
    {
        vusleep(delay);

        /// do Housekeeping on Server Container  
        itsServerContainer.doHouseKeeping();
    }
}

/// End of File
