/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include <iostream>
#include <fstream>
#include <string>

#include "RoundRobinServers.hxx"


/**
 * Format of input file:
 * ---------------------------
 * 5
 * From: 1020@192.168.22.10
 * From: 1100@192.168.5.12
 * From: 1220@192.168.6.92
 * From: 1230@192.168.6.91
 * ---------------------------
 */

int main( int argc, char* argv[] )
{
    if ( argc < 1 )
    {
        cerr << "Usage: inputFileName" << endl;
        exit( -1);
    }


    ifstream infile( argv[1] );
    if ( !infile )
    {
        cerr << "Unable to open: " << argv[1] << endl;
        exit( -1);
    }


    string sipFromString;
    Data sipFromData;

    //RoundRobinServers& rServers = RoundRobinServers::instance( RS );
    RoundRobinServers rrServers;
    int hitList[10];

    ///reading first line of file as number of servers
    getline( infile, sipFromString, '\n' );
    int noServers = atoi( sipFromString.c_str() );
    cout << "Operating under " << noServers << " server(s)" << endl;
    for ( int i = 0; i < noServers; i++)
        hitList[i] = 0;

    while ( getline( infile, sipFromString, '\n' ) )
    {
        cout << "Hashing: " << sipFromString << " into ";
        sipFromString += "\r\n";
        sipFromData = sipFromString;
        ///int hashResult = rServers->getPort(sipFromData);
        int hashResult = rrServers.hashingFunction(sipFromData) % noServers;
        if (!(hashResult < 0)) hitList[hashResult]++;
        else cout << "negative value not counted in hit list";
        cout << hashResult;
        cout << endl;
    }

    for ( int i = 0; i < noServers; i++)
        cout << "Server" << i << " " << hitList[i] << endl;

    return 0;
}
