#ifndef SERVERCONTAINER_HXX
#define SERVERCONTAINER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const ServerContainerVersion = "$Id: ServerContainer.hxx,v 1.25 2001/01/23 23:41:37 maivu Exp $";


#include "Data.hxx"
#include "BaseServerContainer.hxx"


/// Some functions provided for the Provisioning Server Callback
void updateRedirectServers( const string& data, const string& fileName, bool );
void updatePolicyServers( const string& data, const string& fileName, bool );
void updateServerHeartbeatParms( const string& data, const string& fileName, bool );

/**
 * ServerContainer is derived from the base BaseServerContainer class.
 * It also contains a list of all RSs and PolicyServers.  In the constructor,
 * it will configure the RS and POS from provisioning server, register with
 * provisioning server for updating all configuration data.
 */


class ServerContainer: public BaseServerContainer
{
public:
        /**
	 * Return the one Server container or create it if it doesn't already exist.
	 * @return  Reference to the singleton
	 */
        static ServerContainer& instance();

        /// Destructor
        virtual ~ServerContainer();
        /**
	 * Obtain a list of servers for the specified type
	 * @param  type  Type of server to obtain a list of
	 * @return  List of requested servers
	 */
        const Sptr < vector < Data > > getListofServersRef( const ServerType type ) const;
        /**
	 * Obtain a list of active servers of the type specified
	 * @param  type  Server type
	 * @param  listofActiveServers  List of active servers obtained
	 */
        void getListofActive( const ServerType type, Sptr < vector < Data > > & listofActiveServers);
        /**
	 * Deal with a heartbeat event from the given server
	 * @param  key  Server who sent the heartbeat
	 * @return  True if server is recognized, false if not
	 */
        bool processHeartbeat( const Data& key);
        /// Check all servers to see if they are heartbeating correctly
        void doHouseKeeping();
        /// Update RS list due to a change in Provisioning data.
        void updateRSsFromPS();
        /// Update Policy Server list due to a change in Provisioning data.
        void updatePOSsFromPS();
        /// Update Heartbeat data due to a change in Provisioning data.
        void updateHBParmsFromPS();
        /// display RS, POS list
        void displayLists();

protected:
        /** 
	 * Constructors
	 */
        ServerContainer();
        /**
	 * Copy Constructor
	 * @param  rhs  Object from which to copy
	 */
        ServerContainer::ServerContainer( const ServerContainer& rhs );
        /**
	 * Operator overload for "="
	 * @param  rhs  Object from which to copy
	 * @return  Reference to this object with new data
	 */
        ServerContainer& ServerContainer::operator=( const ServerContainer& rhs );
        /// This is the single instance of ServerContainer
        static Sptr < ServerContainer > theSrvContainer;
        /// configure ServerContainer with Provisioning data for RS's
        void configRedirectServers();
        /// configure ServerContainer with Provisioning data for Policy Servers
        void configPolicyServers();
        /**
	 * Obtain all servers of the type specified from Provisioning.
	 * @param  type  Type of server to obtain
	 * @param  retList  Compiled list of servers to pass back
	 */
        void getServersFromPS( const ServerType type, vector < Data > & retList);
        /**
	 * Scan the given list for duplicate servers and remove them.
	 * @param  list  List of servers
	 */
        void removeDuplicatesFromList( Sptr < vector < Data > > & list);

private:
        /**
	 * Copy member data from given object into this one
	 * @param  rhs  Object from which to copy
	 */
        void ServerContainer::copyRhsToThis( const ServerContainer& rhs );

        /// Data member lists
        Sptr < vector < Data > > itsRSList;
        VRWLock rsLock;

        Sptr < vector < Data > > itsPOSList;
        VRWLock posLock;

};

#endif // SERVERCONTAINER_HXX
