#ifndef UAVMCONTAINER_HXX
#define UAVMCONTAINER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const UavmContainer_hxx_Version =
    "$Id: UavmContainer.hxx,v 1.11 2001/01/23 23:41:41 maivu Exp $";


#include "Data.hxx"
#include "BaseServerContainer.hxx"

/**
 *  This class defines a container of uavm servers: it keeps track of the
 *  uavm servers which are active (i.e. send hearthbeat messages on the 
 *  multicast address 226.2.2.5). 
 *  Used in CPLOpVmRedirect operator (voice mail feature server).
 */

class UavmContainer: public BaseServerContainer
{
public:
       /**
	* Return the one Uavm container or create it if it doesn't already exist.
	* @return  The singleton Uavm container object
	*/
        static UavmContainer& instance();

        /// Destructor
        virtual ~UavmContainer();

        /**
	 * Add given server to the container
	 * @param  key  Server address
	 * @param  serverType  Type of server being added
	 * @param  maxMissedHB  Maximum number of heartbeats that can be missed 
	 * before the server is considered to be down.
	 */
        virtual void addServer( const Data& key, const ServerType serverType, const int maxMissedHB);
        /**
	 * Obtain a list of active servers of the type specified
	 * @param  type  Type of servers requested
	 * @param  listofActiveServers  Resulting compiled list of active servers
	 */        
        void getListofActive( const ServerType type, Sptr < vector < Data > > & listofActiveServers);
        /**
	 * Handle an incoming heartbeat event from the specified server and update
	 * hearthbeat data.
	 * @param  key  Server sending the heartbeat
	 * @return  True if server exists in the container, false otherwise
	 */        
        bool processHeartbeat( const Data& key);
        
        /**
	 * Go through server list and check to see if each is still heartbeating
	 */
        void doHouseKeeping();

        ///display UAVM list
        virtual void displayLists();

protected:
        /**
	 * Constructor
	 */
        UavmContainer();
        /**
	 * Copy Constructor
	 * @param  rhs  Object from which to copy
	 */
        UavmContainer::UavmContainer( const UavmContainer& rhs );
        /**
	 * Operator overload for "="
	 * @param  rhs  Object from which to copy
	 * @return  Reference to this object with new data
	 */
        UavmContainer& UavmContainer::operator=( const UavmContainer& rhs );
        /// This is the single instance of UavmContainer
        static Sptr < UavmContainer > theUavmContainer;

private:
        /**
	 * Copy member data from given object into this one
	 * @param  rhs  Object from which to copy
	 */
        void UavmContainer::copyRhsToThis( const UavmContainer& rhs );
};

#endif // UAVMCONTAINER_HXX
