
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const HBAgent_cxx_Version =
    "$Id: HBAgent.cxx,v 1.16 2002/02/25 05:46:19 sprajpat Exp $";

#include <netdb.h>
#include "HBAgent.hxx"
#include "HBServerContainer.hxx"
#include "cpLog.h"

Sptr < HBAgent > HBAgent::theHBAgent = 0;


/**
 * Return the one HBAgent or create it if it doesn't already exist. 
 * @return  Pointer to the singleton agent.
 */
Sptr < HBAgent >
HBAgent::instance()
{
    if ( theHBAgent == 0 )
    {
        theHBAgent = new HBAgent;
    }
    return theHBAgent;
}

/**
 * Constructor
 * @param  application type  
 * @param  application name 
 */
HBAgent::HBAgent() : AgentApi(SERVER_HB, "HeartbeatServer")
{}

/**
 * Destructor
 */
HBAgent::~HBAgent()
{}

/**
 * Build Message to be sent to Network Manager  
 * @param  sender  Address of the message sender
 * @return  The return status
 */
voReturnStatus
HBAgent::buildMessage( NetworkAddress *sender )
{
    HBServerContainer& hbServerContainer = HBServerContainer::instance();
    int numOfServers = 0, parmIndex = 0;

    /// create snmp struct to store server info
    int parm[PARM2SIZE / sizeof(int)];
    memset(&parm, 0, PARM2SIZE/sizeof(int));

    Sptr < vector < Data > > listofServers;
    listofServers = new vector < Data > ;
    hbServerContainer.getListofAllServers( listofServers );

    /// get server stats and store in parameter structure
    for (vector < Data > ::iterator i = listofServers->begin();
            i != listofServers->end(); ++i)
    {
        cpLog( LOG_DEBUG, "server: %s", i->logData() );
        numOfServers++;

        /// Create a copy of '' and strip off the ':'
        string address, port;
        char buf[156];
        (*i).getData(buf, 156);
        string address_port = buf;
        string::size_type colonPos = address_port.find(":");
        if (colonPos != string::npos)
        {
            address = address_port.substr(0, colonPos);
            cpLog( LOG_DEBUG, "address: %s", address.c_str() );
            port = address_port.substr(colonPos + 1);
            cpLog( LOG_DEBUG, "port: %s", port.c_str() );
        }
        else
        {
            /// format is not 'hn:port'
            address = address_port;
            port = "";
        }

        struct hostent tmphostent;
        char buffer[4096];

        int neterr;

        if (NetworkAddress::getHostByName( address.c_str(),
                                           &tmphostent,
                                           buffer,
                                           4096,
                                           &neterr ) == 0 )
        {
            memcpy((void*)&parm[++parmIndex],
                   tmphostent.h_addr,
                   tmphostent.h_length);

            cpLog( LOG_DEBUG, "ip: %lu", parm[parmIndex]);
            parm[++parmIndex] = atoi(port.c_str());
            cpLog( LOG_DEBUG, "port: %d", parm[parmIndex]);

            Data smartServer(*i);
            parm[++parmIndex] =
                (int)hbServerContainer.getType( smartServer );
            parm[++parmIndex] =
                (int)hbServerContainer.getStatus( smartServer );

        }
        else
        {
            cpLog(LOG_ERR, "Failed gethostbyname");
            numOfServers--;
        }
    }

    parm[0] = numOfServers;
    cpLog( LOG_DEBUG, "total servers: %d", parm[0]);
    if ( logLevel == "LOG_DEBUG" &&  numOfServers > 0 )
    {
        for (int j = 1; j <= (numOfServers * 4); j++)
        {
            cpLog( LOG_DEBUG, "parm[%d]: %lu", j, parm[j]);
        }
    }

    voReturnStatus result = sendResponse( parm, sender );
    return result;
}



/**
 * Process the message passed in.
 * @param  message  Message to be processed
 * @param  sender  Address of the message sender
 * @return  The return status
 */
voReturnStatus
HBAgent::processMessage(ipcMessage *message, NetworkAddress *sender)
{

    switch (message->action)
    {
        case Get :
        switch (message->mibVariable)
        {
            case SysStatsTotalServers :
            {
                HBServerContainer& hbServerContainer = HBServerContainer::instance();
                return sendResponse(hbServerContainer.getServerCount(), sender );
            }
            case SysStatsTotalSubscribers :
            return sendResponse(VO_NA_INT, sender);
            case SysStatsTotalProcessDeaths :
            return sendResponse(VO_NA_INT, sender);
            case SysStatsTotalProcessRestarts :
            return sendResponse(VO_NA_INT, sender);
            case AllServerInfo :
            return buildMessage(sender);
            default :
            return voUnknown;
        }
        case Set :
        case Trap :
        case Response :
        case Register :
        default :
        return voFailure;
    }
    return voFailure;
}

/**
 * Set the log level for debugging purpose
 * @param level debug level
 */
void
HBAgent::setLogLevel(string level)
{
  logLevel = level;
}
