
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const HBProxy_cxx_Version =
    "$Id: HBProxy.cxx,v 1.8 2001/02/05 19:50:36 maivu Exp $";

#include "cpLog.h"
#include "HBProxy.hxx"
#include "HBServerContainer.hxx"


/**
 * Ctor
 */
HBProxy::HBProxy()
{
    itsHeartbeatRxThread = new HeartbeatRxThread( HBServerContainer::instance() );
    itsHouseKeepingThread = new HouseKeepingThread( HBServerContainer::instance() );
}


/**
 * Copy member data from given object into this one
 * @param  rhs  Object from which to copy
 */
void
HBProxy::copyRhsToThis( const HBProxy& rhs )
{
    itsHeartbeatRxThread = rhs.itsHeartbeatRxThread;
    itsHouseKeepingThread = rhs.itsHouseKeepingThread;
}


/**
 * Copy Constructor
 * @param  rhs  Object from which to copy
 */
HBProxy::HBProxy( const HBProxy& rhs )
{
    copyRhsToThis( rhs );
}


/**
 * Operator overload for "="
 * @param  rhs  Object from which to copy
 * @return  Reference to this object with new data
 */
HBProxy&
HBProxy::operator=(const HBProxy& rhs )
{
    if ( this != &rhs)
        copyRhsToThis( rhs );

    return *this;
}


/**
 * Begin the heartbeat proxy and its threads
 */
void
HBProxy::run()
{
    if (itsHeartbeatRxThread != 0) itsHeartbeatRxThread->run();
    if (itsHouseKeepingThread != 0 ) itsHouseKeepingThread->run();
}


/**
 * Handle shutdown of threads 
 */
void
HBProxy::shutdown()
{
    if (itsHeartbeatRxThread != 0) itsHeartbeatRxThread->shutdown();
    if (itsHouseKeepingThread != 0 ) itsHouseKeepingThread->shutdown();
}


/**
 * Handle the ending of processes
 */
void
HBProxy::join()
{
    if (itsHeartbeatRxThread != 0) itsHeartbeatRxThread->join();
    if (itsHouseKeepingThread != 0 ) itsHouseKeepingThread->join();
}


/**
 * Destructor
 */
HBProxy::~HBProxy()
{}

// End of File
