
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const ServerWithSnmp_cxx_Version =
    "$Id: ServerWithSnmp.cxx,v 1.13 2002/02/25 05:46:19 sprajpat Exp $";

#include "cpLog.h"
#include "ServerWithSnmp.hxx"
#include "HBAgent.hxx"
#include "HeartbeatUtil.h"

/**
 * Constructor
 * @param  serverType  Type of server
 */
ServerWithSnmp::ServerWithSnmp( const ServerType serverType)
        : Server(serverType)
{}


/**
 * Copy Constructor
 * @param  rhs  Object from which to copy
 */
ServerWithSnmp::ServerWithSnmp( const ServerWithSnmp& rhs ) : Server(rhs)
{
    if (this != &rhs)
    {
        copyRhsToThis( rhs );
    }
}


/**
 * Operator overload for "="
 * @param  rhs  Object from which to copy
 * @return  Reference to this object with new data
 */
ServerWithSnmp&
ServerWithSnmp::operator=( const ServerWithSnmp& rhs )
{
    if ( this != &rhs )
    {
        Server::operator= (rhs);
        copyRhsToThis( rhs );
    }
    return *this;
}


/**
 * Copy member data from given object into this one
 * @param  rhs  Object from which to copy
 */
void
ServerWithSnmp::copyRhsToThis( const ServerWithSnmp& rhs )
{ }


/**
 * Destructor
 */
ServerWithSnmp::~ServerWithSnmp()
{ }


/**
 * Process a heartbeat according to the current status
 * @param  key  Server who sent the heartbeat
 */
void
ServerWithSnmp::processHeartbeat( const Data& key )
{
    incrHeartbeatCountByOne();

    if (itsStatus == Inactive)
    {
        gotFirstHeartbeat(key);
    }
    else if (itsStatus == ActivePending )
    {
        gotHiccup();
    }
}


/**
 * Process the initial heartbeat from a server by changing its status 
 * to active and report server up to snmp  
 * @param  key  Server who sent the heartbeat
 */
void
ServerWithSnmp::gotFirstHeartbeat( const Data& key )
{
    Server::gotFirstHeartbeat(key);

    /// notify snmp server up
    Data serverAddress( convertToString(itsType) );
    serverAddress += ":";
    serverAddress += key;
    char buf[256];
    serverAddress.getData(buf, 256);
    HBAgent::instance()->sendTrap(ServerUp, buf);
}


/**
 * Check to see if server is still active. Mark a server as no longer
 * active if it has reached it's max missed heartbeats and report server
 * down to snmp.
 * @param  serverId  Server on which to perform housekeeping
 */
void
ServerWithSnmp::doHouseKeeping( const Data& serverId)
{
    if ( itsStatus != Inactive )
    {
        if ( heartbeatCount == 0 )
        {
            incrNumOfHeartbeatMissed();

            /// *** keep this code around in case we decide
            /// *** to reinstate the ActivePending state.
            //if ( itsStatus == Active)
            //   setStatus(ActivePending);

            if ( serverIsDead() )
            {
                setStatus(Inactive);

                cpLog( LOG_ALERT, "\n");
                cpLog( LOG_ALERT, "** %s Down: %s **", 
                       convertToString(itsType),
                       serverId.logData() );
                /// notify snmp server down
                Data serverAddress( convertToString(itsType) );
                serverAddress += ":";
                serverAddress += serverId;
                char buf[256];
                serverAddress.getData(buf, 256);
                HBAgent::instance()->sendTrap(ServerDown, buf);

                /// clear HB counters
                clearHeartbeatCount();
                clearNumOfHeartbeatMissed();
            }
        }
        else
        {
	    /// reset heartbeat counters
	  clearHeartbeatCount();
	  clearNumOfHeartbeatMissed();
	}
    }
}
