#ifndef SERVERWITHSNMP_H
#define SERVERWITHSNMP_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



#include "Data.hxx"
#include "Server.hxx"

static const char* const ServerWithSnmpVersion = "$Id: ServerWithSnmp.hxx,v 1.8 2001/02/05 19:50:45 maivu Exp $";

/// This class describes a ServerWithSnmp in the system
class ServerWithSnmp : public Server
{
    public:
        /**
	 * Constructor
	 * @param  serverType  Type of server
	 */
        ServerWithSnmp( const ServerType serverType);

        /**
	 * Copy Constructor
	 * @param  rhs  Object from which to copy
	 */
        ServerWithSnmp::ServerWithSnmp( const ServerWithSnmp& rhs );

        /**
	 * Operator overload for "="
	 * @param  rhs  Object from which to copy
	 * @return  Reference to this object with new data
	 */
        ServerWithSnmp& ServerWithSnmp::operator=( const ServerWithSnmp& rhs );

        /// Destructor
        virtual ~ServerWithSnmp();

        /**
	 * Process a heartbeat according to the current status
	 * @param  key  Server who sent the heartbeat
	 */
        void processHeartbeat( const Data& key);

        /**
	 * Check to see if server is still active. Mark a server as no longer
	 * active if it has reached it's max missed heartbeats and report server
	 * down to snmp.
	 * @param  serverId  Server on which to perform housekeeping
	 */
        void doHouseKeeping( const Data& serverId);

    protected:

        /**
	 * Process the initial heartbeat from a server by changing its status 
	 * to active and report server up to snmp  
	 * @param  key  Server who sent the heartbeat
	 */
        void gotFirstHeartbeat( const Data& key);

    private:

        /**
	 * Copy member data from given object into this one
	 * @param  rhs  Object from which to copy
	 */
        void ServerWithSnmp::copyRhsToThis( const ServerWithSnmp& rhs );

};

#endif //SERVERWITHSNMP_H
