/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const sender_cxx_Version = "$Id: sender.cxx,v 1.2 2002/04/11 23:47:40 bko Exp $";

#include "cpLog.h"
#include "SenderProxy.hxx"
#include "CommandLine.hxx"
#include "Data.hxx"
#include "ProvisionInterface.hxx"

int
main (const int argc, const char** argv)
{
    CommandLine::instance(argc, argv);

    Data pServer;
    Data pServerPort;
    Data sipPort;
    Data backupPServer;
    Data backupPServerPort;

    {
	Data psHostPort = CommandLine::instance()->getString( "PSERVER" );
	char matched;
	Data psHost = psHostPort.matchChar(":", &matched);
	Data psPort = "6005";
	if(matched != '\0') 
	{ 
	    psPort = psHostPort;
	}
	else 
	{
	    // no match, so the remainder must be the psHost
	    psHost = psHostPort;
	}

	pServer = psHost.convertString();
	pServerPort = psPort.convertInt();
    }

    sipPort = CommandLine::instance()->getInt( "SIP_PORT" );

    {
	Data psHostPort = CommandLine::instance()->getString( "PSERVERBACKUP" );
	char matched;
	Data psHost = psHostPort.matchChar(":", &matched);
	Data psPort = "6005";
	if(matched != '\0') 
	{ 
	    psPort = psHostPort;
	}
	else 
	{
	    // no match, so the remainder must be the psHost
	    psHost = psHostPort;
	}

	backupPServer = psHost.convertString();
	backupPServerPort = psPort.convertInt();
    }

    // connect to ProvisionInterface
    try
    {
        ProvisionInterface::instance(pServer, pServerPort,
				     backupPServer, backupPServerPort);
    }
    catch (VException& e)
    {
        cpLog(LOG_ERR, "Failed to connect to Provisioning Server. Stop!!!");
	return 1;
    }

    // create a proxy
    SenderProxy sender(atoi(sipPort.getData()));

    sender.run();
    sender.join();

    return 0;
}

/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
