/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const PS_Common_cxx_Version =
    "$Id: PSCommon.cxx,v 1.3 2001/11/05 18:07:37 icahoon Exp $";

#include "PSCommon.hxx"

///
const string PSCommon::OPEN_BRACKET("<");
///
const string PSCommon::END_ELEMENT_OPEN_BRACKET("</");
///
const string PSCommon::CLOSE_BRACKET(">");


/* Common HTTP Method tags reserved for PS Protocol
 */
///
const string PSCommon::GET_RECORDS_IN_DOMAIN_STR("GetRecordsInDomain");
///
const string PSCommon::GET_FIELDS_IN_DOMAIN_STR("GetFieldsInDomain");
///
const string PSCommon::SET_RECORDS_IN_DOMAIN_STR("SetRecordsInDomain");
///
const string PSCommon::ADD_RECORD_IN_DOMAIN_STR("AddRecordInDomain");
///
const string PSCommon::DELETE_RECORDS_IN_DOMAIN_STR("DeleteRecordsInDomain");
///
const string PSCommon::SESSION_OPEN_STR("SessionOpen");
///
const string PSCommon::SESSION_CLOSE_STR("SessionClose");
///
const string PSCommon::SESSION_SEARCH_DB_STR("SessionSearchDB");
///
const string PSCommon::SESSION_GET_DB_RESULT_STR("SessionGetDBResult");
///
const string PSCommon::SESSION_SORT_DB_RESULT_STR("SessionSortDBResult");
///
const string PSCommon::SESSION_SET_DB_RESULT_STR("SessionSetDBResult");
///
const string PSCommon::SESSION_DELETE_ALL_STR("SessionDeleteAll");
///
const string PSCommon::GET_XML_DESC_STR("GetXMLDescription");
///
const string PSCommon::LIST_XML_CAT_STR("ListXMLCategories");
///
const string PSCommon::LIST_XML_DESC_STR("ListXMLDescriptions");
///
const string PSCommon::REGISTER_DOMAIN_FOR_UPDATE_STR("RegisterDomainForUpdate");
///
const string PSCommon::REGISTER_RECORD_FOR_UPDATE_STR("RegisterRecordForUpdate");
///
const string PSCommon::UPDATE_DOMAIN_STR("UpdateDomain");
///
const string PSCommon::UPDATE_RECORD_STR("UpdateRecord");
///
const string PSCommon::UN_REGISTER_DOMAIN_FOR_UPDATE_STR("UnRegisterDomainForUpdate");
///
const string PSCommon::UN_REGISTER_RECORD_FOR_UPDATE_STR("UnRegisterRecordForUpdate");



/* Common XML tags reserved for PS Protocol
 */

/// GET/PUT/POST request reserved keywords:
const string PSCommon::ARGUMENTS_STR("arguments");
const string PSCommon::ARGUMENTS_START("<arguments>");
const string PSCommon::ARGUMENTS_END("</arguments>");
///
const string PSCommon::DOMAIN_STR("domain");
const string PSCommon::DOMAIN_START("<domain>");
const string PSCommon::DOMAIN_END("</domain>");
///
const string PSCommon::FILTER_STR("filter");
const string PSCommon::FILTER_START("<filter>");
const string PSCommon::FILTER_END("</filter>");
///
const string PSCommon::FIELDLIST_STR("fieldlist");
const string PSCommon::FIELDLIST_START("<fieldlist>");
const string PSCommon::FIELDLIST_END("</fieldlist>");
///
const string PSCommon::LISTITEM_STR("listitem");
const string PSCommon::LISTITEM_START("<listitem>");
const string PSCommon::LISTITEM_END("</listitem>");

/* PUT request and GET response reserved k/w for signifying a 
 * list of k/v pairs:
 */
const string PSCommon::FIELDVALUE_STR("fieldvalue");
const string PSCommon::FIELDVALUE_START("<fieldvalue>");
const string PSCommon::FIELDVALUE_END("</fieldvalue>");

/// GET response reserved keywords:
const string PSCommon::RESPONSE_STR("response");
const string PSCommon::RESPONSE_START("<response>");
const string PSCommon::RESPONSE_END("</response>");
///
const string PSCommon::RESULTCOUNT_STR("resultcount");
const string PSCommon::RESULTCOUNT_START("<resultcount>");
const string PSCommon::RESULTCOUNT_END("</resultcount>");
///
const string PSCommon::MATCH_STR("match");
const string PSCommon::MATCH_START("<match>");
const string PSCommon::MATCH_END("</match>");

/// Session-specific reserved keywords:
const string PSCommon::EXPIRETIME_STR("expiretime");    // for session open
const string PSCommon::EXPIRETIME_START("<expiretime>");    // for session open
const string PSCommon::EXPIRETIME_END("</expiretime>");    // for session open
///
const string PSCommon::TOKEN_STR("token");
const string PSCommon::TOKEN_START("<token>");
const string PSCommon::TOKEN_END("</token>");
///
const string PSCommon::START_STR("start");         // start index
const string PSCommon::START_START("<start>");         // start index
const string PSCommon::START_END("</start>");         // start index
///
const string PSCommon::END_STR("end");             // end index
const string PSCommon::END_START("<end>");             // end index
const string PSCommon::END_END("</end>");             // end index

/// sort related reserved keywords:
const string PSCommon::SORTTYPE_STR("sorttype");
const string PSCommon::SORTTYPE_START("<sorttype>");
const string PSCommon::SORTTYPE_END("</sorttype>");
///
const string PSCommon::SORTFIELD_STR("sortfield");
const string PSCommon::SORTFIELD_START("<sortfield>");
const string PSCommon::SORTFIELD_END("</sortfield>");
///
const string PSCommon::SORT_ASCEND_STR("ascend");         
///
const string PSCommon::SORT_DESCEND_STR("descend");

/// error reserved keyword:
const string PSCommon::ERROR_STR("error");
const string PSCommon::ERROR_START("<error>");
const string PSCommon::ERROR_END("</error>");

/// xml desc reserved keyword:
const string PSCommon::XMLCATEGORY_STR("xmlcategory");
const string PSCommon::XMLCATEGORY_START("<xmlcategory>");
const string PSCommon::XMLCATEGORY_END("</xmlcategory>");

const string PSCommon::XMLDESC_STR("xmldescription");
const string PSCommon::XMLDESC_START("<xmldescription>");
const string PSCommon::XMLDESC_END("</xmldescription>");

/// Registration stuff
const string PSCommon::PRIMARYKEY_STR("primarykey");
const string PSCommon::PRIMARYKEY_START("<primarykey>");
const string PSCommon::PRIMARYKEY_END("</primarykey>");

const string PSCommon::REGISTER_HOSTPORT_STR("register_hostport");
const string PSCommon::REGISTER_HOSTPORT_START("<register_hostport>");
const string PSCommon::REGISTER_HOSTPORT_END("</register_hostport>");

void PSCommon::xmlAddTag( const string& tag,  string& target ) {
    target += tag + "\r\n";
};

void PSCommon::xmlAddTagValue( const string& startTag, const string& val,
                       const string& endTag,   string& target ) {
    target += startTag + val + endTag + "\r\n";
}

