#ifndef PS_COMMON_HXX
#define PS_COMMON_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const PS_Common_hxx_Version =
    "$Id: PSCommon.hxx,v 1.4 2002/02/25 05:49:40 sprajpat Exp $";

#include "global.h"
#include <string>

class PSCommon
{
    public:
        ///
        typedef enum
        {
            SORT_NONE=0,
            SORT_ASCENDING, 
            SORT_DESCENDING,
            MAX_SORT_TYPE
        } PSSortType;
    
        ///
        typedef enum
        {
            HTTPXML = 0,
            // JAVA_SPACES,
            // SNMP,
            MAX_PSIF_TYPE
        } PSIFType;

///
static const string OPEN_BRACKET;               // "<"
///
static const string END_ELEMENT_OPEN_BRACKET;   // "/<"
///
static const string CLOSE_BRACKET;              // ">"

/* Common HTTP URI tags reserved for PS Protocol
 */
///
static const string GET_RECORDS_IN_DOMAIN_STR;
///
static const string GET_FIELDS_IN_DOMAIN_STR;
///
static const string SET_RECORDS_IN_DOMAIN_STR;
///
static const string ADD_RECORD_IN_DOMAIN_STR;
///
static const string DELETE_RECORDS_IN_DOMAIN_STR;
///
static const string SESSION_OPEN_STR;
///
static const string SESSION_CLOSE_STR;
///
static const string SESSION_SEARCH_DB_STR;
///
static const string SESSION_GET_DB_RESULT_STR;
///
static const string SESSION_SORT_DB_RESULT_STR;
///
static const string SESSION_SET_DB_RESULT_STR;
///
static const string SESSION_DELETE_ALL_STR;
///
static const string GET_XML_DESC_STR;
///
static const string LIST_XML_CAT_STR;
///
static const string LIST_XML_DESC_STR;
///
static const string REGISTER_DOMAIN_FOR_UPDATE_STR;
///
static const string REGISTER_RECORD_FOR_UPDATE_STR;
///
static const string UPDATE_DOMAIN_STR;
///
static const string UPDATE_RECORD_STR;
///
static const string UN_REGISTER_DOMAIN_FOR_UPDATE_STR;
///
static const string UN_REGISTER_RECORD_FOR_UPDATE_STR;

    
/* Common XML tags reserved for PS Protocol
 */
/// GET/PUT/POST request reserved keywords:
static const string ARGUMENTS_STR;
static const string ARGUMENTS_START;
static const string ARGUMENTS_END;

static const string DOMAIN_STR;
static const string DOMAIN_START;
static const string DOMAIN_END;

static const string FILTER_STR;
static const string FILTER_START;
static const string FILTER_END;

static const string FIELDLIST_STR;
static const string FIELDLIST_START;
static const string FIELDLIST_END;

static const string LISTITEM_STR;
static const string LISTITEM_START;
static const string LISTITEM_END;

/** PUT request / GET response reserved k/w for signifying 
 *  a list of k/v pairs:
 */
static const string FIELDVALUE_STR;
static const string FIELDVALUE_START;
static const string FIELDVALUE_END;

/// GET response reserved keywords:
static const string RESPONSE_STR;
static const string RESPONSE_START;
static const string RESPONSE_END;

static const string RESULTCOUNT_STR;
static const string RESULTCOUNT_START;
static const string RESULTCOUNT_END;

static const string MATCH_STR;
static const string MATCH_START;
static const string MATCH_END;

/// Session-specific reserved keywords:
static const string EXPIRETIME_STR;    // for session open
static const string EXPIRETIME_START;    // for session open
static const string EXPIRETIME_END;    // for session open

static const string TOKEN_STR;
static const string TOKEN_START;
static const string TOKEN_END;


///
static const string START_STR;         // start index
static const string START_START;         // start index
static const string START_END;         // start index
///
static const string END_STR;           // end index
static const string END_START;           // end index
static const string END_END;           // end index

/// sort related reserved keywords:
static const string SORTTYPE_STR;
static const string SORTTYPE_START;
static const string SORTTYPE_END;

static const string SORTFIELD_STR;
static const string SORTFIELD_START;
static const string SORTFIELD_END;
///
static const string SORT_ASCEND_STR;
///
static const string SORT_DESCEND_STR;

/// error reserved keyword:
static const string ERROR_STR;
static const string ERROR_START;
static const string ERROR_END;

/// xml desc reserved keywords
static const string XMLCATEGORY_STR;
static const string XMLCATEGORY_START;
static const string XMLCATEGORY_END;

static const string XMLDESC_STR;
static const string XMLDESC_START;
static const string XMLDESC_END;

/// Registration stuff
static const string PRIMARYKEY_STR;
static const string PRIMARYKEY_START;
static const string PRIMARYKEY_END;

static const string REGISTER_HOSTPORT_STR;
static const string REGISTER_HOSTPORT_START;
static const string REGISTER_HOSTPORT_END;


/********************************************************************
    Helper functions to build PS XML content:
 ********************************************************************/
///
static void xmlAddTag( const string& tag,  string& target );

///
static void xmlAddTagValue( const string& startTag, const string& val,
                       const string& endTag, string& target );
};

#endif // PS_COMMON_HXX
