
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const PSDBRequest_cxx_Version =
    "$Id: PSDBRequest.cxx,v 1.5 2002/02/25 05:49:40 sprajpat Exp $";



#include <stdio.h>
#include <iostream.h>
#include "PSDBRequest.hxx"
#include "cpLog.h"
#include "VBadDataException.hxx"
#include "Data.hxx"


///
PSDBRequest::PSDBRequest()
  : myDomain(""), myFilter(""), myToken(""), 
    myXmlCategory(""), myXmlDescName(""),
    registerHostport(""), keywordFlag(false)
{ 
    setMessageType( PSMessage::UNKNOWN_MSG_TYPE );
   
} 

///
PSDBRequest::PSDBRequest(PSMessage::msgType type)
  : myDomain(""), myFilter(""), myToken(""), 
    myXmlCategory(""), myXmlDescName(""),
    registerHostport(""), keywordFlag(false)
{
    setMessageType( type ); 
}

///
void PSDBRequest::clear()
{
    setMessageType( PSMessage::UNKNOWN_MSG_TYPE );

    myFieldList.clear();
    // clear myTable
    myTable.clear();

    myDomain = "";
    myFilter = "";
    myToken = "";
    myXmlCategory = "";
    myXmlDescName = "";
    registerHostport = "";
    keywordFlag = false;
}

/// single table of kv pairs
void 
PSDBRequest::setFieldValue(const string& kw, const string& strVal)
                           throw (VException&)
{

    if (kw.size() == 0)
    {
        string err = "setFieldValue fails, NULL keyword";
	cpLog (LOG_ERR, err.c_str());
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }
    Data copied(kw);
    copied.removeSpaces();
    LocalScopeAllocator lo;
    const char* copiedKw = copied.getData(lo);

    TableIter myTableItr = myTable.find(copiedKw);

    if (myTableItr != myTable.end() )
    {
        string strtemp = myTableItr->second;
        strtemp += strVal;
        myTable[copiedKw] = strtemp;
    }
    else
    {
        myTable[copiedKw] = strVal;
    }
}

/// get single instance of a keyword's value
void
PSDBRequest::getFieldValue( const string& kw, string& value )
                           throw (VException&)
{
    cpLog(LOG_DEBUG, "getFieldValue %s", kw.c_str()); 

    value = "";
    if (kw.size() == 0)
    {
        string err = "getFieldValue fails, NULL keyword";
	cpLog (LOG_ERR, err.c_str());
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }
    if (myTable.size() == 0)
    {
        string err = "getFieldValue fails, empty table";
	cpLog (LOG_ERR, err.c_str());
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }
    TableIter i;
    i = myTable.find(kw);
    if ( i != myTable.end() )
    {
        value.assign(i->second);
    }
    else
    {   // no such keyword - throw exception
        string err = "Failed getFieldValue: Keyword" + kw + "not found.";
        cpLog(LOG_ERR, err.c_str());
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }
}


void
PSDBRequest::print(const int loglevel)
{
    cpLog(loglevel, "\n************************************\n"
                    "Session ID = %s\n"
                    "fieldList size : %d\n"
                    "table size : %d ",
                    getSessionID().c_str(), 
	            myFieldList.size(),
                    myTable.size());

    // display parent object
    // PSMessage::print(loglevel);

    cpLog(loglevel, "domain = %s, filterRules = %s",
          myDomain.c_str(), myFilter.c_str());

    for ( unsigned short i = 0; i < myFieldList.size(); i++ )
    {
        cpLog(loglevel, "field list item: %s", myFieldList[i].c_str());
    }

    /// print content of myTable:
    ConstTableIter  itr;
    
    for (itr = myTable.begin(); itr != myTable.end(); itr++)
    {
        cpLog(loglevel, "%s = %s", itr->first.c_str(),
              itr->second.c_str());
    }
         
    cpLog(loglevel, "compiled at: %s\n************************************",
           __DATE__ " " __TIME__);
}

///
void
PSDBRequest::setArguments(const string& arg)
  throw (VException&)
{
    // there should not have any characters
    if (arg != "\n")
    {
        string err = "Duplicate keyword \"arguments\"";
	cpLog (LOG_ERR, err.c_str());
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }
}

///
void
PSDBRequest::setDomain(const string& domain)
  throw (VException&)
{
    // there is only one time to set myDomain.
    if (! myDomain.empty())
    {
        string err = "Duplicate keyword \"domain\"";
	cpLog (LOG_ERR, err.c_str());
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }
    else
        myDomain = domain;

}

///
void 
PSDBRequest::setRegisterHostport(const string& hp) 
  throw (VException&)
{ 
    if (keywordFlag)
    {
        string err = "Duplicate keyword \"register_hostport\"";
	cpLog (LOG_ERR, err.c_str());
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }
    else
        registerHostport = hp; 
}

///
void
PSDBRequest::setFilter( const string& fr ) 
  throw (VException&)
{ 
    if (keywordFlag)
    {
        string err = "Duplicate keyword \"filter\"";
	cpLog (LOG_ERR, err.c_str());
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }
    else
        myFilter += fr; 
}

///
void
PSDBRequest::setSessionID( string token )
  throw (VException&)
{
    if (keywordFlag)
    {
        string err = "Duplicate keyword \"token\"";
	cpLog (LOG_ERR, err.c_str());
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }
    else
        myToken = token; 
}

///
void 
PSDBRequest::addFieldList(const string& field)
  throw (VException&)
{
    // listitem can be only appeared in GET/POST message
    // it can not be associated with a value
    if (keywordFlag)
    {
        string err = "Duplicate keyword \"listitem\"";
	cpLog (LOG_ERR, err.c_str());
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }
    else
    {
	Data copied(field);
	copied.removeSpaces();
        LocalScopeAllocator lo;
        myFieldList.push_back(copied.getData(lo)); 
    }
}


///
void
PSDBRequest::setKeywordFlag(const string& fn, const string& val)
  throw (VException&)
{
    // the keywords are fieldlist, fieldvalue
    // it can not be associated with any characters, except '\n'
    if (val != "\n")
    {
        string err = "Duplicate keyword \"" + fn + "\"";;
	cpLog (LOG_ERR, err.c_str());
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }
    else if (fn != "fieldlist")
    {
        cpLog (LOG_DEBUG, "Set Keyword flag");
        keywordFlag = true;
    }
}


// End of File
