#ifndef PSDBREQUEST_HXX 
#define PSDBREQUEST_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


#include <cassert>
#include <string>
#include <map>
#include "Sptr.hxx"
#include "VException.hxx"
#include "PSMessage.hxx"
#include "PSDBCommonType.h"


static const char* const PSDBRequestVersion = "$Id: PSDBRequest.hxx,v 1.2 2001/09/19 20:31:14 maivu Exp $";

///
class PSDBRequest : public PSMessage
{
    public:

        ///
        PSDBRequest();

        ///
        PSDBRequest( PSMessage::msgType type ); 

        ///
        virtual ~PSDBRequest() { clear(); }

        ///
        void clear();

        ///
        void parseMessage() throw (VException&) {}

        ///
        void setDomain( const string& domain) throw (VException&);

            ///
        void setXmlCategory( const string& category) { myXmlCategory = category; }

            ///
        void setXmlDescName( const string& descName) { myXmlDescName = descName; }

        // exception is thrown if setFilter is called after
        // keywords (fieldlist, fieldvalue) flag is set.
        void setFilter( const string& fr ) throw (VException&);

        ///  tag new field to end of FieldList - for GET/POST requests
        void addFieldList( const string& field )
	  throw (VException&);

        ///
        void setSessionID( string token ) throw (VException&);

        ///
        bool isFieldListEmpty() { return myFieldList.empty(); }

        ///
        const string& getDomain() { return myDomain; }

    ///
        const string& getXmlCategory() { return myXmlCategory; }

    ///
        const string& getXmlDescName() { return myXmlDescName; }

        ///
        const string& getFilter() { return myFilter; }

        const string& getSessionID() { return myToken; }

        /// for PUT requests:
        void setFieldValue( const string& kw, const string& strVal )
             throw (VException&);
  
        /// return first occurrence of a keyword's value
        void getFieldValue( const string& kw, string& value )
             throw (VException&);

        ///
        const FieldListtype getFieldList() { return myFieldList; }

        /** myTable is a list of keyword/value pairs - used to  
         *  store fieldvalues found in e.g. AddRecordToDomain request:
         */
        const tableType& getTable() const 
        { 
            return( myTable );
        }

        /// 
        void print(const int loglevel);

        ///
        void setRegisterHostport(const string& hp) throw (VException&);
        const string& getRegisterHostport() { return registerHostport; }        

        ///
        void setArguments(const string& arg) throw (VException&);

        ///
        void setKeywordFlag(const string& fn, const string& val) 
	  throw (VException&);

    protected:

    private:

        ///  e.g. db table name
        string myDomain;

        ///  filter rules
        string myFilter;  

        ///  session ID
        string myToken;

        /// xml category
        string myXmlCategory;

        /// xml description name
        string myXmlDescName;

        /** field names of interest - applicable to GetRecordsInDomain
         *  DeleteRecordsInDomain, OK to be an empty list
         */
        FieldListtype myFieldList; 

        /**  array container used to store a list of keyword/value
         *   pairs
         */
        tableType   myTable;

        /// used when register/unregister for update
        string registerHostport;

        /// used to validate duplicate keyword
        bool keywordFlag;

};

#endif // PSDBREQUEST_HXX
