
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const PSDBResult_cxx_Version =
    "$Id: PSDBResult.cxx,v 1.8 2001/10/26 21:01:55 maivu Exp $";



#include <stdio.h>
#include <iostream.h>
#include "PSDBResult.hxx"
#include "cpLog.h"
#include "VBadDataException.hxx"
#include "XmlBlobHelper.hxx"



///
void PSDBResult::clear()
{
    myStatus = 0;
    myNumMatches = 0;
    setMessageType( PSMessage::UNKNOWN_MSG_TYPE );

    // clear myTables
    for (unsigned short i = 0; i < myTables.size(); i++)
      {
	myTables[i].clear();
      }

    // clear myFieldList
    myFieldList.clear();
}

/// single table of kv pairs
void 
PSDBResult::setFieldValue(const string& kw, const string& strVal)
                           throw (VException&)
{

    if (kw.size() == 0)
    {
        string err = "setFieldValue failed: NULL keyword";
        cpLog(LOG_ERR, err.c_str());
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }
    (myTables[0])[kw] = strVal;
    //(myTables[0]).push_back( tableType::value_type(kw, strVal) );
}

void
PSDBResult::setFieldValue(int idx, const string& kw, const string& strVal)
                           throw (VException&)
{

    if (kw.size() == 0)
    {
        string err = "setFieldValue failed: NULL keyword";
        cpLog(LOG_ERR, err.c_str());
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }

    if (idx == -1)
    {
        string error = "setFieldValue Failed. NO matching index. ";
	error += "Keyword [";
	error += kw;
	error += "], value [";
	error += strVal;
	error += "]";

        cpLog(LOG_ERR, error.c_str());
        VBadDataException exception(error.c_str(), __FILE__, __LINE__);
	throw exception;
    }

      myTables[idx].insert( tableType::value_type(kw, strVal) );

}
 
///
void 
PSDBResult::findField( int idx, const string& kw, TableIter& itr )
                       throw (VException&)
{
    if (kw.size() == 0)
    {
        string err = "findField fails, NULL keyword";
        cpLog(LOG_ERR, err.c_str());
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;	
    }
    if (myTables.size() == 0)
    {
        string err = "findField fails, empty table";
        cpLog(LOG_ERR, err.c_str());
	VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }
    if (idxInRange( idx))
    {
        TableIter i;

        i = myTables[idx].find(kw); 
        if ( i != myTables[idx].end() )
        {
            itr = i;
        }
        else
        {   
	    string err = "findField fails, keyword " + kw + " not found";
	    cpLog (LOG_ERR, err.c_str());
	    VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	    throw exception;
        }
    }
    else  // idx out of range of myTables
    {
        string err = "findField fails: Index out of range";
        cpLog(LOG_DEBUG, "%s: %d", err.c_str(), idx);
        VBadDataException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }
}

/// get single instance of a keyword's value
void
PSDBResult::getFieldValue( const string& kw, string& value )
                           throw (VException&)
{
    cpLog(LOG_DEBUG, "getFieldValue %s", kw.c_str()); 
    TableIter i;

    findField( 0, kw, i );

    value.assign(i->second);
}


/** given an index for  multiple matches, find the first 
 *  occurrence of a keyword, and update its value
 */
void
PSDBResult::getFieldValue( int idx, const string& kw, string& value )
                           throw (VException&)
{
    value = "";
    cpLog(LOG_DEBUG, "getFieldValue %s, idx %d", kw.c_str(), idx);
    TableIter i;
    
    findField( idx, kw, i );

    value.assign(i->second);

}


//void
//PSDBResult::getFieldValues( const string& kw, vector<string>& values )
//                            throw (VException&)
//{
//}
    
//int
//PSDBResult::getFieldValues(int idx,  const string& kw, vector<string>& values )
//{
//    pair<tableType::const_iterator, tableType::const_iterator> pr =
//         myTables[idx].equal_range(kw);
//    for (tableType::const_iterator i = pr.first; i!=pr.second; ++i)
//    {
//       values[idx] = (*i).second;
//    }
//    return 0;
//}

void
PSDBResult::getFieldValueAsBool( const string& kw, bool& boolVal )
                                   throw (VException&)
{
    cpLog(LOG_DEBUG, "getFieldValueAsBool %s", kw.c_str());
    TableIter i;

    findField( 0, kw, i );

    int x = atoi(i->second.c_str());
    boolVal = (x ? true : false);
}

void
PSDBResult::getFieldValueAsBool( int idx, const string& kw, bool& boolVal )
                                   throw (VException&)
{
    cpLog(LOG_DEBUG, "getFieldValueAsBool %s, idx %d", kw.c_str(), idx);
    TableIter i;

    findField( idx, kw, i );

    int x = atoi(i->second.c_str());
    boolVal = (x ? true : false);

}

void
PSDBResult::getFieldValueAsInt( const string& kw, int& intVal )
                                  throw (VException&)
{
    cpLog(LOG_DEBUG, "getFieldValueAsInt %s", kw.c_str());

    TableIter i;

    findField( 0, kw, i );

    intVal = atoi(i->second.c_str());

}

void
PSDBResult::getFieldValueAsInt( int idx, const string& kw, int& intVal )
                                throw (VException&)
{
    cpLog(LOG_DEBUG, "getFieldValueAsInt %s, idx %d", kw.c_str(), idx);
    TableIter i;

    findField( idx, kw, i );

    intVal = atoi(i->second.c_str());

}


void
PSDBResult::getFieldValueAsLong( const string& kw, long int& longVal )
                                     throw (VException&)
{
    cpLog(LOG_DEBUG, "getFieldValueAsLong %s", kw.c_str());
    TableIter i;

    findField( 0, kw, i );

    longVal = atol(i->second.c_str());
    cout << kw.c_str() << "=" << i->second.c_str() << endl;
    char buf[20];
    sprintf(buf, "%ln", &longVal);
    cout << buf << endl;
}

void
PSDBResult::getFieldValueAsLong( int idx,  const string& kw, 
                                   long int& longVal )
                                   throw (VException&)
{
    cpLog(LOG_DEBUG, "getFieldValueAsLong %s, idx %d", kw.c_str(), idx);
    TableIter i;

    findField( idx, kw, i );

    longVal = atol(i->second.c_str());
    cout << kw.c_str() << "=" << i->second.c_str() << endl;
    char buf[20];
    sprintf(buf, "%li", longVal);
    cout << buf << endl;
}

void
PSDBResult::getFieldValueAsDouble( const string& kw, double& doubleVal )
                                     throw (VException&)
{
    cpLog(LOG_DEBUG, "getFieldValueAsDouble %s", kw.c_str());
    TableIter i;

    findField( 0, kw, i );

    doubleVal = atof(i->second.c_str());

}

void
PSDBResult::getFieldValueAsDouble( int idx,  const string& kw, 
                                   double& doubleVal )
                                   throw (VException&)
{
    cpLog(LOG_DEBUG, "getFieldValueAsDouble %s, idx %d", kw.c_str(), idx);
    TableIter i;

    findField( idx, kw, i );

    doubleVal = atof(i->second.c_str());

}

///
const tableType& 
PSDBResult::getTable(int idx) const throw (VException&)
{
    // need to do range checking
    if (idxInRange( idx ) != true)
      {
	char* error = new char[80]; memset(error, '\0', 80);
	sprintf(error, "PSDBResult::getTable(index=%d): Index Out of Range", idx);
	cpLog (LOG_ERR, error);
        VBadDataException exception(error, __FILE__, __LINE__);
	throw exception;
      }

    return( myTables[idx]); 
}


// Add into the field list
void
PSDBResult::addFieldList(const string & val)
{
    // append it into the end.
    myFieldList.push_back(val);

}


// const list<string>&
// PSDBResult::getList() {}

void
PSDBResult::print(const int loglevel)
{
    cpLog(loglevel, "\n************************************\n"
                    "Domain = %s, Status = %d\n"
                    "Session ID = %s\n"
                    "Error text = %s\n"
                    "Number of records found: %d\n"
                    "************************************",
                    getDomain().c_str(), getStatus(),
                    getSessionID().c_str(), getError().c_str(),
                    myTables.size());

    // display parent object
    // PSMessage::print(loglevel);

    for ( unsigned short i = 0; i < myFieldList.size(); i++ )
    {
        cpLog(loglevel, "field list item: %s", myFieldList[i].c_str());
    }

    /// print content of myTables:
    ConstTableIter  itr;
    for (int i = 0; i < static_cast<int> (myTables.size()); i++)
    {
        const tableType& record = getTable( i );
        for (itr = record.begin(); itr != record.end(); itr++)
        {
            cpLog(loglevel, "%d) %s = %s", i, itr->first.c_str(),
                  itr->second.c_str());
        }
         
    }
    cpLog(loglevel, "compiled at: %s\n************************************",
           __DATE__ " " __TIME__);
}

///
void 
PSDBResult::getFieldValueAsList(const string& kw,
				blobType& listResult)
  throw (VException&)
{
    cpLog (LOG_DEBUG, "GetField Value as list: kw = [%s]", kw.c_str());

    // get the blob string
    string blob;
    getFieldValue(kw, blob);

    XmlBlobHelper helper(blob);
    listResult = helper.getBlobTable();

}

///
void 
PSDBResult::getFieldValueAsList(int idx, 
				const string& kw, 
				blobType& listResult)
  throw (VException&)
{

    cpLog (LOG_DEBUG, "GetField Value as list: index = [%d], kw = [%s]", 
	   idx, kw.c_str());

    // get the blob string
    string blob;
    getFieldValue(idx, kw, blob);

    XmlBlobHelper helper(blob);
    listResult = helper.getBlobTable();

}





// End of File
