#ifndef PSDBRESULT_HXX 
#define PSDBRESULT_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


#include <cassert>
#include <string>
#include <list>
#include "Sptr.hxx"
#include "VException.hxx"
#include "PSMessage.hxx"
#include "PSDBCommonType.h"


static const char* const PSDBResultVersion = "$Id: PSDBResult.hxx,v 1.4 2001/09/25 18:52:20 maivu Exp $";

///
class PSDBResult : public PSMessage
{
    public:

        ///
        PSDBResult() 
        { 
            myStatus = 0; 
            myNumMatches = 0; 
            myToken = -1;
            setMessageType( PSMessage::UNKNOWN_MSG_TYPE );
        } 

        ///
        PSDBResult( PSMessage::msgType type ) 
        {
           myStatus = 0;
           myNumMatches = 0;
           myToken = -1;
           setMessageType( type ); 
        }

        ///
        virtual ~PSDBResult() { clear(); }

        ///
        void clear();

        ///
        void parseMessage() throw (VException&) {}

        ///
        void setStatus( int sts ) { myStatus = sts; }

        ///
        void setDomain( const string& domain) { myDomain = domain; }

            ///
        void setXmlCategory( const string& cat) { myXmlCategory = cat; }

            ///
        void setXmlDescription( const string& desc) { myXmlDescription = desc; }

        ///
        void setError( const string& errTxt) { myError = errTxt; }

        ///
        void setNumMatches( int num ) { myNumMatches = num; }

        ///
        void setSessionID( const string& token ) { myToken = token; }

        ///
        int getStatus() { return myStatus; }

        ///
        const string& getDomain() { return myDomain; }

        ///
        const string& getError() { return myError; }

        ///
        int getNumMatches() { return myNumMatches; }

        ///
        string getSessionID() { return myToken; }

        /// for single db match
        void setFieldValue( const string& kw, const string& strVal )
             throw (VException&);
  
        /// for multiple db matches
        void setFieldValue( int idx, const string& kw, const string& strVal )
             throw (VException&);
        
        /// return first occurrence of a keyword's value
        void getFieldValue( const string& kw, string& value )
             throw (VException&);

        /** for multiple matches, give an index to myTables, return
         *  first occurrence of a keyword's value
         */
        void getFieldValue( int idx, const string& kw, string& value )
             throw (VException&);

        /// find all values for a given keyword 
        // void getFieldValues( const string& kw, vector<string>& values )
        //      throw (VException&);
        
        /// 
        // void getFieldValues( int idx, const string& kw, vector<string>& values )
        //      throw (VException&);
        
        /// 
        void getFieldValueAsBool( const string& kw, bool& boolVal )
             throw (VException&);

        /// 
        void getFieldValueAsBool( int idx, const string& kw, bool& boolVal )
             throw (VException&);

        ///
        void getFieldValueAsInt( const string& kw, int& intVal )
             throw (VException&);

        ///
        void getFieldValueAsInt( int idx, const string& kw, int& intVal )
             throw (VException&);

        ///
        void getFieldValueAsLong( const string& kw, long int& longVal)
             throw (VException&);

        ///
        void getFieldValueAsLong( int idx, const string& kw,
                                    long int& longVal)
             throw (VException&);

        ///
        void getFieldValueAsDouble( const string& kw, double& longVal)
             throw (VException&);

        ///
        void getFieldValueAsDouble( int idx, const string& kw, double& longVal)
             throw (VException&);

        /// blobType is a 2D array == vector < vector <string> >
        void getFieldValueAsList( const string& kw, blobType& listResult)
             throw (VException&);

        ///
        void getFieldValueAsList( int idx, const string& kw, blobType& listResult)
             throw (VException&);

        /** myTables is a vector of tables, each table stores the
         *  content of a db match
         */
        const tableType& getTable(int idx) const throw (VException&);

        ///
        const FieldListtype& getFieldList()
	{
	    return myFieldList;
	};
        ///
        void addFieldList(const string & val);

        ///
        void reserveNumTables( int size ) { myTables.resize( size ); }

        ///  is myTables an empty set?
        bool isEmptyRecordSet() { return(myTables.size() == 0); }

        ///  is this a result of a session request?
        bool isSession() { return( myToken != "-1" ); }


        /// 
        void print(const int loglevel);


    protected:

        ///
        bool idxInRange( int idx ) const
        { 
            int numMatches = myTables.size();
            if ((idx < 0) || (idx >= numMatches))
            {
                 return false;
            }
            return true;
        }

        ///
        void findField( int idx, const string& kw, TableIter& itr )
                        throw (VException&);

    private:


        ///
        int    myStatus;    // status code from Http response

        ///
        string myDomain;

        ///
        string myError;  // if status code != 200, error reason

        /// 
        int    myNumMatches;

        ///  if -1, then this result set is not assoc'd w/ a session
        string   myToken;

    string myXmlCategory;
    string myXmlDescription;
    
        /** a vector of string - list of all fields
         */
        FieldListtype myFieldList;

        /**  array container used to store values whose
         *   keys are identical
         */
        vector < tableType > myTables;

};

#endif // PSDBRESULT_HXX
