
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const PSMessage_cxx_Version =
    "$Id: PSMessage.cxx,v 1.5 2001/12/11 01:36:50 cktam Exp $";



#include <iostream.h>
#include "PSMessage.hxx"
#include "cpLog.h"

string PSMessage::UNKNOWN_MSG_TYPE_STR("UNKNOWN_MSG_TYPE");
string PSMessage::GET_RECORDS_IN_DOMAIN_REQ_STR("GET_RECORDS_IN_DOMAIN");
string PSMessage::GET_RECORDS_IN_DOMAIN_RSP_STR("GET_RECORDS_IN_DOMAIN RESP");
string PSMessage::SET_RECORDS_IN_DOMAIN_REQ_STR("SET_RECORDS_IN_DOMAIN");
string PSMessage::SET_RECORDS_IN_DOMAIN_RSP_STR("SET_RECORDS_IN_DOMAIN RESP");
string PSMessage::ADD_RECORD_IN_DOMAIN_REQ_STR("ADD_RECORD_IN_DOMAIN");
string PSMessage::ADD_RECORD_IN_DOMAIN_RSP_STR("ADD_RECORD_IN_DOMAIN RESP");
string PSMessage::DEL_RECORDS_IN_DOMAIN_REQ_STR("DEL_RECORDS_IN_DOMAIN");
string PSMessage::DEL_RECORDS_IN_DOMAIN_RSP_STR("DEL_RECORDS_IN_DOMAIN RESP");
string PSMessage::GET_CUR_RECORDS_IN_DOMAIN_REQ_STR("GET_CURRENT_RECORDS_IN_DOMAIN");
string PSMessage::GET_CUR_RECORDS_IN_DOMAIN_RSP_STR("GET_CURRENT_RECORDS_IN_DOMAIN RSP");
string PSMessage::SESSION_OPEN_REQ_STR("SESSION_OPEN");
string PSMessage::SESSION_OPEN_RSP_STR("SESSION_OPEN RESP");
string PSMessage::SESSION_CLOSE_REQ_STR("SESSION_CLOSE");
string PSMessage::SESSION_CLOSE_RSP_STR("SESSION_CLOSE RESP");
string PSMessage::SESSION_SEARCH_DB_REQ_STR("SESSION_SEARCH_DB");
string PSMessage::SESSION_SEARCH_DB_RSP_STR("SESSION_SEARCH_DB RESP");
string PSMessage::SESSION_GET_DB_RESULT_REQ_STR("SESSION_GET_DB_RESULT");
string PSMessage::SESSION_GET_DB_RESULT_RSP_STR("SESSION_GET_DB_RESULT RESP");
string PSMessage::SESSION_SORT_DB_RESULT_REQ_STR("SESSION_SORT_DB_RESULT");
string PSMessage::SESSION_SORT_DB_RESULT_RSP_STR("SESSION_SORT_DB_RESULT RESP");
string PSMessage::SESSION_SET_DB_RESULT_REQ_STR("SESSION_SET_DB_RESULT");
string PSMessage::SESSION_SET_DB_RESULT_RSP_STR("SESSION_SET_DB_RESULT RESP");
string PSMessage::SESSION_DELETE_ALL_REQ_STR("SESSION_DELETE_ALL");
string PSMessage::SESSION_DELETE_ALL_RSP_STR("SESSION_DELETE_ALL RESP");

string PSMessage::REGISTER_DOMAIN_FOR_UPDATE_REQ_STR("REGISTER_DOMAIN_FOR_UPDATE");
string PSMessage::REGISTER_DOMAIN_FOR_UPDATE_RSP_STR("REGISTER_DOMAIN_FOR_UPDATE RESP");
string PSMessage::REGISTER_RECORD_FOR_UPDATE_REQ_STR("REGISTER_RECORD_FOR_UPDATE");
string PSMessage::REGISTER_RECORD_FOR_UPDATE_RSP_STR("REGISTER_RECORD_FOR_UPDATE RESP");
string PSMessage::UPDATE_DOMAIN_REQ_STR("UPDATE_DOMAIN REQ");
string PSMessage::UPDATE_RECORD_REQ_STR("UPDATE_RECORD REQ");

string PSMessage::UN_REGISTER_DOMAIN_FOR_UPDATE_REQ_STR("UN_REGISTER_DOMAIN_FOR_UPDATE");
string PSMessage::UN_REGISTER_DOMAIN_FOR_UPDATE_RSP_STR("UN_REGISTER_DOMAIN_FOR_UPDATE RESP");
string PSMessage::UN_REGISTER_RECORD_FOR_UPDATE_REQ_STR("UN_REGISTER_RECORD_FOR_UPDATE");
string PSMessage::UN_REGISTER_RECORD_FOR_UPDATE_RSP_STR("UN_REGISTER_RECORD_FOR_UPDATE RESP");

string PSMessage::GET_XML_DESC_REQ_STR("GET_XML_DESC");
string PSMessage::GET_XML_DESC_RSP_STR("GET_XML_DESC RESP");
string PSMessage::LIST_XML_CAT_REQ_STR("LIST_XML_CAT");
string PSMessage::LIST_XML_CAT_RSP_STR("LIST_XML_CAT RESP");
string PSMessage::LIST_XML_DESC_REQ_STR("LIST_XML_DESC");
string PSMessage::LIST_XML_DESC_RSP_STR("LIST_XML_DESC RESP");

string PSMessage::ERROR_RSP_STR("ERROR!!");

///
PSMessage::PSMessage() 
{ 
    myMessageType = UNKNOWN_MSG_TYPE; 
    blobFlag = false;
    tagLevel = 0; 
    xmlBlob = "";
}

///
PSMessage::PSMessage( msgType type ) 
{ 
    myMessageType = type; 
    blobFlag = false; 
    tagLevel = 0; 
    xmlBlob = "";
}

///
bool 
PSMessage::isRequest()
{
    if ((myMessageType == GET_RECORDS_IN_DOMAIN_REQ) ||
        (myMessageType == SET_RECORDS_IN_DOMAIN_REQ) ||
        (myMessageType == ADD_RECORD_IN_DOMAIN_REQ)  ||
        (myMessageType == DEL_RECORDS_IN_DOMAIN_REQ)  ||
        (myMessageType == SESSION_OPEN_REQ)  ||
        (myMessageType == SESSION_CLOSE_REQ)  ||
        (myMessageType == SESSION_SEARCH_DB_REQ)  ||
        (myMessageType == SESSION_GET_DB_RESULT_REQ)  ||
        (myMessageType == SESSION_SORT_DB_RESULT_REQ)  ||
        (myMessageType == SESSION_SET_DB_RESULT_REQ)  ||
        (myMessageType == SESSION_DELETE_ALL_REQ) ||
        (myMessageType == REGISTER_DOMAIN_FOR_UPDATE_REQ) ||
        (myMessageType == REGISTER_RECORD_FOR_UPDATE_REQ) ||
        (myMessageType == UN_REGISTER_DOMAIN_FOR_UPDATE_REQ) ||
        (myMessageType == UN_REGISTER_RECORD_FOR_UPDATE_REQ) ||
        (myMessageType == UPDATE_DOMAIN_REQ) ||
        (myMessageType == UPDATE_RECORD_REQ) ||
        (myMessageType == GET_XML_DESC_REQ) ||
        (myMessageType == LIST_XML_CAT_REQ) ||
        (myMessageType == LIST_XML_DESC_REQ) )
    {
         return true;
    }
    return false;   // it's a response
}

///
bool PSMessage::isResponse()
{
    if ( (myMessageType == GET_RECORDS_IN_DOMAIN_RSP) ||
         (myMessageType == SET_RECORDS_IN_DOMAIN_RSP) || 
         (myMessageType == ADD_RECORD_IN_DOMAIN_RSP) || 
         (myMessageType == DEL_RECORDS_IN_DOMAIN_RSP) || 
         (myMessageType == SESSION_OPEN_RSP) || 
         (myMessageType == SESSION_CLOSE_RSP) || 
         (myMessageType == SESSION_SEARCH_DB_RSP) || 
         (myMessageType == SESSION_GET_DB_RESULT_RSP) || 
         (myMessageType == SESSION_SORT_DB_RESULT_RSP) || 
         (myMessageType == SESSION_SET_DB_RESULT_RSP) || 
         (myMessageType == SESSION_DELETE_ALL_RSP) ||
         (myMessageType == REGISTER_DOMAIN_FOR_UPDATE_RSP) ||
         (myMessageType == REGISTER_RECORD_FOR_UPDATE_RSP) ||
         (myMessageType == UN_REGISTER_DOMAIN_FOR_UPDATE_RSP) ||
         (myMessageType == UN_REGISTER_RECORD_FOR_UPDATE_RSP) ||
         (myMessageType == GET_XML_DESC_RSP) ||
         (myMessageType == LIST_XML_CAT_RSP) ||
         (myMessageType == LIST_XML_DESC_RSP) )
    {
         return true;
    }
    return false;

}

///
void
PSMessage::print(const int loglevel)
{
    cpLog(loglevel, "************************************\n"
                    "Message type = %d(%s)\nRaw Content: %s",
                    myMessageType, msgTypeStr(myMessageType),
                    myRawMessage.c_str());

    cpLog(loglevel, "compiled at: %s\n************************************",
           __DATE__ " " __TIME__);
}

const char*
PSMessage::msgTypeStr( msgType typ )
{
    switch (typ)
    {
        case GET_RECORDS_IN_DOMAIN_REQ:
            return GET_RECORDS_IN_DOMAIN_REQ_STR.c_str();
            break;
            
        case GET_RECORDS_IN_DOMAIN_RSP:
            return GET_RECORDS_IN_DOMAIN_RSP_STR.c_str();
            break;
            
        case SET_RECORDS_IN_DOMAIN_REQ:
           return SET_RECORDS_IN_DOMAIN_REQ_STR.c_str();
           break;

        case SET_RECORDS_IN_DOMAIN_RSP:
           return SET_RECORDS_IN_DOMAIN_RSP_STR.c_str();
           break;

        case ADD_RECORD_IN_DOMAIN_REQ:
           return ADD_RECORD_IN_DOMAIN_REQ_STR.c_str();
           break;

        case ADD_RECORD_IN_DOMAIN_RSP:
           return ADD_RECORD_IN_DOMAIN_RSP_STR.c_str();
           break;

        case DEL_RECORDS_IN_DOMAIN_REQ:
           return DEL_RECORDS_IN_DOMAIN_REQ_STR.c_str();
           break;

        case DEL_RECORDS_IN_DOMAIN_RSP:
           return DEL_RECORDS_IN_DOMAIN_RSP_STR.c_str();
           break;

        case SESSION_OPEN_REQ:
           return SESSION_OPEN_REQ_STR.c_str();
           break;

        case SESSION_OPEN_RSP:
           return SESSION_OPEN_RSP_STR.c_str();
           break;

        case SESSION_CLOSE_REQ:
           return SESSION_CLOSE_REQ_STR.c_str();
           break;

        case SESSION_CLOSE_RSP:
           return SESSION_CLOSE_RSP_STR.c_str();
           break;

        case SESSION_SEARCH_DB_REQ:
           return SESSION_SEARCH_DB_REQ_STR.c_str();
           break;
        
        case SESSION_SEARCH_DB_RSP:
           return SESSION_SEARCH_DB_RSP_STR.c_str();
           break;

        case SESSION_GET_DB_RESULT_REQ:
           return SESSION_GET_DB_RESULT_REQ_STR.c_str();
           break;

        case SESSION_GET_DB_RESULT_RSP:
           return SESSION_GET_DB_RESULT_RSP_STR.c_str();
           break;

        case SESSION_SORT_DB_RESULT_REQ:
           return SESSION_SORT_DB_RESULT_REQ_STR.c_str();
           break;

        case SESSION_SORT_DB_RESULT_RSP:
           return SESSION_SORT_DB_RESULT_RSP_STR.c_str();
           break;

        case SESSION_SET_DB_RESULT_REQ:
           return SESSION_SET_DB_RESULT_REQ_STR.c_str();
           break;

        case SESSION_SET_DB_RESULT_RSP:
           return SESSION_SET_DB_RESULT_RSP_STR.c_str();
           break;

        case SESSION_DELETE_ALL_REQ:
           return SESSION_DELETE_ALL_REQ_STR.c_str();
           break;

        case SESSION_DELETE_ALL_RSP:
           return SESSION_DELETE_ALL_RSP_STR.c_str();
           break;

        case GET_XML_DESC_REQ:
            return GET_XML_DESC_REQ_STR.c_str();
            break ;
            
        case GET_XML_DESC_RSP:
            return GET_XML_DESC_RSP_STR.c_str();
            break ;
            
        case LIST_XML_CAT_REQ:
            return LIST_XML_CAT_REQ_STR.c_str();
            break ;
            
        case LIST_XML_CAT_RSP:
            return LIST_XML_CAT_RSP_STR.c_str();
            break ;
            
        case LIST_XML_DESC_REQ:
            return LIST_XML_DESC_REQ_STR.c_str();
            break ;
            
        case LIST_XML_DESC_RSP:
            return LIST_XML_DESC_RSP_STR.c_str();
            break ;

        case REGISTER_DOMAIN_FOR_UPDATE_REQ:
            return REGISTER_DOMAIN_FOR_UPDATE_REQ_STR.c_str();
            break ;

        case REGISTER_RECORD_FOR_UPDATE_REQ:
            return REGISTER_RECORD_FOR_UPDATE_REQ_STR.c_str();
            break ;

        case UN_REGISTER_DOMAIN_FOR_UPDATE_REQ:
            return UN_REGISTER_DOMAIN_FOR_UPDATE_REQ_STR.c_str();
            break ;

        case UN_REGISTER_RECORD_FOR_UPDATE_REQ:
            return UN_REGISTER_RECORD_FOR_UPDATE_REQ_STR.c_str();
            break ;

        case UPDATE_DOMAIN_REQ:
            return UPDATE_DOMAIN_REQ_STR.c_str();
            break ;

        case UPDATE_RECORD_REQ:
            return UPDATE_RECORD_REQ_STR.c_str();
            break ;
 
        case REGISTER_DOMAIN_FOR_UPDATE_RSP:
            return REGISTER_RECORD_FOR_UPDATE_RSP_STR.c_str();
            break ;
 
        case REGISTER_RECORD_FOR_UPDATE_RSP:
            return REGISTER_RECORD_FOR_UPDATE_RSP_STR.c_str();
            break ;

        case UN_REGISTER_DOMAIN_FOR_UPDATE_RSP:
            return UN_REGISTER_DOMAIN_FOR_UPDATE_RSP_STR.c_str();
            break ;

        case UN_REGISTER_RECORD_FOR_UPDATE_RSP:
            return UN_REGISTER_RECORD_FOR_UPDATE_RSP_STR.c_str();
            break ;

        case ERROR_RSP:
            return ERROR_RSP_STR.c_str();
            break;
            
        case UNKNOWN_MSG_TYPE:
        default:
            return UNKNOWN_MSG_TYPE_STR.c_str();
            break;      
    }   // end switch
}

///
void
PSMessage::clearXmlBlob()
{
     blobFlag = false; 
     tagLevel = 0; 
     xmlBlob = ""; 

}




// End of File
