#ifndef PSMESSAGE_HXX 
#define PSMESSAGE_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


#include <cassert>
#include <string>
#include "VException.hxx"
#include "PSDBCommonType.h"


static const char* const PSMessageVersion 
    = "$Id: PSMessage.hxx,v 1.7 2001/12/11 01:36:50 cktam Exp $";

///
class PSMessage
{
    public:

        enum msgType
        {
            UNKNOWN_MSG_TYPE = 0,
            GET_RECORDS_IN_DOMAIN_REQ,
            GET_RECORDS_IN_DOMAIN_RSP,
            GET_FIELDS_IN_DOMAIN_REQ,
            GET_FIELDS_IN_DOMAIN_RSP,
            SET_RECORDS_IN_DOMAIN_REQ,
            SET_RECORDS_IN_DOMAIN_RSP,
            ADD_RECORD_IN_DOMAIN_REQ,
            ADD_RECORD_IN_DOMAIN_RSP,
            DEL_RECORDS_IN_DOMAIN_REQ,
            DEL_RECORDS_IN_DOMAIN_RSP,
            GET_CUR_RECORDS_IN_DOMAIN_REQ,
            GET_CUR_RECORDS_IN_DOMAIN_RSP,
            SESSION_OPEN_REQ,
            SESSION_OPEN_RSP,
            SESSION_CLOSE_REQ,
            SESSION_CLOSE_RSP,
            SESSION_SEARCH_DB_REQ,
            SESSION_SEARCH_DB_RSP,
            SESSION_GET_DB_RESULT_REQ, 
            SESSION_GET_DB_RESULT_RSP, 
            SESSION_SORT_DB_RESULT_REQ,
            SESSION_SORT_DB_RESULT_RSP,
            SESSION_SET_DB_RESULT_REQ,
            SESSION_SET_DB_RESULT_RSP,
            SESSION_DELETE_ALL_REQ,
            SESSION_DELETE_ALL_RSP,
            REGISTER_DOMAIN_FOR_UPDATE_REQ,
            REGISTER_DOMAIN_FOR_UPDATE_RSP,
            REGISTER_RECORD_FOR_UPDATE_REQ,
            REGISTER_RECORD_FOR_UPDATE_RSP,
            UPDATE_DOMAIN_REQ,
            UPDATE_RECORD_REQ,
            UN_REGISTER_DOMAIN_FOR_UPDATE_REQ,
            UN_REGISTER_DOMAIN_FOR_UPDATE_RSP,
            UN_REGISTER_RECORD_FOR_UPDATE_REQ,
            UN_REGISTER_RECORD_FOR_UPDATE_RSP,
            GET_XML_DESC_REQ,
            GET_XML_DESC_RSP,
            LIST_XML_CAT_REQ,
            LIST_XML_CAT_RSP,
            LIST_XML_DESC_REQ,
            LIST_XML_DESC_RSP,
            ERROR_RSP,                        // error message type
            NUM_PS_MSG_TYPE = SESSION_DELETE_ALL_RSP
        };            

        static string UNKNOWN_MSG_TYPE_STR;
        static string GET_RECORDS_IN_DOMAIN_REQ_STR;
        static string GET_RECORDS_IN_DOMAIN_RSP_STR;
        static string SET_RECORDS_IN_DOMAIN_REQ_STR;
        static string SET_RECORDS_IN_DOMAIN_RSP_STR;
        static string ADD_RECORD_IN_DOMAIN_REQ_STR;
        static string ADD_RECORD_IN_DOMAIN_RSP_STR;
        static string DEL_RECORDS_IN_DOMAIN_REQ_STR;
        static string DEL_RECORDS_IN_DOMAIN_RSP_STR;
        static string GET_CUR_RECORDS_IN_DOMAIN_REQ_STR;
        static string GET_CUR_RECORDS_IN_DOMAIN_RSP_STR;
        static string SESSION_OPEN_REQ_STR;
        static string SESSION_OPEN_RSP_STR;
        static string SESSION_CLOSE_REQ_STR;
        static string SESSION_CLOSE_RSP_STR;
        static string SESSION_SEARCH_DB_REQ_STR;
        static string SESSION_SEARCH_DB_RSP_STR;
        static string SESSION_GET_DB_RESULT_REQ_STR;
        static string SESSION_GET_DB_RESULT_RSP_STR;
        static string SESSION_SORT_DB_RESULT_REQ_STR;
        static string SESSION_SORT_DB_RESULT_RSP_STR;
        static string SESSION_SET_DB_RESULT_REQ_STR;
        static string SESSION_SET_DB_RESULT_RSP_STR;
        static string SESSION_DELETE_ALL_REQ_STR;
        static string SESSION_DELETE_ALL_RSP_STR;
        static string REGISTER_DOMAIN_FOR_UPDATE_REQ_STR;
        static string REGISTER_DOMAIN_FOR_UPDATE_RSP_STR;
        static string REGISTER_RECORD_FOR_UPDATE_REQ_STR;
        static string REGISTER_RECORD_FOR_UPDATE_RSP_STR;
        static string UPDATE_DOMAIN_REQ_STR;
        static string UPDATE_RECORD_REQ_STR;
        static string UN_REGISTER_DOMAIN_FOR_UPDATE_REQ_STR;
        static string UN_REGISTER_DOMAIN_FOR_UPDATE_RSP_STR;
        static string UN_REGISTER_RECORD_FOR_UPDATE_REQ_STR;
        static string UN_REGISTER_RECORD_FOR_UPDATE_RSP_STR;

        static string GET_XML_DESC_REQ_STR;
        static string GET_XML_DESC_RSP_STR;
        static string LIST_XML_CAT_REQ_STR;
        static string LIST_XML_CAT_RSP_STR;
        static string LIST_XML_DESC_REQ_STR;
        static string LIST_XML_DESC_RSP_STR;

        static string ERROR_RSP_STR;


        ///
        PSMessage();

        ///
        PSMessage( msgType type );

        ///
        virtual ~PSMessage() {}

        ///
        virtual void parseMessage() throw (VException&) = 0; 

        ///
        void setMessageType( msgType type ) { myMessageType = type; }

        ///
        int  getMessageType() { return myMessageType; }

        ///
        void setMessage( const string& msg ) { myRawMessage = msg; }

        ///
        const string& getMessage() { return myRawMessage; }

        ///  Message Type is not defined yet
        bool isUnknownMsg() 
        {
            if (myMessageType == UNKNOWN_MSG_TYPE)
            {
                return true;
            }
            return false;
        }

        ///
        bool isRequest();

        ///
        bool isResponse();

        ///
        bool isError() {
           if (myMessageType == ERROR_RSP)
           {
               return true;       
           }
           return false;
        }
        
        /// 
        void print(const int loglevel);

        ///
        static const char* msgTypeStr( msgType typ );

        bool blobFlag;
        int tagLevel;

        void setXmlBlob(const string& xml) { xmlBlob += xml; }
        const string& getXmlBlob() { return xmlBlob; }

        void clearXmlBlob();

    protected:

    private:

        ///
        msgType    myMessageType;    // type of message

        ///
        string myRawMessage;  // raw message: http header + xml body

        /// Http header + raw xml content
        // HttpMessage myMessage;

        /// string to hold the xml blob
        string xmlBlob;

};

#endif // PSMESSAGE_HXX
