#ifndef PSPROTOCOLSERVICE_HXX
#define PSPROTOCOLSERVICE_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const PSProtocolService_hxx_Version =
    "$Id: PSProtocolService.hxx,v 1.3 2001/09/19 20:31:22 maivu Exp $";


#include <string>
#include <iostream.h>
#include <stdlib.h>
#include "Sptr.hxx"
#include "VNetworkException.hxx"
#include "HttpMessage.hxx"
#include "PSCommon.hxx"
#include "PSDBResult.hxx"
#include "PSDBRequest.hxx"
#include "PSDBCommonType.h"
#include "PSBlobResult.hxx"


using namespace Vocal;



/** Helper class to psLib, used to build the PS protocol message
 *  using other helper classes, HttpMessage and PSXmlContent.
 *  I am making it a singleton as there is only need for one object
 */
class PSProtocolService 
{ 

    public:


        /// instance method to get a singleton PSProtolService object
        static PSProtocolService& getInstance()
                             throw( VException& );

        /// Destructor - will terminate XMLPlatformUtils here
        virtual ~PSProtocolService();

        ///
        const char* getClassName() const { return "PSProtolService"; }

        ///
        Sptr<PSDBRequest> parseRequest( PSMessage::msgType type,
                                 const string& xmlBody )
                                 throw( VException& );
        ///
        Sptr<PSDBResult> parseResponse( PSMessage::msgType type,
                                 const string& rawHttpMsg )
                                 throw( VException& );

       ///
       Sptr<PSBlobResult> parseXmlBlob(const string& xmlBlob)
           throw (VException&);


        // *****************************************************************
        // Generic atomic data accessor Builder functions
        // *****************************************************************

 	/// retrive an XML description file
	Sptr<Http::Message> buildGetXmlDescriptionReq( const string& xmlCategory, 
                                                   const string& xmlDescription)
	  throw( VException& );

     	/// list all catagories of XML descriptions
	Sptr<Http::Message> buildListXmlCatagoriesReq()
	  throw( VException& );

     	/// list all descrptions available for a given catagory
	Sptr<Http::Message> buildListXmlDescriptionsReq( const string& xmlCategory)
	  throw( VException& );



    
 	/// retrive record(s) matching the domain, filter and fields of interest
	Sptr<Http::Message> buildGetRecordsInDomainReq( const string& domain, 
							const string& filterRules,
							FieldListtype & fieldList)
	  throw( VException& );

        ///
        Sptr<Http::Message> buildGetFieldsInDomainReq( const string& domain )
                                throw( VException& );

		/// update existing records with keyword/value pairs provided
		Sptr<Http::Message> buildSetRecordsInDomainReq( const string& domain, 
                                const string& filterRules, 
						        const map<string, string>& fieldValPairs )
				                throw (VException& );

        /// insert a new record
        Sptr<Http::Message> buildAddRecordInDomainReq( const string& domain, 
                               const map<string, string>& filedValPairs)
                               throw( VException& );

        ///
		Sptr<Http::Message> buildDeleteRecordsInDomainReq( const string& domain,
						           const string& filterRules )
				                   throw( VException& );


        // *****************************************************************
        // Session related Builder functions
        // *****************************************************************

		/** psServer returns -1 if PServer does not return
         *  otherwise, return value is the sessioID
         */
        Sptr<Http::Message> buildSessionOpenReq( int timeInSeconds ) 
                        throw(VException&);

		///
        Sptr<Http::Message> buildSessionCloseReq( const string& sessionID ) 
                        throw( VException& );

        ///
        Sptr<Http::Message> buildSessionSearchDBReq( 
                       const string& sessionID, 
                       const string& domain, const string& filterRules,
                       const FieldListtype & fieldList, 
                       PSCommon::PSSortType sType,
                       const string& sortField ) throw(VException&);

        ///
        Sptr<Http::Message> buildSessionGetDBResultReq( 
                       const string& sessionID, int startIdx,
                       int endIdx, const FieldListtype & fieldList)
                       throw (VException&);

        ///
        Sptr<Http::Message> buildSessionSetDBResultReq( const string& sessionID, 
                       const map< string, string>& fieldValPairs)
                       throw (VException&);

        ///
        Sptr<Http::Message> buildSessionSortDBResultReq( 
                       const string& sessionID, PSCommon::PSSortType sType,
                       const string& sortField)
                       throw (VException&);

        ///
        Sptr<Http::Message> buildSessionDeleteAllReq( const string& sessionID ) 
                       throw (VException&);

        // *****************************************************************
        // Registration related Builder functions
        // *****************************************************************
        ///
        Sptr<Http::Message> buildRegisterForUpdateReq(const string& type,
						      const string& hostport,
						      const string& domain,
						      const vector<string>& keyList,
						      callbackFunc func) 
	  throw (VException&);

        ///
        Sptr<Http::Message> buildUpdateReq(const string& type,
					   const string& domain,
					   const vector<string>& keyList)
	  throw (VException&);

        ///
        Sptr<Http::Message> buildUnRegisterForUpdateReq(const string& type,
							const string& hostport,
							const string& domain,
							const vector<string>& keyList,
							callbackFunc func) 
	  throw (VException&);

    protected:
        /// helper function to assemble a HttpMessage
        Sptr<Http::Message> buildHttpMessage( Http::Type,
                       const string& psReqType,
                       const string& body );
    private:

        /// private constructor - will init XMLPlatformUtils here
        PSProtocolService() {} 

        /// suppress equal operator
        friend bool operator==( const PSProtocolService& lhs,
                                const PSProtocolService& rhs );

        /// suppress copy contructor
        PSProtocolService( const PSProtocolService& rhs );

        /// suppress assignment operator
        PSProtocolService& operator =( const PSProtocolService& rhs );

        static PSProtocolService* myInstance;


};

#endif // PSPROTOCOLSERVICE_HXX

