/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const PSReqXmlContentHandlersrCxxVersion = "$Id: PSReqXmlContentHandlers.cxx,v 1.5 2001/12/28 17:25:45 jason Exp $";


// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <stdio.h>
#include <util/XMLUniDefs.hpp>

#include "cpLog.h"

#include "PSCommon.hxx"
#include "XmlContent.hxx"
#include "PSReqXmlContentHandlers.hxx"
#include "VNetworkException.hxx"


// ---------------------------------------------------------------------------
//  PSReqXmlContentHandlers: Constructors
// ---------------------------------------------------------------------------

PSReqXmlContentHandlers::PSReqXmlContentHandlers(Sptr<PSDBRequest> target)
  : myCharacters("")
{
    cpLog(LOG_DEBUG, "PSReqXmlContentHandlers constructor invoked ");
    myParsedContent = target;    // store parsed xml content here

}

// ---------------------------------------------------------------------------
//  PSReqXmlContentHandlers: Destructor
// ---------------------------------------------------------------------------
PSReqXmlContentHandlers::~PSReqXmlContentHandlers()
{
    cpLog(LOG_DEBUG, "PSReqXmlContentHandlers destructor invoked ");
}


// ---------------------------------------------------------------------------
//  PSReqXmlContentHandlers: Overrides of the SAX ErrorHandler interface
// ---------------------------------------------------------------------------
void PSReqXmlContentHandlers::error(const SAXParseException& e)
{
    cpLog(LOG_ERR, "\nSAX Handler throws Error. File %s, line %d, char %d"
                     "\n   Message: %s",
          (StrX(e.getSystemId()), e.getLineNumber(),
          e.getColumnNumber(), StrX(e.getMessage())).localForm());

    string err = "Catching XML Exception: ";
    err += StrX(e.getMessage()).localForm();

    VNetworkException exception(err.c_str(), __FILE__, __LINE__);
    throw exception;

}

void PSReqXmlContentHandlers::fatalError(const SAXParseException& e)
{
    cpLog( LOG_ERR, "\nSAX Handler throws Fatal Error. File %s, line %d, char %d"
                      "\n   Message: %s",
           (StrX(e.getSystemId()), e.getLineNumber(),
            e.getColumnNumber(), StrX(e.getMessage()) ).localForm());

    string err = "Catching XML Exception: ";
    err += StrX(e.getMessage()).localForm();

    VNetworkException exception(err.c_str(), __FILE__, __LINE__);
    throw exception;

}

void PSReqXmlContentHandlers::warning(const SAXParseException& e)
{
    cpLog( LOG_WARNING, "\nSAX Handler throws Warning. File %s, line %d, chr %d"
                      "\n    Message: %s",
           (StrX(e.getSystemId()), e.getLineNumber(),
           e.getColumnNumber(), StrX(e.getMessage()) ).localForm());

    string err = "Catching XML Exception: ";
    err += StrX(e.getMessage()).localForm();

    VNetworkException exception(err.c_str(), __FILE__, __LINE__);
    throw exception;

}


// ---------------------------------------------------------------------------
//  PSReqXmlContentHandlers: Overrides of the SAX DTDHandler interface
// ---------------------------------------------------------------------------
void PSReqXmlContentHandlers::unparsedEntityDecl(const     XMLCh* const name
                                          , const   XMLCh* const publicId
                                          , const   XMLCh* const systemId
                                          , const   XMLCh* const notationName)
{
    // Not using DTD presently
}


void PSReqXmlContentHandlers::notationDecl(const   XMLCh* const name
                                    , const XMLCh* const publicId
                                    , const XMLCh* const systemId)
{
    // Not using DTD presently
}


// ---------------------------------------------------------------------------
//  PSReqXmlContentHandlers: Overrides of the SAX DocumentHandler interface
// ---------------------------------------------------------------------------
void PSReqXmlContentHandlers::characters(const     XMLCh* const    chars,
                                    const   unsigned int    length)
{
    string& fn = myFields.top();   // get current field name
    string  val= StrX( chars ).localForm();
    myCharacters = val;

    cpLog(LOG_DEBUG, "  ****PSReqXmlContentHandlers::characters = %s len = %d",
          val.c_str(), length);

    if (myParsedContent->blobFlag == true)
      {
	cpLog (LOG_DEBUG, "Add the characters into xmlBlob");
	myParsedContent->setXmlBlob(val);
	//cpLog (LOG_DEBUG, "Blob Xml = %s", myParsedContent->getXmlBlob().c_str());
	return;
      }

    if (fn == PSCommon::ARGUMENTS_STR)
    {
	myParsedContent->setArguments(val);
    }
    else if ((fn == PSCommon::FIELDVALUE_STR) ||
	     (fn == PSCommon::FIELDLIST_STR))
    {
        myParsedContent->setKeywordFlag(fn, val);
	myParsedContent->tagLevel = 1;
    }
    else if (fn == PSCommon::DOMAIN_STR)
    {
        myParsedContent->setDomain( val );
        cpLog( LOG_DEBUG, "domain found: %s", val.c_str());
    }
    else if (fn == PSCommon::FILTER_STR)
    {
        myParsedContent->setFilter( val );
        cpLog( LOG_DEBUG, "filter rules = (%s)", val.c_str() );
    }
    else if (fn == PSCommon::TOKEN_STR)
    {
         myParsedContent->setSessionID( val );
         cpLog( LOG_DEBUG, "session token id = %s", val.c_str() );
    }
    else if (fn == PSCommon::LISTITEM_STR)
    {
        myParsedContent->addFieldList(val); // add to end of the fieldlist
	cpLog( LOG_DEBUG, "List item = %s", val.c_str());
    }
    else if (fn == PSCommon::XMLCATEGORY_STR)
    {
        myParsedContent->setXmlCategory( val );
        cpLog( LOG_DEBUG, "Xml category found: %s", val.c_str());
    }
    else if (fn == PSCommon::XMLDESC_STR)
    {
        myParsedContent->setXmlDescName( val );
        cpLog( LOG_DEBUG, "Xml description name found: %s", val.c_str());
    }
    else if (fn == PSCommon::PRIMARYKEY_STR)
    {
        myParsedContent->addFieldList( val );
        cpLog( LOG_DEBUG, "primary key = %s", val.c_str());
    }    
    else if (fn == PSCommon::REGISTER_HOSTPORT_STR)
    {
        myParsedContent->setRegisterHostport(val);
	cpLog (LOG_DEBUG, "HostPort = %s", val.c_str());
    }
    else // anything else, stuff it into myParsedContent's list of kv pairs
    {
        myParsedContent->setFieldValue(fn, val);
        cpLog( LOG_DEBUG, "added %s/%s to table", 
               fn.c_str(), val.c_str());
     }
    
    cpLog(LOG_DEBUG, "**** end characters ");
}


void PSReqXmlContentHandlers::endDocument()
{
    cpLog( LOG_DEBUG, "endDocument() reached" );

    if (myFields.empty() == false)   // stack should be empty by now
    {
        // this should not happen
        cpLog( LOG_ERR, "myFields should be empty!");
    }
}


void PSReqXmlContentHandlers::endElement(const XMLCh* const uri, 
								 const XMLCh* const localname, 
								 const XMLCh* const qname)
{
    string fn = StrX( qname ).localForm();
    cpLog( LOG_DEBUG, "***** endElement qname = %s", fn.c_str() );
    myFields.pop();      

    myParsedContent->tagLevel--;

    if (myParsedContent->blobFlag == true) 
    {
	if (myParsedContent->tagLevel == 1)
	{
	    myParsedContent->setFieldValue(fn, myParsedContent->getXmlBlob());
	    cpLog(LOG_DEBUG, "Final add fieldname [%s] with blob [%s] into table",
		  fn.c_str(), myParsedContent->getXmlBlob().c_str());

	    myParsedContent->clearXmlBlob();
	    //myParsedContent->blobState = false;
	    //myParsedContent->tagLevel = 0;
	    //myParsedContent->xmlBlob = "";
	    return;
	}
	//cpLog (LOG_DEBUG, "Add end of element into xmlBlob: %s", fn.c_str());
	string temp = PSCommon::END_ELEMENT_OPEN_BRACKET + fn + PSCommon::CLOSE_BRACKET;
	myParsedContent->setXmlBlob(temp);
	//cpLog (LOG_DEBUG, "Xml Blob = %s", myParsedContent->getXmlBlob().c_str());
    }
    else if (myCharacters.empty())
    {
	cpLog(LOG_DEBUG, "added field [%s] (with empty value) to talbe", fn.c_str());
	myParsedContent->setFieldValue(fn, "");
    }

}


void PSReqXmlContentHandlers::ignorableWhitespace( const   XMLCh* const chars
                                            ,const  unsigned int length)
{
    cpLog( LOG_DEBUG, "*********  ignorableWhitespace() called " );
}



void PSReqXmlContentHandlers::startDocument()
{
    cpLog( LOG_DEBUG,  "startDocument() ************ ");

}


void PSReqXmlContentHandlers::startElement(const   XMLCh* const    uri,
					   const   XMLCh* const    localname,
					   const   XMLCh* const    qname,
					   const   Attributes&		attributes)
{
    string fn = StrX(qname).localForm();  // field name
    cpLog( LOG_DEBUG, " *** startElement qname : %s", fn.c_str() );
    myFields.push( fn );   // remember it on the stack
    myCharacters = "";

    if (myParsedContent->tagLevel)
    {
	myParsedContent->tagLevel++;
	//cpLog (LOG_DEBUG, "BLOB Counter = %d", myParsedContent->blobCounter);
	if (myParsedContent->tagLevel > 2)
	{
	    myParsedContent->blobFlag = true;
	}
    }

    if (myParsedContent->blobFlag == true)
    {
	cpLog (LOG_DEBUG, "Add start of element into xmlBlob: %s", fn.c_str());
	string temp = PSCommon::OPEN_BRACKET + fn + PSCommon::CLOSE_BRACKET;
	myParsedContent->setXmlBlob(temp);
	//cpLog (LOG_DEBUG, "Xml Blob = %s", myParsedContent->getXmlBlob().c_str());
    }

}


// *****************************************************************
// Single Transaction XML Content Builder
// *****************************************************************

Sptr<string> PSReqXmlContentHandlers::buildGetXmlDescriptionReq(
                                const string& xmlCategory, 
                                const string& xmlDescription)
						        throw( VException& ) 
{
    cpLog(LOG_DEBUG, "buildGetXMLDescriptionReq() catagory %s, description %s",
                     xmlCategory.c_str(), xmlDescription.c_str());

    /* build the XML content
     */ 

    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
    PSCommon::xmlAddTagValue( PSCommon::XMLCATEGORY_START, xmlCategory, 
			      PSCommon::XMLCATEGORY_END, content );
    PSCommon::xmlAddTagValue( PSCommon::XMLDESC_START, xmlDescription, 
			      PSCommon::XMLDESC_END, content );
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    body->assign( content );
    
    cpLog(LOG_DEBUG, "GetXmlDescription xml body: %s", body->c_str());
    return body;
}

Sptr<string> PSReqXmlContentHandlers::buildListXmlCatagoriesReq()
						        throw( VException& ) 
{
    cpLog(LOG_DEBUG, "buildListXMLCatagoriesReq()");
    
    /* build the XML content
     */ 

    Sptr<string> body = new string();
    // this message has an empty body
    body->assign("");
    
    cpLog(LOG_DEBUG, "ListXmlCatagories");
    return body;
}

Sptr<string> PSReqXmlContentHandlers::buildListXmlDescriptionsReq(
    const string& xmlCategory) 
						        throw( VException& ) 
{
    cpLog(LOG_DEBUG, "buildListXMLDescriptionsReq() catagory %s",
                     xmlCategory.c_str());

    /* build the XML content
     */ 

    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
    PSCommon::xmlAddTagValue( PSCommon::XMLCATEGORY_START, xmlCategory, 
			      PSCommon::XMLCATEGORY_END, content );
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    body->assign( content );
    
    cpLog(LOG_DEBUG, "ListXmlDescriptions xml body: %s", body->c_str());
    return body;
}

Sptr<string> PSReqXmlContentHandlers::buildGetRecordsInDomainReq(
                                const string& domain, 
                                const string& filterRules,
                                const FieldListtype & fieldList)
						        throw( VException& ) 
{
    cpLog(LOG_DEBUG, "buildGetRecordsInDomainReq() domain %s, rules %s",
                     domain.c_str(), filterRules.c_str());

    /* build the XML content
     */ 

    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
    PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain, 
			      PSCommon::DOMAIN_END, content );
    if (filterRules.size() != 0)
    {
	PSCommon::xmlAddTagValue( PSCommon::FILTER_START, filterRules,
				  PSCommon::FILTER_END, content );
    }
    if (fieldList.size())
    {
	PSCommon::xmlAddTag( PSCommon::FIELDLIST_START, content );

	for (unsigned short u = 0; u < fieldList.size(); u++)
	{
	    PSCommon::xmlAddTagValue( PSCommon::LISTITEM_START,
				      fieldList[u], PSCommon::LISTITEM_END, content );
	} 
	PSCommon::xmlAddTag( PSCommon::FIELDLIST_END, content );
	
    }
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    body->assign( content );
    
    cpLog(LOG_DEBUG, "GetRecordsInDomain xml body: %s", body->c_str());
    return body;
}

///
Sptr<string> PSReqXmlContentHandlers::buildGetFieldsInDomainReq( 
                                const string& domain )
                                throw( VException& ) 
{
    cpLog(LOG_DEBUG, "buildGetFieldsInDomainReq() domain %s",
                     domain.c_str());

    /* build the XML content
     */
    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
    PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain,
			      PSCommon::DOMAIN_END, content );
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    body->assign( content );

    cpLog(LOG_DEBUG, "GetFieldsInDomain xml body: %s", body->c_str());
    return body;
}

/// update existing records with keyword/value pairs provided
Sptr<string> 
PSReqXmlContentHandlers::buildSetRecordsInDomainReq( 
                                const string& domain, 
                                const string& filter,
				const map<string, string>& fieldValPairs )
				                throw (VException& )
{
    cpLog(LOG_DEBUG, "buildSetRecordsInDomainReq() domain %s",
                     domain.c_str());

    /* build the XML content
     */
    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
    PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain,
			      PSCommon::DOMAIN_END, content );
    if (filter.size() != 0)
    {
	PSCommon::xmlAddTagValue( PSCommon::FILTER_START, filter,
				  PSCommon::FILTER_END, content );
    }
    if (fieldValPairs.size())
    {
	string startField, endField;
	
	PSCommon::xmlAddTag( PSCommon::FIELDVALUE_START, content );
	for ( map<string, string>::const_iterator itr = 
		fieldValPairs.begin();
	      itr != fieldValPairs.end(); itr++ )
	{
	    startField = PSCommon::OPEN_BRACKET + (*itr).first.c_str() +
	      PSCommon::CLOSE_BRACKET;
	    endField = PSCommon::END_ELEMENT_OPEN_BRACKET +
	      (*itr).first.c_str() + 
	      PSCommon::CLOSE_BRACKET;
	    PSCommon::xmlAddTagValue( startField, (*itr).second,
				      endField, content);
	}
	PSCommon::xmlAddTag( PSCommon::FIELDVALUE_END, content );
	
    }
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    body->assign( content );

    cpLog(LOG_DEBUG, "SetRecordsInDomain xml body: %s", body->c_str());
    return body;

}

/// insert a new record
Sptr<string> 
PSReqXmlContentHandlers::buildAddRecordInDomainReq( const string& domain, 
                               const map< string, string>& fieldValPairs)
                               throw( VException& ) 
{
    cpLog(LOG_DEBUG, "AddRecordsInDomain reuses SetRecordsInDomainReq()");

    string nullFilter = "";
    return ( buildSetRecordsInDomainReq( domain, nullFilter, fieldValPairs ) );
}

Sptr<string>
PSReqXmlContentHandlers::buildDeleteRecordsInDomainReq( const string& domain,
						           const string& filterRules )
				                   throw( VException& ) 
{
    cpLog(LOG_DEBUG, "buildDeleteRecordsInDomainReq, domain %s, filter %s",
                     domain.c_str(), filterRules.c_str() );
    /* build the XML content
     */

    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
    PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain,
			      PSCommon::DOMAIN_END, content );
    if (filterRules.size() != 0)
    {
	PSCommon::xmlAddTagValue( PSCommon::FILTER_START, filterRules,
				  PSCommon::FILTER_END, content );
    }
    // if not filter rules, then delete the entire table
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    body->assign( content );
    
    cpLog(LOG_DEBUG, "DeleteRecordsInDomain xml body: %s", body->c_str());
    return body;
}


// *****************************************************************
// Session related XML Content Builder
// *****************************************************************

Sptr<string> 
PSReqXmlContentHandlers::buildSessionOpenReq( int timeInSeconds ) 
                         throw(VException&)
{ 
    cpLog(LOG_DEBUG, "buildSessionOpenReq() time:  %d seconds",
                     timeInSeconds);

    /* build the XML content
     */
    Sptr<string> body = new string();
    string content;

    char buf[20];
    sprintf( buf, "%d", timeInSeconds );
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
    PSCommon::xmlAddTagValue( PSCommon::EXPIRETIME_START, buf,
			      PSCommon::EXPIRETIME_END, content );
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    body->assign( content );

    cpLog(LOG_DEBUG, "SessionOpen xml body: %s", body->c_str());
    return body;
}
         

///
Sptr<string>
PSReqXmlContentHandlers::buildSessionCloseReq( const string& sessionID ) throw( VException& )
{ 
    cpLog(LOG_DEBUG, "buildSessionCloseReq() token:  %s ",
                     sessionID.c_str());

    /* build the XML content
     */
    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
    PSCommon::xmlAddTagValue( PSCommon::TOKEN_START, sessionID.c_str(),
			      PSCommon::TOKEN_END, content );
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    body->assign( content );

    cpLog(LOG_DEBUG, "SessionOpen xml body: %s", body->c_str());

    return body; 
}

///
Sptr<string> 
PSReqXmlContentHandlers::buildSessionSearchDBReq( 
                       const string& sessionID, 
                       const string& domain, const string& filterRules,
                       const FieldListtype& fieldList,  
                       PSCommon::PSSortType sType,
                       const string& sortField ) throw(VException&)
{

    cpLog(LOG_DEBUG, "buildSessionSearchDBReq() token:  %s \n" 
                     "doamin=%s, filter=%s, sortType=%d, sortField=%s",
                     sessionID.c_str(), domain.c_str(), filterRules.c_str(), 
                     (int)sType, sortField.c_str());

    /* build the XML content
     */
    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
    PSCommon::xmlAddTagValue( PSCommon::TOKEN_START, sessionID.c_str(),
			      PSCommon::TOKEN_END, content );
    PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain,
			      PSCommon::DOMAIN_END, content );
    if (filterRules.size() != 0)
    {
	PSCommon::xmlAddTagValue( PSCommon::FILTER_START, filterRules,
				  PSCommon::FILTER_END, content );
    }
    if (fieldList.size())
    {
	PSCommon::xmlAddTag( PSCommon::FIELDLIST_START, content );
	for ( unsigned short i = 0; i < fieldList.size(); i++)
	{
	    PSCommon::xmlAddTagValue( PSCommon::LISTITEM_START,
				      fieldList[i], PSCommon::LISTITEM_END, content );
	}
	PSCommon::xmlAddTag( PSCommon::FIELDLIST_END, content );
    }
    
    if (sType == PSCommon::SORT_ASCENDING)
    {
        PSCommon::xmlAddTagValue( PSCommon::SORTTYPE_START,
				  PSCommon::SORT_ASCEND_STR, PSCommon::SORTTYPE_END, content );
    }
    else if (sType == PSCommon::SORT_DESCENDING)
    {
        PSCommon::xmlAddTagValue( PSCommon::SORTTYPE_START,
				  PSCommon::SORT_DESCEND_STR, PSCommon::SORTTYPE_END, content );
    }
    else
    {
        cpLog (LOG_DEBUG, "No sorting type selected.  Default = \"ascending\"");
        PSCommon::xmlAddTagValue( PSCommon::SORTTYPE_START,
				  PSCommon::SORT_ASCEND_STR, PSCommon::SORTTYPE_END, content );
    }


    if (sortField.size() != 0)
    {
	PSCommon::xmlAddTagValue( PSCommon::SORTFIELD_START,
				  sortField, PSCommon::SORTFIELD_END, content );
    }
        
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    body->assign( content );
    
    cpLog(LOG_DEBUG, "SessionSearchDB xml body: %s", body->c_str());

    return body;

}

///
Sptr<string>
PSReqXmlContentHandlers::buildSessionGetDBResultReq( 
                       const string& sessionID, int startIdx,
                       int endIdx, const FieldListtype& fieldList)
                       throw (VException&)
{
    cpLog(LOG_DEBUG, "buildSessionGetDBResultReq() token:  %s \n"
                     "startIdx=%d, endIdx=%d",
                     sessionID.c_str(), startIdx, endIdx);

    /* build the XML content
     */
    Sptr<string> body = new string();
    string content;

    char buf[20];
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
    PSCommon::xmlAddTagValue( PSCommon::TOKEN_START, sessionID.c_str(),
			      PSCommon::TOKEN_END, content );
    sprintf( buf, "%d", startIdx );
    PSCommon::xmlAddTagValue( PSCommon::START_START, buf,
			      PSCommon::START_END, content );
    sprintf( buf, "%d", endIdx );
    PSCommon::xmlAddTagValue( PSCommon::END_START, buf,
			      PSCommon::END_END, content );
    
    if (fieldList.size())
    {
	PSCommon::xmlAddTag( PSCommon::FIELDLIST_START, content );
	for ( unsigned short i = 0; i < fieldList.size(); i++)
	{
	    PSCommon::xmlAddTagValue( PSCommon::LISTITEM_START,
				      fieldList[i], PSCommon::LISTITEM_END, content );
	}
	PSCommon::xmlAddTag( PSCommon::FIELDLIST_END, content );
    }
    
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    body->assign( content );

    cpLog(LOG_DEBUG, "SessionGetDBResult xml body: %s", body->c_str());

    return body;

}

///
Sptr<string>
PSReqXmlContentHandlers::buildSessionSetDBResultReq( 
                       const string& sessionID, 
                       const map< string, string>& fieldValPairs)
                       throw (VException&)
{
    cpLog(LOG_DEBUG, "buildSessionSetDBReq() sessionID %s", sessionID.c_str());

    /* build the XML content
     */
    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
    PSCommon::xmlAddTagValue( PSCommon::TOKEN_START, sessionID.c_str(),
			      PSCommon::TOKEN_END, content );
    
    if (fieldValPairs.size())
    {
        string startField, endField;
	
	PSCommon::xmlAddTag( PSCommon::FIELDVALUE_START, content );
	for ( map<string, string>::const_iterator itr =
		fieldValPairs.begin();
	      itr != fieldValPairs.end(); itr++ )
	{
	    startField = PSCommon::OPEN_BRACKET + (*itr).first.c_str() +
	      PSCommon::CLOSE_BRACKET;
	    endField = PSCommon::END_ELEMENT_OPEN_BRACKET +
	      (*itr).first.c_str() +
	      PSCommon::CLOSE_BRACKET;
	    PSCommon::xmlAddTagValue( startField, (*itr).second,
				      endField, content);
	}
	PSCommon::xmlAddTag( PSCommon::FIELDVALUE_END, content );
	
    }
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    body->assign( content );

    cpLog(LOG_DEBUG, "SessionSetDBResult xml body: %s", body->c_str());
    return (body);

}

///
Sptr<string>
PSReqXmlContentHandlers::buildSessionSortDBResultReq( 
                       const string& sessionID, PSCommon::PSSortType sType,
                       const string& sortField)
                       throw (VException&) 
{
    cpLog(LOG_DEBUG, "buildSessionSortDBResultReq() token:  %s \n"
                     "sortType=%d, sortField=%s",
                     sessionID.c_str(), (int)sType, sortField.c_str());

    /* build the XML content
     */
    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
    PSCommon::xmlAddTagValue( PSCommon::TOKEN_START, sessionID.c_str(),
			      PSCommon::TOKEN_END, content );
    if (sType == PSCommon::SORT_ASCENDING)
    {
        PSCommon::xmlAddTagValue( PSCommon::SORTTYPE_START,
				  PSCommon::SORT_ASCEND_STR, PSCommon::SORTTYPE_END, content );
    }
    else if (sType == PSCommon::SORT_DESCENDING)
    {
        PSCommon::xmlAddTagValue( PSCommon::SORTTYPE_START,
				  PSCommon::SORT_DESCEND_STR, PSCommon::SORTTYPE_END, content );
    }
    if (sortField.size() != 0)
    {
	PSCommon::xmlAddTagValue( PSCommon::SORTFIELD_START,
				  sortField, PSCommon::SORTFIELD_END, content );
    }
    
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    body->assign( content );
    
    cpLog(LOG_DEBUG, "SessionSetDBResult xml body: %s", body->c_str());

    return body;

}

///
Sptr<string>
PSReqXmlContentHandlers::buildSessionDeleteAllReq( const string& sessionID ) 
                       throw (VException&) 
{
    cpLog(LOG_DEBUG, "buildSessionDeleteAllReq() token:  %s ",
                     sessionID.c_str());

    /* build the XML content
     */
    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_START, content );
    PSCommon::xmlAddTagValue( PSCommon::TOKEN_START, sessionID.c_str(),
			      PSCommon::TOKEN_END, content );
    PSCommon::xmlAddTag( PSCommon::ARGUMENTS_END, content );
    body->assign( content );

    cpLog(LOG_DEBUG, "SessionDeleteAll xml body: %s", body->c_str());

    return body;

}

// *****************************************************************
// Registration Content Builder
// *****************************************************************
///
Sptr<string>
PSReqXmlContentHandlers::buildRegisterForUpdateReq (const string& hostport,
						    const string& domain,
						    const vector<string> keyList,
						    callbackFunc func)
  throw (VException&)
{
    cpLog (LOG_DEBUG, "buildRegisterForUpdate() hostport = %s, domain = %s",
	   hostport.c_str(), domain.c_str());

    /* build the XML content
     */

    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag (PSCommon::ARGUMENTS_START, content);
    PSCommon::xmlAddTagValue(PSCommon::REGISTER_HOSTPORT_START, hostport.c_str(),
			     PSCommon::REGISTER_HOSTPORT_END, content);
    PSCommon::xmlAddTagValue(PSCommon::DOMAIN_START, domain.c_str(),
			     PSCommon::DOMAIN_END, content);

    for (unsigned short i = 0; i < keyList.size(); i++)
      {
	PSCommon::xmlAddTagValue(PSCommon::PRIMARYKEY_START, keyList[i].c_str(),
				 PSCommon::PRIMARYKEY_END, content);
      }

    PSCommon::xmlAddTag (PSCommon::ARGUMENTS_END, content);

    body->assign(content);

    cpLog (LOG_DEBUG, "buildRegisterForUpdateReq xml body: %s",
	   body->c_str());

    return body;
}

///
Sptr <string> 
PSReqXmlContentHandlers::buildUpdateReq (const string& domain,
					 const vector<string>& keyList)
  throw (VException&)
{
    cpLog (LOG_DEBUG, "buildUpdateReq, domain = %s", domain.c_str());

    /* build the XML content
     */
    Sptr <string> body = new string();
    string content, startTag, endTag;

    PSCommon::xmlAddTag(PSCommon::RESPONSE_START, content);
    PSCommon::xmlAddTagValue(PSCommon::DOMAIN_START, domain.c_str(),
			     PSCommon::DOMAIN_END, content);

    PSCommon::xmlAddTag(PSCommon::MATCH_START, content);
    
    for (unsigned short i = 0; i < keyList.size(); i++)
    {
        PSCommon::xmlAddTagValue(PSCommon::PRIMARYKEY_START, keyList[i].c_str(),
				 PSCommon::PRIMARYKEY_END, content);
    }
    PSCommon::xmlAddTag(PSCommon::MATCH_END, content);
    PSCommon::xmlAddTag(PSCommon::RESPONSE_END, content);

    body->assign(content);
    cpLog (LOG_DEBUG, "buildUpdateReq - xml body = %s", body->c_str());

    return body;

}

///
Sptr<string>
PSReqXmlContentHandlers::buildUnRegisterForUpdateReq (const string& hostport,
						      const string& domain,
						      const vector<string> keyList,
						      callbackFunc func)
  throw (VException&)
{
    cpLog (LOG_DEBUG, "buildUnRegisterForUpdate() hostport = %s, domain = %s",
	   hostport.c_str(), domain.c_str());

    /* build the XML content
     */

    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag (PSCommon::ARGUMENTS_START, content);
    PSCommon::xmlAddTagValue(PSCommon::REGISTER_HOSTPORT_START, hostport.c_str(),
			     PSCommon::REGISTER_HOSTPORT_END, content);
    PSCommon::xmlAddTagValue(PSCommon::DOMAIN_START, domain.c_str(),
			     PSCommon::DOMAIN_END, content);

    for (unsigned short i = 0; i < keyList.size(); i++)
      {
	PSCommon::xmlAddTagValue(PSCommon::PRIMARYKEY_START, keyList[i].c_str(),
				 PSCommon::PRIMARYKEY_END, content);
      }

    PSCommon::xmlAddTag (PSCommon::ARGUMENTS_END, content);

    body->assign(content);

    cpLog (LOG_DEBUG, "buildUnRegisterForUpdateReq xml body: %s",
	   body->c_str());

    return body;
}

