#ifndef PSREQXMLCONTENTHANDLERS_HXX
#define PSREQXMLCONTENTHANDLERS_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const PSReqXmlContentHandlersHxxVersion = "$Id: PSReqXmlContentHandlers.hxx,v 1.2 2001/09/19 20:31:26 maivu Exp $";


#include <string>
#include <stack>
#include    <sax2/DefaultHandler.hpp>

#include "XmlContent.hxx"
#include "Sptr.hxx"
#include "VException.hxx"
#include "PSDBRequest.hxx"
#include "PSDBCommonType.h"

///
class PSReqXmlContentHandlers : public DefaultHandler
{
public:

    /**-----------------------------------------------------------------------
     *  Constructors
     * -----------------------------------------------------------------------
     */
    PSReqXmlContentHandlers() {}
    ///
    PSReqXmlContentHandlers(Sptr<PSDBRequest> target);

    /**-----------------------------------------------------------------------
     *  Destructors
     * -----------------------------------------------------------------------
     */
    virtual ~PSReqXmlContentHandlers();


    // -----------------------------------------------------------------------
    //  Implementations of the SAX DocumentHandler interface
    // -----------------------------------------------------------------------
    ///
    void endDocument();

    ///
    void endElement( const XMLCh* const uri, 
					 const XMLCh* const localname, 
					 const XMLCh* const qname);

    ///
    void characters(const XMLCh* const chars, const unsigned int length);

    ///
    void ignorableWhitespace
    (
        const   XMLCh* const    chars
        , const unsigned int    length
    );

    ///
    void startDocument();

    ///
    void startElement(	const   XMLCh* const    uri,
						const   XMLCh* const    localname,
						const   XMLCh* const    qname,
					    const   Attributes&		attributes);



    // -----------------------------------------------------------------------
    //  Implementations of the SAX ErrorHandler interface
    // -----------------------------------------------------------------------
    ///
    void warning(const SAXParseException& exception);
    ///
    void error(const SAXParseException& exception);
    ///
    void fatalError(const SAXParseException& exception);



    // -----------------------------------------------------------------------
    //  Implementation of the SAX DTDHandler interface
    // -----------------------------------------------------------------------
    ///
    void notationDecl
    (
        const   XMLCh* const    name
        , const XMLCh* const    publicId
        , const XMLCh* const    systemId
    );  // not used

    ///
    void unparsedEntityDecl
    (
        const   XMLCh* const    name
        , const XMLCh* const    publicId
        , const XMLCh* const    systemId
        , const XMLCh* const    notationName
    );  // not used

    ///  set a reference to the target where parsed content will be kept
    void setParsedContentTarget( Sptr<PSDBRequest> target )
    {
        myParsedContent = target;
    }

    ///
    Sptr< PSDBRequest >& getParsedContentTarget() { return myParsedContent; }


    // -----------------------------------------------------------------------
    //  Implementation of PS Request XML protocol builder interface
    // -----------------------------------------------------------------------

    ///
    Sptr< string > buildGetXmlDescriptionReq(
                                const string& xmlCategory,
                                const string& xmlDescription)
                                throw( VException& );

        ///
    Sptr< string > buildListXmlCatagoriesReq()
                                throw( VException& );

        ///
    Sptr< string > buildListXmlDescriptionsReq(
        const string& xmlCategory)
                                throw( VException& );


///
    Sptr< string > buildGetRecordsInDomainReq(
                                const string& domain,
                                const string& filterRules,
                                const FieldListtype & fieldList)
                                throw( VException& );



    ///
    Sptr< string > buildGetFieldsInDomainReq( const string& domain )
                                throw( VException& );

    /// update existing records with keyword/value pairs provided
	Sptr< string > buildSetRecordsInDomainReq( const string& domain, 
                                const string& filter,
						        const map<string, string>& fieldValPairs )
				                throw (VException& );

    /// insert a new record
    Sptr< string > buildAddRecordInDomainReq( const string& domain, 
                               const map<string, string>& filedValPairs)
                               throw( VException& );

    ///
	Sptr< string > buildDeleteRecordsInDomainReq( const string& domain,
						           const string& filterRules )
				                   throw( VException& );

    ///
    Sptr< string > buildSessionOpenReq( int timeInSeconds ) 
                        throw(VException&);

	///
    Sptr< string > buildSessionCloseReq( const string& sessionID ) 
                        throw( VException& );

    ///
    Sptr< string > buildSessionSearchDBReq( 
                       const string& sessionID, const string& domain,
                       const string& filterRules,
                       const FieldListtype & fieldList, 
                       PSCommon::PSSortType sType,
                       const string& sortField ) throw(VException&);

    ///
    Sptr< string > buildSessionGetDBResultReq( 
                       const string& sessionID, int startIdx,
                       int endIdx, const FieldListtype & fieldList)
                       throw (VException&);

    ///
    Sptr< string > buildSessionSetDBResultReq( const string& sessionID, 
                       const map< string, string>& fieldValPairs)
                       throw (VException&);

    ///
    Sptr< string > buildSessionSortDBResultReq( 
                       const string& sessionID, PSCommon::PSSortType sType,
                       const string& sortField)
                       throw (VException&);

    ///
    Sptr< string > buildSessionDeleteAllReq( const string& sessionID ) 
                       throw (VException&);

    ///
    Sptr <string> buildRegisterForUpdateReq(const string& hostport,
					    const string& domain,
					    const vector<string> keyList,
					    callbackFunc func)
      throw (VException&);

    ///
    Sptr <string> buildUpdateReq (const string& domain,
				  const vector<string>& keyList)
      throw (VException&);

    ///
    Sptr <string> buildUnRegisterForUpdateReq(const string& hostport,
					      const string& domain,
					      const vector<string> keyList,
					      callbackFunc func)
      throw (VException&);

private :
    // -----------------------------------------------------------------------
    //  Private data members
    // -----------------------------------------------------------------------


    /// parsing helper: name of element last reported is always at the top
    stack< string > myFields;

    ///
    Sptr< PSDBRequest > myParsedContent;

    ///
    string myCharacters;

};
#endif // PSREQXMLCONTENTHANDLERS_HXX
