/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const PSRspXmlContentHandlersCxxVersion = "$Id: PSRspXmlContentHandlers.cxx,v 1.5 2001/12/28 17:25:45 jason Exp $";


// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <stdio.h>

#include <util/XMLUniDefs.hpp>

#include "cpLog.h"

#include "PSCommon.hxx"
#include "XmlContent.hxx"
#include "PSRspXmlContentHandlers.hxx"
#include "VNetworkException.hxx"



// ---------------------------------------------------------------------------
//  PSRspXmlContentHandlers: Constructors
// ---------------------------------------------------------------------------
PSRspXmlContentHandlers::PSRspXmlContentHandlers(Sptr<PSDBResult> target)
  : myCurMatch(-1), myCharacters("")
{
    cpLog(LOG_DEBUG, "PSRspXmlContentHandlers constructor invoked ");
    myParsedContent = target;

}

// ---------------------------------------------------------------------------
//  PSRspXmlContentHandlers: Destructor
// ---------------------------------------------------------------------------
PSRspXmlContentHandlers::~PSRspXmlContentHandlers()
{
    cpLog(LOG_DEBUG, "PSRspXmlContentHandlers destructor invoked ");
}


// ---------------------------------------------------------------------------
//  PSRspXmlContentHandlers: Overrides of the SAX ErrorHandler interface
// ---------------------------------------------------------------------------
void PSRspXmlContentHandlers::error(const SAXParseException& e)
{
    cpLog(LOG_ERR, "\nSAX Hanlder throws Error. File %s, line %d, char %d"
                       "\n   Message: %s",
          (StrX(e.getSystemId()), e.getLineNumber(),
           e.getColumnNumber(), StrX(e.getMessage())).localForm());

    string err = "Catching XML Exception: ";
    err += StrX(e.getMessage()).localForm();
    cpLog (LOG_ERR, err.c_str());

    VNetworkException exception(err.c_str(), __FILE__, __LINE__);
    throw exception;
}

void PSRspXmlContentHandlers::fatalError(const SAXParseException& e)
{
    cpLog( LOG_ERR, "\nSAX Handler throws Fatal Error. File %s, line %d, char %d"
                      "\n   Message: %s",
           (StrX(e.getSystemId()), e.getLineNumber(),
           e.getColumnNumber(), StrX(e.getMessage()) ).localForm());

    string err = "Catching XML Exception: ";
    err += StrX(e.getMessage()).localForm();
    cpLog (LOG_ERR, err.c_str());

    VNetworkException exception(err.c_str(), __FILE__, __LINE__);
    throw exception;
}

void PSRspXmlContentHandlers::warning(const SAXParseException& e)
{
    cpLog( LOG_WARNING, "\nSAX Handler throws Warning. File %s, line %d, chr %d"
                      "\n    Message: %s",
           (StrX(e.getSystemId()), e.getLineNumber(),
            e.getColumnNumber(), StrX(e.getMessage()) ).localForm());
    string err = "Catching XML Exception: ";
    err += StrX(e.getMessage()).localForm();

    VNetworkException exception(err.c_str(), __FILE__, __LINE__);
    throw exception;
}


// ---------------------------------------------------------------------------
//  PSRspXmlContentHandlers: Overrides of the SAX DTDHandler interface
// ---------------------------------------------------------------------------
void PSRspXmlContentHandlers::unparsedEntityDecl(const     XMLCh* const name
                                          , const   XMLCh* const publicId
                                          , const   XMLCh* const systemId
                                          , const   XMLCh* const notationName)
{
    // Not using DTD presently
}


void PSRspXmlContentHandlers::notationDecl(const   XMLCh* const name
                                    , const XMLCh* const publicId
                                    , const XMLCh* const systemId)
{
    // Not using DTD presently
}


// ---------------------------------------------------------------------------
//  PSRspXmlContentHandlers: Overrides of the SAX DocumentHandler interface
// ---------------------------------------------------------------------------
void PSRspXmlContentHandlers::characters(const     XMLCh* const    chars
                                  , const   unsigned int    length)
{
    string& fn = myFields.top();   // get field name
    string  val= StrX( chars ).localForm();
    myCharacters = val;

    cpLog(LOG_DEBUG, "  ****PSRspXmlContentHandlers::characters = %s len = %d",
          val.c_str(), length);

    if (myParsedContent->blobFlag == true)
      {
	cpLog (LOG_DEBUG, "Add the characters into xmlBlob");
	myParsedContent->setXmlBlob(val);
	//cpLog (LOG_DEBUG, "Xml Blob = %s", myParsedContent->getXmlBlob().c_str());
	return;
      }

    if ((fn == PSCommon::RESPONSE_STR) ||
	(fn == PSCommon::MATCH_STR))
    {
        ; // do nothing
    }
    else if ((fn == PSCommon::FIELDLIST_STR) ||
	     (fn == PSCommon::FIELDVALUE_STR))
    {
	myParsedContent->tagLevel = 1;        
    }
    else if (fn == PSCommon::DOMAIN_STR)
    {
        myParsedContent->setDomain( val );
        cpLog( LOG_DEBUG, "domain found: %s", val.c_str());
    }
    else if (fn == PSCommon::RESULTCOUNT_STR)
    {
        int num = atoi(val.c_str()); 
        myParsedContent->setNumMatches( num );
        myParsedContent->reserveNumTables( num );
        cpLog( LOG_DEBUG, "number of matches found %d", num );
    }
    else if (fn == PSCommon::ERROR_STR)
    {
        cpLog( LOG_DEBUG, "error: %s", val.c_str());
        myParsedContent->setError( val );
    }
    else if (fn == PSCommon::TOKEN_STR)
    {
        myParsedContent->setSessionID(val);
	cpLog( LOG_DEBUG, "sessionID = %s", val.c_str());
    }				      
    else if (fn == PSCommon::LISTITEM_STR)
    {
	myParsedContent->addFieldList(val);
	cpLog( LOG_DEBUG, "field = %s", val.c_str());
    }
    else if (fn == PSCommon::XMLCATEGORY_STR)
    {
	myParsedContent->setXmlCategory(val);
	cpLog( LOG_DEBUG, "xmlcategory = %s", val.c_str());
    }
    else if (fn == PSCommon::XMLDESC_STR)
    {
	myParsedContent->setXmlDescription(val);
	cpLog( LOG_DEBUG, "xmldescription = %s", val.c_str());
    }
    else if (fn == PSCommon::PRIMARYKEY_STR)
    {
	myParsedContent->addFieldList(val);
	cpLog( LOG_DEBUG, "primary key = %s", val.c_str());
    }
    else // anything else, stuff it into myParsedContent's table
    {
        // rely on myCurMatch as the index
        myParsedContent->setFieldValue(myCurMatch, fn, val);
        cpLog( LOG_DEBUG, "Table: %d, add %s/%s", 
               myCurMatch, fn.c_str(), val.c_str());
    }
    cpLog(LOG_DEBUG, "**** end characters ");
}


void PSRspXmlContentHandlers::endDocument()
{
    cpLog( LOG_DEBUG, "endDocument() reached" );

    if (myFields.empty() == false)
    {
        // this should not happen
        cpLog( LOG_ERR, "myFields should be empty!");
    }
}


void PSRspXmlContentHandlers::endElement(const XMLCh* const uri, 
								 const XMLCh* const localname, 
								 const XMLCh* const qname)
{
    string fn = StrX( qname ).localForm();
    // do no use name spaces, ignore localname
	//	fFormatter << qname << chCloseAngle;
    cpLog( LOG_DEBUG, "***** endElement qname = %s", fn.c_str() );
    myFields.pop();
    cpLog( LOG_DEBUG, " end endElement ***** ");

    myParsedContent->tagLevel--;

    if (myParsedContent->blobFlag == true) 
    {
	if (myParsedContent->tagLevel == 1)
	{
	    myParsedContent->setFieldValue(myCurMatch, fn, myParsedContent->getXmlBlob());
	    cpLog(LOG_DEBUG, "Final add fieldname [%s] with blob [%s] into table",
		  fn.c_str(), myParsedContent->getXmlBlob().c_str());

	    // clear out the xml blob
	    myParsedContent->clearXmlBlob();
	    //myParsedContent->blobFlag = false;
	    //myParsedContent->tagLevel = 0;
	    //myParsedContent->xmlBlob = "";
	    return;
	}

	//cpLog (LOG_DEBUG, "Add end of element into xml blob: %s", fn.c_str());
	string temp = PSCommon::END_ELEMENT_OPEN_BRACKET + fn + PSCommon::CLOSE_BRACKET;
	myParsedContent->setXmlBlob(temp);
	//cpLog (LOG_DEBUG, "Xml Blobg = %s", myParsedContent->getXmlBlob().c_str());
    }
    else if (myCharacters.empty())
    {
	cpLog(LOG_DEBUG, "added field [%s] (with empty value) to talbe", fn.c_str());
	myParsedContent->setFieldValue(myCurMatch, fn, "");
    }

}


void PSRspXmlContentHandlers::ignorableWhitespace( const   XMLCh* const chars
                                            ,const  unsigned int length)
{
    cout << "*********  ignorableWhitespace() called " << endl;
    // fFormatter.formatBuf(chars, length, XMLFormatter::NoEscapes);
}



void PSRspXmlContentHandlers::startDocument()
{
    cpLog( LOG_DEBUG,  "startDocument() ************ ");

}


void PSRspXmlContentHandlers::startElement(const   XMLCh* const    uri,
					   const   XMLCh* const    localname,
					   const   XMLCh* const    qname,
					   const   Attributes& attributes)
{
    string fn = StrX(qname).localForm();  // field name
    cpLog( LOG_DEBUG, " *** startElement qname : %s", fn.c_str() );

    if (fn == PSCommon::MATCH_STR.c_str())
    {
        myCurMatch++;
        cpLog( LOG_DEBUG, "matching record, idx = %d", myCurMatch );
    }
    myFields.push( fn );
    myCharacters = "";

    if (myParsedContent->tagLevel)
    {
	myParsedContent->tagLevel++;
	//cpLog (LOG_DEBUG, "BLOB Counter = %d", myParsedContent->tagLevel);
	if (myParsedContent->tagLevel > 2)
	{
	    myParsedContent->blobFlag = true;
	}
    }

    if (myParsedContent->blobFlag == true)
    {
	cpLog (LOG_DEBUG, "Add start of element into blobString: %s", fn.c_str());
	string temp = PSCommon::OPEN_BRACKET + fn + PSCommon::CLOSE_BRACKET;
	myParsedContent->setXmlBlob(temp);
	//cpLog (LOG_DEBUG, "Xml Blob = %s", myParsedContent->getXmlBlob().c_str());
    }

}



// *****************************************************************
// Single Transaction Response XML Content Builder
// *****************************************************************

Sptr<string> PSRspXmlContentHandlers::buildGetXmlDescriptionRsp(
    const string& payload) 
			   throw ( VException& ) 
{
    cpLog(LOG_DEBUG, "buildGetXmlDescription() payload %s",
                     payload.c_str());

    /* build the XML content
     */ 

    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
    content += payload;
    PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );

    body->assign( content );

    cpLog(LOG_DEBUG, "GetXmlDescription response xml body:\n %s", body->c_str());
    return body;

}

Sptr<string> PSRspXmlContentHandlers::buildListXmlCategoriesRsp(
    const StringList& resList) 
			   throw ( VException& ) 
{
    cpLog(LOG_DEBUG, "buildListXmlCategories()");

    /* build the XML content
     */ 

    Sptr<string> body = new string();
    string content;
    char buf[256];
    
    PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
    PSCommon::xmlAddTag( PSCommon::RESULTCOUNT_START, content );
    sprintf(buf, "%d", resList.size());
    content += string(buf);
    PSCommon::xmlAddTag( PSCommon::RESULTCOUNT_END, content );
    
    StringList::const_iterator i = resList.begin();
    while ( i != resList.end() )
    {
        string s = *i;
        i++;

        PSCommon::xmlAddTag( PSCommon::XMLCATEGORY_START, content );
        content += s;
        PSCommon::xmlAddTag( PSCommon::XMLCATEGORY_END, content );
    }

    PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );

    body->assign( content );

    cpLog(LOG_DEBUG, "ListXmlCategories response xml body:\n %s", body->c_str());
    return body;

}


Sptr<string> PSRspXmlContentHandlers::buildListXmlDescriptionsRsp(
    const StringList& resList) 
			   throw ( VException& ) 
{
    cpLog(LOG_DEBUG, "buildListXmlDescriptions()");

    /* build the XML content
     */ 

    Sptr<string> body = new string();
    string content;
    char buf[256];
    
    PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
    PSCommon::xmlAddTag( PSCommon::RESULTCOUNT_START, content );
    sprintf(buf, "%d", resList.size());
    content += string(buf);
    PSCommon::xmlAddTag( PSCommon::RESULTCOUNT_END, content );
    
    StringList::const_iterator i = resList.begin();
    while ( i != resList.end() )
    {
        string s = *i;
        i++;

        PSCommon::xmlAddTag( PSCommon::XMLDESC_START, content );
        content += s;
        PSCommon::xmlAddTag( PSCommon::XMLDESC_END, content );
    }

    PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );

    body->assign( content );

    cpLog(LOG_DEBUG, "ListXmlDescriptions response xml body:\n %s", body->c_str());
    return body;

}

///
Sptr<string> PSRspXmlContentHandlers::buildResultSetRsp(
                           const string& domain, 
			   const ResultTabletype & resultTable,
			   const FieldListtype & fieldList )
			   throw ( VException& ) 
{
    cpLog(LOG_DEBUG, "buildResultSetRsp() domain %s",
                     domain.c_str());

    /* build the XML content
     */ 

    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
    PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain, 
			      PSCommon::DOMAIN_END, content );
    int numRecords = resultTable.size();
    string  startTag, endTag;
    char buf[10];
    sprintf( buf, "%d", numRecords );
    PSCommon::xmlAddTagValue( PSCommon::RESULTCOUNT_START,
			      buf, PSCommon::RESULTCOUNT_END, content );
    vector<string> record;

    for (int i = 0; i < numRecords; i++)
    {
        record = resultTable[i];
	PSCommon::xmlAddTag( PSCommon::MATCH_START, content );
	PSCommon::xmlAddTag( PSCommon::FIELDVALUE_START, content );
	
	for (unsigned short j = 0; j < fieldList.size(); j++)
	{
	    startTag = PSCommon::OPEN_BRACKET + fieldList[j] +
	      PSCommon::CLOSE_BRACKET;
	    endTag = PSCommon::END_ELEMENT_OPEN_BRACKET + fieldList[j] +
	      PSCommon::CLOSE_BRACKET;
	    PSCommon::xmlAddTagValue( startTag, record[j], endTag, content);
	}

	PSCommon::xmlAddTag( PSCommon::FIELDVALUE_END, content );
	PSCommon::xmlAddTag( PSCommon::MATCH_END, content );
    } 
    PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );
    body->assign( content );

    cpLog(LOG_DEBUG, "BuildResultSet response xml body:\n %s", body->c_str());
    return body;

}


Sptr<string> PSRspXmlContentHandlers::buildResultCountRsp(
                           const string& domain,
                           const string& resultCount )
                           throw ( VException& )
{
    cpLog(LOG_DEBUG, "buildResultCountRsp() domain %s",
                     domain.c_str());

    /* build the XML content
     */

    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
    PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain,
                              PSCommon::DOMAIN_END, content );

    PSCommon::xmlAddTagValue( PSCommon::RESULTCOUNT_START,
                              resultCount.c_str(), 
                              PSCommon::RESULTCOUNT_END, 
                              content );


    PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );
    body->assign( content );

    cpLog(LOG_DEBUG, "BuildResultCount response xml body:\n %s", body->c_str());
    return body;

}



///
Sptr<string> PSRspXmlContentHandlers::buildGetFieldsRsp( 
                                const string& domain,
				const ResultTabletype & resultTable )
                                throw( VException& ) 
{
    cpLog(LOG_DEBUG, "buildGetFieldsRsp() domain %s",
                     domain.c_str());

    /* build the XML content
     */
    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
    PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain, 
			      PSCommon::DOMAIN_END, content );
    
    int num = resultTable.size();
    char buf[10];
    sprintf( buf, "%d", num );
    
    PSCommon::xmlAddTagValue( PSCommon::RESULTCOUNT_START,
			      buf, PSCommon::RESULTCOUNT_END, content );

    PSCommon::xmlAddTag( PSCommon::MATCH_START, content );
    // build list of field names:
    PSCommon::xmlAddTag( PSCommon::FIELDLIST_START, content );
    for (unsigned short i = 0; i < num; i++)
    {
	PSCommon::xmlAddTagValue( PSCommon::LISTITEM_START,
				  resultTable[i][0],
				  PSCommon::LISTITEM_END, 
				  content );
    }
    PSCommon::xmlAddTag( PSCommon::FIELDLIST_END, content );
    PSCommon::xmlAddTag( PSCommon::MATCH_END, content );

    PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );
    body->assign( content );

    cpLog(LOG_DEBUG, "GetFields response xml body:\n %s", body->c_str());
    return body;
}


/// update existing records with keyword/value pairs provided
Sptr<string> 
PSRspXmlContentHandlers::buildSetRecordsRsp(
                                const string& domain, 
						        const string& resultCount )
				                throw (VException& )
{
    cpLog(LOG_DEBUG, "buildSetRecordsRsp() domain %s, count = %s",
                     domain.c_str(), resultCount.c_str());

    /* build the XML content
     */
    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
    PSCommon::xmlAddTagValue( PSCommon::DOMAIN_START, domain,
			      PSCommon::DOMAIN_END, content );
    
    PSCommon::xmlAddTagValue( PSCommon::RESULTCOUNT_START,
			      resultCount, PSCommon::RESULTCOUNT_END, content );
    
    PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );
    body->assign( content );

    cpLog(LOG_DEBUG, "SetRecords response xml body:\n %s", 
          body->c_str());
    return body;

}

// *****************************************************************
// Session related XML Content Builder
// *****************************************************************

Sptr<string> 
PSRspXmlContentHandlers::buildSessionOpenRsp( const string& token ) 
                         throw(VException&)
{ 
    cpLog(LOG_DEBUG, "buildSessionOpenRsp() token:  %s", token.c_str() );

    /* build the XML content
     */
    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTag( PSCommon::RESPONSE_START, content );
    PSCommon::xmlAddTagValue( PSCommon::TOKEN_START, token.c_str(),
			      PSCommon::TOKEN_END, content );
    PSCommon::xmlAddTag( PSCommon::RESPONSE_END, content );
    body->assign( content );

    cpLog(LOG_DEBUG, "SessionOpen xml body: %s", body->c_str());
    return body;
}
         

///
Sptr<string>
PSRspXmlContentHandlers::buildErrorRsp( const string& errorText )
                       throw (VException&) 
{
    cpLog(LOG_DEBUG, "buildErrorRsp() reason:  %s ",
                     errorText.c_str());

    /* build the XML content
     */
    Sptr<string> body = new string();
    string content;

    PSCommon::xmlAddTagValue( PSCommon::ERROR_START, errorText,
			      PSCommon::ERROR_END, content );
    body->assign( content );

    cpLog(LOG_DEBUG, "ErrorRsp xml body: %s", body->c_str());

    return body;

}

