#ifndef PSRSPXMLCONTENTHANDLERS_HXX
#define PSRSPXMLCONTENTHANDLERS_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const PSRspXmlContentHandlersHxxVersion = "$Id: PSRspXmlContentHandlers.hxx,v 1.2 2001/09/19 20:31:29 maivu Exp $";


#include <string>
#include <stack>
#include <map>
#include <vector>

#include    <sax2/DefaultHandler.hpp>

#include "XmlContent.hxx"
#include "Sptr.hxx"
#include "PSDBResult.hxx"

#include "cpLog.h"
#include "PSDBCommonType.h"
#include "VFileSystem.hxx"


// class PSRspXmlContentHandlers : public DefaultHandler, private XMLFormatTarget
class PSRspXmlContentHandlers : public DefaultHandler
{
public:

    // -----------------------------------------------------------------------
    //  Constructors
    // -----------------------------------------------------------------------
    PSRspXmlContentHandlers(Sptr<PSDBResult> target);

    PSRspXmlContentHandlers() {}


    ///
    virtual ~PSRspXmlContentHandlers();


    // -----------------------------------------------------------------------
    //  Implementations of the SAX DocumentHandler interface
    // -----------------------------------------------------------------------
    void endDocument();

    void endElement( const XMLCh* const uri, 
					 const XMLCh* const localname, 
					 const XMLCh* const qname);

    void characters(const XMLCh* const chars, const unsigned int length);

    void ignorableWhitespace
    (
        const   XMLCh* const    chars
        , const unsigned int    length
    );

    void startDocument();

    void startElement(	const   XMLCh* const    uri,
						const   XMLCh* const    localname,
						const   XMLCh* const    qname,
					    const   Attributes&		attributes);



    // -----------------------------------------------------------------------
    //  Implementations of the SAX ErrorHandler interface
    // -----------------------------------------------------------------------
    void warning(const SAXParseException& exception);
    void error(const SAXParseException& exception);
    void fatalError(const SAXParseException& exception);



    // -----------------------------------------------------------------------
    //  Implementation of the SAX DTDHandler interface
    // -----------------------------------------------------------------------
    void notationDecl
    (
        const   XMLCh* const    name
        , const XMLCh* const    publicId
        , const XMLCh* const    systemId
    );  // not used

    void unparsedEntityDecl
    (
        const   XMLCh* const    name
        , const XMLCh* const    publicId
        , const XMLCh* const    systemId
        , const XMLCh* const    notationName
    );  // not used

    ///  set a reference to the target where parsed content will be kept
    void setParsedContentTarget( Sptr<PSDBResult> target )
    {
        myParsedContent = target;
    }

    ///
    Sptr< PSDBResult >& getParsedContentTarget() { return myParsedContent; }

    // -----------------------------------------------------------------------
    //  Implementation of PS Response XML protocol builder interface
    // -----------------------------------------------------------------------

    Sptr< string > buildGetXmlDescriptionRsp(
        const string& payload) 
                                throw( VException& );

    Sptr< string > buildListXmlCategoriesRsp(
        const StringList& resList) 
                                throw( VException& );
    
    Sptr< string > buildListXmlDescriptionsRsp(
        const StringList& resList) 
        throw( VException& );

    
    
    ///
    Sptr< string > buildResultSetRsp( const string& domain, 
				      const ResultTabletype & resultTable,
				      const FieldListtype & fieldList)
      throw( VException& );

    Sptr< string > buildResultCountRsp( const string& domain,
					const string& resultCount )
      throw( VException& );

    ///
    Sptr< string > buildGetFieldsRsp( const string& domain,
				      const ResultTabletype & resultTable )
      throw( VException& );

    /// update existing records with keyword/value pairs provided
    Sptr< string > buildSetRecordsRsp( const string& domain, 
				       const string& resultCount )
      throw (VException& );

    ///
    Sptr< string > buildSessionOpenRsp( const string& token ) 
                        throw(VException&);

    ///
    Sptr< string > buildErrorRsp( const string& errorText ) 
                       throw (VException&);


private :
    // -----------------------------------------------------------------------
    //  Private data members
    // -----------------------------------------------------------------------

    ///
    int             myCurMatch;

    /// name of element last reported is always at the top
    stack< string > myFields;

    ///
    Sptr< PSDBResult > myParsedContent;

    ///
    string myCharacters;

};
#endif // PSRSPXMLCONTENTHANDLERS_HXX
