// XmlBlobContentHandler.cxx



#include <stdio.h>
#include <util/XMLUniDefs.hpp>

#include "cpLog.h"

#include "PSCommon.hxx"
#include "XmlContent.hxx"
#include "XmlBlobContentHandler.hxx"
#include "VNetworkException.hxx"
#include "PSDBResult.hxx"
#include "Data.hxx"

///
XmlBlobContentHandler::XmlBlobContentHandler(Sptr<PSBlobResult> result)
  : myChars(""), myParsedContent(result), level(0), index(-1)
{
    cpLog(LOG_DEBUG, "XmlBlobContentHandler constructor invoked ");

}

///
XmlBlobContentHandler::~XmlBlobContentHandler()
{
    cpLog(LOG_DEBUG, "XmlBlobContentHandler destructor invoked ");
}



///
void 
XmlBlobContentHandler::startDocument()
{
    cpLog( LOG_DEBUG,  "startDocument() ************ ");

}

///
void 
XmlBlobContentHandler::startElement(const XMLCh* const uri,
			     const XMLCh* const localname,
			     const XMLCh* const qname,
			     const Attributes& attributes)
{
    string fn = StrX(qname).localForm();  // field name
    cpLog( LOG_DEBUG, " *** startElement qname : %s", fn.c_str() );
    myFields.push( fn );   // remember it on the stack
    myChars = "";
    level++;

    cpLog (LOG_DEBUG, "Level = %d", level);
    
    // level 1 ==> table
    // level 2 ==> row
    // level 3 ==> column

    if (level == 2)
      index ++;

}

///
void 
XmlBlobContentHandler::characters(const XMLCh* const chars,
			   const unsigned int length)
{
  //    string& fn = myFields.top();   // get current field name
    string  val= StrX( chars ).localForm();
    myChars = val;

    cpLog(LOG_DEBUG, "  ****XmlBlobContentHandler::characters = %s len = %d",
          val.c_str(), length);

    if (level > 2)
      myParsedContent->setBlobValue(index, val);

    cpLog(LOG_DEBUG, "**** end characters ");
}

///
void 
XmlBlobContentHandler::endElement(const XMLCh* const uri, 
			   const XMLCh* const localname, 
			   const XMLCh* const qname)
{
    string fn = StrX( qname ).localForm();
    level--;

    if (myChars.empty())
        myParsedContent->setBlobValue(index, "");

    cpLog( LOG_DEBUG, "***** endElement qname = %s", fn.c_str() );

    myFields.pop();      

}

///
void 
XmlBlobContentHandler::endDocument()
{
    cpLog( LOG_DEBUG, "endDocument() reached" );

    if (myFields.empty() == false)   // stack should be empty by now
    {
        // this should not happen
        cpLog( LOG_ERR, "myFields should be empty!");
    }
}

///
void 
XmlBlobContentHandler::warning(const SAXParseException& e)
{
    cpLog( LOG_WARNING, "\nSAX Handler throws Warning. File %s, line %d, chr %d"
                      "\n    Message: %s",
           (StrX(e.getSystemId()), e.getLineNumber(),
           e.getColumnNumber(), StrX(e.getMessage()) ).localForm());

    string err = "Catching XML Exception: ";
    err += StrX(e.getMessage()).localForm();

    VNetworkException exception(err.c_str(), __FILE__, __LINE__);
    throw exception;
}

///
void 
XmlBlobContentHandler::error(const SAXParseException& e)
{
    cpLog(LOG_ERR, "\nSAX Handler throws Error. File %s, line %d, char %d"
                     "\n   Message: %s",
          (StrX(e.getSystemId()), e.getLineNumber(),
          e.getColumnNumber(), StrX(e.getMessage())).localForm());

    string err = "Catching XML Exception: ";
    err += StrX(e.getMessage()).localForm();

    VNetworkException exception(err.c_str(), __FILE__, __LINE__);
    throw exception;

}

///
void 
XmlBlobContentHandler::fatalError(const SAXParseException& e)
{
    cpLog( LOG_ERR, "\nSAX Handler throws Fatal Error. File %s, line %d, char %d"
                      "\n   Message: %s",
           (StrX(e.getSystemId()), e.getLineNumber(),
           e.getColumnNumber(), StrX(e.getMessage()) ).localForm());

    string err = "Catching XML Exception: ";
    err += StrX(e.getMessage()).localForm();

    VNetworkException exception(err.c_str(), __FILE__, __LINE__);
    throw exception;

}



// end of file
