
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const HostInterface_cxx_Version =
    "$Id: HostInterface.cxx,v 1.2 2001/09/19 20:32:07 maivu Exp $";

#include "HostInterface.hxx"
#include "Tcp_ClientSocket.hxx"
#include "VException.hxx"


HostInterface::HostInterface(const char *host, const int port) :
    myIsAlive(true), myInUse(false)
{

    // myIsAlive is true -- we assume we are connected till we fail on a connect.
    // myInUse is set by PSInterface
    
    string pServer = host;

    // construct the network address, and pass it to the reader
    myNetworkAddress = new NetworkAddress(pServer);
    myNetworkAddress->setPort(port);

    // instantiate a PSConnection object using this address
    myConnection = new PSConnection(*myNetworkAddress);

}


string
HostInterface::sendRequest(const string& httpMsg)
throw (VException&)
{

    // if this fails, an exception gets thrown up to the PSInterface
    // class, which will mark this as disconnected and try the other one...
    //The return response from ProvisionServer e.g. getRecordsInDomain
    //200 OK HTTP/1.1
    //Content-Length:
    //<?xml version="1.0"?>
    //<response>
    //   <domain>Accounts</domain>
    //   <resultcount>5</resultcount>
    //   <match>
    //   <fieldvalue>
    //      <uid>1001</uid>
    //      <phone>nnnnnnn</phone>
    //   </fieldvalue>
    //   </match>
    //   ....
    //</response>
    //

    return ((myConnection->sendRequest(httpMsg)));

}

