#ifndef HOSTINTERFACE_HXX
#define HOSTINTERFACE_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const HostInterface_hxx_Version =
    "$Id: HostInterface.hxx,v 1.2 2001/09/19 20:32:09 maivu Exp $";

#include <string>
#include "Sptr.hxx"
#include "PSConnection.hxx"
#include "NetworkAddress.h"

class VException;
class NetworkAddress;

class HostInterface
{

    public:

        /// Constructor
        HostInterface(const char* host, const int port);

        /// Destructor
        ~HostInterface()
        {}

        /// Reads an item from provision server

        string sendRequest(const string &httpMessage)
        throw (VException&);

        ///
        const NetworkAddress& getNetAddress()
        {
            return (*myNetworkAddress);
        }

        /// used to let PSInterface class keep track of this guy
        bool isAlive()
        {
            return myIsAlive;
        };

        ///
        void setAlive(bool val)
        {
            myIsAlive = val;
        };

        /// used to let PSInterface class keep track of this object
        bool inUse()
        {
            return myInUse;
        };

        ///
        void setUse(bool val)
        {
            myInUse = val;
        };


    private:


        /// bool to tell if the server is alive 
        bool myIsAlive;

        /// bool to tell if this server is in use (active)
        bool myInUse;

        /// Reader object to serve read request
        Sptr <PSConnection> myConnection;

        /// Network address of the Provision Server
        Sptr <NetworkAddress> myNetworkAddress;


};

#endif // HOSTINTERFACE_HXX
