
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const PSConnection_cxx_Version =
    "$Id: PSConnection.cxx,v 1.2 2001/09/19 20:32:11 maivu Exp $";

#include <sys/types.h>
#include <sys/errno.h>
#include <unistd.h>


#include "PSConnection.hxx"
#include "NetworkAddress.h"
#include "Tcp_ClientSocket.hxx"
#include "Connection.hxx"
#include "cpLog.h"


static const int ONE_K_BUF = 1024;

PSConnection::PSConnection(const NetworkAddress& netAddress)
        : _netAddress(netAddress),
        _clientSocket(NULL)
{}


// Sptr<HttpMessage>
string
PSConnection::sendRequest(const string& httpMsg)
throw (VException&)
{

    cpLog(LOG_DEBUG, "sendRequest - http msg: (%s)",
          httpMsg.c_str());

    Connection& conn = getConnection();
    string temp = httpMsg;
    temp += '\n';

    conn.writeData( (void*)temp.c_str(), temp.size() );

    // Sptr<HttpMessage> msg = new HttpMessage; 
    string dRead;
    char rdbuf[ONE_K_BUF];
    int nRead = 0; 

    // Build a HttpMessage by reading 1K chunk at a time.
    // For each chunk, ask HttpMessage if message is complete.
    // If not, mset rdbuf and continue to read another chunk:
    // while (msg.isComplete() == false)
    // {
    //     memset( rdbuf, 0, ONE_K_BUF );
    //     conn.readn((void*)rdbuf, (sizeof(rdbuf) - 1), nRead);
    //     // stuff chunk into HttpMessage:
    //     msg += rdbuf;
    //     nRead = 0;
    //  }

    while (conn.readn((void*)rdbuf, (sizeof(rdbuf) - 1), nRead))
    {
        rdbuf[nRead] = '\0';
        dRead += rdbuf;
        rdbuf[0] = '\0';
        nRead = 0;
    }

    rdbuf[nRead] = '\0';
    dRead += rdbuf;

    cpLog(LOG_DEBUG, "************** RECEIVE RESPONSE *******************");
    cpLog(LOG_DEBUG, "Data read: (%s)", dRead.c_str());
    // cpLog(LOG_DEBUG, "Data read: (%s)", msg.getString().c_str());
    // return msg;
    return dRead;

}



PSConnection::~PSConnection()
{}



Connection&
PSConnection::getConnection() throw (VException&)
{

    _clientSocket = new TcpClientSocket(_netAddress, true, false);
    _clientSocket->connect();
    return (_clientSocket->getConn());

}
