#ifndef PSHTTPXMLINTERFACE_HXX
#define PSHTTPXMLINTERFACE_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const PSHttpXMLInterface_hxx_Version =
    "$Id: PSHttpXMLInterface.hxx,v 1.2 2001/09/19 20:32:21 maivu Exp $";


#include "VNetworkException.hxx"
#include "ProvisioningInterface.hxx"
#include "PSDBCommonType.h"
#include "HttpMessage.hxx"
#include "PSDBRequest.hxx"

using namespace Vocal;


///
class PSHttpXMLInterface : protected ProvisioningInterface { 

    public:

        /**
           Frees the memory associated the Instance.Gets register with
           atexit(), so that when program exits, it will be called automatically.
        */
        static void destroy();

        /// Destructor
        virtual ~PSHttpXMLInterface();

        const char* getClassName() const { return "PSHttpXMLInterface"; }


        // *****************************************************************
        // Generic atomic data accessor functions
        // *****************************************************************

        /// retrive record(s) matching the domain, filter and fields of interest
	Sptr<PSDBResult> getXmlDescription( const string& xmlCategory, 
                                        const string& xmlDescription)
	  throw( VException& );


	Sptr<PSDBResult> listXmlCatagories()
	  throw( VException& );

    	Sptr<PSDBResult> listXmlDescriptions( const string& xmlCategory)
	  throw( VException& );


    
    /// retrive record(s) matching the domain, filter and fields of interest
	Sptr<PSDBResult> getRecordsInDomain( const string& domain, 
					     const string& filterRules,
					     FieldListtype & fieldList )
	  throw( VException& );

        ///
        Sptr<PSDBResult> getFieldsInDomain( const string& domain )
                                throw( VException& );

		/// update existing records with keyword/value pairs provided
		Sptr<PSDBResult> setRecordsInDomain( const string& domain, 
                                string filterRules,
						        const map<string, string>& fieldValPairs )
				                throw (VException& );

        /// insert a new record
        Sptr<PSDBResult> addRecordInDomain( const string& domain, 
                               const map< string, string>& fieldValPairs)
                               throw( VException& );

        ///
		Sptr<PSDBResult> deleteRecordsInDomain( const string& domain,
						           const string& filterRules )
				                   throw( VException& );


        // *****************************************************************
        // Session related functions
        // *****************************************************************

		/** psServer returns -1 if PServer does not return
         *  otherwise, return value is the sessioID
         */
        string sessionOpen( int timeInSeconds ) throw(VException&);

		///
        bool sessionClose( const string& sessionID ) throw( VException& );

        ///
        Sptr<PSDBResult> sessionSearchDB( const string& sessionID,
                       const string& domain, const string& filterRules,
                       const FieldListtype & fieldList, 
                       PSCommon::PSSortType sType,
                       const string& sortField ) throw(VException&);

        ///
        Sptr<PSDBResult> sessionGetDBResult( const string& sessionID, int startIdx,
                       int endIdx, const FieldListtype& fieldList)
                       throw (VException&);

        ///
        Sptr<PSDBResult> sessionSetDBResult( const string& sessionID, 
                       const map< string, string>& fieldValPairs)
                       throw (VException&);

        ///
        Sptr<PSDBResult> sessionSortDBResult( 
                       const string& sessionID, PSCommon::PSSortType sType,
                       const string& sortField )
                       throw (VException&);

        ///
        Sptr<PSDBResult> sessionDeleteAll( const string& sessionID ) 
                       throw (VException&);

        // *****************************************************************
        // Registration functions
        // *****************************************************************

        /**
           Register client for notification for a given domain
           Callback func get called whenever item is updated.
        */

        void registerDomainForUpdate( const string& hostport,
				      const string& domain,
				      callbackFunc func ) 
	  throw (VException&);

        void registerRecordForUpdate( const string& hostport,
				      const string& domain,
				      const vector<string>& keyList, 
				      callbackFunc func)
	  throw (VException&);  
                       
        void unRegisterDomainForUpdate( const string& hostport,
					const string& domain,
					callbackFunc func ) 
	  throw (VException&);

        void unRegisterRecordForUpdate( const string& hostport,
					const string& domain,
					const vector<string>& keyList, 
					callbackFunc func)
	  throw (VException&); 

    protected:

         friend class PSIFCreator;

         /// instance method to get a singleton PSHttpXMLInterface object
         static ProvisioningInterface& PSHttpXMLInterface::getInstance(
                             const char* primHost, const int primPort,
                             const char* altHost, const int altPort)
                             throw( VException& );

        ///
        void handleRequest(const string& request)
	  throw (VException&);
                       
    private:

        /// private constructor
        PSHttpXMLInterface(const char* primHostname, const int primPort,
                           const char* altHostname, const int altPort);

        /// suppress equal operator
        friend bool operator==( const PSHttpXMLInterface& lhs,
                                const PSHttpXMLInterface& rhs );

        /// suppress copy contructor
        PSHttpXMLInterface( const PSHttpXMLInterface& rhs );

        /// suppress assignment operator
        PSHttpXMLInterface& operator =( const PSHttpXMLInterface& rhs );

        ///
        void HandlePUTRequest(const Http::Message& httpMsg);

        ///
        bool PSHttpXMLInterface::SetCallbackList(const bool& flagDelete,
						 callbackMap& aMap,
						 const string& key,
						 const callbackFunc& func);      
        ///
        void UpdateDomainCallback(Sptr <PSDBRequest> request);

        ///
        void UpdateRecordCallback(Sptr <PSDBRequest> request);


};

#endif // PSHTTPXMLINTERFACE_HXX

