
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const PSIFCreator_cxx_Version = "$Id: PSIFCreator.cxx,v 1.3 2001/10/03 00:03:32 maivu Exp $";


#include "VNetworkException.hxx"
#include "VBadDataException.hxx"
#include "PSIFCreator.hxx"
#include "cpLog.h"

// Concrete ProvisioningInterface classes: 
#include "PSHttpXMLInterface.hxx"


/** PSIFCreator is a concrete class which provides a default
 *  implementation for the factory mehtod.
 *  Usage:
 *
 *  try {
 *      ProvisioningInterface& pif = PSIFCreator::getPSIFInstance(
 *           "host1",6005,"host2",6005,PSIFTYPE::HTTPXML);
 *      long sid = pif.sessionOpen( 3600 );
 *      .....
 *  }
 *  catch (VException& e)
 *  ....
 */
 ProvisioningInterface& PSIFCreator::getPSIFInstance( const char* primHost = 0,
                  const int primPort = 0, const char* altHost = 0,
                  const int altPort = 0,  
                  PSCommon::PSIFType type = PSCommon::HTTPXML )
                  throw (VException&)
{
     if (ProvisioningInterface::myInstance != 0)
     {
         return (*ProvisioningInterface::myInstance);
     }

     if (primHost == 0 || primPort == 0 || altHost == 0 || altPort == 0)
     {
         string err = "PServer hostname and port not provided";
	 cpLog (LOG_ERR, err.c_str());
	 VNetworkException exception(err.c_str(), __FILE__, __LINE__);
	 throw exception;
     }

     switch (type)
     {
         case PSCommon::HTTPXML:
         {
             return  PSHttpXMLInterface::getInstance(
                                           primHost, primPort,
                                           altHost, altPort);
         }
         break;

         default:
	   {
	     string err = "Unsupported PSIFType";
	     cpLog (LOG_ERR, err.c_str());
             VBadDataException exception(err.c_str(), __FILE__, __LINE__, type);
	     throw exception;
	   }
         break;
     }
}


