#ifndef PSIFCREATOR_HXX
#define PSIFCREATOR_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const PSIFCreator_hxx_Version = "$Id: PSIFCreator.hxx,v 1.2 2001/09/19 20:32:24 maivu Exp $";


#include "VException.hxx"
#include "PSCommon.hxx"
#include "ProvisioningInterface.hxx"




/** PSIFCreator is a concrete class which provides a default
 *  implementation for the factory mehtod.
 *  Usage:
 *
 *  try {
 *      PSIFCreator myPSIFCreator;
 *      ProvisioningInterface& pif = myPSIFCreator.getPSIFInstance(
 *           "host1",6005,"host2",6005,HTTPXML);
 *      long sid = pif.sessionOpen( 3600 );
 *      .....
 *  }
 *  catch (VException& e)
 *  ....
 */
class PSIFCreator
{

    public:

        /** Concrete Provisioning Interface Factory Method:
         *  @param primHost = primary host for Provisioning Server
         *  @param primPort = primary port for Provisioning Server
         *  @param altHost  = secondary host
         *  @param altPort  = secondary port
         *  @param type     = enum for Concrete Provisioning Interface obj
         */
        ProvisioningInterface& getPSIFInstance( const char* primHost = 0,
                  const int primPort = 0, const char* altHost = 0,
                  const int altPort = 0,  
                  PSCommon::PSIFType type = PSCommon::HTTPXML )
                  throw(VException&);


};

#endif // PSIFCREATOR_HXX
