/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const PSInterface_cxx_Version =
    "$Id: PSInterface.cxx,v 1.2 2001/09/19 20:32:26 maivu Exp $";

#include <unistd.h>


#include "PSInterface.hxx"
#include "cpLog.h"

const int PSInterface::NUM_TRIES = 10;
const int PSInterface::TIME_DELAY = 10;
const int PSInterface::TIMEOUT = PSInterface::NUM_TRIES*
                                     PSInterface::TIME_DELAY/40;


PSInterface::PSInterface(const char* primhost, const int primport,
                         const char* althost, const int altport) :
        myPrimary(primhost, primport), mySecondary(althost, altport)
{

    cpLog(LOG_DEBUG, "Primary Server is %s:%d, Secondary: %s:%d",
          primhost, primport, althost, altport);

    // start by trying to talk to primary
    myInterface = &myPrimary;
    myInterface->setUse(true);

}


void
PSInterface::swapServers()
{

    cpLog(LOG_ERR, "Failed to talk to one server, swapping");

    // check which is being used, and swap 'em
    if (myPrimary.inUse())
    {

        myPrimary.setUse(false);
        myInterface = &mySecondary;
        mySecondary.setUse(true);

    }
    else
    {

        mySecondary.setUse(false);
        myInterface = &myPrimary;
        myPrimary.setUse(true);

    }

}


// synchronous connection
string
PSInterface::sendRequest(const string& httpMessage)
throw (VException&)
{

    int numTries = 0;
    int elapsedTime = 0;
    string dRet = "";

    while (1)
    {

        while (numTries < NUM_TRIES)
        {

            try
            {
                string dRet = myInterface->sendRequest(httpMessage);
                return dRet;
            }
            catch (VException&)
            {
                swapServers();
            }

            numTries++;

        }

        cpLog(LOG_ERR, "Both Servers are dead. Sleeping.");
        // TODO: impose a timeout or else will loop forever.
        //       if timeout, throw exception
        if (elapsedTime > TIMEOUT)
        {
            cpLog(LOG_WARNING, "sendRequest() gives up retrying");
            // throw an exception?
            break;
        }
        sleep(TIME_DELAY);

        elapsedTime += TIME_DELAY;
        numTries = 0;

    }

    return dRet;

}


