#ifndef PSINTERFACE_HXX
#define PSINTERFACE_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const PSInterface_hxx_Version =
    "$Id: PSInterface.hxx,v 1.2 2001/09/19 20:32:27 maivu Exp $";

#include <string>

//User defined
#include "NetworkAddress.h"
#include "HostInterface.hxx"
#include "VException.hxx"


/**
  Essentially, this class is a wrapper for two HostInterfaces. If one
  fails, the server will fail over and try the other.
*/

class PSInterface
{
    public:

        /**
           Constructor initializes with the name of provision server
           either from a file or from a environment variable.Intentionally
           made private so that it cannot be instantiated directly.
           It also creates an instance of listener in a separate thread
           to listen for a file update.
        */
        PSInterface(const char* myHostname, const int myPort,
                    const char* altHostname, const int altPort);

        /// Destructor
        ~PSInterface()
        {}

        /// Return the address of the PServer currently in use

        const NetworkAddress& getNetAddress()
        {
            return myInterface->getNetAddress();
        }

        /// Reads an item from provision server
        string sendRequest(const string &httpMessage)
        throw (VException&);

    private:

        static PSInterface* myInstance;

        // keep pointers to the two host interfaces
        HostInterface myPrimary;
        HostInterface mySecondary;

        HostInterface *myInterface;

        // swap the active server, which is defined by myInterface
        void swapServers();

    static const int NUM_TRIES;
    static const int TIME_DELAY;
    static const int TIMEOUT;
    
};

#endif // PSINTERFACE_HXX
