
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const PSListener_cxx_Version =
    "$Id: PSListener.cxx,v 1.3 2001/10/03 00:03:33 maivu Exp $";

#include "PSListener.hxx"

#include <sys/time.h>
#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "Sptr.hxx"
#include "Tcp_ServerSocket.hxx"
#include "Connection.hxx"
#include "cpLog.h"
#include "VNetworkException.hxx"
#include "VEnvVar.hxx"
#include "PSIFCreator.hxx"


void*
PSListener::runThr(void* args)
{

    PSListener* self = (PSListener*)args;
    self->run();
    return NULL;

}


void
PSListener::run()
{

    bool socketSuccessful = false;
    Sptr <TcpServerSocket> serverSocket;

    while ( ! socketSuccessful )
    {
        socketSuccessful = true;
        try
        {
            serverSocket = new TcpServerSocket(_listenerPort);
            _init = true;
        }
        catch ( VNetworkException )
        {
            // could not listen here, so retry
            sleep(1);
            _listenerPort++;
            socketSuccessful = false;  // redo

            cpLog(LOG_ALERT, "Trying port %d", _listenerPort);
        }
    }

    if (socketSuccessful)
        cpLog(LOG_DEBUG, "Successfully opened listner port %d", _listenerPort);

    //Loop through for Incoming messages
    for (; ; )
    {
        Connection conn;
        serverSocket->accept(conn);
        processRequest(conn);
        conn.close();
        if (_exitFlg) break;
    }

}


PSListener::~PSListener()
{
    _exitFlg = true;
}

// using dynamic listening port or default = 6006
PSListener::PSListener()
        : _exitFlg(false),
        _listenerPort(0),
        _init(false)
{}


void
PSListener::processRequest(Connection& conn)
{

    char buf[MAXLINE];
    string dRead;
    buf[0] = '\0';
    int nread = 0;

    while (conn.readLine(buf, sizeof(buf), nread) > 0)
    {
        buf[nread] = 0;
        dRead += buf;

    }

    cpLog(LOG_DEBUG, "Processing request: (%s)",
	  dRead.c_str());

    PSIFCreator psFactory;
    try
    {
	psFactory.getPSIFInstance().handleRequest(dRead);
    }
    catch (VException& e)
    {
	cpLog (LOG_ERR, "Failed to handle request: %s",
	       e.getDescription().c_str());
    }

    return;

}

int
PSListener::getListenerPort() const
{

    while (!_init)
    {
        cpLog(LOG_INFO, "PSLIB: PSListener is not initialized waiting..");
        sleep(1);
    }

    return _listenerPort;

}

void
PSListener::setListenPort(int port)
  throw (VException&)
{
    if (_listenerPort != 0)
    {
        string err = "Port is already set. Reject new port";
	cpLog (LOG_ERR, err.c_str());
	VNetworkException exception(err.c_str(), __FILE__, __LINE__);
	throw exception;
    }

    cpLog (LOG_DEBUG, "Got the listen port[%d], start the thread now.", port);
    _listenerPort = port;

    pthread_create( &_ptid, 0, PSListener::runThr, (void*)this);
}
