#ifndef PSLISTENER_HXX
#define PSLISTENER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const PSListener_hxx_Version =
    "$Id: PSListener.hxx,v 1.3 2001/10/03 00:03:35 maivu Exp $";




#include <string>
#include <pthread.h>
#include "VException.hxx"


class Connection;
class ProvisioningInterface;

/**
    Use TCP to listen on a dynamic port for any update notifications,
    that are coming from the provision server.The port number is defined
    by the client and only after the port is set, the new thread will
    be created.  If the port is reset, an exception will throw.
*/

class PSListener
{

    public:

        /// Starts the Listener in separate thread
        static void* runThr(void*);

        /// Destructor
        ~PSListener();

        /// Member to process request on the socket
        void run();

        /// given the port to listen - throw exception if reset to new port.
        void setListenPort (int port)
	  throw (VException&);
  

    private:

        /// Constructor
        PSListener();

        /// Handle an update request. Calls into the ProvisionInterface class
        void processRequest(Connection& conn);

        /// returns the listener port
        int getListenerPort() const ;

    /// thread id
    pthread_t _ptid;

    /// set when time to exit
    bool _exitFlg;

    /// Port to listen for messages from the PS
    int _listenerPort;

friend class ProvisioningInterface;

    /// Have we been initialized (run() called) yet?
    bool _init;
    
};

#endif // PSLISTENER_HXX
