
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const ProvisioningInterface_cxx_Version =
    "$Id: ProvisioningInterface.cxx,v 1.2 2001/09/19 20:32:33 maivu Exp $";
#include <stdlib.h>
#include <stdio.h>
#include <strstream.h>
#include <unistd.h>
#include <errno.h>

#include "ProvisioningInterface.hxx"

#include "VException.hxx"
#include "VBadDataException.hxx"
#include "VMissingDataException.hxx"
#include "VNetworkException.hxx"
// #include "VConditionException.hxx"
#include "LockHelper.hxx"

#include "PSListener.hxx"
#include "cpLog.h"
// #include "VXmlParser.hxx"
#include "NetworkAddress.h"
// #include "PSSession.hxx"

ProvisioningInterface* ProvisioningInterface::myInstance = 0;


ProvisioningInterface::ProvisioningInterface(const char* primHostname,
                                       const int primPort,
                                       const char *altHostname,
                                       const int altPort) :
        myInterface(primHostname, primPort, altHostname, altPort)

{

    cpLog(LOG_DEBUG, "ProvisioningInterface constructor invoked");
    myListener = new PSListener();

#if defined(__linux__)
    //Destroy when program exits
    if (atexit(ProvisioningInterface::destroy) < 0)
    {
        cpLog(LOG_ALERT, "Failed to register ProvisioningInterface::destroy");
    }
#endif

    char buf[256];
    gethostname(buf, 256);
    myHostName = buf;

}

void
ProvisioningInterface::destroy()
{
    delete ProvisioningInterface::myInstance;
    ProvisioningInterface::myInstance = 0;
}

ProvisioningInterface::~ProvisioningInterface()
{
    cpLog(LOG_DEBUG, "ProvisioningInterface destructor invoked");
	delete myListener;
}


///
void
ProvisioningInterface::setListenPort(int port)
  throw (VException&)
{
    return (myListener->setListenPort(port));
}
