/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const GetFields_cxx  = "$Id: GetFields.cxx,v 1.1.1.1 2001/08/21 23:30:24 dabryan Exp $";

#include "Util.h"


int GetFields(ProvisioningInterface& pif)
{
    char buf[80]; memset(buf, 0, 80);
    ReadDomain();

    Sptr<PSDBResult> result = 0;
  
    try
      {
	result = pif.getFieldsInDomain(domain);
      }
    catch (VException& e)
      {
	//cpLog(LOG_ERR, "Error GetFieldsInDomain: ", e.getDescription().c_str());
	cout << "Error from Get Fields in Domain: "
	     << e.getDescription() << endl;
	cout << "Stop !!!" << endl;
	return 1;
      }
    
    if (result == 0)
      {
	cout << "There is no field in domain " << domain << endl;
	cout << "Need to add fields into domain. Stop !!!" << endl;
	return 1;
      }
    
    string error = result->getError();
    if (error.size() != 0)
      {
	cout << "- Error: " << error << ". Stop !!!" << endl;
	return 1;
      }
    
    fieldList = result->getFieldList();

    cout << "====================================" << endl;
    cout << "===>  GetFieldsInDomain Result  <===" << endl;
    cout << "====================================" << endl;
    PrintFieldList();

    return 0;
}


/*
    Sptr<PSDBResult> result = 0;
    
    try
    {
	result = pif.getFieldsInDomain("vocaluser");
    }
    catch (VException& e)
    {
	cpLog(LOG_ERR, "Error GetFieldsInDomain: ", e.getDescription().c_str());
    }
    
    if (result == 0)
      return;
    
    string error = result->getError();
    if (error.size() != 0)
    {
	cout << "-Error: " << error << endl;
	return;
    }
    
    
    cout << "- Domain: " << result->getDomain() << endl;
    cout << "- Number of matches: " << result->getNumMatches() << endl;
    cout << "- Session ID: " << result->getSessionID() << endl;
    cout << "- Status: " << result->getStatus() << endl;
    cout << "- Field List:" << endl;

    FieldListtype fieldList = result->getFieldList();
    for (unsigned short u = 0; u < fieldList.size(); u++)
    {
	cout << u+1 << " ==> " << fieldList[u] << endl;
    }
*/


