/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const RegisterDomain_cxx  = "$Id: RegisterDomain.cxx,v 1.2 2001/09/19 23:16:39 maivu Exp $";

#include "Util.h"


void RegisterDomain(ProvisioningInterface& pif)
{
    char buf[80]; memset(buf, '\0', 80);
    string hostport;

    ReadDomain();
    ReadHostPort(hostport);

    cout << "Which callback test function register ? " << endl << endl;
    cout << "[1] callback test function 1" << endl;
    cout << "[2] callback test function 2" << endl;
    cout << "[3] callback test function 3"  << endl;
    cout << "[4] callback test function 4" << endl;
    cin.getline(buf, 80);

    try
    {
      switch (buf[0])
	{
	case '1': pif.registerDomainForUpdate(hostport, domain, callback1);
	  break;
	case '2': pif.registerDomainForUpdate(hostport, domain, callback2);
	  break;
	case '3': pif.registerDomainForUpdate(hostport, domain, callback3);
	  break;
	case '4': pif.registerDomainForUpdate(hostport, domain, callback4);
	  break;
	default: 
	  cout << "Unknown the choice.  Default is test function 1" << endl;
	  pif.registerDomainForUpdate(hostport, domain, callback1);
	  break;
	}
    }
    catch (VException& e)
    {
	cout << "Catching exception..." << e.getDescription().c_str() << endl;
	return;
    }

    cout << "=============================================" << endl;
    cout << "===>  Register Domain For Update result  <===" << endl;
    cout << "=============================================" << endl;

    cout << "Successful Register !!!" << endl;



}

void callback1(const string& domain, const vector<string>& keyList)
{
    cout << "*************************************************" << endl;
    cout << "***   Callback Function 111111 gets called    ***" << endl;
    cout << "         domain = " << domain.c_str() << endl;
    cout << "*************************************************" << endl;
}

void callback2(const string& domain, const vector<string>& keyList)
{
    cout << "*************************************************" << endl;
    cout << "***   Callback Function 222222 gets called    ***" << endl;
    cout << "         domain = " << domain.c_str() << endl;
    cout << "*************************************************" << endl;
}

void callback3(const string& domain, const vector<string>& keyList)
{
    cout << "*************************************************" << endl;
    cout << "***   Callback Function 333333 gets called    ***" << endl;
    cout << "         domain = " << domain.c_str() << endl;
    cout << "*************************************************" << endl;
}

void callback4(const string& domain, const vector<string>& keyList)
{
    cout << "*************************************************" << endl;
    cout << "***   Callback Function 444444 gets called    ***" << endl;
    cout << "         domain = " << domain.c_str() << endl;
    cout << "*************************************************" << endl;
}


void UnRegisterDomain(ProvisioningInterface& pif)
{
    char buf[80]; memset(buf, '\0', 80);
    string hostport;

    ReadDomain();
    ReadHostPort(hostport);

    cout << "Which callback test function to unregister ? " << endl << endl;
    cout << "[1] callback test function 1" << endl;
    cout << "[2] callback test function 2" << endl;
    cout << "[3] callback test function 3"  << endl;
    cout << "[4] callback test function 4" << endl;
    cin.getline(buf, 80);

    try
    {
      switch (buf[0])
	{
	case '1': pif.unRegisterDomainForUpdate(hostport, domain, callback1);
	  break;
	case '2': pif.unRegisterDomainForUpdate(hostport, domain, callback2);
	  break;
	case '3': pif.unRegisterDomainForUpdate(hostport, domain, callback3);
	  break;
	case '4': pif.unRegisterDomainForUpdate(hostport, domain, callback4);
	  break;
	default: 
	  cout << "Unknown the choice.  Default is test function 1" << endl;
	  pif.unRegisterDomainForUpdate(hostport, domain, callback1);
	  break;
	}
    }
    catch (VException& e)
    {
	cout << "Catching exception..." << e.getDescription().c_str() << endl;
	return;
    }

    cout << "===============================================" << endl;
    cout << "===> Un Register Domain For Update result  <===" << endl;
    cout << "===============================================" << endl;

    cout << "Successful UnRegister !!!" << endl;



}
