/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const RegisterRecord_cxx = "$Id: RegisterRecord.cxx,v 1.2 2001/09/19 23:16:39 maivu Exp $";

#include "Util.h"


void RegisterRecord(ProvisioningInterface& pif)
{
    string hostport;
    ReadHostPort(hostport);
    ReadDomain();
    
    string filter;
    vector<string> keyList;
    ReadKeyList(keyList);

    try
    {
	pif.registerRecordForUpdate(hostport, domain, keyList, UpdateRecord);
    }
    catch (VException& e)
    {
	cout << "Catching exception..." << e.getDescription().c_str() << endl;
	return;
    }
    cout << "=============================================" << endl;
    cout << "===>  Register Record For Update result  <===" << endl;
    cout << "=============================================" << endl;

    cout << "Successful Register !!!" << endl;

}

void UpdateRecord(const string& domain, const vector<string>& keyList)
{
    cout << "********************************************************" << endl;
    cout << "***   Callback Function Update Record gets called    ***" << endl;
    cout << "         domain = " << domain.c_str() << ", key = "
	 << keyList[0].c_str() << endl;
    cout << "********************************************************" << endl;

}
void UnRegisterRecord(ProvisioningInterface& pif)
{
    string hostport;
    ReadHostPort(hostport);
    ReadDomain();

    string filter;
    vector<string> keyList;
    ReadKeyList(keyList);

    try
    {
	pif.unRegisterRecordForUpdate(hostport, domain, keyList, UpdateRecord);
    }
    catch (VException& e)
    {
	cout << "Catching exception..." << e.getDescription().c_str() << endl;
	return;
    }
    cout << "===============================================" << endl;
    cout << "===> Un Register Record For Update result  <===" << endl;
    cout << "===============================================" << endl;

    cout << "Successful UnRegister !!!" << endl;

}
