/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const SessionSearchDB_cxx = "$Id: SessionSearch.cxx,v 1.2 2001/09/19 23:16:39 maivu Exp $";

#include "Util.h"

void SessionSearchDB(ProvisioningInterface& pif)
{
    if (domain.empty() || !(isSessionOpen) )
      {
	cout << "Please Select Session Open First!" << endl;
	return;
      }

    if (fieldList.size() == 0)
      {
	GetFields(pif);
      }

    PrintFieldList();
    FieldListtype aList;
    ReadFieldListRequest(aList);


    string filterRule;
    ReadFilterRule(filterRule);
    filter = filterRule;

    char buf[80]; memset(buf, 0, 80);
    string sortField;
    PSCommon::PSSortType sortType;

    cout << endl << "Do you want to sort by: [a]scending or [d]escending ??? : ";
    cin.getline(buf, 80);

    if ((!strcmp(buf, "a")) || (!strcmp(buf, "A")))
      sortType = PSCommon::SORT_ASCENDING;
    else if ((!strcmp(buf, "d")) || (!strcmp(buf, "D")))
      sortType = PSCommon::SORT_DESCENDING;
    else
      sortType = PSCommon::SORT_NONE;

    cout << endl << "Enter the sort field: ";
    memset(buf, 0, 80);
    cin.getline(buf, 80);
    sortField = buf;

    Sptr<PSDBResult> result = 0;
    try
    {
        result = pif.sessionSearchDB(sessionID, domain, filterRule, aList,
				     sortType, sortField);
    }
    catch (VException& e)
    {
	cout << "Catching exception..." << e.getDescription().c_str() << endl;
	return;
    }

    cout << "=========================================" << endl;
    cout << "===>  Session Search DB Test Result  <===" << endl;
    cout << "=========================================" << endl;

    string error = result->getError();
    if (error.size() != 0)
      {
	cout << "- Error: " << error << ". Stop !!!" << endl;
	return;
      }

    sessionSearchResultCount = result->getNumMatches();

    cout << "- Session ID: " << sessionID.c_str() << endl;
    cout << "- Result Count: " << sessionSearchResultCount << endl;

}

