/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const Util_cxx = "$Id: Util.cxx,v 1.1.1.1 2001/08/21 23:30:24 dabryan Exp $";

#include "Util.h"


// Some Global variables

FieldListtype fieldList;
string sessionID = "";
string domain = "";
int sessionSearchResultCount = 0;
bool isSessionOpen = false;
string filter = "";

char buf[80];

void ReadHostPort(string& hostport)
{
    memset(buf, '\0', 80);
    cout << "Enter \"host:port\" : [localhost:6006] ";

    cin.getline(buf, 80);
    if (buf[0] == '\0')
      hostport = "localhost:6006";
    else
      hostport = buf;

    cout << endl;

}

void ReadDomain()
{
    memset(buf, 0, 80);
    cout << "Enter the domain: ";
    if (!domain.empty())
      cout << "[" << domain << "] ";

    cin.getline(buf, 80);
    if (buf[0] != 0) domain = buf;
    cout << endl;

}

void ReadFieldListRequest(FieldListtype & aList)
{
    cout << "------------------------------------------" << endl;
    cout << "*     Enter the request fields list      *" << endl;
    cout << "------------------------------------------" << endl;

//    PrintFieldList();

    cout << "\n==>> Choose the number from the fields list <<==" << endl;
    cout << "==>>         then <enter> or \"end\"          <<==" << endl;

    memset(buf, 0, 80);
    unsigned short index = 0;
    cin.getline(buf, 80);
    while (strcmp(buf, "end"))
      {
	if (!strcmp(buf, "*"))
	  aList.push_back(buf);
	else
	  {
	index = atoi(buf);
	if (index > fieldList.size() || index == 0)
	  cout << "Wrong field list. Enter again!!!" << endl;
	else
	  aList.push_back(fieldList[index - 1]);
	  }
	memset(buf, 0, 80);
	cin.getline(buf, 80);	
      }

}


void ReadFilterRule(string & filterRule)
{
    cout << "------------------------------------" << endl;
    cout << "* Enter the filter rule            *" << endl;
    cout << "* For ex:    vocalname=\"1000\"      *" << endl;
    cout << "------------------------------------" << endl;

    filterRule = "(";
    memset (buf, 0, 80);
    cin.getline(buf, 80);
    filterRule += buf;
    filterRule += ")";
    cout << "FILTER RULE = " << filterRule << endl;
}


void ReadTableRequest(tableType & aTable)
{
    char buf[80];
    memset(buf, 0, 80);

    cout << "----------------------------------------" << endl;
    cout << "* Enter keyword and value or \"end\":    *" << endl;
    cout << "*  For ex;    vocalname,value          *" << endl;
    cout << "----------------------------------------" << endl;

    cin.getline(buf, 80);
    string aLine, tag, val;

    while (strcmp(buf, "end"))
      {
	aLine = buf;
	string::size_type comma = aLine.find(",");
	if (comma != string::npos)
	{
	    tag = aLine.substr(0, comma);
	    val = aLine.substr(comma + 1, aLine.size());
	    aTable[tag] = val;
	}
	memset(buf, 0, 80);
        cin.getline(buf, 80);
      }

}



void PrintFieldList()
{

    cout << "Here is the fields list from domain " << domain << ": " << endl;
    for (unsigned short u = 0; u < fieldList.size(); u++)
    {
	cout << u+1 << " ==> " << fieldList[u] << endl;
    }
    cout << endl;
}



void PrintGetResult(Sptr <PSDBResult> result)
{

    ConstTableIter  itr;
    int num = result->getNumMatches();

    for (int i = 0; i < num; i++)
      {
	cout << "Match " << i+1 << ":" << endl;
	const tableType& record = result->getTable( i );
	for (itr = record.begin(); itr != record.end(); itr++)
	  {
	    cout << "    " << itr->first << " = " << itr->second << endl;
	  }
      }
}

void ReadKeyList(vector<string>& keyList)
{
    cout << "-----------------------------" << endl;
    cout << "* Enter the key list:       *" << endl;
    cout << "* or \"end\"                   " << endl;
    cout << "-----------------------------" << endl;

    memset(buf, 0, 80);
    cin.getline(buf, 80);
    while (strcmp(buf, "end"))
      {
	keyList.push_back(buf);
	memset(buf, 0, 80);
	cin.getline(buf, 80);
      }

}
