/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const proxyServer_cxx = "$Id: proxyServer.cxx,v 1.1.1.1 2001/08/21 23:30:24 dabryan Exp $";

#include "Util.h"


void ProxyTest (ProvisioningInterface& pif);



int main(const int argc, const char** argv)
{

    cout << endl;
    cout << "|===============================|" << endl;
    cout << "|        PROXY SERVER DEMO      |" << endl;
    cout << "|===============================|" << endl << endl;


    string pServer("localhost"); //, debug("off");
    int port = 6005;
    string logLevel;
    char buf[80];
    memset(buf, 0, 80);

    cout << "==== Connecting to Provision Server ====" << endl;
    cout << "Enter Provision Server name: [localhost]";
    cin.getline(buf, 80);

    if (buf[0] != 0)
      pServer = buf;

    cout << "Enter Provision Server port: [6005]";
    memset(buf, 0, 80);
    cin.getline(buf, 80);

    if (buf[0] != 0)
      port = atoi(buf);

    /*
    cout << "Turn LOG_DEBUG on/off: ";
    cin >> logLevel;

    if (logLevel.c_str() == "on")
      {
	cout << "Open the proxyTest.log to view the log debug" << endl;
	cpLogOpen("proxyTest.log");
	cpLogSetPriority(LOG_DEBUG);
      }
    */
    try
      {
	PSIFCreator psFactory;

	ProvisioningInterface& pif = psFactory.getPSIFInstance(pServer.c_str(), port, pServer.c_str(), port);
	
	ProxyTest(pif);
      }
    catch (VException& e)
      {
	//cpLog (LOG_DEBUG, "Catching exception");
	cout << "Catching exception: " << e.getDescription().c_str() << endl;
	return 1;
      }
    
    cout << endl << "Done the Proxy Server Demo." << endl;
    cout << "Goodbye!!!" << endl << endl;
            
    return 0;
}


void ProxyTest (ProvisioningInterface& pif)
{
    string str, choice = "y";
    char buf[80]; memset(buf, 0, 80);

    while (choice == "y")
    {
	GetRecords(pif);

	cout << endl << "Continue [y/n] ??? : [y] ";
	memset(buf, 0, 80);
	cin.getline(buf, 80);
	if (buf[0] == 0) choice = "y";
	else choice = buf;

    }

    

}


/*
void ProxyTest (ProvisioningInterface& pif)
{
    char choice;

    while (true)
      {
	ShowMenu();
	cout << "Select the test: ";
	cin >> choice;
	cout << endl;

	switch (choice)
	  {
	  case '1': GetRecordsInDomain(pif);
	    break;
	  case '2': GetFieldsInDomain(pif);
	    break;
	  case '3': AutomatedTest(pif);
	    break;
	  case '4': SessionOpen(pif);
	    break;
	  case '5': SessionClose(pif);
	    break;
	  case '9':
	    return;
	  default:
	    cout << "Don't know the selection.  Choose again." << endl;
	  }
	
      }


}



void ShowMenu()
{
    cout << endl;
    cout << "=================================" << endl;
    cout << "|        PROXY TEST             |" << endl;
    cout << "|                               |" << endl;
    cout << "| 1  -- Get Records In Domain   |" << endl;
    cout << "| 2  -- Get Fields In Domain    |" << endl;
    cout << "| 3  -- Automated Test          |" << endl;
    cout << "| 4  -- Session Open            |" << endl;
    cout << "| 5  -- Session Close           |" << endl;
    cout << "|(999) -- END                   |" << endl;
    cout << "|===============================|" << endl << endl;

}

void AutomatedTest(ProvisioningInterface& pif)
{

    cout << "====================================" << endl;
    cout << "===>   Automated Test Result    <===" << endl;
    cout << "====================================" << endl;

    // print out ....
    fieldList.clear();
    cout << " First, Get all fields in the domain: vocaluser" << endl << endl;
    GetFieldsInDomain(pif);

    cout << "\nSecond, get all records." << endl;
    Sptr<PSDBResult> result = 0;

    try 
    {
        result = pif.getRecordsInDomain("vocaluser", 
					"(vocalname=\"*\")", 
					fieldList);

    }
    catch (VException& e)
    {
        cpLog (LOG_ERR, "Failed GetRecordsInDomain: ", e.getDescription().c_str());
    }

    if (result == 0)
      return;

    string error = result->getError();
    if (error.size() != 0)
    {
	cout << "-Error: " << error << endl;
	return;
    }
    
    ConstTableIter  itr;
    for (unsigned short u = 0; u < result->getNumMatches(); u++)
    {
        const tableType& record = result->getTable( u );
	cout << " ******** " << endl;
        for (itr = record.begin(); itr != record.end(); itr++)
        {
	    cout << u << " - " << itr->first.c_str() << " - " <<
	      itr->second.c_str() << endl;
        }
    }
}    
    
*/
