/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const psClientTest_cxx_Version =
    "$Id: psClientTest.cxx,v 1.2 2001/08/23 21:47:41 maivu Exp $";
/*
 *
 * psClientTest.cxx
 *
 */


// Copyright (C) 2000 by Vovida Networks, Inc.

#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include <string>

#include <util/PlatformUtils.hpp>

#include "CommandLine.hxx"
#include "testProvisioningInterface.h"
#include "testPSDBResult.h"
#include "testPSDBRequest.h"
#include "testPSXML.h"
#include "XmlContent.hxx"
#include "testPSRspXmlContentHandlers.h"
#include "testPSReqXmlContentHandlers.h"
#include "VException.hxx"
#include "VNetworkException.hxx"
#include "cpLog.h"
#include "Verify.hxx"

/*
#include "AutomatedTest.h"
*/


char choice[12];
int  intChoice = 0;

static  Sptr<CommandLine> cIn;   // handle command line args


void
ShowMenu()
{
    cout << endl;
    cout << "********************************************************" << endl;
    cout << "******   Test ProvisioningInterface API's         ******" << endl;
    cout << "********************************************************" << endl;
    cout << "(1) getRecordsInDomain          (9) sessionSearchDB" << endl;
    cout << "(2) getFieldsInDomain           (10) sessionGetDBResult" << endl;
    cout << "(3) setRecordsInDomain          (11) sessionSetDBResult" << endl;
    cout << "(4) addRecordInDomain           (12) sessionSortDBResult" << endl;
    cout << "(5) deleteRecordsInDomain       (13) sessionDeleteAll" << endl;
    cout << "(6) getCurrentRecordsInDomain   (14) getXMLDescription"<< endl;
    cout << "(7) sessionOpen                 (15) listXMLCatagories"   << endl;
    cout << "(8) sessionClose                (16) listXMLDescriptions"  << endl;
    cout << "********************************************************" << endl;
    cout << "(20) automated testing" << endl;
    cout << "(21) test PS DB Result's API" << endl;
    cout << "(22) test PS DB Request " << endl;
    cout << "(23) test ProvisioningInterface connectivity" << endl;
    cout << "(24) test PSReqXmlContentHandlers" << endl;
    cout << "(25) test PSRspXmlContentHandlers" << endl;
    cout << "(26) test XML building/parsing" << endl;

    cout << "(Q)ut (Exit, Quit)" << endl;

    cout << "Enter your choice: ";
    cin >> choice;
    cout << endl;

    if ((choice[0] == 'q') || (choice[0] == 'Q'))
    {
        intChoice = 99;
    }
    else {
        intChoice = atoi(choice);
    }
}

/** usage: ./psClientTest -d vvs-abc:6005 vvs-xyz:6005 xxx
 */
int
main(int argc, char** argv)
{

    cIn = CommandLine::instance(argc, (const char**)argv,
            "Usage: psClientTest -d pserver:port bkpserver:port xxxx");
    string pHost = cIn->getString("pshostname");
    int pPort = cIn->getInt("psremoteport");
    string altHost = cIn->getString("psbackuphostname");
    int altPort = cIn->getInt("psbackupremoteport");

    cpLogSetPriority (LOG_DEBUG);

    cpLog(LOG_DEBUG, "pHost=%s, pPort=%d, altHost=%s, altPort=%d",
          pHost.c_str(), pPort, altHost.c_str(), altPort);

    if (altHost.size()==0)
    {
        altHost = pHost;
        altPort = pPort;
    }

    cout << endl;
    cout << "*************************************" << endl;
    cout << "*                                   *" << endl;
    cout << "*        Test Program               *" << endl;
    cout << "*                                   *" << endl;
    cout << "* Note: auto default LOG_DEBUG      *" << endl;
    cout << "* ./psClientTest -d h1:p1 h2:p2 xx  *" << endl;
    cout << "*                                   *" << endl;
    cout << "*************************************" << endl << endl;

    int num = 0;    // number of test iterations

    while (true)
    {
        ShowMenu();

        // Initialize the XML4C2 system - should only do it once/process
        try
        {
             XMLPlatformUtils::Initialize();
        }

        catch (const XMLException& toCatch)
        {
             cerr << "Error during initialization! :\n"
                  << StrX(toCatch.getMessage()) << endl;
             return test_return_code(num);//,0);  // expect all to pass, no failure
        }

        switch (intChoice)
        {
            case 20:     /* auto test all cases */
               for (int testCase = 1; testCase <= 13; testCase++)
               {
                   test_verify(testProvisioningInterface( testCase,
                      pHost.c_str(), pPort, altHost.c_str(), altPort) == 0);
                   num++;
               }
               test_verify( testPSDBResult() == 0 );   num++;
               test_verify( testPSDBRequest() == 0 );  num++;
               return test_return_code(num);//,0);
               
            break;
            case 21 :
                test_verify( testPSDBResult() == 0 );   // test succeeds?
                num++;
                
            break;
            case 22 :
                test_verify( testPSDBRequest() == 0 ); 
                num++;
            break;
            case 23 :
                // test ProvisioningInterface connectivity:
                test_verify( testProvisioningInterface( 1,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );  
                num++;
                
            break;
            case 24 :   // PS request (Xerces SAX) handler
                test_verify( testPSReqXmlContentHandlers() == 0 );   // test succeeds?
                num++;
                
            break;
            case 25 :   // PS response (Xerces SAX) handler
                test_verify( testPSRspXmlContentHandlers() == 0 );   // test succeeds?
                num++;
                
            break;
            case 26:   // build an xml document using Libxml
                test_verify( testPSXMLBuild() == 0 );   
                num++;
            break;
            case 1:    // getRecordsInDomain request
                test_verify( testProvisioningInterface( 1,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;

            break;
            case 2:    // getFieldsInDomain
                test_verify( testProvisioningInterface( 2,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 3:    // setRecordsInDomain
                test_verify( testProvisioningInterface( 3,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 4:   // addRecordInDomain
                test_verify( testProvisioningInterface( 4,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 5:   // deleteRecordsInDomain
                test_verify( testProvisioningInterface( 5,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 6:   // getCurrentRecordsInDomain
                test_verify( testProvisioningInterface( 6,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 7:   // sessionOpen
                test_verify( testProvisioningInterface( 7,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 8:   // sessionClose
                test_verify( testProvisioningInterface( 8,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 9:   // sessionSearchDB
                test_verify( testProvisioningInterface( 9,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 10:  // sessionGetDBResult
                test_verify( testProvisioningInterface( 10,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 11:  // sessionSetDBResult
                test_verify( testProvisioningInterface( 11,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 12:  // sessionSortDBResult 
                test_verify( testProvisioningInterface( 12,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 13:  // sessionDeleteAll
                test_verify( testProvisioningInterface( 13,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 14:  // sessionDeleteAll
                test_verify( testProvisioningInterface( 14,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 15:  // sessionDeleteAll
                test_verify( testProvisioningInterface( 15,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 16:  // sessionDeleteAll
                test_verify( testProvisioningInterface( 16,
                    pHost.c_str(), pPort, altHost.c_str(), altPort) == 0 );
                num++;
            break;
            case 99:                // out, exit, quit, terminate
                // And call the termination method
                XMLPlatformUtils::Terminate();
                return test_return_code(num);//,0);  // expect all to pass, no failure
            break;

            default:
                cout << "unknown choice: " << choice << ". Stop!!!" << endl;
                // And call the termination method
                XMLPlatformUtils::Terminate();
                return test_return_code(num);//,0);
            break;
        }
        
    }

}
