/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const testPSDBRequest_cxx  = "$Id: testPSDBRequest.cxx,v 1.1.1.1 2001/08/21 23:30:24 dabryan Exp $";


#include "cpLog.h"
#include "testPSDBRequest.h"
#include "PSCommon.hxx"
#include "PSDBRequest.hxx"
#include "PSProtocolService.hxx"
#include "PSDBCommonType.h"

using namespace Vocal;


// test PSDBRequest API 
int
testPSDBRequest()
{
    try
    {

        // build a canned request
        //list<string> myFieldList;
        FieldListtype myFieldList;
        myFieldList.push_back("uid");
        myFieldList.push_back("phone");
        myFieldList.push_back("foo");
        myFieldList.push_back("bar");

        map<string, string> myKVPairs;
        myKVPairs["foo"] = "bar";
        myKVPairs["defaultProxy"] = "69.4.127.15";
        myKVPairs["authentification"] = "none";

        PSProtocolService& proto = PSProtocolService::getInstance();

        cout << "\n1) *** test building/ parsing GetRecordsInDomainReq :" << endl;
        Sptr< Http::Message > req = proto.buildGetRecordsInDomainReq(
                            "Accounts", "uid=200*", myFieldList );

        // now parse the XML content
        const string& xmlBody1 = req->body();
        Sptr<PSDBRequest> myRequest = proto.parseRequest( 
                            PSMessage::GET_RECORDS_IN_DOMAIN_REQ, xmlBody1 );

        cout << "\n2) *** test building/parsing GetFieldsInDomain :" << endl;
        req = proto.buildGetFieldsInDomainReq( "Marshals" );

        const string& xmlBody2 = req->body();
        myRequest = myRequest = proto.parseRequest(
                      PSMessage::GET_FIELDS_IN_DOMAIN_REQ, xmlBody2 );
 
        cout << "\n3) *** test building/parsing SetRecordsInDomain :" << endl;
        req = proto.buildSetRecordsInDomainReq("Accounts", 
                        "uid=100*",myKVPairs );

        const string& xmlBody3 = req->body();
        myRequest = myRequest = proto.parseRequest(
                      PSMessage::SET_RECORDS_IN_DOMAIN_REQ, xmlBody3 );

        cout << "\n4) *** test building/parsing AddRecordInDomain :" << endl;
        myKVPairs["uid"] = "Wendy";
        myKVPairs["phone"] = "5068";
        req = proto.buildAddRecordInDomainReq("Accounts", myKVPairs );

        const string& xmlBody4 = req->body();
        myRequest = myRequest = proto.parseRequest(
                      PSMessage::ADD_RECORD_IN_DOMAIN_REQ, xmlBody4 );

        cout << "\n5) *** test building/parsing DeleteRecordsInDomain :" << endl;
        req = proto.buildDeleteRecordsInDomainReq("Accounts", "uid=100*" );

        const string& xmlBody5 = req->body();
        myRequest = myRequest = proto.parseRequest(
                      PSMessage::DEL_RECORDS_IN_DOMAIN_REQ, xmlBody5 );

        cout << "\n6) *** test building/parsing GetCurrentRecordsInDomain :" << endl;
        cout << "*********** Not supported ************" << endl;
	//req = proto.buildGetCurrentRecordsInDomainReq("Accounts", "uid=100*",
	//myFieldList );

        const string& xmlBody6 = req->body();
        myRequest = myRequest = proto.parseRequest(
                      PSMessage::GET_CUR_RECORDS_IN_DOMAIN_REQ, xmlBody6 );


        cout << "\n7) *** test building/parsing SessionOpen :" << endl;
        req = proto.buildSessionOpenReq(1240);   // expiry time = 1240 seconds

        const string& xmlBody7 = req->body();
        myRequest = myRequest = proto.parseRequest(
                      PSMessage::SESSION_OPEN_REQ, xmlBody7 );

        cout << "\n8) *** test building/parsing SessionClose :" << endl;
        req = proto.buildSessionCloseReq("34567");

        const string& xmlBody8 = req->body();
        myRequest = myRequest = proto.parseRequest(
                      PSMessage::SESSION_CLOSE_REQ, xmlBody8 );

        cout << "\n9) *** test building/parsing SessionSearchDB :" << endl;
        myFieldList.clear();
        string token = "12345";
        string domain = "Marshals";
        string filter = "host=vvs-honda";
        PSCommon::PSSortType sType = PSCommon::SORT_ASCENDING;
        string sortField = "port";

        myFieldList.push_back("gateway");
        myFieldList.push_back("retryTimer");
        myFieldList.push_back("bridgeNumber");
        myFieldList.push_back("foobar");
        //req = proto.buildSessionSearchDBReq(token, domain, filter, 
	//myFieldList, sType, sortField );

        const string& xmlBody9 = req->body();
        myRequest = myRequest = proto.parseRequest(
                      PSMessage::SESSION_SEARCH_DB_REQ, xmlBody9 );

        cout << "\n10) *** test building/parsing SessionGetDBResult :" << endl;
        //req = proto.buildSessionGetDBResultReq(token, startIdx, endIdx,
	//myFieldList );

        const string& xmlBody10 = req->body();
        myRequest = myRequest = proto.parseRequest(
                      PSMessage::SESSION_GET_DB_RESULT_REQ, xmlBody10 );

        cout << "\n11) *** test building/parsing SessionSetDBResult :" << endl;
        myKVPairs.clear();
        myKVPairs["foo"] = "bar";
        myKVPairs["defaultproxy"] = "0.0.0.0";
        myKVPairs["authentification"] = "none";

        req = proto.buildSessionSetDBResultReq( token, myKVPairs );

        const string& xmlBody11 = req->body();
        myRequest = myRequest = proto.parseRequest(
                      PSMessage::SESSION_SET_DB_RESULT_REQ, xmlBody11);

        cout << "\n12) *** test building/parsing SessionSortDBResult :" << endl;
        sType = PSCommon::SORT_DESCENDING;
        req = proto.buildSessionSortDBResultReq( token, sType, sortField ); 

        const string& xmlBody12 = req->body();
        myRequest = myRequest = proto.parseRequest(
                      PSMessage::SESSION_SORT_DB_RESULT_REQ, xmlBody12 );

        cout << "\n13) *** test building/parsing SessionDeleteAll :" << endl;
        req = proto.buildSessionDeleteAllReq( token );   

        const string& xmlBody13 = req->body();
        myRequest = myRequest = proto.parseRequest(
                      PSMessage::SESSION_DELETE_ALL_REQ, xmlBody13 );


    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }

    return 0;

}

/* 
void
ospServers()
{
    cout << "\n===> Osp Servers <===\n";

    int ch;
    while (true)
    {
        cout << "\n(1) Connect to pServer" << endl;
        cout << "(2) Get Osp Server Data" << endl;
        cout << "(999) Exit" << endl;
        cout << "Enter your choice: ";
        cin >> ch;
        cout << endl;

        switch (ch)
        {
            case 1:
            connectPserver();
            break;
            case 2: getOspServerData();
            break;
            case 999: return ;
            break;
            default:
            cout << "Don't know the option.\n\n";
            break;
        }

    }
}
*/
