/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999-2000 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xerces" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache\@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation, and was
 * originally based on software copyright (c) 1999, International
 * Business Machines, Inc., http://www.ibm.com .  For more information
 * on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */



// ---------------------------------------------------------------------------
//  Includes
// ---------------------------------------------------------------------------
#include <util/PlatformUtils.hpp>
#include <util/TransService.hpp>
#include <sax2/SAX2XMLReader.hpp>
#include <sax2/XMLReaderFactory.hpp>
#include <framework/MemBufInputSource.hpp> 

#include "cpLog.h"
#include "Sptr.hxx"
#include "PSCommon.hxx"
#include "PSDBResult.hxx"
#include "PSRspXmlContentHandlers.hxx"
#include "testPSRspXmlContentHandlers.h"


// ---------------------------------------------------------------------------
//  Local data
//
//  encodingName
//      The encoding we are to output in. If not set on the command line,
//      then it is defaulted to LATIN1.
//
//  xmlFile
//      The path to the file to parser. Set via command line.
//
//  valScheme
//      Indicates what validation scheme to use. It defaults to 'auto', but
//      can be set via the -v= command.
//
//	expandNamespaces
//		Indicates if the output should expand the namespaces Alias with
//		their URI's, defaults to false, can be set via the command line -e
// ---------------------------------------------------------------------------
static const char*              encodingName    = "LATIN1";
// static XMLFormatter::UnRepFlags unRepFlags      = XMLFormatter::UnRep_CharRef;
static char*                    xmlFile         = 0;
static SAX2XMLReader::ValSchemes valScheme      = SAX2XMLReader::Val_Auto;
static bool						expandNamespaces= false ;

static const char*  gMemBufId = "psInfo";
static const char*  gXMLInMemBuf = "\
<response>\n\
<domain> Accounts </domain>\n\
<resultcount> 5 </resultcount>\n\
<match>\n\
<fieldvalue>\n\
<uid>1001</uid>\n\
<phone>4083831001</phone>\n\
</fieldvalue>\n\
</match>\n\
<match>\n\
<fieldvalue>\n\
<uid>1002</uid>\n\
<phone>4083839999</phone>\n\
<foo>bar</foo>\n\
</fieldvalue>\n\
</match>\n\
<match>\n\
<fieldvalue>\n\
<uid>1003</uid>\n\
<phone>4083831003</phone>\n\
<foo>bar</foo>\n\
</fieldvalue>\n\
</match>\n\
<match>\n\
<fieldvalue>\n\
<uid>1114</uid>\n\
<phone>4083831004</phone>\n\
<foo>bar</foo>\n\
</fieldvalue>\n\
</match>\n\
</response>\n\
<match>\n\
<fieldvalue>\n\
<uid>1005</uid>\n\
<phone>4083831005</phone>\n\
<foo>bar</foo>\n\
</fieldvalue>\n\
</match>\n\
</response>";



// ---------------------------------------------------------------------------
//  Local helper methods
// ---------------------------------------------------------------------------
static void usage()
{
    cout <<  "\nUsage: testPSRspXmlContentHandlers file\n"
             "This program prints the data returned by the various SAX2\n"
             "handlers for the specified input data.\n"
             "The parser has intrinsic support for the following encodings:\n"
             "    UTF-8, USASCII, ISO8859-1, UTF-16[BL]E, UCS-4[BL]E,\n"
             "    WINDOWS-1252, IBM1140, IBM037\n"
         <<  endl;
}



// ---------------------------------------------------------------------------
//  Program 
// ---------------------------------------------------------------------------
int testPSRspXmlContentHandlers()
{


    //
    //  Create a SAX parser object. Then, according to what we were told on
    //  the command line, set it to validate or not.
    //
    SAX2XMLReader* parser = XMLReaderFactory::createXMLReader();
	if (valScheme == SAX2XMLReader::Val_Auto)
	{
	  parser->setFeature(XMLString::transcode("http://xml.org/sax/features/validation"), true);
	  parser->setFeature(XMLString::transcode("http://apache.org/xml/features/validation/dynamic"), true);
	}

    //
    //  Create the handler object and install it as the document and error
    //  handler for the parser. Then parse the file and catch any exceptions
    //  that propogate out
    //
    Sptr<PSDBResult> target= new 
              PSDBResult( PSMessage::GET_RECORDS_IN_DOMAIN_RSP);
    try
    {
        PSRspXmlContentHandlers handler( target );
        parser->setContentHandler(&handler);
        parser->setErrorHandler(&handler);
        MemBufInputSource* memBufIS = new MemBufInputSource (
            (const XMLByte*)gXMLInMemBuf, strlen(gXMLInMemBuf), gMemBufId,
             false);

        // parser->parse(xmlFile);
        parser->parse(*memBufIS);
        delete parser;
    }
    catch (const XMLException& toCatch)
    {
        cerr << "\nAn error occured\n  Error: "
             << StrX(toCatch.getMessage())
             << "\n" << endl;
        delete parser;
        return -1;
    }

    // And call the termination method
    // XMLPlatformUtils::Terminate();

    target->print(LOG_DEBUG);

    return 0;
}

