/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const testProvisioningInterface_cxx  = "$Id: testProvisioningInterface.cxx,v 1.2 2001/09/17 23:32:15 maivu Exp $";

#include "testProvisioningInterface.h"
#include "PSCommon.hxx"
#include "PSIFCreator.hxx"


int tc1( ProvisioningInterface& pif )
{
    try {
        vector<string> myFieldList;
	myFieldList.push_back("Number_Services");
        //myFieldList.push_back("vocalname");
        //myFieldList.push_back("fieldvalue");

        Sptr<PSDBResult> res;
        res = pif.getRecordsInDomain("Carafe", "(hostport='*')", myFieldList);
				     //"vocaluser","(vocalname='100*')",myFieldList);

    }
    catch (VException& e)
    {
        cout << "Test Case 1 exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
}

int tc2( ProvisioningInterface& pif )
{
    try {

        Sptr<PSDBResult> res;
        res = pif.getFieldsInDomain("Accounts");

    }
    catch (VException& e)
    {
        cout << "Test Case 2 exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
}


int tc3( ProvisioningInterface& pif )
{
    try {
        map<string, string> myKVPairs;
        myKVPairs["Number_Services"] = 
"<table>
  <row>
    <Number_Services_Contact>911</Number_Services_Contact>
    <Number_Services_Priority>EMERGENCY</Number_Services_Priority>
    <Number_Services_Route_Target>sip-server.cisco.com</Number_Services_Route_Target>
  </row>
</table>";
        //myKVPairs["defaultProxy"] = "69.4.27.3";
        //myKVPairs["authentification"] = "none";

        Sptr<PSDBResult> res;
        res = pif.setRecordsInDomain("Carafe","(hostport='*')",myKVPairs);

    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }

  /*
    try {
        map<string, string> myKVPairs;
        myKVPairs["foo"] = "bar";
        myKVPairs["defaultProxy"] = "69.4.27.3";
        myKVPairs["authentification"] = "none";

        Sptr<PSDBResult> res;
        res = pif.setRecordsInDomain("Accounts","uid=100*",myKVPairs);

    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
  */
    return 0;
}

int tc4( ProvisioningInterface& pif )
{
    try {
        map<string, string> myKVPairs;
        myKVPairs["vocalname"] = "<level1><level2>abc</level2><level2><level3><level4>kadfj</level4><level4>kdfjld</level4></level3><level3>ajfkdg</level3></level2></level1>";

        Sptr<PSDBResult> res;
        res = pif.addRecordInDomain("vocaluser",myKVPairs);

    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
}


int tc5( ProvisioningInterface& pif )
{
    try {

        Sptr<PSDBResult> res;
        res = pif.deleteRecordsInDomain("Accounts","uid=100*");

    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
}

int tc6( ProvisioningInterface& pif )
{
	cout << "Not support this call" << endl;
	return 0;
/*
    try {
        list<string> myFieldList;
        myFieldList.push_back("uid");
        myFieldList.push_back("phone");
        myFieldList.push_back("foo");

        Sptr<PSDBResult> res;
        res = pif.getCurrentRecordsInDomain("Accounts","uid=100*",myFieldList);

    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
*/
}


int tc7( ProvisioningInterface& pif )
{
    try {

        string token = "0";
        token = pif.sessionOpen( 120 ); // expiretime = 120 sec
        cout << "token received = %s" << token.c_str() << endl;

    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
}

int tc8( ProvisioningInterface& pif )
{
    try {

        bool rtn = false;
        rtn = pif.sessionClose( "12345" );
        cout << "sessionClose returns " << rtn << endl;

    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
}

int tc9( ProvisioningInterface& pif )
{
    try {
        string token = "12345";
        string domain = "Marshals";
        string filter = "host=vvs-toyota";
        PSCommon::PSSortType sType = PSCommon::SORT_ASCENDING;
        string sortField = "port"; 
        FieldListtype myFieldList;
        myFieldList.push_back("gateway");
        myFieldList.push_back("retryTimer");
        myFieldList.push_back("bridgeNumber");
        myFieldList.push_back("foobar");

        Sptr<PSDBResult> res;
        res = pif.sessionSearchDB( token, domain, filter, myFieldList, 
                     sType, sortField );

    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
}

int tc10( ProvisioningInterface& pif )
{
    try {
        string token = "12345";
        int startIdx = 10;
        int endIdx   = 20;
        FieldListtype myFieldList;
        myFieldList.push_back("uid");
        myFieldList.push_back("phone");

        Sptr<PSDBResult> res;
        res = pif.sessionGetDBResult( token, startIdx, endIdx, myFieldList );
    
    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
}

int tc11( ProvisioningInterface& pif )
{
    try {
        string token = "12345";
        map<string, string> myKVPairs;
        myKVPairs["marshalgroup"] = "hostname";
        myKVPairs["authentification"] = "none";

        Sptr<PSDBResult> res;
        res = pif.sessionSetDBResult(token, myKVPairs );

    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
}


int tc12( ProvisioningInterface& pif )
{
    try {
        string token = "12345";
        PSCommon::PSSortType sType = PSCommon::SORT_DESCENDING;
        string sortField = "port";

        Sptr<PSDBResult> res;
        res = pif.sessionSortDBResult( token, sType, sortField );

    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
}


int tc13( ProvisioningInterface& pif )
{
    try {

        Sptr<PSDBResult> res;
        res = pif.sessionDeleteAll( "12345");

    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
}

int tc14( ProvisioningInterface& pif )
{
    try {

        Sptr<PSDBResult> res;
        res = pif.getXmlDescription(string("servers"), string("GatewayMarshal.xml"));

        
    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
}

int tc15( ProvisioningInterface& pif )
{
    try {

        Sptr<PSDBResult> res;
        res = pif.listXmlCatagories();
        cerr << "back in 15" << endl;

    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
}

int tc16( ProvisioningInterface& pif )
{
    try {

        Sptr<PSDBResult> res;
        res = pif.listXmlDescriptions(string("servers"));

    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }
    return 0;
}


// test ProvisioningInterface connectivity
int
testProvisioningInterface(int tc, const char*  ps1, int port1, 
                           const char*  ps2, int port2 )
{
    int rtn = 0;

    try
    {
        PSIFCreator myPifCreator;
        ProvisioningInterface& pif = myPifCreator.getPSIFInstance(
           ps1, port1, ps2, port2, PSCommon::HTTPXML);

        switch (tc) {
        case 1:
            rtn = tc1( pif );            
        break;
        case 2:
            rtn = tc2( pif );
        break;
        case 3:
            rtn = tc3(pif );
        break;
        case 4:
            rtn = tc4( pif );            
        break;
        case 5:
            rtn = tc5( pif );            
        break;
        case 6:
            rtn = tc6( pif );            
        break;
        case 7:
            rtn = tc7( pif );
        break;
        case 8:
            rtn = tc8( pif );            
        break;
        case 9:
            rtn = tc9( pif );
        break;
        case 10:
            rtn = tc10(pif );
        break;
        case 11:
            rtn = tc11( pif );            
        break;
        case 12:
            rtn = tc12( pif );            
        break;
        case 13:
            rtn = tc13( pif );
        break;
        case 14:
            rtn = tc14( pif );
        break;
        case 15:
            rtn = tc15( pif );
        break;
        case 16:
            rtn = tc16( pif );
        break;
        default:
            cout << "unknown test case: " << tc << endl;
            rtn = -1;
        break;
        }
        
    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
        return -1;
    }

    return rtn;

}

