/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const testProxy_cxx = "$Id: testProxy.cxx,v 1.1.1.1 2001/08/21 23:30:24 dabryan Exp $";

#include "Util.h"


void start (ProvisioningInterface& pif);
void Menu();


int main(const int argc, const char** argv)
{

    cout << endl;
    cout << "|===============================|" << endl;
    cout << "|      PROXY SERVER TEST        |" << endl;
    cout << "|===============================|" << endl << endl;


    string pServer("localhost"); //, debug("off");
    int port = 6005, listenPort = 6006;
    string logLevel;
    char buf[80];
    memset(buf, 0, 80);

    cout << "==== Connecting to Provision Server ====" << endl;
    cout << "Enter Provision Server name: [localhost]";
    cin.getline(buf, 80);

    if (buf[0] != 0)
      pServer = buf;

    cout << "Enter Provision Server port: [6005]";
    memset(buf, 0, 80);
    cin.getline(buf, 80);

    if (buf[0] != 0)
      port = atoi(buf);

    cout << "Enter the listen port: [6006] ";
    memset(buf, 0, 80);
    cin.getline(buf, 80);

    if (buf[0] != 0)
      listenPort = atoi(buf);

    cout << "Turn LOG_DEBUG on/off: [off] ";
    memset(buf, 0, 80);
    cin.getline(buf, 80);

    if (buf[0] != 0)
      {
	//cout << "Open the proxyTest.log to view the log debug" << endl;
	//cpLogOpen("testProxy.log");
	cpLogSetPriority(LOG_DEBUG);
      }

    try
      {
	PSIFCreator psFactory;

	ProvisioningInterface& pif = psFactory.getPSIFInstance(pServer.c_str(), port, pServer.c_str(), port);
	pif.setListenPort(listenPort);
	start(pif);
      }
    catch (VException& e)
      {
	//cpLog (LOG_DEBUG, "Catching exception");
	cout << "Catching exception: " << e.getDescription().c_str() << endl;
	return 1;
      }
    
    cout << endl << "Done the Proxy Test." << endl;
    cout << "Goodbye!!!" << endl << endl;
            
    return 0;
}


void start (ProvisioningInterface& pif)
{
    string choice = "y";
    char buf[80];
    memset(buf, 0, 80);
    int testcase = 0;

    while (choice == "y")
    {
        Menu();

	cout << "Enter the test: ";
	memset(buf, 0, 80);
	cin.getline(buf, 80);
	testcase = atoi(buf);

	switch (testcase)
	  {
	  case 1: GetFields(pif);
	    break;
	  case 2: GetRecords(pif);
	    break;
	  case 3: SetRecords(pif);
	    break;
	  case 4: AddRecords(pif);
	    break;
	  case 5: DeleteRecords(pif);
	    break;
	  case 10: SessionOpen(pif);
	    break;
	  case 11: SessionSearchDB(pif);
	    break;
	  case 12: SessionGetDBResult(pif);
	    break;
	  case 13: SessionSetDBResult(pif);
	    break;
	  case 14: SessionSortDBResult(pif);
	    break;
	  case 15: SessionDeleteAll(pif);
	    break;
	  case 16: SessionClose(pif);
	    break;
	  case 20: RegisterDomain(pif);
	    break;
	  case 21: RegisterRecord(pif);
	    break;
	  case 22: UnRegisterDomain(pif);
	    break;
	  case 23: UnRegisterRecord(pif);
	    break;
	  case 999:
	    return;
	  default:
	    cout << "Don't know the selection.  Choose again." << endl;
	  }

	cout << endl << "Continue [y/n] ??? : [y]";
	memset(buf, 0, 80);
	cin.getline(buf, 80);
	if (buf[0] != 0) choice = buf;
	
	cout << endl;

    }

    

}

void Menu()
{
    cout << endl;
    cout << "===================================" << endl;
    cout << "|        PROXY TEST               |" << endl;
    cout << "|                                 |" << endl;
    cout << "| 1  -- Get Fields In Domain      |" << endl;
    cout << "| 2  -- Get Records In Domain     |" << endl;
    cout << "| 3  -- Set Records In Domain     |" << endl;
    cout << "| 4  -- Add Records In Domain     |" << endl;
    cout << "| 5  -- Delete Records In Domain  |" << endl;
    cout << "|                                 |" << endl;
    cout << "| 10 -- Session Open              |" << endl;
    cout << "| 11 -- Session Search DB         |" << endl;
    cout << "| 12 -- Session Get DB Result     |" << endl;
    cout << "| 13 -- Session Set DB Result     |" << endl;
    cout << "| 14 -- Session Sort DB Result    |" << endl;
    cout << "| 15 -- Session Delete All        |" << endl;
    cout << "| 16 -- Session Close             |" << endl;
    cout << "|                                 |" << endl;
    cout << "| 20 -- RegisterDomainForUpdate   |" << endl;
    cout << "| 21 -- RegisterRecordForUpdate   |" << endl;
    cout << "| 22 -- UnRegisterDomainForUpdate |" << endl;
    cout << "| 23 -- UnRegisterRecordForUpdate |" << endl;
    cout << "|                                 |" << endl;
    cout << "|(999) -- END                     |" << endl;
    cout << "|=================================|" << endl << endl;

}
/*
void AutomatedTest(ProvisioningInterface& pif)
{

    cout << "====================================" << endl;
    cout << "===>   Automated Test Result    <===" << endl;
    cout << "====================================" << endl;

    // print out ....
    fieldList.clear();
    cout << " First, Get all fields in the domain: vocaluser" << endl << endl;
    GetFields(pif);

    cout << "\nSecond, get all records." << endl;
    Sptr<PSDBResult> result = 0;

    try 
    {
        result = pif.getRecords("vocaluser", 
					"(vocalname=\"*\")", 
					fieldList);

    }
    catch (VException& e)
    {
        cpLog (LOG_ERR, "Failed GetRecordsInDomain: ", e.getDescription().c_str());
    }

    if (result == 0)
      return;

    string error = result->getError();
    if (error.size() != 0)
    {
	cout << "-Error: " << error << endl;
	return;
    }
    
    ConstTableIter  itr;
    for (unsigned short u = 0; u < result->getNumMatches(); u++)
    {
        const tableType& record = result->getTable( u );
	cout << " ******** " << endl;
        for (itr = record.begin(); itr != record.end(); itr++)
        {
	    cout << u << " - " << itr->first.c_str() << " - " <<
	      itr->second.c_str() << endl;
        }
    }
}    
    
*/
