#ifndef CACHEMANAGER_HXX
#define CACHEMANAGER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const CacheManager_hxx_Version =
    "$Id: CacheManager.hxx,v 1.1.1.1 2001/08/21 23:30:24 dabryan Exp $";


#include <pthread.h>
#include "VRWLock.hxx"
#include "TimeVal.hxx"
#include "Data.hxx"

#include "PSDBCommonType.h"
#include "DBResultset.hxx"

#include "VMissingDataException.hxx"
/*
#ifdef USE_YARROW
#include "YarrowPseudoRandom.hxx"
#endif

#include "CryptoRandom.hxx"

#define NUM_RANDOMNESS 16  // 128 bits of randomness
*/


typedef struct
{
    string domain;
    string filter;
    string sortType;
    string sortField;
    DBResultset* resultset;
    string sessionId; 
    timeval timeStamp;    
} ProvData;


#if defined(__linux__)
#define PROV_USE_HASHMAP 1
#else
#define PROV_USE_HASHMAP 0
#endif

#if (PROV_USE_HASHMAP == 1)
#include "StringHash.hxx"
#include <hash_map>
typedef hash_map <string, ProvData*, StringHash > ProvRequestMap; 
#else
#include <map>
typedef map < string, ProvData* > ProvRequestMap;
#endif 


class VException;


/** CacheManager manages the caching of the session information for the
 *  provisioning server.
 */
class CacheManager
{
    public:
       /** Default Constructor.  Starts cache thread used for deleteing
        *  entries that exceeds the expire time.
        */
       CacheManager();

       /** Default Destructor.  Does nothing for now.
        */
       ~CacheManager();

       /** Starts the cache thread.
        */
       static void* startCacheManagerThread(void *);

       /** Calls cleanCache() in a specified interval ( CACHE_POLLING_TIME ) 
        *  to remove old cached entries.
        */
       void run();

       /** Generates a time stamp for the cached entry.
        */
       timeval getTimeStamp();        

       /** Generates a session id used for the cached entry.
        */
       string genSessionId();

       /** Adds a new cache entry.
        */
       void addCacheRequest( const string sessionId,
                             const string domain,
                             const string filter,
                             const string sortType,
                             const string sortField,
                             DBResultset* resultset);

       /** Sets all the values in the cache entry.
        */
       int setCache( const string sessionId,
                     const string domain,
                     const string filter,
                     const string sortType,
                     const string sortField );

       /** Similiar to setCache() but allows for modification of selected
        *  parameters.
        */
       int modifyCache( const string sessionId,
                        const string domain,
                        const string filter,
                        const string sortType,
                        const string sortField ); 

       /** Returns a copy of the cached data based on the session id.
        */
       void getCacheRequest( const string sessionId, 
                             ProvData& pData) throw (VMissingDataException&);

       /** Deletes the cached entry.
        */
       int deleteCacheRequest( string sessionId );

       /**
        */
       void showCacheEntry(const string sessionID);

       /** Goes through the cache and cleans up old cached entries.
        */ 
       void cleanCache();

    private:
        /** Stores the cached information for a session.
         *  map< string, ProvData* >
         */
        ProvRequestMap myCache;

        /** Read/Write lock for the cache.
         */
        static VRWLock cacheLock;

        /**  pthread id 
         */
        pthread_t myPid;
};


#endif
