#ifndef FILESTORE_HXX
#define FILESTORE_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const FileStoreVersion =
    "$Id: FileStore.hxx,v 1.1.1.1 2001/08/21 23:30:24 dabryan Exp $";

#include "VRWLock.hxx"
#include "VFileSystem.hxx"

/// Class that store data in file system
class FileStore 
{
    public:
    
        ///
        FileStore( const string &fileRootP );

        ///
        virtual ~FileStore();

        /// get the specified item
        virtual string
        getItem( const string& category, const string& name )
        throw(VException&);

    /// used when no directories are there (or managed by user)
    // there are no other functions for this instance
    virtual string
    getItemRaw(const string& item)
        throw(VException&);
    
        /// get the last modification time for the specfied item
        virtual TimeStamp
        getItemTimeStamp( const string& category, const string& name )
        throw(VException&);

        /// get the size in bytes of the data in the specified item
        virtual int
        getItemSize( const string& category, const string& name )
        throw(VException&);

        /**
         * store the data into the specified item, 
         * it time is not specifeid, current time is used
         */
        virtual void
        putItem( const string& category,
                 const string& name,
                 const string& data,
                 TimeStamp timeStamp = 0 )
        throw(VException&);

        /// Remove the specified item
        virtual void
        removeItem( const string& category, const string& name)
        throw(VException&);

        /// Return true if the specified item exists
        virtual bool
        isItem( const string& category, const string& name)
        throw(VException&);

        /// get list of all items in this category
        virtual StringList
        listItems(const string& category)
        throw(VException&);

        /// add a new category
        virtual void
        addCategory( const string& category )
        throw(VException&);

        /// remove a category
        virtual void
        removeCategory( const string& category)
        throw(VException&);

        /// return true if the category exists
        virtual bool
        isCategory( const string& category)
        throw(VException&);

        /// get a list of all categorys
        virtual StringList
        listCategories();

    protected:

    private:

        /// get the full path directoy name for a given category
        string
        dirName(const string& category ) const;

        /// get the full path file name for a given item
        string
        fileName(const string& category, const string& name ) const;

        /// path to root of file system in use
        string fileRoot;

        /// lock to block concurent file access
        VRWLock lock;
};

#endif
