
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const PSCommandLine_cxx_Version =
    "$Id: PSCommandLine.cxx,v 1.3 2001/09/18 00:20:42 maivu Exp $";

#if defined(__FreeBSD__) || defined (__svr4__)
#include "getopt.h"
#else
#include <getopt.h>
#endif
#include <cstdio>

#include "parse3tuple.h"
#include "cpLog.h"
#include "PSCommandLine.hxx"


///
Sptr<PSCommandLine> PSCommandLine::myInstance = 0;
///
bool PSCommandLine::useConfigFile = false;


///
PSCommandLine::PSCommandLine()
{
    myPSConfiguration = PSConfiguration::instance();
}


///
void
PSCommandLine::instance( const int argc,
                         const char** argv,
                         const char* applicationUsage )
{
    if (myInstance == 0)
    {
        myInstance = new PSCommandLine();
    }
    
    myInstance->parseCommandLine( argc, argv, applicationUsage );
}


///
void
PSCommandLine::parseCommandLine( const int argc,
                                 const char** argv,
                                 const char* applicationUsage )
{
    int c = 0;
    
    while( ( c = getopt( argc, const_cast<char**>( argv ),
                         "c:v:f:p:t:d" ) ) != EOF )
    {
        if (useConfigFile == true)
            break;

        switch( c )
        {
            case 'd':
                // daemon level
                PSConfiguration::instance()->setDaemonMode( false );            
            break;

            case 'v':
                // loglevel
                PSConfiguration::instance()->setDebugLevel( optarg );
            break;

            case 't':
                // num of threads
                PSConfiguration::instance()->setNumThreads( atoi(optarg) );
            break;

            case 'f':
                // log file
                PSConfiguration::instance()->setLogFileName( optarg );
                cpLogOpen(optarg);
            break;

            case 'p':
                // localport
                PSConfiguration::instance()->setLocalPort( atoi(optarg) );
            break;

            case 'c':
                // config file
                PSConfiguration::instance()->setConfigFile( optarg );
                useConfigFile = true;
            break;

            case '?':
                cerr << "Unknown arg" << endl;
                cerr << "Usage:  " << argv[0] << " [OPTION]..." << endl;
                cerr << endl;
                cerr << "  Where [OPTION] is one of: " << endl;
                cerr << "    -v <log_level>                      " <<
                "    set loglevel" << endl;
                cerr << "    -f <logfile>                        " <<
                "    set log file" << endl;
                cerr << "    -c <config file>                    " <<
                "    set config file (overrides command options) " << endl;
                cerr << "    -p <local port>                     " <<
                "    set port to listen for incoming\n" <<
                "                                        " <<
                "      requests" << endl;
                cerr << "    -d                                  " <<
                "    disable daemon mode (run in \n" <<
                "                                        " <<
                "      foreground)" << endl;
                cerr << "    -t <number of threads (25)>         " <<
                "    number of request proccess threads." << endl;
                cerr << endl;
                exit( -1);

            break;

        };
    }

    if ( useConfigFile )
    {
        //  Need to read and parse config file if this option is set.
        cpLog(LOG_ALERT, "*** Using config file <%s> ***",
              PSConfiguration::instance()->getConfigFile().c_str() );
        PSConfiguration::instance()->parseConfigFile();
    }

    if ( ( argc - optind ) > 0 )
    {
        cpLog( LOG_ERR, "Unknown option: %s", argv[ optind ] );
        exit( -1 );
    }
}


///
Sptr < PSCommandLine >
PSCommandLine::instance()
{
    if (myInstance == 0 )
    {
        myInstance = new PSCommandLine();
    }
 
    return myInstance;
}


string 
PSCommandLine::getDebugLevel()
{
    return myPSConfiguration->getDebugLevel();
};

string 
PSCommandLine::getLogFileName()
{
    return myPSConfiguration->getLogFileName();
};

int 
PSCommandLine::getLocalPort()
{
    return myPSConfiguration->getLocalPort();
};

bool 
PSCommandLine::getDaemonMode()
{
    return myPSConfiguration->getDaemonMode();
};

int 
PSCommandLine::getNumThreads()
{
    return myPSConfiguration->getNumThreads();
};


