#ifndef PROVISIONSERVER_HXX
#define PROVISIONSERVER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const ProvisionServer_hxx_Version =
    "$Id: ProvisionServer.hxx,v 1.1.1.1 2001/08/21 23:30:25 dabryan Exp $";

#include <sys/time.h>
#include <sys/types.h>
#include <map>
#include <string>

#include "Tcp_ServerSocket.hxx"
#include "LockedConnection.hxx"
// #include "RedundancyManager.hxx"
// #include "HouseKeepingThread.hxx"
#include "VMutex.h"
#include "VIoException.hxx"


#include "HttpMessage.hxx"
#include "VThreadPool.hxx"

#include "DBConnection.hxx"
#include "DBResultset.hxx"
#include "ConnectionManager.hxx"
#include "VDbException.hxx"
#include "CacheManager.hxx"
#include "RegisterManager.hxx"

#include "PSDBRequest.hxx"
#include "StringHash.hxx"
#include "VRWLock.hxx"

#include "FileStore.hxx"


using namespace Vocal;


// class VThreadPool;

class ProvisionServer
{
    public:

        /// Interface to get reference to singleton object.
        static ProvisionServer& instance();

        /// Interface to get reference to singleton
        static ProvisionServer& instance( int nthreads );

        /// Return the name of the class
        string className()
        {
            return "ProvisionServer";
        }

        /// Main process loop. Waits on connections and handles them
        void run();

        /// Destructor
        ~ProvisionServer();

        // Symbolic string aliases for common group names
        static string SYS_CONF;
        static string FEATURES;
        static string ACCOUNTS;
        static string CALLEDLIST;
        static string CALLINGLIST;


        /// Called when we notice that the heartbeat is gone
        void lostBeat()
        {
            //_copy.heartBeatGone();
        }

    protected:


    void getHtmlfromFunc(LockedConnection &conn, string url);
    
    private:

        static const int THREAD_DELAY;
        static const int FAIL_SLEEP_TIME;
    
        /** 
         *  Private constructor to build the class (public access using
         *  the instance() operator since this is a singleton) 
         */
        ProvisionServer(int nthreads);

        /** 
         *  Basically a big set of if's to decide how to handle an incoming
         *  message.
         */
        void processHttpRequest(LockedConnection& conn) throw (VException&);

        void processSyncRequest(LockedConnection& conn) throw (VException&);

        /**
         *  Frees the memory associated with singelton instance.
         *  gets register to atexit() function at the time of creation.
         */
        static void destroy(void);

        /// A thread from the pool is passed this with a conenction to service
        static void processRequestFunc(void* con);

        /**
         *  Signal handler for SIGHUP signal, causes the client list to
         *  be reinitialized.
         */
        static void sigHupHandler(int sigNo);

        /// Signal handler for SIGTERM
        static void sigTerm(int signo);

        /// Signal handler to cleanup the child
        static void sigChld(int signo);

        /// ***************************************************************

        /**
         *  Message handling
         */

        /**
         */
        void handleGetRequest( LockedConnection& conn,
                               const Http::Message &httpmsg );

        /**
         */
        void handlePostRequest( LockedConnection& conn, 
                                const Http::Message &httpmsg );

        /**
         */
        void handlePutRequest( LockedConnection& conn,
                               const Http::Message &httpmsg);

       /**
         */
        void handleDeleteRequest( LockedConnection& conn, 
                                  const Http::Message &httpmsg );

        /** Get Requests 
         */

        /**
         */
        void handleGetRecordsInDomain( LockedConnection& conn,
                                       const string& httpBody);

        void handleGetFieldsInDomain( LockedConnection& conn,
                                      const string& httpBody);


        void handleSessionOpen( LockedConnection& conn,
                                const string& httpBody);

        void handleSessionGetDBResult( LockedConnection& conn,
                                       const string& httpBody);

        void handleGetXmlDescription( LockedConnection& conn,
                                      const string& httpBody);

        void handleListXmlCategories( LockedConnection& conn,
                                      const string& httpBody);

        void handleListXmlDescriptions( LockedConnection& conn,
                                        const string& httpBody);
    
    
        // Post Requests 
        void handleSessionClose( LockedConnection& conn,
                                 const string& httpBody);

        void handleSessionSearchDB( LockedConnection& conn,
                                    const string& httpBody);

        void handleSessionSortDBResult( LockedConnection& conn,
                                        const string& httpBody);

        void handleRegisterDomainForUpdate( LockedConnection& conn,
                                            const string& httpBody);

        void handleUnRegisterDomainForUpdate( LockedConnection& conn,
                                              const string& httpBody);

        void handleRegisterRecordForUpdate( LockedConnection& conn,
                                            const string& httpBody);

        void handleUnRegisterRecordForUpdate( LockedConnection& conn,
                                              const string& httpBody);


        // Put Requests 
        void handleSetRecordsInDomain( LockedConnection& conn,
                                       const string& httpBody);

        void handleAddRecordInDomain( LockedConnection& conn,
                                      const string& httpBody);

        void handleSessionSetDBResult( LockedConnection& conn,
                                       const string& httpBody);

        // Delete Requests 

        void handleDeleteRecordsInDomain( LockedConnection& conn,
                                          const string& httpBody);

        void handleSessionDeleteAll( LockedConnection& conn,
                                     const string& httpBody);


        /// Check the register list for updates.

        void updateCheck( Sptr<PSDBRequest> request,
                          DBResultset* resultset = 0);

        /// Make a 400 and send on the provided connection
        void sendHttpFail(LockedConnection &conn, string retData);

        /// Make a 200 and send on the provided connection
        void sendHttpOk(LockedConnection &conn, string retData);


        /// ***************************************************************

        // the word "file" appears in these as a backward compatability thing...

        /// Reads the content of item from the data store
        string readFile(const string& fileName, TimeStamp *timestamp = 0)
        throw (VIoException&);

        /// Writes the contents of item on the data store, returns timestamp
        void writeFile(const string& fileName, const string& contents,
                       TimeStamp *timestamp)
        throw (VIoException&);

        /// If a new server register for update, update the client list
        void updateClientList();

        /**
         *  Initialises the list of hosts, to which,
         *  notification has to be sent.Gets called when
         *  object is first created or a HUP signal is received.         
         */
        void initializeClientList() throw (VIoException&);

        /// 
        void initializeFeatureServerList() throw (VIoException&);


        /* Various locks */
        // _clientListLock and _featureListLock not used
        // static pthread_mutex_t _clientListLock;
        // static pthread_mutex_t _featureListLock;

	// Use VMutex instead of pthread_mutex directly
        static VMutex _MutexconnectionMapLock;
        static VMutex _MutextimerLock;

        typedef map < int, LockedConnection > ConnectionMap;

        /// Map of int to LockedConnection, tracks the active connections
        ConnectionMap _connectionMap;

        /// pointer to the singleton instance
        static ProvisionServer* myInstance;

        /// Socket to listen on
        TcpServerSocket _serverSocket;

        /// Holds the PorvisionServer port
        int _pServerPort;

        /// Pool of worker threads
        Sptr < VThreadPool > _threadPool;

        /// File system root (location of the data)
        string _root;

        /// The redundant server manages all contact with the other server
//        RedundancyManager _copy;

        /// Timer to track if we are waiting for a sync timer to expire
        bool timerPending;

        /// Timer to track if we have a timeout set for sync 
        bool failTimerPending;

        /// Smart pointer to the DataStore where the files reside
        Sptr < FileStore > _dataStore;
    Sptr < FileStore > myWebStore;

        /// Register manager to track client registrations
        RegisterManager myRegisterManager;

        /// Number of bins in the DataStore
        int _nbins;

        /// Number of threads in thread pool
        int _nthreads;

        /// Connection Manager to handle connection to databases
        ConnectionManager myDBManager;

        /// CacheManager
        CacheManager myCacheManager;
};

#endif
