// #include <iostream>
#include "VDbException.hxx"
#include "PGSQLConnection.hxx"
#include "MYSQLConnection.hxx"
#include "LDAPConnection.hxx"
#include "ConnectionManager.hxx"
#include "cpLog.h"

using namespace std;

//  Return a new connection to a database
DBConnection* 
ConnectionManager::GetDBConnection(const DBInfo dbInfo)
{
    if (dbInfo.dbBackend == "PGSQL")
    {
        if (myDBConnectionP == 0)
        {
            myDBConnectionP = new PGSQLConnection(
                    dbInfo.database,
                    dbInfo.host,
                    dbInfo.user, 
                    dbInfo.password);
        };
    }
    else if (dbInfo.dbBackend == "MYSQL")
    {
        if (myDBConnectionP == 0)
        {
            myDBConnectionP = new MYSQLConnection(
                    dbInfo.database, 
                    dbInfo.host, 
                    dbInfo.user, 
                    dbInfo.password);
        };
    }
    else if (dbInfo.dbBackend == "LDAP")
    {
        if (myDBConnectionP == 0)
        {
            cpLog (LOG_DEBUG, "LDAP: not supported for now.");
            // myDBConnectionP = new LDAPConnection(dbInfo.database, 
            //               dbInfo.host,
            //               dbInfo.port, 
            //               dbInfo.dn, 
            //               dbInfo.pw);
        };
    }
    else
    {
        cpLog (LOG_ERR, "DB Backend not supported.");
        myDBConnectionP = NULL;
    }

    return myDBConnectionP;
}
