#ifndef DBCONNECTION_HXX
#define DBCONNECTION_HXX

#include <string>
#include <vector>
#include <map>
#include "DBTypes.h"


/// Forward declare DBResultset
class DBResultset;

/// DBConnection class- Abstract client interface class
class DBConnection
{
    public:
        /// Constructor
        DBConnection(){};

        /// Destructor
        virtual ~DBConnection(){};

        /// Return a resultset object
        virtual DBResultset* DBSelect (const string& Domain, 
                const string& SearchFilter, 
                const FieldListtype& FieldList, 
                const Ordertype& Order, 
                const string &OrderField) = 0;

        /// Return a resultset object
        virtual DBResultset* DBSelect (const string& Domain, 
                const string& SearchFilter, 
                const FieldListtype& FieldList) = 0;

        /// Return a resultset object
        virtual DBResultset* DBSelect(const string& Domain, 
                const string& SearchFilter) = 0;

        /// Return a resultset object
        virtual DBResultset* DBSelect (const string& Domain,
                const string& SearchFilter,
                const string& start,
                const string& end,
                const FieldListtype& FieldList) = 0;

        /// Return a resultset object
        virtual DBResultset* DBSelect( const string& Domain,
                const string& SearchFilter,
                const string& start,
                const string& end,
                const Ordertype& Order,
                const string &OrderField,
                const FieldListtype& FieldList) = 0;

        /// Return a string containing primary key of Domain
        virtual string DBPrimaryKey ( const string& Domain) = 0;

        /// Return a table describing fields/types/keys in Domain
        virtual ResultTabletype& DBDescribe (const string& Domain) = 0;

        /// Return a list of domains in database
        virtual ResultRecordtype& DBDescribe (void) = 0;

        /// Update Domain, based on searchfilter
        virtual void DBUpdate (const string& Domain, 
                const string& SearchFilter, 
                const FieldValuetype& FieldValueSet) = 0;

        /// Update Domain, based on previous resultset
        virtual void DBUpdate (const string& Domain, DBResultset* Resultset, 
                const FieldValuetype& FieldValueSet) = 0;

        /// Insert record
        virtual void DBInsert (const string& Domain, 
                const FieldValuetype& FieldValueSet) = 0;

        /// Delete rows based on a seachfilter
        virtual void DBDelete (const string& Domain, 
                const string& SearchFilter) = 0;

        /// Delete rows based on a previous resultset
        virtual void DBDelete (const string& Domain, 
                DBResultset* Resultset) = 0;

        /// Create a new domain, use STRINGTYPE for all columns
        /// with or without checking current existence
        virtual void DBCreate (const string& Domain, 
                const FieldListtype& FieldsList, 
                const string& PrimaryKey,
                bool ExistenceCheck ) = 0;

        /// Create a new domain, specify types for each column
        /// with or without checking current existence
        virtual void DBCreate (const string& Domain, 
                const FieldListtype& FieldsList, 
                const FieldListDatatype& Datatypelist, 
                const string& PrimaryKey,
                bool ExistenceCheck ) = 0;

        /// Alter a domain, drop field
        virtual void DBAlter(const string& Domain, 
                const string& Fieldname) = 0;
         
        /// Alter a domain, change a type of a field
        virtual void DBAlter(const string& Domain, const string& Fieldname,
                const Datatype Newtype) = 0;

        /// Drop a domain, with or without checking current existence
        virtual void DBDrop(const string& Domain, 
                bool ExistenceCheck ) = 0;

        /// Returns true if Domain exists, false otherwise
        virtual bool DBVerify(const string& Domain) = 0;
};
#endif
