#ifndef DBTYPES_H
#define DBTYPES_H


#include <map>
#include <string>
#include <vector>


/// Supported Data types
enum Datatype
{
    STRINGTYPE,
    INTEGERTYPE,
    FLOATTYPE,
    TEXTBLOBTYPE,
    BINBLOBTYPE,
    TIMESTAMPTYPE,
    DATETYPE,
    TIMETYPE
};

/// Supported Order Types
enum Ordertype
{
    ASCENDING,
    DESCENDING,
    UNDEFINED
};

struct DBInfo
{
  // "MYSQL" - "LDAP" - ????
  string dbBackend;

  string database;
  string host;
  int port;
  string user;
  string password;
  string dn;
};


/// When using DBCreate() or DBDrop() to create or dop domains, 
/// set the ExistenceCheck argument to one of the following:
static const bool IGNOREEXISTENCE = false;
///
static const bool CHECKEXISTENCE = true;

/// Maximum length of a Database query string
//static const unsigned short MAXQUERYLENGTH = 512;

static const unsigned short MAXQUERYLENGTH = 32768;  
// 2^15 = 32768 (a short is 16 bits)

/// Type used to describe field datatype
typedef vector<Datatype> FieldListDatatype;

/// Type used to describe a single record
typedef vector<string> ResultRecordtype;

/// Type used to describe a group of records
typedef vector<ResultRecordtype> ResultTabletype;

/// Type used to describe a group of fields
typedef vector< string > FieldListtype;

/// Type used to describe a group of Field-Value pairs
typedef map< string, string > FieldValuetype;



#endif // DBTYPES_H
