#include <iostream>
#include <string>
#include "VDbException.hxx"
#include "LDAPResultset.hxx"
#include "LDAPFilterparser.hxx"
#include "LDAPConnection.hxx"

using namespace std;

//
LDAPConnection::LDAPConnection(const string& LdapBaseDN, const string& LdapHost,
        const unsigned short LdapPort, const string& LdapBindDN, 
        const string& LdapBindPW):
myBaseDN(LdapBaseDN)
{ 
    int LDAPqueryresult;

    // Pass non-consts char* to LDAP API functions
    char* ldaphost = strdup(LdapHost.c_str());
    char* binddn = strdup(LdapBindDN.c_str());
    char* bindpw = strdup(LdapBindPW.c_str());

    // ldap_init will point myLDAPhandle to a live server
    if ((myLDAPhandle = ldap_init(ldaphost, LdapPort)) == NULL)
    {
        // Failed to authenticate to LDAP server, throw exception
        string err = "LDAPConnection Error: Failed to attached to ldap server";
        throw VDbException(err, __FILE__, __LINE__, 0);                         
    }
 
    // ldap_simple_bind_s will authenticate against an LDAP server as a user
    if ( ( LDAPqueryresult = ldap_simple_bind_s(myLDAPhandle, binddn, bindpw) )
            != LDAP_SUCCESS)
    {
        // Failed to authenticate to LDAP server, throw exception
        string LDAPstringerr = ldap_err2string( LDAPqueryresult );
        string err = "LDAPConnection Error: " + LDAPstringerr;
        throw VDbException(err, __FILE__, __LINE__, 0);                         
    }

    // Set search size limit to infinite
#if defined(__svr4__) || defined(__SUNPRO_CC)
    ldap_set_option(myLDAPhandle, LDAP_OPT_SIZELIMIT,& 0);
#else
 #if (LDAP_VERSION >= LDAP_VERSION2)
    ldap_set_option(myLDAPhandle, LDAP_OPT_SIZELIMIT, 0);
 #else
    myLDAPhandle->ld_sizelimit = 0;
 #endif
#endif

};
 
// DBSelect- Returns a DBResultset
DBResultset* 
LDAPConnection::DBSelect(const string& Domain,
    const string& SearchFilter, const FieldListtype& FieldList, 
    const Ordertype& Order, const string &OrderField)
{
    LDAPMessage* ldapresult;
    char ldapsearchfilter[MAXQUERYLENGTH];
    char* searchfields[FieldList.size() + 1];
    unsigned int i = 0;
    int LDAPqueryresult;

    // Pass non-consts char* to LDAP API functions
    char* ldapbasedn = strdup(myBaseDN.c_str());

    // Create LDAPsearch filter based on domain, and searchfilter
    CreateLDAPsearchstring(ldapsearchfilter, Domain, SearchFilter);

    // For troubleshooting, print ldapsearch parameters
//    cout << "LDAP Command: ldapsearch BaseDN=";
//    cout << myBaseDN << ", " << ldapsearchfilter << endl;

    // Convert STL vector of strings to array of char pointers
    // to be used by ldap_search_s()
    for (i = 0; i < FieldList.size(); i++)
    {
        searchfields[i] = strdup(FieldList[i].c_str());
    }
    searchfields[i] = NULL;

    if ( ( LDAPqueryresult = ldap_search_s( myLDAPhandle, 
           ldapbasedn, LDAP_SCOPE_SUBTREE, ldapsearchfilter, 
           searchfields, 0, &ldapresult) )
         != LDAP_SUCCESS )
    {
        //  ldap_perror(myLDAPhandle, "ldap_search_s");
        string LDAPstringerr = ldap_err2string( LDAPqueryresult );
        string err = "DBSelect Error: " + LDAPstringerr;
        throw VDbException(err, __FILE__, __LINE__, 0);                         
    }

    myLDAPResultsetP = new LDAPResultset(ldapresult, 
            myLDAPhandle, SearchFilter);

    return myLDAPResultsetP;
};

//
DBResultset* 
LDAPConnection::DBSelect(const string& Domain,
     const string& SearchFilter, const FieldListtype& FieldList)
{
    LDAPMessage* ldapresult;
    char ldapsearchfilter[MAXQUERYLENGTH];
    char* searchfields[FieldList.size() + 1];
    unsigned int i = 0;
    int LDAPqueryresult;

    // Pass non-consts char* to LDAP API functions
    char* ldapbasedn = strdup(myBaseDN.c_str());

    // Create LDAPsearch filter based on domain, and searchfilter
    CreateLDAPsearchstring(ldapsearchfilter, Domain, SearchFilter);

    // For troubleshooting, print ldapsearch parameters
    // cout << "LDAP Command: ldapsearch BaseDN=";
    // cout << myBaseDN << ", " << ldapsearchfilter << endl;

    // Convert STL vector of strings to array of char pointers
    // to be used by ldap_search_s()
    for (i = 0; i < FieldList.size(); i++)
    {
        searchfields[i] = strdup(FieldList[i].c_str());
    }
    searchfields[i] = NULL;

    if ( ldap_search_s( myLDAPhandle, ldapbasedn, LDAP_SCOPE_SUBTREE, 
         ldapsearchfilter, searchfields, 0, &ldapresult) 
         != LDAP_SUCCESS )
    {
        //  ldap_perror(myLDAPhandle, "ldap_search_s");
        string LDAPstringerr = ldap_err2string( LDAPqueryresult );
        string err = "DBSelect Error: " + LDAPstringerr;
        throw VDbException(err, __FILE__, __LINE__, 0);                         
    }

    myLDAPResultsetP = new LDAPResultset(ldapresult, 
            myLDAPhandle, SearchFilter);

    return myLDAPResultsetP;
};

//
ResultTabletype& 
LDAPConnection::DBDescribe(const string& Domain)
{
    cout << "Not implemented yet\n";
    myDescribeTable.clear();
    return myDescribeTable;
};

//
ResultRecordtype& 
LDAPConnection::DBDescribe(void)
{
    cout << "Not implemented yet\n";
    myDescribeRecord.clear();
    return myDescribeRecord;
};

//
void 
LDAPConnection::DBUpdate(const string& Domain, const string& SearchFilter,
     const FieldValuetype& FieldValueSet)
{
    cout << "Not implemented yet\n";
};

//
void 
LDAPConnection::DBUpdate(const string& Domain, DBResultset* Resultset,
              const FieldValuetype& FieldValueSet)
{
    cout << "Not implemented yet\n";
};

//
void 
LDAPConnection::DBInsert(const string& Domain,
              const FieldValuetype& FieldValueSet)
{
    cout << "Not implemented yet\n";
};

//
void 
LDAPConnection::DBDelete(const string& Domain, const string& SearchFilter)
{
    cout << "Not implemented yet\n";
};

//
void 
LDAPConnection::DBDelete(const string& Domain, DBResultset* Resultset)
{
    cout << "Not implemented yet\n";
};

//
void 
LDAPConnection::DBCreate(const string& Domain, const FieldListtype& FieldList,
        const string& PrimaryKey, bool ExistenceCheck)
{
    cout << "Not implemented yet\n";
};

//
void 
LDAPConnection::DBCreate(const string& Domain, const FieldListtype& FieldList, 
        const FieldListDatatype& DatatypeList, const string& PrimaryKey, 
        bool ExistenceCheck)
{
    cout << "Not implemented yet\n";
};

//
void 
LDAPConnection::DBAlter(const string& Domain, const string& Fieldname)
{
    cout << "Not implemented yet\n";
};

//
void 
LDAPConnection::DBAlter(const string& Domain, const string& Fieldname,
        const Datatype Newtype)
{
    cout << "Not implemented yet\n";
};

//
void 
LDAPConnection::DBDrop(const string& Domain, bool ExistenceCheck)
{
    cout << "Not implemented yet\n";
};

//
bool 
LDAPConnection::DBVerify(const string& Domain)
{
    cout << "Not implemented yet\n";
    return false;
};

// Generate an ldap_search() search filter based on the DBSelect arguments
// This functions uses the 'Filterparser' class to parser the
// search filter. 
void 
LDAPConnection::CreateLDAPsearchstring(char* ldapsearchfilter, 
     const string& Domain, const string& SearchFilter)              
{

    // Use a strstream to create ldapfilter.
    strstream filterbuff(ldapsearchfilter, MAXQUERYLENGTH);

    // Prepend (&(objectclass=domain to every DBSelect search
    filterbuff << " (&(objectclass=" << Domain << ")";

    // Use Filterparser to get ldap_search filter, from SearchFilter
    // Then, add LDAPsearch clause to ldapsearchfilter
    LDAPFilterparser filter(SearchFilter);
    filterbuff << filter.GetParsedresult();

    // End string by closing paren from and'ing objectclass
    filterbuff << ")" << ends;

}
