#ifndef LDAPCONNECTION_HXX
#define LDAPCONNECTION_HXX

#include <string>
#include <lber.h>
#include <ldap.h>
#include "DBConnection.hxx"

/// Foward Declare LDAPResultset
class LDAPResultset;

/// LDAPConnection class
class LDAPConnection : public DBConnection
{
    private:
        /// Constructor
        LDAPConnection(const string& basedh, const string& ldaphost, 
                       const unsigned short ldapport, const string& binddn, 
                       const string& bindpw);

        /// Destructor
        virtual ~LDAPConnection()
        { 
            ldap_unbind(myLDAPhandle);
        }; 
 
        ///
        DBResultset* DBSelect(const string& Domain, 
                const string& SearchFilter, const FieldListtype& FieldList, 
                const Ordertype& Order, const string &OrderField);
        ///
        DBResultset* DBSelect(const string& Domain, 
                const string& SearchFilter, const FieldListtype& FieldList);

        ///  Not implemented
        DBResultset* DBSelect(const string& Domain,
                const string& SearchFilter, const string& start, const string& end,
                const FieldListtype& FieldList)
        {
            return 0;
        };

        ///  Not implemented
        DBResultset* DBSelect( const string& Domain,
                               const string& SearchFilter, 
                               const string& start, 
                               const string& end,
                               const Ordertype& Order,
                               const string& OrderField,
                               const FieldListtype& FieldList)
        {
            return 0;
        };

        ///  Not implemented
        DBResultset* DBSelect(const string& Domain,
                              const string& SearchFilter )
        {
            return 0;
        };


        ///
        void DBUpdate(const string& Domain, const string& SearchFilter, 
                const FieldValuetype& FieldValueSet);

        ///
        void DBUpdate(const string& Domain, DBResultset* Resultset, 
                const FieldValuetype& FieldValueSet);

        ///
        void DBInsert(const string& Domain, 
                const FieldValuetype& FieldValueSet);

        ///
        void DBDelete(const string& Domain, const string& SearchFilter);

        ///
        void DBDelete(const string& Domain, DBResultset* Resultset);

        /// Not implemented
        string DBPrimaryKey( const string& Domain)
        {
            return 0;
        };


        ///
        ResultTabletype& DBDescribe(const string& Domain);

        ///
        ResultRecordtype& DBDescribe(void);

        ///
        void DBCreate(const string& Domain, const FieldListtype& FieldsList, 
                const string& PrimaryKey,
                bool ExistenceCheck);

        ///
        void DBCreate(const string& Domain, const FieldListtype& FieldsList,
                const FieldListDatatype& DatatypeList, 
                const string& PrimaryKey,
                bool ExistenceCheck);

        ///
        void DBAlter(const string& Domain, const string& Fieldname);

        ///
        void DBAlter(const string& Domain, const string& Fieldname, 
                const Datatype Newtype);

        ///
        void DBDrop(const string& Domain, bool ExistenceCheck);

        ///
        bool DBVerify(const string& Domain);

        ///
        void CreateLDAPsearchstring(char *ldapsearchfilter, 
                const string& Domain, const string& SearchFilter);

        /// LDAP handle
        LDAP* myLDAPhandle; 

        /// LDAP Base DN (Distinguished Name)
        const string& myBaseDN;

        /// vector<vector <string> > used by DBDescribe(Domain) 
        ResultTabletype myDescribeTable;

        /// vector <string> used by DBDescribe() 
        ResultRecordtype myDescribeRecord; 

        /// myLDAPResultsethandle is a pointer to the resultset
        LDAPResultset* myLDAPResultsetP;

    public:
        ///
        friend class ConnectionManager;
};
#endif
