#include <iostream>
#include <string>
#include "VDbException.hxx"
#include "MYSQLFilterparser.hxx"
#include "MYSQLResultset.hxx"
#include "MYSQLConnection.hxx"

using namespace std;

// MYSQLConnection Constructor
MYSQLConnection::MYSQLConnection(const string& dbname, const string& sqlhost,
              const string& sqluser, const string& sqlpw)
{
    try
    {
        myMySQLConnectionP = new MysqlConnection(use_exceptions);
        myMySQLConnectionP->connect(dbname.c_str(), sqlhost.c_str(),
                sqluser.c_str(), sqlpw.c_str());
    }
    catch (BadQuery mysqle)
    {
        // To hide the MySQL specific exceptions, catch and throw your own.
        string err = "SQLConnection Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
};

// DBSelect- Returns a DBResultset
DBResultset*
MYSQLConnection::DBSelect(const string& Domain,
     const string& SearchFilter, const FieldListtype& FieldList,
     const Ordertype& Order, const string &OrderField)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;

    // Create SQL statement
    CreateSQLselect(Sqlcmd, Domain, SearchFilter, FieldList,
            Order, OrderField);

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBSelect Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    mySQLResultsetP = new MYSQLResultset(mysqlresult, SearchFilter);
    return mySQLResultsetP;
}

// DBSelect- Returns a DBResultset
DBResultset*
MYSQLConnection::DBSelect(const string& Domain,
     const string& SearchFilter, const FieldListtype& FieldList)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;

    //
    CreateSQLselect(Sqlcmd, Domain, SearchFilter, FieldList);

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBSelect Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    mySQLResultsetP = new MYSQLResultset(mysqlresult, SearchFilter);
    return mySQLResultsetP;
};

DBResultset*
MYSQLConnection::DBSelect(const string& Domain,
                        const string& SearchFilter )
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;

    CreateSQLselect(Sqlcmd, Domain, SearchFilter );

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();

    }
    catch (BadQuery mysqle)
    {
        string err = "DBSelect Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    mySQLResultsetP = new MYSQLResultset(mysqlresult, SearchFilter);
    return mySQLResultsetP;
}


DBResultset*
MYSQLConnection::DBSelect(const string& Domain,
                        const string& SearchFilter,
                        const string& start,
                        const string& end,
                        const Ordertype& Order, 
                        const string &OrderField,
                        const FieldListtype& FieldList)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;

    //
    CreateSQLselect( Sqlcmd, 
                     Domain, 
                     SearchFilter, 
                     start, 
                     end, 
                     Order, 
                     OrderField, 
                     FieldList );

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBSelect Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    mySQLResultsetP = new MYSQLResultset(mysqlresult, SearchFilter);
    return mySQLResultsetP;
};


DBResultset*
MYSQLConnection::DBSelect(const string& Domain,
                        const string& SearchFilter, 
                        const string& start,
                        const string& end,
                        const FieldListtype& FieldList)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;

    //
    CreateSQLselect(Sqlcmd, Domain, SearchFilter, start, end, FieldList);

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBSelect Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    mySQLResultsetP = new MYSQLResultset(mysqlresult, SearchFilter);
    return mySQLResultsetP;
};


// DBPrimaryKey - returns the primary key for a domain
// NOTE:  assumes only one primary key exists.  If more
//        than one, then it'll get the first one from
//        the list.

string
MYSQLConnection::DBPrimaryKey (const string& Domain)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;
    MysqlRow row;
    ostrstream buff(Sqlcmd, MAXQUERYLENGTH);
    string fieldbuff;
    unsigned short recordnumber;
    string resultString = "";

    // Clear contents of DescribeRecord for each query
    myDescribeRecord.clear();

    // Static SQL statement for getting table information
    buff << "SHOW INDEX FROM " << Domain << ends;

    // Diagnostic only
//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBDescribe Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    // Iterate through records, find primary key 
    for (recordnumber = 0; recordnumber < mysqlresult.size(); recordnumber++)
    {
        row = mysqlresult[recordnumber];

        // First column:   Table
        // Second column:  uniqueness( 0 = unique, 1 = non-unique)
        // Third column:   Key_name       <----
        // Fourth column:  Seq_in_index
        // Fifth column:   Column_name    <----  

        // Want the value search the value in Key_name for PRIMARY
        fieldbuff = row[mysqlresult.names(2)];

        // If it is primary, then return Column_name
        if (fieldbuff == "PRIMARY" )
        {
            resultString = row[ mysqlresult.names(4) ];
            return resultString;
        }
    }

    // returns an empty string if nothing
    return resultString; 
}



// DBDescribe- Returns information on one domain
ResultTabletype&
MYSQLConnection::DBDescribe(const string& Domain)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;
    ostrstream buff(Sqlcmd, MAXQUERYLENGTH);
    ResultRecordtype recordbuff;
    MysqlRow row;
    string fieldbuff;
    unsigned short recordnumber;

    // Clear contents of DescribeTable for each query
    myDescribeTable.clear();

    // Static SQL statement for getting table information
    buff << "DESCRIBE " << Domain << ends;

    // Diagnostic only
//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBDescribe Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    // Iterate through records, filling myDescribeTable
    for (recordnumber = 0; recordnumber < mysqlresult.size(); recordnumber++)
    {
        row = mysqlresult[recordnumber];
        // First column of Describe table is Fieldname
        fieldbuff = row[mysqlresult.names(0)];
        recordbuff.push_back(fieldbuff);

        // Second column of Describe table is Datatype
        // Convert the MySQL datatype, to the DBmodule
        // supported Datatype.
        fieldbuff = row[mysqlresult.names(1)];

        if ( !fieldbuff.find("char(") || !fieldbuff.find("CHAR(") )
              recordbuff.push_back("STRINGTYPE");
        else if ( !fieldbuff.find("varchar(") || !fieldbuff.find("VARCHAR(") )
              recordbuff.push_back("STRINGTYPE");
        else if ( !fieldbuff.find("float") || !fieldbuff.find("FLOAT") )
              recordbuff.push_back("FLOATTYPE");
        else if ( !fieldbuff.find("timestamp(") || 
                  !fieldbuff.find("TIMESTAMP(") )
              recordbuff.push_back("TIMESTAMPTYPE");

        else if ( fieldbuff == "time" || fieldbuff == "TIME" )
              recordbuff.push_back("TIMETYPE");
        else if ( fieldbuff == "date" || fieldbuff == "DATE" )
              recordbuff.push_back("DATETYPE");
        else if ( fieldbuff == "text" || fieldbuff == "TEXT" ||
		  fieldbuff == "longblob" )
              recordbuff.push_back("TEXTBLOBTYPE");
        else if ( fieldbuff == "blob" || fieldbuff == "BLOB" )
              recordbuff.push_back("BINBLOBTYPE");
        else
        {
            string err = "DBDescribe Error: unkown datatype";
            throw VDbException(err, __FILE__, __LINE__, 0);
        }

        // Third column of Describe table is boolean 
        // describing whether the field can be NULL, or 'NOT NULL'
        // Ignore this column since the DBAPI currently only
        // supports NULL fields.  A NULL field is a field that can
        // have a NULL value.  The only fields that cannot have NULL
        // values are Primary Key fields.

        // Fourth column of Describe table is Index type.
        // MySQL has three different index types: PRIMARY, 
        // UNIQUE, and INDEX.  To keep compliant with other backends
        // We are only concerned with the PRIMARY key field
        fieldbuff = row[mysqlresult.names(3)];
        if ( fieldbuff == "PRI" ) 
            recordbuff.push_back("PRIMARYKEY");

/*        for (i = 0; i < mysqlresult.columns(); i++)
        {
            fieldbuff = row[mysqlresult.names(i)];
            recordbuff.push_back(fieldbuff);
        }
*/

        // Insert record buffer into result table
        myDescribeTable.push_back(recordbuff);

        // Clear recordbuff for next record
        recordbuff.clear();
    }

    return myDescribeTable;
};

// DBDescribe- Returns a list of all domains in database
ResultRecordtype&
MYSQLConnection::DBDescribe()
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;
    MysqlRow row;
    ostrstream buff(Sqlcmd, MAXQUERYLENGTH);
    string fieldbuff;
    unsigned short recordnumber;

    // Clear contents of DescribeRecord for each query
    myDescribeRecord.clear();

    // Static SQL statement for getting table information
    buff << "SHOW TABLES " << ends;

    // Diagnostic only
//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBDescribe Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    // Iterate through records, filling myDescribeRecord
    for (recordnumber = 0; recordnumber < mysqlresult.size(); recordnumber++)
    {
        row = mysqlresult[recordnumber];

        // SHOW TABLES always returns 1 column,
        // Therefore row[0] will always contain the table name
        fieldbuff = row[0];

        // Insert field buffer into result table
        myDescribeRecord.push_back(fieldbuff);
    }

    return myDescribeRecord;
};

// DBUpdate- Updates a record in an existing domain
void
MYSQLConnection::DBUpdate(const string& Domain, const string& SearchFilter,
     const FieldValuetype& FieldValueSet)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;

    // Create SQL statement
    CreateSQLupdate(Sqlcmd, Domain, SearchFilter, FieldValueSet);

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBUpdate Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
};

// DBUpdate- Updates a record in an existing domain, based on a
//           previous resultset.
void
MYSQLConnection::DBUpdate(const string& Domain, DBResultset* Resultset,
     const FieldValuetype& FieldValueSet)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;

    // Get SearchFilter from resultset, use it to generate SQL command
    const string searchfilter = Resultset->GetSearchFilter();

    // Create SQL statement
    CreateSQLupdate(Sqlcmd, Domain, searchfilter, FieldValueSet);

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBUpdate Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
};

// DBInsert- Adds a record to an existing domain
void
MYSQLConnection::DBInsert(const string& Domain,
     const FieldValuetype& FieldValueSet)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;

    // Create SQL statement
    CreateSQLinsert(Sqlcmd, Domain, FieldValueSet);

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBInsert Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
};

// DBDelete- Deletes a record from an existing domain, based on a
//           searchfilter
void
MYSQLConnection::DBDelete(const string& Domain, const string& SearchFilter)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;

    // Create SQL statement
    CreateSQLdelete(Sqlcmd, Domain, SearchFilter);

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBDelete Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
};

// DBDelete- Deletes a record from an existing domain, based on a
//           previous resultset
void
MYSQLConnection::DBDelete(const string& Domain, DBResultset* Resultset)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;

    // Get SearchFilter from resultset, use it to generate SQL command
    const string searchfilter = Resultset->GetSearchFilter();

    // Create SQL statement
    CreateSQLdelete(Sqlcmd, Domain, searchfilter);

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBDelete Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
};

// DBCreate- Creates a new domain. This version assumes all columns to
//           be of datatype STRING, which is msyql CHAR(255)
void
MYSQLConnection::DBCreate(const string& Domain, const FieldListtype& FieldList,
        const string& PrimaryKey, bool ExistenceCheck)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;

    // Create SQL statement
    CreateSQLcreate(Sqlcmd, Domain, FieldList, PrimaryKey, ExistenceCheck);
    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBCreate Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
};

// DBCreate- Creates a new domain. This version requires the client to
//           specify the datatype of each column
void
MYSQLConnection::DBCreate(const string& Domain, const FieldListtype& FieldList,
        const FieldListDatatype& DatatypeList, const string& PrimaryKey, 
        bool ExistenceCheck)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;

    // Check if a data type exists for each field
    if (FieldList.size() != DatatypeList.size())
    {
        throw VDbException("DBCreate Error: \
            Number of Datatypes does not match number of Fields",
            __FILE__, __LINE__, 0);
    }

    // Create SQL statement
    CreateSQLcreate(Sqlcmd, Domain, FieldList, DatatypeList, PrimaryKey, 
            ExistenceCheck);

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBCreate Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
};

// DBAlter- Drop a column in a table
void
MYSQLConnection::DBAlter(const string& Domain, const string& Fieldname)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;
    ostrstream buff(Sqlcmd, MAXQUERYLENGTH);

    buff << "ALTER TABLE " << Domain << " ";
    buff << "DROP COLUMN " << Fieldname << ends;

    // Diagnostic only
//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBAlter Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
};

// DBAlter- Add a column to a table
void
MYSQLConnection::DBAlter(const string& Domain, const string& Fieldname,
        const Datatype Newtype)
{
    char Sqlcmd[MAXQUERYLENGTH];
    string Sqltype;
    MysqlRes mysqlresult;
    ostrstream buff(Sqlcmd, MAXQUERYLENGTH);

    GetSQLtypestring(Sqltype, Newtype);

    buff << "ALTER TABLE " << Domain << " ";
    buff << "ADD COLUMN " << Fieldname << " ";
    buff << Sqltype << ends;

    // Diagnostic only
//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBAlter Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
};

// DBDrop- Delete a domain.  If ExistenceCheck is true, use default
// SQL DROP TABLE which errors if table does not exist. If ExistenceCheck
// is false, use SQL DROP TABLE IF EXISTS which doesn't error if
// table does not exist.
void
MYSQLConnection::DBDrop(const string& Domain, bool ExistenceCheck)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;
    ostrstream buff(Sqlcmd, MAXQUERYLENGTH);

    // Static SQL statement for Deleting Table
    if (ExistenceCheck)
    {
        buff << "DROP TABLE " << Domain << ends;
    }
    else
    {
        buff << "DROP TABLE IF EXISTS " << Domain << ends;
    }

    // Diagnostic only
//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBDrop Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
};

// DBVerify- Verify the existence of a domain.
bool
MYSQLConnection::DBVerify(const string& Domain)
{
    char Sqlcmd[MAXQUERYLENGTH];
    MysqlRes mysqlresult;
    ostrstream buff(Sqlcmd, MAXQUERYLENGTH);
    MysqlRow row;
    string fieldbuff;

    buff << "CHECK TABLE " << Domain << ends;

    // Diagnostic only
//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;

    try
    {
        MysqlQuery query = myMySQLConnectionP->query();
        query << Sqlcmd;
        mysqlresult = query.store();
    }
    catch (BadQuery mysqle)
    {
        string err = "DBVerify Error: " + mysqle.error ;
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    // Check the last column of the last record.  If this field is "OK",
    // the table exists, otherwise the table doesn't exist.
    row = mysqlresult[ mysqlresult.size() - 1 ];
    fieldbuff = row[ mysqlresult.columns() - 1 ];

    // Return true if table exists, false otherwise
    if ( fieldbuff == "OK" )
        return true;
    else
        return false;
};

// Convert from public enumerated data type to private data type
// that this dbmodule supports
void
MYSQLConnection::GetSQLtypestring(string& Sqltype, Datatype type)
{
    /// MySQL Equivalent Data types
    const char SQLtypestring[][12] =
    {
        "CHAR(255)",   // STRING
        "INTEGER",     // INTEGER
        "FLOAT",       // FLOAT
        "TEXT",        // TEXTBLOB
        "BLOB",        // BINBLOB
        "TIMESTAMP",   // TIMESTAMP
        "DATE",        // DATE
        "TIME",        // TIME
    };

    Sqltype = SQLtypestring[type];
}

// Generate an SQL statement based on the DBSelect arguments
void 
MYSQLConnection::CreateSQLselect(char* Sqlcmd, const string& Domain,
     const string& SearchFilter, const FieldListtype& FieldList)
{
    // Use a strstream to create SQL command
    ostrstream strbuff(Sqlcmd, MAXQUERYLENGTH);

    // Field List iterator to used to fill fields in SQL statement
    FieldListtype::const_iterator p;

    // All SQL SELECT statements begin with SELECT
    strbuff << "SELECT ";

    // Add FieldList elements to SQL statement
    for (p = FieldList.begin(); p != FieldList.end(); p++)
    {
        strbuff << *p ;
        if ((p + 1) != FieldList.end())
        {
            // Insert a comma between field elements
            strbuff << ",";
        }
        else
        {
            // Insert a space, instead of a comma after the last element
            strbuff << " ";
        }
    }

    // Add Domain to SQL statement
    strbuff << "FROM " << Domain  << " ";

    // Use Filterparser to get SQL 'WHERE' clause from SearchFilter
    // then add WHERE clause to SQL statement
    MYSQLFilterparser filter(SearchFilter);
    strbuff << "WHERE " << filter.GetParsedresult() << ends;

    // Diagnostic only
    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;

}

// Generate an SQL statement based on the DBSelect arguments
void
MYSQLConnection::CreateSQLselect(char* Sqlcmd, 
                               const string& Domain,
                               const string& SearchFilter,
                               const string& start,
                               const string& end,
                               const FieldListtype& FieldList)
{
    // Use a strstream to create SQL command
    ostrstream strbuff(Sqlcmd, MAXQUERYLENGTH);

    // Field List iterator to used to fill fields in SQL statement
    FieldListtype::const_iterator p;

    // All SQL SELECT statements begin with SELECT
    strbuff << "SELECT ";

    // Add FieldList elements to SQL statement
    for (p = FieldList.begin(); p != FieldList.end(); p++)
    {
        strbuff << *p ;
        if ((p + 1) != FieldList.end())
        {
            // Insert a comma between field elements
            strbuff << ",";
        }
        else
        {
            // Insert a space, instead of a comma after the last element
            strbuff << " ";
        }
    }

    // Add Domain to SQL statement
    strbuff << "FROM " << Domain  << " ";

    // Use Filterparser to get SQL 'WHERE' clause from SearchFilter
    // then add WHERE clause to SQL statement
    MYSQLFilterparser filter(SearchFilter);
    // strbuff << "WHERE " << filter.GetParsedresult() << ends;
    strbuff << "WHERE " << filter.GetParsedresult() << " ";

    int diff = atoi(end.c_str()) - atoi(start.c_str());
    char buf [16];
    sprintf(buf, "%d", diff + 1);


    strbuff << "LIMIT " << start << "," << buf << ends;

    // Diagnostic only
     cout << endl << "SQL Command: " << Sqlcmd << endl << endl;

}


// Generate an SQL statement based on the DBSelect arguments
void
MYSQLConnection::CreateSQLselect(char* Sqlcmd,
                               const string& Domain,
                               const string& SearchFilter)
{
    // Use a strstream to create SQL command
    ostrstream strbuff(Sqlcmd, MAXQUERYLENGTH);

    // All SQL SELECT statements begin with SELECT
    strbuff << "SELECT COUNT(*) FROM " << Domain << " ";

    // Use Filterparser to get SQL 'WHERE' clause from SearchFilter
    // then add WHERE clause to SQL statement
    MYSQLFilterparser filter(SearchFilter);

    strbuff << "WHERE " << filter.GetParsedresult() << ends;

    // Diagnostic only
//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}


// Generate an SQL statement based on the DBSelect arguments
void
MYSQLConnection::CreateSQLselect( char* Sqlcmd, 
                                const string& Domain,
                                const string& SearchFilter,
                                const string& start,
                                const string& end,
                                const Ordertype& Order, 
                                const string& OrderField, 
                                const FieldListtype& FieldList )
{
    // Use a strstream to create SQL command
    ostrstream sqlbuff(Sqlcmd, MAXQUERYLENGTH);

    // Field List iterator to used to fill fields in SQL statement
    FieldListtype::const_iterator p;

    // SQL Order string
    string SQLorderstring;

    // All SQL SELECT statements begin with SELECT
    sqlbuff << "SELECT ";

    // Add FieldList elements to SQL statement
    for (p = FieldList.begin(); p != FieldList.end(); p++)
    {
        sqlbuff << *p ;
        if ((p + 1) != FieldList.end())
        {
            // Insert a comma between field elements
            sqlbuff << ",";
        }
        else
        {
            // Insert a space, instead of a comma after the last element
            sqlbuff << " ";
        }
    }

    // Add Domain to SQL statement
    sqlbuff << "FROM " << Domain  << " ";

    // Use Filterparser to get SQL 'WHERE' clause from SearchFilter
    // then add WHERE clause to SQL statement
    MYSQLFilterparser filter(SearchFilter);
    sqlbuff << "WHERE " << filter.GetParsedresult() << " ";

    // Set SQL Order string, based on Order being ASCENDING OR DESCENDING
    if (Order == ASCENDING)
        SQLorderstring = "ASC";
    else
        if (Order == DESCENDING)
            SQLorderstring = "DESC";
        else
            throw VDbException("Order must be ASCENDING or DESCEDING",
                __FILE__, __LINE__, 0);

    // Add Order clause
    sqlbuff << "ORDER BY " << OrderField << " " << SQLorderstring << " ";

    // Add Limit

    int diff = atoi(end.c_str()) - atoi(start.c_str());
    char buf [16];
    sprintf(buf, "%d", diff + 1);

    sqlbuff << "LIMIT " << start << "," << buf << ends;
}



// Generate an SQL statement based on the DBSelect arguments
void
MYSQLConnection::CreateSQLselect(char* Sqlcmd, const string& Domain,
     const string& SearchFilter, const FieldListtype& FieldList,
     const Ordertype& Order, const string& OrderField )
{
    // Use a strstream to create SQL command
    ostrstream sqlbuff(Sqlcmd, MAXQUERYLENGTH);

    // Field List iterator to used to fill fields in SQL statement
    FieldListtype::const_iterator p;

    // SQL Order string
    string SQLorderstring;

    // All SQL SELECT statements begin with SELECT
    sqlbuff << "SELECT ";

    // Add FieldList elements to SQL statement
    for (p = FieldList.begin(); p != FieldList.end(); p++)
    {
        sqlbuff << *p ;
        if ((p + 1) != FieldList.end())
        {
            // Insert a comma between field elements
            sqlbuff << ",";
        }
        else
        {
            // Insert a space, instead of a comma after the last element
            sqlbuff << " ";
        }
    }

    // Add Domain to SQL statement
    sqlbuff << "FROM " << Domain  << " ";

    // Use Filterparser to get SQL 'WHERE' clause from SearchFilter
    // then add WHERE clause to SQL statement
    MYSQLFilterparser filter(SearchFilter);
    sqlbuff << "WHERE " << filter.GetParsedresult() << " ";

    // Set SQL Order string, based on Order being ASCENDING OR DESCENDING
    if (Order == ASCENDING)
        SQLorderstring = "ASC";
    else
        if (Order == DESCENDING)
            SQLorderstring = "DESC";
        else
            throw VDbException("Order must be ASCENDING or DESCEDING",
                __FILE__, __LINE__, 0);

    // Add Order clause
    sqlbuff << "ORDER BY " << OrderField << " " << SQLorderstring << ends;

    // Diagnostic only
    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}

// Generate an SQL statement based on the DBUpdate arguments
void
MYSQLConnection::CreateSQLupdate(char* Sqlcmd, const string& Domain,
     const string& SearchFilter, const FieldValuetype& FieldValueSet)
{
    // Use a strstream to create SQL command
    ostrstream sqlbuff(Sqlcmd, MAXQUERYLENGTH);

    // Field List iterator to used to fill fields in SQL statement
    FieldValuetype::const_iterator p;

    // All supported SQL UPDATE statements begin with UPDATE Domain SET
    sqlbuff << "UPDATE " << Domain << " SET " ;

    // Add FieldList elements to SQL statement
    for (p = FieldValueSet.begin(); p != FieldValueSet.end(); p++)
    {

        sqlbuff << p->first << " = ";
//        sqlbuff << "\"" << p->second << "\"" ;

        // use single quotes instead.
        sqlbuff << "\'" << p->second << "\'" ;


        // p + 1 doesn't work on a STL map iterator? so increment then decrement
        if ((++p) != FieldValueSet.end())
        {
            p--;
            // Insert a comma between fieldvalue pairs
            sqlbuff << ", ";
        }
        else
        {
            p--;
            // Insert a space, instead of a comma after the last element
            sqlbuff << " ";
        }
    }

    // Use Filterparser to get SQL 'WHERE' clause from SearchFilter
    // then add WHERE clause to SQL statement
    MYSQLFilterparser filter(SearchFilter);
    sqlbuff << "WHERE " << filter.GetParsedresult() << ends;

    // Diagnostic only
    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}

// Generate an SQL statement based on the DBInsert arguments
void
MYSQLConnection::CreateSQLinsert(char* Sqlcmd, const string& Domain,
     const FieldValuetype& FieldValueSet)
{
    // Use a strstream to create SQL command
    ostrstream sqlbuff(Sqlcmd, MAXQUERYLENGTH);

    // Field List iterator to used to fill fields in SQL statement
    FieldValuetype::const_iterator p;

    // All supported SQL INSERT statements begin with INSERT INTO Domain
    sqlbuff << "INSERT INTO " << Domain ;

    // Add Field names
    sqlbuff << " ( " ;
    for (p = FieldValueSet.begin(); p != FieldValueSet.end(); p++)
    {
        sqlbuff << p->first;

        // p + 1 doesn't work on a map iterator? so increment then decrement
        if ((++p) != FieldValueSet.end())
        {
            p--;
            // Insert a comma between fieldvalue pairs
            sqlbuff << ", ";
        }
        else
        {
            p--;
            // Insert a space, instead of a comma after the last element
            sqlbuff << " ";
        }
    }
    sqlbuff << " ) " ;

    // Add Value names
    sqlbuff << "VALUES ( " ;
    for (p = FieldValueSet.begin(); p != FieldValueSet.end(); p++)
    {
        // sqlbuff << "\"" << p->second << "\"" ;
        sqlbuff << "\'" << p->second << "\'" ;

        // p + 1 doesn't work on a map iterator? so increment then decrement
        if ((++p) != FieldValueSet.end())
        {
            p--;
            // Insert a comma between fieldvalue pairs
            sqlbuff << ", ";
        }
        else
        {
            p--;
            // Insert a space, instead of a comma after the last element
            sqlbuff << " ";
        }
    }
    sqlbuff << " ) " << ends;

    // Diagnostic only
    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}

// Generate an SQL statement based on the DBDelete arguments
void
MYSQLConnection::CreateSQLdelete(char* Sqlcmd, const string& Domain,
     const string& SearchFilter)
{
    // Use a strstream to create SQL command
    ostrstream sqlbuff(Sqlcmd, MAXQUERYLENGTH);

    // All supported SQL DELETE statements begin with DELETE FROM Domain
    sqlbuff << "DELETE FROM " << Domain << " ";

    // Use Filterparser to get SQL 'WHERE' clause from SearchFilter
    // then add WHERE clause to SQL statement
    MYSQLFilterparser filter(SearchFilter);
    sqlbuff << "WHERE " << filter.GetParsedresult() << ends;

    // Diagnostic only
//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}

// Generate an SQL statement based on the DBCreate arguments
// This version of DBCreate does not take FieldDataTypeList which
// allows you to specify a different data type for each field.
// Therefore the data type for each field will be CHAR(255) or STRINGTYPE
void
MYSQLConnection::CreateSQLcreate(char* Sqlcmd, const string& Domain,
     const FieldListtype& FieldList, const string& PrimaryKey, 
     bool ExistenceCheck)
{
    // Use a strstream to create SQL command
    ostrstream sqlbuff(Sqlcmd, MAXQUERYLENGTH);

    // Field List iterator to used to fill fields in SQL statement
    FieldListtype::const_iterator p;

    // All supported SQL CREATE statements begin with...
    if (ExistenceCheck)
    {
        sqlbuff << "CREATE TABLE " << Domain <<  " ( " << endl;
    }
    else
    {
        sqlbuff << "CREATE TABLE IF NOT EXISTS " << Domain <<  " ( " << endl;
    }

    // Add FieldList elements to SQL statement
    for (p = FieldList.begin(); p != FieldList.end(); p++)
    {
        // If the field is the PRIMARY KEY, it must have the
        // NOT NULL option set.
        if ( *p == PrimaryKey )
            sqlbuff << *p << " CHAR(255)   NOT NULL, " << endl;
        else
            sqlbuff << *p << " CHAR(255), " << endl;
    }

    // Insert Primary Key
    sqlbuff << "PRIMARY KEY ( " << PrimaryKey << " ) ) " << ends;


    // Diagnostic only
//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}

// Generate an SQL statement based on the DBCreate arguments
void
MYSQLConnection::CreateSQLcreate(char* Sqlcmd, const string& Domain,
     const FieldListtype& FieldList, const FieldListDatatype& DatatypeList, 
     const string& PrimaryKey, bool ExistenceCheck)
{
    // Use a strstream to create SQL command
    ostrstream sqlbuff(Sqlcmd, MAXQUERYLENGTH);
    string Sqltype;
    string fieldnamebuff;
    Datatype Datatypebuff;

    // Field List iterator, and DataType iterator
    FieldListtype::const_iterator p1;
    FieldListDatatype::const_iterator p2;

    // All supported SQL CREATE statements begin with...
    if (ExistenceCheck)
    {
        sqlbuff << "CREATE TABLE " << Domain <<  " ( " << endl;
    }
    else
    {
        sqlbuff << "CREATE TABLE IF NOT EXISTS " << Domain <<  " ( " << endl;
    }

    // Add FieldList elements to SQL statement
    for (p1 = FieldList.begin(), p2 = DatatypeList.begin();
         p1 != FieldList.end(); p1++, p2++)
    {
        fieldnamebuff = *p1;
        Datatypebuff = *p2;

        GetSQLtypestring(Sqltype, Datatypebuff);

        sqlbuff << fieldnamebuff  << "  " << Sqltype << "  ";

        // If the field is the PRIMARY KEY, it must have the
        // NOT NULL option set.
        if ( *p1 == PrimaryKey )
            sqlbuff << "NOT NULL, " << endl;
        else
            sqlbuff << ", " << endl;
    }

    // Insert Primary Key
    sqlbuff << "PRIMARY KEY ( " << PrimaryKey << " ) ) " << ends;

    // Diagnostic only
//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}
