#ifndef MYSQLCONNECTION_HXX
#define MYSQLCONNECTION_HXX

#if defined(__svr4__) || defined(__SUNPRO_CC)
    #include <mysqlcppapi.h>
#else
    #include <sqlplus.hh>
#endif 

#include <string>
#include "DBConnection.hxx"

/// Forward Declare MYSQLResultset
class MYSQLResultset;

/// MYSQLConnection class
class MYSQLConnection : public DBConnection
{
    private:
        /// Constructor
        MYSQLConnection(const string& dbname, const string& mysqlhost,
                const string& mysqluser, const string& mysqlpw);

        /// Destructor
        virtual ~MYSQLConnection()
        { 
            delete myMySQLConnectionP;
        };

        ///
        DBResultset* DBSelect(const string& Domain,
                const string& SearchFilter, const FieldListtype& FieldList,
                const Ordertype& Order, const string &OrderField);

        ///
        DBResultset* DBSelect(const string& Domain,
                const string& SearchFilter, const FieldListtype& FieldList);

        ///
        DBResultset* DBSelect(const string& Domain,
                              const string& SearchFilter);

        ///
        DBResultset* DBSelect(const string& Domain,
                const string& SearchFilter, const string& start, const string& end, 
                const FieldListtype& FieldList);

        ///
        DBResultset* DBSelect(const string& Domain,
                              const string& SearchFilter,
                              const string& start,
                              const string& end,
                              const Ordertype& Order,
                              const string &OrderField,
                              const FieldListtype& FieldList);

        ///
        void DBUpdate(const string& Domain, const string& SearchFilter,
                const FieldValuetype& FieldValueSet);

        ///
        void DBUpdate(const string& Domain, DBResultset* Resultset,
                const FieldValuetype& FieldValueSet);

        ///
        void DBInsert(const string& Domain,
                const FieldValuetype& FieldValueSet);

        ///
        void DBDelete(const string& Domain, const string& SearchFilter);

        ///
        void DBDelete(const string& Domain, DBResultset* Resultset);
 
        ///
        string DBPrimaryKey( const string& Domain);

        ///
        ResultTabletype& DBDescribe(const string& Domain);

        ///
        ResultRecordtype& DBDescribe(void);

        ///
        void DBCreate(const string& Domain, const FieldListtype& FieldsList, 
                const string& PrimaryKey, bool ExistenceCheck);

        ///
        void DBCreate(const string& Domain, const FieldListtype& FieldsList, 
                const FieldListDatatype& Datatype, 
                const string& PrimaryKey, bool ExistenceCheck);

        ///
        void DBAlter(const string& Domain, const string& Fieldname);

        ///
        void DBAlter(const string& Domain, const string& Fieldname, 
                const Datatype Newtype);

        ///
        void DBDrop(const string& Domain, bool ExistenceCheck);

        ///
        bool DBVerify(const string& Domain);

        ///
        void CreateSQLselect(char *Sqlcmd, const string& Domain,
                const string& SearchFilter, const FieldListtype& FieldList,
                const Ordertype& Order, const string& OrderField);

        ///
        void CreateSQLselect(char *Sqlcmd, const string& Domain, 
                const string& SearchFilter, const FieldListtype& FieldList);

        ///
        void CreateSQLselect(char *Sqlcmd, const string& Domain,
                const string& SearchFilter, const string& start, const string& end,
                const FieldListtype& FieldList);

        ///
        void CreateSQLselect( char* Sqlcmd,
                              const string& Domain,
                              const string& SearchFilter, 
                              const string& start,
                              const string& end,
                              const Ordertype& Order, 
                              const string& OrderField,
                              const FieldListtype& FieldList );

        ///
        void CreateSQLselect(char *Sqlcmd, const string& Domain,
                const string& SearchFilter );

        ///
        void CreateSQLupdate(char *Sqlcmd, const string& Domain, 
                const string& SearchFilter, 
                const FieldValuetype& FieldValueSet);
        ///
        void CreateSQLinsert(char *Sqlcmd, const string& Domain, 
                const FieldValuetype& FieldValueSet);

        ///
        void CreateSQLdelete(char *Sqlcmd, const string& Domain, 
                const string& SearchFilter);
        ///
        void CreateSQLcreate(char *Sqlcmd, const string& Domain, 
                const FieldListtype& FieldList, 
                const string& PrimaryKey, bool ExistenceCheck);
        ///
        void CreateSQLcreate(char *Sqlcmd, const string& Domain, 
                const FieldListtype& FieldList, 
                const FieldListDatatype& DatatypeList, 
                const string& PrimaryKey, 
                bool ExistenceCheck);

        /// Return the MySQL equivalent of DBAPI Datatype
        void GetSQLtypestring(string& Sqltype, Datatype type);

        /// MySQL Connection handle
        MysqlConnection* myMySQLConnectionP;

        /// vector<vector <string> > used by DBDescribe(Domain)
        ResultTabletype myDescribeTable;

        /// vector <string> used by DBDescribe()
        ResultRecordtype myDescribeRecord;

        /// mySQLResultsethandle a the pointer to the resultset
        MYSQLResultset* mySQLResultsetP;

    public:
        ///
        friend class ConnectionManager;
};
#endif
