#include <iostream>
#include "MYSQLResultset.hxx"
#include "VDbException.hxx"

using namespace std;

// SQLResetset constructor
MYSQLResultset::MYSQLResultset(const MysqlRes& result, const string& searchfilter):
mySearchFilter(searchfilter)
{
    myMySQLresult = result;
    myRecordCount = myMySQLresult.size();
    myFieldList = GetFields();
};

// GetRecords[offset] - returns the record at offset
const ResultRecordtype&
MYSQLResultset::operator[](unsigned short offset) const
{
    static ResultRecordtype returnResult;
    string fieldbuff;
    MysqlRow row;
    unsigned short i;

    // Check offsets are within range of results
    if (offset > myRecordCount) 
    {
        throw VDbException("Record Offset out of range", __FILE__, __LINE__, 0);
    }

    // Clear contents of returnResult for now
    returnResult.clear();

    row = myMySQLresult[offset];

    // If Records exist, iterate through fields filling returnResult
    if (myRecordCount)
    {
        for (i = 0; i < myFieldList.size(); i++)
        {
            fieldbuff = row[myFieldList[i]];
            returnResult.push_back(fieldbuff);
        }
    }

    return returnResult;
}

// GetRecord(offset) - returns the record at offset
const ResultRecordtype&
MYSQLResultset::GetRecord(unsigned short offset) const
{
    static ResultRecordtype returnResult;
    string fieldbuff;
    MysqlRow row;
    unsigned short i;

    // Check offsets are within range of results
    if (offset > myRecordCount) 
    {
        throw VDbException("Record Offset out of range", __FILE__, __LINE__, 0);
    }

    // Clear contents of returnResult for now
    returnResult.clear();

    row = myMySQLresult[offset];

    // If Records exist, iterate through fields filling returnResult
    if (myRecordCount)
    {
        for (i = 0; i < myFieldList.size(); i++)
        {
            fieldbuff = row[myFieldList[i]];
            returnResult.push_back(fieldbuff);
        }
    }

    return returnResult;
};

// GetRecords(offset1, offset2) - returns all records between offset1
// and offset2
const ResultTabletype&
MYSQLResultset::GetRecords(unsigned short offset1, unsigned short offset2) const
{
    static ResultTabletype returnResult;
    string fieldbuff;
    ResultRecordtype recordbuff;
    MysqlRow row;
    unsigned short recordnumber;
    unsigned short i;

    // Check offsets are within range of results
    if ((offset1 > offset2) || (offset1 > myRecordCount) 
            || (offset2 > myRecordCount))
    {
        throw VDbException("Record Offsets out of range", __FILE__, __LINE__, 0);
    }

    // Clear contents of returnResult for now
    returnResult.clear();

    for (recordnumber = offset1; recordnumber <= offset2; recordnumber++)
    {
        row = myMySQLresult[recordnumber];
        for (i = 0; i < myFieldList.size(); i++)
        {
            fieldbuff = row[myFieldList[i]];
            recordbuff.push_back(fieldbuff);
        }

        // Insert record buffer into result table
        returnResult.push_back(recordbuff);

        // Clear recordbuff for next record
        recordbuff.clear();
    }
    return returnResult;
};
 
// GetRecords - returns all records from a search
const ResultTabletype&
MYSQLResultset::GetRecords() const
{
    static ResultTabletype returnResult;
    ResultRecordtype recordbuff;
    MysqlRow row;
    string fieldbuff;
    unsigned short recordnumber;
    unsigned short i;

    // Clear contents of returnResult for now
    returnResult.clear();

    for (recordnumber = 0; recordnumber < myRecordCount; recordnumber++)
    {
        row = myMySQLresult[recordnumber];
        for (i = 0; i < myFieldList.size(); i++)
        {
            fieldbuff = row[myFieldList[i]];
            recordbuff.push_back(fieldbuff);
        }

        // Insert record buffer into result table
        returnResult.push_back(recordbuff);

        // Clear recordbuff for next record
        recordbuff.clear();
    }
    return returnResult;
};

// GetRecordCount- Return Record count
const unsigned int
MYSQLResultset::GetRecordCount(void) const
{
    return myRecordCount;
};

// GetFields - Return fields on a resultset
const FieldListtype&
MYSQLResultset::GetFields(void) const
{
    static FieldListtype returnResult;
    unsigned short i;

    // Clear contents of returnResult for now
    returnResult.clear();

    // If Records exist, fill myFieldList
    if (myRecordCount) 
    {
        for (i = 0; i < myMySQLresult.columns(); i++)
        {
            returnResult.push_back(myMySQLresult.names(i));
        }
    }

    return returnResult;
}

// GetmySearchFilter- Return searchfilter that was used to create this
//    resultset.  This is normally used when a resultset is passed back 
//    to DBUpdate() or DBSelect()
const string&
MYSQLResultset::GetSearchFilter(void) const
{
    return mySearchFilter;
};
