#include <iostream>
#include <string>
#include <stdio.h>
#include "VDbException.hxx"
#include "PGSQLFilterparser.hxx"
#include "PGSQLResultset.hxx"
#include "PGSQLConnection.hxx"

using namespace std;

// PGSQLConnection Constructor
PGSQLConnection::PGSQLConnection(const string& dbname, const string& dbhost,
        const string& dbuser, const string& dbpassword)
{
    // make a connection to postgresql backend
    char* pgport = NULL;      // port of the server
    char* pgoptions = NULL;   // special options to start up the server
    char* pgtty = NULL;       // debugging tty for the server

    myPgSQLConnectionP = PQsetdb(
            dbhost.c_str(),
            pgport, 
            pgoptions, 
            pgtty, 
            dbname.c_str());

    // check connection
    if (PQstatus(myPgSQLConnectionP) == CONNECTION_BAD)
    {
        cout << "PGSQLConnection Error: Connection failed" << endl;
        cout << "dbname: " << dbname.c_str() << endl;
        cout << "dbhost: " << dbhost.c_str() << endl;
        cout << PQerrorMessage(myPgSQLConnectionP) << endl;
        string err = "PgSQLConnection Error: Connection failed to database ";
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
};

// DBSelect- Returns a DBResultset
DBResultset*
PGSQLConnection::DBSelect(const string& Domain,
     const string& SearchFilter, const FieldListtype& FieldList,
     const Ordertype& Order, const string &OrderField)
{
    char Sqlcmd[MAXQUERYLENGTH];
    PGresult*  pgresult;

    // Create SQL statement
    CreateSQLselect(
            Sqlcmd, 
            Domain, 
            SearchFilter, 
            FieldList,
            Order, 
            OrderField);

    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_TUPLES_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBSelect Error: PostgreSQL error: " + Pgerr;
                   
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    else
        myPGSQLResultsetP = new PGSQLResultset(pgresult, SearchFilter);

    return myPGSQLResultsetP;
}

// DBSelect- Returns a DBResultset
DBResultset*
PGSQLConnection::DBSelect(const string& Domain,
     const string& SearchFilter, const FieldListtype& FieldList)
{
    char Sqlcmd[MAXQUERYLENGTH];
    PGresult*  pgresult;

    // Create SQL statement
    CreateSQLselect(Sqlcmd, Domain, SearchFilter, FieldList);
    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_TUPLES_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBSelect Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    else
        myPGSQLResultsetP = new PGSQLResultset(pgresult, SearchFilter);

    return myPGSQLResultsetP;
};

DBResultset*
PGSQLConnection::DBSelect(const string& Domain,
                        const string& SearchFilter )
{
    char Sqlcmd[MAXQUERYLENGTH];
    PGresult*  pgresult;

    // Create SQL statement
    CreateSQLselect(Sqlcmd, 
            Domain, 
            SearchFilter );

    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_TUPLES_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBSelect Error: PostgreSQL error: " + Pgerr;
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    else
        myPGSQLResultsetP = new PGSQLResultset(pgresult, SearchFilter);

    return myPGSQLResultsetP;
}


DBResultset*
PGSQLConnection::DBSelect(const string& Domain,
                        const string& SearchFilter,
                        const string& start,
                        const string& end,
                        const Ordertype& Order, 
                        const string &OrderField,
                        const FieldListtype& FieldList)
{
    char Sqlcmd[MAXQUERYLENGTH];
    PGresult*  pgresult;

    // Create SQL statement
    CreateSQLselect( Sqlcmd, 
                     Domain, 
                     SearchFilter, 
                     start, 
                     end, 
                     Order, 
                     OrderField, 
                     FieldList );

    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_TUPLES_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBSelect Error: PostgreSQL error: " + Pgerr;
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    else
        myPGSQLResultsetP = new PGSQLResultset(pgresult, SearchFilter);

    return myPGSQLResultsetP;
};


DBResultset*
PGSQLConnection::DBSelect(const string& Domain,
                        const string& SearchFilter, 
                        const string& start,
                        const string& end,
                        const FieldListtype& FieldList)
{
    char Sqlcmd[MAXQUERYLENGTH];
    PGresult*  pgresult;

    // Create SQL statement
    CreateSQLselect( Sqlcmd, 
                     Domain, 
                     SearchFilter, 
                     start, 
                     end, 
                     FieldList );

    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_TUPLES_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBSelect Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    else
        myPGSQLResultsetP = new PGSQLResultset(pgresult, SearchFilter);

    // Free memory used by PGresult pointer
    PQclear(pgresult);

    return myPGSQLResultsetP;
};

// DBPrimaryKey - returns the primary key for a domain
// NOTE:  assumes only one primary key exists.  If more
//        than one, then it'll get the first one from
//        the list. If none exists, don't throw exception,
//        just return null
string
PGSQLConnection::DBPrimaryKey (const string& Domain)
{
    char Sqlcmd[MAXQUERYLENGTH];
    ostrstream buff(Sqlcmd, MAXQUERYLENGTH);
    string resultString;
    PGresult* pgresult;

    // Clear contents of DescribeRecord for each query
    myDescribeRecord.clear();

    //  Check if table even exists first
    if(!DBVerify(Domain))
    {
        string err = "DBPrimaryKey Error: Table does not exist: " + Domain;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
   
    // This is PostgreSQL equivalent of mySQL's 'DESCRIBE TABLE' 
    buff << "SELECT pg_class.relname, pg_attribute.attname "
         << "FROM pg_class, pg_attribute, pg_index "
         << "WHERE pg_class.oid = pg_attribute.attrelid AND "
         << " pg_class.oid = pg_index.indrelid AND "
         << " pg_index.indkey[0] = pg_attribute.attnum AND "
         << " pg_class.relname = '" << Domain << "' AND "
         << " pg_index.indisprimary = 't'; "
         << ends;

    // cout << endl << "SQL Command: " << Sqlcmd << endl << endl;

    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);
    if (!pgresult || PQresultStatus(pgresult) != PGRES_TUPLES_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBPrimaryKey Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    // The tablename, index are the only value returned from 
    // this query. This query will be valid even if there
    // are no records returned, so return an empty string if 
    // no primary key exists
    if (PQntuples(pgresult) > 0 )
        resultString = PQgetvalue(pgresult, 0, 1);
    else
        resultString = "";

    // Free memory used by PGresult pointer
    PQclear(pgresult);
    // return string by value, an empty string if nothing found
    return resultString; 
}

// DBDescribe- Returns information on one domain
ResultTabletype&
PGSQLConnection::DBDescribe(const string& Domain)
{
    char Sqlcmd[MAXQUERYLENGTH];
    ostrstream buff(Sqlcmd, MAXQUERYLENGTH);
    ResultRecordtype recordbuff;
    string fieldbuff;
    unsigned short recordnumber;
    PGresult* pgresult;

    // Clear contents of DescribeTable for each query
    myDescribeTable.clear();

    // SQL statement for getting table information
    buff << "SELECT a.attname, a.atttypid, a.attnotnull "
         << " FROM pg_class c, pg_attribute a WHERE c.relname = '"
         << Domain << "' AND a.attnum > 0 AND a.attrelid = c.oid" << ends;

    // cout << endl << "SQL Command: " << Sqlcmd << endl << endl;

    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_TUPLES_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBDescribe Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    // 
    for (recordnumber = 0; recordnumber < PQntuples(pgresult); recordnumber++)
    {
        // First column is Fieldname
        fieldbuff = PQgetvalue(pgresult, recordnumber, 0);
        recordbuff.push_back(fieldbuff);

        // Second column is integer type id format.
        // Convert the PgSQL datatype, to the DBmodule
        // supported Datatype.
        char* typebuff = PQgetvalue(pgresult, recordnumber, 1);
        switch (atoi(typebuff))
        {
            case 1043:   //varchar
                recordbuff.push_back("STRINGTYPE");
                break;
            case 23:     //integer
                recordbuff.push_back("INTEGERTYPE");
                break;
            case 700:    //float  
                recordbuff.push_back("FLOATTYPE");
                break;
            case 25:     //text
                recordbuff.push_back("TEXTBLOBTYPE");
                break;
            case 26:     //blob
                recordbuff.push_back("BINBLOBTYPE");
                break;
            case 1184:   //timestamp
                recordbuff.push_back("TIMESTAMPTYPE");
                break;
            case 1082:   //date
                recordbuff.push_back("DATETYPE");
                break;
            case 1083:   //time
                recordbuff.push_back("TIMETYPE");
                break;
            default:
                string err = "DBDescribe Error: Unkown datatype ";
                throw VDbException(err, __FILE__, __LINE__, 0);
            break;
        }

        // Use the DBPrimaryKey method to determine if this field
        // is the primary key.
        string primarykey = DBPrimaryKey(Domain);
        if (fieldbuff == primarykey)
        {
            recordbuff.push_back("PRIMARYKEY");
        }


        // Insert record buffer into result table
        myDescribeTable.push_back(recordbuff);

        // Clear recordbuff for next record
        recordbuff.clear();
    }
    // Free memory used by PGresult pointer
    PQclear(pgresult);

    return myDescribeTable;
};

// DBDescribe- Returns a list of all domains in database
ResultRecordtype&
PGSQLConnection::DBDescribe()
{
    char Sqlcmd[MAXQUERYLENGTH];
    ostrstream buff(Sqlcmd, MAXQUERYLENGTH);
    string fieldbuff;
    unsigned short recordnumber;
    PGresult* pgresult;

    // Clear contents of DescribeRecord for each query
    myDescribeRecord.clear();

    // This is mysql's equivalent of "SHOW TABLES"
     buff  << "SELECT c.relname as \"Name\" "
       << "FROM pg_class c, pg_user u "
       << "WHERE c.relowner = u.usesysid AND c.relkind = 'r' "
       << "  AND c.relname !~ '^pg_' "
       << "  ORDER BY c.relname " << ends;

    // cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_TUPLES_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBDescribe Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    //  Loop for each row
    for (recordnumber = 0; recordnumber < PQntuples(pgresult); recordnumber++)
    {
        // Only one column is retreived, the table name
        fieldbuff = PQgetvalue(pgresult, recordnumber, 0);
        myDescribeRecord.push_back(fieldbuff);
    }
    // Free memory used by PGresult pointer
    PQclear(pgresult);

    return myDescribeRecord;
};

// DBUpdate- Updates a record in an existing domain
void
PGSQLConnection::DBUpdate(const string& Domain, const string& SearchFilter,
     const FieldValuetype& FieldValueSet)
{
    char Sqlcmd[MAXQUERYLENGTH];
    PGresult* pgresult;

    // Create SQL statement
    CreateSQLupdate(Sqlcmd, Domain, SearchFilter, FieldValueSet);

    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_COMMAND_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBUpdate Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    // Free memory used by PGresult pointer
    PQclear(pgresult);
};

// DBUpdate- Updates a record in an existing domain, based on a
//           previous resultset.
void
PGSQLConnection::DBUpdate(const string& Domain, DBResultset* Resultset,
     const FieldValuetype& FieldValueSet)
{
    char Sqlcmd[MAXQUERYLENGTH];
    PGresult* pgresult;

    // Get SearchFilter from resultset, use it to generate SQL command
    const string searchfilter = Resultset->GetSearchFilter();

    // Create SQL statement
    CreateSQLupdate(Sqlcmd, Domain, searchfilter, FieldValueSet);

    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_COMMAND_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBSelect Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    // Free memory used by PGresult pointer
    PQclear(pgresult);
};

// DBInsert- Adds a record to an existing domain
void
PGSQLConnection::DBInsert(const string& Domain,
     const FieldValuetype& FieldValueSet)
{
    char Sqlcmd[MAXQUERYLENGTH];
    PGresult* pgresult;

    // Create SQL statement
    CreateSQLinsert(Sqlcmd, Domain, FieldValueSet);

    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_COMMAND_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBInsert Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    // Free memory used by PGresult pointer
    PQclear(pgresult);
};

// DBDelete- Deletes a record from an existing domain, based on a
//           searchfilter
void
PGSQLConnection::DBDelete(const string& Domain, const string& SearchFilter)
{
    char Sqlcmd[MAXQUERYLENGTH];
    PGresult* pgresult;

    // Create SQL statement
    CreateSQLdelete(Sqlcmd, Domain, SearchFilter);

    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_COMMAND_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBDelete Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    // Free memory used by PGresult pointer
    PQclear(pgresult);
};

// DBDelete- Deletes a record from an existing domain, based on a
//           previous resultset
void
PGSQLConnection::DBDelete(const string& Domain, DBResultset* Resultset)
{
    char Sqlcmd[MAXQUERYLENGTH];
    PGresult* pgresult;

    // Get SearchFilter from resultset, use it to generate SQL command
    const string searchfilter = Resultset->GetSearchFilter();

    // Create SQL statement
    CreateSQLdelete(Sqlcmd, Domain, searchfilter);

    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_COMMAND_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBDelete Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    // Free memory used by PGresult pointer
    PQclear(pgresult);
};

// DBCreate- Creates a new domain. This version assumes all columns to
//           be of datatype STRING, which is msyql CHAR(255)
void
PGSQLConnection::DBCreate(const string& Domain, const FieldListtype& FieldList,
        const string& PrimaryKey, bool ExistenceCheck)
{
    char Sqlcmd[MAXQUERYLENGTH];
    PGresult* pgresult;

    // Create SQL statement
    CreateSQLcreate(Sqlcmd, 
            Domain, 
            FieldList, 
            PrimaryKey);

    if (ExistenceCheck)
    {
        //  CHECKEXISTENCE sent, so let the backend throw the flag
        //  if the table exists
        pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);
    }
    else
    {
        //  IGNOREEXISTENCE sent, so quietly do nothing if 
        //  table already exists
        if(!DBVerify(Domain))
        {
            pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);
        }
        else
            return;
    }

    if (!pgresult || PQresultStatus(pgresult) != PGRES_COMMAND_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBCreate Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    // Free memory used by PGresult pointer
    PQclear(pgresult);
};

// DBCreate- Creates a new domain. This version requires the client to
//           specify the datatype of each column
void
PGSQLConnection::DBCreate(const string& Domain, const FieldListtype& FieldList,
        const FieldListDatatype& DatatypeList, const string& PrimaryKey, 
        bool ExistenceCheck)
{
    char Sqlcmd[MAXQUERYLENGTH];
    PGresult* pgresult;

    // Check if a data type exists for each field
    if (FieldList.size() != DatatypeList.size())
    {
        throw VDbException("DBCreate Error: \
            Number of Datatypes does not match number of Fields",
            __FILE__, __LINE__, 0);
    }

    // Create SQL statement
    CreateSQLcreate(Sqlcmd, 
            Domain, 
            FieldList, 
            DatatypeList, 
            PrimaryKey);

    if (ExistenceCheck)
    {
        //  CHECKEXISTENCE sent, so let the backend throw the flag
        //  if the table exists
        pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);
    }
    else
    {
        //  IGNOREEXISTENCE sent, so quietly do nothing if 
        //  table already exists
        if(!DBVerify(Domain))
        {
            pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);
        }
        else
            return;
    }
    if (!pgresult || PQresultStatus(pgresult) != PGRES_COMMAND_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBCreate Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    PQclear(pgresult);
};

// DBAlter- Drop a column in a table
void
PGSQLConnection::DBAlter(const string& Domain, const string& Fieldname)
{
    char Sqlcmd[MAXQUERYLENGTH];
    ostrstream buff(Sqlcmd, MAXQUERYLENGTH);
    ResultTabletype oldtable;
    PGresult*  pgresult;
    bool foundfieldflag = false;
    ResultTabletype::const_iterator tp;
    FieldListtype::const_iterator fp;

    // PostgreSQL doesn't support DROP COLUMN.
    // Workaround:
    // SELECT ...  -- all columns but the one you want to remove
    // INTO TABLE temp_
    // FROM old_table;
    // DROP TABLE old_table;
    // ALTER TABLE temp_ RENAME TO old_table;

    //  Check if table exists first
    if(!DBVerify(Domain))
    {
        string err = "DBPrimaryKey Error: Table does not exist: " + Domain;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    // All SQL SELECT statements begin with SELECT
    buff << "SELECT ";

    // Use DBDescribe to get old fieldnames
    oldtable = DBDescribe(Domain);

    for (tp = oldtable.begin(); tp != oldtable.end(); tp++)
    {
        fp = tp->begin();

        // Add all but the field you're dropping to the temp table
        if(*fp != Fieldname) 
        {
            buff << *fp;
            if ((tp + 1) != oldtable.end())
                buff << ","; // comma between field elements
            else 
                buff << " "; // Space after last element
        }
        else
            foundfieldflag = true;
    }

    if(!foundfieldflag)
    {
        string err = "DBAlter Error: Column: " + Fieldname + 
            " does not exist in: " + Domain;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    buff << "INTO TABLE temp_" << Domain << " ";
    buff << "FROM " << Domain << " ; ";
    buff << "DROP TABLE " << Domain << " ; ";
    buff << "ALTER TABLE temp_" << Domain << " ";
    buff << "RENAME TO " << Domain << ends;

    // cout << endl << "SQL Command: " << Sqlcmd << endl << endl;

    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_COMMAND_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBAlter Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    PQclear(pgresult);
};

// DBAlter- Add a column to a table
void
PGSQLConnection::DBAlter(const string& Domain, const string& Fieldname,
        const Datatype Newtype)
{
    char Sqlcmd[MAXQUERYLENGTH];
    string Sqltype;
    ostrstream buff(Sqlcmd, MAXQUERYLENGTH);
    PGresult*  pgresult;

    //  Check if table even exists first
    if(!DBVerify(Domain))
    {
        string err = "DBPrimaryKey Error: Table does not exist: " + Domain;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    GetSQLtypestring(Sqltype, Newtype);

    buff << "ALTER TABLE " << Domain << " ";
    buff << "ADD COLUMN " << Fieldname << " ";
    buff << Sqltype << ends;

    // cout << endl << "SQL Command: " << Sqlcmd << endl << endl;

    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_COMMAND_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBAlter Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    // Free memory used by PGresult pointer
    PQclear(pgresult);
};

// DBDrop- Delete a domain.  If ExistenceCheck is true, use default
// SQL DROP TABLE which errors if table does not exist. If ExistenceCheck
// is false, use LOCK TABLE to check if table exists.
void
PGSQLConnection::DBDrop(const string& Domain, bool ExistenceCheck)
{
    char Sqlcmd[MAXQUERYLENGTH];
    ostrstream Sqlbuff(Sqlcmd, MAXQUERYLENGTH);
    PGresult* pgresult;

    // Static SQL statement for Deleting Table
    if (ExistenceCheck)
    {
        //  CHECKEXISTENCE sent, so let the backend throw the flag
        //  if the tables does not exist
        Sqlbuff << "DROP TABLE " << Domain << ends;
    }
    else
    {
        //  IGNOREEXISTENCE sent, so quietly do nothing if 
        //  table does not exist
        if(DBVerify(Domain))
        {
            Sqlbuff << "DROP TABLE " << Domain << ends;
        }
        else
        {
            // Table doesn't exist, so return
            return;
        }
    }

    // cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_COMMAND_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBDrop Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }
    // Free memory used by PGresult pointer
    PQclear(pgresult);
};

// DBVerify- Verify the existence of a domain.
bool
PGSQLConnection::DBVerify(const string& Domain)
{
    char Sqlcmd[MAXQUERYLENGTH];
    ostrstream buff(Sqlcmd, MAXQUERYLENGTH);
    string fieldbuff;
    unsigned short recordnumber;
    PGresult* pgresult;

    // Clear contents of DescribeRecord for each query
    myDescribeRecord.clear();

    // This is mysql's equivalent of "SHOW TABLES"
     buff  << "SELECT c.relname as \"Name\" "
       << "FROM pg_class c, pg_user u "
       << "WHERE c.relowner = u.usesysid AND c.relkind = 'r' "
       << "  AND c.relname !~ '^pg_' " << ends;

    // cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
    pgresult = PQexec(myPgSQLConnectionP, Sqlcmd);

    if (!pgresult || PQresultStatus(pgresult) != PGRES_TUPLES_OK)
    {
        string Pgerr = PQresultErrorMessage(pgresult);
        string err = "DBDescribe Error: PostgreSQL error: " + Pgerr;
        // Free memory used by PGresult pointer
        PQclear(pgresult);
        throw VDbException(err, __FILE__, __LINE__, 0);
    }

    //  Loop for each row
    for (recordnumber = 0; recordnumber < PQntuples(pgresult); recordnumber++)
    {
        // Only one column is retreived, the table name
        fieldbuff = PQgetvalue(pgresult, recordnumber, 0);
        if (fieldbuff == Domain)
        {
            // Free memory used by PGresult pointer
            PQclear(pgresult);
            return true;
        }
    }
    // Free memory used by PGresult pointer
    PQclear(pgresult);
    return false;
};

// Convert from public enumerated data type to private data type
// that this dbmodule supports
void
PGSQLConnection::GetSQLtypestring(string& Sqltype, Datatype type)
{
    /// PgSQL Equivalent Data types
    const char SQLtypestring[][16] =
    {
        "varchar(255)",   // STRING
        "integer",        // INTEGER
        "float",          // FLOAT
        "text",           // TEXTBLOB
        "blob",           // BINBLOB
        "timestamp",      // TIMESTAMP
        "date",           // DATE
        "time",           // TIME
    };

    Sqltype = SQLtypestring[type];
}

// Generate an SQL statement based on the DBSelect arguments
void 
PGSQLConnection::CreateSQLselect(char* Sqlcmd, const string& Domain,
     const string& SearchFilter, const FieldListtype& FieldList)
{
    // Use a strstream to create SQL command
    ostrstream strbuff(Sqlcmd, MAXQUERYLENGTH);

    // Field List iterator to used to fill fields in SQL statement
    FieldListtype::const_iterator p;

    // All SQL SELECT statements begin with SELECT
    strbuff << "SELECT ";

    // Add FieldList elements to SQL statement
    for (p = FieldList.begin(); p != FieldList.end(); p++)
    {
        strbuff << *p ;
        if ((p + 1) != FieldList.end())
        {
            // Insert a comma between field elements
            strbuff << ",";
        }
        else
        {
            // Insert a space, instead of a comma after the last element
            strbuff << " ";
        }
    }

    // Add Domain to SQL statement
    strbuff << "FROM " << Domain  << " ";

    // Use Filterparser to get SQL 'WHERE' clause from SearchFilter
    // then add WHERE clause to SQL statement
    PGSQLFilterparser filter(SearchFilter);
    strbuff << "WHERE " << filter.GetParsedresult() << ends;

    // cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}

// Generate an SQL statement based on the DBSelect arguments
void
PGSQLConnection::CreateSQLselect(char* Sqlcmd, 
                               const string& Domain,
                               const string& SearchFilter,
                               const string& start,
                               const string& end,
                               const FieldListtype& FieldList)
{
    // Use a strstream to create SQL command
    ostrstream strbuff(Sqlcmd, MAXQUERYLENGTH);

    // Field List iterator to used to fill fields in SQL statement
    FieldListtype::const_iterator p;

    // All SQL SELECT statements begin with SELECT
    strbuff << "SELECT ";

    // Add FieldList elements to SQL statement
    for (p = FieldList.begin(); p != FieldList.end(); p++)
    {
        strbuff << *p ;
        if ((p + 1) != FieldList.end())
        {
            // Insert a comma between field elements
            strbuff << ",";
        }
        else
        {
            // Insert a space, instead of a comma after the last element
            strbuff << " ";
        }
    }

    // Add Domain to SQL statement
    strbuff << "FROM " << Domain  << " ";

    // Use Filterparser to get SQL 'WHERE' clause from SearchFilter
    // then add WHERE clause to SQL statement
    PGSQLFilterparser filter(SearchFilter);
    // strbuff << "WHERE " << filter.GetParsedresult() << ends;
    strbuff << "WHERE " << filter.GetParsedresult() << " ";

    int diff = atoi(end.c_str()) - atoi(start.c_str());
    char buf [16];
    sprintf(buf, "%d", diff + 1);

    strbuff << "LIMIT " << start << "," << buf << ends;

    // cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}

// Generate an SQL statement based on the DBSelect arguments
void
PGSQLConnection::CreateSQLselect(char* Sqlcmd,
                               const string& Domain,
                               const string& SearchFilter)
{
    // Use a strstream to create SQL command
    ostrstream strbuff(Sqlcmd, MAXQUERYLENGTH);

    // All SQL SELECT statements begin with SELECT
    strbuff << "SELECT COUNT(*) FROM " << Domain << " ";

    // Use Filterparser to get SQL 'WHERE' clause from SearchFilter
    // then add WHERE clause to SQL statement
    PGSQLFilterparser filter(SearchFilter);

    strbuff << "WHERE " << filter.GetParsedresult() << ends;

    // cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}


// Generate an SQL statement based on the DBSelect arguments
void
PGSQLConnection::CreateSQLselect( char* Sqlcmd, 
                                const string& Domain,
                                const string& SearchFilter,
                                const string& start,
                                const string& end,
                                const Ordertype& Order, 
                                const string& OrderField, 
                                const FieldListtype& FieldList )
{
    // Use a strstream to create SQL command
    ostrstream sqlbuff(Sqlcmd, MAXQUERYLENGTH);

    // Field List iterator to used to fill fields in SQL statement
    FieldListtype::const_iterator p;

    // SQL Order string
    string SQLorderstring;

    // All SQL SELECT statements begin with SELECT
    sqlbuff << "SELECT ";

    // Add FieldList elements to SQL statement
    for (p = FieldList.begin(); p != FieldList.end(); p++)
    {
        sqlbuff << *p ;
        if ((p + 1) != FieldList.end())
        {
            // Insert a comma between field elements
            sqlbuff << ",";
        }
        else
        {
            // Insert a space, instead of a comma after the last element
            sqlbuff << " ";
        }
    }

    // Add Domain to SQL statement
    sqlbuff << "FROM " << Domain  << " ";

    // Use Filterparser to get SQL 'WHERE' clause from SearchFilter
    // then add WHERE clause to SQL statement
    PGSQLFilterparser filter(SearchFilter);
    sqlbuff << "WHERE " << filter.GetParsedresult() << " ";

    // Set SQL Order string, based on Order being ASCENDING OR DESCENDING
    if (Order == ASCENDING)
        SQLorderstring = "ASC";
    else
        if (Order == DESCENDING)
            SQLorderstring = "DESC";
        else
            throw VDbException("Order must be ASCENDING or DESCEDING",
                __FILE__, __LINE__, 0);

    // Add Order clause
    sqlbuff << "ORDER BY " << OrderField << " " << SQLorderstring << " ";

    // Add Limit

    int diff = atoi(end.c_str()) - atoi(start.c_str());
    char buf [16];
    sprintf(buf, "%d", diff + 1);

    sqlbuff << "LIMIT " << start << "," << buf << ends;
}



// Generate an SQL statement based on the DBSelect arguments
void
PGSQLConnection::CreateSQLselect(char* Sqlcmd, const string& Domain,
     const string& SearchFilter, const FieldListtype& FieldList,
     const Ordertype& Order, const string& OrderField )
{
    // Use a strstream to create SQL command
    ostrstream sqlbuff(Sqlcmd, MAXQUERYLENGTH);

    // Field List iterator to used to fill fields in SQL statement
    FieldListtype::const_iterator p;

    // SQL Order string
    string SQLorderstring;

    // All SQL SELECT statements begin with SELECT
    sqlbuff << "SELECT ";

    // Add FieldList elements to SQL statement
    for (p = FieldList.begin(); p != FieldList.end(); p++)
    {
        sqlbuff << *p ;
        if ((p + 1) != FieldList.end())
        {
            // Insert a comma between field elements
            sqlbuff << ",";
        }
        else
        {
            // Insert a space, instead of a comma after the last element
            sqlbuff << " ";
        }
    }

    // Add Domain to SQL statement
    sqlbuff << "FROM " << Domain  << " ";

    // Use Filterparser to get SQL 'WHERE' clause from SearchFilter
    // then add WHERE clause to SQL statement
    PGSQLFilterparser filter(SearchFilter);
    sqlbuff << "WHERE " << filter.GetParsedresult() << " ";

    // Set SQL Order string, based on Order being ASCENDING OR DESCENDING
    if (Order == ASCENDING)
        SQLorderstring = "ASC";
    else
        if (Order == DESCENDING)
            SQLorderstring = "DESC";
        else
            throw VDbException("Order must be ASCENDING or DESCEDING",
                __FILE__, __LINE__, 0);

    // Add Order clause
    sqlbuff << "ORDER BY " << OrderField << " " << SQLorderstring << ends;

//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}

// Generate an SQL statement based on the DBUpdate arguments
void
PGSQLConnection::CreateSQLupdate(char* Sqlcmd, const string& Domain,
     const string& SearchFilter, const FieldValuetype& FieldValueSet)
{
    // Use a strstream to create SQL command
    ostrstream sqlbuff(Sqlcmd, MAXQUERYLENGTH);

    // Field List iterator to used to fill fields in SQL statement
    FieldValuetype::const_iterator p;

    // All supported SQL UPDATE statements begin with UPDATE Domain SET
    sqlbuff << "UPDATE " << Domain << " SET " ;

    // Add FieldList elements to SQL statement
    for (p = FieldValueSet.begin(); p != FieldValueSet.end(); p++)
    {

        sqlbuff << p->first << " = ";
//        sqlbuff << "\"" << p->second << "\"" ;

        // use single quotes instead.
        sqlbuff << "\'" << p->second << "\'" ;


        // p + 1 doesn't work on a STL map iterator? so increment then decrement
        if ((++p) != FieldValueSet.end())
        {
            p--;
            // Insert a comma between fieldvalue pairs
            sqlbuff << ", ";
        }
        else
        {
            p--;
            // Insert a space, instead of a comma after the last element
            sqlbuff << " ";
        }
    }

    // Use Filterparser to get SQL 'WHERE' clause from SearchFilter
    // then add WHERE clause to SQL statement
    PGSQLFilterparser filter(SearchFilter);
    sqlbuff << "WHERE " << filter.GetParsedresult() << ends;

//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}

// Generate an SQL statement based on the DBInsert arguments
void
PGSQLConnection::CreateSQLinsert(char* Sqlcmd, const string& Domain,
     const FieldValuetype& FieldValueSet)
{
    // Use a strstream to create SQL command
    ostrstream sqlbuff(Sqlcmd, MAXQUERYLENGTH);

    // Field List iterator to used to fill fields in SQL statement
    FieldValuetype::const_iterator p;

    // All supported SQL INSERT statements begin with INSERT INTO Domain
    sqlbuff << "INSERT INTO " << Domain ;

    // Add Field names
    sqlbuff << " ( " ;
    for (p = FieldValueSet.begin(); p != FieldValueSet.end(); p++)
    {
        sqlbuff << p->first;

        // p + 1 doesn't work on a map iterator? so increment then decrement
        if ((++p) != FieldValueSet.end())
        {
            p--;
            // Insert a comma between fieldvalue pairs
            sqlbuff << ", ";
        }
        else
        {
            p--;
            // Insert a space, instead of a comma after the last element
            sqlbuff << " ";
        }
    }
    sqlbuff << " ) " ;

    // Add Value names
    sqlbuff << "VALUES ( " ;
    for (p = FieldValueSet.begin(); p != FieldValueSet.end(); p++)
    {
        // sqlbuff << "\"" << p->second << "\"" ;
        sqlbuff << "\'" << p->second << "\'" ;

        // p + 1 doesn't work on a map iterator? so increment then decrement
        if ((++p) != FieldValueSet.end())
        {
            p--;
            // Insert a comma between fieldvalue pairs
            sqlbuff << ", ";
        }
        else
        {
            p--;
            // Insert a space, instead of a comma after the last element
            sqlbuff << " ";
        }
    }
    sqlbuff << " ) " << ends;

//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}

// Generate an SQL statement based on the DBDelete arguments
void
PGSQLConnection::CreateSQLdelete(char* Sqlcmd, const string& Domain,
     const string& SearchFilter)
{
    // Use a strstream to create SQL command
    ostrstream sqlbuff(Sqlcmd, MAXQUERYLENGTH);

    // All supported SQL DELETE statements begin with DELETE FROM Domain
    sqlbuff << "DELETE FROM " << Domain << " ";

    // Use Filterparser to get SQL 'WHERE' clause from SearchFilter
    // then add WHERE clause to SQL statement
    PGSQLFilterparser filter(SearchFilter);
    sqlbuff << "WHERE " << filter.GetParsedresult() << ends;

//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}

// Generate an SQL statement based on the DBCreate arguments
// This version of DBCreate does not take FieldDataTypeList which
// allows you to specify a different data type for each field.
// Therefore the data type for each field will be VARCHAR(255) 
// which is STRINGTYPE
void
PGSQLConnection::CreateSQLcreate(char* Sqlcmd, const string& Domain,
     const FieldListtype& FieldList, const string& PrimaryKey) 
{
    // Use a strstream to create SQL command
    ostrstream Sqlbuff(Sqlcmd, MAXQUERYLENGTH);

    // Field List iterator to used to fill fields in SQL statement
    FieldListtype::const_iterator p;

    Sqlbuff << "CREATE TABLE " << Domain << " ( " << endl;

    // Add FieldList elements to SQL statement
    for (p = FieldList.begin(); p != FieldList.end(); p++)
    {
        // If the field is the PRIMARY KEY, it must have the
        // NOT NULL option set.
        if ( *p == PrimaryKey )
            Sqlbuff << *p << " VARCHAR(255)   NOT NULL, " << endl;
        else
            Sqlbuff << *p << " VARCHAR(255), " << endl;
    }

    // Insert Primary Key
    Sqlbuff << "PRIMARY KEY ( " << PrimaryKey << " ) ) " << ends;


//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}

// Generate an SQL statement based on the DBCreate arguments
void
PGSQLConnection::CreateSQLcreate(char* Sqlcmd, const string& Domain,
     const FieldListtype& FieldList, const FieldListDatatype& DatatypeList, 
     const string& PrimaryKey)
{
    // Use a strstream to create SQL command
    ostrstream Sqlbuff(Sqlcmd, MAXQUERYLENGTH);
    string Sqltype;
    string fieldnamebuff;
    Datatype Datatypebuff;

    // Field List iterator, and DataType iterator
    FieldListtype::const_iterator p1;
    FieldListDatatype::const_iterator p2;

    Sqlbuff << "CREATE TABLE " << Domain <<  " ( " << endl;

    // Add FieldList elements to SQL statement
    for (p1 = FieldList.begin(), p2 = DatatypeList.begin();
         p1 != FieldList.end(); p1++, p2++)
    {
        fieldnamebuff = *p1;
        Datatypebuff = *p2;

        GetSQLtypestring(Sqltype, Datatypebuff);

        Sqlbuff << fieldnamebuff  << "  " << Sqltype << "  ";

        // If the field is the PRIMARY KEY, it must have the
        // NOT NULL option set.
        if ( *p1 == PrimaryKey )
            Sqlbuff << "NOT NULL, " << endl;
        else
            Sqlbuff << ", " << endl;
    }

    // Insert Primary Key
    Sqlbuff << "PRIMARY KEY ( " << PrimaryKey << " ) ) " << ends;

//    cout << endl << "SQL Command: " << Sqlcmd << endl << endl;
}
