#ifndef PGSQLCONNECTION_HXX
#define PGSQLCONNECTION_HXX

#if defined(__svr4__) || defined(__SUNPRO_CC)
    #include <libpq-fe.h>
#else
    #include <libpq-fe.h>
#endif 

#include <string>
#include "DBConnection.hxx"

/// Forward Declare PGSQLResultset
class PGSQLResultset;

/// PGSQLConnection class
class PGSQLConnection : public DBConnection
{
    private:
        /// Constructor- make connection to PostgreSQL 
        PGSQLConnection(const string& dbname, 
                const string& dbhost,
                const string& dbuser, 
                const string& dbpassword);

        /// Destructor- free memory associated with PGconn pointer
        virtual ~PGSQLConnection()
        { 
            PQfinish(myPgSQLConnectionP);
        };

        /// Return a resultset object based on query
        DBResultset* DBSelect(const string& Domain, 
                const string& SearchFilter, 
                const FieldListtype& FieldList, 
                const Ordertype& Order, 
                const string &OrderField);

        /// Return a resultset object based on query
        DBResultset* DBSelect(const string& Domain,
                const string& SearchFilter, 
                const FieldListtype& FieldList);

        /// Return a resultset object based on query
        DBResultset* DBSelect(const string& Domain, 
                const string& SearchFilter);

        /// Return a resultset object based on query
        DBResultset* DBSelect(const string& Domain,
                const string& SearchFilter, 
                const string& start, 
                const string& end, 
                const FieldListtype& FieldList);

        /// Return a resultset object based on query
        DBResultset* DBSelect(const string& Domain,
                const string& SearchFilter,
                const string& start,
                const string& end,
                const Ordertype& Order,
                const string &OrderField,
                const FieldListtype& FieldList);

        /// Return a string containing primary key of Domain
        string DBPrimaryKey( const string& Domain);

        /// Return a table containing fields/types/keys of Domain
        ResultTabletype& DBDescribe(const string& Domain);

        /// Return a list of domains in database
        ResultRecordtype& DBDescribe(void);

        /// Update Domain, based on searchfilter
        void DBUpdate(const string& Domain, const string& SearchFilter,
                const FieldValuetype& FieldValueSet);

        /// Update Domain, based on previous resultset
        void DBUpdate(const string& Domain, DBResultset* Resultset,
                const FieldValuetype& FieldValueSet);

        /// Insert record
        void DBInsert(const string& Domain,
                const FieldValuetype& FieldValueSet);

        /// Delete record, based on searchfilter
        void DBDelete(const string& Domain, 
                const string& SearchFilter);

        /// Delete record, based on previous resultset
        void DBDelete(const string& Domain, 
                DBResultset* Resultset);
 
        /// Create a new domain, use STRINGTYPE for all columns
        /// with or without checking current existence
        void DBCreate(const string& Domain, 
                const FieldListtype& FieldsList, 
                const string& PrimaryKey, 
                bool ExistenceCheck);

        /// Create a new domain, specify types for each column
        /// with or without checking current existence
        void DBCreate(const string& Domain, 
                const FieldListtype& FieldsList, 
                const FieldListDatatype& Datatype, 
                const string& PrimaryKey, 
                bool ExistenceCheck);

        /// Alter domain, drop field
        void DBAlter(const string& Domain, 
                const string& Fieldname);

        /// Alter domain, change a type of a field
        void DBAlter(const string& Domain, 
                const string& Fieldname, 
                const Datatype Newtype);

        /// Drop a Domain, with or without checking current existence
        void DBDrop(const string& Domain, 
                bool ExistenceCheck);

        /// Returns true if Domain exists, falose otherwise
        bool DBVerify(const string& Domain);

        /// Create the SQL for the respective query
        void CreateSQLselect(char *Sqlcmd, 
                const string& Domain,
                const string& SearchFilter, 
                const FieldListtype& FieldList,
                const Ordertype& Order, 
                const string& OrderField);

        /// Create the SQL for the respective query
        void CreateSQLselect(char *Sqlcmd, 
                const string& Domain, 
                const string& SearchFilter, 
                const FieldListtype& FieldList);

        /// Create the SQL for the respective query
        void CreateSQLselect(char *Sqlcmd, 
                const string& Domain,
                const string& SearchFilter, 
                const string& start, 
                const string& end,
                const FieldListtype& FieldList);

        /// Create the SQL for the respective query
        void CreateSQLselect( char* Sqlcmd,
                const string& Domain,
                const string& SearchFilter, 
                const string& start,
                const string& end,
                const Ordertype& Order, 
                const string& OrderField,
                const FieldListtype& FieldList );

        /// Create the SQL for the respective query
        void CreateSQLselect(char *Sqlcmd, const string& Domain,
                const string& SearchFilter );

        /// Create the SQL for the respective query
        void CreateSQLupdate(char *Sqlcmd, const string& Domain, 
                const string& SearchFilter, 
                const FieldValuetype& FieldValueSet);

        /// Create the SQL for the respective query
        void CreateSQLinsert(char *Sqlcmd, const string& Domain, 
                const FieldValuetype& FieldValueSet);

        /// Create the SQL for the respective query
        void CreateSQLdelete(char *Sqlcmd, const string& Domain, 
                const string& SearchFilter);

        /// Create the SQL for the respective query
        void CreateSQLcreate(char *Sqlcmd, const string& Domain, 
                const FieldListtype& FieldList, 
                const string& PrimaryKey);

        /// Create the SQL for the respective query
        void CreateSQLcreate(char *Sqlcmd, const string& Domain, 
                const FieldListtype& FieldList, 
                const FieldListDatatype& DatatypeList, 
                const string& PrimaryKey);

        /// Return the PostgreSQL equivalent of DBModule Datatype
        void GetSQLtypestring(string& Sqltype, Datatype type);

        /// PostgreSQL Connection handle
        PGconn* myPgSQLConnectionP;

        /// vector<vector <string> > used by DBDescribe(Domain)
        ResultTabletype myDescribeTable;

        /// vector <string> used by DBDescribe()
        ResultRecordtype myDescribeRecord;

        /// myPgSQLResultsethandle a the pointer to the resultset
        PGSQLResultset* myPGSQLResultsetP;

    public:
        /// The only class authorized to use PGSQLConnection is 
        /// ConnectionManager
        friend class ConnectionManager;
};
#endif
