#ifndef PGSQLRESULTSET_HXX
#define PGSQLRESULTSET_HXX

#if defined(__svr4__) || defined(__SUNPRO_CC)
    #include <libpq-fe.h>
#else
    #include <libpq-fe.h>
#endif

#include <string>
#include <vector>
#include "DBResultset.hxx"

///
class PGSQLResultset : public DBResultset
{
    private:
        /// Constructor-
        PGSQLResultset(PGresult* result, const string& searchfilter);

        /// Destructor-
        virtual ~PGSQLResultset();

        /// Return all records from a query
        virtual const ResultTabletype& GetRecords(void) const;

        /// Return records between offset1, and offset2
        virtual const ResultTabletype& GetRecords(
                unsigned short offset1, 
                unsigned short offset2) const;

        /// Return the record at [offset]
        virtual const ResultRecordtype& operator[](
                unsigned short offset) const;

        /// Return the record at offset
        virtual const ResultRecordtype& GetRecord(
                unsigned short offset) const;

        /// Return field list from query
        virtual const FieldListtype& GetFields(void) const;

        /// Return number of records in query
        virtual const unsigned int GetRecordCount(void) const;

        /// Return serach filter from query
        virtual const string& GetSearchFilter(void) const;

        /// PgSQL API Result 
        PGresult* myPgSQLresultP;
       
        /// Store the search filter in case the resultset object is
        /// used as input to DBUpdate() or DBDelete()
        const string mySearchFilter;

        /// STL vector of strings representing fieldlist
        FieldListtype myFieldList;

        /// Number of records in resultset
        int myRecordCount;

    public:
        /// The only class that needs access
        friend class PGSQLConnection;
};

#endif
