#include "ParsedDef.hxx"
#include <algo.h>
#include <stdio.h>

const DifferenceSet ParsedDef::compareOld( const ParsedDef &oldDef ) {

    DifferenceSet returnVal;
    FieldListtype::const_iterator walker, result;

    // If the primary keys don't agree, don't even bother checking anything
    // else, since we have to wipe the old version of the database
    if (myPrimaryKey != oldDef.getPrimaryKey()) {
        returnVal.setChangedPrimary();
        return returnVal;
    }
        
    // Walk the new list and see if the entries are in the old list. If they
    // are not, then these are new entries -- add them to the newList
    for (walker = myFieldList.begin(); walker != myFieldList.end();
         walker++) {

        result = find((oldDef.myFieldList).begin(),
                      (oldDef.myFieldList).end(),
                      *walker);

        if (result == (oldDef.myFieldList).end())
            returnVal.addNewField(*walker);

    }

    // Next walk the old one and look for any that don't seem to be in the
    // new list. We will add these to the delete list.
    for (walker = (oldDef.myFieldList).begin();
         walker != (oldDef.myFieldList).end();
         walker++) {
        
        result = find(myFieldList.begin(), myFieldList.end(),
                      *walker);
        
        if (result == myFieldList.end())
            returnVal.addDelField(*walker);
        
    }

    return returnVal;

}


void ParsedDef::addField( const string newField )
    throw (VBadDataException &) {

    FieldListtype::iterator result;

    result = find(myFieldList.begin(), myFieldList.end(), newField);
    
    if (result != myFieldList.end()) {
        char buf[256];
        sprintf (buf, "Duplicate field name %s encountered!",
                 newField.c_str());
        throw VBadDataException( buf, __FILE__, __LINE__, 0 );
    }

    myFieldList.push_back(newField); 

}

void ParsedDef::addFieldType(const Datatype newType) 
    throw (VBadDataException &)
{
  if (myFieldList.size() != (myFieldTypes.size()+1))
    {
      throw VBadDataException("Number of fields does not \
          match number of types", __FILE__, __LINE__, 0);
    }

  myFieldTypes.push_back(newType);
}
