package org.vovida.prov.authentication;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.TextOutputCallback;
import javax.security.auth.callback.UnsupportedCallbackException;

import java.io.IOException;
import javax.security.auth.login.LoginException;






/**
 * Prompts the user to input his user name and password. <p>
 * Uses a helper class to display the actual gui input fields which will
 * gather the requested input. This class just determines what type of input
 * has been requested (name, password) and prompts the gui panel to generate
 * those widgets. Once the user has submitted the gui panel, this class
 * gathers the input string from the panel and inserts them into the
 * Callback objects. <p>
 */
public class AuthInput implements CallbackHandler
{
    public void handle(Callback[] callbacks) throws IOException,
        UnsupportedCallbackException
    {
        AuthInputScreen screen = new AuthInputScreen();

        for (int i=0; i<callbacks.length; i++)
        {
            if (TextOutputCallback.class.isInstance(callbacks[i]))
            {
                //this should be the banner identifying the login module
                TextOutputCallback callback = (TextOutputCallback)callbacks[i];
                screen.addBannerOutput(callback.getMessage());
            }
            else if (NameCallback.class.isInstance(callbacks[i]))
            {
                //this should be the request for the account identifier
                NameCallback callback = (NameCallback)callbacks[i];
                screen.addTextInputField(callback.getPrompt(),callback.getDefaultName(),callback);
            }
            else if (PasswordCallback.class.isInstance(callbacks[i]))
            {
                //this should be the request for the password
                PasswordCallback callback = (PasswordCallback)callbacks[i];
                screen.addPasswordInputField(callback.getPrompt(),callback);
            }
            else
            {
                throw new UnsupportedCallbackException(callbacks[i], "Unrecognized callback");
            }
        }

        screen.pack();
        screen.setVisible(true); //modal

        if (screen.getLastButton() == screen.OK_BUTTON)
        {
            for (int i=0; i<callbacks.length; i++)
            {
                String value = screen.getValue(callbacks[i]);
                if (value != null)
                {
                    if (NameCallback.class.isInstance(callbacks[i]))
                    {
                        NameCallback name = (NameCallback)callbacks[i];
                        name.setName(value);
                    }
                    else if (PasswordCallback.class.isInstance(callbacks[i]))
                    {
                        PasswordCallback password = (PasswordCallback)callbacks[i];
                        password.setPassword(value.toCharArray());
                    }
                }
            }
        }
        else //cancel button
        {
			throw new LoginCancelledException();
        }

        screen.dispose();
    }
}
