package org.vovida.prov.authentication;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.border.EmptyBorder;

import java.util.HashMap;




/**
 * Contains the actual gui widgets which reqest
 */
public class AuthInputScreen extends JDialog
{
    /**
     * Keeps track of the mapping of the textInput field used to request input
     * from the user and the key object which will be used to retrieve the
     * requested data by the caller.
     */
    private HashMap inputFields;

    /**
     * Will contain the actual input fields and their labels.
     */
    private JPanel inputPanel;

    private int lastButton = -1;
    public static final int OK_BUTTON = 1;
    public static final int CANCEL_BUTTON = 0;

    private JPanel contentPanel;

    public AuthInputScreen()
    {
        setTitle("Enter password");

        contentPanel = new JPanel();
        contentPanel.setLayout(new BorderLayout());
        contentPanel.setBorder(new EmptyBorder(3,3,3,3));
        getContentPane().add(contentPanel);

		inputFields = new HashMap();
        setModal(true);

        inputPanel = new JPanel(new GridLayout(0,2));
        contentPanel.add(inputPanel,BorderLayout.CENTER);

        JPanel temp = new JPanel();
        temp.setLayout(new BoxLayout(temp,BoxLayout.X_AXIS));
        temp.setBorder(new EmptyBorder(5,0,0,0));

        JButton ok = new JButton("Ok");
        JButton cancel = new JButton ("Cancel");

        temp.add(Box.createHorizontalGlue());
        temp.add(ok);
        temp.add(cancel);

        ok.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent event)
            {
                okPressed();
            }
        });

        cancel.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent event)
            {
                lastButton = CANCEL_BUTTON;
                setVisible(false);
            }
        });

        contentPanel.add(temp,BorderLayout.SOUTH);
    }

    /**
     * Return the value of the flag which indicates which button was pressed
     * when the dialog was hidden. <p>
     * This may be the OK or CANCEL.<p>
     */
    public int getLastButton()
    {
        return lastButton;
    }

    /**
     * Adds a label at the top of the dialog which displays the given text.
     */
    public void addBannerOutput(String banner)
    {
        JLabel bannerLabel = new JLabel(banner);
        bannerLabel.setBorder(new EmptyBorder(0,0,5,0)); //some space below the banner
        contentPanel.add(bannerLabel, BorderLayout.NORTH);
    }

    /**
     * Center the dialog on the screen before calling the superclass setVisible
     */
    public void setVisible(boolean visible)
    {
        Dimension dimension = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((dimension.width - this.getSize().width)/2,
            (dimension.height - this.getSize().height)/2);
        super.setVisible(visible);
    }

    /**
     * Add a text input field. <p>
     * This would be the field where the user enters the account id. <p>
     */
    public void addTextInputField(String label, String defaultText, Object key)
    {
        JLabel labelField = new JLabel(label);
        JTextField textField;
        if (defaultText != null)
        {
            textField = new JTextField(defaultText);
            textField.select(0,defaultText.length());
        }
        else
        {
            textField = new JTextField();
        }
        inputPanel.add(labelField);
        inputPanel.add(textField);

        inputFields.put(key,textField);
    }

    /**
     * Hide the dialog remember that the last button pressed was ok (not cancel).<p>
     * This is called form both the action listener for the ok button and for
     * the password field.<p>
     */
    private void okPressed()
    {
        lastButton = OK_BUTTON;
        setVisible(false);
    }

    /**
     * Adds a password input field. <p>
     * The field is a JPassword field (instead of a JTextField) and has an
     * associated action listener which does the same thing as pressing the
     * "ok" button.<p>
     */
    public void addPasswordInputField(String label, Object key)
    {
        JLabel labelField = new JLabel(label);
        JPasswordField password = new JPasswordField();
        inputPanel.add(labelField);
        inputPanel.add(password);

        //when the user presses enter in the password field, hide the dialog
        //and mark the last button just as if he had pressed ok
        password.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent event)
            {
                okPressed();
            }
        });

        inputFields.put(key,password);
    }

    /**
     * Return the text value which is retrieved from the input field which was
     * added with the given key. <p>
     * If a field for the given key is not found, returns null.
     */
    public String getValue(Object key)
    {
        JTextField field = (JTextField)inputFields.get(key);
        if (field != null)
        {
	        return field.getText();
        }
        return null;
    }
}

