package org.vovida.prov.authentication;

import org.vovida.prov.psLib.ProvisioningInterface;


public class LoginModuleConstants
{
    /**
     * Gives the host name of the provisioning server to contact for
     * authentication.
     **/
	public static final String AUTH_SERVER = "auth_server";

    /**
     * Gives the port to contact the authenticating provisioning server on
     */
    public static final String AUTH_PORT = "auth_port";

    /**
     * Flag which indicates whether to try using an existing connection to the
     * provisioning server (and initialize if it does not exist) or whether to
     * re-initialize the connection by default
     */
    public static final String REUSE_CONNECTION = "reuse_conn";

    /**
     * Flag which indicates the class name of the provisioning interface to use
     * by the login module.<p>
     * This is usefull for debugging/testing so that the responses from the
     * provisioning server may be simulated via a provisioning interface stub
     * class.
     */
    public static final String PROVISIONING_INTERFACE = "prov_int";


    /**
     * Flag which indicates whether to to supply a TextOutputCallback to the
     * CallbackHander which will describe this login module.
     */
    public static final String USE_MODULE_BANNER = "module_banner";

    public static final String DEFAULT_BANNER_TEXT = "Provisioning Server Login Module v 0.1";
    public static final String DEFAULT_USERID_PROMPT = "Account id:";
    public static final String DEFAULT_USERID = "vovida";
    public static final String DEFAULT_PASSWORD_PROMPT = "Password:";


    //Default values for the above flags
    public static final String DEFAULT_AUTH_SERVER = "localhost";
    public static final int DEFAULT_AUTH_PORT = 6005;
    public static final String DEFAULT_REUSE_CONNECTION = "true";
    public static final String DEFAULT_USE_MODULE_BANNER = "true";

    /**
     * This should ONLY be set if the login module is being debugged.<p>
     * Set this to an instance of the provisioning interface stub to be used
     * for testing. Otherwise, the PServerLoginModule should use a
     * SocketProvisioningInterface as specified by its options.
     */
    public static ProvisioningInterface provisioningInterface = null;
}
