package org.vovida.prov.authentication.test;

import java.io.IOException;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;

import org.vovida.prov.authentication.AuthConstants;
import org.vovida.prov.authentication.AuthTopLevel;
import org.vovida.prov.authentication.LoginInitFailedException;
import org.vovida.prov.authentication.LoginModuleConstants;

import org.vovida.prov.psLib.AuthenticationException;



public class LoginFailed extends PSTest 
{
    /**
     * Keep track of the number of times the login module requests and account
     * id and password from the callback handler.
     */
    private static int retryCount = 0; //static because login module instantiates its own callbackHandler and want to share that result with this instance

    /**
     * Keep track of whether the main class of the target application was
     * executed or not.
     */
    private static boolean appExecuted = false;

	protected int startTest() throws Throwable
    {
		//setup the fake ProvisioningInterface
        getProvisioningInterface().setNextResult(getProvisioningInterface().RESULT_FAILURE);
        getProvisioningInterface().setNextError(new AuthenticationException("Simulated failure"));

        AuthTopLevel.main(new String [] {AuthTopLevel.OPT_LOGIN_CALLBACK_HANDLER,
            this.getClass().getName()});

        //make sure the correct number of login retries occured.
        if (retryCount != AuthConstants.NUM_LOGIN_RETRIES)
        {
            lastMessage = "Wrong number of retries. Expected "
                + AuthConstants.NUM_LOGIN_RETRIES + " but got " + retryCount;
            return FAILURE;
        }
        if (appExecuted)
        {
            lastMessage = "The application was executed even though all logins failed";
            return FAILURE;
        }

        LoginModuleConstants.provisioningInterface = null;
        retryCount = 0;
        lastMessage = "Application did not execute after "
            + AuthConstants.NUM_LOGIN_RETRIES + " login retries. ";
        return SUCCESS;
    }


    public String getTestName()
    {
        return "Test failed login and login retry";
    }

    public String getTestDescription()
    {
        return "Use a ProvisioningInterface stub to simulate a failed login. "
            + "Make sure that the AuthTopLevel allows the login to be retried "
            + "AuthConstants.NUM_LOGIN_RETRIES times. "
            + "Make sure the application does not get executed if all "
            + "login retries fail";
    }

    public void handle(Callback[] callbacks) throws IOException,
        UnsupportedCallbackException
    {
        for (int i=0; i<callbacks.length; i++)
        {
            if (NameCallback.class.isInstance(callbacks[i]))
            {
                 retryCount ++; //increment the number of requests for account id (retries)
                 NameCallback name = (NameCallback)callbacks[i];
                 name.setName("this login");
            }
            else if (PasswordCallback.class.isInstance(callbacks[i]))
            {
                 PasswordCallback password = (PasswordCallback)callbacks[i];
                 password.setPassword("should fail".toCharArray());
            }
            else
            {
                //just quietly ignore any unknown callbacks
            }
        }
    }

    public static void main(String [] args)
    {
       appExecuted = true;
    }
}
