/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package org.vovida.prov.content;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.ListIterator;

import org.jdom.Element;

import org.apache.log4j.Category;

import org.vovida.prov.dynamicGui.DTDConstants;
import org.vovida.prov.dynamicGui.ProvisionableInstance;
import org.vovida.prov.widget.GuiFactory;
import org.vovida.prov.widget.GuiComponent;
import org.vovida.prov.widget.ListGuiComponent;
import org.vovida.prov.widget.HostPortField;
import org.vovida.prov.widget.Table;

/**
 * @author Barbara Samson
 */
public class ContentFactory implements DTDConstants
{
  static Category logCategory = 
    Category.getInstance(ContentFactory.class.getName());

  public static ContentData newContentData(String name, GuiComponent widget, 
          ProvisionableInstance pInstance, Element contentElement)
  {
    Element typeElement = null;
    ContentData content = null;

    // default is StringType
    String typeName = STRINGTYPE;

    if (contentElement != null)
    {
      List contentChildren = contentElement.getChildren();

      if (!contentChildren.isEmpty())
      {
        typeElement = (Element) contentChildren.get(0);
        typeName = typeElement.getName();
      }
    }

    try
    {
      Class contentClass = null;

      // First try to get the class from the content package.
      // If that doesn't work, try getting the class as given.
      try
      {
        contentClass = Class.forName("org.vovida.prov.content." + typeName);
        typeName = "org.vovida.prov.content." + typeName;
      }
      catch (ClassNotFoundException e)
      {
        contentClass = Class.forName(typeName);
      }

      Constructor contentConstructor = contentClass.getConstructor(new Class[]
      {
        String.class, GuiComponent.class, ProvisionableInstance.class, 
        Element.class
      });

      content = (ContentData) contentConstructor.newInstance(new Object[]
      {
        name, widget, pInstance, typeElement
      });

      // If the widget is a HostPortField, then the
      // content needs to be a HostPortType.
      // If some other content was specified, just throw it away.
      if (widget instanceof HostPortField)
      {
        if (!(content instanceof HostPortType))
        {
          content = new HostPortType(name, widget, pInstance, null);
        }
      }

      // If the widget is a List Component of some sort (ie ComboBox or
      // Radio Button), then the content needs to be a ListType.
      if (widget instanceof ListGuiComponent)
      {
        if (!(content instanceof ListType))
        {
          content = new ListType(content, pInstance, typeElement);
        }
      }
    }
    catch (ClassCastException e1)
    {
      logCategory.error("ClassCastException for:\n" + name + " (" + typeName 
                        + ")" + " is not of type ContentData");
    }
    catch (ClassNotFoundException e2)
    {
      logCategory.error("ClassNotFoundException for:\n" + name + " (" 
                        + typeName + ")" + " cannot be found");
    }
    catch (InstantiationException e3)
    {
      logCategory.error("InstantiationException for:\n" + name + " (" 
                        + typeName + ")" + " cannot be instantiated");
    }
    catch (IllegalAccessException e4)
    {
      logCategory.error("IllegalAccessException for:\n" + name + " (" 
                        + typeName + ")\n" + e4.getMessage());
    }
    catch (NoSuchMethodException e5)
    {
      logCategory.error("NoSuchMethodException for:\n" + name + " (" 
                        + typeName + ")\n" + e5.getMessage());
    }
    catch (InvocationTargetException e6)
    {
      logCategory.error("InvocationTargetException for:\n" + name + " (" 
                        + typeName + ")\n" + e6.getMessage());
      e6.printStackTrace();
    }
    catch (IllegalArgumentException e7)
    {
      logCategory.error("IllegalArgumentException for:\n" + name + " (" 
                        + typeName + ")\n" + e7.getMessage());
    }

    if (content == null)
    {
      logCategory.error("No content created");

      return null;
    }

    return content;
  }

}

