/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.content;

import org.vovida.prov.dynamicGui.ProvisionableInstance;
import org.vovida.prov.widget.GuiComponent;
import org.vovida.prov.widget.TextField;
import org.vovida.prov.widget.HostPortField;
import org.vovida.prov.widget.ComboBox;
import org.vovida.prov.widget.GuiFactory;
import org.vovida.prov.psLib.ProvisioningInterface;
import org.vovida.prov.psLib.OperationException;
import org.vovida.prov.util.XMLProcessor;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JFrame;
import javax.swing.text.PlainDocument;
import javax.swing.text.Document;
import javax.swing.text.BadLocationException;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JOptionPane;
import java.io.IOException;
//import java.io.Reader;
//import java.io.StringReader;

import org.xml.sax.helpers.DefaultHandler;
//import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.Attributes;
//import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import org.jdom.Element;

import org.apache.log4j.Category;

/**
 * @author Barbara Samson
 */
public class HostType extends ListType
{

  static Category logCategory = 
    Category.getInstance(HostType.class.getName());


  /**
   * Class constructor.
   * @param typeElement could be null
   */
  public HostType(String name, GuiComponent gui, 
                  ProvisionableInstance instance, Element typeElement)
  {
    super(name, gui, instance, typeElement);

    setUserInputOK(true);

    // get the list of host names from file.
    ProvisioningInterface ps = ProvisioningInterface.getInstance();
    String xmlString = null;

    try
    {
      xmlString = ps.getXMLFile("config", "Hosts.xml");
    }
    catch (OperationException e1)
    {

      // if the Hosts file is not present, log a warning message.
      logCategory.warn("Hosts file not available: " + e1.getMessage());

      return;
    }
    catch (IOException e2)
    {
      JOptionPane.showMessageDialog(new JFrame(), 
              "error getting Hosts file: " + e2.getMessage(), "IOException", 
              JOptionPane.ERROR_MESSAGE);

      return;
    }

    try
    {
      XMLProcessor.doSAXParse(xmlString, new HandleHostsParsing());
    }
    catch (SAXException e)
    {
      JOptionPane.showMessageDialog(new JFrame(), 
              "badly formed Hosts file: " + e.getMessage(), "SAXException", 
              JOptionPane.ERROR_MESSAGE);
      return;
    }


    /*
    Reader reader = new StringReader(xmlString);
    InputSource source = new InputSource(reader);

    try
    {
      SAXParser parser = new SAXParser();

      parser.setContentHandler(new HandleHostsParsing());

      parser.parse(source);
    }
    catch (SAXException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    */

  }


  /**
   * Handles the callbacks for the SAX parsing of the Hosts file.
   */
  class HandleHostsParsing extends DefaultHandler
  {
    private boolean processingHost = false;

    public void startElement(String ignore1, String ignore2, String rawName, 
            Attributes attrs)
    {
      if (rawName.equals("host"))
      {
        processingHost = true;
      }
    }

    public void characters(char[] ch, int start, int length)
    {
      if (processingHost)
      {
        String value = new String(ch, start, length);

        addEnumeration(value);
        logCategory.info("added Host: " + value);

        processingHost = false;
      }
    }

  }

}




