/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package org.vovida.prov.content;

import org.vovida.prov.dynamicGui.DTDConstants;
import org.vovida.prov.dynamicGui.ProvisionableInstance;
import org.vovida.prov.widget.GuiComponent;
import org.vovida.prov.widget.ComboBox;
import org.vovida.prov.widget.RadioButtons;
import org.vovida.prov.widget.ListListenerAdapter;
import org.vovida.prov.psLib.ProvisioningInterface;
import org.vovida.prov.psLib.PSDBResult;

import javax.swing.JFrame;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.List;
import java.util.ListIterator;

import org.jdom.Element;

import org.apache.log4j.Category;

/**
 * @author Barbara Samson
 */
public class ListType extends org.vovida.prov.content.ContentData 
  implements DTDConstants
{

  // private List myList = new Vector();
  private Object mySelectedObject = null;

  private ListListenerAdapter myListAdapter;

  /**
   * true if the user can input a value that is not in the list
   */
  private boolean myUserInputOK = false;

  /**
   * Used for type validation. This could be null.
   */
  ContentData myContentData;

  /**
   * for logging
   */
  private static Category logCategory = 
    Category.getInstance(ListType.class.getName());

  /**
   * Class constructor.
   * @param typeElement could be null.
   */
  public ListType(ContentData content, ProvisionableInstance instance, 
                  Element typeElement)
  {
    super(content.getName(), instance, typeElement);

    setGuiComponent(content.getGuiComponent());

    myContentData = content;

    setListAdapter(new ListListenerAdapter(this));

    // If there was a typeElement, check it for further restrictions
    if (typeElement != null)
    {
      List typeChildren = typeElement.getChildren();

      if (!typeChildren.isEmpty())
      {
        Element restrictionElement = (Element) typeChildren.get(0);
        String restrictionName = restrictionElement.getName();

        if (restrictionName.equals(ENUMERATION))
        {
          setUpEnumeration(restrictionElement);
        }
        else if (restrictionName.equals(PSERVER))
        {
          setUpPserverQuery(restrictionElement);
        }
      }
    }
  }

  protected ListType(String name, GuiComponent gui, 
                     ProvisionableInstance instance, Element typeElement)
  {
    super(name, instance, typeElement);

    setGuiComponent(gui);
    setListAdapter(new ListListenerAdapter(this));
  }

  /**
   * Cloning allows new table rows to be created.
   */
  public Object clone() throws CloneNotSupportedException
  {
    ListType clone = (ListType) super.clone();

    clone.setListAdapter(new ListListenerAdapter(clone));
    clone.setDefault(getDefault());

    return clone;
  }

  protected void setListAdapter(ListListenerAdapter adapter)
  {
    myListAdapter = adapter;

    GuiComponent gui = getGuiComponent();

    myListAdapter.setUpListener(gui);
  }

  /**
   * Fill in content from PServer query.
   */
  private void setUpPserverQuery(Element pserverElement)
  {
    logCategory.info("found a PServer query");

    Element queryElement = null;
    ProvisioningInterface ps = ProvisioningInterface.getInstance();

    queryElement = pserverElement.getChild(GETRECORDSINDOMAIN);

    if (queryElement != null)
    {
      logCategory.info("query is GetRecordsInDomain");

      PSDBResult result = callGetRecordsInDomain(queryElement);

      if (result == null)
      {
        logCategory.warn("null result returned from getRecordsInDomain query");

        return;
      }

      Hashtable[] rows = result.getRows();

      for (int i = 0; i < rows.length; i++)
      {
        Enumeration enum = rows[i].elements();

        if (enum.hasMoreElements())
        {
          addEnumeration((String) enum.nextElement());
        }
      }
    }
    else
    {
      queryElement = pserverElement.getChild(GETDOMAINS);

      if (queryElement != null)
      {
        logCategory.info("query is GetDomains");

        String[] domains = callGetDomains(queryElement);

        if ((domains == null) || (domains.length == 0))
        {
          logCategory.warn("null result returned from getDomains query");

          return;
        }

        for (int i = 0; i < domains.length; i++)
        {
          addEnumeration(domains[i]);
        }
      }
    }
  }

  protected void setUpEnumeration(Element enumElement)
  {
    List enumList = enumElement.getChildren(ENTRY);
    ListIterator enumIter = enumList.listIterator();

    while (enumIter.hasNext())
    {
      Element entry = (Element) enumIter.next();

      addEnumeration(entry.getTextTrim());
    }
  }

  void addEnumeration(String entry)
  {
    myListAdapter.addChoice(entry);
  }

  protected String getHint()
  {
    GuiComponent gui = getGuiComponent();

    if (gui instanceof ComboBox)
    {
      return "select a value from the drop-down list";
    }
    else if (gui instanceof RadioButtons)
    {
      return "select a radio button";
    }
    else
    {
      return "select one of the values given";
    }
  }

  public String getContentAsString()
  {
    if (mySelectedObject == null)
    {
      return "";
    }

    return (String) mySelectedObject;
  }

  public void validate(String value) throws InvalidDataException
  {
    if (myContentData != null)
    {
      myContentData.validate(value);
    }
    else
    {
      checkIfRequired(value);
      checkAlert(value);
    }
  }

  public void setUserInputOK(boolean inputOK)
  {
    super.setUserInputOK(inputOK);

    myUserInputOK = inputOK;
  }

  public void setContent(String newContent)
  {
    myListAdapter.setSelection(newContent);

    mySelectedObject = newContent;

    super.setContent(newContent);
  }

  public void selectionChanged(String newSelection)
  {
    mySelectedObject = newSelection;
  }

}


