/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.content;

import org.vovida.prov.widget.GuiException;
import org.vovida.prov.widget.GuiComponent;
import org.vovida.prov.widget.TextField;
import org.vovida.prov.widget.Table;

import java.util.Vector;
import java.util.Enumeration;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;
import javax.swing.CellEditor;
import javax.swing.DefaultCellEditor;
import javax.swing.AbstractCellEditor;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import java.awt.Component;

/**
 * @author Barbara Samson
 */
public class MyTableModel extends AbstractTableModel
{
  private CellEditor myCellEditor = new CellEditor();
  private CellRenderer myCellRenderer = new CellRenderer();

  // a vector of arrays of ContentDatas
  private Vector myData = new Vector();

  private Vector myColumns = new Vector();

  // these get cloned to create a new row
  private Vector myColumnContents = new Vector();

  private int myNumColumns = 0;

  public void addColumn(String name, String label, ContentData content, 
          int position, JTable table)
  {
    TableColumn column = new TableColumn(position);

    if (label != null)
    {
      column.setHeaderValue(label);
    }
    else
    {
      column.setHeaderValue(name);
    }

    column.setCellEditor(myCellEditor);
    column.setCellRenderer(myCellRenderer);
    table.addColumn(column);

    myColumns.add(name);

    myNumColumns = myColumns.size();

    myColumnContents.add(content);
    fireTableStructureChanged();
  }


  public String getColumnName(int column)
  {
    if (column >= myNumColumns)
    {
      return null;
    }

    return (String) myColumns.elementAt(column);
  }


  public int getRowCount()
  {
    return myData.size();
  }

  public int getColumnCount()
  {
    return myNumColumns;
  }

  public ContentData getContentAt(int row, int column)
  {
    ContentData[] rowValues = (ContentData[]) myData.elementAt(row);

    return rowValues[column];
  }

  public void setData(Vector data)
  {
    myData = data;

    fireTableDataChanged();
  }

  public Object getValueAt(int row, int column)
  {
    ContentData content = getContentAt(row, column);

    if (content == null)
    {
      return "";
    }

    return content.getContentAsString();
  }

  public void setValueAt(Object aValue, int rowIndex, int columnIndex)
  {
    if (rowIndex >= myData.size())
    {
      return;
    }

    ContentData[] rowValues = (ContentData[]) myData.elementAt(rowIndex);
    ContentData content = rowValues[columnIndex];

    content.setContent(aValue.toString());
  }

  public boolean isCellEditable(int rowIndex, int columnIndex)
  {
    return true;
  }

  public ContentData[] createNewEntry()
  {
    ContentData[] newRow = new ContentData[myNumColumns];

    try
    {
      for (int i = 0; i < myColumnContents.size(); i++)
      {
        newRow[i] = 
          (ContentData) ((ContentData) myColumnContents.elementAt(i)).clone();
      }
    }
    catch (CloneNotSupportedException e)
    {
      e.printStackTrace();

      return null;
    }

    return newRow;
  }

  public void addNewEntry(int selection)
  {
    ContentData[] newRow = createNewEntry();

    if (selection < 0)
    {
      myData.add(newRow);
    }
    else
    {
      myData.add(selection + 1, newRow);
    }

    int rowInsertedAt = myData.size();

    fireTableRowsInserted(rowInsertedAt, rowInsertedAt);
    fireTableDataChanged();
  }

  public void deleteRow(int row)
  {
    myData.remove(row);
    fireTableRowsDeleted(row, row);
  }

  public boolean tryToMoveUp(int selectedRow)
  {
    if (selectedRow > 0)
    {
      Object row = myData.remove(selectedRow);

      myData.insertElementAt(row, selectedRow - 1);
      fireTableRowsUpdated(selectedRow - 1, selectedRow);

      return true;
    }

    return false;
  }

  public boolean tryToMoveDown(int selectedRow)
  {
    if ((selectedRow > -1) && (selectedRow < myData.size() - 1))
    {
      Object row = myData.remove(selectedRow);

      myData.insertElementAt(row, selectedRow + 1);
      fireTableRowsUpdated(selectedRow, selectedRow + 1);

      return true;
    }

    return false;
  }

  public Enumeration getRows()
  {
    return myData.elements();
  }

  public String getSnapshot()
  {
    StringBuffer buf = new StringBuffer("snapshot\n");
    Enumeration enum = getRows();

    while (enum.hasMoreElements())
    {
      ContentData[] row = (ContentData[]) enum.nextElement();

      for (int i = 0; i < row.length; i++)
      {
        buf.append(row[i].getContentAsString() + "\t");
      }

      buf.append("\n");
    }

    return new String(buf);
  }

  class CellEditor extends AbstractCellEditor implements TableCellEditor
  {
    private ContentData myContent = null;
    private int myRow = -1;
    private int myColumn = -1;

    public Component getTableCellEditorComponent(JTable table, Object value, 
            boolean isSelected, int row, int column)
    {
      ContentData content = getContentAt(row, column);

      myContent = content;
      myRow = row;
      myColumn = column;

      Component returnVal = content.getGuiComponent().getEditorComponent();

      return returnVal;
    }

    public Object getCellEditorValue()
    {
      if (myContent == null)
      {
        return new String();
      }

      String retVal = myContent.getContentAsString();

      return retVal;
    }

  }


  class CellRenderer implements TableCellRenderer
  {
    public Component getTableCellRendererComponent(JTable table, 
            Object value, boolean isSelected, boolean hasFocus, int row, 
            int column)
    {
      ContentData content = getContentAt(row, column);
      String currentVal = content.getContentAsString();
      Component returnVal = 
        content.getGuiComponent().getTableCellRendererComponent(currentVal);

      returnVal.setEnabled(table.isEnabled());

      return returnVal;
    }

  }


}




