/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.content;

import java.util.HashSet;
import java.util.Iterator;
import java.awt.Toolkit;
import javax.swing.text.PlainDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;

/**
 * @author Barbara Samson
 */
public class NumberDocument extends PlainDocument
{
  private HashSet allowedCharacters = new HashSet();

  public void insertString(int offs, String str, 
          AttributeSet a) throws BadLocationException
  {
    char[] source = (str == null ? new char[0] : str.toCharArray());
    char[] result = new char[source.length];
    int j = 0;

    for (int i = 0; i < result.length; i++)
    {
      if (isAllowed(source[i]))
      {
        result[j++] = source[i];
      }
      else
      {
        Toolkit.getDefaultToolkit().beep();
      }
    }

    super.insertString(offs, new String(result, 0, j), a);
  }

  protected void addAllowedCharacter(Character toAdd)
  {
    allowedCharacters.add(toAdd);
  }

  protected void removeAllowedCharacter(Character toRemove)
  {
    allowedCharacters.remove(toRemove);
  }

  private boolean isAllowed(char charToCheck)
  {
    if (Character.isDigit(charToCheck))
    {
      return true;
    }

    Character character = new Character(charToCheck);
    Iterator iter = allowedCharacters.iterator();

    while (iter.hasNext())
    {
      Character iterChar = (Character) iter.next();

      if (character.equals(iterChar))
      {
        return true;
      }
    }

    return false;
  }

}




