/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.content.condition;

import org.vovida.prov.content.ContentData;
import org.vovida.prov.dynamicGui.ProvisionableInstance;

import java.util.Vector;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * @author Barbara Samson
 */
public class Condition implements ActionListener
{
  private String myValue;

  /**
   * @link aggregationByValue
   * @clientCardinality 1
   * @supplierCardinality 1..*
   */
  private Action[] myActions;

  /**
   * @link aggregation
   * @clientCardinality 1..*
   * @supplierCardinality 1
   * @clientRole checkCondition
   */
  private ContentData myTarget;

  public Condition(ContentData target, String value)
  {
    myTarget = target;
    myValue = value;

    // adds this as ActionListener to the GuiComponent
    // of the target ContentData
    myTarget.addActionListener(this);
  }

  public void initialize(ProvisionableInstance instance)
  {
    if (myActions != null)
    {
      for (int i = 0; i < myActions.length; i++)
      {
        String targetString = myActions[i].getTargetString();

        if ((targetString != null) && (!targetString.equals("")))
        {
          ContentData actionContent = instance.getContentByName(targetString);

          myActions[i].setTarget(actionContent);
        }
      }

      actionPerformed(null);
    }
  }

  public void actionPerformed(ActionEvent e)
  {
    String targetValue = myTarget.getContentAsString();

    if (targetValue.equals(myValue))
    {
      for (int i = 0; i < myActions.length; i++)
      {
        myActions[i].performAction();
      }
    }
  }

  public void addAction(Action action)
  {

    // in most cases a Condition will only have one Action,
    // making this not as expensive as it looks.
    if (myActions == null)
    {
      myActions = new Action[]
      {
        action
      };

      return;
    }

    Vector vector = new Vector(myActions.length + 1);

    for (int i = 0; i < myActions.length; i++)
    {
      vector.addElement(myActions[i]);
    }

    vector.addElement(action);

    Action[] newArray = new Action[vector.size()];

    myActions = (Action[]) vector.toArray(newArray);
  }

}




