/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.content.condition;

import java.util.List;
import java.util.ListIterator;

import org.jdom.Element;

import org.apache.log4j.Category;

import org.vovida.prov.content.ContentData;

/**
 * @author Barbara Samson
 */
public class ConditionFactory
{
  static Category logCategory = 
    Category.getInstance(ConditionFactory.class.getName());

  /**
   * Parse a conditionElement branch of the JDOM to create a new
   * Condition.
   */
  public static Condition newCondition(ContentData content, 
          Element conditionElement)
  {

    String value = conditionElement.getAttributeValue("value");
    Condition condition = new Condition(content, value);
    List actions = conditionElement.getChildren("Action");

    logCategory.info("Condition " + value);

    ListIterator actIter = actions.listIterator();

    while (actIter.hasNext())
    {
      Element actionElement = (Element) actIter.next();
      String actionTarget = actionElement.getAttributeValue("target");

      logCategory.info("Target: " + actionTarget);

      List taskList = actionElement.getChildren();
      Action action = null;

      if (!taskList.isEmpty())
      {
        Element taskElement = (Element) taskList.get(0);
        String taskName = taskElement.getName();

        if (taskName.equals("Enable"))
        {
          action = new Enable(actionTarget);

          logCategory.info("Action is Enable");
        }
        else if (taskName.equals("Disable"))
        {
          action = new Disable(actionTarget);

          logCategory.info("Action is Disable");
        }
      }

      if (action != null)
      {
        condition.addAction(action);
      }
    }

    return condition;
  }

}

