/**
	DialogLayout2
*/
package org.vovida.prov.dl;

import java.awt.*;
import java.util.*;

import javax.swing.*;

public class DialogLayout2 
	implements LayoutManager 
{
	protected static final int COMP_THREE_COL = 0;
	protected static final int COMP_BIG = 1;
	protected static final int COMP_BUTTON = 2;

	protected int m_divider1 = -1;
	protected int m_divider2 = -1;
	protected int m_hGap = 10;
	protected int m_vGap = 5;
	protected Vector m_v = new Vector();

	public DialogLayout2() {}

	public DialogLayout2(int hGap, int vGap) 
	{
		m_hGap = hGap;
		m_vGap = vGap;
	}

	public void addLayoutComponent(String name, Component comp) {}

	public void removeLayoutComponent(Component comp) {}

	public Dimension preferredLayoutSize(Container parent)
	{
		m_v.removeAllElements();
		int w = 0;
		int h = 0;
	//	int type = -1;
		int type = COMP_BIG;

		for (int k=0 ; k<parent.getComponentCount(); k++) 
		{
			Component comp = parent.getComponent(k);
			int newType = getLayoutType(comp);
			if (k == 0)
				type = newType;

			if (type != newType)
			{
				Dimension d = preferredLayoutSize(m_v, type);
				w = Math.max(w, d.width);
				h += d.height + m_vGap;
				m_v.removeAllElements();
				type = newType;
			}

			m_v.addElement(comp);
		}

		Dimension d = preferredLayoutSize(m_v, type);
		w = Math.max(w, d.width);
		h += d.height + m_vGap;

		h -= m_vGap;

		Insets insets = parent.getInsets();
		return new Dimension(w+insets.left+insets.right, 
			h+insets.top+insets.bottom);
	}

	protected Dimension preferredLayoutSize(Vector v, int type)
	{
		int w = 0;
		int h = 0;
		switch (type)
		{
		case COMP_THREE_COL:
			int divider1 = getDivider1(v);
			int divider2 = getDivider2(v);
			for (int k=1 ; k<v.size(); k+=3) 
			{
				Component comp = (Component)v.elementAt(k);
				Dimension d = comp.getPreferredSize();
				w = Math.max(w, d.width);
				h += d.height + m_vGap;
			}
			h -= m_vGap;
			//return new Dimension(divider1+w, h);
			return new Dimension(divider2+10, h);

		case COMP_BIG:
			for (int k=0 ; k<v.size(); k++) 
			{
				Component comp = (Component)v.elementAt(k);
				Dimension d = comp.getPreferredSize();
				w = Math.max(w, d.width);
				h += d.height + m_vGap;
			}
			h -= m_vGap;
			return new Dimension(w, h);

		case COMP_BUTTON:
			Dimension d = getMaxDimension(v);
			w = d.width + m_hGap;
			h = d.height;
			return new Dimension(w*v.size()-m_hGap, h);
		}
		throw new IllegalArgumentException("Illegal type "+type);
	}

	public Dimension minimumLayoutSize(Container parent)
	{
		return preferredLayoutSize(parent);
	}

	public void layoutContainer(Container parent)
	{
		m_v.removeAllElements();
		//int type = -1;
		int type = COMP_BIG;

		Insets insets = parent.getInsets();
		int w = parent.getWidth() - insets.left - insets.right;
		int x = insets.left;
		int y = insets.top;

		for (int k=0 ; k<parent.getComponentCount(); k++) 
		{
			Component comp = parent.getComponent(k);
			int newType = getLayoutType(comp);
			if (k == 0)
				type = newType;

			if (type != newType)
			{
				y = layoutComponents(m_v, type, x, y, w);
				m_v.removeAllElements();
				type = newType;
			}
			
			m_v.addElement(comp);
		}

		y = layoutComponents(m_v, type, x, y, w);
		m_v.removeAllElements();
	}

	protected int layoutComponents(Vector v, int type,
		int x, int y, int w)
	{
		switch (type)
		{
		case COMP_THREE_COL:
			int divider1 = getDivider1(v);
			int divider2 = getDivider2(v);
			for (int k=2 ; k<v.size(); k+=3) 
			{
				Component comp1 = (Component)v.elementAt(k-2);
				Component comp2 = (Component)v.elementAt(k-1);
				Component comp3 = (Component)v.elementAt(k);
				Dimension d = comp2.getPreferredSize();
				
				comp1.setBounds(x, y, divider1, d.height);

				comp2.setBounds(x+divider1, y, divider2-divider1, d.height);

        // we add 2 points to the width so that the start symbol
        // displays correctly
				comp3.setBounds(x+divider2, y, w-divider2+2, d.height);

        y += d.height + m_vGap;

			}
			//y -= m_vGap;
			return y;

		case COMP_BIG:
			for (int k=0 ; k<v.size(); k++) 
			{
				Component comp = (Component)v.elementAt(k);
				Dimension d = comp.getPreferredSize();
				comp.setBounds(x, y, w, d.height);
				y += d.height + m_vGap;
			}
			//y -= m_vGap;
			return y;

		case COMP_BUTTON:
			Dimension d = getMaxDimension(v);
			int ww = d.width*v.size() + m_hGap*(v.size()-1);
			int xx = x + Math.max(0, (w - ww)/2);
			for (int k=0 ; k<v.size(); k++) 
			{
				Component comp = (Component)v.elementAt(k);
				comp.setBounds(xx, y, d.width, d.height);
				xx += d.width + m_hGap;
			}
			return y + d.height;
		}
		throw new IllegalArgumentException("Illegal type "+type);
	}

	public int getHGap()
	{
		return m_hGap;
	}

	public int getVGap()
	{
		return m_vGap;
	}

	protected int getDivider1(Vector v)
	{
		int divider = 0;
		for (int k=0 ; k<v.size(); k+=3) 
		{
			Component comp = (Component)v.elementAt(k);
			Dimension d = comp.getPreferredSize();
			divider = Math.max(divider, d.width);
		}
		divider += m_hGap;
    m_divider1 = divider;
		return m_divider1;
	}

	protected int getDivider2(Vector v)
	{
		int divider = 0;
		for (int k=1 ; k<v.size(); k+=3) 
		{
			Component comp = (Component)v.elementAt(k);
			Dimension d = comp.getPreferredSize();
			divider = Math.max(divider, d.width);
		}
		divider += m_hGap;
		divider += m_divider1;

		m_divider2 = divider;
		return m_divider2;
	}

	protected Dimension getMaxDimension(Vector v)
	{
		int w = 0;
		int h = 0;
		for (int k=0 ; k<v.size(); k++) 
		{
			Component comp = (Component)v.elementAt(k);
			Dimension d = comp.getPreferredSize();
			w = Math.max(w, d.width);
			h = Math.max(h, d.height);
		}
		return new Dimension(w, h);
	}

	protected int getLayoutType(Component comp)
	{
		if (comp instanceof AbstractButton)
			return COMP_BUTTON;
		else if (comp instanceof JPanel || 
			comp instanceof JScrollPane || 
			comp instanceof DialogSeparator)
			return COMP_BIG;
		else
			return COMP_THREE_COL;
	}

	public String toString() 
	{
		return getClass().getName() + "[hgap=" + m_hGap + ",vgap=" 
			+ m_vGap + ",divider1=" + m_divider1 + ",divider2=" + m_divider2 + "]";
	}
}

