/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.dynamicGui;

import org.vovida.prov.content.ContentFactory;
import org.vovida.prov.content.condition.ConditionFactory;
import org.vovida.prov.content.ContentData;
import org.vovida.prov.content.condition.Condition;
import org.vovida.prov.widget.ListGuiComponent;
import org.vovida.prov.widget.GuiComponent;
import org.vovida.prov.widget.GuiFactory;
import org.vovida.prov.widget.Table;
import org.vovida.prov.widget.LayoutPanel;
import org.vovida.prov.widget.GuiException;
import org.vovida.prov.psLib.ProvisioningInterface;
import org.vovida.prov.psLib.PSDBResult;
import org.vovida.prov.psLib.OperationException;

import java.awt.Container;
import javax.swing.JOptionPane;
import javax.swing.JFrame;
import java.util.List;
import java.util.ListIterator;
import java.util.Vector;
import java.util.ListIterator;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.IOException;
import org.jdom.Element;
import org.apache.log4j.Category;

/**
 * This class is parses a branch of an XMLDescription tree
 * and builds the necessary classes to represent an instance of it.
 *
 * @author Barbara Samson
 */
public class DescriptionReader implements DTDConstants
{

  static Category logCategory = 
    Category.getInstance(DescriptionReader.class.getName());

  /**
   * Create a ContentData according to a DBEntry branch.
   */
  static public ContentData addEntry(Element topElement, Container container, 
          ProvisionableInstance pInstance)
  {
    String name = topElement.getAttributeValue(NAME);

    if (logCategory.isInfoEnabled())
    {
      logCategory.info("adding " + topElement.getName() + " " + name);
    }

    String enabled = topElement.getAttributeValue(ENABLED);

    String label = topElement.getChildText(LABEL);
    String tooltip = topElement.getChildText(TOOLTIP);
    String alert = topElement.getChildText(ALERT);
    String def = topElement.getChildTextTrim(DEFAULT);

    // String reference    = null;
    // String pserverQuery = null;
    boolean isRequired = (topElement.getChild(REQUIRED) == null ? false 
            : true);
    boolean isHidden = (topElement.getChild(HIDDEN) == null ? false : true);
    boolean isKey = false;
    ContentData content = null;

    // the default value for label is name.
    if (label == null)
    {
      label = name;
    }

    // create GuiComponent
    GuiComponent widget = null;
    Element widgetElement = topElement.getChild(WIDGET);

    if (widgetElement == null)
    {
      widgetElement = new Element(WIDGET);
    }

    widget = GuiFactory.newGuiComponent(label, widgetElement);

    if (logCategory.isInfoEnabled())
    {
      logCategory.info("widget is " + widget.getClass().getName());
    }

    if ((tooltip != null) && (!tooltip.equals("")))
    {
      widget.setTooltip(tooltip);
    }

    // create ContentData
    Element contentElement = topElement.getChild(CONTENT);

    if (contentElement == null)
    {
      contentElement = new Element(CONTENT);
    }

    // If the Content is TableType, then the widget must be Table.
    // We have to take care of that here.
    Element testForTableElement = contentElement.getChild(TABLETYPE);

    if (testForTableElement != null)
    {
      if (!(widget instanceof Table))
      {
        widgetElement = new Element(WIDGET);

        widgetElement.addContent(new Element(TABLE));

        widget = GuiFactory.newGuiComponent(label, widgetElement);

        logCategory.info("changed widget to Table: " 
                         + widget.getClass().getName());
      }
    }

    // If the Content is Boolean or an Enumeration,
    // then the widget must be a ListGuiComponent
    Element testForBooleanElement = contentElement.getChild(BOOLEANTYPE);

    if (testForBooleanElement != null)
    {
      if (!(widget instanceof ListGuiComponent))
      {
        widgetElement = new Element(WIDGET);

        widgetElement.addContent(new Element(CHECKBOX));

        widget = GuiFactory.newGuiComponent(label, widgetElement);

        logCategory.info("changed widget to Checkbox: " 
                         + widget.getClass().getName());
      }
    }
    else
    {
      List childrenOfContent = contentElement.getChildren();
      ListIterator iter = childrenOfContent.listIterator();

      // we only look at the first child
      if (iter.hasNext())
      {
        Element childOfContent = (Element) iter.next();
        Element enumerationElement = 
          (Element) childOfContent.getChild(ENUMERATION);

        if ((enumerationElement != null) 
            &&!(widget instanceof ListGuiComponent))
        {
          widgetElement = new Element(WIDGET);

          widgetElement.addContent(new Element(COMBOBOX));

          widget = GuiFactory.newGuiComponent(label, widgetElement);

          logCategory.info("changed widget to ComboBox: " 
                           + widget.getClass().getName());
        }
      }
    }

    content = ContentFactory.newContentData(name, widget, pInstance, 
            contentElement);

    if (logCategory.isInfoEnabled())
    {
      logCategory.info("content type is " + content.getClass().getName());
    }

    // set key information
    if (pInstance.getType().hasKey(name))
    {
      widget.setKey(true);
      content.setKey(true);

      isKey = true;
    }

    // set required information
    if (isRequired)
    {
      content.setRequired(true);
      widget.setRequired(true);
    }

    // set to be hidden
    {
      if (isHidden)
      {
        widget.setHidden(true);
      }
    }

    // check for Condition
    List conditionChildren = topElement.getChildren(CONDITION);

    if (logCategory.isInfoEnabled() &&!conditionChildren.isEmpty())
    {
      logCategory.info("It has condition(s)");
    }

    ListIterator condIter = conditionChildren.listIterator();

    while (condIter.hasNext())
    {
      Element conditionElement = (Element) condIter.next();
      Condition condition = ConditionFactory.newCondition(content, 
              conditionElement);

      content.addCondition(condition);
    }

    // set alert
    if (alert != null)
    {
      content.setAlert(alert);
      logCategory.info("It has an alert: " + alert);
    }

    // set default
    if (def != null)
    {
      content.setDefault(def);
      logCategory.info("It has a default: " + def);
    }

    try
    {
      if (container != null)
      {
        widget.renderSelf(container);
      }
    }
    catch (GuiException e)
    {
      e.printStackTrace();
    }

    if (enabled != null)
    {
      boolean isEnabled = (enabled.equals(FALSE) ? false : true);

      content.setEnabled(isEnabled);
      logCategory.info("It " + (isEnabled ? "is" : "is not") + " enabled");
      ;
    }

    return content;
  }

  /**
   * Create a ContentData for a Border element.
   */
  static Container addContainer(Element element, Container container)
  {
    String label = element.getChildTextTrim(LABEL);
    LayoutPanel panel = GuiFactory.newPanel(label);

    try
    {
      panel.renderSelf(container);
    }
    catch (GuiException e)
    {
      e.printStackTrace();
    }

    return panel.getContainer();
  }

  static Container addPage(Element element, Container parent)
  {
	String name = element.getAttribute("name").getValue();
    String label = element.getChildTextTrim(LABEL);
    if ( (label != null) && (!"".equals(label)) )
    {
        name = label;
    }

    Container page = GuiFactory.newPage();
    parent.add(name, page);

    return page;
  }

  static Container addPagedContainer(Element element, Container parent)
  {
    Container paged = GuiFactory.newPagedContainer();

    //for some reason, the layout manager used by the parent by default does
    //not seem to handle the tabbed pane correctly - it does not show up at all
    parent.setLayout(new java.awt.BorderLayout());
    parent.add(paged, java.awt.BorderLayout.CENTER);

    return paged;
  }

  /**
   * Performs the database call for the GetRecordsInDomain element.
   * @param argsElement the actual GetRecordsInDomain element, containing
   * attributes that give parameters to the database query.
   * @result the response from the database.
   */
  static private List callGetRecordsInDomain(Element argsElement) 
    throws OperationException
  {
    String domain = argsElement.getAttributeValue(DOMAIN);

    if ((domain == null) || (domain.equals("")))
    {
      logCategory.warn("missing domain attribute");

      return null;
    }

    String field = argsElement.getAttributeValue(FIELD);

    if ((field == null) || (field.equals("")))
    {
      logCategory.warn("missing field attribute");

      return null;
    }

    String filter = argsElement.getAttributeValue(FILTER);

    if ((filter == null) || (filter.equals("")))
    {
      logCategory.warn("missing filter attribute");

      return null;
    }

    List returnList = new Vector();

    try
    {
      ProvisioningInterface ps = ProvisioningInterface.getInstance();
      PSDBResult result = ps.getRecordsInDomain(domain, filter, new String[]
      {
        field
      });

      if (result == null)
      {
        return null;
      }

      Hashtable[] rows = result.getRows();

      // we can only have one piece of data per row
      for (int i = 0; i < rows.length; i++)
      {
        Enumeration enum = rows[i].elements();

        if (enum.hasMoreElements())
        {
          returnList.add(enum.nextElement());
        }
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }

    return returnList;
  }

  /**
   * Performs the database call for the GetDomains element.
   * @param argsElement the actual GetDomains element, containing
   * attributes that give parameters to the database query.
   * @result the response from the database.
   */
  static private List callGetDomains(Element argsElement) 
    throws OperationException
  {
    String category = argsElement.getAttributeValue(CATEGORY_ATTR);

    if ((category == null) || (category.equals("")))
    {
      logCategory.warn("missing category attribute");

      return null;
    }

    ProvisioningInterface ps = ProvisioningInterface.getInstance();
    String[] domains = null;

    try
    {
      domains = ps.getListXMLDescriptions(category);
    }
    catch (IOException e1)
    {
      e1.printStackTrace();

      return null;
    }

    if ((domains == null) || (domains.length == 0))
    {
      logCategory.warn("null response from PServer");
    }

    List returnList = new Vector();

    for (int i = 0; i < domains.length; i++)
    {
      returnList.add(domains[i]);
    }

    return returnList;
  }

}




