/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.dynamicGui.test;

import org.vovida.prov.dynamicGui.TypeBuilder;
import org.vovida.prov.dynamicGui.ProvisionableType;
import org.vovida.prov.dynamicGui.ProvisionableInstance;
import org.vovida.prov.content.InvalidDataException;
import org.vovida.prov.widget.SmartLayoutManager;

import javax.swing.*;
import javax.swing.event.CaretListener;
import javax.swing.event.CaretEvent;
import javax.swing.text.BadLocationException;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.net.URL;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.Category;

public class BasicApplet extends JApplet
{
  private String descriptionString;
  private JFrame mainFrame = new JFrame("Test GUI");
  private JTextArea textArea = new JTextArea();
  private CaretListener myCaretListener;

  static Category logCategory
    = Category.getInstance(BasicApplet.class.getName());

  private ProvisionableType myType;
  private ProvisionableInstance myInstance;

  boolean isStandalone = false;

  // Get a parameter value
  public String getParameter(String key, String def)
  {
    return isStandalone ? System.getProperty(key, def)
    : (getParameter(key) != null ? getParameter(key) : def);
  }

  // Initialize the applet
  public void init()
  {
    JPanel panel = new JPanel();
    JScrollPane scrollPane = new JScrollPane();
    panel.setLayout(
      new BorderLayout());
    panel.add(scrollPane, BorderLayout.CENTER);
    scrollPane.getViewport().add(textArea);
    setUpApplet(panel);
    setContentPane(panel);
  }

  public void start()
  {
  }

  private void buildGUI()
  {
    try
    {
      myType = TypeBuilder.getType(null, textArea.getText());
      String[] fields = myType.getDBFields();
      if (fields != null)
      {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < fields.length; i++)
        {
          buf.append(fields[i]);
          buf.append(" ");
        }
        System.out.println(
          new String(buf));
      }
      JScrollPane scrollPane = new JScrollPane();
      JPanel contentPanel = new JPanel();
      contentPanel.setLayout(
        new BorderLayout());
      mainFrame.setContentPane(contentPanel);
      contentPanel.add(scrollPane, BorderLayout.CENTER);
      SmartLayoutManager smartLayout = new SmartLayoutManager();
      JPanel dynamicPanel = new JPanel(smartLayout);
      dynamicPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
      setUpFrame(contentPanel);

      myInstance = myType.buildInstance(dynamicPanel);

      scrollPane.getViewport().add(dynamicPanel);

      mainFrame.pack();
      mainFrame.setVisible(true);

    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  private void setUpFrame(JPanel contentPane)
  {
    JPanel bottomPanel      = new JPanel();
    JPanel buttonPanel      = new JPanel();
    GridLayout buttonLayout = new GridLayout();
    buttonLayout.setColumns(3);
    buttonLayout.setHgap(5);
    JButton OKButton = new JButton("OK");
    OKButton.setToolTipText("Show field values in XML format");
    OKButton.addActionListener(
      new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
         // try
          //{
            JOptionPane.showMessageDialog(
              new JFrame(), "");
// broken for now.
//              myInstance.getData("", true));
          //}
//          catch (InvalidDataException e1)
          //{
            //e1.printStackTrace();
          //}
        }
      });
    JButton CancelButton = new JButton("Cancel");
    CancelButton.setToolTipText("Cancel unsaved changes");
    CancelButton.addActionListener(
      new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          myInstance.cancel();
        }
      });

    JButton ResetButton = new JButton("Reset");
    ResetButton.setToolTipText("Revert to default values");
    ResetButton.addActionListener(
      new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          myInstance.reset();
        }
      });


    buttonPanel.setLayout(buttonLayout);
    buttonPanel.add(OKButton);
    buttonPanel.add(CancelButton);
    buttonPanel.add(ResetButton);
    bottomPanel.add(buttonPanel);
    contentPane.add(bottomPanel, BorderLayout.SOUTH);
  }

  private void setText(String filename)
  {
    StringBuffer buf = new StringBuffer();
    textArea.setText("");
    try
    {
      URL resourceURL
      = TypeBuilder.class.getResource("resources/" + filename);
      if (resourceURL == null)
      {
        System.out.println("sample is null");
      }
      else
      {
        InputStream stream = resourceURL.openStream();
        BufferedReader reader
        = new BufferedReader(
          new InputStreamReader(stream));
        String line = "";
        while ((line = reader.readLine()) != null)
        {
          buf.append(line + "\n");
        }
        textArea.setText(
          new String(buf));
      }
    }
    catch (IOException e1)
    {
      e1.printStackTrace();
    }
  }

  private void setUpApplet(JPanel contentPane)
  {
    JPanel bottomPanel      = new JPanel();
    JPanel buttonPanel      = new JPanel();
    GridLayout buttonLayout = new GridLayout();
    buttonLayout.setColumns(4);
    buttonLayout.setHgap(5);
    JButton SampleButton = new JButton("Sample");
    SampleButton.addActionListener(
      new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          setText("sample.xml");
        }
      });
    JButton BlankButton = new JButton("Blank Form");
    BlankButton.addActionListener(
      new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          setText("blank.xml");
        }
      });

    JButton ShowButton = new JButton("Show GUI");
    ShowButton.addActionListener(
      new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          buildGUI();
        }
      });

    final JLabel linePosition = new JLabel("Line: ");
    linePosition.setHorizontalAlignment(JLabel.CENTER);
    if (myCaretListener != null)
    {
      textArea.removeCaretListener(myCaretListener);
    }
    myCaretListener = new CaretListener()
    {
      public void caretUpdate(CaretEvent e)
      {
        try
        {
          int pos = textArea.getLineOfOffset(textArea.getCaretPosition());
          linePosition.setText("Line: " + (pos + 1));
        }
        catch (BadLocationException e1)
        {
          linePosition.setText("Line: unknown");
        }
      }

    };
    textArea.addCaretListener(myCaretListener);

    buttonPanel.setLayout(buttonLayout);
    buttonPanel.add(SampleButton);
    buttonPanel.add(BlankButton);
    buttonPanel.add(ShowButton);
    buttonPanel.add(linePosition);
    bottomPanel.add(buttonPanel);
    contentPane.add(bottomPanel, BorderLayout.SOUTH);
  }

  // Main method
  public static void main(String[] args)
  {
    System.out.println("This test is broken");
    return;
    /*
    // initialize logging
    URL logConfig = BasicApplet.class.getResource
      ("resources/" + "logConfig");
    if (logConfig != null)
    {
      PropertyConfigurator.configure(logConfig);
      logCategory.warn("Logging configured from file");
    }
    else
    {
      // no configuration file, so just log to console
      BasicConfigurator.configure();
      logCategory.info("Logging configured from BasicConfigurator");
    }
    BasicApplet applet = new BasicApplet();

    applet.isStandalone = true;

    JFrame frame = new JFrame();
    frame.setDefaultCloseOperation(3);
    frame.getContentPane().add(applet, BorderLayout.CENTER);

    applet.init();

    applet.start();

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    frame.setSize(
      new Dimension(500, 400));

    frame.setLocation((d.width - frame.getSize().width) / 2,
    (d.height - frame.getSize().height) / 2);
    frame.setVisible(true);
    */
  }


}
