package org.vovida.prov.dynamicGui.test;

import org.vovida.prov.dynamicGui.TypeBuilder;
import org.vovida.prov.dynamicGui.ProvisionableType;
import org.vovida.prov.dynamicGui.ProvisionableInstance;
import org.vovida.prov.content.InvalidDataException;
import org.vovida.prov.content.ReasonCodes;
import org.vovida.prov.widget.SmartLayoutManager;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;
import org.jdom.JDOMException;

import java.util.List;
import java.util.ListIterator;
import java.util.Hashtable;
import java.io.InputStream;
import java.io.StringReader;
import java.net.URL;
import java.io.IOException;
import java.awt.Container;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * This class does the automated unit testing.
 * Each test results in either success or failure.
 * If the test fails, then that failure is thrown
 * back to the caller (ie the build program);
 */
public class TestApplication implements ReasonCodes
{

  private static final int BUFSIZE = 1024;

  private static final int OK = 0;
  private static final int BAD_FILE = 1;
  private static final int UNEXPECTED_EXCEPTION = 2;
  private static final int BAD_REASON = 3;
  private static final int NO_EXCEPTION = 4;
  private static final int BAD_DATA = 5;

  private static final String[] results = new String[]
  {
    "OK",
    "BAD_FILE",
    "UNEXPECTED_EXCEPTION",
    "BAD_REASON",
    "NO_EXCEPTION",
    "BAD_DATA"
  };

  private JFrame mainFrame;
  private Document myDocument = null;
  private Element myRootElement = null;
  private ProvisionableType myType = null;
  private ProvisionableInstance myInstance = null;
  private String myReason = null;
  private String myExpectedReason = null;
  private String myData = null;
  private String myExpectedData = null;

  /*
  private String[] myFilesToTest = new String[]
  {
    "changeKeyfield.xml",
    "noChangeKeyfield.xml",
    "setDefaults.xml",
    "setFieldsBlank.xml",
    "setAndGetData.xml"
  };
  */


  public static void main(String[] args)
  {
    System.out.println("This test is broken");
    return;
    /*
    try
    {
      new TestApplication().start();
    }
    catch (Exception e)
    {
      e.printStackTrace();
      System.exit(1);
    }
    System.exit(0);
    */
  }

  private void start()
  {
    try
    {
      mainFrame = new JFrame("Test Application");
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
    mainFrame.pack();
    mainFrame.setVisible(true);
    Document listOfTests = null;
    InputStream listOfTestsStream
      = TestApplication.class.getResourceAsStream("resources/listOfTests.xml");
    SAXBuilder builder = new SAXBuilder("org.apache.xerces.parsers.SAXParser");
    try
    {
      listOfTests = builder.build(listOfTestsStream);
    }
    catch (JDOMException e1)
    {
      System.out.println("JDOMException reading listOfTests.xml");
      System.out.println(e1.getMessage());
      return;
    }
    Element root = listOfTests.getRootElement();
    if (root == null)
    {
      System.out.println("Error: listOfTests has no root element");
      return;
    }
    List children = root.getChildren("test");
    ListIterator iter = children.listIterator();
    while (iter.hasNext())
    {
      String test = ((Element)iter.next()).getTextTrim();
      System.out.println("Testing " + test);
      int result = runTest(test);
      System.out.println("Result: " + results[result]);
      switch(result)
      {
        case BAD_FILE:
        {
          System.out.println("terminating because bad file encountered: "
            + test);
          System.exit(1);
        }
        case BAD_REASON:
        {
          System.out.println("reason expected: " + myExpectedReason);
          System.out.println("reason received: " + myReason);
          System.exit(1);
        }
        case BAD_DATA:
        {
          System.out.println("data expected: " + myExpectedData);
          System.out.println("data received: " + myData);
          System.exit(1);
        }
        case NO_EXCEPTION:
        {
          System.out.println("exception expected: " + myExpectedReason);
          System.exit(1);
        }
        default:
        break;
      }
    }
  }

  private ProvisionableType createType(String filename)
  {
    myDocument = null;
    myRootElement = null;
    myType = null;
    myInstance = null;
    myReason = null;
    myExpectedReason = null;
    myData = null;
    myExpectedData = null;
    InputStream stream
      = TestApplication.class.getResourceAsStream("resources/" + filename);
    if (stream == null)
    {
      System.out.println("File not found: " + filename);
      return null;
    }
    SAXBuilder builder = new SAXBuilder("org.apache.xerces.parsers.SAXParser");
    try
    {
      myDocument = builder.build(stream);
    }
    catch (JDOMException e1)
    {
      System.out.println("JDOMException thrown reading " + filename);
      System.out.println(e1.getMessage());
    }
    if (myDocument == null)
    {
      return null;
    }
    myRootElement = myDocument.getRootElement();
    Element dataDefn = myRootElement.getChild("DataDefn");
    // unfortunately we need to write this back to a string
    // so that the dtd can be prepended for validation
    String dataDefnString = null;
    try
    {
      dataDefnString = new XMLOutputter().outputString(dataDefn);
    }
    catch (IOException e2)
    {
      e2.printStackTrace();
    }
    if (dataDefnString == null)
    {
      System.out.println("Data Definition is null");
      return null;
    }

    return TypeBuilder.getType(null, dataDefnString);
  }

  private int checkException(InvalidDataException e)
  {
    int reasonReceived = e.getReason();
    myReason = reasons[reasonReceived];
    int reasonExpected
      = InvalidDataException.getReason(myExpectedReason);
    if (reasonReceived != reasonExpected)
    {
      return BAD_REASON;
    }
    return OK;
  }

    // the actual tests

  private int runTest(String fileName)
  {
      XMLOutputter outputter = new XMLOutputter();
      outputter.setTrimText(true);
      Container contentPane = mainFrame.getContentPane();
      contentPane.removeAll();
      JPanel panel = new JPanel();
      panel.setLayout(new SmartLayoutManager());
      contentPane.add(panel);
  	  myType = createType(fileName);
      if (myType == null)
      {
        System.out.println("type created from " + fileName
            + " is null");
        return BAD_FILE;
      }
  	  myInstance = myType.buildInstance(panel);
      myInstance.setTest(true);
      mainFrame.pack();

      // start by setting up the expected result
      Element expectedResult = myRootElement.getChild("expectedResult");
      Element expectedException =
        (expectedResult == null) ? null : expectedResult.getChild("exception");
      myExpectedReason =
        (expectedException == null) ? null : expectedException.getAttributeValue("resultCode");
      boolean gotException = false;
      Element expectedData =
        (expectedResult == null) ? null : expectedResult.getChild("fieldValues");
      if (expectedData != null)
      {
        try
        {
          myExpectedData = outputter.outputString(expectedData);
        }
        catch (IOException e)
        {}
      }

      // now set data
  	  List fieldValuesList = myRootElement.getChildren("fieldValues");
      Hashtable hash = new Hashtable();
      ListIterator iter = fieldValuesList.listIterator();
      while(iter.hasNext())
      {
        Element fieldValue = (Element)iter.next();
        hash.put(fieldValue.getName(), fieldValue.getTextTrim());
      }
      try
      {
        myInstance.setData(hash);
// broken for now.
//          myInstance.getData("", true);
      }
      catch (InvalidDataException e)
      {
       gotException = true;
        if (expectedException == null)
        {
          return UNEXPECTED_EXCEPTION;
        }
        int exceptionResult = checkException(e);
        if (exceptionResult != OK)
        {
          return exceptionResult;
        }
      }
    if( (expectedException != null) && !gotException)
    {
      return NO_EXCEPTION;
    }
    if (myExpectedData != null)
    {
      // try
      //{
// broken for now
//    myData = outputter.outputString(myInstance.getDataAsDocument().getRootElement());
      //}
      //catch (IOException e1)
      //{}
      //catch (InvalidDataException e2)
      //{
      // if getData() is throwing an exception,
      // then myData will always be null
      //}
      if (myExpectedData.equals(myData))
      {
        // all ok
      }
      else
      {
        return BAD_DATA;
      }
    }
    return OK;
  }

}
