/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.endpoint;

import org.vovida.prov.navigation.ConfigConstants;
import org.vovida.prov.navigation.NavBarClient;
import org.vovida.prov.navigation.TopLevel;
import org.vovida.prov.psLib.ProvisioningInterface;
import org.vovida.prov.dynamicGui.TypeBuilder;
import org.vovida.prov.template.NestedException;
import org.vovida.prov.template.BulkAddController;
import org.vovida.prov.template.TemplateBuilder;
import org.vovida.prov.template.XMLTemplateLoader;
import org.vovida.prov.template.TemplateLoader;

import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.table.AdministratorPanel;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JOptionPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;

import org.apache.log4j.Category;



/**
 * @author Barbara Samson
 */
public class EndpointClient extends NavBarClient implements ConfigConstants
{
  private JFrame myParent;
  private ProvisioningInterface ps;
  private JButton tbButton;
  private JButton addButton;
  private JButton showTable;
  private AdministratorPanel panel;

  private boolean init = false;
  String [] allEndpoints = null;



  private static Category logCategory =
    Category.getInstance(EndpointClient.class.getName());

  public EndpointClient(String name, TopLevel parent)
  {
    super(name, parent);

    myParent = parent;

    ps = ProvisioningInterface.getInstance();
    tbButton = new JButton("Template Builder");
    addButton = new JButton("Add With Template");
    showTable = new JButton("Show Endpoints");
  }

  protected void showSelf()
  {
    try
    {
      start();
    }
    catch (NestedException e)
    {
      JOptionPane.showMessageDialog(myParent, e.getMessage(), 
              "Error in Endpoint Provisioning", JOptionPane.ERROR_MESSAGE);
      e.printStackTrace();
    }
  }


  private void init() throws NestedException
  {
    if (!init)
    {
        init = true;

	    try
	    {
	      if (allEndpoints == null)
	      {
	 	     allEndpoints = ps.getListXMLDescriptions(ENDPOINTS);
	      }
	    }
	    catch (Exception e1)
	    {
	      JOptionPane.showMessageDialog(myParent, 
	              "error in list of XML Descriptions" + ": " + e1.getMessage(), 
	              e1.getClass().toString(), JOptionPane.ERROR_MESSAGE);
	
	      return;
	    }

	    String endpointFile = "";
	    for (int i = 0; i < allEndpoints.length; i++)
	    {
	      endpointFile = allEndpoints[i];
	
	      String xmlString = null;
	
	      try
	      {
	        xmlString = ps.getXMLFile(ENDPOINTS, endpointFile);
	      }
	      catch (Exception e2)
	      {
	        JOptionPane.showMessageDialog(myParent,
	                "error in " + endpointFile + ": " + e2.getMessage(), 
	                e2.getClass().toString(), JOptionPane.ERROR_MESSAGE);
	
	        continue;
	      }
	
	      // create all ProvisionableTypes in hashtable.
	      TypeBuilder.buildType(xmlString);
	    }

	    final TemplateLoader tl = new XMLTemplateLoader();
	
	    // 
	    // Action stuff for Template Builder
	    // 
	    tbButton.addActionListener(new ActionListener()
	    {
	      public void actionPerformed(ActionEvent ae)
	      {
	        JPanel panel = new JPanel();
	
	        add(panel);
	
	        final TemplateBuilder tb = new TemplateBuilder(myParent, tl);
	
	        tb.pack();
	        tb.setVisible(true);
	      }
	
	    });
	
	    // 
	    // Action stuff for Adding with Templates
	    // 
	    addButton.addActionListener(new BulkAddController(myParent, tl));

        showTable.addActionListener(new ActionListener()
        {
          public void actionPerformed(ActionEvent event)
          {
            final JFrame frame = new JFrame("Endpoints");
            frame.getContentPane().setLayout(new BorderLayout());

            ProvisioningInterface pInterface = org.vovida.prov.psLib.SocketProvisioningInterface.getInstance();
            ProvisioningServerWrapper wrapper = new ProvisioningServerWrapper(pInterface);

            if (panel == null)
            {
	            panel = new AdministratorPanel("", "adminAccounts",
                    "endpoints",  wrapper, myParent);
                add(panel);
                panel.init();
            }

            frame.getContentPane().add(panel,BorderLayout.CENTER);
            JPanel temp = new JPanel();
            temp.setLayout(new BoxLayout(temp,BoxLayout.X_AXIS));
            temp.add(Box.createHorizontalGlue());
            JButton hide = new JButton("Hide");
            hide.addActionListener(new ActionListener()
            {
              public void actionPerformed (ActionEvent e)
              {
                  frame.setVisible(false);
              }
            });
            temp.add(hide);
            frame.getContentPane().add(temp, BorderLayout.SOUTH);
            frame.pack();
            frame.setVisible(true);
          }
        });
    }
  }


  private void start() throws NestedException
  {
    init();

    JPanel temp = new JPanel();
    temp.setLayout(new BoxLayout(temp,BoxLayout.Y_AXIS));
    temp.add(tbButton);
    temp.add(Box.createHorizontalStrut(10));
    temp.add(addButton);
    temp.add(Box.createHorizontalStrut(10));
    temp.add(showTable);
    temp.add(Box.createVerticalGlue());
    setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
    temp.setVisible(true);
    add(temp);
    add(Box.createVerticalGlue());
    myParent.pack();
  }

}




