package org.vovida.prov.group;

import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;

import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.DataFlavor;

import java.util.Vector;

import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.table.TableModel;
import org.vovida.prov.group.GroupNode;
import org.vovida.prov.util.logging.Util;
import org.vovida.prov.util.FieldNames;
import org.vovida.prov.util.ParseUtils;

import org.apache.log4j.Category;




public class GroupDragAndDropTree extends DragAndDropTree
{
    private static Category logCategory = Category.getInstance(GroupDragAndDropTree.class.getName());

    private ProvisioningServerWrapper server;
    private String childDomain;
    private String parentAccount;


    public GroupDragAndDropTree (DefaultTreeModel model,
        ProvisioningServerWrapper wrapper, String domain, String parent)
    {
        super(model);
        server = wrapper;
        childDomain = domain;
        parentAccount = parent;
    }

     protected boolean isDataFlavourSupported(Transferable transferable)
     {
        return transferable.isDataFlavorSupported(DataFlavor.stringFlavor);
     }

	 /**
      * The gui portion of the drag-and-drop operation has completed
      * successfully. Now update the provisioning server with the information
      * about what changed groups. <p>
      */
     protected void dragEnded(DefaultMutableTreeNode sourceParent,
        DefaultMutableTreeNode destinationParent, DefaultMutableTreeNode movedNode)
     {
		//get the old group name
        String sourceGroup = ((GroupNode)sourceParent.getUserObject()).getGroupName();

		//get the new group name
        String destinationGroup = ((GroupNode)destinationParent.getUserObject()).getGroupName();

        if (sourceGroup.equals(destinationGroup))
        {
            //nothing to do
            return;
        }

        //get the account that was moved
		GroupMemberNode account = (GroupMemberNode)movedNode.getUserObject();

        //get the current group string for this account
		Vector groups = account.getGroups();
        if (groups == null)
        {
            try
            {
	            String [] groupList = server.getGroups(childDomain,account.getName());
	            groups = new Vector();
				for (int i=0; i<groupList.length; i++)
	            {
	                groups.addElement(groupList[i]);
	            }
	            account.setGroups(groups);
            }
            catch (Exception ex)
            {
                String message = "Could not get list of groups for account " + account.getName();
                logCategory.error(message, ex);
				Util.showException(message,ex);
                return;
            }
        }

        //modify it according to the drag and drop changes
		if (!sourceGroup.equals("")) //if the source was not the default group
        {
        	if (groups.contains(sourceGroup))
	        {
                groups.remove(sourceGroup);
	        }
        }

        if (!destinationGroup.equals(""))
        {
            if (!groups.contains(destinationGroup))
            {
                groups.addElement(destinationGroup);
            }
        }

        //save the list of groups in the node
        account.setGroups(groups);

        //write it back to the provisioning server
        String groupString = ParseUtils.generateCommaDelimitedString(groups);

        try
        {
	        server.setFields(childDomain, account.getName(), /**parentAccount,**/
                new String [] {FieldNames.groupColumnName},
                new String [] {groupString});
        }
        catch (Exception ex)
        {
            String message = "Could not write back the new list of groups for account " + account.getName();
            logCategory.error(message,ex);
            Util.showException(message,ex);
            return;
        }

        //TO DO: update the table (?)
     }

     protected boolean okToDropNode(DefaultMutableTreeNode parent,
       DefaultMutableTreeNode newChild)
     {
		GroupNode group = (GroupNode)parent.getUserObject();
        if(group.isDataLoaded())
        {
            return true;
        }
        return false;
     }

}
