package org.vovida.prov.group;

import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.tree.TreePath;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import org.vovida.prov.util.assert.*;
import org.vovida.prov.group.task.TaskLoadGroupMembers;
import org.vovida.prov.util.task.Task;
import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.util.task.TaskListener;

import org.apache.log4j.Category;


/**
 * A TreeExpansionListener which is registered on the group tree and loads
 * data from the provisioning server whenever a group node is expanded. <p>
 **/
public class GroupSelectionListener implements TreeExpansionListener, TaskListener
{
    private static Category logCategory = Category.getInstance(GroupSelectionListener.class.getName());

    private String accountIdentifier;

	/** The domain which contains the accounts that are members of the group */
    private String domain;

    private ProvisioningServerWrapper server;
	private DefaultTreeModel model;

    /** @link dependency */
    /*#GroupNode lnkGroupNode;*/

    public GroupSelectionListener(String aDomain, String accountId,
        ProvisioningServerWrapper psInt, DefaultTreeModel aModel)
    {
        Assert.assert("psInt != null", psInt != null);
        Assert.assert("model != null", aModel != null);
        Assert.assert("accountId != null", accountId != null);
//        Assert.assert("accountId != null && != \"\"", accountId != null && accountId != "");
        server = psInt;
        accountIdentifier = accountId;
        model = aModel;
        domain = aDomain;
    }

    public void treeExpanded (TreeExpansionEvent event)
    {
        logCategory.debug("Tree expanded");
		//if the list of members has not been loaded yet, start a new thread
        //to load it.

        //determine which group was expanded
        TreePath path = event.getPath();
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();
        GroupNode group = (GroupNode)node.getUserObject();

        if (group.isDataLoaded())
        {
            return;
        }

		//mark that the data has been loaded for this group
        group.setDataLoaded();

        TaskLoadGroupMembers task = new TaskLoadGroupMembers(node,domain, accountIdentifier,this,server);
        task.start();
    }

    public void treeCollapsed(TreeExpansionEvent event)
    {
        //no implementation necessary
    }

   //------------------start TaskListener interface----------------------------
   	public void taskFinished(Task task, boolean status, Object returnValue)
    {
        //if the task completed successfully
		if (true)
        {
			//remove the node that says "data not loaded"
			final DefaultMutableTreeNode node = (DefaultMutableTreeNode)returnValue;
            node.remove(0);

            Runnable runnable = new Runnable()
            {
            	public void run()
	            {
					logCategory.debug("task load group members finished in thread " + Thread.currentThread());
                    model.reload(node);
    	        }
            };
            SwingUtilities.invokeLater(runnable);
        }
        else
        {
			JOptionPane.showMessageDialog(null,"Could not load data because: \n\n"
                + task.getException());
        }
    }
   //------------------end TaskListener interface------------------------------

}
