package org.vovida.prov.group.action;

import java.awt.event.ActionEvent;
import java.awt.Component;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.JTree;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import java.util.Enumeration;

import org.vovida.prov.util.FieldNames;
import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.util.ParseUtils;
import org.vovida.prov.util.assert.*;
import org.vovida.prov.util.logging.Util;
import org.vovida.prov.util.task.TaskListener;
import org.vovida.prov.util.task.Task;
import org.vovida.prov.group.*;
import org.vovida.prov.group.task.TaskDeleteGroup;

import org.apache.log4j.Category;



public class ActionDeleteGroup extends ActionGroup
{
    private static Category logCategory = Category.getInstance(ActionDeleteGroup.class.getName());
    private String childDomain;


	public ActionDeleteGroup(String actionName, String accountId, String parentDomain,
        String aChildDomain, ProvisioningServerWrapper psInt, JTree aTree)
    {
		super(actionName, accountId, parentDomain, psInt, aTree);
        childDomain = aChildDomain;
    }

    public void actionPerformed(ActionEvent event)
    {
		Assert.assert("path component is a DefaultMutableTreeNode", DefaultMutableTreeNode.class.isInstance(clickedPath.getLastPathComponent()));
        final DefaultMutableTreeNode node = (DefaultMutableTreeNode)clickedPath.getLastPathComponent();
        final String groupName = ((GroupNode)node.getUserObject()).toString();

        //if the user did not click on a group
        if (!GroupNode.class.isInstance(node.getUserObject()))
        {
            return;
        }

        new TaskDeleteGroup(domain, accountIdentifier, childDomain,
            new TaskListener()
	        {
			    public void taskFinished(Task task, boolean status, Object returnValue)
                {
                    if (status) //task completed successfully
                    {
                        try
                        {
					        //contact the provisioning server and update the administrator account
	                        String [] groups = server.getDefinedGroups(domain, accountIdentifier);
                            String newGroups = ParseUtils.removeStringFromList(groupName, groups);

                            server.setFields(domain, accountIdentifier,
                                new String [] {FieldNames.definedGroupsConlumnName},
                                new String [] {newGroups});
                        }
                        catch (Exception e)
                        {
                            String message = "Could not modify list of groups for administrator \"" + accountIdentifier + "\".";
							logCategory.error(message,e);
                            Util.showException(message,e);
                            return;
                        }

				        final TreePath parent = clickedPath.getParentPath();

						//get all the child nodes
                        int numChildren  = node.getChildCount();
                        for (int i=0; i<numChildren; i++)
                        {
							DefaultMutableTreeNode child = (DefaultMutableTreeNode)node.getChildAt(0);
                            String name = ((GroupMemberNode)child.getUserObject()).getName();
                            try
                            {
	                            String [] groups = server.getGroups(childDomain,name);
	                            if (groups.length == 0) //if this account does not belong to any other groups
	                            {
	                            	((DefaultMutableTreeNode)node.getParent()).add(child); //add it to the parent node of the group
	                            }
                            }
                            catch (Exception ex)
                            {
                                String message = "Could not get groups for account \"" + name + "\".";
                                logCategory.error(message,ex);
                                Util.showException(message,ex);
                            }
                        }

				        //remove the group node from the tree
				        ((DefaultMutableTreeNode)parent.getLastPathComponent()).remove(node);

						SwingUtilities.invokeLater(new Runnable()
                        {
                            public void run()
                            {
						        //repaint the tree
						        ((DefaultTreeModel)tree.getModel()).reload((DefaultMutableTreeNode)parent.getLastPathComponent());
                            }
                        });
                    }
                }
	        }, server, node).start();
    }

}
