package org.vovida.prov.group.action;

import java.awt.event.ActionEvent;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.JOptionPane;
import javax.swing.JTree;

import org.vovida.prov.group.*;
import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.util.FieldNames;
import org.vovida.prov.util.logging.Util;

import org.apache.log4j.Category;



public class ActionNewGroup extends ActionGroup
{
    private static Category logCategory = Category.getInstance(ActionNewGroup.class.getName());


	public ActionNewGroup(String actionName, String accountId, String domain,
        ProvisioningServerWrapper psInt, JTree aTree)
    {
		super(actionName, accountId, domain, psInt, aTree);
    }

    public void actionPerformed(ActionEvent event)
    {
        //show a dialog to prompt the user for the name of the group to create
        String groupName = JOptionPane.showInputDialog(null, "Enter a name for the group to create:");

        //contact the provisioning server and ask it to add the group to
        //the administrator's account data

        try
        {
        	//find out what the groups for this administrator are
			String [] groups = server.getDefinedGroups(domain, accountIdentifier);

			//see if a group with the given name already exists
	        boolean found = false;
	        for (int i=0; i<groups.length; i++)
	        {
	            if (groups[i].equals(groupName))
	            {
	                found = true;
	                break;
	            }
	        }

	        if (found)
	        {
				//if the group exists, complain and return
	            JOptionPane.showMessageDialog(null, "The group \"" + groupName + "\" already exists.");
	            return;
	        }

			//create a comma delimited string containing all the groups
	        String groupList = "";
	        for (int i=0; i<groups.length; i++)
	        {
				groupList += groups[i] + ",";
	        }
	        groupList += groupName;

			try
            {
		        //write back a new list of groups
//		        String parentAccount = accountIdentifier.substring(0,accountIdentifier.lastIndexOf("."));
//		        accountIdentifier = accountIdentifier.substring(accountIdentifier.lastIndexOf(".")+1,accountIdentifier.length());

		        server.setFields(domain, accountIdentifier, /**parentAccount, **/
                    new String [] {FieldNames.definedGroupsConlumnName},
                    new String [] {groupList});
            }
            catch (Exception exc)
            {
                String message = "Could not write back new list of groups.";
                logCategory.error(message,exc);
                Util.showException(message,exc);
                return;
            }
        }
        catch (Exception ex)
        {
            String message = "Could not find the list of groups defined by the administrator \"" + accountIdentifier + "\".";
        	logCategory.error(message,ex);
            Util.showException(message,ex);
            return;
        }

        //create a new node in the tree for the group
        DefaultMutableTreeNode node = new DefaultMutableTreeNode(new GroupNode(groupName, groupName));
        DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
		DefaultMutableTreeNode root = (DefaultMutableTreeNode)model.getRoot();
		root.add(node);

        //repaint the tree
        model.reload(root);
    }
}
