package org.vovida.prov.group.task;

import org.vovida.prov.util.task.Task;
import org.vovida.prov.util.task.TaskListener;
import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.util.FieldNames;
import org.vovida.prov.util.logging.Util;
import org.vovida.prov.util.ParseUtils;
import org.vovida.prov.group.GroupNode;

import java.util.Hashtable;
import java.util.Enumeration;

import javax.swing.tree.DefaultMutableTreeNode;

import org.apache.log4j.Category;



/**
 * Deletes a group from the provisioning server.<P>
 * Gets a list of all the accounts that belong to this group, removes this group
 * from their list of member groups and writes them all back. Then gets the list
 * of defined groups for the administrator, removes the given group from the
 * list and writes back the administrator account.<p>
 * Does not perform any gui operations.
 */
public class TaskDeleteGroup extends Task
{
    private static Category logCategory = Category.getInstance(TaskLoadGroupMembers.class.getName());

    private String accountIdentifier;
    private ProvisioningServerWrapper server;
    private String parentDomain;
    private String childDomain;
    private DefaultMutableTreeNode parentGroup;


	/**
     * Find all the accounts that belong to the given group. For each account,
     * get the list of groups and remove the given group from the list, then
     * write back the new list.
     */
    public TaskDeleteGroup(String aParentDomain, String parentAcctId,
        String aChildDomain, TaskListener listener,
        ProvisioningServerWrapper psInt, DefaultMutableTreeNode parent)
    {
        super (listener);
        accountIdentifier = parentAcctId;
        server = psInt;
        parentDomain = parentDomain;
        childDomain = aChildDomain;
        parentGroup = parent;
    }

    public void doWork() throws Exception
    {
        String group = ((GroupNode)parentGroup.getUserObject()).toString();

        //find all the accounts that belong to this group and their list of groups
        try
        {
	      	Hashtable [] results = server.get(childDomain,"(&("+
                FieldNames.userIdConlumnName+"=\""+accountIdentifier+".*\")("+
                FieldNames.groupColumnName+"=\"*"+group+"*\"))",
                new String[] {FieldNames.userIdConlumnName,FieldNames.groupColumnName});

			if (results == null)
			{
                logCategory.info("Group \"" + group + "\" did not contain any accounts.");
                return;
			}

	        for (int i=0; i<results.length; i++)
	        {
		        //get the list of groups and remove this group from the list
				String listOfGroups = (String)results[i].get(FieldNames.groupColumnName);
                String newListOfGroups = ParseUtils.removeStringFromList(group,listOfGroups);

                //write back the new list of groups for this account
                try
                {
					server.setFields(childDomain,
	                    (String)results[i].get(FieldNames.userIdConlumnName),
	                    new String [] {FieldNames.groupColumnName},
	                    new String [] {newListOfGroups});
                }
                catch (Exception exc)
                {
                    String message = "Could not write back list of groups for acount \"" + results[i].get(FieldNames.userIdConlumnName) + "\".";
                    logCategory.error(message,exc);
                    Util.showException(message,exc);
                    //continue
                }
	        }
        }
        catch (Exception ex)
        {
            String message = "Could not find out list of groups for member accounts.";
            logCategory.error(message,ex);
            Util.showException(message,ex);
            throw ex;
        }
    }
}
