package org.vovida.prov.group.task;

import org.vovida.prov.util.task.Task;
import org.vovida.prov.util.task.TaskListener;
import org.vovida.prov.group.*;
import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.util.assert.*;
import org.vovida.prov.util.logging.Util;

import javax.swing.tree.DefaultMutableTreeNode;

import org.apache.log4j.Category;




/**
 * A task which runs in its own thread and load the list of members for a
 * particular group and (administrator) account. <p>
 * This is so that the gui event processing thread is not tied up if loading
 * the group members from the provisioning server takes a long time.
 */
public class TaskLoadGroupMembers extends Task
{
    private static Category logCategory = Category.getInstance(TaskLoadGroupMembers.class.getName());

    private DefaultMutableTreeNode group;
    private String accountIdentifier;
    private ProvisioningServerWrapper server;

    /**
     * The domain which will be searched for accounts which belong to the given
     * group.
     */
    private String domain;


    public TaskLoadGroupMembers(DefaultMutableTreeNode aGroup, String aDomain,
        String acctId, TaskListener listener, ProvisioningServerWrapper psInt)
    {
        super (listener);
        group = aGroup;
        accountIdentifier = acctId;
        server = psInt;
        domain = aDomain;
    }

    public void doWork() throws Exception
    {
        Assert.assert("Tree node has a GroupNode", group.getUserObject() != null);
        //get the list of members for this group from the provisioning server
        GroupNode node = (GroupNode)group.getUserObject();
        try
        {
	        final String [] members = server.getGroupMembers(domain,
                accountIdentifier, node.getGroupName());

            if (logCategory.isDebugEnabled())
            {
                String listOfGroups = "";
                for (int i=0; i<members.length; i++)
                {
                    listOfGroups += members[i] + " ";
                }
                logCategory.debug("Members of group " + node.toString() + " are " + listOfGroups);
            }

	        //add them to the tree
	        for (int i=0; i<members.length; i++)
	        {
				GroupMemberNode memberNode = new GroupMemberNode(members[i]);
	            DefaultMutableTreeNode member = new DefaultMutableTreeNode(memberNode,false);
	            group.add(member);
	        }
	
	        saveReturnValue(group);
        }
        catch (Exception ex)
        {
            String message =  "Could not retrieve list of group members for account "
                + accountIdentifier + " and group " + node.toString();
			logCategory.error(message, ex);
			Util.showException(message,ex);
            throw ex;
        }
    }
}
