/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.navigation;

import javax.swing.JPanel;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;
import javax.swing.Box;
import javax.swing.JOptionPane;
import java.awt.FlowLayout;
import java.net.URL;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Graphics;
import java.io.IOException;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import org.apache.log4j.Category;

import org.vovida.prov.psLib.ProvisioningInterface;
import org.vovida.prov.psLib.OperationException;
import org.vovida.prov.util.XMLProcessor;

/**
 * This Singleton class provides the Radio Buttons for navigating
 * between top-level screens.
 * 
 * @author Barbara Samson
 */
public class NavBar extends JPanel implements ConfigConstants
{

  /**
   * @link aggregationByValue
   */
  private ButtonGroup myButtonGroup;

  /**
   * @clientRole singleton
   */
  private static NavBar myInstance;

  private static String myLogoFilename = null;
  private static Image myLogoImage = null;
  private static int myLogoHeight = 50;
  private static int myLogoWidth = 100;

  private Box myBox;

  private boolean myFirstButtonAdded = false;

  /**
   * for logging
   */
  private static Category logCategory = 
    Category.getInstance(NavBar.class.getName());

  private NavBar()
  {
    ProvisioningInterface ps = ProvisioningInterface.getInstance();
    String xmlString = null;

    try
    {
      xmlString = ps.getXMLFile(CONFIG, NAVBARLOGO_XML);
    }
    catch (OperationException e1)
    {
      JOptionPane.showMessageDialog(this, 
              "Error in NavBar logo file: " + e1.getMessage(), 
              "OperationException", JOptionPane.ERROR_MESSAGE);

      return;
    }
    catch (IOException e2)
    {
      JOptionPane.showMessageDialog(this, 
              "Error getting NavBar logo file.\n" 
              + "Check connection to PServer\n" + e2.getMessage(), 
              "IOException", JOptionPane.ERROR_MESSAGE);

      return;
    }

    try
    {
      XMLProcessor.doSAXParse(xmlString, new DefaultHandler()
      {
        public void startElement(String ignore1, String ignore2, 
                String rawName, Attributes attrs)
        {
          if (rawName.equals(NAVBARLOGO))
          {
            myLogoFilename = attrs.getValue(NAME);

            String height = attrs.getValue(ConfigConstants.HEIGHT);
            String width = attrs.getValue(ConfigConstants.WIDTH);

            try
            {
              myLogoHeight = Integer.parseInt(height);
              myLogoWidth = Integer.parseInt(width);
            }
            catch (NumberFormatException e)
            {
              logCategory.error(e.getMessage());
            }
          }
        }

      });

      URL imageURL = null;

      if ((myLogoFilename != null) && (!myLogoFilename.equals("")))
      {
        imageURL = NavBar.class.getResource("resources/" + myLogoFilename);
      }

      if (imageURL != null)
      {
        myLogoImage = getToolkit().createImage(imageURL);
      }

    }
    catch (SAXException e)
    {
      e.printStackTrace();
    }

    setLayout(new FlowLayout(FlowLayout.LEFT));
    logCategory.info("myLogo height: " + myLogoHeight + ", width: " 
                     + myLogoWidth);

    myBox = Box.createVerticalBox();

    myBox.add(Box.createVerticalStrut(myLogoHeight));
    add(myBox);
    setPreferredSize(new Dimension(myLogoWidth, 250));

    myButtonGroup = new ButtonGroup();
  }

  public static NavBar getInstance()
  {
    if (myInstance == null)
    {
      myInstance = new NavBar();
    }

    return myInstance;
  }

  public void add(String name, JRadioButton button)
  {
    myButtonGroup.add(button);
    myBox.add(Box.createVerticalStrut(20));
    myBox.add(button);

    if (!myFirstButtonAdded)
    {
      myButtonGroup.setSelected(button.getModel(), true);

      myFirstButtonAdded = true;
    }
  }

  public void paintComponent(Graphics g)
  {
    super.paintComponent(g);

    if (myLogoImage != null)
    {
      g.drawImage(myLogoImage, 0, 0, this);
    }
  }

}




