/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.server;

import java.util.Hashtable;
import java.io.IOException;
import javax.swing.JDialog;
import javax.swing.JFrame;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.FlowLayout;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JScrollPane;

import org.apache.log4j.Category;

import org.vovida.prov.widget.LayoutPanel;
import org.vovida.prov.dynamicGui.ProvisionableInstance;
import org.vovida.prov.psLib.ProvisioningInterface;
import org.vovida.prov.psLib.OperationException;
import org.vovida.prov.content.InvalidDataException;

/**
 * @author Barbara Samson
 */
public class EditServerDialog extends JDialog implements ServerConstants
{
  private JPanel myPanel;
  private JButton myCancelButton;
  private JButton myOKButton;
  private JButton myResetButton;

  /**
   * This is used to remember the hostport key saved to database in case
   * that field changes during editing.
   */
  private String myHostPort = "";

  /**
   * for logging
   */
  private static Category logCategory = 
    Category.getInstance(EditServerDialog.class.getName());

  public EditServerDialog(JFrame parent, String title, String tooltip)
  {
    super(parent);

    setUpDialog(title, tooltip);
  }

  public EditServerDialog(JDialog parent, String title, String tooltip)
  {
    super(parent);

    setUpDialog(title, tooltip);
  }

  /**
   * Called from the Edit button of TableDialog.
   */
  public void setHostPort(String hostport)
  {
    myHostPort = hostport;
  }

  private void setUpDialog(String title, String tooltip)
  {
    JPanel titlePanel = new JPanel();

    titlePanel.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));

    if ((tooltip != null) && (!tooltip.equals("")))
    {
      titlePanel.setToolTipText(tooltip);
    }

    JLabel titleLabel = new JLabel(title);

    titleLabel.setForeground(Color.black);
    titlePanel.add(titleLabel);
    getContentPane().add(titlePanel, BorderLayout.NORTH);

    JScrollPane scrollPane = new JScrollPane();

    getContentPane().add(scrollPane, BorderLayout.CENTER);

    JPanel buttonPanel = new JPanel();
    JPanel gridPanel = new JPanel();

    gridPanel.setLayout(new GridLayout(1, 3));
    buttonPanel.add(gridPanel);

    myOKButton = new JButton("OK");
    myCancelButton = new JButton("Cancel");
    myResetButton = new JButton("Reset");

    gridPanel.add(myOKButton);
    gridPanel.add(myCancelButton);
    gridPanel.add(myResetButton);
    getContentPane().add(buttonPanel, BorderLayout.SOUTH);

    LayoutPanel layoutPanel = new LayoutPanel(null);

    myPanel = layoutPanel.getContainer();

    scrollPane.getViewport().add(myPanel);
  }

  public JPanel getPanel()
  {
    return myPanel;
  }

  protected void createListeners(final ProvisionableInstance instance, 
          boolean isNewInstance)
  {
    myCancelButton.addActionListener(new CancelButtonListener(instance));
    myResetButton.addActionListener(new ResetButtonListener(instance));
    myOKButton.addActionListener(new OKButtonListener(instance, 
            isNewInstance));
  }

  protected void addOKListener(ActionListener listener)
  {
    myOKButton.addActionListener(listener);
  }

  /**
   * The ActionListener for the Cancel Button
   */
  protected class CancelButtonListener implements ActionListener
  {
    private ProvisionableInstance myInstance;

    CancelButtonListener(ProvisionableInstance instance)
    {
      myInstance = instance;
    }

    public void actionPerformed(ActionEvent e)
    {
      myInstance.cancel();
      setVisible(false);
    }

  }


  /**
   * The ActionListener for the Reset Button
   */
  protected class ResetButtonListener implements ActionListener
  {
    private ProvisionableInstance myInstance;

    ResetButtonListener(ProvisionableInstance instance)
    {
      myInstance = instance;
    }

    public void actionPerformed(ActionEvent e)
    {
      myInstance.reset();
      myInstance.clearKeyValue();
    }

  }


  /**
   * The ActionListener for the OK Button
   */
  protected class OKButtonListener implements ActionListener
  {
    private ProvisionableInstance myInstance;
    private boolean myIsNewInstance;

    OKButtonListener(ProvisionableInstance instance, boolean isNewInstance)
    {
      myInstance = instance;
      myIsNewInstance = isNewInstance;
    }

    public void actionPerformed(ActionEvent e)
    {
      try
      {
        Hashtable data = myInstance.getData();
        ProvisioningInterface ps = ProvisioningInterface.getInstance();

        if (myIsNewInstance)
        {
          if (ps.addRecordInDomain(myInstance.getName(), data) == false)
          {
            JOptionPane.showMessageDialog(EditServerDialog.this, 
                    "Add Record failed");
          }
        }
        else
        {
          String hostport;

          if (myHostPort.equals(""))
          {
            hostport = (String) data.get(HOSTPORT);
          }
          else
          {
            hostport = myHostPort;
          }

          String filter = new String("(" + HOSTPORT + "=\"" + hostport 
                  + "\")");

          if (ps.setRecordsInDomain(myInstance.getName(), filter, data) 
              == false)
          {
            JOptionPane.showMessageDialog(EditServerDialog.this, 
                    "Set Record failed");
          }
        }

        setVisible(false);
      }
      catch (InvalidDataException e1)
      {

        // a pop-up should already explain the problem
      }
      catch (IOException e2)
      {
        e2.printStackTrace();
      }
      catch (OperationException e3)
      {
        JOptionPane.showMessageDialog(EditServerDialog.this, e3.getMessage());
        e3.printStackTrace();
      }
    }

  }

}




