/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.server;

import java.util.Vector;
import javax.swing.table.AbstractTableModel;

import org.apache.log4j.Category;

/**
 * @author Barbara Samson
 */
public class ServerTableModel extends AbstractTableModel
{

  private static final int STRING = 0;
  private static final int INT = 1;

  private static Category logCategory = 
    Category.getInstance(ServerTableModel.class.getName());

  private String[] myColumns = new String[]
  {
    "Host", "Port", "Type"
  };

  private int[] myColumnTypes = new int[]
  {
    STRING, INT, STRING
  };

  private Vector myData = new Vector();

  public ServerTableModel()
  {
    myData.clear();
  }

  public void clear()
  {
    myData = new Vector();
  }

  public void addRows(int numRows)
  {
    for (int i = 0; i < numRows; i++)
    {
      myData.add(new String[3]);
    }

    fireTableDataChanged();
  }

  public void setValueAt(Object value, int row, int column)
  {
    String[] rowArray = (String[]) myData.get(row);

    rowArray[column] = (String) value;

    fireTableDataChanged();
  }

  public Object getValueAt(int row, int column)
  {
    String[] rowData = (String[]) myData.elementAt(row);

    if ((rowData[column] == null) || (rowData[column].equals("")))
    {
      return null;
    }

    switch (myColumnTypes[column])
    {

      case INT:
      {
        return new Integer(rowData[column]);
      }

      default:
      {
        return rowData[column];
      }
    }
  }

  public int getColumnCount()
  {
    return myColumns.length;
  }

  public int getRowCount()
  {
    return myData.size();
  }

  public String getColumnName(int position)
  {
    return myColumns[position];
  }

  public Class getColumnClass(int aColumn)
  {
    switch (myColumnTypes[aColumn])
    {

      case INT:
      {
        return Integer.class;
      }

      default:
      {
        return String.class;
      }
    }
  }

  public boolean isCellEditable(int row, int column)
  {
    return false;
  }


}




