/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.server;

import java.util.Vector;
import java.util.Hashtable;
import java.io.InputStream;
import java.io.IOException;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import org.xml.sax.Attributes;

import org.apache.log4j.Category;

import org.vovida.prov.psLib.ProvisioningInterface;
import org.vovida.prov.psLib.PSDBResult;
import org.vovida.prov.psLib.OperationException;
import org.vovida.prov.dynamicGui.ProvisionableInstance;
import org.vovida.prov.dynamicGui.ProvisionableType;
import org.vovida.prov.dynamicGui.TypeBuilder;
import org.vovida.prov.content.InvalidDataException;

/**
 * @author Barbara Samson
 */
public class TableDialog extends JDialog implements ServerConstants
{
  private static final int HOST = 0;
  private static final int PORT = 1;
  private static final int TYPE = 2;

  /**
   * @link aggregationByValue
   */
  private JTable myTable;

  /**
   * @link aggregationByValue
   */
  private ServerTableModel myModel;

  /**
   * A TableModel that sits between the Table and its true model.
   */
  private TableSorter myProxyModel;

  private Vector myColumns;
  private JFrame myParentFrame;

  /**
   * Set of server types that this table is built of.
   */
  private String[] myServerTypes;

  /**
   * for logging
   */
  private static Category logCategory = 
    Category.getInstance(TableDialog.class.getName());

  // constants used by delete confirm dialog
  private final static int YES = 0;
  private final static int NO = 1;
  private final static int YES_TO_ALL = 2;

  public TableDialog(JFrame parentFrame)
  {
    super(parentFrame);

    myParentFrame = parentFrame;

    // set up buttons
    JButton newButton = new JButton("New");

    newButton.setToolTipText("Create new server");
    newButton.addActionListener(new NewServerListener());

    JButton editButton = new JButton("Edit");

    editButton.setToolTipText("Edit selected server(s)");
    editButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        try
        {
          editSelectedRows();
        }
        catch (IOException e1)
        {
          e1.printStackTrace();
        }
        catch (OperationException e2)
        {
          e2.printStackTrace();
        }
      }

    });

    JButton copyButton = new JButton("Copy");

    copyButton.setToolTipText("Copy selected server(s)");
    copyButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        try
        {
          copySelectedRows();
        }
        catch (IOException e1)
        {
          e1.printStackTrace();
        }
        catch (OperationException e2)
        {
          e2.printStackTrace();
        }
      }

    });


    JButton deleteButton = new JButton("Delete");

    deleteButton.setToolTipText("Delete selected server(s)");
    deleteButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        try
        {
          deleteSelectedRows();
        }
        catch (IOException e1)
        {
          e1.printStackTrace();
        }
        catch (OperationException e2)
        {
          e2.printStackTrace();
        }
      }

    });

    JButton closeButton = new JButton("Close");

    closeButton.setToolTipText("Close window");
    closeButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        setVisible(false);
      }

    });

    JPanel buttonPanel = new JPanel();
    JPanel gridPanel = new JPanel();

    gridPanel.setLayout(new GridLayout(1, 5));
    gridPanel.add(newButton);
    gridPanel.add(editButton);
    gridPanel.add(copyButton);
    gridPanel.add(deleteButton);
    gridPanel.add(closeButton);
    buttonPanel.add(gridPanel);
    getContentPane().add(buttonPanel, BorderLayout.SOUTH);

    // set up the table
    JScrollPane scrollPane = new JScrollPane();

    // this is indirectly the model of myTable
    myModel = new ServerTableModel();

    myProxyModel = new TableSorter(myModel);
    myTable = new JTable(myProxyModel);

    myProxyModel.addMouseListenerToHeaderInTable(myTable);

    myTable.createDefaultColumnsFromModel();

    TableColumnModel columnModel = myTable.getColumnModel();

    columnModel.setColumnMargin(5);

    TableColumn portColumn = myTable.getColumnModel().getColumn(PORT);

    portColumn.setPreferredWidth(70);
    portColumn.setMaxWidth(70);

    myTable.setToolTipText("Click on column header to sort ascending. " 
            + "Shift-click to sort descending.");

    getContentPane().add(scrollPane, BorderLayout.CENTER);
    scrollPane.getViewport().add(myTable);

    addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
      {
        setVisible(false);
      }

    });
  }

  /**
   * Rebuild a table of server types.
   */
  public void buildTable()
  {
    if (myServerTypes != null)
    {
      buildTable(myServerTypes);
    }
    else
    {
      logCategory.error("myServerTypes is null");
    }
  }

  /**
   * Build a table of given serverTypes.
   */
  public void buildTable(String[] serverTypes)
  {
    myServerTypes = serverTypes;

    ProvisioningInterface ps = ProvisioningInterface.getInstance();
    String filter = "(" + HOSTPORT + "=\"*\")";
    String[] fieldList = new String[]
    {
      HOSTPORT
    };

    myModel.clear();
    myProxyModel.reallocateIndexes();
    logCategory.debug("building table of servers");

    // keeps track of position to add next row.
    int rowPosition = 0;

    for (int i = 0; i < serverTypes.length; i++)
    {
      if (logCategory.isDebugEnabled())
      {
        logCategory.debug("querying PServer for server type " 
                          + serverTypes[i]);
      }

      PSDBResult result = null;

      try
      {
        result = ps.getRecordsInDomain(serverTypes[i], filter, fieldList);
      }
      catch (IOException e1)
      {
        JOptionPane.showMessageDialog(this, 
                e1.getMessage() + "\nCheck to make sure PServer is running.", 
                "IO Exception", JOptionPane.ERROR_MESSAGE);
        e1.printStackTrace();

        continue;
      }
      catch (OperationException e2)
      {
        JOptionPane.showMessageDialog(this, e2.getMessage(), 
                "Operation Exception", JOptionPane.ERROR_MESSAGE);
        e2.printStackTrace();

        continue;
      }

      if (result == null)
      {
        logCategory.error("null result from query of " + serverTypes[i]);

        continue;
      }

      int numRecords = result.getNumRecords();

      if (logCategory.isDebugEnabled())
      {
        logCategory.debug(serverTypes[i] + " has " + numRecords 
                          + " instances");
      }

      myModel.addRows(numRecords);

      for (int j = 0; j < numRecords; j++)
      {
        String[] row = new String[3];
        String hostport = result.getFieldValue(j, HOSTPORT);

        setHostPortValue(row, hostport);

        row[TYPE] = serverTypes[i];

        addRow(row, rowPosition++);

        if (logCategory.isDebugEnabled())
        {
          logCategory.debug("added server " + hostport + " of type " 
                            + serverTypes[i]);
        }
      }
    }

    myModel.fireTableDataChanged();
  }

  public void showWindow()
  {

    setSize(500, 300);
    setVisible(true);
  }

  private void editSelectedRows() throws IOException, OperationException
  {
    ProvisioningInterface ps = ProvisioningInterface.getInstance();
    int[] selectedRows = myTable.getSelectedRows();

    if (selectedRows.length < 1)
    {
      JOptionPane.showMessageDialog(this, 
              "Select one or more servers to edit", "No Selection", 
              JOptionPane.WARNING_MESSAGE);

      return;
    }

    for (int i = 0; i < selectedRows.length; i++)
    {
      int position = selectedRows[i];
      String hostport = getHostPortValue(position);
      String typeName = (String) myProxyModel.getValueAt(position, TYPE);
      String filter = "(" + HOSTPORT + "=\"" + hostport + "\")";
      ProvisionableType type = TypeBuilder.getType(typeName);
      String label = type.getLabel();
      String tooltip = type.getTooltip();
      EditServerDialog editDialog = new EditServerDialog(this, 
              "Edit " + label, tooltip);

      editDialog.setHostPort(hostport);

      ProvisionableInstance instance = 
        type.buildInstance(editDialog.getPanel());

      // we need the OKListener in case host:port is changed.
      editDialog.addOKListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          buildTable();
        }

      });

      // the false argument means that this instance is not new.
      editDialog.createListeners(instance, false);

      String[] fieldNames = type.getDBFields();
      PSDBResult result = ps.getRecordsInDomain(typeName, filter, fieldNames);

      if (result.getNumRecords() < 1)
      {
        JOptionPane.showMessageDialog(this, 
                "Database query " + filter + " failed for domain " 
                + typeName, "Bad Query", +JOptionPane.ERROR_MESSAGE);

        continue;
      }

      Hashtable data = result.getRow(0);

      try
      {
        instance.setData(data);
      }
      catch (InvalidDataException e)
      {
        e.printStackTrace();
      }

      editDialog.pack();
      editDialog.setVisible(true);
    }

  }

  private void copySelectedRows() throws IOException, OperationException
  {
    ProvisioningInterface ps = ProvisioningInterface.getInstance();
    int[] selectedRows = myTable.getSelectedRows();

    if (selectedRows.length < 1)
    {
      JOptionPane.showMessageDialog(this, 
              "Select one or more servers to copy", "No Selection", 
              JOptionPane.WARNING_MESSAGE);

      return;
    }

    for (int i = 0; i < selectedRows.length; i++)
    {
      int position = selectedRows[i];
      String hostport = getHostPortValue(position);
      String typeName = (String) myProxyModel.getValueAt(position, TYPE);
      String filter = "(" + HOSTPORT + "=\"" + hostport + "\")";
      ProvisionableType type = TypeBuilder.getType(typeName);
      String label = type.getLabel();
      String tooltip = type.getTooltip();
      EditServerDialog copyDialog = new EditServerDialog(this, 
              "New " + label, tooltip);

      ProvisionableInstance instance = 
        type.buildInstance(copyDialog.getPanel());

      // add this action listener first so that it will be invoked last.
      copyDialog.addOKListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          buildTable();
          myProxyModel.sortByLastColumn();
        }

      });

      // the true argument means that this instance is new.
      copyDialog.createListeners(instance, true);

      String[] fieldNames = type.getDBFields();
      PSDBResult result = ps.getRecordsInDomain(typeName, filter, fieldNames);

      if (result.getNumRecords() < 1)
      {
        JOptionPane.showMessageDialog(this, 
                "Database query " + filter + " failed for domain " 
                + typeName, "Bad Query", +JOptionPane.ERROR_MESSAGE);

        continue;
      }

      Hashtable data = result.getRow(0);

      data.remove(HOSTPORT);

      try
      {
        instance.setData(data);
      }
      catch (InvalidDataException e)
      {
        e.printStackTrace();
      }

      copyDialog.pack();
      copyDialog.setVisible(true);
    }

  }


  private void deleteSelectedRows() throws IOException, OperationException
  {
    ProvisioningInterface ps = ProvisioningInterface.getInstance();
    boolean deleteAll = false;
    boolean deleteThis = false;
    int[] selectedRows = myTable.getSelectedRows();

    if (selectedRows.length < 1)
    {
      JOptionPane.showMessageDialog(this, 
              "Select one or more servers to delete", "No Selection", 
              JOptionPane.WARNING_MESSAGE);

      return;
    }

    // The user will only have yes/no to select from if only
    // one server is selected for deletion.
    // If more than one is selected, the user will select
    // from yes/no/delete all.
    Object[] options;

    if (selectedRows.length == 1)
    {
      options = new Object[]
      {
        "yes", "no"
      };
    }
    else
    {
      options = new Object[]
      {
        "yes", "no", "yes to all"
      };
    }

    for (int i = 0; i < selectedRows.length; i++)
    {
      int position = selectedRows[i];
      String hostport = getHostPortValue(position);
      String type = (String) myProxyModel.getValueAt(position, TYPE);
      String filter = "(" + HOSTPORT + "=\"" + hostport + "\")";

      if (!deleteAll)
      {

        int confirmResult = JOptionPane.showOptionDialog(this, 
                "delete " + hostport + "?", "Confirm Delete", 
                JOptionPane.DEFAULT_OPTION, JOptionPane.QUESTION_MESSAGE, 
                null, options, options[0]);

        switch (confirmResult)
        {

          case YES_TO_ALL:
          {
            deleteAll = true;
            deleteThis = true;

            break;
          }

          case YES:
          {
            deleteThis = true;

            break;
          }

          default:
          {
            JOptionPane.showMessageDialog(this, "delete cancelled");

            deleteThis = false;

            return;
          }
        }
      }

      if (deleteThis)
      {
        boolean deleteSuccess = ps.deleteRecordsInDomain(type, filter);

        if (!deleteSuccess)
        {
          JOptionPane.showMessageDialog(this, "delete failed");
        }
      }
    }

    buildTable();
    myProxyModel.sortByLastColumn();

  }

  public String getHostPortValue(int row)
  {
    String host = (String) myProxyModel.getValueAt(row, HOST);
    Integer port = (Integer) myProxyModel.getValueAt(row, PORT);

    if ((host == null) || (host.equals("")))
    {
      return (port == null) ? "" : (":" + port.toString());
    }

    if ((port == null) || (port.equals("")))
    {
      return host;
    }

    return (host + ":" + port.toString());
  }

  /**
   * Take a value that looks like host:port and separate it
   * into host and port values.
   */
  private void setHostPortValue(String[] row, String hostport)
  {
    int separatorPos = hostport.indexOf(":");

    // if there is no separator
    // or if the separator is at the end of the string,
    // then assume the whole string is the hostname
    if ((separatorPos == -1) || (separatorPos == hostport.length() - 1))
    {
      row[HOST] = hostport;

      return;
    }

    row[HOST] = hostport.substring(0, separatorPos);
    row[PORT] = hostport.substring(separatorPos + 1, hostport.length());
  }


  protected void addRow(String[] newRow, int rowPosition)
  {
    myProxyModel.setValueAt(newRow[HOST], rowPosition, HOST);
    myProxyModel.setValueAt(newRow[PORT], rowPosition, PORT);
    myProxyModel.setValueAt(newRow[TYPE], rowPosition, TYPE);
  }

  /**
   * The ActionListener for the New button.
   */
  protected class NewServerListener implements ActionListener
  {

    /**
     * Create a Dynamic GUI screen to fill in this server's parameters.
     * If more than one type of server is available, start by making the
     * user choose one.
     */
    public void actionPerformed(ActionEvent e)
    {
      if ((myServerTypes == null) || (myServerTypes.length < 1))
      {
        return;
      }

      int selectedRow = myTable.getSelectedRow();
      String selectedType;

      if (selectedRow == -1)
      {
        selectedType = myServerTypes[0];
      }
      else
      {
        selectedType = (String) myProxyModel.getValueAt(selectedRow, TYPE);
      }

      // Present the user with a drop-down list so that he
      // can select a type other than the one currently selected.
      // Only do this if there is more than one type to choose from.
      if (myServerTypes.length > 1)
      {
        selectedType = (String) JOptionPane.showInputDialog(null, 
                "Choose one", "Choose a Server Type", 
                JOptionPane.INFORMATION_MESSAGE, null, myServerTypes, 
                selectedType);
      }

      ProvisionableType type = TypeBuilder.getType(selectedType);

      if (type == null)
      {
        return;
      }

      String label = type.getLabel();
      String tooltip = type.getTooltip();
      EditServerDialog newDialog = new EditServerDialog(myParentFrame, label, 
              tooltip);
      ProvisionableInstance instance = 
        type.buildInstance(newDialog.getPanel());

      newDialog.addOKListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          buildTable();
          myProxyModel.sortByLastColumn();
        }

      });

      // the true argument means that this is a new instance.
      newDialog.createListeners(instance, true);

      newDialog.pack();
      newDialog.setVisible(true);

    }

  }

}




