package org.vovida.prov.table;

import org.vovida.prov.util.logging.LogScreen;
import org.vovida.prov.util.logging.Util;
import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.util.FieldNames;
import org.vovida.prov.userEdit.UIRepository;
import org.vovida.prov.psLib.OperationException;
import org.vovida.prov.group.GroupScreen;

import javax.swing.Box;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JScrollPane;

import java.awt.BorderLayout;
import java.awt.Window;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.io.IOException;

import org.apache.log4j.Category;



/**
 * This is the main screen for an account administrator. <p>
 * It combines the table screen, the group screen and a log screen.
 */
public class AdministratorPanel extends JPanel
{
    private String userId;
    private String parentDomain;
    private String childCategory;
    private ProvisioningServerWrapper server;
    private JTabbedPane tabs;
    private TableScreen screen;
    private Window parentComponent;

    private static Category logCategory = Category.getInstance(AdministratorPanel.class.getName());

    public AdministratorPanel(String aUserId, String parent, String category,
        ProvisioningServerWrapper aServer, Window parentWindow)
    {
        userId = aUserId;
        parentDomain = parent;
        childCategory = category;
        server = aServer;
        parentComponent = parentWindow;

        setLayout(new BorderLayout());
        tabs = new JTabbedPane();
        tabs.setTabPlacement(tabs.BOTTOM);
        add(tabs,BorderLayout.CENTER);

        UIRepository.setChildCategory(childCategory);


//        tabs.addTab("Log", null, createLogPanel(), "View application log");
    }

    public void init()
    {
		//find the child domain for the current administrator
        String childDomain = "";

        try
        {
        	childDomain = (String)server.get(parentDomain,
                "("+FieldNames.userIdConlumnName+"=\""+userId+"\")",
                new String[]{FieldNames.CHILD_DOMAIN})[0].get(FieldNames.CHILD_DOMAIN);
        }
        catch (Exception ex)
        {
            String message = "Could not determine the domain of the child accounts for the administrator \"" + userId + "\".";
            logCategory.error(message,ex);
            Util.showException(message,ex);
            return;
        }

		//create tabs for the group screen and table screen
/**
        tabs.insertTab("Groups", null, createGroupPanel(userId,parentDomain,
            childDomain,server), "View accounts organized by group",0);
**/

        tabs.insertTab("Accounts",null, createAccountTablePanel(userId,
            parentDomain,childDomain,server),"View table of accounts", 0);

        //start loading data into the table screen
		screen.startLoadingData();
    }


    private JPanel createGroupPanel(String userId, String parentDomain, String childDomain,
        ProvisioningServerWrapper server)
    {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());

		GroupScreen group = new GroupScreen(parentDomain,childDomain,userId,server);
        panel.add(group,BorderLayout.CENTER);

        return panel;
    }

    private JPanel createAccountTablePanel(String userId, String parentDomain,
        String childDomain, ProvisioningServerWrapper aServer)
    {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
                                                
		String [] columnId2 = new String [] {};
        String [] columnId = new String [] {};
        String [] allColumns = new String [] {};

        try
        {
			columnId = aServer.getListOfColumns(childDomain); //the columns which contain additional (non-feature) information
            allColumns = columnId;
	        columnId2 = aServer.getListOfFeatures(parentDomain,userId); //the columns which correspond to features
	        allColumns = new String [columnId.length + columnId2.length]; //one vector which contains all the column identifers combined
	        int i=0;
	        for (i=0; i<columnId.length; i++)
	        {
	            allColumns[i] = columnId[i];
	        }
	        for (; i<allColumns.length; i++)
	        {
	            allColumns[i] = columnId2[i-columnId.length];
	        }
		}
        catch (Exception ex)
        {
            String msg = "Could not find the list of column identifiers.";
            logCategory.error(msg + ex);
            Util.showException(msg,ex);
        }

        screen = new TableScreen(userId,parentDomain,childDomain,allColumns,
            aServer,parentComponent);

        //now mark the feature columns as non-domain
        NamedTableColumn [] columns = screen.getTableModel().getColumns();
        for (int i=0; i<columnId2.length; i++)
        {
            for (int j=0; j<columns.length; j++)
            {
                if (columns[j].getColumnId().equals(columnId2[i]))
                {
					columns[j].setIsDomainColumn(false);
                    break;
                }
            }
        }

        //now get the xml description for the table to set the column "nice"
        //names for columns which do not display features
        try
        {
			String xml = server.getXMLDescription(childCategory,childDomain);
            screen.setColumnDescriptions(xml);
        }
        catch (Exception ex)
        {
            String message = "Could not retrieve xml description for the \"" +
                childDomain +
                "\" domain. Cannot determine nice names for columns.";
            logCategory.warn(message,ex);
            Util.showException(message,ex);
            //continue
        }

        //get the xml descriptions for all the columns which display features
        //to set the "nice" names for these columns as well
        if ( (columnId2!=null) && (columnId2.length > 0) )
        {
            for (int i=0; i<columnId2.length; i++)
            {
                try
                {
                    String xml = server.getFeatureXMLDescription(columnId2[i]);
                    screen.setColumnDescriptions(xml);
                }
                catch (Exception ex)
                {
		            String message = "Could not retrieve xml description for the \"" +
		                columnId2[i] +
		                "\" domain. Cannot determine nice names for columns.";
                    logCategory.warn(message,ex);
                    System.out.println(message);
                    ex.printStackTrace();
                }
            }
        }

        screen.sizeColumnsToFitData();
        panel.add(screen,BorderLayout.CENTER);

        JMenuBar menuBar = new JMenuBar();

        JMenu menu = screen.getTable().getMenu();
        menu.addSeparator();
        JMenuItem item = new JMenuItem("Resize columns");
        item.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent event)
            {
                screen.sizeColumnsToFitData();
            }
        });
        menu.add(item);
        menuBar.add(menu);

        item = new JMenuItem("Reload!");
        item.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent event)
            {
                reload();
            }
        });
        menuBar.add(item);

        menuBar.add(Box.createHorizontalGlue());
        panel.add(menuBar,BorderLayout.NORTH);

        return panel;
    }


    /**
     * Create a panel that contains a LogScreen in the center and the logScreen's
     * menu at the top. <p>
     */
    private JPanel createLogPanel()
    {
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        LogScreen logScreen = new LogScreen();
        panel.add(logScreen,BorderLayout.CENTER);

        JMenuBar menuBar = new JMenuBar();
        menuBar.add(logScreen.getMenu());
        panel.add(menuBar,BorderLayout.NORTH);

        return panel;
    }


	/**
     * Remove the data that has been stored in the accounts table and cause
     * new data to be loaded. This includes re-reading the account count.
     */
    public void reload()
    {
        clear();
        screen.startLoadingData();
    }

	/**
     * Remove all the data that has been stored in the accounts table.
     */
    public void clear()
    {
        screen.clear();
    }
}
