package org.vovida.prov.table;

import org.vovida.prov.util.RuntimeInfo;
import org.vovida.prov.psLib.ProvisioningInterface;
import org.vovida.prov.table.NamedTableColumn;

import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.ImageIcon;
import javax.swing.JTable;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import java.awt.Component;
import java.net.URL;

import org.apache.log4j.Category;



/**
 * This class should be registered as the renderer component on the table
 * header of the main table. It is responsible for
 */
public class ColumnOrderRenderer extends DefaultTableCellRenderer
{
    private static Category logCategory = Category.getInstance(ColumnOrderRenderer.class.getName());

    /** Icon which indicates ascending sort */
    private static ImageIcon ascending;

    /** Icon which indicates descending sort */
    private static ImageIcon descending;

    /** Knows what order the column is sorted in. Use to select the correct icon */
    private SortRequestListener order;

    /** Need this to get the standard appearance of the headers */
    private static final DefaultTableCellRenderer headerRenderer = (DefaultTableCellRenderer) new JTable().getTableHeader().getDefaultRenderer();


    public ColumnOrderRenderer(SortRequestListener listener)
    {
        order = listener;
        ascending = loadIcon("org/vovida/prov/table/resources/ascending04.gif");
        descending = loadIcon("org/vovida/prov/table/resources/descending04.gif");
    }

    private ImageIcon loadIcon (String path)
    {
		ImageIcon img = null;
		URL url = ClassLoader.getSystemResource(path);
		if (url == null)
		{
            img = new ImageIcon(path);
		}
		else
		{
            img = new ImageIcon(url);
		}

        logCategory.info("Image load status for " + path + " = " +
            RuntimeInfo.getImageLoadStatusString(img.getImageLoadStatus()));

	    return (img);
    }

    public Component getTableCellRendererComponent(JTable table, Object value,
        boolean isSelected, boolean hasFocus, int row, int column)
    {
        DefaultTableCellRenderer renderer = (DefaultTableCellRenderer)headerRenderer.getTableCellRendererComponent(table,value,isSelected,hasFocus,row,column);
        //column here is the index into the rowModel, not the tableModel
        column = table.convertColumnIndexToModel(column);

        //determine what order the column is actually sorted in
        ImageIcon iconToUse;
        if (order.getSortOrder().equals(ProvisioningInterface.SORT_TYPE_ASC))
        {
            iconToUse = ascending;
        }
        else
        {
            iconToUse = descending;
        }

        //if this is a NamedTableColumn, get its tooltip and show it, otherwise
        //clear the tooltip
        if (NamedTableColumn.class.isInstance(table.getColumnModel().getColumn(column)))
        {
            String tip = ((NamedTableColumn)table.getColumnModel().getColumn(column)).getTooltip();
            renderer.setToolTipText(tip);
        }
        else
        {
            renderer.setToolTipText(null);
        }

        //determine what is the heading for the column
        String name = table.getModel().getColumnName(column);

        //set the text and appropriate icon
        renderer.setText(name);
        renderer.setHorizontalTextPosition(SwingConstants.LEFT);

        if (order.getCurrentSortColumn() == column)
        {
            //The table is sorted on this column so display an icon in the header
            renderer.setIcon(iconToUse);
        }
        else
        {
            //The table is sorted on some other column, so don't display an
            //icon in the header of this one.
            renderer.setIcon(null);
        }

        return renderer;
   }
}
