package org.vovida.prov.table;

import javax.swing.table.TableColumn;

/**
 * This class represents a table column along with its name. <p>
 **/
public class NamedTableColumn extends TableColumn
{
    /**
     * The name of this column. <p>
     * This string should be used in the table header when this column is
     * visible.
     **/
    private String columnName = null;


    /**
     * If this column is a column that is actually found in a table, set this
     * to true. Set this to false if this column is a parsed column. The case
     * for this is the various feature columns which
     */
    private boolean isDomainColumn = true;

    /**
     * The identifier of this column. <p>
     * This is different from the column name in that this should be the
     * xml element name used to identify the data represented by this column.
     * This should be the identifier passed to the provisioning server.
     */
    private String columnId = null;

    private String tooltip = null;



    public NamedTableColumn(String id)
    {
        columnId = id;
    }

    /**
     * Set the name of the column. <p>
     */
    public void setColumnName(String name)
    {
    	columnName = name;
    }

    /**
     * @return the name of the column
     */
    public String getColumnName()
    {
        if (columnName == null)
        {
        	return columnId;
        }
        return columnName;
    }

    public String getColumnId()
    {
		return columnId;
    }

    public boolean isDomainColumn()
    {
        return isDomainColumn;
    }

    public void setIsDomainColumn(boolean set)
    {
        isDomainColumn = set;
    }

    public void setTooltip(String tip)
    {
        tooltip = tip;
    }

    public String getTooltip()
    {
        return tooltip;
    }
}
