
package org.vovida.prov.table;

import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.table.TableModel;

import java.awt.Component;

import javax.swing.table.DefaultTableCellRenderer;


/**
 * Used in a table to draw cells whose value is the string "true" or "false"
 * as a correspondingly selected check box. <p>
 * A checkBoxRenderer cannot be used as the default renderer for the table
 * because there may be cases where the table is visible on screen but has
 * no data loaded. In these cases, an empty string will cause an exception
 * in the standard check box renderer. This class will use the check box only
 * for the cases where it finds a recognizable boolean value in the field and
 * a text label for other cases.
 */
public class RowRenderer extends DefaultTableCellRenderer
{
    JCheckBox check = new JCheckBox();

    public RowRenderer()
    {
        check.setHorizontalAlignment(check.CENTER);
    }

    public Component getTableCellRendererComponent(JTable table, Object value,
        boolean isSelected,boolean hasFocus,int row,int column)
    {
		TableModel model = table.getModel();
        Component renderer = this;

        int modelColumn = table.convertColumnIndexToModel(column);

        if ("true".equals(value))
        {
            check.setSelected(true);
            renderer = check;
        }
        else  if ("false".equals(value))
        {
            check.setSelected(false);
            renderer = check;
        }

        setValue(value);
        setFont(table.getFont());
        if (isSelected)
        {
            renderer.setBackground(table.getSelectionBackground());
            renderer.setForeground(table.getSelectionForeground());
        }
        else
        {
            renderer.setBackground(table.getBackground());
            renderer.setForeground(table.getForeground());
        }

        return renderer;
    }
}
