package org.vovida.prov.table.action;

import javax.swing.JDialog;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;

import java.util.Arrays;

import org.vovida.prov.table.TableScreen;
import org.vovida.prov.userEdit.UIRepository;
import org.vovida.prov.userEdit.UserEditorPanel;
import org.vovida.prov.util.FieldNames;
import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.util.logging.Util;

import org.apache.log4j.Category;





public abstract class ActionChangeUsers extends ActionTable
{
    private static Category logCategory = Category.getInstance(ActionEdit.class.getName());

    static protected JDialog userEditor;
    protected String adminName;
    protected String adminDomain;
    protected String userDomain;
	static protected UserEditorPanel panel;
    protected String [] adminFeatures = null;

    public ActionChangeUsers(ProvisioningServerWrapper wrapper, TableScreen screen,
        String actionName, String anAdminName, String anAdminDomain,
        String aUserDomain, Window aParent)
    {
		super(actionName,wrapper,screen,aParent);
        userDomain = aUserDomain;
        adminDomain = anAdminDomain;
        adminName = anAdminName;

		UIRepository.setServer(wrapper);
    }


    protected void setChangedFeaturesInTable(int row)
    {
		String [] selected = panel.getSelectedFeatures();
        String [] changed = panel.getChangedFeatures();

        Arrays.sort(selected);
        Arrays.sort(changed);


        for (int i=0; i<changed.length; i++)
        {
            if (Arrays.binarySearch(selected,changed[i])>=0) //if found
            {
                tableScreen.getTableModel().setValueAt(row,changed[i],"true");
            }
            else
            {
                tableScreen.getTableModel().setValueAt(row,changed[i],null);
            }
        }
    }


	public void actionPerformed(ActionEvent event)
	{
		//find the features which are available to the administrator
        if (adminFeatures == null)
        {
            try
            {
	            adminFeatures = server.getListOfFeatures(adminDomain, adminName);
            }
            catch (Exception ex)
            {
                String message = "Could not determine the list of features for the administrator \""
                    + adminName + "\" in domain \"" + adminDomain + "\".";
                logCategory.error(message,ex);
                Util.showException(message,ex);
                return;
            }
        }

        if (panel == null)
        {
            //create the panel
            userEditor = new JDialog((Frame)parent,"Edit user account",true);
            panel = new UserEditorPanel(adminFeatures, userDomain, server, userEditor);
            userEditor.getContentPane().setLayout(new BorderLayout());
            userEditor.getContentPane().add(panel,BorderLayout.CENTER);

		    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		    userEditor.setLocation((d.width - userEditor.getSize().width) / 2,
		                      (d.height - userEditor.getSize().height) / 2);
        }

		if (!prepare())
        {
			return;
        }

   		userEditor.pack();
        userEditor.setVisible(true);

        doWork();

        panel.clear();
    }

    protected abstract boolean prepare();

    protected abstract void doWork();
}
