package org.vovida.prov.table.action;

import org.apache.log4j.Category;
import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.util.logging.Util;
import org.vovida.prov.table.TableScreen;

import java.awt.Window;
import java.awt.event.ActionEvent;
import javax.swing.JOptionPane;

public class ActionDelete extends ActionTable
{
    private static Category logCategory = Category.getInstance(ActionDelete.class.getName());
    private String childDomain;


    public ActionDelete(ProvisioningServerWrapper wrapper,
        TableScreen screen, String actionName, String aUserDomain, Window parent)
    {
        super(actionName, wrapper, screen, parent);
        childDomain = aUserDomain;
    }

    public void actionPerformed(ActionEvent event)
    {
        if(tableScreen.getTable().getSelectedRowCount() < 1)
        {
			logCategory.debug("No rows selected for delete");
            return;
        }

        //find the selected rows
        int [] selectedRows = tableScreen.getTable().getSelectedRows();

        //for each selected row:
        for (int i=0; i<selectedRows.length; i++)
        {
            int row = selectedRows[i];
            //after each row has been removed in the table, the remaining rows
            //are shifted up by one so the next row to be removed is then at
            //and index one higher than it was when the selected rows were
            //retrieved and the table was still complete
            row = row - i;

            // - get the account identifer
			String accountId = (String)tableScreen.getTableModel().getValueAt(row,findNameIndex());

            try
            {
                // - get the list of selected features
	    		String [] features = server.getListOfFeatures(childDomain,accountId);

                // - remove the entry from each feature table on the provisioning server
                for (int feature=0; feature<features.length; feature++)
                {
					if (!server.deleteRecord(features[feature],accountId))
                    {
                        String message = "Cound not delete entry from feature domain \"" + features[feature] + "\"\nfor account \"" + accountId + "\".";
                        logCategory.warn(message);
                        JOptionPane.showMessageDialog(parent,message,"Warning",JOptionPane.WARNING_MESSAGE);
                    }
                }
            }
            catch (Exception ex)
            {
                String message = "Could not remove entries from the feature domains for account \"" + accountId + "\".";
                logCategory.error(message,ex);
				Util.showException(message,ex);
            }

            // - remove the entry from the account table on the provisioning server
			try
            {
	            if (!server.deleteRecord(childDomain,accountId))
                {
                    String message = "Cound not delete entry from domain \"" +
                        childDomain + "\"\nfor account \"" + accountId + "\".";
                    logCategory.warn(message);
                    JOptionPane.showMessageDialog(parent,message,"Warning",
                        JOptionPane.WARNING_MESSAGE);
                }
            }
            catch (Exception ex)
            {
                String message = "Could note remove entry for account \"" +
                    accountId + "\" from domain \"" + childDomain + "\".";
                logCategory.error(message,ex);
                Util.showException(message,ex);
            }

            //remove the row from the table
            tableScreen.getTableModel().removeRow(row);
        } //end for each row

        tableScreen.getTableModel().fireTableStructureChanged();
    }
}

