package org.vovida.prov.table.action;

import java.awt.Frame;
import java.awt.event.ActionEvent;

import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.util.assert.Assert;
import org.vovida.prov.util.logging.Util;
import org.vovida.prov.table.SortInfo;
import org.vovida.prov.table.TableScreen;
import org.vovida.prov.table.search.SearchInputDialog;
import org.vovida.prov.psLib.ProvisioningInterface;

import org.apache.log4j.Category;



public class ActionSearch extends ActionTable
{
    private static Category logCategory = Category.getInstance(ActionSearch.class.getName());

    private SearchInputDialog input;



    public ActionSearch(ProvisioningServerWrapper wrapper,
        TableScreen screen, String actionName, Frame parent)
    {
        super(actionName, wrapper, screen, parent);
        input = new SearchInputDialog(parent,screen.getTableModel());
    }


    public void actionPerformed(ActionEvent event)
    {
		Assert.assert("Sort info is not null", tableScreen.getSortInfo() != null);

		//show a dialog to allow the user to input a search criteria
		input.setDefaultSearchDomain(tableScreen.getTableModel().getDomain());
        input.pack();
        input.setVisible(true);
        String querry = input.getSearchString();
        String domain = input.getSearchDomain();
    	logCategory.debug("Starting to search table. Filter = " + querry + ". Domain = " + domain);
        if ((querry != null) && (!"".equals(querry))) //if the user entered a seach string
        {
            String sortOrder = tableScreen.getSortInfo().sortOrder;
            String sortField = tableScreen.getTableModel().getColumns()[tableScreen.getSortInfo().sortColumn].getColumnId();

            try
            {
		        //search the table
		        int count = server.search(domain, querry, sortOrder, sortField);
	            logCategory.debug("There are " + count + " search results");

		        //set the size of the scroll bars;
		        tableScreen.getTableModel().ensureDataCapacity(count);

                reloadData();
            }
            catch (Exception ex)
            {
	            String message =  "Could not search table with filter " + querry;
	        	logCategory.error(message, ex);
				Util.showException(message,ex);
            }
        }
    }
}
