package org.vovida.prov.table.task;

import org.vovida.prov.table.TableModel;
import org.vovida.prov.util.task.Task;
import org.vovida.prov.util.task.TaskListener;
import org.vovida.prov.util.ProvisioningServerWrapper;


/**
 * A base class for all tasks that invovle communication with the provisioning
 * server. <p>
 * Stores a reference to the provisioning server interface, as well as the
 * indices of the accounts to process, and an account type reference.
 */
abstract public class TableTask extends Task
{
    /**
     * The index of the first account to download information for.
     */
    protected int startIndex;

    /**
     * The index of the last account to download information for.
     */
    protected int stopIndex;

    /**
     * The identifier of the type of account to get.
     */
    protected String domain;

    /**
     * The name of the account to get data for.
     */
    protected String accountIdentifier;

    /**
     * Used to communicate with the provisioning server to get the account
     * names. <p>
     */
    protected ProvisioningServerWrapper server;

    /**
     * The data retrieved from the server is added to the model.
     */
    protected TableModel model;

    /**
     * Create a new task that will download account names in the given index
     * range (inclusive) for the given type of account. <p>
     * Assuming that indices can be passed to the server relative to the current
     * sort order. Otherwise, this needs work.
     */
    public TableTask(int from, int to, String domain, String acctId,
        ProvisioningServerWrapper psInt, TableModel data)
    {
        this(domain, acctId, psInt, data);
        startIndex = from;
        stopIndex = to;
    }

    public TableTask(int from, int to, String aDomain, String acctId,
        ProvisioningServerWrapper psInt, TableModel data, TaskListener listener)
    {
        super(listener);
        startIndex = from;
        stopIndex = to;
        server = psInt;
        model = data;
        domain = aDomain;
        accountIdentifier = acctId;
    }

    private TableTask( String aDomain, String acctId,
        ProvisioningServerWrapper psInt, TableModel data)
    {
        domain = aDomain;
        accountIdentifier = acctId;
        server = psInt;
        model = data;
    }
}
