package org.vovida.prov.table.task;

import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.table.TableModel;
import org.vovida.prov.table.NamedTableColumn;
import org.vovida.prov.util.logging.Util;

import java.util.Hashtable;
import javax.swing.SwingUtilities;

import org.apache.log4j.Category;



/**
 * Runs in its own thread and is responsible for loading data for a particular
 * set of accounts from the provisioning server. <p>
 */
public class TaskLoadAccountData extends TableTask 
{
     private static Category logCategory = Category.getInstance(TaskLoadAccountData.class.getName());

    /**
     * The list of the identifiers (as understood by provisioning server's
     * database) of the fields which will be loaded.
     */
    private String [] listOfFields;

    public TaskLoadAccountData(int from, int to, String domain,
        String acctIdentifier, ProvisioningServerWrapper server, TableModel data,
        String [] fields)
    {
    	super(from,to,domain, acctIdentifier,server,data);
		listOfFields = fields;
    }

    public void doWork() throws Exception
    {
        logCategory.debug("TaskLoadAccountData started: " + startIndex + " - "
            + stopIndex);
        try
        {
            //load the data from the provisioning server
	        Hashtable [] accountData = server.getAccountData(domain,
                accountIdentifier, startIndex, stopIndex, listOfFields);
	        saveReturnValue(accountData);

            for (int row=startIndex; row<=stopIndex; row++)
            {
                model.setRow(row,accountData[row-startIndex]);
            }
        }
        catch (Exception ex)
        {
            logCategory.error("Load account data failed", ex);
			Util.showException("Load account data failed ", ex);
            throw ex;
        }

        //cause the table to be updated in the swing event thread
        Runnable runnable = new Runnable()
        {
            public void run()
            {
                logCategory.debug("tableDataChanged from TaskLoadAccountData");
                model.fireTableDataChanged();
            }
        };
        SwingUtilities.invokeLater(runnable);
    } //end method doWork
 }
