package org.vovida.prov.table.task;

import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.util.logging.Util;
import org.vovida.prov.util.task.TaskListener;
import org.vovida.prov.table.TableModel;

import javax.swing.SwingUtilities;

import org.apache.log4j.Category;



/**
 * This task is responsible for loading a list of account names.
 */
public class TaskLoadAccountKeys extends TableTask
{
    private static Category logCategory = Category.getInstance(TaskLoadAccountKeys.class.getName());

    /**
     * The identifier (the column name as understood by the provisioning server)
     * which is the key field for this data. Ie, this would be something like
     * "accountName" when this is used with a table of end users. <p>
     * This is used when adding the data to the table and becomes the data id.
     */
    private String keyField;

    /**
     * Indicates the sort order which will be applied to the account keys before
     * they are loaded form the provisioning server.
     */
    private String sortOrder;


    private void setFields (String key, String aSortOrder)
    {
        keyField = key;
        sortOrder = aSortOrder;
    }

    public TaskLoadAccountKeys(int from, int to, String aDomain, String acctId,
        String key, String aSortOrder, ProvisioningServerWrapper psInt,
        TableModel data, TaskListener listener)
    {
        super(from,to,aDomain,acctId,psInt,data,listener);
        setFields(key,aSortOrder);
    }


    public TaskLoadAccountKeys(int from, int to, String domain, String acctId,
        String key, String aSortOrder, ProvisioningServerWrapper psInt,
        TableModel data)
    {
        super(from,to,domain,acctId,psInt,data);
        setFields(key,aSortOrder);
    }

    public void doWork() throws Exception
    {
        logCategory.info("loading account keys for " + startIndex + " to " + stopIndex);
        try
        {
            String [] accountKeys = server.getAccountKeys(domain,startIndex,
                stopIndex, keyField, sortOrder);

	         if(accountKeys.length > 0)
	         {
	             logCategory.debug("Adding account keys to table");
	         }
	
	         for (int i=0; i<accountKeys.length; i++)
	         {
	             model.addAccount(accountKeys[i],startIndex+i);
	         }

	        saveReturnValue(accountKeys);
        }
        catch (Exception ex)
        {
            logCategory.error("Could not load account keys", ex);
			Util.showException("Could not load account keys", ex);
            throw ex;
        }

        //cause the table to be updated in the swing event thread
        Runnable runnable = new Runnable()
        {
            public void run()
            {
				logCategory.debug("tableDataChanged from TaskLoadAccountKeys");
                model.fireTableDataChanged();
            }
        };
        SwingUtilities.invokeLater(runnable);
   }
}
