package org.vovida.prov.table.task;

import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.table.TableScreen;
import org.vovida.prov.util.task.Task;
import org.vovida.prov.util.logging.Util;
import org.vovida.prov.table.TableModel;

import org.apache.log4j.Category;



public class TaskSearchTable extends Task
{
    private static Category logCategory = Category.getInstance(TaskSearchTable.class.getName());

    private String domain;
    private String filter;
    private ProvisioningServerWrapper server;
    private TableScreen tableScreen;
    private String sortType;
    private String sortField;

    public TaskSearchTable(String aDomain, String aFilter, String aSortType,
        String aSortField, ProvisioningServerWrapper wrapper, TableScreen screen)
    {
		domain = aDomain;
        filter = aFilter;
        server = wrapper;
        tableScreen = screen;
        sortType = aSortType;
        sortField = aSortField;
    }

    public void doWork() throws Exception
    {
    	logCategory.debug("Starting to search table. Filter = " + filter);
        try
        {
	        //search the table
	        int count = server.search(domain, filter, sortType, sortField);
            logCategory.debug("There are " + count + " search results");

	        //set the size of the scroll bars;
	        TableModel model = tableScreen.getTableModel();
			model.ensureDataCapacity(count);

	        //reload the data
			tableScreen.loadScreenOfData();
        }
        catch (Exception ex)
        {
            String message =  "Could not search table with filter =" + filter;
        	logCategory.error(message, ex);
			Util.showException(message,ex);
            throw ex;
        }
    }
}
