/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package org.vovida.prov.template;

import javax.swing.text.Document;
import javax.swing.JTextField;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JLabel;

import org.vovida.prov.content.NumberDocument;

/**
 * Generates a sequence of numbers between two bounds.
 *
 * @author Matt Naish
 */
public class AutoNumberDataProvider extends DataProvider
{
  private String start;
  private String end;

  private int startInt;
  private int endInt;
  private int index;

  private JPanel panel;
  private JTextField startTextField;
  private JTextField endTextField;

  public AutoNumberDataProvider() {}

  public AutoNumberDataProvider(String start, String end)
  {
    this.start = start;
    this.end = end;
  }

  public void saveUIValues() throws IllegalArgumentException
  {
    if (panel != null)
    {
      String startTemp = startTextField.getText();
      String endTemp = endTextField.getText();

      try
      {
        if (Integer.parseInt(endTemp) <= Integer.parseInt(startTemp))
        {
          throw new IllegalArgumentException
            ("'End' must be larger than 'Start'");
        }

        start = startTemp;
        end = endTemp;
      }
      catch (NumberFormatException nfe)
      {
        throw new IllegalArgumentException
          ("Both 'Start' and 'End' must be integer values.");
      }
    }
  }

  public void clearUI()
  {
    if (panel != null)
    {
      startTextField.setText(null);
      endTextField.setText(null);
    }
  }

  public void populateUI()
  {
    if (panel != null)
    {
      startTextField.setText(start);
      endTextField.setText(end);
    }
  }

  public String getStart()
  {
    return start;
  }

  public String getEnd()
  {
    return end;
  }

  public String getName()
  {
    return "Auto Number";
  }

  public boolean next()
  {
    return ++index <= endInt;
  }

  public String getData()
  {
    return String.valueOf(index);
  }

  public JPanel getUI()
  {
    if (panel == null)
    {
      panel = new JPanel();

      panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

      JPanel startPanel = new JPanel();

      startPanel.add(new JLabel("Start:"));
      startPanel.add(startTextField = new JTextField(new NumberDocument(), 
              start, 15));

      JPanel endPanel = new JPanel();

      endPanel.add(new JLabel("End:"));
      endPanel.add(endTextField = new JTextField(new NumberDocument(), end, 
              15));

      panel.add(startPanel);
      panel.add(endPanel);
    }

    return panel;
  }

  public void connect() throws NestedException
  {
    try
    {
      startInt = Integer.parseInt(start);
      endInt = Integer.parseInt(end);
      index = startInt - 1;
    }
    catch (Exception e)
    {
      throw new NestedException(e);
    }
  }

  public void close()
  {

    // do nothing.
  }

}

