/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package org.vovida.prov.template;

/**
 * A TemplateLoader takes care of loading DataProvider and the bar code scanner
 * field configuration information from a source.  It also handles saving this
 * modified information back to the source.
 *
 * @author Matt Naish
 */
public interface TemplateLoader
{

  /**
   * Returns a list of available templates
   */
  public String[] getAvailableTemplates();

  /**
   * Returns a list of available definitions
   */
  public String[] getDefinitions();

  /**
   * Loads a specified template.  The data from the template is then
   * available via the getFields() and getBarCodeField() methods
   */
  public void load(String templateName) throws NestedException;

  /**
   * Return all the fields from the template identified by the last call to
   * load().
   * @throws IllegalStateException if no call to load has been made yet.
   */
  public DataProviderWrapper[] getFields();

  /**
   * Return the bar code field from the template identified by the last call
   * to load().
   * @throws IllegalStateException if no call to load has been made yet.
   */
  public String getBarCodeField();

  /**
   * Return all the information pertaining to a RecordUpdater object
   * identified by the last call to load().
   * @throws IllegalStateException if no call to load has been made yet.
   */
  public RecordUpdaterData getRecordUpdaterData();

  /**
   * Return the name of the parent definition identified by the last call
   * to load()
   * @throws IllegalStateException if no call to load has been made yet.
   */
  public String getParent();

  /**
   * Save the information for the last load()-ed template.
   */
  public void save(String barCodeField) throws NestedException;

  /**
   * Create a new template with the given name and based on the supplied
   * definition file.
   */
  public void createNew(String newTemplateName, 
          String definition) throws NestedException;

  /**
   * Create a new template from a copy of the current template
   */
  public void copy(String newTemplateName) throws NestedException;

  /**
   * Delete the current template
   */
  public void delete() throws NestedException;

}

