/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.template;

import org.jdom.Element;
import java.util.List;

/**
 * A utility class that facilitates creating a DataProvider from and Element and
 * facilitates saving data from a DataProvider to an Element.
 *
 * @author Matt Naish
 */
public class XMLDataProviderUtil
{

  /**
   * Create a DataProvider from Element data
   */
  public static DataProvider createDataProviderFromElement(Element elem)
  {
    DataProvider dp = null;
    Element valueElement = elem.getChild(XMLTemplateLoader.VALUE);
    List valueChildren;
    Element valueChild;

    if (valueElement == null)
    {
      return null;
    }

    valueChildren = valueElement.getChildren();

    if (valueChildren.size() == 0)
    {
      return null;
    }

    valueChild = (Element) valueChildren.get(0);

    if (valueChild.getName().equals(XMLTemplateLoader.DEFAULT))
    {
      return new DefaultDataProvider(valueChild.getText());
    }

    if (valueChild.getName().equals(XMLTemplateLoader.AUTO_NUMBER))
    {
      return new AutoNumberDataProvider(valueChild.getAttributeValue
        (XMLTemplateLoader.START), 
              valueChild.getAttributeValue(XMLTemplateLoader.END));
    }

    if (valueChild.getName().equals(XMLTemplateLoader.DATABASE))
    {
      return new DatabaseDataProvider
        (valueChild.getAttributeValue(XMLTemplateLoader.DOMAIN), 
              valueChild.getAttributeValue(XMLTemplateLoader.FILTER), 
              valueChild.getAttributeValue(XMLTemplateLoader.FIELD));
    }

    return null;
  }

  /**
   * Save the data from the DataProvider into the Element.
   */
  public static void saveDataProviderToElement(DataProvider dp, Element elem)
  {
    if (dp == null)
    {
      elem.removeChildren();
    }
    else
    {
      Element valueElement = elem.getChild(XMLTemplateLoader.VALUE);

      if (valueElement == null)
      {
        valueElement = new Element(XMLTemplateLoader.VALUE);

        elem.addContent(valueElement);
      }
      else
      {
        valueElement.removeChildren();
      }

      if (dp instanceof DefaultDataProvider)
      {
        Element defaultElement = new Element(XMLTemplateLoader.DEFAULT);

        defaultElement.setText(((DefaultDataProvider) dp).getDefaultValue());
        valueElement.addContent(defaultElement);
      }
      else if (dp instanceof AutoNumberDataProvider)
      {
        Element autoNumberElement = 
          new Element(XMLTemplateLoader.AUTO_NUMBER);
        AutoNumberDataProvider andp = (AutoNumberDataProvider) dp;

        autoNumberElement.addAttribute(XMLTemplateLoader.START, 
                andp.getStart());
        autoNumberElement.addAttribute(XMLTemplateLoader.END, andp.getEnd());
        valueElement.addContent(autoNumberElement);
      }
      else if (dp instanceof DatabaseDataProvider)
      {
        Element dbElement = new Element(XMLTemplateLoader.DATABASE);
        DatabaseDataProvider dbdp = (DatabaseDataProvider) dp;

        dbElement.addAttribute(XMLTemplateLoader.DOMAIN, dbdp.getDomain());
        dbElement.addAttribute(XMLTemplateLoader.FILTER, dbdp.getFilter());
        dbElement.addAttribute(XMLTemplateLoader.FIELD, dbdp.getField());
        valueElement.addContent(dbElement);
      }
      else
      {

        // throw error.  DataProvider is not known to this block of code!
        throw new IllegalArgumentException("DataProvider is not known");
      }
    }
  }

}

