package org.vovida.prov.userEdit;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Window;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

import java.util.Hashtable;
import java.util.Iterator;

import org.vovida.prov.dynamicGui.ProvisionableInstance;
import org.vovida.prov.content.ContentData;
import org.vovida.prov.util.logging.Util;
import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.util.FieldNames;

import org.apache.log4j.Category;





public class FeatureEditPanel extends JPanel
{
    private static Category logCategory = Category.getInstance(FeatureEditPanel.class.getName());


    private JTabbedPane tabs;
    private String [] accountIds;
    private ProvisioningServerWrapper server;
    private String childDomain;
    private Window parent;


    public FeatureEditPanel(ProvisioningServerWrapper wrapper,
        String domain, Window aParent)
    {
        server = wrapper;
        childDomain = domain;
        parent = aParent;

        setLayout(new BorderLayout());

        boolean hasFeatures = true;
        try
        {
            hasFeatures = server.hasFeatures(domain);
        }
        catch (Exception ex)
        {
            String message = "Could not determine whether the domain \"" +
                domain + "\" has features.";
            logCategory.warn(message,ex);
        }

        tabs = new JTabbedPane();

        if (hasFeatures)
        {
	        add(tabs,BorderLayout.CENTER);
        }

        //add the tab which will allow the data from the account table to be
        //edited
        try
        {
        	ProvisionableInstance instance = UIRepository.get(childDomain);
            if (hasFeatures)
            {
	            addTab(instance);
            }
            else
            {
                add(instance.getContainer(), BorderLayout.CENTER);
            }
            parent.pack();
        }
        catch (Exception ex)
        {
            String message = "Could not load user interface description for domain \"" + childDomain + "\".";
            Util.showException(message,ex);
            logCategory.error(message,ex);
        }

        tabs.addChangeListener(new ChangeListener()
        {
            public void stateChanged(ChangeEvent event)
            {
		        if ( (accountIds != null) && (accountIds.length == 1))
		        {
					int selectedIndex = tabs.getSelectedIndex();
                    if (selectedIndex >= 0) //if the user has no features enabled, there may not be any tabs (and so no selected tab)
                    {
		                TabContentPanel panel = (TabContentPanel)tabs.getComponentAt(selectedIndex);
		                loadData(panel,accountIds[0]);
                    }
		        }
            }
        });
    }


    public Hashtable getChangedData (String domain)
    {
        try
        {
		    ProvisionableInstance instance = UIRepository.get(domain);
	        if (instance != null)
	        {
                return instance.readChangedData();
//	            return instance.getChangedData();
	        }
	        else
	        {
	            String message = "There is no data in the UIRepository for the domain \"" + domain + "\".";
	            logCategory.warn(message);
	        }
        }
        catch (Exception ex)
        {
            String message = "Could not get the user interface for domain \"" + domain + "\" while trying to retrieve data from the FeatureEditPanel.";
            logCategory.error(message,ex);
            Util.showException(message,ex);
        }
        return null;
    }


    public Hashtable getData (String domain)
    {
        try
        {
	        ProvisionableInstance instance = UIRepository.get(domain);
	        if (instance != null)
	        {
	            return instance.getData();
	        }
	        else
	        {
	            String message = "There is no data in the UIRepository for the domain \"" + domain + "\".";
	            logCategory.warn(message);
	        }
        }
        catch (Exception ex)
        {
            String message = "Could not get the user interface for domain \"" + domain + "\" while trying to retrieve data from the FeatureEditPanel.";
            logCategory.error(message,ex);
            Util.showException(message,ex);
        }
        return null;
    }

    public void removeAllFeatureTabs()
    {
        int tabCount = tabs.getTabCount();
        int at = 0;
        for (int i=0; i<tabCount; i++)
        {
            TabContentPanel panel = (TabContentPanel)tabs.getComponent(at);
            if (panel.isFeature())
            {
	            tabs.removeTabAt(at);
            }
            else
            {
                at++;
            }
        }
    }

    public void removeTab(String featureName)
    {
		for (int i=0; i<tabs.getTabCount(); i++)
        {
			String featureId = ((TabContentPanel)tabs.getComponentAt(i)).getTabId();
            if (featureId.equals(featureName))
            {
                tabs.removeTabAt(i);
                return;
            }
        }
    }


    private void addTab(ProvisionableInstance instance)
    {
		TabContentPanel panel = (TabContentPanel)instance.getContainer();
		tabs.addTab(instance.getType().getLabel(),panel);
        tabs.setToolTipTextAt(tabs.getTabCount()-1,instance.getType().getTooltip());
		tabs.validate();

		parent.pack();
    }

    public void addFeatureTabs()
    {
        if (accountIds.length == 1)
        {
            String account = accountIds[0];

            //get the list of features for this account
            try
            {
            	String [] features = server.getListOfFeatures(childDomain, account);
	            for (int i=0; i<features.length; i++)
	            {
	                addFeatureTab(features[i]);
	            }
            }
            catch (Exception ex)
            {
                String message = "Could not determine list of features for account \"" + account + "\" in domain \"" + childDomain + "\".";
                logCategory.error(message,ex);
                Util.showException(message,ex);
            }
        }
        else
        {
            logCategory.warn("Attempted to add all feature tabs when editing multiple accounts. addAllFeatureTabs() only applies when editing single account.");
        }
    }

    public void addFeatureTab(String featureName)
    {
        try
        {
	    	ProvisionableInstance instance = UIRepository.getFeature(featureName);
            addTab(instance);
        }
        catch (Exception ex)
        {
            String message = "Could not load user interface description for feature " + featureName;
            Util.showException(message,ex);
            logCategory.error(message,ex);
        }
    }


    /**
     * Set the names of the accounts which are being edited by this panel.<p>
     * This will clear the data loaded in all the feature tabs and load data
     * for the first visible tab (if only one new account is set. Otherwise, the
     * screen will be blank). <p>
     */
    public void setAccount(String [] accounts)
    {
        accountIds = accounts;

		//it should not be possible to change the account id field when editing
        //more than one account at a time
        if (accountIds.length > 1)
        {
            enableAccountIdFields(false);
        }
        else
        {
            enableAccountIdFields(true);
        }

		if (accounts.length == 1)
        {
            try
            {
	            ProvisionableInstance domainInstance = UIRepository.get(childDomain);
	            if (domainInstance != null)
	            {
	            	TabContentPanel panel = (TabContentPanel)domainInstance.getContainer();
	                loadData(panel,accountIds[0]);
	            }
            }
            catch (Exception ex)
            {
                String message = "Could not load account data for domain \"" +
                    childDomain + "\".";
                logCategory.error(message,ex);
                Util.showException(message,ex);
            }
        }
    }

	private void enableAccountIdFields(boolean set)
    {
        Iterator allUis = UIRepository.getAllUIs().iterator();
		while (allUis.hasNext())
        {
            ProvisionableInstance instance = (ProvisionableInstance) allUis.next();
	        JComponent component = instance.getComponentByName(FieldNames.keyField);
            if (component != null)
            {
                component.setEnabled(set);
            }
        }
    }


    private void loadData(TabContentPanel panel, String account)
	{
		if (!panel.isDataLoaded())
		{
			String featureId = panel.getTabId();

			try
			{
				Hashtable data = server.getData(account,featureId);
				if (data != null)
				{
					UIRepository.getFeature(featureId).setData(data);
				}
				else
				{
				    logCategory.info("No data for feature " + featureId +
				        " and account \"" + accountIds[0] + "\"");
				}
			}
			catch (Exception ex)
			{
				String message = "Could not get feature data for feature \"" +
				    featureId + "\" and account \"" + accountIds[0] + "\".";
				logCategory.error(message, ex);
				Util.showException(message,ex);
			}
			panel.setDataLoaded(true);
        }
    }


    public void revert()
    {
		Iterator iterator = UIRepository.getAllUIs().iterator();
        while (iterator.hasNext())
        {
            ProvisionableInstance instance = (ProvisionableInstance)iterator.next();
            instance.cancel();
        }
    }

    /**
     * Write back to the provisioning server, the data for each of the feature
     * panels that are currently visible in the tabs. <p>
     */
    public boolean submit()
    {
        boolean allSubmited = true;

        //if using the tab control, it will contain all the currently selected
        //feature panels and the main account info tab
        if (tabs.getTabCount() > 0)
        {
            for (int i=0; i<tabs.getTabCount(); i++)
            {
                //find the feature id of the selected panel
	            TabContentPanel panel = (TabContentPanel)tabs.getComponentAt(i);
	            String featureId = panel.getTabId();

	            try
	            {
                    //get the gui elements for the current feature id
	                ProvisionableInstance instance = UIRepository.getFeature(featureId);
	            }
	            catch (Exception ex)
	            {
	                String message = "Could not get user interface for feature \"" + featureId + "\".";
	                logCategory.warn(message,ex);
	                Util.showException(message,ex);
	            }

                //write back the changed data for the current feature id
				if (!writeBackFeature(featureId))
                {
                    //remember that a write failed to report this to the caller
                    //(which may use it to allow user to re-enter the data)
                    allSubmited = false;
                }
            }
        }
        //if not using the tab control, just write back the main account info
        //tab directly
        else
        {
            allSubmited = writeBackFeature(childDomain);
        }

        return allSubmited;
    }

    public void clear()
    {
        Iterator allUis = UIRepository.getAllUIs().iterator();
        while (allUis.hasNext())
        {
            ProvisionableInstance instance = (ProvisionableInstance)allUis.next();
			TabContentPanel panel = (TabContentPanel)instance.getContainer();
            panel.setDataLoaded(false);
        }

        UIRepository.clearAllData();
        accountIds = null;
    }

    public String getSelectedAccount()
    {
        try
        {
	        ProvisionableInstance instance = UIRepository.get(childDomain);
	        ContentData data = instance.getContentByName(FieldNames.userIdConlumnName);
	        String userId = data.getContent();
	        return userId;
        }
        catch (Exception ex)
        {
            String message = "Could not retrieve the account identifier entered by the user.";
            logCategory.error(message,ex);
            Util.showException(message,ex);
        }
        return "";
    }


    private boolean writeBackFeature(String feature)
    {
        Hashtable newData = new Hashtable();

        if ( (accountIds == null) || (accountIds.length == 1) )
        {
            if (!feature.equals(childDomain)) //if not saving the main account tab
            {
                try
                {
                    //get the account identifier which has been filled in the main
                    //account tab
			        ProvisionableInstance instance = UIRepository.get(childDomain);
    	            String accountId = instance.getContentByName(FieldNames.userIdConlumnName).getContent();

					//set it as the account identifier for this tab
	                instance = UIRepository.get(feature);
	                ContentData data = instance.getContentByName(FieldNames.userIdConlumnName);
	                if (data != null)
	                {
	                    data.setContent(accountId);
	                }
                }
                catch (Exception ex)
                {
                    String message = "Could not copy the account identifier into the " + feature + " tab.";
                    logCategory.error(message,ex);
                    Util.showException(message,ex);
                }
            }
        }
		else
        {
            if(accountIds.length > 1) //if editing multiple accounts
            {
//----------temporary--------
		        try
		        {
			        ProvisionableInstance instance = UIRepository.getFeature(feature);
                    instance.getContentByName(FieldNames.keyField).setContent("kleenex");
//					instance.getContentByName(FieldNames.userIdConlumnName).setContent("kleenex");
		        }
		        catch (Exception ex)
		        {
		            System.out.println(ex);
		        }
//----------temporary--------
            }
        }

        try
        {
            ProvisionableInstance instance = UIRepository.getFeature(feature);
			try
	        {
				newData = instance.readChangedData();
//				newData = instance.getChangedData();
	        }
	        catch (org.vovida.prov.content.InvalidDataException ex)
	        {
	            return false;
	        }
        }
        catch (Exception ex)
        {
            String message = "Could not retrieve data for domain \"" + feature + "\".";
            logCategory.error(message,ex);
            Util.showException(message,ex);
        }

        if (newData.size() < 1)
        {
            logCategory.info("No data to be saved for tab " + feature);
            return true; //no changed data is ok
        }

        if (accountIds != null) //editing account(s)
        {
            for (int i=0; i<accountIds.length; i++)
            {
                newData.put(FieldNames.keyField,accountIds[i]);
                write(feature,newData);
            }
        }
        else //adding account
        {
            try
            {
	            newData = UIRepository.getFeature(feature).getData();
            }
            catch (Exception ex)
            {
	            String message = "Could not get data for feature \"" + feature + "\".";
				logCategory.error(message,ex);
	            Util.showException(message,ex);
            }
            write(feature,newData);
        }

        return true;
    }

    private void write(String domain, Hashtable data)
    {
            String account = (String)data.get(FieldNames.keyField);

			//check whether there is any entry in the table for
            //this account
            Hashtable oldData = null;
            try
            {
				oldData = server.getData(account,domain);
            }
            catch (Exception ex)
            {
                logCategory.warn("Could not determine whether an entry exists for account \""
                    + account + "\" in domain \"" + domain
                    + "\".\nAssuming it does not exist",ex);
            }

			if (oldData != null)
			{
               //the feature table does contain an entry for this account
               //therefore, it only needs to be modified
               try
               {
                  server.set(domain,account,data);
               }
               catch (Exception exc)
               {
                   String message = "Could not set data for domain \""
                       + domain + "\" and account \"" + account + "\".";
                   logCategory.error(message,exc);
                   Util.showException(message,exc);
               }
			}
			else
			{
				//the feature table does not contain an entry for this
				//account. Therefore, it needs to be added.
                try
                {
	                server.add(domain,data);
                }
                catch (Exception ex)
                {
                    String message = "Could not create new entry in domain \"" +
                        domain + "\" for account \"" + account +"\".";
                    logCategory.error(message,ex);
                    Util.showException(message,ex);
                }
			}
    }
}
