package org.vovida.prov.userEdit;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import org.vovida.prov.util.ProvisioningServerWrapper;

import org.apache.log4j.Category;



public class FeatureSelectPanel extends JPanel
{
    private static Category logCategory = Category.getInstance(UserEditorPanel.class.getName());

    private FeatureCheckBox[] features;
    private FeatureEditPanel featureEditor;
    private ProvisioningServerWrapper server;


    /**
     * @param featureNames the names of all the feature avalialbe to the admin
     */
    public FeatureSelectPanel(String [] featureIds, FeatureEditPanel panel,
        ProvisioningServerWrapper wrapper)
    {
        server = wrapper;
        featureEditor = panel;

		setLayout(new BorderLayout());
        setBorder(new EmptyBorder(3,3,3,3));
        add(new JLabel("Select features: "),BorderLayout.NORTH);

		JPanel temp = new JPanel();
        temp.setLayout(new BoxLayout(temp,BoxLayout.Y_AXIS));

        features = new FeatureCheckBox[featureIds.length];
		for (int i=0; i<featureIds.length; i++)
        {
            final FeatureCheckBox check = new FeatureCheckBox(featureIds[i]);
			features[i] = check;
            features[i].addActionListener(new FeatureToggleListener());
            temp.add(features[i]);

            try
            {
	            String xml = server.getFeatureXMLDescription(featureIds[i]);

                org.apache.xerces.parsers.SAXParser parser = new org.apache.xerces.parsers.SAXParser();
                org.xml.sax.helpers.DefaultHandler handler = new org.xml.sax.helpers.DefaultHandler()
                {
		        	private boolean inLabel = false;
		            private boolean inTooltip = false;
		            private boolean inDataDef = false;

				    public void startElement(String uri, String local, String raw, org.xml.sax.Attributes attrs)
		            {
						if (local.equals("DataDefn"))
		                {
		                    inDataDef = true;
		                }
		                else if (local.equals("Label"))
		                {
		                    inLabel = true;
		                }
		                else if (local.equals("Tooltip"))
		                {
		                    inTooltip = true;
		                }
		                else if (local.equals("DBEntry"))
		                {
		                    inDataDef = false;
		                }
		            }

		            public void endElement(String uri, String local, String qName)
		            {
		                if (local.equals("DataDefn"))
		                {
		                    inDataDef = false;
		                }
		                else if (local.equals("Label"))
		                {
		                    inLabel = false;
		                }
		                else if (local.equals("Tooltip"))
		                {
		                    inTooltip = false;
		                }
		            }

		            public void characters(char[] ch, int start, int length)
		            {
						if (inDataDef)
		                {
			                if (inLabel)
			                {
		                        check.setName(new String(ch,start,length));
			                }
			                else if (inTooltip)
			                {
		                        check.setToolTipText(new String(ch,start,length));
			                }
		                }
		            }
		        };

		        parser.setContentHandler(handler);
		        try
		        {
					parser.parse(new org.xml.sax.InputSource(new java.io.StringReader(xml)));
		        }
		        catch (Exception exc)
		        {
                    String message = "Could not parse xml description for feature \"" + features[i] + "\".";
                    logCategory.warn(message, exc);
                    System.out.print(message);
                    exc.printStackTrace();
		        }
            }
            catch (Exception ex)
            {
				String message = "Could not find xml description for feature \"" + featureIds[i] + "\".";
                logCategory.warn(message,ex);
                System.out.println(message);
                ex.printStackTrace();
            }
        }

        add(temp,BorderLayout.CENTER);
        add(Box.createHorizontalStrut(10),BorderLayout.WEST);
    }


    private class FeatureToggleListener implements ActionListener
    {
       public void actionPerformed(ActionEvent event)
        {
            FeatureCheckBox feature = (FeatureCheckBox)event.getSource();
            String featureName = feature.getId();

            if (feature.isSelected())
            {
				featureEditor.addFeatureTab(featureName);
            }
            else
            {
                featureEditor.removeTab(featureName);
            }
        }
    }


    public String getChangedFeatures()
    {
        String changed = "";
        for (int i=0; i<features.length; i++)
        {
        	if (features[i].isChanged())
	        {
                changed += features[i].getId() + ",";
	        }
        }

        if (changed.length() > 0)
        {
	       changed = changed.substring(0,changed.length()-1); //drop the last ","
        }

        return changed;
    }


    public String getSelectedFeatures()
    {
        String selected = "";

        for (int i=0; i<features.length; i++)
        {
            if (features[i].isSelected())
            {
                selected += features[i].getId() + ",";
            }
        }

        if (selected.length() > 0)
        {
	       selected = selected.substring(0,selected.length()-1); //drop the last ","
        }

        return selected;
    }

    public boolean hasFeatures()
    {
        if ((features != null) && (features.length > 0))
        {
            return true;
        }
        return false;
    }


    public void setSelectedFeatures(String [] featureIds)
    {
		//set all the check boxes to false
        for (int i=0; i<features.length; i++)
        {
            features[i].setSelected(false);
        }

        //remove all the tabs for all the features
        featureEditor.removeAllFeatureTabs();

        //set check boxes and add tabs for the selected features
        for (int i=0; i<featureIds.length; i++)
        {
            for (int j=0; j<features.length; j++)
            {
                if (features[j].getId().equals(featureIds[i]))
                {
                    features[j].setSelected(true);
                    featureEditor.addFeatureTab(featureIds[i]);
                    break;
                }
            }
        }

        //clear the change flags for all the check boxes
        for (int i=0; i<features.length; i++)
        {
            features[i].resetChanged();
        }
    }
}
