package org.vovida.prov.userEdit;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;

import org.vovida.prov.util.AppletApp;
import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.psLib.SocketProvisioningInterface;

import org.apache.log4j.Category;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.PatternLayout;




public class StandaloneUserGUI extends AppletApp
{
    private static Category logCategory = Category.getInstance(StandaloneUserGUI.class.getName());

    private FeatureEditPanel editor;
    private ProvisioningServerWrapper server;
    private String userAccountId;
    private String userDomain;

    private static StandaloneUserGUI configure;


    public StandaloneUserGUI()
    {
        try
        {
//	        UIManager.setLookAndFeel(new com.incors.plaf.kunststoff.KunststoffLookAndFeel());
        }
        catch (Exception ex)
        {
            logCategory.warn("Could not load look and feel because ", ex);
        }

		Category.getRoot().addAppender(new ConsoleAppender(new PatternLayout("[%t] %-6p %c{3} - %m%n")));
    }


    public void createPanel (ProvisioningServerWrapper wrapper, String account, String domain)
    {
        server = wrapper;
        userAccountId = account;
        userDomain = domain;
        UIRepository.setServer(server);
        UIRepository.setChildCategory("user");

        final JFrame frame = createFrame();

        editor = new FeatureEditPanel(server,userDomain,frame);
        frame.getContentPane().add(editor,BorderLayout.CENTER);

        frame.setTitle("Configure - " + account);

		JButton ok = new JButton("Ok");
        ok.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent event)
            {
                editor.submit();
                frame.setVisible(false);
                if (isStandalone)
                {
                    System.exit(0);
                }
            }
        });

        JButton cancel = new JButton("Cancel");
        cancel.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent event)
            {
                frame.setVisible(false);
                if (isStandalone)
                {
                    System.exit(0);
                }
            }
        });

        JButton revert = new JButton("Revert all");
        revert.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent event)
            {
                editor.revert();
            }
        });

        JPanel panel = new JPanel();
        panel.setBorder(new EmptyBorder(10,5,5,5));
        panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
        panel.add(revert);
		panel.add(Box.createHorizontalGlue());
        panel.add(ok);
        panel.add(cancel);

        frame.getContentPane().add(panel,BorderLayout.SOUTH);

        frame.pack();
    }

    private static void showUsage()
    {
        System.out.println("Usage:");
        System.out.println("  UserConfigure host port account domain");
        System.out.println("where:");
        System.out.println("  host - host name of provisioning server");
        System.out.println("  port - port number of provisioning server");
        System.out.println("  account - account identifier of the user");
        System.out.println("  domain - name of the domain in which the user account may be found on the provisioning server");
    }

    public void init()
    {
        String host = getParameter("host", "localhost");
        int port = Integer.parseInt(this.getParameter("port", "6005"));
        String account = getParameter("account", "");
        String domain = getParameter("domain", "accounts");

        SocketProvisioningInterface.initialize(host,port);
        ProvisioningServerWrapper server = new ProvisioningServerWrapper(SocketProvisioningInterface.getInstance());

        configure.createPanel(server, account, domain);
        configure.editor.setAccount(new String [] {configure.userAccountId});
        configure.editor.addFeatureTabs();

        configure.setVisible(true);
    }


    public static void main(String [] arg)
    {
		if (arg.length < 4)
        {
            showUsage();
            return;
        }

        System.setProperty("host", arg[0]);
        System.setProperty("port", arg[1]);
        System.setProperty("account", arg[2]);
        System.setProperty("domain", arg[3]);

        configure = new StandaloneUserGUI();
        configure.isStandalone = true;
        configure.init();
    }
}
