package org.vovida.prov.userEdit;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Collection;
import java.io.IOException;
import javax.swing.Box;

import org.vovida.prov.dynamicGui.ProvisionableInstance;
import org.vovida.prov.util.ProvisioningServerWrapper;
import org.vovida.prov.psLib.OperationException;
import org.vovida.prov.dynamicGui.*;


public class UIRepository
{
    private static Hashtable uis = new Hashtable();
    private static ProvisioningServerWrapper server;
    private static String childCategory = "accounts";



    public static void setChildCategory(String category)
    {
        childCategory = category;
    }


    public static ProvisionableInstance get(String domain) throws IOException,
        OperationException
    {
        if (server == null)
        {
        	throw new IOException("ProvisioningServerWrapper not set for UIRepository. Use setServer()");
        }

        ProvisionableInstance ui = (ProvisionableInstance)uis.get(domain);
        if (ui == null)
        {
            String xml = server.getXMLDescription(childCategory,domain);
            ui = putUI(domain, xml, false);
        }

        return ui;
    }

    public static ProvisionableInstance getFeature(String featureName) throws
        IOException, OperationException
    {
        if (server == null)
        {
        	throw new IOException("ProvisioningServerWrapper not set for UIRepository. Use setServer()");
        }

	    ProvisionableInstance ui = (ProvisionableInstance)uis.get(featureName);
        if (ui == null)
        {
			String xml = server.getFeatureXMLDescription(featureName);
            ui = putUI(featureName, xml, true);
        }
    	return ui;
    }

    private static ProvisionableInstance putUI(String domain, String xml, boolean isFeature)
    {
		ProvisionableType type = TypeBuilder.getType(domain,xml);
		ProvisionableInstance ui = type.buildInstance(new TabContentPanel(domain,isFeature));
		uis.put(domain,ui);
        return ui;
    }

    public static void setServer(ProvisioningServerWrapper wrapper)
    {
        server = wrapper;
    }

    public static void clearAllData()
    {
        Iterator features = uis.values().iterator();
        while(features.hasNext())
        {
            ProvisionableInstance instance = (ProvisionableInstance)features.next();
            instance.reset();
            instance.clearKeyValue();
        }
    }

    public static Collection getAllUIs()
    {
        return uis.values();
    }
}
