package org.vovida.prov.util;

import javax.swing.JApplet;
import javax.swing.JPanel;
import javax.swing.JFrame;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;

import org.apache.log4j.Category;





public abstract class AppletApp extends JApplet
{
	private static final Category logCategory = Category.getInstance(AppletApp.class.getName());

    protected boolean isStandalone = false;

    protected HashMap argMap;


    protected void addArg(String argSwitch, int argNum)
    {
        if (argMap == null)
        {
            argMap = new HashMap();
        }

        argMap.put(argSwitch, new Integer(argNum));
        System.getProperties().remove(argSwitch);
    }

    protected String [] parseArgs(String [] args)
    {
		if (logCategory.isDebugEnabled())
        {
            String commandLine = "";
            for (int i=0; i<args.length; i++)
            {
                commandLine += args[i] + " ";
            }
            logCategory.debug("Command line = " + commandLine);
        }

        Vector extraArgs = new Vector();

        ArrayList arguments = new ArrayList();
        for (int i=0; i<args.length; i++)
        {
            arguments.add(args[i]);
        }

        Iterator keys = argMap.keySet().iterator();
        while (keys.hasNext())
        {
            String switchName = (String)keys.next();
            int occurances = ((Integer)argMap.get(switchName)).intValue();

            int switchOccurance = arguments.indexOf(switchName);
			if (switchOccurance != -1)
            {
                logCategory.debug("Found command line switch " + switchName);
	            arguments.remove(switchOccurance);
	            String [] values = new String[occurances];
	            for (int i=0; i<occurances; i++)
	            {
	                values[i] = (String)arguments.remove(switchOccurance);
	            }
	
	            if (values.length > 0)
	            {
		            System.setProperty(switchName, values[0]);
	            }
	            else
	            {
	                System.setProperty(switchName, "");
	            }
            }
        }

		return (String []) arguments.toArray(new String [] {});
    }


    protected String getParameter(String key, String defaultValue)
    {
        if (isStandalone)
        {
            String value = System.getProperty(key);
            if (value != null)
            {
                return value;
            }
            else
            {
                return defaultValue;
            }
        }
        else
        {
            String value = getParameter(key);
            if (value != null)
            {
                return value;
            }
            else
            {
                return defaultValue;
            }
        }
    }

    /**
     *  Initialize the applet.
     */
    abstract public void init();

    protected static JFrame createFrame(JPanel content)
    {
        JFrame frame = createFrame();
        frame.getContentPane().add(content,BorderLayout.CENTER);
        frame.pack();

        Dimension dimension = Toolkit.getDefaultToolkit().getScreenSize();
        frame.setLocation((dimension.width - frame.getSize().width)/2,
            (dimension.height - frame.getSize().height)/2);

        return frame;
    }

    protected static JFrame createFrame()
    {
        JFrame frame = new JFrame();
        frame.setDefaultCloseOperation(frame.EXIT_ON_CLOSE);
        frame.getContentPane().setLayout(new BorderLayout());
        frame.setVisible(true);
        return frame;
    }
}

