package org.vovida.prov.util;

import java.util.Vector;

import org.apache.log4j.Category;


/**
 * Given a series of integer values, coalesce them into pairs of ranges. The
 * integers must be sorted into ascending order before being passed to the
 * coalesce method. <p>
 * For example, given the folowing set of integers: <br>
 *   1,2,3,4,5,9,10,11,12,56 <br>
 * the following set of ranges will be generated: <br>
 *   (1,5) (9,12) (56,56) <br>
 */
public class Coalesce
{
    private static Category logCategory = Category.getInstance(Coalesce.class.getName());


    public static IntPair [] coalesce(int [] values)
    {
        if (logCategory.isDebugEnabled()) //don't build string if not debugging
        {
            String temp = "";
            for (int i=0;i<values.length; i++)
            {
                temp += values[i] + " ";
            }
            logCategory.debug(temp);
        }

        if (values.length == 0)
        {
            return null;
        }
        if (values.length == 1)
        {
            return new IntPair[] {new IntPair(values[0],values[0])};
        }
        else
        {
            Vector pairs = new Vector();
            int min = values[0];
            int i;
            for (i=0; i<values.length-1; i++)
            {
                if (values[i+1]-values[i] != 1)
                {
                    pairs.addElement(new IntPair(min,values[i]));
                    logCategory.debug("New pair " + min + ":" + values[i]);

                    if (values.length>i)
                    {
                        min=values[i+1];
                    }
                }
            }

            if(values[i]-values[i-1] == 1) //make a pair for the last index
            {
                pairs.addElement(new IntPair(min,values[i]));
                logCategory.debug("New pair " + min + ":" + values[i]);
            }

            IntPair [] returnValue = new IntPair[pairs.size()];
            pairs.copyInto(returnValue);
            return returnValue;
        }
    }
}
