package org.vovida.prov.util.logging;

import javax.swing.JPanel;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.Box;
import javax.swing.JOptionPane;
import javax.swing.border.TitledBorder;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import org.apache.log4j.*;

/**
 * A panel which sets the priority for a particular category. <p>
 * Contains a combo box which allows the user to select any of the available
 * priorities (prority names are determined by querying the logging system)
 * and then set that value as the priority for a particular category. The
 * category is set via the setCategoryName() method which is called from the
 * LogConfigurer whenever the category selected in its tree changes.
 */
public class CategoryConfigure extends JPanel
{
    private JLabel categoryName;
    private JComboBox priority;
    private JButton setPriority;

    public CategoryConfigure()
    {
        setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
        setBorder(new TitledBorder("Select priority for category: "));

        JPanel temp = new JPanel();
        temp.setLayout(new BoxLayout(temp,BoxLayout.X_AXIS));
        categoryName = new JLabel(" ");
        temp.add(categoryName);
        temp.add(Box.createHorizontalGlue());
        add(temp);

        temp = new JPanel();
        temp.setLayout(new BoxLayout(temp,BoxLayout.X_AXIS));
        priority = new JComboBox();
        priority.addItem("");
        Priority [] priorities = Priority.getAllPossiblePriorities();
        for (int i=0; i<priorities.length; i++)
        {
            priority.addItem(priorities[i]);
        }
        priority.addActionListener(new ActionListener()
        {
            public void actionPerformed (ActionEvent event)
            {
                String name = categoryName.getText();
                Category category;
                if (name.equals(Category.getRoot().getName()))
                {
                    category = Category.getRoot();
                }
                else
                {
                    category = Category.getInstance(name);
                }
                Object selectedItem = priority.getSelectedItem();
                if ("".equals(selectedItem))
                {
                    if (category.equals(Category.getRoot()))
                    {
                        JOptionPane.showMessageDialog(CategoryConfigure.this,"The priority for the root category cannot be set to null.");
                        priority.setSelectedItem(Category.getRoot().getPriority());
                    }
                    else
                    {
                        //clear the priority so this category inherits it from its
                        //parent
                        category.setPriority(null);
                    }
                }
                else
                {
                    category.setPriority((Priority)priority.getSelectedItem());
                }
            }
        });
        temp.add(priority);
        temp.add(Box.createHorizontalGlue());
        add(temp);

/**
        temp = new JPanel();
        temp.setLayout(new BoxLayout(temp,BoxLayout.Y_AXIS));
        setPriority = new JButton("Set");
        setPriority.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent event)
            {
                Category category = Category.getInstance(categoryName.getText());
                category.setPriority((Priority)priority.getSelectedItem());
            }
        });
        temp.add(Box.createHorizontalGlue());
        temp.add(setPriority);
        add(temp);
**/
    }

    public void setCategory(String name)
    {
        categoryName.setText(name);
        Category category;
        if (name.equals(Category.getRoot().getName()))
        {
            category = Category.getRoot();
        }
        else
        {
            category = Category.getInstance(name);
        }
        Priority currentPriority = category.getPriority();
        priority.setSelectedItem(currentPriority);
    }
}
