/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.util.logging;

import org.vovida.prov.navigation.NavBarClient;
import org.vovida.prov.navigation.TopLevel;
import org.vovida.prov.psLib.ProvisioningInterface;
import org.vovida.prov.psLib.OperationException;

import java.util.Properties;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.awt.BorderLayout;
import javax.swing.JPanel;
import javax.swing.JMenuBar;

import org.apache.log4j.Category;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.PropertyConfigurator;

/**
 * @author Barbara Samson
 */
public class LogClient extends NavBarClient
{
  static Category logCategory = 
    Category.getInstance(LogClient.class.getName());

  public LogClient(String name, TopLevel parent)
  {
    super(name, parent);

    JPanel panel = new JPanel();

    panel.setLayout(new BorderLayout());

    LogScreen logScreen = new LogScreen();

    panel.add(logScreen, BorderLayout.CENTER);

    JMenuBar menuBar = new JMenuBar();

    menuBar.add(logScreen.getMenu());
    panel.add(menuBar, BorderLayout.NORTH);
    add(panel, BorderLayout.CENTER);
  }

  /**
   * Set up logging from configuration file
   */
  public static void configureFromFile()
  {

    // Until we get config file, log everything.
    BasicConfigurator.configure();
    logCategory.info("Logging configured from BasicConfigurator");

    ProvisioningInterface ps = ProvisioningInterface.getInstance();

    String logConfig = null;

    try
    {
      logConfig = ps.getRawFile("logConfig");
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
    }

    // If logConfig cannot be read, we get an error message instead.
    if ((logConfig != null) && (!logConfig.startsWith("<error>")))
    {
      Properties props = new Properties();

      try
      {
        props.load(new ByteArrayInputStream(logConfig.getBytes()));
      }
      catch (IOException e)
      {
        e.printStackTrace();
      }

      new PropertyConfigurator().doConfigure(props, 
              Category.defaultHierarchy);
      logCategory.warn("Logging configured from file");
    }

  }

}

