package org.vovida.prov.util.logging;

import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.JScrollPane;
import javax.swing.tree.*;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;

import java.awt.BorderLayout;

import java.util.Enumeration;
import java.util.StringTokenizer;

import org.apache.log4j.*;


public class LogConfigurer extends JPanel
{
    private static Category logCategory = Category.getInstance(LogConfigurer.class.getName());

    private DefaultTreeModel model;
    private JTree tree;
    private CategoryConfigure conf;

    public LogConfigurer()
    {
        DefaultMutableTreeNode root = new DefaultMutableTreeNode(Category.getRoot().getName());
        model = new DefaultTreeModel(root);
        tree = new JTree(model);
        JScrollPane scroller = new JScrollPane(tree);

        addCategories();
        initListener();

        setLayout(new BorderLayout());
        add(scroller, BorderLayout.CENTER);
        conf = new CategoryConfigure();
        add(conf, BorderLayout.SOUTH);
    }

    private void initListener()
    {
        tree.addTreeSelectionListener(new TreeSelectionListener()
        {
            public void valueChanged(TreeSelectionEvent event)
            {
                TreePath path = event.getNewLeadSelectionPath();
                String categoryName = "";
                Object [] pathComponents = path.getPath();
                if (pathComponents.length == 1)
                {
                    categoryName = Category.getRoot().getName();
                }
                else
                {
                    for (int i=0; i<pathComponents.length; i++)
                    {
                        DefaultMutableTreeNode component = (DefaultMutableTreeNode)pathComponents[i];
                        if (i != 0) //the first component is the root
                        {
                            categoryName += component.getUserObject().toString();
                        }
                        if (i != 0 && i != pathComponents.length-1)
                        {
                            categoryName += ".";
                        }
                    }
                }
                conf.setCategory(categoryName);
            }
        });
    }

    private void addCategories()
    {
        Enumeration categories = Category.getCurrentCategories();
        DefaultMutableTreeNode root = (DefaultMutableTreeNode)model.getRoot();

        while (categories.hasMoreElements())
        {
            String name = ((Category)categories.nextElement()).getName();
            StringTokenizer tokenizer = new StringTokenizer(name,".");
            DefaultMutableTreeNode currentNode = root;
            while (tokenizer.hasMoreElements())
            {
                String nameComponent = tokenizer.nextToken();
                currentNode = addNameComponent(nameComponent, currentNode);
            }
        }

        model.reload();
    }

    private DefaultMutableTreeNode addNameComponent(String name, DefaultMutableTreeNode parent)
    {
        Enumeration children = parent.children();
        while (children.hasMoreElements())
        {
            DefaultMutableTreeNode child = (DefaultMutableTreeNode)children.nextElement();
            if (child.getUserObject().equals(name))
            {
                //this node already exists in the tree
                return child;
            }
        }

        //didn't find this node so add it
        DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(name);
        parent.add(newNode);
        return newNode;
    }
}
