package org.vovida.prov.util.logging;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JMenuItem;
import javax.swing.ButtonGroup;
import javax.swing.BoxLayout;
import javax.swing.Box;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.Enumeration;

import org.apache.log4j.Category;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.Priority;

import org.vovida.prov.util.logging.TextPaneAppender;


/**
 * Contains a text pane which acts as a destination for logging messages. <p>
 * Also contains a menu bar which allows a single priority to be assigned to
 * all categories.
 */
public class LogScreen extends JPanel
{
    JMenu menu = null;
    TextPaneAppender appender;
    JFrame configure = null;

    public LogScreen ()
    {
        appender = new TextPaneAppender(new PatternLayout("%-6p %d %c:  %m%n"),"Debug");
        Category.getRoot().addAppender(appender);
        //log to the text console as well
//		Category.getRoot().addAppender(new ConsoleAppender(new PatternLayout("[%t] %-6p %c{3} - %m%n")));
        JScrollPane scroller = new JScrollPane(appender.getTextPane());
        setLayout(new BorderLayout());
        add(scroller,BorderLayout.CENTER);
        validate();
    }

    private class PriorityChangeListener implements ActionListener
    {
        private Priority priority;

        public PriorityChangeListener (Priority aPriority)
        {
            priority = aPriority;
        }

        public void actionPerformed(ActionEvent event)
        {
            //clear the priority for all categories so that they inherit from
            //the root category
            Enumeration allCategories = Category.getCurrentCategories();
            while (allCategories.hasMoreElements())
            {
                Category category = (Category)allCategories.nextElement();
                category.setPriority(null);
            }

            JRadioButtonMenuItem item = (JRadioButtonMenuItem)event.getSource();
            if (item.isSelected())
            {
                Category.getRoot().setPriority(priority);
            }
        }
    }

    public JMenu getMenu()
    {
        if (menu == null)
        {
            menu = new JMenu("Log");

            JMenuItem item = new JMenuItem("Clear");
            item.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    appender.getTextPane().setText("");
                }
            });
            menu.add(item);

            item = new JMenuItem ("Configure ...");
            item.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    if (configure == null)
                    {
                        configure = new JFrame("Configure log categories");
                        configure.getContentPane().setLayout(new BorderLayout());
                        configure.getContentPane().add(new LogConfigurer(), BorderLayout.CENTER);
                        JPanel temp = new JPanel();
                        temp.setLayout(new BoxLayout(temp, BoxLayout.X_AXIS));
                        temp.add(Box.createHorizontalGlue());
                        JButton close = new JButton("Close");
                        close.addActionListener(new ActionListener()
                        {
                            public void actionPerformed(ActionEvent event)
                            {
                                configure.setVisible(false);
                            }
                        });
                        temp.add(close);
                        configure.getContentPane().add(temp, BorderLayout.SOUTH);
                        configure.pack();
                        configure.setSize(300,400);

				        Dimension dimension = Toolkit.getDefaultToolkit().getScreenSize();
				        configure.setLocation((dimension.width - configure.getSize().width)/2,
				            (dimension.height - configure.getSize().height)/2);
                    }
                    configure.setVisible(true);
                }
            });
            menu.add(item);
        }
        return menu;
    }
}
