/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.widget;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.util.List;
import java.awt.event.ItemListener;
import java.awt.Container;
import javax.swing.JCheckBox;
import javax.swing.JComponent;

import org.jdom.Element;

/**
 * @author Barbara Samson
 */
public class CheckBox extends GuiComponent implements ListGuiComponent
{
  private JCheckBox myCheckBox;

  /**
   * Class Constructor.
   * @param widgetElement could be null.
   */
  public CheckBox(String label, Element widgetElement)
  {
    super(widgetElement);

    myCheckBox = new JCheckBox(label, false);
  }

  /**
   * Cloning allows new table rows to be created.
   */
  public Object clone() throws CloneNotSupportedException
  {
    CheckBox checkBox = (CheckBox) super.clone();
    JCheckBox box = new JCheckBox();

    box.setHorizontalAlignment(JCheckBox.CENTER);

    checkBox.myCheckBox = box;

    checkBox.setSelection(getCurrentSelection());

    return checkBox;
  }

  public Component getTableCellRendererComponent(String value)
  {
    return myCheckBox;
  }

  public JComponent getEditorComponent()
  {
    return myCheckBox;
  }

  public JComponent getLabelComponent()
  {
    return myCheckBox;
  }

  public void setTooltip(String tooltip)
  {
    myCheckBox.setToolTipText(tooltip);
  }

  public void setLabel(String label)
  {
    myCheckBox.setText(label);
  }

  public String getLabel()
  {
    return myCheckBox.getText();
  }

  public void renderSelf(Container container)
  {
    if (!isHidden())
    {
      container.add(myCheckBox);
    }
  }

  public void addChoice(Object choice)
  {

    // we already know it can only be true or false
  }

  public void addListener(ListListenerAdapter listener)
  {
    myCheckBox.addItemListener((ItemListener) listener);
  }

  public String getCurrentSelection()
  {
    String retValue = (myCheckBox.isSelected()) ? "true" : "false";

    return retValue;
  }

  public void setSelection(String newSelection)
  {
    if (newSelection.equals("true"))
    {
      myCheckBox.setSelected(true);
    }
    else
    {
      myCheckBox.setSelected(false);
    }
  }

  public void addActionListener(ActionListener listener)
  {
    myCheckBox.addActionListener(listener);
  }

}




