/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.widget;

import org.vovida.prov.widget.GuiException;
import org.vovida.prov.widget.GuiComponent;

import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.Vector;
import java.awt.Container;
import javax.swing.JComboBox;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JLabel;
import javax.swing.JComponent;
import javax.swing.event.ListDataListener;

import org.jdom.Element;

/**
 * @author Barbara Samson
 */
public class ComboBox extends GuiComponent implements ListGuiComponent
{
  private JComboBox myComboBox;
  private DefaultComboBoxModel myComboBoxModel;
  private JLabel myLabel;

  /**
   * Class constructor.
   * @param label The initial string given to myLabel. Note that this
   * may change.
   * @param widgetElement could be null.
   */

  public ComboBox(String label, Element widgetElement)
  {
    super(widgetElement);

    myLabel = new JLabel(label);
    myComboBox = new JComboBox();
    myComboBoxModel = new DefaultComboBoxModel();

    myComboBox.setModel(myComboBoxModel);

    Font font = myComboBox.getFont();

    myComboBox.setFont(new Font(font.getName(), font.PLAIN, font.getSize()));
  }

  protected void setModel(DefaultComboBoxModel model)
  {
    myComboBoxModel = model;
  }

  /**
   * Cloning allows new table rows to be created.
   */
  public Object clone() throws CloneNotSupportedException
  {
    ComboBox clone = (ComboBox) super.clone();
    JComboBox comboBox = new JComboBox();
    DefaultComboBoxModel cloneModel = new DefaultComboBoxModel();

    comboBox.setModel(cloneModel);
    clone.setModel(cloneModel);

    Font font = comboBox.getFont();

    comboBox.setFont(new Font(font.getName(), font.PLAIN, font.getSize()));

    clone.myComboBox = comboBox;

    clone.setUserInputOK(myComboBox.isEditable());

    int itemCount = myComboBox.getItemCount();

    for (int i = 0; i < itemCount; i++)
    {
      clone.addChoice(myComboBox.getItemAt(i));
    }

    return clone;
  }

  public Component getTableCellRendererComponent(String value)
  {
    return myComboBox;
  }

  public void setUserInputOK(boolean inputOK)
  {
    myComboBox.setEditable(inputOK);
  }

  public JComponent getEditorComponent()
  {
    return myComboBox;
  }

  public JComponent getLabelComponent()
  {
    return myLabel;
  }

  /**
   * Change the text of the label. This happens when we encounter a
   * label annotation in the Schema.
   */
  public void changeLabel(String newLabel)
  {
    myLabel.setText(newLabel);
  }

  public void renderSelf(Container parent) throws GuiException
  {
    if (!isHidden())
    {
      parent.add(myLabel);
      parent.add(myComboBox);
      parent.add(mySymbol);
    }
  }

  public void setLabel(String newLabel)
  {
    myLabel.setText(newLabel);
  }

  public String getLabel()
  {
    return myLabel.getText();
  }

  public void setTooltip(String tooltip)
  {
    myLabel.setToolTipText(tooltip);
  }

  public void addChoice(Object choice)
  {
    myComboBoxModel.addElement(choice);
  }

  public String getCurrentSelection()
  {
    Object selectedItem = myComboBox.getModel().getSelectedItem();

    if (selectedItem != null)
    {
      return (selectedItem.toString());
    }

    return "";
  }

  public void addListener(ListListenerAdapter myListener)
  {
    myComboBox.getModel().addListDataListener((ListDataListener) myListener);
    myListener.contentsChanged(null);
  }

  public void setSelection(String newSelection)
  {
    myComboBox.getModel().setSelectedItem(newSelection);
  }

  public void addActionListener(ActionListener listener)
  {
    myComboBox.addActionListener(listener);
  }

  public void addListDataListener(ListDataListener listener)
  {
    myComboBox.getModel().addListDataListener(listener);
  }

}

