/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.widget;

import org.vovida.prov.dynamicGui.DTDConstants;

import java.util.List;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.jdom.Element;

import org.apache.log4j.Category;

/**
 * This class takes schema information as it is parsed by the TreeBuilder
 * and uses that information to create a UI object, which will be associated
 * with the ContentData of the node that the TreeBuilder is currently building.
 *
 * @author Barbara Samson
 */
public class GuiFactory implements DTDConstants
{
  static Category logCategory = 
    Category.getInstance(GuiFactory.class.getName());


  public static GuiComponent newGuiComponent(String label, 
          Element widgetElement)
  {
    String typeName = TEXTFIELD;

    if (widgetElement != null)
    {
      List childList = widgetElement.getChildren();

      if (!childList.isEmpty())
      {
        Element child = (Element) childList.get(0);

        typeName = child.getName();
      }
    }

    GuiComponent widget = null;

    // default widget is TextField
    Class widgetClass = null;

    try
    {

      // First try to get the class from the widget package.
      // If that doesn't work, try getting the class as given.
      try
      {
        widgetClass = Class.forName("org.vovida.prov.widget." + typeName);
        typeName = "org.vovida.prov.widget." + typeName;
      }
      catch (ClassNotFoundException e)
      {
        widgetClass = Class.forName(typeName);
      }

      Constructor widgetConstructor = widgetClass.getConstructor(new Class[]
      {
        String.class, Element.class
      });

      widget = (GuiComponent) widgetConstructor.newInstance(new Object[]
      {
        label, widgetElement
      });
    }
    catch (ClassCastException e1)
    {
      logCategory.error("ClassCastException:\n" + typeName 
                        + " is not of type GuiComponent");
    }
    catch (ClassNotFoundException e2)
    {
      logCategory.error("ClassNotFoundException:\n" + typeName 
                        + " cannot be found");
    }
    catch (InstantiationException e3)
    {
      logCategory.error("InstantiationException:\n" + typeName 
                        + " cannot be instantiated");
    }
    catch (IllegalAccessException e4)
    {
      logCategory.error("IllegalAccessException for " + typeName + ":\n" 
                        + e4.getMessage());
    }
    catch (NoSuchMethodException e5)
    {
      logCategory.error("NoSuchMethodException for " + typeName + ":\n" 
                        + "Expected Constructor does not exist");
    }
    catch (InvocationTargetException e6)
    {
      logCategory.error("InvocationTargetException for " + typeName + ":\n" 
                        + e6.getMessage());
      e6.printStackTrace();
    }
    catch (IllegalArgumentException e7)
    {
      logCategory.error("IllegalArgumentException in Constructor for " 
                        + typeName + ":\n" + e7.getMessage());
    }

    return widget;
  }

  /**
   * Create a new panel in the GUI.
   */
  public static LayoutPanel newPanel(String label)
  {
    return new LayoutPanel(label);
  }

  public static java.awt.Container newPagedContainer()
  {
	javax.swing.JTabbedPane pane = new javax.swing.JTabbedPane();
    return pane;
  }

  public static java.awt.Container newPage()
  {
    SmartLayoutManager layoutManager = new SmartLayoutManager();
    javax.swing.JPanel panel = new javax.swing.JPanel();
    panel.setLayout(layoutManager);
    return panel;
  }
}

