/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.widget;

import org.vovida.prov.dynamicGui.DTDConstants;
import org.vovida.prov.content.ContentData;
import org.vovida.prov.content.HostPortType;

import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JComponent;
import java.awt.FlowLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import javax.swing.text.Document;

import org.jdom.Element;

/**
 * Renders a JLabel JTextField pair.
 *
 * @author Barbara Samson
 */
public class HostPortField extends TextField implements DTDConstants
{
  private ComboBox myHostField;

  private TextField myPortField;

  // a "required" symbol for the port field
  private JLabel myPortSymbol;

  // a "required symbol for the host field
  private JLabel myHostSymbol;

  private JPanel myPanel;

  /**
   * Class constructor.
   */
  public HostPortField(String label, Element element)
  {

    // we ignore the passed-in label
    super(HostPortType.HOSTLABEL, element);

    myPanel = new JPanel();

    myPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

    Element hostElem = new Element(WIDGET);

    hostElem.addContent(new Element(COMBOBOX));

    myHostField = 
      (ComboBox) GuiFactory.newGuiComponent(HostPortType.HOSTLABEL, hostElem);

    myHostField.setEnabled(true);

    Element portElem = new Element(WIDGET);

    portElem.addContent(new Element(TEXTFIELD));

    myPortField = 
      (TextField) GuiFactory.newGuiComponent(HostPortType.PORTLABEL, 
            portElem);

    ((JTextField) myPortField.getEditorComponent()).setColumns(PORT_SIZE);

    try
    {
      myHostField.renderSelf(myPanel);

      myHostSymbol = new JLabel();

      myPanel.add(myHostSymbol);
      myPortField.renderSelf(myPanel);

      myPortSymbol = new JLabel();

      myPanel.add(myPortSymbol);
    }
    catch (GuiException e)
    {
      e.printStackTrace();
    }
  }

  /**
   * Cloning allows new table rows to be created.
   */
  public Object clone() throws CloneNotSupportedException
  {
    HostPortField clone = (HostPortField) super.clone();

    clone.setHostField((ComboBox) GuiFactory.newGuiComponent
      (HostPortType.HOSTLABEL, 
            new Element(COMBOBOX)));
    clone.setPortField((TextField) GuiFactory.newGuiComponent
      (HostPortType.PORTLABEL, 
            new Element(TEXTFIELD)));

    return clone;
  }

  public void setHostField(ComboBox field)
  {
    myHostField = field;
  }

  public void setPortField(TextField field)
  {
    myPortField = field;
  }

  public ComboBox getHostField()
  {
    return myHostField;
  }

  public TextField getPortField()
  {
    return myPortField;
  }

  /**
   * Returns a reference to the ComboBox for Host.
   * There is currently no way to get the TextField for Port.
   */
  public JComponent getEditorComponent()
  {
    return myHostField.getEditorComponent();
  }

  public void renderSelf(Container parent) throws GuiException
  {
    if (!isHidden())
    {
      setSymbolProperties(myPortSymbol);
      setSymbolProperties(myHostSymbol);
      parent.add(myPanel);
    }
  }

  public void setEnabled(boolean canEdit)
  {
    super.setEnabled(canEdit);
    myPortField.setEnabled(canEdit);
  }

}




