/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.widget;

import org.vovida.prov.content.ListType;

import java.util.List;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import javax.swing.ButtonModel;
import javax.swing.DefaultComboBoxModel;

import org.apache.log4j.Category;


/**
 * This class exists as an adapter between the change notification
 * mechanism of DefaultComboBoxModel and ButtonModel.
 *
 * @author Barbara Samson
 */
public class ListListenerAdapter implements ActionListener, ListDataListener, 
        ItemListener
{
  private ListType myListener;
  private ListGuiComponent myGuiComponent;

  static Category logCategory = 
    Category.getInstance(ListListenerAdapter.class.getName());

  public ListListenerAdapter(ListType content)
  {
    myListener = content;
  }

  public void setSelection(String newSelection)
  {
    myGuiComponent.setSelection(newSelection);
  }

  /**
   * Comes from a ButtonModel
   */
  public void actionPerformed(ActionEvent e)
  {
    String selection = myGuiComponent.getCurrentSelection();

    myListener.selectionChanged(selection);
  }

  /**
   * Comes from a DefaultComboBoxModel
   */
  public void contentsChanged(ListDataEvent e)
  {
    String selection = myGuiComponent.getCurrentSelection();

    myListener.selectionChanged(selection);
  }

  /**
   * Comes from a CheckBox
   */
  public void itemStateChanged(ItemEvent e)
  {
    String selection = myGuiComponent.getCurrentSelection();

    myListener.selectionChanged(selection);
  }

  public void intervalAdded(ListDataEvent e) {}

  public void intervalRemoved(ListDataEvent e) {}

  public void addChoice(Object choice)
  {
    myGuiComponent.addChoice(choice);
  }

  public void setUpListener(GuiComponent gui)
  {
    if (gui instanceof ListGuiComponent)
    {
      myGuiComponent = (ListGuiComponent) gui;

      myGuiComponent.addListener(this);
    }
    else
    {
      logCategory.warn("setUpListener called with " + gui);
    }

    // myGuiComponent.setListOfChoices(choices);
  }

}




