/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.widget;

import java.util.Vector;
import java.util.List;
import java.util.ListIterator;
import java.util.Enumeration;
import java.awt.Container;
import javax.swing.JPanel;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.Color;
import javax.swing.JLabel;
import javax.swing.JComponent;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import java.awt.event.ActionListener;

import org.jdom.Element;

/**
 * @author Barbara Samson
 */
public class RadioButtons extends GuiComponent implements ListGuiComponent
{

  private ButtonGroup myGroup = new ButtonGroup();
  private JPanel mainPanel;
  private JPanel buttonPanel;
  private JLabel myLabel = new JLabel();

  /**
   * Class Constructor.
   * @param widgetElement could be null.
   */
  public RadioButtons(String label, Element widgetElement)
  {
    super(widgetElement);

    setLabel(label);

    mainPanel = new JPanel();

    mainPanel.setLayout(new FlowLayout(FlowLayout.CENTER));

    JPanel labelPanel = new JPanel();

    labelPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
    labelPanel.add(myLabel);

    buttonPanel = new JPanel();

    buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER));

    JPanel symbolPanel = new JPanel();

    symbolPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
    symbolPanel.add(mySymbol);
    mainPanel.add(labelPanel);
    mainPanel.add(buttonPanel);
    mainPanel.add(symbolPanel);
  }

  public void addChoice(Object choice)
  {
    String nameOfButton = choice.toString();
    JRadioButton button = new JRadioButton(nameOfButton);

    myGroup.add(button);
  }

  public String getCurrentSelection()
  {
    Enumeration enum = myGroup.getElements();

    while (enum.hasMoreElements())
    {
      JRadioButton button = (JRadioButton) enum.nextElement();

      if (button.isSelected())
      {
        return (button.getText());
      }
    }

    return "";
  }

  public JComponent getEditorComponent()
  {
    return null;
  }

  public void setEnabled(boolean canEdit)
  {
    Enumeration enum = myGroup.getElements();

    while (enum.hasMoreElements())
    {
      JRadioButton button = (JRadioButton) enum.nextElement();

      button.setEnabled(canEdit);
      myLabel.setEnabled(canEdit);
    }
  }

  public JComponent getLabelComponent()
  {
    return myLabel;
  }

  public void setTooltip(String tooltip)
  {
    myLabel.setToolTipText(tooltip);
  }

  public String getLabel()
  {
    return myLabel.getText();
  }

  public void setLabel(String label)
  {
    myLabel.setText(label);
  }

  public void renderSelf(Container container)
  {
    if (!isHidden())
    {
      Enumeration enum = myGroup.getElements();

      while (enum.hasMoreElements())
      {
        JRadioButton button = (JRadioButton) enum.nextElement();

        container.add(button);
        buttonPanel.add(button);
      }

      container.add(mainPanel);
    }
  }

  public void addListener(ListListenerAdapter myListener)
  {
    Enumeration enum = myGroup.getElements();

    while (enum.hasMoreElements())
    {
      JRadioButton button = (JRadioButton) enum.nextElement();

      button.getModel().addActionListener((ActionListener) myListener);
    }
  }

  public void setSelection(String newSelection)
  {
    Enumeration enum = myGroup.getElements();

    while (enum.hasMoreElements())
    {
      JRadioButton button = (JRadioButton) enum.nextElement();

      if (button.getText().equals(newSelection))
      {
        button.doClick();
      }
    }
  }

  public void addActionListener(ActionListener listener)
  {
    Enumeration buttons = myGroup.getElements();

    while (buttons.hasMoreElements())
    {
      ((JRadioButton) buttons.nextElement()).addActionListener(listener);
    }
  }


}




