/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.widget;

import org.vovida.prov.content.TableType;
import org.vovida.prov.content.MyTableModel;

import java.util.Enumeration;
import java.awt.Component;
import javax.swing.JTable;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.table.JTableHeader;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.Container;
import javax.swing.border.LineBorder;
import java.awt.Color;
import java.awt.GridLayout;
import javax.swing.border.Border;
import javax.swing.JButton;
import javax.swing.border.TitledBorder;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.UIManager;
import javax.swing.table.TableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableCellEditor;

import org.jdom.Element;

/**
 * @author Barbara Samson
 */
public class Table extends GuiComponent
{
  private String myLabel;

  /**
   * Used only to determine disabled color.
   */
  private JTextField myTextField = new JTextField();
  private Color borderColor;
  private TableWidget myTable;
  private JPanel myPanel = new JPanel();
  private TitledBorder myBorder;
  private JButton myAddButton = new JButton("Add Row");
  private JButton myDeleteButton = new JButton("Delete Row");
  private JButton myMoveUpButton = new JButton("Move Up");
  private JButton myMoveDownButton = new JButton("Move Down");

  /**
   * Class Constructor.
   * @param widgetElement could be null.
   */
  public Table(String label, Element widgetElement)
  {
    super(widgetElement);

    myLabel = label;
  }

  public void setContent(final TableType content)
  {
    myTable = new TableWidget(content.getTableModel());

    myTable.setAutoCreateColumnsFromModel(false);
    myPanel.setLayout(new BorderLayout());

    JScrollPane scrollPane = new JScrollPane();

    scrollPane.getViewport().add(myTable);
    myPanel.add(scrollPane, BorderLayout.CENTER);

    myBorder = new TitledBorder(myLabel);

    myPanel.setBorder(myBorder);

    borderColor = myBorder.getTitleColor();

    myAddButton.setForeground(borderColor);
    myAddButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        TableCellEditor editor = myTable.getCellEditor();

        if (editor != null)
        {
          editor.stopCellEditing();
        }

        int selectedRow = myTable.getSelectedRow();

        content.addRowAfter(selectedRow);
      }

    });
    myDeleteButton.setForeground(borderColor);
    myDeleteButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        int selectedRow = myTable.getSelectedRow();

        if (selectedRow > -1)
        {
          content.deleteRow(selectedRow);
        }
      }

    });
    myMoveUpButton.setForeground(borderColor);
    myMoveUpButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        TableCellEditor editor = myTable.getCellEditor();

        if (editor != null)
        {
          editor.stopCellEditing();
        }

        int selectedRow = myTable.getSelectedRow();
        boolean success = 
          ((MyTableModel) myTable.getModel()).tryToMoveUp(selectedRow);

        if (success)
        {
          myTable.setRowSelectionInterval(selectedRow - 1, selectedRow - 1);
          myTable.scrollRectToVisible(myTable.getCellRect(selectedRow - 1, 
                  myTable.getSelectedColumn(), false));
        }
      }

    });
    myMoveDownButton.setForeground(borderColor);
    myMoveDownButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        TableCellEditor editor = myTable.getCellEditor();

        if (editor != null)
        {
          editor.stopCellEditing();
        }

        int selectedRow = myTable.getSelectedRow();
        boolean success = 
          ((MyTableModel) myTable.getModel()).tryToMoveDown(selectedRow);

        if (success)
        {
          myTable.setRowSelectionInterval(selectedRow + 1, selectedRow + 1);
          myTable.scrollRectToVisible(myTable.getCellRect(selectedRow + 1, 
                  myTable.getSelectedColumn(), false));
        }
      }

    });

    JPanel buttonPanel = new JPanel();
    JPanel gridPanel = new JPanel(new GridLayout(1, 4));

    gridPanel.add(myAddButton);
    gridPanel.add(myDeleteButton);
    gridPanel.add(myMoveUpButton);
    gridPanel.add(myMoveDownButton);
    buttonPanel.add(gridPanel);
    myPanel.add(buttonPanel, BorderLayout.SOUTH);
    myTable.setPreferredScrollableViewportSize(new Dimension(300, 55));
  }

  public JComponent getEditorComponent()
  {
    return myTable;
  }

  public void setEnabled(boolean isEditable)
  {
    TableCellEditor editor = myTable.getCellEditor();

    if (editor != null)
    {
      editor.stopCellEditing();
    }

    myTable.setEnabled(isEditable);
    myTable.setCellSelectionEnabled(isEditable);
    myAddButton.setEnabled(isEditable);
    myDeleteButton.setEnabled(isEditable);
    myMoveUpButton.setEnabled(isEditable);
    myMoveDownButton.setEnabled(isEditable);
    myTable.setForeground(isEditable ? Color.black 
            : myTextField.getDisabledTextColor());
    myTable.getTableHeader().setForeground(isEditable ? Color.black 
            : myTextField.getDisabledTextColor());
    myBorder.setTitleColor(isEditable ? borderColor 
            : myTextField.getDisabledTextColor());
    myPanel.repaint();
  }

  public JComponent getLabelComponent()
  {
    return myTable;
  }

  public void addColumn(TableColumn column)
  {
    myTable.addColumn(column);
  }

  public void renderSelf(Container parent) throws GuiException
  {
    if (!isHidden())
    {
      parent.add(myPanel);
    }
  }

  public void setLabel(String newLabel)
  {
    myLabel = newLabel;
  }

  public String getLabel()
  {
    return myLabel;
  }

  public void setTooltip(String tooltip)
  {
    myPanel.setToolTipText(tooltip);
  }

  public void add(java.awt.Component child) {}

  public void add(java.awt.Component child, Object constraints) {}

  /**
   * @link aggregation
   */
  private JTable lnkJTable;

  /**
   * @link aggregation
   */
  private JLabel lnkJLabel;
}




