/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

package org.vovida.prov.widget;

import org.vovida.prov.content.ContentData;

import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JComponent;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;

import org.jdom.Element;

/**
 * Renders a JLabel JTextField pair.
 */
public class TextField extends GuiComponent
{

  /**
   * @link aggregation
   */
  protected JTextField myTextField;

  /**
   * @link aggregation
   */
  protected JLabel myLabel;


  /**
   * Class constructor.
   * @param label The initial string given to myLabel.
   * Note that this may change.
   * @param widgetElement could be null.
   */
  public TextField(String label, Element widgetElement)
  {
    super(widgetElement);

    myLabel = new JLabel(label);
    myTextField = new JTextField(FIELD_SIZE);
  }

  /**
   * Cloning allows new table rows to be created.
   */
  public Object clone() throws CloneNotSupportedException
  {
    TextField clone = (TextField) super.clone();

    clone.setTextField(new JTextField());

    return clone;
  }

  public void setModel(Document doc)
  {
    myTextField.setDocument(doc);
  }

  public JComponent getEditorComponent()
  {
    return myTextField;
  }

  public JComponent getLabelComponent()
  {
    return myLabel;
  }

  /**
   * Change the text of the label. This happens when we encounter a
   * label annotation in the Schema.
   */
  public void changeLabel(String newLabel)
  {
    myLabel.setText(newLabel);
  }

  public void renderSelf(Container parent) throws GuiException
  {
    if (!isHidden())
    {
      parent.add(myLabel);
      parent.add(myTextField);
      parent.add(mySymbol);
    }
  }

  public void setLabel(String newLabel)
  {
    myLabel.setText(newLabel);
  }

  public String getLabel()
  {
    return myLabel.getText();
  }

  public void setTooltip(String tooltip)
  {
    myLabel.setToolTipText(tooltip);
  }

  protected void setTextField(JTextField field)
  {
    myTextField = field;
  }

  public void addActionListener(ActionListener listener)
  {
    myTextField.addActionListener(listener);
  }

  public void addDocumentListener(DocumentListener listener)
  {
    myTextField.getDocument().addDocumentListener(listener);
  }

}




