/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const FileDataStoreWrapper_cxx_Version =
    "$Id: FileDataStoreWrapper.cxx,v 1.4 2001/06/29 20:26:52 bko Exp $";

#include <stdio.h>
#include <unistd.h>
#include "FileDataStore.hxx"
#include "VIoException.hxx"
#include "cpLog.h"
#include <iostream>

void putFile (	int nbins,
               string psRoot,
               string dirPath,
               string filename,
               string fileContents )
{
    FileDataStore dataStore(nbins, psRoot);
    cout << "Attempting to write " << filename
    << " to " << dirPath << " at root " << psRoot << endl;
    try
    {
        dataStore.putItem(dirPath, filename, fileContents, 0);
    }
    catch (VIoException& e)
    {
        string reason = e.getDescription();
        cerr << "Failed to write item " << filename << ": " << reason.c_str() << endl;
        return ;
    }
}

int main(int argc, char** argv)
{
    char c;
    extern char *optarg;
    int nbins = 6;
    string psRoot = "/usr/local/provisioning_data";
    string dirpath = "";
    string filename = "";
    string fileContents = "";

    try
    {
        // parse command line options
        while (1)
        {
            c = getopt(argc, argv, "b:r:d:f:c:");
            if (c == -1)
            {
                break;
            }
            switch (c)
            {
                case 'b':
                nbins = atoi(optarg);
                break;
                case 'r':
                psRoot = optarg;
                break;
                case 'd':
                dirpath = optarg;
                break;
                case 'f':
                filename = optarg;
                break;
                case 'c':
                fileContents = optarg;
                break;
                default:
                cerr << "Usage: FileDataStoreWrapper -b <number bins>" << endl;
                cerr << "                            -r <file system root>" << endl;
                cerr << "                            -d <directory path>" << endl;
                cerr << "                            -f <file name>" << endl;
                cerr << "                            -c <file contents>" << endl;
            }
        }
    }
    catch (...)
    {
        cerr << "Application exiting, reason:Unknown" << endl;
    }
    putFile(nbins, psRoot, dirpath, filename, fileContents);
}
