/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const FileDataStoreWrapper_cxx_Version =
    "$Id: InstallFileDataStore.cxx,v 1.3 2002/04/17 03:33:27 bko Exp $";

#include <stdio.h>
#include <unistd.h>
#include "FileDataStore.hxx"
#include "VIoException.hxx"
#include "cpLog.h"
#include <iostream>
#include <fstream>
#include "vocalconfig.h"

const char* defaultPsRoot = VOCAL_INSTALL_PATH "/provisioning_data";
const int defaultBins = 6;

bool verbose = false;


/// Obtain the contents of the given file
bool 
getContents ( const string& filename, string* fileContents )
{
    ifstream inRead(filename.c_str());
    if(inRead.bad())
    {
        return false;
    }

    char c;
    while(inRead.get(c))
    {
	*fileContents += c;
    }

    if(inRead.bad())
    {
        return false;
    }

    return true;
} /// end of getContents


void putFile (	
    int nbins,
    string psRoot,
    string dirPath,
    string filename )
{

    string fileContents;
    if(!getContents(filename, &fileContents))
    {
	cerr << "could not read from filename." << endl;
	exit(-1);
    }

    if(verbose)
    {
	cout << "contents:\n" << fileContents << endl;
    }

    int pos = filename.rfind(".xml");

    if(pos != EOF)
    {
	if((pos + 4) == static_cast<int>(filename.size()))
	{
	    filename.erase(pos, 4);
	}
    }

    FileDataStore dataStore(nbins, psRoot);
    if(verbose)
    {
	cout << "Attempting to write " 
	     << filename
	     << " to " 
	     << dirPath 
	     << " at root " 
	     << psRoot << endl;
    }
    try
    {
        dataStore.putItem(dirPath, filename, fileContents, 0);
    }
    catch (VIoException& e)
    {
        string reason = e.getDescription();
        cerr << "Failed to write item " << filename << ": " << reason.c_str() << endl;
        return ;
    }
}


void usage(const char* program)
{
    cerr << "InstallFileDataStore installs the file filename into\n" 
	 << "the provisioning file directory provisioning-root.  If the data\n"
	 << "belongs in a subdirectory (i.e. the SystemConfiguration directory)\n"
	 << "that should be specified as the second argument, or . for no\n"
	 << "subdirectory.\n"
	 << endl;

    cerr << "Usage:\n" 
	 << "  " << program << " [options...] filename directory-path\n"
	 << "\n"
	 << "Options:\n"
	 << "        -v             turn on verbose mode\n"
	 << "        -b <bin-count> [default: " << defaultBins << " ]\n"
	 << "        -r <provisioning-root> [default: " << defaultPsRoot << " ]\n"
	 << endl;
}


int main(int argc, char** argv)
{
    char c;
    extern char *optarg;
    int nbins = defaultBins;
    string psRoot = defaultPsRoot;
    string dirpath = "";
    string sourceFilename = "";

    try
    {
        // parse command line options
        while (1)
        {
            c = getopt(argc, argv, "vb:r:");
            if (c == -1)
            {
                break;
            }
            switch (c)
            {
	    case 'v':
		verbose = true;
		break;

	    case 'b':
                nbins = atoi(optarg);
                break;
                
	    case 'r':
                psRoot = optarg;
                break;

	    default:
		usage(argv[0]);
		exit(-1);
		break;
            }
        }

	if((argc - optind) != 2)
	{
	    usage(argv[0]);
	    exit(-1);
	}

	sourceFilename = argv[0 + optind];
	dirpath = argv[1 + optind];
	putFile(nbins, psRoot, dirpath, sourceFilename);
    }
    catch (...)
    {
        cerr << "Application exiting, reason:Unknown" << endl;
    }
}
