/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const PutMultipleFiles_cxx_Version =
    "$Id: PutMultipleFiles.cxx,v 1.2 2001/06/29 20:26:52 bko Exp $";


#include <fstream.h>
#include <strstream>
#include <stdio.h>
#include <unistd.h>
#include "FileDataStore.hxx"
#include "VFile.hxx"
#include "VIoException.hxx"
#include "cpLog.h"
#include <iostream>

/// Create a file in the given directory 
void putFile (	int nbins,
               string psRoot,
               string dirPath,
               string filename,
               string fileContents )
{
    FileDataStore dataStore(nbins, psRoot);
    cout << "Attempting to write " << filename
    << " to " << dirPath << " at root " << psRoot << endl;
    try
    {
        dataStore.putItem(dirPath, filename, fileContents);
    }
    catch (VIoException& e)
    {
        string reason = e.getDescription();
        cerr << "Failed to write item " << filename << ": " << reason.c_str() << endl;
        return ;
    }
} /// end of putFile


/// Obtain the contents of the given file
void getContents ( const string& filename, string& fileContents )
{
    char buf[256];
    VFile vf( "./", filename );
    ifstream inRead(vf.getFullName().c_str());
    if(inRead.bad())
    {
        ostrstream ostr(buf, 256);
        ostr << "Failed to open config file ("
             << vf.getFullName()
             << ")."
             << ends;
        cpLog(LOG_DEBUG, ostr.str());
        return;
    }

    while(inRead.getline(buf, 256))
    {
        fileContents += buf;
        fileContents += '\n';
    }

} /// end of getContents


/**
 * Num of bins, path to provisioning_data, target directory, starting number, number of users
 */
int main(int argc, char** argv)
{

    int num = atoi(argv[1]);
    int nbins = (num/200)+1; /// Algorithm for determining the number of hash bins.
    string psRoot = argv[2]; /// Path to provisioning_data directory
    string dirpath = argv[3]; /// Directory name in which files will be placed
    int idBase = atoi(argv[4]); /// Starting phone number
    int numUsers = atoi(argv[5]); /// Number of users to create
    string fileName = "";
    string fileContents = "";

/*
    cout << "PutMultipleFiles" << endl;
    cout << "num: " << num << endl;
    cout << "num bins: " << nbins << endl;
    cout << "psRoot: " << psRoot << endl;
    cout << "dirpath: " << dirpath << endl;
    cout << "idBase: " << idBase << endl;
    cout << "numUsers: " << numUsers << endl;
*/

    int currentUser = idBase;
    int finalUser = idBase + numUsers;
    /// loop through the users and put their files into the correct hash directory
    while (currentUser < finalUser)
    {
        char buf[16];
        sprintf(buf, "%d", currentUser);
        fileName.assign(buf);

        /// cout << "Current user: " << fileName << endl;
        /// clear out file contents
        fileContents = "";
        getContents(fileName, fileContents);

        putFile(nbins, psRoot, dirpath, fileName, fileContents);

        ++currentUser;
    }
} /// end of main

/// end of file
