/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

/*
 * The Apache Software License, Version 1.1
 * 
 * 
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 * 
 * 4. The names "Xerces" and "Apache Software Foundation" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact apache@apache.org.
 * 
 * 5. Products derived from this software may not be called "Apache",
 * nor may "Apache" appear in their name, without prior written
 * permission of the Apache Software Foundation.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 * 
 * Based on DOMTree by Andy Clark, IBM
 * Note that the Apache license applies to the original
 * software and the Vovida license applies to changes
 * to that software.
 */
package vocal.data;

import java.util.Vector;
import java.io.Serializable;
import javax.swing.JTree;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreeSelectionModel;

import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Attr;
import org.w3c.dom.traversal.NodeFilter;

import java.util.Hashtable;
import java.util.Enumeration;

/**
 * This class manages the JTree that is the basis of
 * navigation through server-side provisioning.
 * This class has an outer and inner component. The outer
 * component provides an API for the GUI classes in the ui package.
 * Each type of screen (ie Marshal, Feature, Group, etc)
 * has its own pair of methods to call, either to populate
 * its textfields with data from an existing server or to set
 * it up for creating a new server.
 * <p>
 * The inner class component contains a Document Object Model
 * that holds all server data in a tree and the classes for
 * converting the nodes of this DOM to be rendered as JTree
 * nodes.
 * 
 * @author Barbara Samson, Vovida Networks
 */
public class ConfigTree extends JTree implements TreeNodeTypes
{

  /**
   * The data manager that gets passed from all requests.
   */
  DataToDOM dataManager = new DataToDOM(this);

  // 
  // Constructors
  // 

  /**
   * Default Constructor.
   */
  public ConfigTree()
  {
    super(new Model());

    // set tree properties
    setRootVisible(false);

    getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

    Vector nodes = ((Model) getModel()).getTreeNodesToExpand();

    for (int i = 0; i < nodes.size(); i++)
    {
      DefaultMutableTreeNode node = 
        (DefaultMutableTreeNode) nodes.elementAt(i);

      expandPath(new TreePath(node.getPath()));
    }
  }     // <init>()

  // 
  // Public methods
  // 


  /**
   * Returns the document's root element.
   */
  public Element getRootElement()
  {
    return getTreeDOM().getDocumentElement();
  }

  /**
   * returns the DOM created from the document
   */
  public Document getTreeDOM()
  {
    return ((Model) getModel()).getTreeDOM();
  }

  /**
   * get the org.w3c.Node for a MutableTreeNode.
   */
  public Node getNode(Object treeNode)
  {
    return ((Model) getModel()).getNode(treeNode);
  }

  /**
   * Checks to see if the same host:port combination already
   * exist in the tree.
   * @param host hostname to check
   * @param port port number to check
   * @return the Element that matches host:port, or null if
   * no Element matches
   */
   public Element hostPortExists(String host, String port)
   {
     System.out.println("calling hostPortExists with host " + host
     + ", port " + port);
     Element rootElement = getRootElement();
     System.out.println(new XMLUtils().buildXMLString(rootElement));
     return (traverseChildren(rootElement, host, port));
   }

  private Element traverseChildren(Element parent, String host, String port)
  {
    NodeList children = parent.getChildNodes();
    if (children != null)
    {
      for (int i = 0; i < children.getLength(); i++)
      {
        Node node = children.item(i);
        if (node.getNodeType() == Node.ELEMENT_NODE)
        {
          Element child = (Element)node;
          String hostString = child.getAttribute("host");
          String portString = child.getAttribute("port");
          // we won't check empty strings
          if( (!hostString.equals("")) && (!portString.equals("")) )
          {
            if (hostString.equals(host) && portString.equals(port))
            {
              return child;
            }
          }
          Element testVal = traverseChildren(child, host, port);
          if (testVal != null)
          {
            return testVal;
          }
        }
      }
    }
    return null;
  }

  /*
   * Causes the TreeSelectionEvent to fire again for the current node.
   */
  public void refreshCurrentSelection()
  {
    DefaultMutableTreeNode currentNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();

    TreePath path = new TreePath(currentNode.getPath());

    // this causes the node to be reselected, which magically
    // sets all fields back to their last saved values.
    fireValueChanged(new TreeSelectionEvent(this, path, false, path, path));
  }

  /**
   * Replace selected node.
   * @param replacement replacement node
   */
  public void replaceSelectedNode(Node replacement)
  {
    DefaultMutableTreeNode currentNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();

    ((Model) getModel()).replaceNodeInMap(currentNode, replacement);
    setSelectionPath(new TreePath(currentNode.getPath()));

    DefaultMutableTreeNode selectedNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();
  }


  /**
   * We can only do this if the node has no children.
   * @return true if the node could be deleted, false otherwise.
   */
  public boolean deleteData()
  {
    Element element = getCurrentElement();
    int elementType = getSelectedNodeType();

    switch (elementType)
    {

      case FEATURE_GROUP:

      case MARSHAL_GROUP:

      case CDR_GROUP:

      case HEARTBEAT_GROUP:

      case PDP_GROUP:

      case REDIRECT_GROUP:

      case JTAPI_GROUP:
      {

        // cannot do delete if there are element child nodes.
        boolean canDelete = true;

        if (element.hasChildNodes())
        {
          NodeList childNodes = element.getChildNodes();

          for (int i = 0; i < childNodes.getLength(); i++)
          {
            Node childNode = childNodes.item(i);

            if (childNode.getNodeType() == Node.ELEMENT_NODE)
            {
              canDelete = false;
            }
          }
        }

        if (canDelete)
        {
          deleteSelectedNode();

          return true;
        }

        return false;
      }

      // note that the user must understand the consequences of
      // deleting a feature server.
      case FEATURE_SERVER:

      case MARSHAL_SERVER:

      case CDR_SERVER:

      case HEARTBEAT_SERVER:

      case PDP_SERVER:

      case REDIRECT_SERVER:

      case JTAPI_SERVER:
      {
        deleteSelectedNode();

        return true;
      }

      default:
      {
        return false;
      }
    }
  }

  /**
   * delete the selected tree node and its associated org.w3c.node
   */
  protected void deleteSelectedNode()
  {
    DefaultMutableTreeNode currentNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();

    DOMToFile.deleteFile((Element) getNode(currentNode));

    DefaultMutableTreeNode newSelection = 
      ((Model) getModel()).deleteNode(currentNode);

    setSelectionPath(new TreePath(newSelection.getPath()));

    try
    {
      DOMToFile.writeAllFilesInPath(getCurrentElement());
    }
    catch (FailedToWriteFileException e)
    {

      // not much we can do about it.
      refreshCurrentSelection();
    }
  }

  /**
   * remove the children of the selected node
   */
  public void deleteChildrenOfSelectedNode()
  {
    DefaultMutableTreeNode parent = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();

    if (parent == null)
    {
      System.out.println("parent is null");

      return;
    }

    ((Model) getModel()).deleteChildrenOfNode(parent);

    // we must set selection because we fired a nodeChanged event
    setSelectionPath(new TreePath(parent.getPath()));

    if (parent != getLastSelectedPathComponent())
    {
      System.out.println("Error selection mismatch");
    }
  }


  /**
   * Rebuild the tree from the currently selected TreeNode
   * down.
   */
  public void saveSelectedNode()
  {
    DefaultMutableTreeNode currentNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();
    DefaultMutableTreeNode newSelection = 
      ((Model) getModel()).rebuildTree(currentNode);

    setSelectionPath(new TreePath(newSelection.getPath()));
  }

  /**
   * Rebuild the children of the currently selected TreeNode.
   */
  public void saveChildrenOfSelectedNode()
  {
    DefaultMutableTreeNode treeNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();
    Node parent = ((Model) getModel()).getNode(treeNode);
    NodeList childNodes = parent.getChildNodes();

    for (int i = 0; i < childNodes.getLength(); i++)
    {
      Node child = childNodes.item(i);

      if (child.getNodeType() == Node.ELEMENT_NODE)
      {
        ((Model) getModel()).addElementToParent(child, treeNode);
      }
    }

    setSelectionPath(new TreePath(treeNode));
  }

  /**
   * Insert a DocumentFragment as first child of the specified TreeNode.
   */
  public void insertDocumentFragment(DocumentFragment fragment, 
          MutableTreeNode where)
  {
    DefaultMutableTreeNode newSelection = 
      ((Model) getModel()).insertFragmentNode(fragment, where);

    setSelectionPath(new TreePath(newSelection.getPath()));
  }

  /**
   * get Element corresponding to currently selected TreeNode
   */
  public Element getCurrentElement()
  {
    DefaultMutableTreeNode currentNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();
    Node node = getNode(currentNode);

    if (node == null)
    {
      System.out.println("DOM node corresponding to currently selected treeNode is null");

      return null;
    }

    Element element = null;

    try
    {
      element = (Element) node;
    }
    catch (ClassCastException e)
    {
      System.out.println("currently selected node is not an element: " 
                         + node.getNodeValue());
    }

    return element;
  }

  /**
   * get type of currently selected TreeNode.
   */
  public int getSelectedNodeType()
  {
    DefaultMutableTreeNode currentNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();

    if (currentNode == null)
    {
      System.out.println("selected node is null");

      return UNKNOWN;
    }

    UserObject theObject = (UserObject) (currentNode.getUserObject());

    return theObject.getNodeType();
  }

  /**
   * 
   */
  public MarshalData getMarshalData()
  {
    return dataManager;
  }

  /**
   * 
   */
  public MarshalData getNewMarshalData()
  {
    DefaultMutableTreeNode thisNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();

    return new DataFragment(MARSHAL_SERVER, this, thisNode);
  }

  /**
   * 
   */
  public FeatureData getFeatureData()
  {
    return dataManager;
  }

  /**
   * 
   */
  public FeatureData getNewFeatureData()
  {
    DefaultMutableTreeNode thisNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();

    return new DataFragment(FEATURE_SERVER, this, thisNode);
  }

  /**
   * 
   */
  public GroupData getGroupData()
  {
    int type = getSelectedNodeType();

    return dataManager;
  }

  /**
   * 
   */
  public GroupData getNewGroupData()
  {
    System.out.println("calling getNewGroupData");

    DefaultMutableTreeNode thisNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();
    int type = getSelectedNodeType();

    return new DataFragment(FEATURE_GROUP, this, thisNode);
  }

  /**
   * 
   */
  public RedirectData getRedirectData()
  {
    return dataManager;
  }

  /**
   * 
   */
  public RedirectData getNewRedirectData()
  {
    DefaultMutableTreeNode thisNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();

    return new DataFragment(REDIRECT_SERVER, this, thisNode);
  }

  /**
   * 
   */
  public PDPData getPDPData()
  {
    return dataManager;
  }

  /**
   * 
   */
  public PDPData getNewPDPData()
  {
    DefaultMutableTreeNode thisNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();

    return new DataFragment(PDP_SERVER, this, thisNode);
  }

  /**
   * 
   */
  public OSPData getOSPData()
  {
    return dataManager;
  }

  /**
   * 
   */
  public GlobalData getGlobalData()
  {
    return dataManager;
  }

  /**
   * 
   */
  public DialPlanData getDialPlanData()
  {
    return dataManager;
  }

  /**
   * 
   */
  public DialPlanData getNewDialPlanData()
  {
    return dataManager;
  }

  /**
   * 
   */
  public CdrData getCdrData()
  {
    return dataManager;
  }

  /**
   * 
   */
  public CdrData getNewCdrData()
  {
    DefaultMutableTreeNode thisNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();

    return new DataFragment(CDR_SERVER, this, thisNode);
  }

  /**
   * 
   */
  public HeartbeatData getHeartbeatData()
  {
    return dataManager;
  }

  /**
   * 
   */
  public HeartbeatData getNewHeartbeatData()
  {
    DefaultMutableTreeNode thisNode = 
      (DefaultMutableTreeNode) getLastSelectedPathComponent();

    return new DataFragment(HEARTBEAT_SERVER, this, thisNode);
  }

  // 
  // Classes
  // 

  /**
   * DOM tree model.
   */
  static class Model extends DefaultTreeModel implements Serializable
  {

    // 
    // Data
    // 

    /**
     * Document.
     */
    private Document treeDOM;

    /**
     * Node Map.
     */
    private Hashtable nodeMap = new Hashtable();

    /**
     * tree nodes to be expanded at initialization
     */
    Vector treeNodesToExpand = new Vector();

    // 
    // Constructors
    // 

    /**
     * Default constructor.
     */
    public Model()
    {
      super(new DefaultMutableTreeNode(new UserObject(UNKNOWN, "")));

      System.out.println("Building DOM tree");

      // create document
      try
      {
        this.treeDOM = FileToDOM.readFiles();
      }
      catch (Exception e)
      {
        e.printStackTrace();
      }

      // clear tree and re-populate
      ((DefaultMutableTreeNode) getRoot()).removeAllChildren();
      nodeMap.clear();
      buildTree();
      fireTreeStructureChanged(this, new Object[]
      {
        getRoot()
      }, new int[0], new Object[0]);

    }

    // 
    // Public methods
    // 

    /**
     * rebuilds the tree after the DOM has been modified
     */
    public synchronized DefaultMutableTreeNode rebuildTree(DefaultMutableTreeNode currentNode)
    {

      // clear branch and re-populate
      Node domNode = getNode(currentNode);
      DefaultMutableTreeNode parent = 
        (DefaultMutableTreeNode) (currentNode.getParent());

      parent.remove(currentNode);

      DefaultMutableTreeNode newNode = 
        (DefaultMutableTreeNode) (insertElementNode(domNode, parent));

      nodeStructureChanged(parent);

      return newNode;
    }

    /**
     * 
     */
    public Vector getTreeNodesToExpand()
    {
      return treeNodesToExpand;
    }

    /**
     * Returns the document.
     */
    public Document getTreeDOM()
    {
      return treeDOM;
    }

    /**
     * get the org.w3c.Node for a MutableTreeNode.
     */
    public Node getNode(Object treeNode)
    {
      return (Node) nodeMap.get(treeNode);
    }

    /**
     * replace an entry in nodeMap
     */
    public void replaceNodeInMap(Object key, Object replacement)
    {
      nodeMap.put(key, replacement);
    }

    /**
     * remove the org.w3c.Node from the nodeMap
     * and remove the associated TreeNode from the tree.
     * @return the parent of the node removed.
     */
    public DefaultMutableTreeNode deleteNode(DefaultMutableTreeNode treeNode)
    {
      DefaultMutableTreeNode treeParent = 
        (DefaultMutableTreeNode) treeNode.getParent();
      Enumeration childrenEnum = treeNode.children();
      DefaultMutableTreeNode child = null;

      while (childrenEnum.hasMoreElements())
      {
        child = (DefaultMutableTreeNode) childrenEnum.nextElement();

        deleteNode(child);
      }

      Object removedObject = nodeMap.remove(treeNode);

      System.out.println("removing " + removedObject.toString() 
                         + " from tree");

      if (removedObject instanceof Node)
      {

        // the nodemap also holds strings and possibly other things
        Node removedNode = (Node) removedObject;
        Node nodeParent = removedNode.getParentNode();

        if (nodeParent != null)
        {
          nodeParent.removeChild(removedNode);
        }
      }

      treeParent.remove(treeNode);
      nodeStructureChanged(treeNode);

      return treeParent;
    }

    /**
     * For all children of this node,
     * remove the org.w3c.Node from the nodeMap
     * and remove the associated TreeNode from the tree.
     */
    public void deleteChildrenOfNode(DefaultMutableTreeNode treeNode)
    {
      while (treeNode.getChildCount() > 0)
      {
        DefaultMutableTreeNode child = 
          (DefaultMutableTreeNode) treeNode.getFirstChild();

        deleteNode(child);
      }
    }

    // 
    // Private methods
    // 

    /**
     * Builds the tree.
     * Modified original to include a NodeFilter
     */

    private void buildTree()
    {


      // is there anything to do?
      if (treeDOM == null)
      {
        return;
      }

      // iterate over children of this node
      NodeList nodes = treeDOM.getChildNodes();
      int len = (nodes != null) ? nodes.getLength() : 0;
      MutableTreeNode root = (MutableTreeNode) getRoot();

      for (int i = 0; i < len; i++)
      {
        Node node = nodes.item(i);

        switch (node.getNodeType())
        {

          case Node.DOCUMENT_NODE:
          {
            root = insertDocumentNode(node, root);

            break;
          }

          case Node.ELEMENT_NODE:
          {
            insertElementNode(node, root);

            break;
          }

          default:      // ignore

        }               // switch

      }                 // for

    }                   // buildTree()

    /**
     * Inserts a node and returns a reference to the new node.
     */
    private MutableTreeNode insertNode(String what, MutableTreeNode where)
    {
      return insertNode(what, where, UNKNOWN);
    }

    /**
     * Inserts a typed node and returns a reference to the new node.
     */
    private MutableTreeNode insertNode(String what, MutableTreeNode where, 
            int type)
    {

      MutableTreeNode node = new DefaultMutableTreeNode(new UserObject(type, 
              what));

      insertNodeInto(node, where, where.getChildCount());

      return node;

    }   // insertNode(Node,MutableTreeNode):MutableTreeNode


    /**
     * Inserts the document node.
     */
    private MutableTreeNode insertDocumentNode(Node what, 
            MutableTreeNode where)
    {
      MutableTreeNode treeNode = insertNode("<" + what.getNodeName() + '>', 
              where);

      nodeMap.put(treeNode, what);

      return treeNode;
    }

    /**
     * Inserts a document fragment.
     * Note that this only works for a fragment with a single
     * child. Returns the single child.
     */
    private DefaultMutableTreeNode insertFragmentNode(DocumentFragment what, 
            MutableTreeNode where)
    {

      Node parent = getNode(where);

      parent.appendChild(what);

      Node child = parent.getLastChild();
      DefaultMutableTreeNode treeNode = 
        (DefaultMutableTreeNode) (insertElementNode(child, where));

      return treeNode;
    }

    /**
     * The public version of insertElementNode
     */
    public void addElementToParent(Node what, MutableTreeNode where)
    {
      insertElementNode(what, where);
      nodeStructureChanged(where);
    }

    /**
     * Inserts an element node.
     */
    private MutableTreeNode insertElementNode(Node what, 
            MutableTreeNode where)
    {
      NodeFilter elementFilter = new NodeFilter()
      {

        /**
         * 
         */
        public short acceptNode(Node node)
        {
          String nodeName = node.getNodeName();

          if ((nodeName.equals("group")) || (nodeName.equals("type")) 
              || (nodeName.equals("expiryTimer")) 
              || (nodeName.equals("multicastHost")) 
              || (nodeName.equals("maxMissedHeartbeats")) 
              || (nodeName.equals("heartbeatInterval")) 
              || (nodeName.equals("proxyAuthKey")) 
              || (nodeName.equals("redirectReasonInSIP")) 
              || (nodeName.equals("servicePoints")) 
              || (nodeName.equals("localValidation")) 
              || (nodeName.equals("sslLifetime")) 
              || (nodeName.equals("http")) 
              || (nodeName.equals("ospExtension")) 
              || (nodeName.equals("customerID")) 
              || (nodeName.equals("deviceID")) 
              || (nodeName.equals("auditURL")) 
              || (nodeName.equals("uaVMServers")) 
              || (nodeName.equals("gateway")) 
              || (nodeName.equals("noResponseTimer")) 
              || (nodeName.equals("allowUnbillableCalls")) 
              || (nodeName.equals("conferenceBridge")) 
              || (nodeName.equals("syncPort")) 
              || (nodeName.equals("radiusServer")) 
              || (nodeName.equals("billing")) 
              || (nodeName.equals("maxThreads")) 
              || (nodeName.equals("keepAliveTimer")) 
              || (nodeName.equals("responseTime")) 
              || (nodeName.equals("clientPort")) 
              || (nodeName.equals("multiGroup")) || (nodeName.equals("key")))
          {

            // Uncomment for debug:
            // System.out.println("skipping " + nodeName);
            return NodeFilter.FILTER_SKIP;
          }

          return NodeFilter.FILTER_ACCEPT;
        }

      };

      if (elementFilter.acceptNode(what) != NodeFilter.FILTER_ACCEPT)
      {
        return null;
      }

      String elementName = what.getNodeName();
      String parentName = "";
      String grandparentName = "";
      Node parentNode = what.getParentNode();
      Node grandparentNode = null;

      if (parentNode != null)
      {
        parentName = parentNode.getNodeName();
        grandparentNode = parentNode.getParentNode();

        if (grandparentNode != null)
        {
          grandparentName = grandparentNode.getNodeName();
        }
      }

      int type = UNKNOWN;

      if (elementName.equals("system"))
      {
        type = SYSTEM;
      }
      else if (elementName.equals("servers"))
      {
        type = SERVERS;
      }
      else if (elementName.equals("globalConfiguration"))
      {
        type = GLOBAL;
      }
      else if (elementName.equals("featureServers"))
      {
        type = FEATURE_SERVERS;
      }
      else if (elementName.equals("marshalServers"))
      {
        type = MARSHAL_SERVERS;
      }
      else if (elementName.equals("jtapiServers"))
      {
        type = JTAPI_SERVERS;
      }
      else if (elementName.equals("redirectServers"))
      {
        type = REDIRECT_SERVERS;
      }
      else if (elementName.equals("cdrServers"))
      {
        type = CDR_SERVERS;
      }
      else if (elementName.equals("heartbeatServers"))
      {
        type = HEARTBEAT_SERVERS;
      }
      else if (elementName.equals("pdpServers"))
      {
        type = PDP_SERVERS;
      }
      else if (elementName.equals("serverType"))
      {
        if (parentName.equals("featureServers"))
        {
          type = FEATURE_TYPE;
        }
        else if (parentName.equals("marshalServers"))
        {
          type = MARSHAL_TYPE;
        }
      }
      else if (elementName.equals("serverGroup"))
      {
        if (grandparentName.equals("featureServers"))
        {
          type = FEATURE_GROUP;
        }
        else if (grandparentName.equals("marshalServers"))
        {
          type = MARSHAL_GROUP;
        }
        else if (parentName.equals("jtapiServers"))
        {
          type = JTAPI_GROUP;
        }
        else if (parentName.equals("redirectServers"))
        {
          type = REDIRECT_GROUP;
        }
        else if (parentName.equals("cdrServers"))
        {
          type = CDR_GROUP;
        }
        else if (parentName.equals("heartbeatServers"))
        {
          type = HEARTBEAT_GROUP;
        }
        else if (parentName.equals("pdpServers"))
        {
          type = PDP_GROUP;
        }
      }
      else if (elementName.equals("featureServer"))
      {
        type = FEATURE_SERVER;
      }
      else if (elementName.equals("marshalServer"))
      {
        type = MARSHAL_SERVER;
      }
      else if (elementName.equals("jtapiServer"))
      {
        type = JTAPI_SERVER;
      }
      else if (elementName.equals("redirectServer"))
      {
        type = REDIRECT_SERVER;
      }
      else if (elementName.equals("cdrServer"))
      {
        type = CDR_SERVER;
      }
      else if (elementName.equals("heartbeatServer"))
      {
        type = HEARTBEAT_SERVER;
      }
      else if (elementName.equals("pdpServer"))
      {
        type = PDP_SERVER;
      }
      else if (elementName.equals("ospServer"))
      {
        type = OSP_SERVER;
      }
      else if (elementName.equals("ipplan"))
      {
        type = IP_DIAL_PLAN;
      }
      else if (elementName.equals("digitalplan"))
      {
        type = DIGITAL_DIAL_PLAN;
      }

      StringBuffer name = new StringBuffer();

      name.append(what.getNodeName());

      NamedNodeMap attrs = what.getAttributes();

      if (attrs != null)
      {
        Node qualifier = attrs.getNamedItem("value");

        if (qualifier != null)
        {
          name.append(" " + ((Attr) qualifier).getValue());
        }
        else
        {
          qualifier = attrs.getNamedItem("host");

          if (qualifier != null)
          {
            name.append(" " + ((Attr) qualifier).getValue());
          }

          qualifier = attrs.getNamedItem("port");

          if (qualifier != null)
          {
            name.append(":" + ((Attr) qualifier).getValue());
          }

          qualifier = attrs.getNamedItem("callingPort");

          if (qualifier != null)
          {
            name.append(":" + ((Attr) qualifier).getValue());
          }

          qualifier = attrs.getNamedItem("calledPort");

          if (qualifier != null)
          {
            name.append(":" + ((Attr) qualifier).getValue());
          }
        }
      }



      MutableTreeNode element = insertNode(name.toString(), where, type);

      // if this is a Servers type of node, expand it.
      if ((type == SYSTEM) || (type == SERVERS))
      {
        treeNodesToExpand.addElement(element);
      }

      nodeMap.put(element, what);

      int attrCount = (attrs != null) ? attrs.getLength() : 0;

      for (int i = 0; i < attrCount; i++)
      {
        Attr attr = (Attr) (attrs.item(i));

        if ((attr.getName().equals("host")) 
            || (attr.getName().equals("port")) 
            || (attr.getName().equals("callingPort")) 
            || (attr.getName().equals("calledPort")) 
            || (attr.getName().equals("value")) 
            || (attr.getName().equals("fileDir")) 
            || (attr.getName().equals("fileName")) 
            || (attr.getName().equals("fileType")))
        {
          continue;
        }

        StringBuffer line = new StringBuffer();

        line.append(attr.getName());
        line.append(" = ");
        line.append(attr.getValue());

        MutableTreeNode attrNode = insertNode(line.toString(), element);

        nodeMap.put(attrNode, attr.getName());
      }

      // if this is a multiGroup, we do not include the children
      if (elementName.equals("multiGroup"))
      {
        return element;
      }

      // gather up attributes and children nodes
      NodeList children = what.getChildNodes();
      int len = (children != null) ? children.getLength() : 0;

      for (int i = 0; i < len; i++)
      {
        Node node = children.item(i);

        switch (node.getNodeType())
        {

          case Node.CDATA_SECTION_NODE:
          {
            insertCDataSectionNode(node, element);      // Add a Section Node

            break;
          }

          case Node.TEXT_NODE:
          {
            insertTextNode(node, element);

            break;
          }

          case Node.ELEMENT_NODE:
          {
            insertElementNode(node, element);

            break;
          }
        }
      }

      return element;

    }   // insertElementNode(Node,MutableTreeNode):MutableTreeNode

    /**
     * Inserts a text node.
     */
    private MutableTreeNode insertTextNode(Node what, MutableTreeNode where)
    {
      String value = what.getNodeValue().trim();

      if (value.length() > 0)
      {
        MutableTreeNode treeNode = insertNode(value, where);

        nodeMap.put(treeNode, what);

        return treeNode;
      }

      return null;
    }


    /**
     * Inserts a CData Section Node.
     */
    private MutableTreeNode insertCDataSectionNode(Node what, 
            MutableTreeNode where)
    {
      StringBuffer CSectionBfr = new StringBuffer();

      // --- optional --- CSectionBfr.append( "<![CDATA[" );
      CSectionBfr.append(what.getNodeValue());

      // --- optional --- CSectionBfr.append( "]]>" );
      if (CSectionBfr.length() > 0)
      {
        MutableTreeNode treeNode = insertNode(CSectionBfr.toString(), where);

        nodeMap.put(treeNode, what);

        return treeNode;
      }

      return null;
    }


  }     // class Model



}       // class ConfigTree

/**
 * There is one UserObject per DefaultMutableTreeNode.
 */
class UserObject implements TreeNodeTypes
{
  private int nodeType = UNKNOWN;
  private String displayString = "";

  /**
   * 
   */
  public String toString()
  {
    return displayString;
  }

  /**
   * 
   * @param string
   */
  public void setDisplayString(String string)
  {
    displayString = string;
  }

  /**
   * 
   */
  public int getNodeType()
  {
    return nodeType;
  }

  /**
   * 
   * @param type
   * @param string
   */
  UserObject(int type, String string)
  {
    nodeType = type;

    setDisplayString(string);
  }

}       // class UserObject

