/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.data;

import vocal.comm.VPPTransactionWrapper;

import vocal.ui.AdministrativeLogin;

import org.apache.xerces.parsers.DOMParser;


import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.InputSource;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.XMLReader;
import java.io.BufferedReader;
import java.io.StringReader;
import java.util.Vector;
import java.util.Stack;

/**
 * This class generates CPL scripts. a script is generated for
 * each feature that a user has turned on. The script is duplicated
 * for each server that implements that feature. In general, we define
 * the main body of the script as a static string, then parse it
 * into a DOM. From there, we can create and insert Elements as
 * necessary to modify the script for a particular user's requirements.
 * Finally when the script is complete, we write the DOM to an XML string
 * which is stored in files named by the user and the server names of
 * all feature servers that implement the feature.
 */
public class CplGenerator implements DataDefaults
{
  private static VPPTransactionWrapper connection = null;


  // a DOM built from ListOfFeatureServers
  private static Document fsDOM;
  private static String[][] cplFilePaths;

  private static String FIRST_LINE = "<?xml version=\"1.0\" ?>\n";
  private static String SECOND_LINE = "<!DOCTYPE cpl SYSTEM \"cpl.dtd\">\n";

  /**
   * Call Forward No Answer Busy Document
   */
  private static String FNAB_DOCUMENT = new String("<cpl>" + "<incoming>" 
          + "<lookup clear=\"yes\" source=\"sip:USERNAME\">" + "<success>" 
          + "<proxy ordering=\"first-only\" timeout=\"15\">" + "<failure>" 
          + "<location clear=\"yes\" url=\"sip:FAILURECASE\">" 
          + "<redirect/>" + "</location>" + "</failure>" + "</proxy>" 
          + "</success>" + "<notfound>" + "<reject status = \"reject\"/>" 
          + "</notfound>" + "</lookup>" + "</incoming>" + "</cpl>");

  /**
   * The Document Node that gets inserted for No Answer
   */
  private static String NO_ANSWER_FRAGMENT = new String("<noanswer>" 
          + "<location clear=\"yes\" url=\"sip:FORWARDNOANS\">" 
          + "<redirect/>" + "</location>" + "</noanswer>");

  /**
   * The Document Node that gets inserted for Busy
   */
  private static String BUSY_FRAGMENT = new String("<busy>" 
          + "<location clear=\"yes\" url=\"sip:FORWARDBUSY\">" 
          + "<redirect/>" + "</location>" + "</busy>");


  /**
   * Forward All Calls Document
   */
  private static String CFA_DOCUMENT = new String("<cpl>" + "<incoming>" 
          + "<location url=\"sip:FORWARD\" clear=\"yes\">" + "<redirect/>" 
          + "</location>" + "</incoming>" + "</cpl>");

  /**
   * Call Blocking Document
   */
  private static String CLBL_DOCUMENT = new String("<cpl>" 
          + "<subaction id=\"rejectcall\">" 
          + "<reject status=\"reject\" reason=\"feature activated\"/>" 
          + "</subaction>" + "<outgoing>" 
          + "<address-switch field=\"original-destination\" subfield=\"user\">" 
          + "<!--dummy placeholder -->" + "<address subdomain-of=\";\">" 
          + "<sub ref=\"rejectcall\" />" + "</address>" + "<otherwise>" 
          + "<lookup clear=\"yes\" timeout=\"2\" source=\"registration\">" 
          + "<success>" + "<proxy ordering=\"first-only\"/>" + "</success>" 
          + "<notfound>" + "<sub ref=\"rejectcall\"/>" + "</notfound>" 
          + "</lookup>" + "</otherwise>" + "</address-switch>" 
          + "</outgoing>" + "</cpl>");

  /**
   * Call Screening Document
   */
  private static String CS_DOCUMENT = new String("<cpl>" 
          + "<subaction id=\"rejectcall\">" 
          + "<reject status=\"reject\" reason=\"feature activated\"/>" 
          + "</subaction>" + "<incoming>" 
          + "<address-switch field=\"origin\" subfield=\"user\">" 
          + "<!--dummy placeholder -->" + "<address subdomain-of=\";\">" 
          + "<sub ref=\"rejectcall\" />" + "</address>" + "<otherwise>" 
          + "<lookup clear=\"yes\" timeout=\"2\" source=\"registration\">" 
          + "<success>" + "<proxy ordering=\"first-only\"/>" + "</success>" 
          + "<notfound>" + "<sub ref=\"rejectcall\"/>" + "</notfound>" 
          + "</lookup>" + "</otherwise>" + "</address-switch>" 
          + "</incoming>" + "</cpl>");

  /**
   * The Document Fragment that gets inserted to block long distance calls
   */
  private static String BLOCKLD_FRAGMENT = new String("<fragment>" 
          + "<!--Block long distance numbers (except for 1-800) -->" 
          + "<address subdomain-of=\"12\">" + "<sub ref=\"rejectcall\" />" 
          + "</address>" + "<address subdomain-of=\"13\">" 
          + "<sub ref=\"rejectcall\" />" + "</address>" 
          + "<address subdomain-of=\"14\">" + "<sub ref=\"rejectcall\" />" 
          + "</address>" + "<address subdomain-of=\"15\">" 
          + "<sub ref=\"rejectcall\" />" + "</address>" 
          + "<address subdomain-of=\"16\">" + "<sub ref=\"rejectcall\" />" 
          + "</address>" + "<address subdomain-of=\"17\">" 
          + "<sub ref=\"rejectcall\" />" + "</address>" 
          + "<address subdomain-of=\"19\">" + "<sub ref=\"rejectcall\" />" 
          + "</address>" + "</fragment>");

  /**
   * The Document Fragment that gets inserted to block 900/976 calls
   * Note that this does not take care of the case where someone
   * calls a 976 number long-distance.
   */
  private static String BLOCK900_FRAGMENT = new String("<fragment>" 
          + "<address subdomain-of=\"1900\">" + "<sub ref=\"rejectcall\"/>" 
          + "</address>" + "<address subdomain-of=\"976\">" 
          + "<sub ref=\"rejectcall\"/>" + "</address>" + "</fragment>");

  /**
   * This String generates the user Cpl script for caller ID blocking.
   * The Cpl script is generated when the user turns on
   * the feature.
   * It is generated one per user. We assume only one server for
   * this feature.
   * 
   * Note that we do not build a Document from this string
   * because it never changes.
   */
  private static String CALLER_ID_BLOCKING_USER_STRING = new String("<cpl>\n" 
          + "  <outgoing>\n" 
          + "    <lookup clear=\"yes\" source=\"registration\">\n" 
          + "      <success>\n" 
          + "        <remove-location location=\"complete_calleridblock\">\n" 
          + "          <proxy ordering=\"first-only\"/>\n" 
          + "        </remove-location>\n" + "      </success>\n" 
          + "        <notfound>\n" 
          + "          <reject status=\"reject\"/>\n" 
          + "        </notfound>\n" + "    </lookup>\n" + "  </outgoing>\n" 
          + "</cpl>");

  /**
   * This String generates the server Cpl script for *69 calls.
   * The Cpl script is generated when the Call Return feature
   * server is created, not when a user selects it.
   * It is generated one per call return feature server.
   * 
   * Note that we do not build a Document from this string
   * because it never changes.
   */
  private static String CALL_RETURN_SERVER_STRING = new String("<cpl>\n" 
          + "  <incoming>\n" 
          + "    <location clear=\"yes\" url=\"callreturn\">\n" 
          + "      <redirect/>\n" + "    </location>" + "    </incoming>\n" 
          + "</cpl>");

  /**
   * This String generates the caller Cpl script for *69 calls.
   * The Cpl script is generated when the administrator allows
   * the feature.
   * It is generated one per user. We assume only one server for
   * this feature.
   * 
   * Note that we do not build a Document from this string
   * because it never changes.
   */
  private static String CALL_RETURN_USER_STRING = new String("<cpl>\n" 
          + "  <incoming>\n" + "    <log name=\"lastcaller\" comment=\"\">\n" 
          + "      <lookup clear=\"yes\" source=\"registration\">\n" 
          + "        <success>\n" 
          + "          <proxy ordering=\"first-only\"/>\n" 
          + "        </success>\n" + "        <notfound>\n" 
          + "          <reject status=\"reject\"/>\n" 
          + "        </notfound>\n" + "      </lookup>\n" + "    </log>\n" 
          + "  </incoming>" + "</cpl>");

  /**
   * This String generates the caller Cpl script for callers who
   * do not have a feature enabled.
   * The Cpl script is generated when the administrator disallows
   * the feature. It is generated one per user per call return feature server.
   * 
   * Note that we do not build a Document from this string
   * because it never changes.
   */
  private static String FEATURE_DISABLED_STRING = new String("<cpl>\n" 
          + "  <incoming>\n" + "    <reject status=\"reject\"/>\n" 
          + "  </incoming>\n" + "</cpl>");

  // fnab DOM the nodes where you insert username and failurecase
  private Document fnabDOM;
  private Element fnabIncomingLookupElement;
  private Element fnabProxyElement;
  private Element fnabBusyLocationElement;
  private Element fnabNoAnswerLocationElement;
  private Element fnabFailureElement;
  private Element fnabFailureLocationElement;
  private Node fnabBusyNode;
  private Node fnabNoAnswerNode;

  // cfa DOM and the node where you insert the forwardTo address
  private Document cfaDOM;
  private Element cfaForwardElement;

  // clbl DOM (this gets cloned)
  private Document clblDOM;
  private Element clblAddressSwitchElement;
  private Element clblDummyElement;

  // cs DOM and the node where you insert the callScreening nodes
  private Document csDOM;
  private Element csAddressSwitchElement;
  private Element csDummyElement;

  // clbl DOM and the nodes for 900 and long distance blocking
  private DocumentFragment blockLongDistanceFrag;
  private DocumentFragment block900Frag;
  private Node noAnswerNode;
  private Node busyNode;

  /**
   * 
   */
  private DocumentFragment buildDocumentFragment(final Document doc, 
          String fragString)
  {
    final DocumentFragment frag = doc.createDocumentFragment();

    // read the string in with a SAX parser to build the fragment
    StringReader fragReader = new StringReader(fragString);
    XMLReader parser = new SAXParser();

    parser.setContentHandler(new DefaultHandler()
    {
      Stack elementStack = new Stack();

      /**
       * 
       * @param ignore1
       * @param ignore2
       * @param rawName
       * @param attrs
       */
      public void startElement(String ignore1, String ignore2, 
              String rawName, Attributes attrs)
      {
        if (rawName.equals("fragment"))
        {

          // the fragment root element is necessary for the
          // SAX parser to work.
          return;
        }

        Element newElement = doc.createElement(rawName);

        for (int i = 0; i < attrs.getLength(); i++)
        {
          newElement.setAttribute(attrs.getRawName(i), attrs.getValue(i));
        }

        if (!elementStack.isEmpty())
        {
          Element lastElement = (Element) (elementStack.peek());

          lastElement.appendChild(newElement);
        }

        elementStack.push(newElement);
      }

      /**
       * 
       * @param ignore1
       * @param ignore2
       * @param rawName
       */
      public void endElement(String ignore1, String ignore2, String rawName)
      {
        if (!rawName.equals("fragment"))
        {
          Element lastElement = (Element) (elementStack.pop());

          if (elementStack.isEmpty())
          {
            frag.appendChild(lastElement);
          }
        }
      }

    });

    try
    {
      parser.parse(new InputSource(fragReader));
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

    return frag;
  }


  /**
   * 
   */
  private Node buildNode(final Document doc, String fragString, 
          final String rootElementName)
  {
    final Node returnNode = doc.createElement(rootElementName);

    // read the string in with a SAX parser to build the node
    StringReader fragReader = new StringReader(fragString);
    XMLReader parser = new SAXParser();

    parser.setContentHandler(new DefaultHandler()
    {
      Stack elementStack = new Stack();

      /**
       * 
       * @param ignore1
       * @param ignore2
       * @param rawName
       * @param attrs
       */
      public void startElement(String ignore1, String ignore2, 
              String rawName, Attributes attrs)
      {
        if (rawName.equals(rootElementName))
        {

          // the node root element is necessary for the
          // SAX parser to work.
          return;
        }

        Element newElement = doc.createElement(rawName);

        for (int i = 0; i < attrs.getLength(); i++)
        {
          newElement.setAttribute(attrs.getRawName(i), attrs.getValue(i));
        }

        if (!elementStack.isEmpty())
        {
          Element lastElement = (Element) (elementStack.peek());

          lastElement.appendChild(newElement);
        }

        elementStack.push(newElement);
      }

      /**
       * 
       * @param ignore1
       * @param ignore2
       * @param rawName
       */
      public void endElement(String ignore1, String ignore2, String rawName)
      {
        if (!rawName.equals(rootElementName))
        {
          Element lastElement = (Element) (elementStack.pop());

          if (elementStack.isEmpty())
          {
            returnNode.appendChild(lastElement);
          }
        }
      }

    });

    try
    {
      parser.parse(new InputSource(fragReader));
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

    return returnNode;
  }


  /**
   * 
   */
  public String buildCsXML(Document userDOM) 
    throws NoSuchNodeException, NotTextNodeException
  {

    long startTime = new java.util.Date().getTime();
    NodeList nodes = userDOM.getElementsByTagName("screenfrom");

    for (int i = 0; i < nodes.getLength(); i++)
    {
      Element element = (Element) nodes.item(i);
      String number = element.getAttribute("number");
      Element addressElement = csDOM.createElement("address");

      addressElement.setAttribute("subdomain-of", number);

      Element subElement = csDOM.createElement("sub");

      subElement.setAttribute("ref", "rejectcall");
      addressElement.appendChild(subElement);
      csAddressSwitchElement.insertBefore(addressElement, csDummyElement);
    }

    // make up the new xml string
    String returnString = createXMLString(csDOM);

    // replace the address-switch element with an empty one.
    Element newCsAddressSwitchElement = csDOM.createElement("address-switch");
    Node parent = csAddressSwitchElement.getParentNode();

    parent.replaceChild(newCsAddressSwitchElement, csAddressSwitchElement);

    csAddressSwitchElement = newCsAddressSwitchElement;

    csAddressSwitchElement.appendChild(csDummyElement);

    long endTime = new java.util.Date().getTime();

    System.out.println("time to build cs: " 
                       + Long.toString(endTime - startTime));

    return returnString;
  }

  /**
   * 
   * @param block900 true if 900 calls should be blocked.
   * @param blockLongDistance true if long distance calls should be blocked.
   * @param prefix if not null or empty, gets prepended to all numbers.
   */
  public String buildClblXML(boolean block900, boolean blockLongDistance,
    String prefix) 
    throws NoSuchNodeException, NotTextNodeException
  {
    long startTime = new java.util.Date().getTime();

    if (block900)
    {

      // insert the 1-900 node in first part of address-switch
      Node block900Node = block900Frag.cloneNode(true);

      clblAddressSwitchElement.insertBefore(block900Node, clblDummyElement);
    }
    else
    {
      System.out.println("900 calls are not blocked");
    }

    if (blockLongDistance)
    {

      // insert the long distance node in the clbl tree.
      Node blockLongDistanceNode = blockLongDistanceFrag.cloneNode(true);

      clblAddressSwitchElement.insertBefore(blockLongDistanceNode, 
              clblDummyElement);
    }
    else
    {
      System.out.println("long distance calls are not blocked");
    }

    // if prefix is given, prepend it.
    if ( (prefix != null) && (!prefix.equals("")) )
    {
      NodeList addressNodes = clblDOM.getElementsByTagName("address");
      for (int i = 0; i < addressNodes.getLength(); i++)
      {
        Node node = addressNodes.item(i);
        if (node instanceof Element)
        {
          Element element = (Element)node;
          String number = element.getAttribute("subdomain-of");
          if ( (number.equals(";")) || (number.equals("")) )
          {
            continue;
          }
          StringBuffer newNumber = new StringBuffer(prefix);
          newNumber.append(number);
          element.setAttribute("subdomain-of", newNumber.toString());
        }
      }
    }

    // make up the new xml string
    String returnString = createXMLString(clblDOM);

    // replace the address-switch element to get rid of unwanted children
    Node parent = clblAddressSwitchElement.getParentNode();
    Element newClblAddressSwitchElement = 
      clblDOM.createElement("address-switch");

    parent.replaceChild(newClblAddressSwitchElement, 
            clblAddressSwitchElement);

    clblAddressSwitchElement = newClblAddressSwitchElement;

    clblAddressSwitchElement.appendChild(clblDummyElement);

    long endTime = new java.util.Date().getTime();

    System.out.println("time to build clbl: " 
                       + Long.toString(endTime - startTime));

    return returnString;
  }     // buildClblXML

  /**
   * 
   */
  public String buildFnabXML(String userName, boolean noAnswerIsOn, 
          String fnaForwardAddress, boolean busyIsOn, 
          String cfbForwardAddress, 
          String failureCase) throws NoSuchNodeException, NotTextNodeException
  {
    long startTime = new java.util.Date().getTime();

    System.out.println("in buildFnamXML, user name is " + userName);

    // put the user address into location element
    fnabIncomingLookupElement.setAttribute("source", 
            "sip:" + userName + ";user=phone");

    // put the failure case address into the failure location element
    fnabFailureLocationElement.setAttribute("url", "sip:" + failureCase);

    Node cfbNode = null;
    Node fnaNode = null;

    if (busyIsOn)
    {
      System.out.println("busy is on");

      // put the forward busy address into the busy location element
      fnabBusyLocationElement.setAttribute("url", 
              "sip:" + cfbForwardAddress + ";user=phone");

      cfbNode = busyNode.cloneNode(true);

      fnabProxyElement.insertBefore(cfbNode, fnabFailureElement);

    }

    if (noAnswerIsOn)
    {
      System.out.println("no answer is on");

      // put the forward no answer address into the fna location element
      fnabNoAnswerLocationElement.setAttribute("url", 
              "sip:" + fnaForwardAddress + ";user=phone");

      fnaNode = noAnswerNode.cloneNode(true);

      fnabProxyElement.insertBefore(fnaNode, fnabFailureElement);

    }

    // make up the new xml string
    String returnString = createXMLString(fnabDOM);

    // remove cfb node, if it exists.
    if (cfbNode != null)
    {
      NodeList children = fnabProxyElement.getChildNodes();

      fnabProxyElement.removeChild(cfbNode);
    }

    // remove fna node, if it exists.
    if (fnaNode != null)
    {
      fnabProxyElement.removeChild(fnaNode);
    }

    long endTime = new java.util.Date().getTime();

    System.out.println("time to build fnab: " 
                       + Long.toString(endTime - startTime));

    return returnString;
  }     // buildFnabXML


  /**
   * 
   */
  public String buildCfaXML(String forwardAddress) 
    throws NoSuchNodeException, NotTextNodeException
  {
    long startTime = new java.util.Date().getTime();

    Element cfaRootElement = cfaDOM.getDocumentElement();       // cpl

    // find the node for the forward URL attribute
    NodeList nodes = cfaRootElement.getElementsByTagName("incoming");
    Element element = (Element) (nodes.item(0));

    nodes = element.getElementsByTagName("location");
    element = (Element) (nodes.item(0));

    if (element == null)
    {
      throw new NoSuchNodeException("location node is missing");
    }

    element.setAttribute("url", "sip:" + forwardAddress);

    // make up the new xml string
    String returnString = createXMLString(cfaDOM);
    long endTime = new java.util.Date().getTime();

    System.out.println("time to build cfa: " 
                       + Long.toString(endTime - startTime));

    return returnString;
  }     // buildCfaXML

  // the following methods can be static because they only refer
  // to static strings.

  /**
   * 
   */
  public static String buildCallerIdBlockingXML()
  {
    return (FIRST_LINE + SECOND_LINE + CALLER_ID_BLOCKING_USER_STRING);
  }

  /**
   * 
   */
  public static String buildCallReturnServerXML()
  {
    return (FIRST_LINE + SECOND_LINE + CALL_RETURN_SERVER_STRING);
  }

  /**
   * 
   */
  public static String buildCallReturnUserXML()
  {
    return (FIRST_LINE + SECOND_LINE + CALL_RETURN_USER_STRING);
  }

  /**
   * 
   */
  public static String buildFeatureDisabledXML()
  {
    return (FIRST_LINE + SECOND_LINE + FEATURE_DISABLED_STRING);
  }

  /**
   * 
   */
  private Element getFirstElementChild(Element parent)
  {
    NodeList nodes = parent.getChildNodes();

    for (int i = 0; i < nodes.getLength(); i++)
    {
      Node node = nodes.item(i);

      if (node.getNodeType() == Node.ELEMENT_NODE)
      {
        return (Element) node;
      }
    }

    System.out.println("getFirstElementChild returning null");

    return null;
  }

  /**
   * return the list of file paths for all features belonging
   * to a particular group. Return null if the group is not
   * in cplFilePaths.
   * @return array of file path names
   */
  public static String[] getAddressArray(String groupName)
  {
    String filePath[] = null;

    if (cplFilePaths == null)
    {
      return null;
    }

    for (int i = 0; i < cplFilePaths.length; i++)
    {
      filePath = cplFilePaths[i];

      if (groupName.compareToIgnoreCase(filePath[0]) == 0)
      {
        return filePath;
      }
    }

    return null;
  }

  /**
   * Called at startup.
   */
  private static String[][] buildCplFilePaths()
  {
    DOMParser parser = null;

    try
    {
      parser = new DOMParser();

      if (connection == null)
      {

        connection = AdministrativeLogin.getConnection();

        if (connection == null)
        {
          System.out.println("ERROR: null connection in CPL generator");
          System.out.println("Could not get one from the AdministrativeLogin either");
          new Exception().printStackTrace();

          return null;
        }
      }

      BufferedReader response = connection.request("GET", SYSTEM_DIR, 
              LIST_OF_FEATURE_SERVERS_FILE);

      parser.parse(new InputSource(response));

      fsDOM = parser.getDocument();
    }
    catch (Exception e)
    {
      e.printStackTrace();

      return null;
    }

    if (fsDOM == null)
    {
      System.out.println("required file ListOfFeatureServers missing");
      new Exception().printStackTrace();

      return null;
    }

    Vector pathArrayVector = new Vector();
    Vector pathVector;
    String pathArray[] = null;
    Element rootElement = fsDOM.getDocumentElement();

    NodeList featureNodes = rootElement.getElementsByTagName("serverType");

    for (int i = 0; i < featureNodes.getLength(); i++)
    {

      // the pathVector holds the group name and all serverPaths in group
      Element featureElement = (Element) (featureNodes.item(i));

      // there must be one type per feature
      String typeName = featureElement.getAttribute("value");

      // eventually we need to replace this with an XML attribute
      String callingCalledString;

      if ((typeName.equals("ForwardNoAnswerBusy")) 
          || (typeName.equals("ForwardAllCalls")) 
          || (typeName.equals("CallScreening")) 
          || (typeName.equals("Voicemail")) 
          || (typeName.equals("CallReturn")))
      {
        callingCalledString = "Called";
      }
      else if ((typeName.equals("CallBlocking")) 
               || (typeName.equals("CallerIdBlocking")))
      {
        callingCalledString = "Calling";
      }
      else
      {
        System.out.println("Error: type name " + typeName 
                           + " not recognized");

        return null;
      }

      // there may be zero or more groups per feature
      NodeList groupNodes = 
        featureElement.getElementsByTagName("serverGroup");

      for (int j = 0; j < groupNodes.getLength(); j++)
      {
        pathVector = new Vector();

        String serverGroupName = null;
        Element groupElement = (Element) (groupNodes.item(j));

        serverGroupName = groupElement.getAttribute("value");

        // the first thing in the vector is the group name
        pathVector.add(serverGroupName);


        // each additional entry in the vector is a pathname for that group
        NodeList serverNodes = groupElement.getElementsByTagName("server");


        for (int k = 0; k < serverNodes.getLength(); k++)
        {
          Element serverElement = (Element) (serverNodes.item(k));

          String hostName = serverElement.getAttribute("host");
          String portNumber = serverElement.getAttribute("port");

          String serverPath = FEATURE_DIR + "_" + hostName + "_" + portNumber 
                  + "_" + callingCalledString;

          pathVector.add(serverPath);

          pathArray = new String[pathVector.size()];

          pathVector.copyInto(pathArray);
        }

        if (pathVector.size() == 1)     // serverGroupName only
        {

          // no cpl will be generated for this group today.
          System.out.println("Error: " + pathVector.firstElement() 
                             + " has no associated server");
        }
        else
        {
          pathArrayVector.add(pathArray);
        }
      }
    }

    if (pathArrayVector.isEmpty())
    {
      System.out.println("Error: no path information");

      return null;
    }

    String allPathArrays[][] = new String[pathArrayVector.size()][];

    pathArrayVector.copyInto(allPathArrays);

    // take a look at what's in this mess
    for (int n = 0; n < allPathArrays.length; n++)
    {
      String[] serverGroupPaths = allPathArrays[n];

      if (serverGroupPaths == null)
      {
        System.out.println("serverGroupPaths is null");
        new Exception().printStackTrace();

      }

      for (int p = 0; p < serverGroupPaths.length; p++)
      {
        System.out.println(serverGroupPaths[p]);
      }
    }

    return allPathArrays;
  }

  /**
   * 
   * @param con
   */
  public CplGenerator(VPPTransactionWrapper con)
  {
    connection = con;

    init();
  }

  /**
   * Set up the Element references we need.
   */
  private void init()
  {
    try
    {
      cplFilePaths = buildCplFilePaths();
      fnabDOM = createDocument(FNAB_DOCUMENT);

      Element rootElement = fnabDOM.getDocumentElement();

      fnabIncomingLookupElement = 
        getFirstElementChild(getFirstElementChild(rootElement));

      Element fnabSuccessElement = 
        getFirstElementChild(fnabIncomingLookupElement);

      fnabProxyElement = getFirstElementChild(fnabSuccessElement);
      fnabFailureElement = getFirstElementChild(fnabProxyElement);
      fnabFailureLocationElement = getFirstElementChild(fnabFailureElement);
      busyNode = buildNode(fnabDOM, BUSY_FRAGMENT, "busy");
      fnabBusyLocationElement = (Element) busyNode.getFirstChild();
      noAnswerNode = buildNode(fnabDOM, NO_ANSWER_FRAGMENT, "noanswer");
      fnabNoAnswerLocationElement = (Element) noAnswerNode.getFirstChild();
      cfaDOM = createDocument(CFA_DOCUMENT);
      rootElement = cfaDOM.getDocumentElement();
      cfaForwardElement = 
        getFirstElementChild(getFirstElementChild(rootElement));
      clblDOM = createDocument(CLBL_DOCUMENT);

      NodeList nodes = clblDOM.getElementsByTagName("outgoing");
      Element outgoingElement = (Element) nodes.item(0);

      if (outgoingElement == null)
      {
        throw new NoSuchNodeException("outgoing");
      }

      clblAddressSwitchElement = getFirstElementChild(outgoingElement);
      clblDummyElement = getFirstElementChild(clblAddressSwitchElement);
      csDOM = createDocument(CS_DOCUMENT);
      nodes = csDOM.getElementsByTagName("incoming");

      Element incomingElement = (Element) nodes.item(0);

      if (incomingElement == null)
      {
        throw new NoSuchNodeException("incoming");
      }


      csAddressSwitchElement = getFirstElementChild(incomingElement);
      csDummyElement = getFirstElementChild(csAddressSwitchElement);
      blockLongDistanceFrag = buildDocumentFragment(clblDOM, 
              BLOCKLD_FRAGMENT);
      block900Frag = buildDocumentFragment(clblDOM, BLOCK900_FRAGMENT);
    }
    catch (NoSuchNodeException e)
    {
      System.out.println(e.getMessage());
      e.printStackTrace();
    }
  }

  /**
   * 
   */
  protected Document createDocument(String rawXML)
  {
    StringReader reader = new StringReader(rawXML);

    try
    {
      DOMParser parser = new DOMParser();

      parser.parse(new InputSource(reader));

      return parser.getDocument();
    }
    catch (Exception e)
    {
      e.printStackTrace();

      return null;
    }
  }

  /**
   * 
   */
  private static String createXMLString(Node node)
  {
    String theXML = new XMLUtils().buildXMLString(node);

    return (FIRST_LINE + SECOND_LINE + theXML);
  }

}



