/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.data;

import vocal.comm.VPPTransactionWrapper;
import vocal.comm.VPPException;
import vocal.ui.AdministrativeLogin;

import org.apache.xml.serialize.*;
import org.apache.xerces.dom.*;
import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Attr;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.traversal.NodeFilter;
import org.w3c.dom.traversal.DocumentTraversal;
import org.w3c.dom.traversal.NodeIterator;
import org.w3c.dom.traversal.TreeWalker;
import org.apache.xerces.dom.TreeWalkerImpl;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringReader;

/**
 * $RCSfile: DOMToFile.java,v $
 * 
 * @author $Author: bsamson $, $Date: 2002/04/17 21:21:53 $
 * @version $Revision: 1.30 $
 */
public class DOMToFile implements TreeNodeTypes
{
  private static Document baseDocument;
  private static Element rootElement;
  private static Element systemElement;
  private static Element serversElement;


  public static void deleteFile(Element currentNode)
  {

    String fileNameString = null;
    String fileDirString = null;

    fileNameString = currentNode.getAttribute("fileName");
    fileDirString = currentNode.getAttribute("fileDir");

    VPPTransactionWrapper connection = AdministrativeLogin.getConnection();

    try
    {
      connection.doDel(fileDirString, fileNameString);
    }
    catch (VPPException e)
    {
      VPPTransactionWrapper.showVPPException(e, "Error deleting file");
    }

  }


  /**
   * Write currentNode to file if it has an attribute "fileName"
   * @param currentNode the node we will try to write.
   * @return true if the file was successfully written, or false if
   * currentNode was not writable (did not have a filename);
   * @throws FailedToWriteFileException if currentNode should be writable but
   * the write failed
   */
  public static boolean tryToWriteFile(Element currentNode) 
    throws FailedToWriteFileException
  {
    String fileTypeString = null;
    String fileNameString = null;
    String fileDirString = null;

    try
    {
      fileNameString = currentNode.getAttribute("fileName");
      fileDirString = currentNode.getAttribute("fileDir");

      // uncomment for debugging
      // System.out.println("writing to file name: " + fileNameString);

      if (!fileNameString.equals(""))
      {
        fileTypeString = currentNode.getAttribute("fileType");

        int nodeType = Integer.parseInt(fileTypeString);

        writeFileFromNode(currentNode, nodeType, fileDirString, 
                          fileNameString);

        return true;
      }
    }
    catch (NumberFormatException e)
    {
      System.out.println("Error: corrupted data in xml for " + fileNameString 
                         + "\n" + fileTypeString 
                         + " cannot be parsed as a node type");
      e.printStackTrace();

    }

    return false;
  }     // tryToWriteFile

  /**
   * Starting from the root of a tree,
   * write to file any node that has
   * the attribute "fileName"
   */
  public static void writeAllFilesInTree(Document doc)
  {
    try
    {
      NodeIterator iterator = 
        ((DocumentImpl) doc).createNodeIterator(doc.getDocumentElement(), 
              Node.ELEMENT_NODE, null, true);
      Node currentNode = iterator.nextNode();

      while (currentNode != null)
      {
        if (currentNode instanceof Element)
        {
          tryToWriteFile((Element) currentNode);
        }

        currentNode = iterator.nextNode();
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  /**
   * Starting from currentNode, write to file any node that has
   * an attribute "filename".
   */
  public static void writeAllFilesInPath(Element endOfPath) 
    throws FailedToWriteFileException
  {
    Element currentNode = endOfPath;

    try
    {
      Node parentNode = null;

      // uncomment for debugging
      // System.out.println("current node: " + currentNode.getNodeName());
      tryToWriteFile(currentNode);

      while (true)
      {
        parentNode = currentNode.getParentNode();

        if (parentNode != null 
            && parentNode.getNodeType() == Node.ELEMENT_NODE)
        {
          currentNode = (Element) parentNode;

          // uncomment for debugging
          // System.out.println("current node: " + currentNode.getNodeName());
          tryToWriteFile(currentNode);
        }
        else
        {
          if (parentNode == null)
          {

            // uncomment for debugging
            // System.out.println("returning because parent is null");
          }
          else if (parentNode.getNodeType() != Node.ELEMENT_NODE)
          {

            // uncomment for debugging
            // System.out.println("returning because parent type is "
            // + parentNode.getNodeType());
          }

          return;
        }
      }
    }
    catch (FailedToWriteFileException e)
    {
      System.out.println("Failed to write file at node " 
                         + currentNode.getNodeName());

      throw e;
    }

  }     // writeAllFilesInPath

  /**
   * Starting from currentNode, write to file first node that has
   * an attribute "filename".
   */
  public static void writeFirstFileInPath(Element endOfPath) 
    throws FailedToWriteFileException
  {
    Element currentNode = endOfPath;
    Node parentNode = null;

    System.out.println("current node: " + currentNode.getNodeName());

    if (tryToWriteFile(currentNode))
    {
      return;
    }

    while (true)
    {
      parentNode = currentNode.getParentNode();

      if (parentNode != null && parentNode.getNodeType() == Node.ELEMENT_NODE)
      {
        currentNode = (Element) parentNode;

        System.out.println("current node: " + currentNode.getNodeName());

        if (tryToWriteFile(currentNode))
        {
          return;
        }
      }
      else
      {
        return;
      }
    }
  }     // writeFirstFileInPath

  /**
   * 
   * @param nodeToWrite
   * @param nodeType
   * @param fileDir
   * @param filename
   * 
   * @throws NumberFormatException
   */
  public static void writeFileFromNode(Element nodeToWrite, int nodeType, 
          String fileDir, 
          String filename) throws NumberFormatException, 
          FailedToWriteFileException
  {
    System.out.println("in writeFileFromNode, fileDir: " + fileDir 
                       + ", filename: " + filename);

    switch (nodeType)
    {

      case GLOBAL:

      case DIGITAL_DIAL_PLAN:

      case IP_DIAL_PLAN:

      case FEATURE_SERVER:

      case MARSHAL_SERVER:

      case CDR_SERVER:

      case HEARTBEAT_SERVER:

      case PDP_SERVER:

      case OSP_SERVER:

      case REDIRECT_SERVER:
      {
        Attr fileDirAttr = nodeToWrite.getAttributeNode("fileDir");
        Attr fileNameAttr = nodeToWrite.getAttributeNode("fileName");
        Attr fileTypeAttr = nodeToWrite.getAttributeNode("fileType");

        if (fileDirAttr != null)
        {
          nodeToWrite.removeAttributeNode(fileDirAttr);
        }

        if (fileNameAttr != null)
        {
          nodeToWrite.removeAttributeNode(fileNameAttr);
        }

        if (fileTypeAttr != null)
        {
          nodeToWrite.removeAttributeNode(fileTypeAttr);
        }

        String xmlString = new XMLUtils().buildXMLString(nodeToWrite);

        writeAsIs(xmlString, fileDir, filename);

        if (fileDirAttr != null)
        {
          nodeToWrite.setAttributeNode(fileDirAttr);
        }

        if (fileNameAttr != null)
        {
          nodeToWrite.setAttributeNode(fileNameAttr);
        }

        if (fileTypeAttr != null)
        {
          nodeToWrite.setAttributeNode(fileTypeAttr);
        }

        break;
      }

      case FEATURE_SERVERS:

      case MARSHAL_SERVERS:

      case CDR_SERVERS:

      case HEARTBEAT_SERVERS:

      case REDIRECT_SERVERS:

      case PDP_SERVERS:
      {
        writeListOfServers(nodeToWrite, fileDir, filename);

        break;
      }

      default:
      {
        String nodeTypeString = null;

        try
        {
          nodeTypeString = TreeNodeTypes.strings[nodeType];
        }
        catch (ArrayIndexOutOfBoundsException e)
        {
          nodeTypeString = Integer.toString(nodeType);
        }

        System.out.println("cannot write file for node of type " 
                           + nodeTypeString);

        throw new NumberFormatException();
      }
    }
  }     // writeFileFromNode

  /**
   * Write the given string to the given file.
   * My apologies for the connection kluge. It is for Deployment.
   * I realized at the last moment that I need to
   * establish a connection through something other
   * than the AdministrativeLogin screen.
   * @param xmlString content of file to be written
   * @param fileDir directory path of file to be written
   * @param filename name of file to be written
   * @throws FailedToWriteFileException if the write was not successful
   */
  protected static void writeAsIs(String xmlString, String fileDir, 
          String filename) throws FailedToWriteFileException
  {

    // if the AdministrativeLogin screen returns a null connection,
    // get one from the Deployment class.
    VPPTransactionWrapper connection = AdministrativeLogin.getConnection();

    if (connection == null)
    {
      connection = DeployServers.getConnection();

      if (connection == null)
      {
        System.out.println("Error: null connection");

        return;
      }
    }

    try
    {
      connection.doPut(fileDir, filename, xmlString);
    }
    catch (VPPException e)
    {
      VPPTransactionWrapper.showVPPException(e, 
              "Error writing " + filename + " to " + fileDir);

      throw new FailedToWriteFileException(e.getMessage());
    }
  }

  /**
   * 
   * @param nodeToWrite
   * @param fileDir
   * @param filename
   */
  private static void writeListOfServers(final Element nodeToWrite, 
          String fileDir, String filename) throws FailedToWriteFileException
  {
    System.out.println("in writeListOfServers\n" + "fileDir: " + fileDir 
                       + "filename: " + filename);

    int whatToWrite = NodeFilter.SHOW_ELEMENT | NodeFilter.SHOW_ATTRIBUTE;
    NodeFilter filter = new NodeFilter()
    {

      /**
       * 
       */
      public short acceptNode(Node n)
      {
        if ((n.getNodeType() == Node.ATTRIBUTE_NODE) 
            || (n.getNodeName().equals("featureServers")) 
            || (n.getNodeName().equals("marshalServers")) 
            || (n.getNodeName().equals("cdrServers")) 
            || (n.getNodeName().equals("heartbeatServers")) 
            || (n.getNodeName().equals("pdpServers")) 
            || (n.getNodeName().equals("redirectServers")) 
            || (n.getNodeName().equals("serverType")) 
            || (n.getNodeName().equals("serverGroup")) 
            || (n.getNodeName().equals("featureServer")) 
            || (n.getNodeName().equals("marshalServer")) 
            || (n.getNodeName().equals("cdrServer")) 
            || (n.getNodeName().equals("heartbeatServer")) 
            || (n.getNodeName().equals("redirectServer")) 
            || (n.getNodeName().equals("pdpServer")) )
        {
          return FILTER_ACCEPT;
        }
        else
        {
          return FILTER_REJECT;
        }
      }

    };
    TreeWalker treeWalker = 
      new org.apache.xerces.dom.TreeWalkerImpl(nodeToWrite, whatToWrite, 
            filter, false);

    String xmlString = traverseListOfServersWalker(treeWalker);

    writeAsIs(xmlString, fileDir, filename);
  }

  /**
   * 
   */
  private static String traverseListOfServersWalker(TreeWalker treeWalker)
  {
    StringBuffer outputBuf = new StringBuffer();
    StringBuffer indentBuf = new StringBuffer();

    if (treeWalker.getRoot() == null)
    {
      System.out.println("Error null treeWalker");

      return "";
    }

    outputBuf.append(printNodeStart(treeWalker.getCurrentNode(), "", false));

    LOOP1:
    while (true)
    {
      if (treeWalker.firstChild() != null)
      {
        indentBuf.append("  ");
        outputBuf.append(printNodeStart(treeWalker.getCurrentNode(), 
                indentBuf.toString(), true));

        continue;
      }
      else
      {
        outputBuf.append(printNodeEnd(treeWalker.getCurrentNode(), 
                indentBuf.toString()));

        LOOP2:
        while (true)
        {
          if (treeWalker.nextSibling() != null)
          {
            outputBuf.append(printNodeStart(treeWalker.getCurrentNode(), 
                    indentBuf.toString(), true));

            continue LOOP1;
          }
          else
          {
            if (treeWalker.parentNode() == null)
            {

              // we have traversed back to the root.
              break LOOP1;
            }
            else
            {
              indentBuf.delete(0, 2);
              outputBuf.append(printNodeEnd(treeWalker.getCurrentNode(), 
                      indentBuf.toString()));
            }
          }

          // continue LOOP2
        }
      }
    }

    System.out.println("at end of traversal, outputBuf:\n" 
                       + new String(outputBuf));

    return new String(outputBuf);
  }

  /**
   * 
   */
  private static String printNodeStart(Node node, String indentString, 
          boolean makeNewLine)
  {
    StringBuffer sbuf = new StringBuffer();
    String nodeName = node.getNodeName();

    if ((nodeName.equals("featureServer")) 
        || (nodeName.equals("marshalServer")) 
        || (nodeName.equals("cdrServer")) 
        || (nodeName.equals("heartbeatServer")) 
        || (nodeName.equals("redirectServer")) 
        || (nodeName.equals("pdpServer")) )
    {
      nodeName = "server";
    }

    if (makeNewLine)
    {
      sbuf.append("\n");
    }

    sbuf.append(indentString + "<" + nodeName);

    String attrIndent = (indentString + "    ");
    NamedNodeMap attributes = node.getAttributes();

    if (attributes != null)
    {
      for (int i = 0; i < attributes.getLength(); i++)
      {
        Attr attr = (Attr) attributes.item(i);
        String attrName = attr.getName();

        if ((attrName.equals("value")) || (attrName.equals("host")) 
            || (attrName.equals("port")))
        {
          sbuf.append("\n" + attrIndent + attrName + "=" + "\"" 
                      + attr.getValue() + "\"");
        }
      }
    }

    sbuf.append(">");

    return new String(sbuf);
  }

  /**
   * 
   */
  private static String printNodeEnd(Node node, String indentString)
  {
    String nodeName = node.getNodeName();

    if ((nodeName.equals("featureServer")) 
        || (nodeName.equals("marshalServer")) 
        || (nodeName.equals("cdrServer")) 
        || (nodeName.equals("heartbeatServer")) 
        || (nodeName.equals("redirectServer")) 
        || (nodeName.equals("pdpServer")) )
    {
      nodeName = "server";
    }

    return ("\n" + indentString + "</" + nodeName + ">");
  }

}



