/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */
package vocal.pw;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;

import vocal.comm.VPPTransactionWrapper;
import vocal.comm.VPPException;

import java.net.ConnectException;


public class AdminAcctManager extends JApplet
{

  /**
   * Identifier which defines the default close operation for a frame. <p.
   * This does not exist as an identifier in java 1.2 so define it here.
   */
  private static final int EXIT_ON_CLOSE = 3;


  boolean isStandalone = false;
  String hostName;
  int portNumber;

  AdminAcctManagerPanel panel;

  // Get a parameter value
  public String getParameter(String key, String def)
  {
    return isStandalone ? System.getProperty(key, def) 
           : (getParameter(key) != null ? getParameter(key) : def);
  }


  public boolean initialize()
  {
    try
    {
      hostName = this.getParameter("host", "localhost");
    }
    catch (Exception e)
    {
      e.printStackTrace();

      return false;
    }

    try
    {
      portNumber = Integer.parseInt(this.getParameter("port", "6005"));
    }
    catch (Exception e)
    {
      e.printStackTrace();

      return false;
    }

    panel = new AdminAcctManagerPanel(isStandalone);

    try
    {
      VPPTransactionWrapper con = new VPPTransactionWrapper(hostName, 
              portNumber);

      panel.setConnection(con);
    }
    catch (VPPException e)
    {
      VPPTransactionWrapper.showVPPException(e, 
              "Error connecting to pserver");

      return false;
    }

    getContentPane().add(panel);
    panel.init();

    return true;
  }


  // Initialize the applet
  public void init()
  {
    initialize();
  }

  // Get parameter info
  public String[][] getParameterInfo()
  {
    String[][] pinfo = 
    {
      {
        "host", "String", ""
      }, 
      {
        "port", "int", ""
      }, 
    };

    return pinfo;
  }

  // Main method
  public static void main(String[] args)
  {
    if (args.length < 2)
    {
      System.out.println("Usage:");
      System.out.println("  AcctManager host port");
    }

    System.setProperty("host", args[0]);
    System.setProperty("port", args[1]);

    AdminAcctManager applet = new AdminAcctManager();

    applet.isStandalone = true;

    JFrame frame = new JFrame();

    frame.setDefaultCloseOperation(EXIT_ON_CLOSE);
    frame.setTitle("Manage administrative accounts");

    if (!applet.initialize())
    {
      System.exit(0);
    }

    frame.getContentPane().add(applet.panel, BorderLayout.CENTER);
    applet.start();
    frame.pack();

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

    frame.setLocation((d.width - frame.getSize().width) / 2, 
                      (d.height - frame.getSize().height) / 2);
    frame.setVisible(true);
  }

}

