/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */
package vocal.systemStatus;

import vocal.comm.VPPTransactionWrapper;
import vocal.comm.VPPException;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;

import java.net.URL;


public class SNMPApplet extends JApplet
{

  static final int SNMP_PORT = 33606;

  // these must be static because they are set in main
  static boolean isStandalone = false;
  static String host = "localhost";
  static int port = SNMP_PORT;

  private SystemStatus theClient;

  private static VPPTransactionWrapper connection;

  public static ImageIcon blueball;
  public static ImageIcon redball;


  /**
   * Get a parameter value.
   * This is only used by the applet.
   * @param key Parameter we are looking from from the html page.
   * @param def Default if parameter is not found.
   * @exception Exception thrown if this is called in standalone mode.
   */
  public String getParameter(String key, String def) throws Exception
  {
    if (isStandalone)
    {
      throw new Exception("getParameters called on standalone application");
    }

    return (getParameter(key) != null ? getParameter(key) : def);
  }

  // Construct the applet
  public SNMPApplet() {}

  // Initialize the applet
  public void init()
  {
    System.out.println("Initializing Applet");

    try
    {
      if (!isStandalone)
      {
        host = this.getParameter("host", "localhost");
        port = Integer.parseInt(this.getParameter("port", "SNMP_PORT"));

        URL codebase = getCodeBase();

        blueball = new ImageIcon(new URL(codebase, "images/blueball.gif"));
        redball = new ImageIcon(new URL(codebase, "images/redball.gif"));
      }
      else
      {
        blueball = new ImageIcon("images/blueball.gif");
        redball = new ImageIcon("images/redball.gif");
      }

      if (blueball.getImageLoadStatus() != MediaTracker.COMPLETE)
      {
        System.out.println("Error loading blueball.gif: " 
                           + blueball.getImageLoadStatus());
      }

      if (redball.getImageLoadStatus() != MediaTracker.COMPLETE)
      {
        System.out.println("Error loading redball.gif: " 
                           + redball.getImageLoadStatus());
      }

      jbInit();
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  /*
   * public void stop()
   * {
   * theClient.stop();
   * if(isStandalone)
   * {
   * System.exit(0);
   * }
   * super.stop();
   * }
   */

  public void start()
  {
    System.out.println("Starting Applet");

    try
    {
      connection = new VPPTransactionWrapper(host, port);
    }
    catch (VPPException e)
    {
      System.out.println("Could not connect to " + host + ":" + port 
                         + " because:");
      e.printStackTrace();
      JOptionPane.showMessageDialog(this,
            "Could not open connection to Network Manager because: \n" + e,
            "ERROR", JOptionPane.ERROR_MESSAGE);

      return;
    }

    try
    {
      theClient = new SystemStatus(this);

      System.out.println("starting SNMPClient with host " + host + " port " 
                         + port);
      init();
      theClient.init(host, port);
      theClient.start();
    }
    catch (Exception e)
    {
      System.out.println(e.getMessage());
      e.printStackTrace();
    }
  }

  public static void main(String args[])
  {
    isStandalone = true;

    if (args.length < 2)
    {
      System.out.println("Usage: SNMPClient hostname port");

      return;
    }

    try
    {
      int portNumber = Integer.parseInt(args[1]);

      port = portNumber;
      host = args[0];

      new SNMPApplet().start();
    }
    catch (NumberFormatException e)
    {
      System.out.println("Exception parsing port number: " + e.getMessage());

      return;
    }
  }


  // Component initialization
  private void jbInit() throws Exception
  {
    this.setSize(new Dimension(400, 300));
  }

  // Get parameter info
  public String[][] getParameterInfo()
  {
    String[][] pinfo = 
    {
      {
        "host", "String", ""
      }, 
      {
        "port", "int", ""
      }, 
    };

    return pinfo;
  }

  public static VPPTransactionWrapper getConnectionInstance()
  {
    return connection;
  }

}

