/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.systemStatus;

import vocal.comm.VPPTransactionWrapper;
import vocal.comm.VPPException;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;

import java.awt.event.*;

import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Calendar;
import java.util.TimeZone;

import java.net.URL;
import java.net.MalformedURLException;

import java.io.*;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;

import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.plaf.metal.MetalIconFactory;
import javax.swing.tree.TreePath;

import org.apache.xerces.parsers.SAXParser;
import org.apache.xerces.framework.XMLAttrList;
import org.xml.sax.AttributeList;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.InputSource;
import org.xml.sax.Attributes;


/**
 * This class displays the System status as a GUI. It receives the information
 * about the systems from SNMP network manager.
 * This class extends JFrame to implement the frame container for our JTree
 */
public class SystemStatus extends JFrame
{

  /**
   * A reference to the Applet that started this up.
   */
  private SNMPApplet theApplet;

  /**
   * Flag which indicates whether to keep querying the server for new status info.
   */
  private boolean run = true;

  /**
   * Name of the System Status host
   */
  private String hostname = "";

  /**
   * Port the System Status host is listening on
   */
  private int port = 0;

  /**
   * The current version number to pass to Network Manager
   * This must be initialized to < 0 so that the Network
   * Manager will send over data on the first request.
   * The Network Manager may send back an incremented
   * version number, which is what we use on subsequent calls.
   */
  private int versionNumber = -1;

  /**
   * Handler for SAX callbacks
   */
  private SAXHandler handler = new SAXHandler();

  private DefaultMutableTreeNode top;
  private DefaultMutableTreeNode selectedNode;

  // the label that tells us which node is being controlled.
  private JLabel controlLabel;

  // the button used to send SNMP Start
  private JRadioButton startButton;

  // the button used to send SNMP Stop
  private JRadioButton stopButton;

  // an invisible button to make it look like none are selected
  private JRadioButton dummyButton;

  /**
   * Text Field and message pane to display the status messages
   */
  private JTextField m_display;
  private JEditorPane messagePane;


  /**
   * The container object
   */
  Container contents;
  JRootPane rootP;
  SNMPCellRenderer renderer;
  JTree m_tree;
  DefaultTreeModel m_model;

  /**
   * 
   */
  public SystemStatus(SNMPApplet applet)
  {
    this();

    theApplet = applet;
  }

  /**
   * Default constructor.
   */
  public SystemStatus()
  {

    super("SNMP GUI ");

    setSize(500, 400);

    OidNode oNode = new OidNode("root", null, OidNode.ROOT);

    oNode.setDisplayString("vocal.com");

    top = new DefaultMutableTreeNode(oNode);

    controlLabel = new JLabel(" ");


    startButton = new JRadioButton("Start Process");

    startButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        if (selectedNode == null)
        {
          System.out.println("selected node is null");
        }
        else
        {
          OidNode node = (OidNode) (selectedNode.getUserObject());

          synchronized (this)
          {
            sendSNMPStart(node);
          }
        }
      }

    });

    dummyButton = new JRadioButton("none");

    stopButton = new JRadioButton("Stop Process");

    stopButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        if (selectedNode == null)
        {
          System.out.println("selected node is null");
        }
        else
        {
          OidNode node = (OidNode) (selectedNode.getUserObject());

          synchronized (this)
          {
            sendSNMPStop(node);
          }
        }
      }

    });


    /**
     * Defines the root and each node of the tree as TreeNode instances
     */
    m_model = new DefaultTreeModel(top);
    m_tree = new JTree(m_model);

    m_tree.addTreeSelectionListener(new TreeSelectionListener()
    {
      public void valueChanged(TreeSelectionEvent e)
      {
        DefaultMutableTreeNode candidateNode = 
          (DefaultMutableTreeNode) (e.getPath().getLastPathComponent());

        if (candidateNode != selectedNode)
        {

          // unselect the stop button
          dummyButton.setSelected(true);
        }

        OidNode userObject = (OidNode) (candidateNode.getUserObject());


        if (userObject.getType() == OidNode.PORT)
        {
          selectedNode = candidateNode;

          controlLabel.setText(userObject.getDisplayString());
          stopButton.setEnabled(true);
          startButton.setEnabled(true);
        }
        else
        {
          selectedNode = null;

          controlLabel.setText(" ");
          stopButton.setEnabled(false);
          startButton.setEnabled(false);
        }
      }

    });

    // allow only one node to be selected at a time.
    m_tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

    m_tree.setRootVisible(false);


    /**
     * Renderer object to render a tree cell based on its current states
     */
    renderer = new SNMPCellRenderer();


    m_tree.setCellRenderer(renderer);
    m_tree.setShowsRootHandles(true);
    m_tree.setEditable(false);

    m_tree.setSelectionPath(new TreePath(top));
    m_tree.addTreeSelectionListener(new TreeSelectionListener()
    {

      /**
       */
      public void valueChanged(TreeSelectionEvent e)
      {
        DefaultMutableTreeNode node = 
          (DefaultMutableTreeNode) m_tree.getLastSelectedPathComponent();

        if (node == null)
        {
          return;
        }

        Object nodeInfo = node.getUserObject();

        if (nodeInfo instanceof OidNode)
        {
          OidNode nd = (OidNode) nodeInfo;
        }
      }

    });

    /**
     * The root pane and Container object
     */
    rootP = getRootPane();
    contents = rootP.getContentPane();

    /**
     * JScrollPane is created to provide scrolling capabilities,  tree is added to
     * its JViewport
     */
    JScrollPane treeView = new JScrollPane();

    treeView.getViewport().add(m_tree);

    /**
     * Create the messages viewing pane.
     */
    messagePane = new JEditorPane();

    messagePane.setEditable(false);

    JScrollPane msgView = new JScrollPane(messagePane);

    // Add the scroll panes to a split pane
    JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

    // Make a panel for the treeView and stopStart panels.
    JPanel topPanel = new JPanel();

    topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.X_AXIS));

    // Make a  panel for the stop/start buttons.
    JPanel controlPanel = new JPanel();

    controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.Y_AXIS));
    controlPanel.setBorder(new TitledBorder(new EmptyBorder(5, 5, 5, 5), 
            "SNMP Control"));

    // Make the things that go in the startStop box.
    ButtonGroup startStopGroup = new ButtonGroup();

    startStopGroup.add(startButton);
    startStopGroup.add(stopButton);
    startStopGroup.add(dummyButton);

    controlPanel.add(controlLabel);
    controlPanel.add(startButton);
    controlPanel.add(stopButton);
    controlPanel.add(Box.createGlue());

    topPanel.add(treeView);
    topPanel.add(controlPanel);

    splitPane.setTopComponent(topPanel);
    splitPane.setBottomComponent(msgView);

    this.setSize(new Dimension(500, 500));

    // args to Dimension are width, height

    treeView.setMinimumSize(new Dimension(200, 50));
    treeView.setPreferredSize(new Dimension(300, 300));
    msgView.setMinimumSize(new Dimension(500, 50));
    msgView.setPreferredSize(new Dimension(500, 200));
    splitPane.setOneTouchExpandable(true);
    splitPane.setPreferredSize(new Dimension(500, 500));

    /**
     * Add the split pane to this frame
     */
    contents.add(splitPane, BorderLayout.CENTER);

    /**
     * Object for text field
     */
    m_display = new JTextField();

    m_display.setEditable(false);

    /**
     * Add the text field to this frame
     */
    contents.add(m_display, BorderLayout.SOUTH);
    getContentPane().add(m_display, BorderLayout.SOUTH);

    addWindowListener(new WindowAdapter()
    {

      /**
       * 
       */
      public void windowClosing(WindowEvent e)
      {
        stop();
      }

    });

    setVisible(true);
  }     // end of SystemStatus constructor

  private void sendSNMPStart(OidNode node)
  {
    System.out.println("---TRACE---SystemStatus:sendSNMPStart");

    String requestString = ("START_REQ," + node.getDescriptorString() + "\n");

    VPPTransactionWrapper connection = SNMPApplet.getConnectionInstance();

    try
    {
      String response = connection.request("START_REQ", requestString);

      if ((response == null) || (response.equals("")))
      {
        System.out.println("No response to Start request");

        return;
      }
      else
      {
        try
        {
          decipherResponse("START_REQ", response);
        }
        catch (SAXException e1)
        {
          throw new VPPException(e1.getMessage() + " from SAXException");
        }
        catch (IOException e2)
        {
          throw new VPPException(e2.getMessage() + " from IOException");
        }
      }
    }
    catch (VPPException e)
    {
      VPPTransactionWrapper.showVPPException(e, "Start request failed");
    }
  }

  private void decipherResponse(String requestType, 
          String response) throws SAXException, IOException
  {
    if ((response == null) || (response.trim().equals("")))
    {
      System.out.println("empty response to " + requestType);

      return;
    }

    BufferedReader reader = new BufferedReader(new StringReader(response));
    XMLReader parser = new SAXParser();

    parser.setContentHandler(handler);
    parser.setErrorHandler(handler);
    parser.parse(new InputSource(reader));
  }

  private void sendSNMPStop(OidNode node)
  {
    System.out.println("---TRACE---SystemStatus:sendSNMPStop");

    String requestString = ("STOP_REQ," + node.getDescriptorString() + "\n");

    System.out.println(requestString);

    VPPTransactionWrapper connection = SNMPApplet.getConnectionInstance();

    try
    {
      String response = connection.request("STOP_REQ", requestString);

      if (response == null)
      {
        System.out.println("No response to STOP_REQ");
      }
      else
      {
        try
        {
          decipherResponse("STOP_REQ", response);
        }
        catch (SAXException e1)
        {
          throw new VPPException(e1.getMessage() + " from SAXException");
        }
        catch (IOException e2)
        {
          throw new VPPException(e2.getMessage() + " from IOException");
        }
      }
    }
    catch (VPPException e)
    {
      VPPTransactionWrapper.showVPPException(e, "Stop request failed");
    }
  }

  /**
   * Checks whether this parent has a child node with this identifier string.
   * 
   * @param parent The parent whose children we are checking.
   * @param identifier The identifier we are checking for.
   * @return The child that contains identifier, or null if no child
   * contains identifier.
   */
  public DefaultMutableTreeNode checkIdentifier(DefaultMutableTreeNode parent, 
          String identifier)
  {
    DefaultMutableTreeNode node = null;

    for (Enumeration e = parent.children(); e.hasMoreElements(); )
    {
      node = (DefaultMutableTreeNode) (e.nextElement());

      if (node == null)
      {
        throw new RuntimeException("Current child is null");
      }

      OidNode nd = (OidNode) node.getUserObject();
      String nodeId = nd.getIdentifier();

      if (nodeId.equals(identifier))
      {
        return node;
      }
    }

    return null;
  }

  /**
   * createNodes : Builds the nodes dynamically
   * @param ip, ip address of the node (or could be hostname)
   * @param port, port of the node
   * @param type, name of the node
   * @param status, represents the status of the node
   */
  public void createNodes(String ip, String port, String type, String status)
  {
    DefaultMutableTreeNode ipNode = null;
    DefaultMutableTreeNode portNode = null;
    DefaultMutableTreeNode statusNode = null;

    ipNode = checkIdentifier(top, ip);

    if (ipNode == null)
    {

      /*
       * create the new IP node
       */
      ipNode = createAndAddChild(top, ip, "HOST:" + ip, OidNode.HOST);

      // make the ip node be displayed at the start
      m_tree.expandPath(new TreePath(top));

      /*
       * create the new port node
       */
      portNode = createAndAddChild(ipNode, port, type + ":" + port, 
              OidNode.PORT);

      ((OidNode) (portNode.getUserObject())).setDescriptorString(ip, port, 
              type);

      /*
       * create the new status node
       */
      createAndAddChild(portNode, null, status, OidNode.STATUS);

      updateMessage(type, port, status, ip);
    }
    else
    {
      OidNode userObject = null;

      /*
       * MATCH FOUND FOR IP
       */
      portNode = checkIdentifier(ipNode, port);

      if (portNode == null)
      {

        /*
         * create the new port node
         */
        portNode = createAndAddChild(ipNode, port, type + ":" + port, 
                OidNode.PORT);

        ((OidNode) (portNode.getUserObject())).setDescriptorString(ip, port, 
                type);

        /*
         * create the new status node
         */
        statusNode = createAndAddChild(portNode, null, status, 
                OidNode.STATUS);
        userObject = (OidNode) (statusNode.getUserObject());

        updateMessage(type, port, status, ip);
      }
      else
      {

        statusNode = (DefaultMutableTreeNode) (portNode.getChildAt(0));
        userObject = (OidNode) statusNode.getUserObject();

        /*
         * Check if the status has changed
         */
        String oldStatus = userObject.toString();

        if (oldStatus.equals(status))
        {
          return;
        }

        /*
         * They are Different
         */
        userObject.setDisplayString(status);
        System.out.println("\n status.setName() = " + userObject);

        if (statusNode != null)
        {
          m_model.nodeChanged(statusNode);
        }

        if (portNode != null)
        {
          m_model.nodeChanged(portNode);
        }

        if (ipNode != null)
        {
          m_model.nodeChanged(ipNode);
        }

        updateMessage(type, port, status, ip);
      }
    }
  }


  /**
   * 
   * @param parent
   * @param id
   * @param displayString
   * @param type ROOT, HOST, PORT, or STATUS
   * 
   * @return
   */
  private DefaultMutableTreeNode createAndAddChild(DefaultMutableTreeNode parent, 
          String id, String displayString, int type)
  {
    OidNode oNode = new OidNode(id, parent, type);

    oNode.setDisplayString(displayString);

    DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(oNode);

    m_model.insertNodeInto(childNode, parent, 0);

    return childNode;

  }


  /**
   * updateMessage : Update the GUI with the new messages received
   */
  public void updateMessage(String type, String port, String status, 
          String ip)
  {
    java.util.Calendar cal;
    String textStr = " ";
    String redString = " ";
    String DATE_FORMAT = "MM/dd/yyyy HH:mm:ss";
    java.text.SimpleDateFormat sdf;

    cal = Calendar.getInstance(TimeZone.getDefault());

    sdf = new java.text.SimpleDateFormat(DATE_FORMAT);

    sdf.setTimeZone(TimeZone.getDefault());

    /*
     * Display the latest message in the text field of the GUI screen
     */
    m_display.setText("Server name : " + type + " : " + port 
                      + " ,  Status : " + status);

    textStr = messagePane.getText();

    /*
     * Append the new message in the message pane of the GUI screen
     */
    messagePane.setText(sdf.format(cal.getTime()) + " --> " + ip + "   " 
            + type + " : " + port + "  " + status + "\n" + textStr);
  }


  /**
   * interface to the SNMP manager
   * @param ip The IP address of the device that changed.
   * @param port The port number of the device that changed.
   * @param type The type of the device that changed, for example "ms" or "fs".
   * @param status The new status of the device that changed,
   * for example "active" or "inactive".
   */
  public void changeEvent(String ip, String port, String type, String status)
  {
    createNodes(ip, port, type, status);
  }

  public void stop()
  {
    try
    {
      SNMPApplet.getConnectionInstance().close();
    }
    catch (VPPException e)
    {
      System.out.println("Error closing connection: " + e.getMessage());
    }

    run = false;

    if ((theApplet == null) || (theApplet.isStandalone))
    {

      // uncomment for debug
      // System.out.println("CALLING SYSTEM EXIT");
      System.exit(0);
    }

    // uncomment for debug
    // System.out.println("STOPPING APPLET");
    theApplet.stop();
  }

  public void start() throws SNMPClientException
  {
    run = true;

    VPPTransactionWrapper connection = SNMPApplet.getConnectionInstance();
    String response;
    String request, message;

    request = "CONFIG_REQ";

    while (run)
    {
      try
      {
        message = request + "," + versionNumber + ",0\n";

        // uncomment for debugging
        // System.out.println("Sending request " + request + message);
        synchronized (this)
        {
          response = connection.request(request, message);
        }
      }
      catch (VPPException e)
      {
        VPPTransactionWrapper.showVPPException(e, "Status request failed");

        return;
      }

      // if the connection timed out or the response could not be gotten for
      // some other reason, the response will be null.
      if (response == null)
      {
        System.out.println("---TRACE---SNMPApplet:init - no response to " 
                           + request + " " + message);
      }
      else
      {

        try
        {
          decipherResponse(request, response);
        }
        catch (SAXException e1)
        {
          throw new SNMPClientException(e1.getMessage() 
                  + " from SAXException");
        }
        catch (IOException e2)
        {
          throw new SNMPClientException(e2.getMessage() 
                  + " from IOException");
        }

        try
        {
          int time = 500;

          // uncomment for debugging
          // System.out.println("Sleeping for " + time + " ms");
          Thread.sleep(time);
        }
        catch (InterruptedException e)
        {
          throw new SNMPClientException("sleep unexpectedly interrupted: " 
                  + e.getMessage());
        }
      }         // end else (response received)
    }
  }


  /**
   * This is where this object starts from.
   * @param hostname Host to connect to.
   * @param port Port that host is listening on.
   * 
   * @throws SNMPClientException when a connection error occurs.
   * @throws SystemStatusException when there is a GUI error.
   * 
   */
  void init(String aHostname, 
            int aPort) throws SNMPClientException, SystemStatusException
  {
    System.out.println("starting init");
    System.out.println("init called with aHostname: " + aHostname + " aPort " 
                       + new Integer(aPort).toString());

    hostname = aHostname;
    port = aPort;
  }


  /**
   * Method declaration
   * 
   * 
   * @param numberString
   * 
   */
  private void handleVersionNumber(String numberString)
  {
    int newVersionNumber = 0;

    try
    {
      newVersionNumber = Integer.parseInt(numberString);

      versionNumber = newVersionNumber;
    }
    catch (NumberFormatException e)
    {
      System.out.println("version number " + numberString 
                         + " threw NumberFormatException: " + e.getMessage());
    }
  }

  private void handleError(String errorMessage)
  {
    System.out.println("Error: " + errorMessage);
    JOptionPane.showMessageDialog(this, errorMessage, "Error", 
            JOptionPane.ERROR_MESSAGE);
  }

  /**
   * This inner class handles SAX callbacks.
   */
  class SAXHandler extends DefaultHandler
  {

    /**
     * static definitions for tags
     */
    private final static String ERROR = "error";
    private final static String VERSION = "version";
    private final static String DATA = "data";
    private final static String SERVERCONFIG = "serverconfig";
    private final static String IP = "ip";
    private final static String PORT = "port";
    private final static String SERVERTYPE = "servertype";
    private final static String SERVERSTATUS = "serverstatus";

    /**
     * When an element comes in, this is set to the element type
     */
    private Vector elementTypes;


    /**
     * Receive notification of the start of an element.
     * @param uri Ignored.
     * @param localName Ignored.
     * @param rawName The raw XML name.
     * @param attrs the attributes attached to the element.
     * If there are no attributes, an empty Attributes object.
     */
    public void startElement(String uri, String localName, String rawName, 
            Attributes attributes)
    {

      // if rawName is ERROR that will be handled in
      // characters method.

      if (rawName.equals(DATA))
      {
        elementTypes = new Vector();
        String version = attributes.getValue(VERSION);

        // uncomment for debugging
        // System.out.println("VERSION is " + version);

        if (version != null)
        {
          handleVersionNumber(version);
        }
      }
      else if (rawName.equals(SERVERCONFIG))
      {

        String ip = attributes.getValue(IP);
        String port = attributes.getValue(PORT);
        String type = attributes.getValue(SERVERTYPE);
        String status = attributes.getValue(SERVERSTATUS);

        try
        {

          changeEvent(ip, port, type, status);

        }
        catch (Exception e)
        {
          System.out.println("GUI changeEvent threw " + e.getMessage());
        }
      }

      elementTypes.add(rawName);

      // the character data will now be handled by the characters method.
    }   // startElement



    /**
     * Receive notification of the end of an element.
     * @param namespaceuri Ignored.
     * @param localName Ignored.
     * @param rawName The raw XML name.
     */
    public void endElement(String namespaceuri, String localName, 
            String rawName)
    {
      if (rawName.equals(ERROR))
      {
        System.out.println("name is error");
      }
      else if (rawName.equals(VERSION))
      {
        System.out.println("name is version");
      }

      String lastElementType = 
        (String) (elementTypes.remove(elementTypes.size() - 1));

      if (!rawName.equals(lastElementType))
      {
        System.out.println("Element mismatch: ending " + rawName 
                           + " but lastElementType is " + lastElementType);
      }
    }   // endElement


    /**
     * Method declaration
     * 
     * 
     * @param ch
     * @param start
     * @param length
     * 
     */
    public void characters(char[] ch, int start, int length)
    {

      String lastElementType = (String) (elementTypes.lastElement());

      if (lastElementType.equalsIgnoreCase(ERROR))
      {
        handleError(new String(ch, start, length));
      }
    }

  }

}

/**
 * This class encapsulates a single object identifier as a number, a String name
 * describing the associated object, String ip address and String port name.
 * Instances of this class are passed directly to the DefaultMutableTreeNode constructor
 * to act as a node.
 */

class OidNode
{

  final static int ROOT = 0;
  final static int HOST = 1;
  final static int PORT = 2;
  final static int STATUS = 3;

  /**
   * This identifier is used to find a unique node.
   * No two siblings may have the same identifier.
   */
  private String identifier;

  /**
   * This string is sent to the SNMP Manager for a start or stop request.
   * It is only needed by tyep PORT.
   */
  private String descriptorString = "";

  /**
   * The parent of the DefaultMutableTreeNode that created this user object.
   */
  private DefaultMutableTreeNode parentNode;

  /**
   * This type is used instead of subclassing.
   */
  private int type;

  /**
   * Empty if no alarms are raised.
   */
  private Vector alarms = new Vector();

  /**
   * 
   */
  public int getType()
  {
    return type;
  }

  /**
   * @return true if an alarm should be raised.
   */
  public synchronized boolean isAlarmed()
  {
    if (alarms.isEmpty())
    {
      return false;
    }

    return true;
  }

  public void setDescriptorString(String host, String port, String type)
  {
    descriptorString = (host + ":" + port + "," + type);
  }

  public String getDescriptorString()
  {
    return descriptorString;
  }


  /**
   * Used by this class and by child node to raise or
   * clear an alarm.
   * 
   * @param newValue True if alarm should be raised.
   * @param requester Object requesting that alarm be raised.
   * This object will be added to the queue of objects that
   * have requested an alarm be raised. The alarm will not
   * be cleared until all objects that raised an alarm have
   * cleared it.
   */
  protected synchronized void setAlarmed(boolean newValue, Object requester)
  {
    if (newValue)
    {

      /*
       * We are raising an alarm. If the requester is not
       * already in the alarms queue, then put it there.
       */
      if (!alarms.contains(requester))
      {
        alarms.add(requester);
      }
    }
    else
    {

      /*
       * We are clearing an alarm. Remove the requester frm
       * the alarm queue.
       */
      alarms.remove(requester);
    }

    if (parentNode != null)
    {
      ((OidNode) (parentNode.getUserObject())).setAlarmed(newValue, this);
    }
  }

  /**
   * display string of the node.
   */
  private String displayString;

  /**
   * Constructor for creating the node
   */
  public OidNode(String id, DefaultMutableTreeNode theParentNode, int aType)
  {
    identifier = id;
    parentNode = theParentNode;
    type = aType;
  }

  /**
   * Returns the name of the node
   */
  public String getIdentifier()
  {
    return identifier;
  }

  public String getDisplayString()
  {
    return displayString;
  }

  /**
   * Sets the displayString of the node
   */
  public void setDisplayString(String newDisplay)
  {
    displayString = newDisplay;

    if (type == STATUS)
    {
      if (displayString.equals("Inactive"))
      {
        setAlarmed(true, this);
      }
      else
      {
        setAlarmed(false, this);
      }
    }
  }

  /**
   * Return the display String so that the tree's cell renderer will display
   * each node correctly.
   */
  public String toString()
  {
    return displayString;
  }

}

/**
 * $RCSfile: SystemStatus.java,v $
 * 
 * @author $Author: bsamson $, $Date: 2001/08/13 23:33:12 $
 * @version $Revision: 1.13 $
 */
class SNMPCellRenderer extends DefaultTreeCellRenderer
{

  /**
   */
  public SNMPCellRenderer()
  {
    super();

    setOpenIcon(SNMPApplet.blueball);
    setClosedIcon(SNMPApplet.blueball);
  }

  /**
   * 
   * @param tree
   * @param value
   * @param sel
   * @param expanded
   * @param leaf
   * @param row
   * @param hasFocus
   * 
   * @return
   */
  public Component getTreeCellRendererComponent(JTree tree, Object value, 
          boolean sel, boolean expanded, boolean leaf, int row, 
          boolean hasFocus)
  {

    // value is a DefaultMutableTreeNode
    boolean isAlarmed = false;

    if (value instanceof DefaultMutableTreeNode)
    {
      isAlarmed = 
        ((OidNode) ((DefaultMutableTreeNode) value).getUserObject()).isAlarmed();
    }

    if (isAlarmed)
    {
      if (leaf)
      {
        setLeafIcon(SNMPApplet.redball);
      }
      else
      {
        setClosedIcon(SNMPApplet.redball);
        setOpenIcon(SNMPApplet.redball);
      }
    }
    else
    {
      setLeafIcon(SNMPApplet.blueball);
      setClosedIcon(SNMPApplet.blueball);
      setOpenIcon(SNMPApplet.blueball);
    }

    return super.getTreeCellRendererComponent(tree, value, sel, expanded, 
            leaf, row, hasFocus);
  }

}

