/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.ui;

import vocal.comm.VPPTransactionWrapper;
import vocal.comm.VPPException;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;
import java.net.ConnectException;

public class AdministrativeLogin extends JApplet
{
  boolean isStandalone = false;
  String hostName;
  int portNumber;

  AdministrativeLoginPanel panel;


  private static VPPTransactionWrapper connection = null;

  // Get a parameter value
  public String getParameter(String key, String def)
  {
    return isStandalone ? System.getProperty(key, def) 
           : (getParameter(key) != null ? getParameter(key) : def);
  }

  // Initialize the applet
  public void init()
  {
    try
    {
      hostName = this.getParameter("host", "localhost");
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

    try
    {
      portNumber = Integer.parseInt(this.getParameter("port", "6005"));
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

    try
    {
      connection = new VPPTransactionWrapper(hostName, portNumber);
    }
    catch (VPPException e)
    {
      System.out.println("Could not open connection to pserver");
      e.printStackTrace();
      JOptionPane.showMessageDialog(this, 
              "Could not open connection to pserver because: \n" + e, 
              "ERROR", JOptionPane.ERROR_MESSAGE);

      return;
    }

    panel = new AdministrativeLoginPanel(isStandalone, connection, 
            getRootPane());

    getContentPane().add(panel);
  }


  public String[][] getParameterInfo()
  {
    String[][] pinfo = 
    {
      {
        "host", "String", "Host name of machine running pserver"
      }, 
      {
        "port", "int", "Port number of pserver"
      }, 
    };

    return pinfo;
  }

  // Main method
  public static void main(String[] args)
  {
    if (args.length < 2)
    {
      System.out.println("Usage:");
      System.out.println("  AdministrativeLogin host port");

      return;
    }

    System.setProperty("host", args[0]);
    System.setProperty("port", args[1]);

    AdministrativeLogin applet = new AdministrativeLogin();

    applet.isStandalone = true;

    JFrame frame = new JFrame();

    // EXIT_ON_CLOSE == 3
    frame.setDefaultCloseOperation(3);
    frame.setTitle("Vovida Open Communication Application Library");

    applet.init();

    if (connection == null)
    {
      return;
    }

    applet.panel = new AdministrativeLoginPanel(applet.isStandalone, 
            applet.connection, frame.getRootPane());

    vocal.util.Logger.log("Test log message");

    applet.start();
    frame.getContentPane().add(applet.panel, BorderLayout.CENTER);
    frame.pack();

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

    frame.setLocation((d.width - frame.getSize().width) / 2, 
                      (d.height - frame.getSize().height) / 2);
    frame.setVisible(true);
    frame.setResizable(false);
  }


  public static VPPTransactionWrapper getConnection()
  {
    return connection;
  }

}

