/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.ui;

import java.util.Vector;
import javax.swing.table.AbstractTableModel;

/**
 * $RCSfile: BasicTableModel.java,v $
 * 
 * @author $Author: bsamson $, $Date: 2001/01/27 00:42:33 $
 * @version $Revision: 1.8 $
 */
public class BasicTableModel extends AbstractTableModel
{
  private Vector data = new Vector();

  // a weirdness that is necessary because mouse clicks add rows.
  private boolean addButtonPressed = false;

  // an optional list of names for each column in the table
  private String columnNames[] = null;

  /**
   * 
   */
  public Vector getTableData()
  {
    System.out.println("in tableModel getTableData, data is:\n" 
                       + data.toString());

    return data;
  }

  /**
   * 
   * @param newVector
   */
  public void setData(Vector newVector)
  {
    data = newVector;

    System.out.println("in tableModel setData, data is:\n" + data.toString());
    fireTableDataChanged();
  }

  /**
   * 
   * @param entry
   */
  public void addNewEntry(String entry)
  {
    data.add(0, entry);
  }

  /**
   * 
   */
  public Object getValueAt(int row, int column)
  {
    return data.elementAt(row);
  }

  /**
   * 
   */
  public void setAddButtonPressed(boolean pressed)
  {
    addButtonPressed = pressed;
  }

  /**
   * 
   */
  public boolean isAddButtonPressed()
  {
    return addButtonPressed;
  }

  /**
   * 
   * @param value
   * @param row
   * @param column
   */
  public void setValueAt(Object value, int row, int column)
  {

    // need to ensure that the add event came from the Add button.
    System.out.println("attempting to put " + value + " at " 
                       + Integer.toString(row));
    data.remove(row);
    data.add(row, value);
  }

  /**
   * 
   */
  public Class getColumnClass(int column)
  {
    return new String().getClass();
  }

  /**
   * 
   */
  public int getColumnCount()
  {
    return 0;
  }

  /**
   * 
   */
  public int getRowCount()
  {
    return data.size();
  }

  /**
   * 
   * @param row
   */
  public void deleteRow(int row)
  {
    data.remove(row);
  }

  /**
   * 
   * @param names
   */
  public void setColumnNames(String names[])
  {
    columnNames = names;
  }

  /**
   * 
   */
  public String getColumnName(int column)
  {
    if ((columnNames == null) || (column > columnNames.length))
    {
      return "";
    }

    return columnNames[column];
  }

  /**
   * 
   */
  public boolean isCellEditable(int row, int column)
  {
    if (row < data.size())
    {
      return true;
    }

    return false;
  }

}



