/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.ui;

import java.awt.*;
import java.util.*;

/**
 * This is sample code from "Java Swing"
 * written by Matthew Robinson and Pavel Vorobiev.
 */
public class DialogLayout implements LayoutManager
{
  protected int m_divider = -1;
  protected int m_hGap = 10;
  protected int m_vGap = 5;

  /**
   */
  public DialogLayout() {}

  /**
   * 
   * @param hGap
   * @param vGap
   */
  public DialogLayout(int hGap, int vGap)
  {
    m_hGap = hGap;
    m_vGap = vGap;
  }

  /**
   * 
   * @param name
   * @param comp
   */
  public void addLayoutComponent(String name, Component comp) {}

  /**
   * 
   */
  public void removeLayoutComponent(Component comp) {}

  /**
   * 
   */
  public Dimension preferredLayoutSize(Container parent)
  {
    int divider = getDivider(parent);

    int w = 0;
    int h = 0;

    for (int k = 1; k < parent.getComponentCount(); k += 2)
    {
      Component comp = parent.getComponent(k);
      Dimension d = comp.getPreferredSize();

      w = Math.max(w, d.width);
      h += d.height + m_vGap;
    }

    h -= m_vGap;

    Insets insets = parent.getInsets();

    return new Dimension(divider + w + insets.left + insets.right, 
            h + insets.top + insets.bottom);
  }

  /**
   * 
   */
  public Dimension minimumLayoutSize(Container parent)
  {
    return preferredLayoutSize(parent);
  }

  /**
   * 
   */
  public void layoutContainer(Container parent)
  {
    int divider = getDivider(parent);

    Insets insets = parent.getInsets();
    int w = parent.getWidth() - insets.left - insets.right - divider;
    int x = insets.left;
    int y = insets.top;

    for (int k = 1; k < parent.getComponentCount(); k += 2)
    {
      Component comp1 = parent.getComponent(k - 1);
      Component comp2 = parent.getComponent(k);
      Dimension d = comp2.getPreferredSize();

      comp1.setBounds(x, y, divider - m_hGap, d.height);
      comp2.setBounds(x + divider, y, w, d.height);

      y += d.height + m_vGap;
    }
  }

  /**
   * 
   */
  public int getHGap()
  {
    return m_hGap;
  }

  /**
   * 
   */
  public int getVGap()
  {
    return m_vGap;
  }

  /**
   * 
   * @param divider
   */
  public void setDivider(int divider)
  {
    if (divider > 0)
    {
      m_divider = divider;
    }
  }

  /**
   * 
   */
  public int getDivider()
  {
    return m_divider;
  }

  /**
   * 
   */
  protected int getDivider(Container parent)
  {
    if (m_divider > 0)
    {
      return m_divider;

    }

    int divider = 0;

    for (int k = 0; k < parent.getComponentCount(); k += 2)
    {
      Component comp = parent.getComponent(k);
      Dimension d = comp.getPreferredSize();

      divider = Math.max(divider, d.width);
    }

    divider += m_hGap;

    return divider;
  }

  /**
   * 
   */
  public String toString()
  {
    return getClass().getName() + "[hgap=" + m_hGap + ",vgap=" + m_vGap 
           + ",divider=" + m_divider + "]";
  }

}


