/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */
package vocal.ui;

import javax.swing.*;
import javax.swing.text.*;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;
import java.util.StringTokenizer;


public class HostNameField extends ValidatingField
{

  public HostNameField(String desc)
  {
    super(desc);

    emptyOk = false;
  }

  public HostNameField(int columns, String desc)
  {
    super(columns, desc);

    emptyOk = false;
  }

  public HostNameField(String text, int columns, String desc)
  {
    super(text, columns, desc);

    emptyOk = false;
  }

  public void verifyContent() throws ParseException
  {

    String contentString = getText();

    if (emptyOk && (contentString.length() == 0))
    {
      return;
    }


    StringTokenizer myTokenizer = new StringTokenizer(contentString, ".", 
            true);

    // Check for correct form (7 tokens since I tell the tokenizer to
    // pass in the periods as well
    if (myTokenizer.countTokens() != 7)
    {
      throw new ParseException(getFieldDescription() + " : " 
              + "Fields should be of the form " + "###.###.###.###", 0);
    }

    // Are we expecting a dot this token? No when we start looking...
    boolean expectingdot = false;

    while (myTokenizer.hasMoreTokens())
    {

      String tok = myTokenizer.nextToken();


      if (!expectingdot)
      {         // should be a number

        char[] tokarray = tok.toCharArray();

        if (tokarray.length > 3)
        {
          throw new ParseException(getFieldDescription() + " : " 
                  + "Each section should have no " 
                  + "more than three digits", 0);
        }

        for (int i = 0; i < tokarray.length; i++)
        {
          if (!(Character.isDigit(tokarray[i])))
          {
            throw new ParseException(getFieldDescription() + " : " 
                    + "Only numbers may appear " + "between the dots, and " 
                    + "fields may not start or " + "end with a dot", i);
          }
        }

        int val = new Integer(tok).intValue();

        if (val > 255)
        {
          throw new ParseException(getFieldDescription() + " : " 
                  + "No number >255 may appear " + "between the dots", 0);
        }

        expectingdot = true;

      }
      else
      {         // should be a dot

        if (!tok.equalsIgnoreCase("."))
        {
          throw new ParseException(getFieldDescription() + " : " 
                  + "Fields must be of the form " + "###.###.###.###", 0);
        }

        expectingdot = false;

      }

    }           // end parser loop over tokens

  }

  public void setEmptyOk()
  {
    emptyOk = true;
  }

  private boolean emptyOk;

}       // end class

