/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.ui;

import javax.swing.JTextField;
import javax.swing.text.Caret;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

/**
 * $RCSfile: InternalClipboardField.java,v $
 * 
 * @author $Author: bsamson $, $Date: 2001/01/27 00:42:33 $
 * @version $Revision: 1.7 $
 */
public class InternalClipboardField extends JTextField
{

  /**
   * Used to store cut/copied text for all classes of this type.
   */
  protected static String internalClipboard = "";

  // constructors
  public InternalClipboardField()
  {
    super();
  }

  public InternalClipboardField(Document doc, String text, int columns)
  {
    super(doc, text, columns);
  }

  public InternalClipboardField(int columns)
  {
    super(columns);
  }

  public InternalClipboardField(String text)
  {
    super(text);
  }

  public InternalClipboardField(String text, int columns)
  {
    super(text, columns);
  }

  /**
   * 
   */
  protected static synchronized void setInternalClipboard(String newString)
  {
    internalClipboard = newString;
  }

  /**
   * 
   */
  protected static synchronized String getInternalClipboard()
  {
    return internalClipboard;
  }

  /**
   * Transfer the currently selected range to the internal clipboard,
   * removing the contents from the model.
   */
  public void cut()
  {
    if (isEditable() && isEnabled())
    {
      Caret caret = getCaret();
      int p0 = Math.min(caret.getDot(), caret.getMark());
      int p1 = Math.max(caret.getDot(), caret.getMark());

      if (p0 != p1)
      {
        try
        {
          Document doc = getDocument();
          String srcData = doc.getText(p0, p1 - p0);

          setInternalClipboard(srcData);
          doc.remove(p0, p1 - p0);
        }
        catch (BadLocationException e)
        {
          return;
        }
      }
    }
    else
    {
      getToolkit().beep();
    }
  }

  /**
   * Transfer the currently selected range to the internal clipboard,
   * leaving the current selection intact.
   */
  public void copy()
  {
    Caret caret = getCaret();
    int p0 = Math.min(caret.getDot(), caret.getMark());
    int p1 = Math.max(caret.getDot(), caret.getMark());

    if (p0 != p1)
    {
      try
      {
        Document doc = getDocument();
        String srcData = doc.getText(p0, p1 - p0);

        setInternalClipboard(srcData);
      }
      catch (BadLocationException e)
      {
        return;
      }
    }
  }

  /**
   * Tranfer the contents of the internal clipboard to the text model.
   * Any selected text is replaced with the transferred contents.
   */
  public void paste()
  {
    String dstData = getInternalClipboard();

    if (!dstData.equals(""))
    {
      try
      {
        dstData = getInternalClipboard();

        replaceSelection(dstData);
      }
      catch (Exception e)
      {
        getToolkit().beep();
      }
    }
  }

}



