/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.ui;

import vocal.comm.VPPTransactionWrapper;
import vocal.comm.VPPException;
import vocal.comm.VPPNoSuchFileException;
import vocal.pw.*;
import vocal.pw.CreateAdminAccountPanel;

import java.security.NoSuchAlgorithmException;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.BufferedReader;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;

import org.xml.sax.SAXException;


public abstract class LoginPanel extends JPanel
{
  private JTextField loginId;
  private JPasswordField password;

  private JButton submit;
  private JButton exit;

  private PasswordManager passwordManager;

  private JDialog createAdminAcct = null;
  private CreateAdminAccountPanel createAdminAcctPanel = null;


  protected VPPTransactionWrapper connect;


  public LoginPanel(boolean isStandAlone, VPPTransactionWrapper aConnection, 
                    JRootPane root)
  {
    connect = aConnection;

    JMenuBar menuBar = new JMenuBar();

    root.setJMenuBar(menuBar);

    JMenu menu = new JMenu("About");

    menuBar.add(Box.createHorizontalGlue());
    menuBar.add(menu);

    JMenuItem item = new JMenuItem("Version");

    menu.add(item);

    item.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        JOptionPane.showMessageDialog(LoginPanel.this.getTopLevelAncestor(), 
                "Vovida Open Communication Application Library\n\nVersion " 
                + Version.VERSION, "About", JOptionPane.INFORMATION_MESSAGE);
      }

    });

    passwordManager = new PasswordManager(connect);

    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    setBorder(new EmptyBorder(7, 7, 7, 7));

    loginId = new JTextField("");
    password = new JPasswordField();

    submit = new JButton("Login");
    exit = new JButton("Exit");

    JPanel temp = new JPanel();

    temp.setLayout(new BoxLayout(temp, BoxLayout.Y_AXIS));
    temp.add(Box.createVerticalStrut(10));
    temp.add(new JLabel("Login ID: "));
    temp.add(loginId);
    temp.add(Box.createVerticalStrut(10));
    add(temp);

    temp = new JPanel();

    temp.setLayout(new BoxLayout(temp, BoxLayout.Y_AXIS));
    temp.add(new JLabel("Password: "));
    temp.add(password);
    temp.add(Box.createVerticalStrut(10));
    add(temp);

    temp = new JPanel();

    temp.setLayout(new BoxLayout(temp, BoxLayout.X_AXIS));
    temp.add(Box.createHorizontalStrut(150));
    temp.add(submit);

    if (isStandAlone)
    {
      temp.add(Box.createHorizontalStrut(5));
      temp.add(exit);
    }

    add(temp);

    addActionListeners();
  }


  protected void addActionListeners()
  {
    exit.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {

        // the exit button is only shown on standalone apps.
        System.exit(0);
      }

    });

    password.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        startLogin();
      }

    });

    submit.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        startLogin();
      }

    });
  }

  private void startLogin()
  {
    if (loginId.getText().equals(""))
    {
      JOptionPane.showMessageDialog(LoginPanel.this, 
              "Please enter a login id.");

      return;
    }

    try
    {
      if (passwordManager.isAccessAllowed(getLoginId(), getPassword(), 
              getAccessLevel()))
      {
        loginId.setText(passwordManager.getLoginName());
        doLogin();

        password.setText("");
      }
      else
      {
        String message = "Incorrect password";

        if ((getAccessLevel() == AccessLevels.ADMINISTRATOR) 
            || (getAccessLevel() == AccessLevels.TECHNICIAN))
        {
          message += " or access level";
        }

        message += ".";

        JOptionPane.showMessageDialog(LoginPanel.this.getTopLevelAncestor(), 
                message);
        password.setText("");
      }
    }
    catch (UnknownLoginIdException ex)
    {

      // for users, just say the login id does not exist and move on
      if (getAccessLevel() == AccessLevels.USER)
      {
        JOptionPane.showMessageDialog(LoginPanel.this.getTopLevelAncestor(), 
                "Unknown user " + getLoginId() + ".");
      }

      // for administrators or technicians
      else if ((getAccessLevel() == AccessLevels.ADMINISTRATOR) 
               || (getAccessLevel() == AccessLevels.TECHNICIAN))
      {

        // See if there are any administrative accounts set up on the system
        // If this is the first time the login screen is run on a new system,
        // there won't be any.
        try
        {

          String response = connect.requestString("NLIST", "Admin_Accounts", 
                  "");

          if (response.length() == 1)
          {

            // there were no administrative accounts in the system -> create a
            // new one.

            // If there are no admin accounts, assume this is a new system and
            // create a default vovida/vovida account.
            JOptionPane.showMessageDialog(LoginPanel.this.getTopLevelAncestor(), 
                    "There are currently no administrator or technician accounts on the system.\n" 
                    + "Creating account with login/password vovida/vovida.");

            try
            {
              connect.doPut("Admin_Accounts", "vovida", 
                            "<admin><name>vovida</name><password>6f7d5634f65192a3b9c8479cee3b655</password><accessLevel type=\"administrator\" enabled=\"true\"></accessLevel><accessLevel type=\"technician\" enabled=\"true\"></accessLevel></admin>");
            }
            catch (VPPException e)
            {
              connect.showVPPException(e, "");
            }


            // now that the account has been created, log it in
            loginId.setText("vovida");
            password.setText("vovida");
            doLogin();

            // clear the password when control returns to this screen
            password.setText("");
          }
          else
          {

            // there were some admin accounts
            JOptionPane.showMessageDialog(LoginPanel.this.getTopLevelAncestor(), 
                    "Unknown login id " + getLoginId() + ".");
          }
        }
        catch (VPPException e)
        {
          connect.showVPPException(e, "");
        }
      }
    }
    catch (NoSuchAlgorithmException ex)
    {
      JOptionPane.showMessageDialog(LoginPanel.this.getTopLevelAncestor(), 
              "Could not find a provider for the requested hash algorithm.\n\n" 
              + ex, "ERROR", JOptionPane.ERROR_MESSAGE);
    }
    catch (SAXException ex)
    {
      ex.printStackTrace();
      JOptionPane.showMessageDialog(LoginPanel.this.getTopLevelAncestor(), 
              "Could not process the xml file for the given login id because:\n\n" 
              + ex);
    }
    catch (IOException ex)
    {
      ex.printStackTrace();
      JOptionPane.showMessageDialog(LoginPanel.this.getTopLevelAncestor(), 
              "Could not process the xml file for the given login id because:\n\n" 
              + ex);
    }
  }

  protected abstract void doLogin();

  protected abstract int getAccessLevel();

  protected String getLoginId()
  {
    return loginId.getText();
  }

  protected String getPassword()
  {
    return new String(password.getPassword());
  }

}


