/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.ui;

import java.util.Vector;
import javax.swing.UIManager;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;
import javax.swing.border.EmptyBorder;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Component;

/**
 * Used to render all a Vector's values into one table cell.  Each vector value
 * is placed on a separate line.
 */
public class VectorCellRenderer extends JPanel implements TableCellRenderer
{

  // a bin of label objects waiting to be used.  Since these can are
  // reused let's store them.  If more are needed they are dynamically
  // created later.
  private JLabel[] labels = 
  {
    new JLabel(), new JLabel(), new JLabel()
  };

  private EmptyBorder noFocusBorder = new EmptyBorder(1, 2, 1, 2);

  public VectorCellRenderer()
  {
    super.setLayout(new GridLayout(0, 1));
  }

  // implements javax.swing.table.TableCellRenderer
  public Component getTableCellRendererComponent(JTable table, Object value, 
          boolean isSelected, boolean hasFocus, int row, int column)
  {

    // first, remove any labels that may be left over from the last call
    super.removeAll();

    if (isSelected)
    {
      super.setForeground(table.getSelectionForeground());
      super.setBackground(table.getSelectionBackground());
    }
    else
    {
      super.setForeground(table.getForeground());
      super.setBackground(table.getBackground());
    }

    super.setFont(table.getFont());

    if (hasFocus)
    {
      setBorder(UIManager.getBorder("Table.focusCellHighlightBorder"));

      if (table.isCellEditable(row, column))
      {
        super.setForeground(UIManager.getColor("Table.focusCellForeground"));
        super.setBackground(UIManager.getColor("Table.focusCellBackground"));
      }
    }
    else
    {
      setBorder(noFocusBorder);
    }

    Vector valueVec;

    if (value instanceof Vector)
    {
      valueVec = (Vector) value;

      for (int i = 0; i < valueVec.size(); i++)
      {
        if (i < this.labels.length)
        {
          labels[i].setText(valueVec.get(i).toString());
          labels[i].setForeground(this.getForeground());
          labels[i].setBackground(this.getBackground());
          labels[i].setFont(table.getFont());
          super.add(labels[i]);
        }
        else
        {
          JLabel label = new JLabel(valueVec.get(i).toString());

          label.setForeground(this.getForeground());
          label.setBackground(this.getBackground());
          label.setFont(table.getFont());
          super.add(label);
        }
      }
    }
    else if (value != null)
    {
      labels[0].setText(value.toString());
      labels[0].setForeground(this.getForeground());
      labels[0].setBackground(this.getBackground());
      labels[0].setFont(table.getFont());
      super.add(labels[0]);
    }

    return this;
  }

}

