/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */


package vocal.userEditor;

import vocal.util.*;
import vocal.comm.VPPException;


import java.util.Vector;
import java.io.IOException;
import org.xml.sax.SAXException;
import javax.swing.JOptionPane;
import java.awt.Dimension;
import java.awt.Toolkit;


public abstract class EditorTask extends Task
{
  protected Editor editor;

  /**
   * Constructs a new EditorTask for the given Editor. <p>
   * @param editor this will be prompted by the task to contact the
   * provisioning server to perform operations such as loading account data
   * as needed.
   */
  public EditorTask(Editor anEditor)
  {
    editor = anEditor;
  }


  /**
   * Loads user account data for the accounts at the given rows. <p>
   * Retrieves the table model from the editor and causes it contact the
   * provisioning server to retrieve the account data for the given rows.
   * @param a list of the rows for which to load user data.
   */
  protected void loadUsers(int[] rows)
  {
    for (int i = 0; i < rows.length; i++)
    {
      Vector user = editor.getData().getUserAt(rows[i]);

      if (user.size() <= 1)     // if the user's data has not been loaded yet
      {
        if (i % 10 == 0)        // report every 10th user
        {
          Logger.swingLog("Loading user # " + i + " - " 
                          + editor.getData().getValueAt(rows[i], 
                          UserTableModel.USER_NAME));
        }

        try
        {
          editor.getData().loadUserAt(rows[i]);
        }
        catch (IOException ex)
        {
          ex.printStackTrace();
          JOptionPane.showMessageDialog(editor.getTopLevelAncestor(), 
                  "Could not load user data for user " 
                  + editor.getData().getValueAt(rows[i], UserTableModel.USER_NAME) 
                  + "because: \n\n" + ex);
        }
        catch (SAXException ex)
        {
          ex.printStackTrace();
          JOptionPane.showMessageDialog(editor.getTopLevelAncestor(), 
                  "Could not load user data for user " 
                  + editor.getData().getValueAt(rows[i], UserTableModel.USER_NAME) 
                  + "because: \n\n" + ex);
        }
        catch (VPPException ex)
        {
          ex.printStackTrace();
          JOptionPane.showMessageDialog(editor.getTopLevelAncestor(), 
                  "Could not load user data for user " 
                  + editor.getData().getValueAt(rows[i], UserTableModel.USER_NAME) 
                  + "because: \n\n" + ex);
        }
      }
    }                           // end for all rows

    // under jdk 1.3, it seems that the above operation causes the row selection
    // to be lost. It seems to occur after the fireTableChanged method is called
    // So we need to restore the row selection.
    for (int i = 0; i < rows.length; i++)
    {
      editor.getTable().addRowSelectionInterval(rows[i], rows[i]);
    }

    Logger.swingLog("Done loading users");
  }

}

