/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.userEditor;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;


public class FeatureCallerIdBlocking extends Feature
{
  private JCheckBox userSet;
  private boolean userSetChanged = false;


  protected FeatureCallerIdBlocking(String[] newGroups, String name, 
          PServerInterface psInt)
  {
    super(newGroups, name, psInt);

    userSet = new JCheckBox("Set");

    userSet.setToolTipText("check box to hide your phone number when you make a call");

    JPanel temp = new JPanel();

    temp.setLayout(new BoxLayout(temp, BoxLayout.X_AXIS));
    temp.add(userSet);
    temp.add(Box.createHorizontalGlue());
    userLayout.add(temp);

    addChangeListener();
  }


  public void setMode(int mode)
  {

    // TODO: implement this vocal.userEditor.Feature abstract method
  }


  protected void addChangeListener()
  {
    userSet.addChangeListener(new ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        FeatureCallerIdBlocking.this.userSetChanged = true;
      }

    });

    adminEnabled.addChangeListener(new ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        if (!adminEnabled.isSelected())
        {
          userSet.setEnabled(false);
          userSet.setSelected(false);

          userSetChanged = true;
        }
        else
        {
          userSet.setEnabled(true);
        }
      }

    });
  }


  public String getData(int dataId)
  {
    switch (dataId)
    {

      case UserTableModel.CALLER_ID_BLOCKING_ADMIN_ENABLED:
      {
        if (adminEnabledChanged)
        {
          return adminEnabled.isSelected() + "";
        }
      }

        break;

      case UserTableModel.CALLER_ID_BLOCKING_GROUP:
      {
        if (!featureGroups.getSelectedItem().equals(""))        // if the value chaned
        {
          return featureGroups.getSelectedItem().toString();
        }
      }

        break;

      case UserTableModel.CALLER_ID_BLOCKING_USER_SET:
      {
        if (userSetChanged)
        {
          if (userSet.isSelected())
          {
            return "ON";
          }
          else
          {
            return "OFF";
          }
        }
      }

        break;
    }

    return null;
  }


  public void clearFeatureSpecificFields()
  {
    userSet.setSelected(false);

    userSetChanged = false;
  }


  public boolean setData(int id, String data)
  {
    switch (id)
    {

      case UserTableModel.CALLER_ID_BLOCKING_ADMIN_ENABLED:
      {
        if (data.equalsIgnoreCase("true"))
        {
          adminEnabled.setSelected(true);
        }
        else
        {
          adminEnabled.setSelected(false);
        }

        return true;
      }

      case UserTableModel.CALLER_ID_BLOCKING_GROUP:
      {
        featureGroups.setSelectedItem(data);

        return true;
      }

      case UserTableModel.CALLER_ID_BLOCKING_USER_SET:
      {
        if (data.equalsIgnoreCase("ON"))
        {
          userSet.setSelected(true);
        }
        else
        {
          userSet.setSelected(false);
        }
      }
    }   // end switch dataId

    return false;
  }


  private void enableUserFields(boolean enable)
  {
    userSet.setEnabled(enable);
  }


  protected void disableFeature()
  {
    enableUserFields(false);

    // turn the feature off
    userSet.setSelected(false);
  }


  protected void enableFeature()
  {
    enableUserFields(adminEnabled.isSelected());
  }


  public boolean isDataValid()
  {
    return true;        // no validating fields avaliable for this component yet
  }

}

