/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.userEditor;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

import java.awt.Dimension;



class FeatureForwardAllCalls extends Feature
{
  private JCheckBox cfaSet;
  private JTextField cfaTarget;

  private boolean cfaSetChanged = false;


  protected FeatureForwardAllCalls(String[] newGroups, String name, 
          PServerInterface psInt)
  {
    super(newGroups, name, psInt);

    cfaSet = new JCheckBox("Set");

    cfaSet.setToolTipText("check box to forward all calls");

    cfaTarget = new JTextField();

    cfaTarget.setToolTipText("number to forward all calls to");

    cfaSet.setPreferredSize(new Dimension(COLUMN_WIDTH, 
            cfaSet.getHeight()));       // bit of a hack

    Dimension size = cfaTarget.getPreferredSize();
    Dimension size2 = cfaTarget.getSize();

    cfaTarget.setPreferredSize(new Dimension(COLUMN_WIDTH / 2, size.height));

    JPanel temp = new JPanel();

    temp.setLayout(new BoxLayout(temp, BoxLayout.X_AXIS));
    temp.add(cfaSet);
    temp.add(new JLabel("Forward To: "));
    temp.add(cfaTarget);

    userLayout.add(temp);

    addChangeListener();
  }


  public String getData(int dataId)
  {
    switch (dataId)
    {

      case UserTableModel.FORWARD_ALL_ADMIN_ENABLED:
      {
        if (adminEnabledChanged)
        {
          return adminEnabled.isSelected() + "";
        }
      }

        break;

      case UserTableModel.FORWARD_ALL_GROUP:
      {
        if (!featureGroups.getSelectedItem().equals(""))        // if the value chaned
        {
          return (String) featureGroups.getSelectedItem();
        }
      }

        break;

      case UserTableModel.FORWARD_ALL_USER_SET:
      {
        if (cfaSetChanged)
        {
          if (cfaSet.isSelected())
          {
            return "ON";
          }
          else
          {
            return "OFF";
          }
        }
      }

        break;

      case UserTableModel.FORWARD_ALL_DESTINATION:
      {
        if (!cfaTarget.getText().equals(""))
        {
          return cfaTarget.getText();
        }
      }
    }   // end switch

    return null;
  }     // end method getData


  public boolean setData(int id, String data)
  {
    switch (id)
    {

      case UserTableModel.FORWARD_ALL_ADMIN_ENABLED:
      {
        if (data.equalsIgnoreCase("true"))
        {
          adminEnabled.setSelected(true);
        }
        else
        {
          adminEnabled.setSelected(false);
        }

        return true;
      }

      case UserTableModel.FORWARD_ALL_GROUP:
      {
        featureGroups.setSelectedItem(data);

        return true;
      }

      case UserTableModel.FORWARD_ALL_USER_SET:
      {
        if (data.equalsIgnoreCase("ON"))
        {
          cfaSet.setSelected(true);
        }
        else
        {
          cfaSet.setSelected(false);
        }

        return true;
      }

      case UserTableModel.FORWARD_ALL_DESTINATION:
      {
        cfaTarget.setText(data);

        return true;
      }
    }   // end switch

    return false;
  }


  protected void addChangeListener()
  {
    cfaSet.addChangeListener(new ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        FeatureForwardAllCalls.this.cfaSetChanged = true;
      }

    });

    adminEnabled.addChangeListener(new ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        if (adminEnabled.isSelected())
        {
          enableUserFields(true);
        }
        else
        {
          enableUserFields(false);
          cfaSet.setSelected(false);

          cfaSetChanged = true;
        }
      }

    });
  }


  public void clearFeatureSpecificFields()
  {
    cfaTarget.setText("");
    cfaSet.setSelected(false);

    cfaSetChanged = false;
  }


  public void setMode(int mode)
  {

    // nothing to do so far
  }


  private void enableUserFields(boolean enable)
  {
    cfaSet.setEnabled(enable);
    cfaTarget.setEnabled(enable);
  }


  protected void disableFeature()
  {
    enableUserFields(false);

    // turn the feature off
    cfaSet.setSelected(false);

    cfaSetChanged = true;
  }


  protected void enableFeature()
  {
    enableUserFields(adminEnabled.isSelected());
  }


  public boolean isDataValid()
  {
    return true;        // no validating fields avaliable for this component yet
  }

}

