/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.userEditor;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.awt.Dimension;


class FeatureForwardNoAnswerBusy extends Feature
{
  private JCheckBox fnaSet;
  private JCheckBox fbSet;
  private JTextField fnaTarget;
  private JTextField fbTarget;
  private JTextField failureCase;

  private boolean fbSetChanged = false;
  private boolean fnaSetChanged = false;


  protected FeatureForwardNoAnswerBusy(String[] newGroups, String name, 
          PServerInterface psInt)
  {
    super(newGroups, name, psInt);

    fnaSet = new JCheckBox("Forward Unanswered");

    fnaSet.setToolTipText("check box to forward unanswered calls to another number");

    fbSet = new JCheckBox("Forward Busy");

    fbSet.setToolTipText("check box to forward calls when your line is busy");
    fnaSet.setToolTipText("check box to forward calls that you do not answer");

    fnaTarget = new JTextField("");

    fnaTarget.setToolTipText("forward unanswered calls to this number");

    fbTarget = new JTextField("");

    fbTarget.setToolTipText("forward busy calls to this number");

    failureCase = new JTextField("");

    failureCase.setToolTipText("fall-back number to route calls to");

    fnaSet.setPreferredSize(new Dimension(COLUMN_WIDTH, 
            fnaSet.getHeight()));       // bit of a hack
    fbSet.setPreferredSize(new Dimension(COLUMN_WIDTH, 
            fbSet.getHeight()));        // bit of a hack

    JPanel temp = new JPanel();

    temp.setLayout(new BoxLayout(temp, BoxLayout.X_AXIS));
    temp.add(new JLabel("Failure Case: "));
    temp.add(Box.createHorizontalGlue());
    temp.add(failureCase);
    adminLayout.add(temp);

    temp = new JPanel();

    temp.setLayout(new BoxLayout(temp, BoxLayout.X_AXIS));
    temp.add(fnaSet);
    temp.add(new JLabel("Forward To: "));
    temp.add(fnaTarget);
    userLayout.add(temp);

    temp = new JPanel();

    temp.setLayout(new BoxLayout(temp, BoxLayout.X_AXIS));
    temp.add(fbSet);
    temp.add(new JLabel("Forward To: "));
    temp.add(fbTarget);
    userLayout.add(temp);

    addChangeListener();
  }


  public String getData(int dataId)
  {
    switch (dataId)
    {

      case UserTableModel.FORWARD_BUSY_DESTINATION:
      {
        if (!fbTarget.getText().equals(""))
        {
          return fbTarget.getText();
        }
      }

        break;

      case UserTableModel.FORWARD_BUSY_USER_SET:
      {
        if (fbSetChanged)
        {
          return fbSet.isSelected() + "";
        }
      }

        break;

      case UserTableModel.FORWARD_UNANSWERED_DESTINATION:
      {
        if (!fnaTarget.getText().equals(""))
        {
          return fnaTarget.getText();
        }
      }

        break;

      case UserTableModel.FORWARD_UNANSWERED_USER_SET:
      {
        if (fnaSetChanged)
        {
          return fnaSet.isSelected() + "";
        }
      }

        break;

      case UserTableModel.FORWARD_BUSY_ADMIN_ENABLED:
      {
        if (adminEnabledChanged)
        {
          return adminEnabled.isSelected() + "";
        }
      }

        break;

      case UserTableModel.FORWARD_BUSY_GROUP:
      {
        if (!featureGroups.getSelectedItem().equals(""))
        {
          return featureGroups.getSelectedItem().toString();
        }
      }

        break;

      case UserTableModel.FAILURE_CASE:
      {
        if (!failureCase.getText().equals(""))
        {
          return failureCase.getText();
        }
      }

        break;
    }

    return null;
  }


  public boolean setData(int dataId, String data)
  {
    switch (dataId)
    {

      case UserTableModel.FORWARD_BUSY_DESTINATION:
      {
        fbTarget.setText(data);

        return true;
      }

      case UserTableModel.FORWARD_BUSY_USER_SET:
      {
        if (data.equalsIgnoreCase("true"))
        {
          fbSet.setSelected(true);
        }
        else
        {
          fbSet.setSelected(false);
        }

        return true;
      }

      case UserTableModel.FORWARD_UNANSWERED_DESTINATION:
      {
        fnaTarget.setText(data);

        return true;
      }

      case UserTableModel.FORWARD_UNANSWERED_USER_SET:
      {
        if (data.equalsIgnoreCase("true"))
        {
          fnaSet.setSelected(true);
        }
        else
        {
          fnaSet.setSelected(false);
        }

        return true;
      }

      case UserTableModel.FORWARD_BUSY_ADMIN_ENABLED:
      {
        if (data.equalsIgnoreCase("true"))
        {
          adminEnabled.setSelected(true);

          enableUserFields(true);
        }
        else
        {
          adminEnabled.setSelected(false);

          // if the administrator has disabled this feature, disable all the
          // user editable fields as well
          enableUserFields(false);
        }

        return true;
      }

      case UserTableModel.FORWARD_BUSY_GROUP:
      {
        featureGroups.setSelectedItem(data);

        return true;
      }

      case UserTableModel.FAILURE_CASE:
      {
        failureCase.setText(data);

        return true;
      }
    }

    return false;
  }


  protected void addChangeListener()
  {
    fnaSet.addChangeListener(new ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        FeatureForwardNoAnswerBusy.this.fnaSetChanged = true;
      }

    });

    adminEnabled.addChangeListener(new ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        if (adminEnabled.isSelected())
        {
          enableUserFields(true);
        }
        else
        {
          enableUserFields(false);
          fbSet.setSelected(false);

          fbSetChanged = true;

          fnaSet.setSelected(false);

          fnaSetChanged = true;
        }
      }

    });

    fbSet.addChangeListener(new ChangeListener()
    {
      public void stateChanged(ChangeEvent e)
      {
        FeatureForwardNoAnswerBusy.this.fbSetChanged = true;
      }

    });
  }


  public void clearFeatureSpecificFields()
  {
    fnaSet.setSelected(false);
    fbSet.setSelected(false);

    fnaSetChanged = false;
    fbSetChanged = false;

    fnaTarget.setText("");
    fbTarget.setText("");
  }


  public void setMode(int mode)
  {

    // nothing to do so far
  }


  private void enableUserFields(boolean enable)
  {
    fnaSet.setEnabled(enable);
    fbSet.setEnabled(enable);
    fnaTarget.setEnabled(enable);
    fbTarget.setEnabled(enable);
  }


  protected void disableFeature()
  {
    failureCase.setEditable(false);
    enableUserFields(false);

    // Turn the feature off
    fbSet.setSelected(false);
    fnaSet.setSelected(false);

    fbSetChanged = true;        // make sure that the change is saved
    fnaSetChanged = true;
  }


  protected void enableFeature()
  {
    failureCase.setEditable(true);
    enableUserFields(adminEnabled.isSelected());
  }


  public boolean isDataValid()
  {
    return true;        // no validating fields avaliable for this component yet
  }

}

