/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.userEditor;

import vocal.comm.VPPTransactionWrapper;
import vocal.comm.VPPException;
import vocal.comm.VPPNoSuchFileException;
import vocal.data.XMLUtils;
import vocal.util.Logger;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.helpers.DefaultHandler;


import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.Vector;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringReader;
import java.net.ConnectException;

public class PServerInterface
{
  private VPPTransactionWrapper connection = null;
  private DOMParser parser = new DOMParser();
  private Hashtable featureGroups = null;
  private Hashtable groupServers = null;
  private String[] marshalTypes = null;


  public PServerInterface(VPPTransactionWrapper con)
  {
    connection = con;
  }


  public PServerInterface(String aHost, int aPort) throws VPPException
  {
    connection = new VPPTransactionWrapper(aHost, aPort);
  }

  public void deleteUser(String name) throws VPPNoSuchFileException
  {
    try
    {
      connection.request("DELETEUSER", name, "");
    }
    catch (VPPException ex)
    {
      if (VPPNoSuchFileException.class.isInstance(ex))
      {
        throw (VPPNoSuchFileException) ex;
      }
      else
      {
        connection.showVPPException(ex, "");
      }
    }
  }

  public void deleteAdmin(String name) throws VPPNoSuchFileException
  {
    try
    {
      connection.request("REMOVE", "Admin_Accounts", name);
    }
    catch (VPPException ex)
    {
      if (VPPNoSuchFileException.class.isInstance(ex))
      {
        throw (VPPNoSuchFileException) ex;
      }
      else
      {
        connection.showVPPException(ex, "");
      }
    }
  }

  public void saveUser(String name, Document doc)
  {
    String xml = new XMLUtils().buildXMLString(doc);

    try
    {
      connection.doPut("Accounts", name, xml);
    }
    catch (VPPException e)
    {
      connection.showVPPException(e, "");
    }
  }

  /**
   * Pass through for doPut. <p>
   */
  public void put(String group, String filename, String data)
  {
    try
    {
      connection.doPut(group, filename, data);
    }
    catch (VPPException e)
    {
      connection.showVPPException(e, "");
    }
  }

  public String get(String group, 
                    String filename) throws VPPNoSuchFileException
  {
    try
    {
      return connection.doGet(group, filename);
    }
    catch (VPPException ex)
    {
      if (VPPNoSuchFileException.class.isInstance(ex))
      {
        throw (VPPNoSuchFileException) ex;
      }
      else
      {
        connection.showVPPException(ex, "");
      }
    }

    return "";
  }

  private Document getAccountNamed(String group, 
          String filename) throws VPPException, IOException, SAXException
  {
    BufferedReader user = connection.request("GET", group, filename);

    parser.parse(new InputSource(user));

    return parser.getDocument();
  }


  public Document getAdminNamed(String name) 
    throws VPPException, IOException, SAXException
  {
    return getAccountNamed("Admin_Accounts", name);
  }

  /**
   * Retrieve the xml file for a user with the given name and parse it.
   */
  public Document getUserNamed(String name) 
    throws VPPException, IOException, SAXException
  {
    return getAccountNamed("Accounts", name);
  }

  public String[] getAllAdminNames() throws VPPNoSuchFileException
  {
    return getAllNames("Admin_Accounts");
  }

  public String[] getAllAliasNames() throws VPPNoSuchFileException
  {
    return getAllNames("Aliases");
  }

  public String getAliasNamed(String alias) throws VPPNoSuchFileException
  {
    try
    {
      return connection.doGet("Aliases", alias);
    }
    catch (VPPException ex)
    {
      if (VPPNoSuchFileException.class.isInstance(ex))
      {
        throw (VPPNoSuchFileException) ex;
      }
      else
      {
        connection.showVPPException(ex, "");
      }
    }

    return "";
  }

  private String[] getAllNames(String group) throws VPPNoSuchFileException
  {
    String[] names = new String[]{};

    Logger.swingLog("Downloading list of user names from server ...");


    String fileList = null;

    try
    {
      fileList = connection.requestString("NLIST", group, "");
    }
    catch (VPPException ex)
    {
      if (VPPNoSuchFileException.class.isInstance(ex))
      {
        Logger.swingLog("No names found");

        throw (VPPNoSuchFileException) ex;
      }
      else
      {
        connection.showVPPException(ex, "");
      }
    }

    if (fileList == null)
    {
      return names;
    }

    if (fileList.charAt(fileList.length() - 1) == '\n')
    {
      fileList = fileList.substring(0, Math.max(0, fileList.length() - 2));
    }

    if (fileList.length() == 0)
    {
      Logger.swingLog("No names found");

      return names;
    }

    Logger.swingLog("Starting to tokenize user name list ...");

    StringTokenizer tokenizer = new StringTokenizer(fileList, ",", false);

    Vector namesTemp = new Vector();
    String name;

    while (tokenizer.hasMoreTokens())
    {
      name = tokenizer.nextToken();

      namesTemp.addElement(name);
    }

    Logger.swingLog("Done tokenizing user name list ...");

    names = new String[namesTemp.size()];

    namesTemp.copyInto(names);

    Logger.swingLog("Starting to sort user name list ...");

    java.util.Arrays.sort(names);

    Logger.swingLog("Done sorting ");

    return names;
  }

  /**
   * Retrieve a directory listing for all the /Accounts/ dir to get a list of
   * the names of all the users.
   */
  public String[] getAllUserNames() throws VPPNoSuchFileException
  {
    return getAllNames("Accounts");
  }

  public String[] getMarshalGroups() 
    throws VPPNoSuchFileException, IOException, SAXException
  {
    if (marshalTypes == null)
    {
      String filegroup = "SystemConfiguration";
      String filename = "ListOfMarshalServers";

      try
      {
        BufferedReader configInfo = connection.request("GET", filegroup, 
                filename);

        parser.parse(new InputSource(configInfo));

        Document doc = parser.getDocument();

        NodeList marshals = doc.getElementsByTagName("serverType");

        if (marshals.getLength() == 0)
        {
          throw new FileNotFoundException("There are no marshal servers defined in " 
                  + filegroup + " " + filename);
        }

        for (int i = 0; i < marshals.getLength(); i++)
        {
          Element marshal = (Element) marshals.item(i);
          String type = marshal.getAttribute("value");

          if (type.equals("UserAgent"))
          {
            NodeList groups = marshal.getElementsByTagName("serverGroup");

            marshalTypes = new String[groups.getLength()];

            for (int j = 0; j < groups.getLength(); j++)
            {
              Element group = (Element) groups.item(j);
              String name = group.getAttribute("value");

              marshalTypes[j] = name;
            }
          }
        }
      }
      catch (VPPException ex)
      {
        if (VPPNoSuchFileException.class.isInstance(ex))
        {
          throw new VPPNoSuchFileException("Could not load file " + filegroup 
                  + " " + filename + "\n\nbecause:\n\n" + ex.getMessage());
        }
        else
        {
          connection.showVPPException(ex, "");
        }
      }
    }

    if ((marshalTypes == null) || (marshalTypes.length == 0))
    {
      throw new FileNotFoundException("Could not find any marshal types. Probably could not parse file.");
    }

    return marshalTypes;
  }

  /**
   * Find out whether a particular group for a particular feature type has at
   * least one server associated with it.
   */
  public boolean hasServer(String featureType, String group) 
    throws IOException, SAXException, VPPNoSuchFileException
  {
    if (groupServers == null)
    {
      getFeatureInformation();
    }

    Boolean has = (Boolean) groupServers.get(featureType + group);

    if (has != null)
    {
      return true;
    }

    return false;
  }


  /**
   * Get the groups which are configured for the feature with the given name. <p>
   * The data comes from the SystemConfiguration_ListofFeatureServers file
   * on the pserver.
   */
  public String[] getFeatureGroups(String featureName) 
    throws IOException, SAXException, VPPNoSuchFileException
  {
    if (featureGroups == null)
    {
      getFeatureInformation();
    }

    Vector groups = (Vector) featureGroups.get(featureName);
    String[] groupNames = new String[groups.size()];

    for (int i = 0; i < groups.size(); i++)
    {
      groupNames[i] = (String) groups.elementAt(i);
    }

    return groupNames;
  }

  /**
   * Retrieves a list of the names of the features which are configured. <p>
   * The data comes from the SystemConfiguration_ListofFeatureServers file
   * on the pserver.
   */
  public String[] getFeatureTypes() 
    throws IOException, SAXException, VPPNoSuchFileException
  {
    if (featureGroups == null)
    {
      getFeatureInformation();
    }

    Set keys = featureGroups.keySet();
    String[] types = new String[keys.size()];
    Iterator iterator = keys.iterator();
    int i = 0;

    while (iterator.hasNext())
    {
      types[i] = (String) iterator.next();
      i++;
    }

    return types;
  }

  /**
   * Reads the SystemConfiguration_ListOfFeatureServers file from the pserver
   * to determine the feature types and the group names for each feature which
   * are configured on the pserver.
   */
  private void getFeatureInformation() 
    throws IOException, SAXException, VPPNoSuchFileException
  {
    featureGroups = new Hashtable();
    groupServers = new Hashtable();

    String filegroup = "SystemConfiguration";
    String filename = "ListOfFeatureServers";

    try
    {
      BufferedReader configInfo = connection.request("GET", filegroup, 
              filename);

      parser.parse(new InputSource(configInfo));

      Document doc = parser.getDocument();

      NodeList features = doc.getElementsByTagName("serverType");

      for (int i = 0; i < features.getLength(); i++)
      {

        // get the feature name
        String typeName = 
          features.item(i).getAttributes().getNamedItem("value").getNodeValue();

        // get a list of groups associated with that feature
        NodeList groups = 
          ((Element) features.item(i)).getElementsByTagName("serverGroup");
        Vector groupNames = new Vector();

        for (int j = 0; j < groups.getLength(); j++)
        {
          Element group = (Element) groups.item(j);
          String groupName = 
            group.getAttributes().getNamedItem("value").getNodeValue();

          if (groupName != null)
          {
            groupNames.addElement(groupName);

            // find out if this feature group has any servers associated with it
            if (group.hasChildNodes())
            {
              NodeList servers = 
                group.getElementsByTagName("server");   // get all server nodes

              if (servers.getLength() > 0)
              {
                String host = 
                  ((Element) servers.item(0)).getAttribute("host");

                if (host != null)
                {
                  groupServers.put(typeName + groupName, 
                                   new Boolean(true));  // key is featureType+groupName
                }       // end if host attribute is defined
              }         // end if there are server elements
            }           // end if group has child nodes
          }             // end if group has name

        }

        if (groupNames.size() > 0)
        {
          featureGroups.put(typeName, groupNames);
        }
      }
    }
    catch (VPPException ex)
    {
      if (VPPNoSuchFileException.class.isInstance(ex))
      {
        throw new VPPNoSuchFileException("Could not load file " + filegroup 
                + " " + filename + "\n\nbecause:\n\n" + ex.getMessage());
      }
      else
      {
        connection.showVPPException(ex, "");
      }
    }
  }                     // end method getFeatureGroups

  /**
   * Get the underlying VPPTransactionWrapper which is used by this class
   * to communicate with the pserver.
   */
  public VPPTransactionWrapper getConnection()
  {
    return connection;
  }     // end method getConnection

  public static String getMasterUserNameFromXml(String xml)
  {

    // this should be done using an xml parser ...
    int index = xml.indexOf("masterUser=\"") + "masterUser=\"".length();
    String masterName = xml.substring(index, xml.indexOf("\"", index + 1));

    return masterName;
  }

  /**
   * Takes the given xml string and extracts the value of the password element.
   * Returns null if the element is not present.
   */
  public String getPasswordFromXml(String xml)
  {
    SAXParser parser = new SAXParser();
    PasswordHandler handler = new PasswordHandler();

    parser.setContentHandler(handler);

    try
    {
      parser.parse(new InputSource(new StringReader(xml)));

      return handler.getPassword();
    }
    catch (Exception e)
    {

      // quietly suppress this exception
      e.printStackTrace();
    }

    return null;
  }


  /**
   * A content handler for the sax parser used to retrieve the password element
   * from the file. Note that the password is returned in its raw (hashed)
   * form.
   */
  class PasswordHandler extends DefaultHandler
  {
    private boolean atPasswordElement = false;
    private String password = null;

    public void startElement(String ignore1, String ignore2, String rawName, 
            Attributes attr)
    {
      if (rawName.equals("password"))
      {
        atPasswordElement = true;
      }
    }

    public void characters(char[] ch, int start, int length)
    {
      if (atPasswordElement)
      {
        password = new String(ch, start, length);
      }
    }

    public void endElement(java.lang.String uri, java.lang.String localName, 
            java.lang.String rawName)
    {
      if (rawName.equals("password"))
      {
        atPasswordElement = false;
      }
    }

    public String getPassword()
    {
      return password;
    }

  }     // end inner class PasswordHandler


  /**
   * Check whenter the supplied xml string allows access level of the supplied
   * type (ie: admin or tech).
   * This parses the file, finds the "accessLevel" elements (returns false if
   * they do not exist), matches the "type" attribute to the supplied type and
   * then returns the value of the "enabled" attribute.
   */
  public boolean allowsAccessOfType(String xml, 
          String type) throws IOException, SAXException
  {
    DOMParser parser = new DOMParser();

    parser.parse(new InputSource(new StringReader(xml)));

    Document doc = parser.getDocument();

    NodeList accessLevels = doc.getElementsByTagName("accessLevel");

    if (accessLevels == null)
    {
      return false;
    }

    if (accessLevels.getLength() < 1)
    {
      return false;
    }

    for (int i = 0; i < accessLevels.getLength(); i++)
    {
      Element accessLevel = (Element) accessLevels.item(i);
      String typeFromFile = accessLevel.getAttribute("type");

      if (typeFromFile.equals(type))
      {
        String enabled = accessLevel.getAttribute("enabled");

        if (enabled.equals("true"))
        {
          return true;
        }
        else
        {
          return false;
        }
      }
    }

    return false;
  }

  /**
   * @return true if the alias file was successfully created and saved, false
   * otherwise.
   */
  public boolean saveAlias(String aliasName, String masterName)
  {

    // want aliases to be unique, so check whether there is a user with the
    // same name as the proposed alias
    try
    {
      connection.doGet("Accounts", aliasName);
    }
    catch (VPPException exx)
    {
      if (VPPNoSuchFileException.class.isInstance(exx))
      {

        // this user name does not exist -> ok
        // now check whether this alias already exists
        String response = "";

        try
        {
          response = connection.doGet("Aliases", aliasName);
        }
        catch (VPPException ex)
        {
          if (VPPNoSuchFileException.class.isInstance(ex))
          {

            // there is not a user with this name either
            // so its ok to create the alias
            String xml = "<userAlias masterUser=\"" + masterName 
                         + "\" alias=\"" + aliasName + "\"></userAlias>";

            try
            {
              connection.doPut("Aliases", aliasName, xml);
            }
            catch (VPPException e)
            {
              connection.showVPPException(e, "");
            }

            return true;
          }     // end if no such user
          else
          {
            connection.showVPPException(ex, "");
          }
        }

        // if the alias exists, that is not necessarily and error: it may belong
        // to this user
        if (response.indexOf(masterName) != -1)

        // if the user name is contained anywhere in the alias file, then assume
        // that this alias belongs to this user. This should probably be done
        // by actually parsing the file, but for now its good enough.
        {
          return true;
        }
      }         // end if no such alias
      else
      {
        connection.showVPPException(exx, "");
      }
    }

    return false;
  }


  public void deleteAliasFile(String alias) throws VPPNoSuchFileException
  {
    try
    {
      connection.request("REMOVE", "Aliases", alias);
    }
    catch (VPPException ex)
    {
      if (VPPNoSuchFileException.class.isInstance(ex))
      {
        throw (VPPNoSuchFileException) ex;
      }
      else
      {
        connection.showVPPException(ex, "");
      }
    }
  }

}       // end class PServerInterface

