/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.userEditor;

import vocal.pw.*;
import vocal.comm.VPPTransactionWrapper;

import javax.swing.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.border.EmptyBorder;

import java.security.NoSuchAlgorithmException;

public class PasswordPanel extends JPanel
{
  private JPasswordField oldPassword;
  private JPasswordField newPassword;
  private JPasswordField reEnterPassword;

  private JButton ok;
  private JButton cancel;

  /**
   * String representation of the new password if the change was successfull
   */
  String password = null;

  private String userName;

  private PasswordManager pwManager;

  private int lastButtonClicked = Buttons.BUTTON_NONE;


  public PasswordPanel(VPPTransactionWrapper con)
  {
    pwManager = new PasswordManager(con);

    oldPassword = new JPasswordField();
    newPassword = new JPasswordField();
    reEnterPassword = new JPasswordField();


    ok = new JButton("OK");

    ok.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        String new1 = new String(newPassword.getPassword());
        String new2 = new String(reEnterPassword.getPassword());

        if (!new1.equals(new2))
        {
          JOptionPane.showMessageDialog(PasswordPanel.this.getTopLevelAncestor(), 
                  "The new passwords did not match. Please try again.");
          newPassword.setText("");
          reEnterPassword.setText("");
        }
        else
        {
          PasswordPanel.this.password = new1;
          PasswordPanel.this.lastButtonClicked = Buttons.BUTTON_OK;

          PasswordPanel.this.getTopLevelAncestor().setVisible(false);
        }
      }

    });

    cancel = new JButton("Cancel");

    cancel.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        PasswordPanel.this.getTopLevelAncestor().setVisible(false);
      }

    });


    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

    JPanel temp;

    temp = new JPanel();

    temp.setLayout(new BoxLayout(temp, BoxLayout.X_AXIS));
    temp.add(new JLabel("New password: "));
    temp.add(Box.createHorizontalGlue());

    add(temp);
    add(newPassword);
    add(Box.createVerticalStrut(5));

    temp = new JPanel();

    temp.setLayout(new BoxLayout(temp, BoxLayout.X_AXIS));
    temp.add(new JLabel("Re-enter new password: "));
    temp.add(Box.createHorizontalGlue());

    add(temp);
    add(reEnterPassword);
    add(Box.createVerticalStrut(5));
    add(Box.createVerticalStrut(5));

    setBorder(new EmptyBorder(5, 5, 5, 5));

    temp = new JPanel();

    temp.setLayout(new BoxLayout(temp, BoxLayout.X_AXIS));
    temp.add(Box.createHorizontalGlue());
    temp.add(ok);
    temp.add(Box.createHorizontalStrut(5));
    temp.add(cancel);
    add(temp);
  }

  public void clear()
  {
    newPassword.setText("");
    reEnterPassword.setText("");
    oldPassword.setText("");

    password = null;

    lastButtonClicked = Buttons.BUTTON_NONE;
  }

  public int getLastButtonClicked()
  {
    return lastButtonClicked;
  }

  public void setUserId(String newId)
  {
    userName = newId;
  }

  public String getPassword()
  {
    try
    {
      return pwManager.computeHash(password);
    }
    catch (NoSuchAlgorithmException e)
    {
      e.printStackTrace();
      JOptionPane.showMessageDialog(this.getTopLevelAncestor(), 
              "Could not compute hash of password because: \n\n" + e);
    }

    return null;
  }

}

