/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.userEditor;

import vocal.ui.UserNameField;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import java.text.ParseException;



public class UserInfo extends JPanel implements DataEditor
{
  private JTextField group;
  private UserNameField name;

  private static Color lightGray = new Color(235, 235, 235);

  boolean nameChanged = false;
  boolean groupChanged = false;

  private int mode = UserPanel.ADD_NEW_MODE;



  public UserInfo()
  {
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    setBorder(new EmptyBorder(5, 5, 5, 5));

    name = new UserNameField("User name");

    name.setToolTipText("primary phone number for this user");

    group = new JTextField();

    group.setToolTipText("arbitrary text string for classifying this user");

    add(createLabelComponentPair("Name:", name));
    add(createLabelComponentPair("Group:", group));

    final Color backgroundColor = name.getBackground();

    name.setBackground(lightGray);
    group.setBackground(lightGray);

    name.addKeyListener(new KeyAdapter()
    {
      public void keyTyped(KeyEvent e)
      {
        name.setBackground(backgroundColor);

        nameChanged = true;
      }

    });

    group.addKeyListener(new KeyAdapter()
    {
      public void keyTyped(KeyEvent e)
      {
        group.setBackground(backgroundColor);

        groupChanged = true;
      }

    });
  }


  private JPanel createLabelComponentPair(String labelText, Component comp)
  {
    JPanel temp = new JPanel();

    temp.setBorder(new EmptyBorder(1, 1, 1, 1));

    JLabel label = new JLabel(labelText);
    Dimension dim = new Dimension(80, label.getHeight());

    label.setPreferredSize(dim);
    temp.setLayout(new BoxLayout(temp, BoxLayout.X_AXIS));
    temp.add(label);
    temp.add(comp);

    return temp;
  }


  // ----------------------DataEditor implementation------------------------------

  public String getData(int id)
  {
    switch (id)
    {

      case UserTableModel.USER_NAME:
      {
        if (nameChanged)
        {
          return name.getText();
        }
      }

        break;

      case UserTableModel.USER_GROUP:
      {
        if (groupChanged)
        {
          return group.getText();
        }
      }

        break;
    }

    return null;
  }


  /**
   * Clear the value of all editor components in this panel and set their
   * corresponding "changed" flags to false.
   */
  public void clear()
  {
    name.setText("");
    group.setText("");

    nameChanged = false;
    groupChanged = false;

    name.setBackground(lightGray);
    group.setBackground(lightGray);
  }


  public boolean setData(int id, String data)
  {
    switch (id)
    {

      case UserTableModel.USER_NAME:
      {
        name.setText(data);

        return true;
      }

      case UserTableModel.USER_GROUP:
      {
        group.setText(data);

        return true;
      }
    }

    return false;
  }


  public void setMode(int newMode)
  {
    if (mode != newMode)
    {
      mode = newMode;

      if (newMode == UserPanel.EDIT_EXISTING_MODE)
      {
        name.setEditable(false);
        name.setEnabled(false);
      }
      else if (newMode == UserPanel.ADD_NEW_MODE)
      {
        name.setEditable(true);
        name.setEnabled(true);
      }

      validate();
    }
  }


  public boolean isDataValid()
  {
    try
    {
      name.verifyContent();
    }
    catch (ParseException ex)
    {
      ex.printStackTrace();
      JOptionPane.showMessageDialog(this.getTopLevelAncestor(), 
              ex.getMessage(), "Invalid input", JOptionPane.ERROR_MESSAGE);

      return false;
    }

    return true;
  }

  // ----------------------End DataEditor implementation---------------------------
}

