/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
package vocal.userEditor;

import vocal.util.Logger;
import vocal.comm.VPPException;
import vocal.comm.VPPNoSuchFileException;

import vocal.data.XMLUtils;
import vocal.data.CplGenerator;
import vocal.data.ContactListGenerator;
import vocal.data.NoSuchNodeException;
import vocal.data.NotTextNodeException;

import javax.swing.table.AbstractTableModel;
import javax.swing.JOptionPane;

import java.util.Vector;
import java.util.Hashtable;
import java.util.StringTokenizer;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringReader;

import org.apache.xerces.dom.ElementImpl;
import org.apache.xerces.dom.DocumentImpl;
import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;


class UserTableModel extends AbstractTableModel
{

  // NOTE: these have to be in the same order as the columns displayed in the
  // table in Editor.java. Also, any data elements that are not displayed in
  // that table (such as the list of call screen numbers) must be entered here
  // at the end, after those that are in the table.
  public static final int USER_NAME =                     0;
  public static final int USER_GROUP =                    1;
  public static final int IP =                            2;
  public static final int MARSHAL =                       3;

  public static final int FORWARD_ALL_ADMIN_ENABLED =     4;
  public static final int FORWARD_ALL_GROUP =             5;
  public static final int FORWARD_ALL_USER_SET =          6;
  public static final int FORWARD_ALL_DESTINATION =       7;

  public static final int FORWARD_BUSY_ADMIN_ENABLED =    8;
  public static final int FORWARD_BUSY_GROUP =            9;
  public static final int FORWARD_BUSY_USER_SET =         10;
  public static final int FORWARD_BUSY_DESTINATION =      11;
  public static final int FORWARD_UNANSWERED_USER_SET =   12;
  public static final int FORWARD_UNANSWERED_DESTINATION =13;

  public static final int FAILURE_CASE =                  14;

  public static final int CALL_SCREEN_ADMIN_ENABLED =     15;
  public static final int CALL_SCREEN_GROUP =             16;

  public static final int CALL_BLOCK_PREFIX =             17;
  public static final int CALL_BLOCK_ADMIN_ENABLED =      18;
  public static final int CALL_BLOCK_GROUP =              19;
  public static final int BLOCK_LONG_DISTANCE_ADMIN_ENABLED = 20;
  public static final int BLOCK_LONG_DISTANCE_USER_SET =  21;
  public static final int BLOCK_900_ADMIN_ENABLED =       22;
  public static final int BLOCK_900_USER_SET =            23;

  public static final int CALL_RETURN_ADMIN_ENABLED =     24;
  public static final int CALL_RETURN_GROUP =             25;

  public static final int CALLER_ID_BLOCKING_ADMIN_ENABLED = 26;
  public static final int CALLER_ID_BLOCKING_GROUP =      27;
  public static final int CALLER_ID_BLOCKING_USER_SET =   28;

  public static final int AUTHENTICATION_TYPE =           29;
  public static final int AUTHENTICATION_PASSWORD =       30;
  public static final int STATIC_REGISTRATION_ENABLED =   31;
  public static final int TERMINATING_HOST =              32;
  public static final int TERMINATING_PORT =              33;

  public static final int PASSWORD =                      34;
  public static final int CALL_SCREEN_NUMBERS =           35;

  public static final int IS_ALIAS =                      36;
  public static final int MASTER_NAME =                   37;
  public static final int MASTER_INDEX =                  38;
  public static final int ALIASES =                       39;

  public static final int MAX_COLUMN_ID = ALIASES;

  private static BufferedReader sampleUserXML =
    new BufferedReader(new StringReader("<user><name>1000</name>"
          + "<password>6f7d5634f65192a3b9c8479cee3b655</password>"
          +     // it's "vovida"
  "<ipaddr>000.000.000.000</ipaddr>" + "<marshalgroup>UserAgentGroup</marshalgroup>"
          + "<staticRegistration value=\"false\">"
          + "<terminatingContactList>"
          + "<contact value=\"UserAgentGroup\"></contact>"
          + "<contact value=\"000.000.000.000:0000\"></contact>"
          + "</terminatingContactList>" + "</staticRegistration>"
          + "<authenticationType value=\"None\">"
          + "<authenticationPassword value=\"password\"></authenticationPassword>"
          + "</authenticationType>" + "<failurecase>unknown</failurecase>"
          + "<cfa enabled=\"false\"><setfeat>OFF</setfeat> "
          + "<featuregroup>ForwardAllCallsGroup</featuregroup>"
          + "<forwardto>none</forwardto>" + "</cfa>"
          + "<fnab enabled=\"false\"><setfeat>OFF</setfeat>"
          + "<featuregroup>ForwardNoAnswerBusyGroup</featuregroup>"
          + "<fna set=\"false\">" + "<forwardto>none</forwardto>" + "</fna>"
          + "<cfb set=\"false\">" + "<forwardto>none</forwardto>" + "</cfb>"
          + "</fnab>" + "<cs enabled=\"false\"><setfeat>OFF</setfeat>"
          + "<featuregroup>CallScreeningGroup</featuregroup>" + "</cs>"
          + "<clbl enabled=\"false\"><setfeat>OFF</setfeat>"
          + "<prefix></prefix>"
          + "<featuregroup>CallBlockingGroup</featuregroup>"
          + "<nineHundredBlocked adminSet=\"false\" userSet=\"false\"></nineHundredBlocked>"
          + "<longDistanceBlocked adminSet=\"false\" userSet=\"false\"></longDistanceBlocked>"
          + "</clbl>" + "<callReturn enabled=\"false\"><setfeat>OFF</setfeat>"
          + "<featuregroup>CallReturnGroup</featuregroup>" + "</callReturn>"
          + "<callerIdBlocking enabled=\"false\"><setfeat>OFF</setfeat>"
          + "<featuregroup>CallerIdBlockingGroup</featuregroup>"
          + "</callerIdBlocking>"
          + "<voicemail enabled =\"false\"><setfeat>OFF</setfeat>"
          + "<featuregroup>VoiceMailGroup</featuregroup>" + "</voicemail>"
          + "<userGroup>none</userGroup>" + "</user>"));

  private static Document dom;

  private static Vector dummyUser;

  private boolean isShowingAliases = false;
  private boolean aliasesLoadedFromServer = false;

  private Vector userColumnIndices = new Vector();
  private Vector adminColumnIndices = new Vector();

  private static ColumnInfo[] columns = new ColumnInfo[]
  {
    new ColumnInfo("Name", 175, USER_NAME),
    new ColumnInfo("User Group", 175, USER_GROUP),
    new ColumnInfo("IP", 173, IP),
    new ColumnInfo("Marshal", 170, MARSHAL),
    new ColumnInfo("Forward All Enabled", 150, FORWARD_ALL_ADMIN_ENABLED),
    new ColumnInfo("Forward All Group", 150, FORWARD_ALL_GROUP),
    new ColumnInfo("Forward All Set", 150, FORWARD_ALL_USER_SET),
    new ColumnInfo("Forward All To", 150, FORWARD_ALL_DESTINATION),
    new ColumnInfo("Forward Busy/No Ans Enabled", 150, FORWARD_BUSY_ADMIN_ENABLED),
    new ColumnInfo("Forward Busy/No Ans Group", 150, FORWARD_BUSY_GROUP),
    new ColumnInfo("Forward Busy Set", 150, FORWARD_BUSY_USER_SET),
    new ColumnInfo("Forward Busy To", 150, FORWARD_BUSY_DESTINATION),
    new ColumnInfo("Forward No Ans Set", 150, FORWARD_UNANSWERED_USER_SET),
    new ColumnInfo("Forward No Ans To", 150, FORWARD_UNANSWERED_DESTINATION),
    new ColumnInfo("Failure Case", 150, FAILURE_CASE),
    new ColumnInfo("Call Screen Enabled", 150, CALL_SCREEN_ADMIN_ENABLED),
    new ColumnInfo("Call Screen Group", 150, CALL_SCREEN_GROUP),
    new ColumnInfo("Call Block Prefix", 150, CALL_BLOCK_PREFIX),
    new ColumnInfo("Call Block Enabled", 150, CALL_BLOCK_ADMIN_ENABLED),
    new ColumnInfo("Call Block Group", 150, CALL_BLOCK_GROUP),
    new ColumnInfo("Long Distance Admin Block", 150, BLOCK_LONG_DISTANCE_ADMIN_ENABLED),
    new ColumnInfo("Long Distance User Block", 150, BLOCK_LONG_DISTANCE_USER_SET),
    new ColumnInfo("900 # Admin Block", 150, BLOCK_900_ADMIN_ENABLED),
    new ColumnInfo("900 # User Block", 150, BLOCK_900_USER_SET),
    new ColumnInfo("Call Return Enabled", 150, CALL_RETURN_ADMIN_ENABLED),
    new ColumnInfo("Call Return Group", 150, CALL_RETURN_GROUP),
    new ColumnInfo("Caller ID Block Enabled", 150, CALLER_ID_BLOCKING_ADMIN_ENABLED),
    new ColumnInfo("Caller ID Group", 150, CALLER_ID_BLOCKING_GROUP),
    new ColumnInfo("Caller ID Set", 150, CALLER_ID_BLOCKING_USER_SET),
    new ColumnInfo("Authentication Type", 150, AUTHENTICATION_TYPE),
    new ColumnInfo("Password", 150, AUTHENTICATION_PASSWORD),
    new ColumnInfo("Static Reg Enabled", 150, STATIC_REGISTRATION_ENABLED),
    new ColumnInfo("Terminating host", 150, TERMINATING_HOST),
    new ColumnInfo("Terminating port", 150, TERMINATING_PORT)
  };

  /**
   * Stores the list of users (as a vector of vectors)
   */
  private Vector data = new Vector();

  /**
   * Stores the list of aliases (as a vector of vectors). The each alias would
   * be stored in a vector of the same form as the master user names.
   */
  private Vector aliases = new Vector();

  /**
   * Handles communication with pServer
   */
  PServerInterface psInterface;

  /**
   * Used to find paths and generate cpl files
   */
  private CplGenerator cplGen;

  static
  {
    DOMParser parser = new DOMParser();

    try
    {
      parser.parse(new InputSource(sampleUserXML));

      dom = parser.getDocument();
    }
    catch (Exception e)
    {

      // If this happens, there was an error parsing the hard coded sample user
      // XML. That should never happen. It would suggest a coding error.
      e.printStackTrace();
      JOptionPane.showMessageDialog(null, "Could not parse sample user XML",
              "ERROR", JOptionPane.ERROR_MESSAGE);
    }
  }


  public UserTableModel(PServerInterface con)
  {
    psInterface = con;

    dummyUser = convertUserToVector(dom);

    userColumnIndices.add(new Integer(FORWARD_ALL_USER_SET));
    userColumnIndices.add(new Integer(FORWARD_ALL_DESTINATION));
    userColumnIndices.add(new Integer(FORWARD_BUSY_USER_SET));
    userColumnIndices.add(new Integer(FORWARD_BUSY_DESTINATION));
    userColumnIndices.add(new Integer(FORWARD_UNANSWERED_USER_SET));
    userColumnIndices.add(new Integer(FORWARD_UNANSWERED_DESTINATION));
    userColumnIndices.add(new Integer(BLOCK_LONG_DISTANCE_USER_SET));
    userColumnIndices.add(new Integer(BLOCK_900_USER_SET));
    userColumnIndices.add(new Integer(CALLER_ID_BLOCKING_USER_SET));

    adminColumnIndices.add(new Integer(FORWARD_ALL_ADMIN_ENABLED));
    adminColumnIndices.add(new Integer(FORWARD_ALL_GROUP));
    adminColumnIndices.add(new Integer(FORWARD_BUSY_ADMIN_ENABLED));
    adminColumnIndices.add(new Integer(FORWARD_BUSY_GROUP));
    adminColumnIndices.add(new Integer(FAILURE_CASE));
    adminColumnIndices.add(new Integer(CALL_SCREEN_ADMIN_ENABLED));
    adminColumnIndices.add(new Integer(CALL_SCREEN_GROUP));
    adminColumnIndices.add(new Integer(CALL_BLOCK_PREFIX));
    adminColumnIndices.add(new Integer(CALL_BLOCK_ADMIN_ENABLED));
    adminColumnIndices.add(new Integer(CALL_BLOCK_GROUP));
    adminColumnIndices.add(new Integer(BLOCK_LONG_DISTANCE_ADMIN_ENABLED));
    adminColumnIndices.add(new Integer(BLOCK_900_ADMIN_ENABLED));
    adminColumnIndices.add(new Integer(CALL_RETURN_ADMIN_ENABLED));
    adminColumnIndices.add(new Integer(CALL_RETURN_GROUP));
    adminColumnIndices.add(new Integer(CALLER_ID_BLOCKING_ADMIN_ENABLED));
    adminColumnIndices.add(new Integer(CALLER_ID_BLOCKING_GROUP));
    adminColumnIndices.add(new Integer(AUTHENTICATION_TYPE));
    adminColumnIndices.add(new Integer(AUTHENTICATION_PASSWORD));
    adminColumnIndices.add(new Integer(STATIC_REGISTRATION_ENABLED));
    adminColumnIndices.add(new Integer(TERMINATING_HOST));
    adminColumnIndices.add(new Integer(TERMINATING_PORT));
  }

  public Vector getUserColumnIndices()
  {
    return userColumnIndices;
  }

  public Vector getAdminColumnIndices()
  {
    return adminColumnIndices;
  }

  public void addUser(Vector user)
  {
    data.addElement(user);
  }

  public void addUser(Document doc)
  {
    Vector user = convertUserToVector(doc);

    data.addElement(user);
  }

  private Document convertUserAtToDocument(int row)
  {
    return convertUserToDocument(getUserAt(row));
  }

  public static Document convertUserToDocument(Vector user)
  {
    dom.getElementsByTagName("name").item(0).getFirstChild().setNodeValue((String) user.elementAt(UserTableModel.USER_NAME));
    dom.getElementsByTagName("ipaddr").item(0).getFirstChild().setNodeValue((String) user.elementAt(UserTableModel.IP));
    dom.getElementsByTagName("marshalgroup").item(0).getFirstChild().setNodeValue((String) user.elementAt(UserTableModel.MARSHAL));
    dom.getElementsByTagName("failurecase").item(0).getFirstChild().setNodeValue((String) user.elementAt(UserTableModel.FAILURE_CASE));

    // caller id blocking
    NodeList idBlockings = dom.getElementsByTagName("callerIdBlocking");
    Element idBlocking = (Element) idBlockings.item(0);

    idBlocking.setAttribute("enabled",
            (String) user.elementAt(CALLER_ID_BLOCKING_ADMIN_ENABLED));

    NodeList setfeats = idBlocking.getElementsByTagName("setfeat");
    Element setfeat = (Element) setfeats.item(0);

    setfeat.getFirstChild().setNodeValue((String) user.elementAt(CALLER_ID_BLOCKING_USER_SET));

    NodeList groups = idBlocking.getElementsByTagName("featuregroup");
    Element group = (Element) groups.item(0);

    group.getFirstChild().setNodeValue((String) user.elementAt(CALLER_ID_BLOCKING_GROUP));

    // usr group
    NodeList userGroups = dom.getElementsByTagName("userGroup");
    Element userGroup = (Element) userGroups.item(0);

    userGroup.getFirstChild().setNodeValue((String) user.elementAt(USER_GROUP));


    // call return
    NodeList callReturns = dom.getElementsByTagName("callReturn");
    Element callReturn = (Element) callReturns.item(0);

    callReturn.setAttribute("enabled",
            (String) user.elementAt(UserTableModel.CALL_RETURN_ADMIN_ENABLED));

    setfeats = callReturn.getElementsByTagName("setfeat");
    setfeat = (Element) setfeats.item(0);

    // There is no "user set" gui element for the call return feature since
    // this is not something that the user sets for himself. (To keep the xml
    // consistent) the user setfeat is set to the same state as the admin
    // enabled field
    if (user.elementAt(UserTableModel.CALL_RETURN_ADMIN_ENABLED).equals("true"))
    {
      setfeat.getFirstChild().setNodeValue("ON");
    }
    else
    {
      setfeat.getFirstChild().setNodeValue("OFF");
    }

    groups = callReturn.getElementsByTagName("featuregroup");
    group = (Element) groups.item(0);

    group.getFirstChild().setNodeValue((String) user.elementAt(UserTableModel.CALL_RETURN_GROUP));


    // set the authentication type
    NodeList authTypes = dom.getElementsByTagName("authenticationType");
    Element authType = (Element) authTypes.item(0);

    authType.setAttribute("value",
            (String) user.elementAt(UserTableModel.AUTHENTICATION_TYPE));

    NodeList authPasswords =
      dom.getElementsByTagName("authenticationPassword");
    Element authPassword = (Element) authPasswords.item(0);
    authPassword.setAttribute("value",
            (String) user.elementAt(UserTableModel.AUTHENTICATION_PASSWORD));

    // set the static registration
    NodeList statRegs = dom.getElementsByTagName("staticRegistration");
    Element statReg = (Element) statRegs.item(0);
    statReg.setAttribute("value",
            (String) user.elementAt(UserTableModel.STATIC_REGISTRATION_ENABLED));

    Element oldTerminatingContacts =
      (Element) statReg.getElementsByTagName("terminatingContactList").item(0);
    Element terminatingContacts = dom.createElement("terminatingContactList");

    statReg.replaceChild(terminatingContacts, oldTerminatingContacts);

    statReg.appendChild(terminatingContacts);

    Element contact = dom.createElement("contact");

    contact.setAttribute("value",
            (String) user.elementAt(UserTableModel.MARSHAL));
    terminatingContacts.appendChild(contact);

    contact = dom.createElement("contact");

    String contactStr = user.elementAt(UserTableModel.TERMINATING_HOST) + ":"
            + user.elementAt(UserTableModel.TERMINATING_PORT);

    contact.setAttribute("value", contactStr);
    terminatingContacts.appendChild(contact);


    // set the password element
    NodeList passwords = dom.getElementsByTagName("password");

    if (passwords != null)
    {
      if (passwords.getLength() > 0)
      {
        Node password = passwords.item(0);

        if (password.hasChildNodes())
        {
          password.getFirstChild().setNodeValue((String) user.elementAt(UserTableModel.PASSWORD));
        }
      }
      else
      {
        if (!user.elementAt(UserTableModel.PASSWORD).equals("not loaded from xml"))
        {
          ElementImpl password = new ElementImpl((DocumentImpl) dom,
                  "password");

          password.setNodeValue((String) user.elementAt(UserTableModel.PASSWORD));
          dom.appendChild(password);
        }
      }
    }
    else
    {
      if (!user.elementAt(UserTableModel.PASSWORD).equals("not loaded from xml"))
      {
        ElementImpl password = new ElementImpl((DocumentImpl) dom,
                "password");

        password.setNodeValue((String) user.elementAt(UserTableModel.PASSWORD));
        dom.appendChild(password);
      }
    }

    // create/set the alias list

    // create a new "aliases" element
    Element newAliases = dom.createElement("aliases");

    // replace the existing "aliases" element with
    // the one just created above, or if it does not exist, insert it into the
    // dom
    NodeList aliasesList = dom.getElementsByTagName("aliases");

    if (aliasesList != null)
    {
      if (aliasesList.getLength() > 0)
      {
        Element aliases = (Element) aliasesList.item(0);

        // an "aliases" element existed -> replace it
        dom.getDocumentElement().replaceChild(newAliases, aliases);
      }
      else      // the "aliases" element did not exist...
      {
        if (!user.elementAt(ALIASES).equals("not loaded from xml"))
        {

          // add it to the document
          dom.getDocumentElement().appendChild(newAliases);
        }
      }
    }
    else        // the "aliases" element was null
    {
      if (!user.elementAt(ALIASES).equals("not loaded from xml"))
      {

        // add it to the document
        dom.appendChild(newAliases);
      }
    }

    // create a new "alias" element for each alias
    String aliasString = (String) user.elementAt(ALIASES);

    while (aliasString.length() > 0)
    {
      String alias = aliasString.substring(0, aliasString.indexOf(";"));

      aliasString = aliasString.substring(aliasString.indexOf(";") + 1);

      Element element = new ElementImpl((DocumentImpl) dom, "alias");

      element.setAttribute("value", alias);

      newAliases.appendChild(element);
    }


    Element clblNode = (Element) dom.getElementsByTagName("clbl").item(0);
    Element csNode = (Element) dom.getElementsByTagName("cs").item(0);
    Element fnabNode = (Element) dom.getElementsByTagName("fnab").item(0);
    Element cfaNode = (Element) dom.getElementsByTagName("cfa").item(0);

    NodeList clblPrefixNodes = clblNode.getElementsByTagName("prefix");
    if (clblPrefixNodes.getLength() > 0)
    {
      Element clblPrefixNode = (Element)clblPrefixNodes.item(0);
      if (clblPrefixNode.hasChildNodes())
      {
        clblPrefixNode.getFirstChild().setNodeValue((String)user.elementAt(UserTableModel.CALL_BLOCK_PREFIX));
      }
      else
      {
        Node value = dom.createTextNode((String)user.elementAt(UserTableModel.CALL_BLOCK_PREFIX));
        clblPrefixNode.appendChild(value);
        clblNode.appendChild(clblPrefixNode);
      }
    }
    else
    {
      Element clblPrefixNode = dom.createElement("prefix");
      Node value = dom.createTextNode((String)user.elementAt(UserTableModel.CALL_BLOCK_PREFIX));
      clblPrefixNode.appendChild(value);
      clblNode.appendChild(clblPrefixNode);

/**
        ElementImpl clblPrefixNode = new ElementImpl((DocumentImpl) dom,
                "prefix");
        clblPrefixNode.setNodeValue((String)user.elementAt(UserTableModel.CALL_BLOCK_PREFIX));
        clblNode.appendChild(clblPrefixNode);
**/
    }


    try
    {
      Element block900Node = XMLUtils.getChildByName(clblNode,
              "nineHundredBlocked");

      block900Node.setAttribute("adminSet",
              (String) user.elementAt(UserTableModel.BLOCK_900_ADMIN_ENABLED));
      block900Node.setAttribute("userSet",
              (String) user.elementAt(UserTableModel.BLOCK_900_USER_SET));

      Element blockLongDist = XMLUtils.getChildByName(clblNode,
              "longDistanceBlocked");

      blockLongDist.setAttribute("adminSet",
              (String) user.elementAt(UserTableModel.BLOCK_LONG_DISTANCE_ADMIN_ENABLED));
      blockLongDist.setAttribute("userSet",
              (String) user.elementAt(UserTableModel.BLOCK_LONG_DISTANCE_USER_SET));

      // set the userset field of the clbl feature based on the values of userSet
      // for block900 and blockLongDist
      // if either nineHundredBlocked or LongDistanceBlocked are set true by the
      // user, or the admin the setfeat should be ON. Otherwise, it should be OFF
      if (user.elementAt(BLOCK_900_USER_SET).equals("true")
          || user.elementAt(BLOCK_LONG_DISTANCE_USER_SET).equals("true")
          || user.elementAt(BLOCK_900_ADMIN_ENABLED).equals("true")
          || user.elementAt(BLOCK_LONG_DISTANCE_ADMIN_ENABLED).equals("true"))
      {
        clblNode.getElementsByTagName("setfeat").item(0).getFirstChild().setNodeValue("ON");
      }
      else
      {
        clblNode.getElementsByTagName("setfeat").item(0).getFirstChild().setNodeValue("OFF");
      }


      // Set feature groups for all features
      XMLUtils.getChildByName(clblNode,
              "featuregroup").getChildNodes().item(0).setNodeValue((String) user.elementAt(UserTableModel.CALL_BLOCK_GROUP));
      XMLUtils.getChildByName(csNode,
              "featuregroup").getChildNodes().item(0).setNodeValue((String) user.elementAt(UserTableModel.CALL_SCREEN_GROUP));
      XMLUtils.getChildByName(cfaNode,
              "featuregroup").getChildNodes().item(0).setNodeValue((String) user.elementAt(UserTableModel.FORWARD_ALL_GROUP));
      XMLUtils.getChildByName(fnabNode,
              "featuregroup").getChildNodes().item(0).setNodeValue((String) user.elementAt(UserTableModel.FORWARD_BUSY_GROUP));
      XMLUtils.getChildByName(cfaNode,
              "setfeat").getChildNodes().item(0).setNodeValue((String) user.elementAt(UserTableModel.FORWARD_ALL_USER_SET));

      // set forward to field for call forward busy
      Element cfbNode = XMLUtils.getChildByName(fnabNode, "cfb");

      XMLUtils.getChildByName(cfbNode,
              "forwardto").getChildNodes().item(0).setNodeValue((String) user.elementAt(UserTableModel.FORWARD_BUSY_DESTINATION));
      cfbNode.setAttribute("set",
              (String) user.elementAt(UserTableModel.FORWARD_BUSY_USER_SET));

      // set forward to field for forward no answer
      Element fnaNode = XMLUtils.getChildByName(fnabNode, "fna");

      XMLUtils.getChildByName(fnaNode,
              "forwardto").getChildNodes().item(0).setNodeValue((String) user.elementAt(UserTableModel.FORWARD_UNANSWERED_DESTINATION));
      fnaNode.setAttribute("set",
              (String) user.elementAt(UserTableModel.FORWARD_UNANSWERED_USER_SET));

      // set forward to field for call forward all
      XMLUtils.getChildByName(cfaNode,
              "forwardto").getChildNodes().item(0).setNodeValue((String) user.elementAt(UserTableModel.FORWARD_ALL_DESTINATION));

      // chose the setfeat value for the forward no answer blocking feature based
      // on the values of the forward no answer and forward blocking nodes
      // if either cfb or fna is enabled, the setfeat is ON. Otherwise, it is OFF
      if (user.elementAt(UserTableModel.FORWARD_BUSY_USER_SET).equals("true")
          || user.elementAt(UserTableModel.FORWARD_UNANSWERED_USER_SET).equals("true"))
      {
        fnabNode.getElementsByTagName("setfeat").item(0).getFirstChild().setNodeValue("ON");
      }
      else
      {
        fnabNode.getElementsByTagName("setfeat").item(0).getFirstChild().setNodeValue("OFF");
      }
    }
    catch (NoSuchNodeException e)
    {
      System.out.println("Could not convert user data to a Document because:");
      e.printStackTrace();
    }

    // set whether each feature is enabled
    cfaNode.setAttribute("enabled",
            (String) user.elementAt(UserTableModel.FORWARD_ALL_ADMIN_ENABLED));
    fnabNode.setAttribute("enabled",
            (String) user.elementAt(UserTableModel.FORWARD_BUSY_ADMIN_ENABLED));
    csNode.setAttribute("enabled",
            (String) user.elementAt(UserTableModel.CALL_SCREEN_ADMIN_ENABLED));
    clblNode.setAttribute("enabled",
            (String) user.elementAt(UserTableModel.CALL_BLOCK_ADMIN_ENABLED));

    // save the data about who to screen calls from
    // deleting the "screenfrom" child nodes does not seem to work, so instead
    // create a whole new node so it has not "screenfrom" children and then use
    // it to replace the original csNode.
    Element newCsNode = new ElementImpl((DocumentImpl) dom, "cs");

    newCsNode.setAttribute("enabled", csNode.getAttribute("enabled"));
    newCsNode.appendChild(csNode.getElementsByTagName("setfeat").item(0));
    newCsNode.appendChild(csNode.getElementsByTagName("featuregroup").item(0));
    dom.getDocumentElement().replaceChild(newCsNode, csNode);

    csNode = newCsNode;

    // get the list of numbers to screen
    String screenFrom =
      (String) user.elementAt(UserTableModel.CALL_SCREEN_NUMBERS);


    if (user.elementAt(CALL_SCREEN_ADMIN_ENABLED).equals("true"))
    {

      // if the administrator has enabled this feature, then
      // if the list contains some numbers, that means that the user has enabled
      // that feature
      if (screenFrom.length() > 0)
      {
        csNode.getElementsByTagName("setfeat").item(0).getFirstChild().setNodeValue("ON");
      }
      else
      {
        csNode.getElementsByTagName("setfeat").item(0).getFirstChild().setNodeValue("OFF");
      }
    }
    else
    {

      // if the administrator set the feature to disabled, set the feature off for
      // the user even if there are some numbers to screen on the list
      csNode.getElementsByTagName("setfeat").item(0).getFirstChild().setNodeValue("OFF");
    }

    // for each entry in the call screen list, create a new element named "screenfrom",
    // create and attribute named "name" and another attribute named "number" then
    // add the new node as a child of the csNode
    while (screenFrom.length() > 0)
    {
      String name = screenFrom.substring(0, screenFrom.indexOf(","));
      String number = screenFrom.substring(screenFrom.indexOf(",") + 1,
              screenFrom.indexOf(";"));

      screenFrom = screenFrom.substring(screenFrom.indexOf(";") + 1);

      Element element = new ElementImpl((DocumentImpl) dom, "screenfrom");

      element.setAttribute("name", name);
      element.setAttribute("number", number);

      csNode.appendChild(element);
    }

    return dom;
  }


  private static Vector setFeatureGroup(Vector data, int dataId,
          Element parent)
  {
    NodeList nodeList = parent.getElementsByTagName("featuregroup");

    if (nodeList.getLength() > 0)
    {
      Node node = nodeList.item(0);

      if (node.hasChildNodes())
      {
        data.set(dataId, node.getFirstChild().getNodeValue());
      }
    }

    return data;
  }

  private static Vector setSetfeat(Vector data, int dataId, Element parent)
  {
    NodeList nodeList = parent.getElementsByTagName("setfeat");

    if (nodeList.getLength() > 0)
    {
      Node node = nodeList.item(0);

      if (node.hasChildNodes())
      {
        Node value = node.getFirstChild();

        data.set(dataId, value.getNodeValue());
      }
    }

    return data;
  }

  public static Vector convertUserToVector(Document user)
  {
    String temp = "not loaded from xml";

    // alocate enough space for all data elements
    Vector userData = new Vector(MAX_COLUMN_ID+6);

    for (int i = 0; i <= MAX_COLUMN_ID; i++)
    {
      userData.addElement(temp);
    }


    // user name
    NodeList elements = user.getElementsByTagName("name");

    if (elements.getLength() > 0)
    {
      Element element = (Element) elements.item(0);

      if (element.hasChildNodes())
      {
        userData.set(USER_NAME, element.getFirstChild().getNodeValue());
      }
    }


    // user group
    elements = user.getElementsByTagName("userGroup");

    if (elements.getLength() > 0)
    {
      Element element = (Element) elements.item(0);

      if (element.hasChildNodes())
      {
        userData.set(USER_GROUP, element.getFirstChild().getNodeValue());
      }
    }


    // ip address
    elements = user.getElementsByTagName("ipaddr");

    if (elements.getLength() > 0)
    {
      Element element = (Element) elements.item(0);

      if (element.hasChildNodes())
      {
        userData.set(IP, element.getFirstChild().getNodeValue());
      }
    }

    // marshalgroup
    elements = user.getElementsByTagName("marshalgroup");

    if (elements.getLength() > 0)
    {
      Element element = (Element) elements.item(0);

      if (element.hasChildNodes())
      {
        userData.set(MARSHAL, element.getFirstChild().getNodeValue());
      }
    }


    // load the password element, if it is present in the xml
    NodeList passwords = user.getElementsByTagName("password");

    if (passwords != null)
    {
      if (passwords.getLength() > 0)
      {
        if (passwords.item(0).hasChildNodes())
        {
          Node password = passwords.item(0).getFirstChild();

          if (password != null)
          {
            userData.set(UserTableModel.PASSWORD, password.getNodeValue());
          }
        }
      }
    }


    // Call blocking
    Element clblNode = null;
    NodeList nodeList = user.getElementsByTagName("clbl");

    if (nodeList.getLength() > 0)
    {
      clblNode = (Element) nodeList.item(0);
    }

    Element block900Node = null;
    Element blockLongDistNode = null;

    if (clblNode != null)
    {

      // 900 calls blocked
      nodeList = clblNode.getElementsByTagName("nineHundredBlocked");

      if (nodeList.getLength() > 0)
      {
        block900Node = (Element) nodeList.item(0);

        userData.set(UserTableModel.BLOCK_900_ADMIN_ENABLED,
                     block900Node.getAttribute("adminSet"));
        userData.set(UserTableModel.BLOCK_900_USER_SET,
                     block900Node.getAttribute("userSet"));
      }

      // long distance calls blocked
      nodeList = clblNode.getElementsByTagName("longDistanceBlocked");

      if (nodeList.getLength() > 0)
      {
        blockLongDistNode = (Element) nodeList.item(0);

        userData.set(UserTableModel.BLOCK_LONG_DISTANCE_ADMIN_ENABLED,
                     blockLongDistNode.getAttribute("adminSet"));
        userData.set(UserTableModel.BLOCK_LONG_DISTANCE_USER_SET,
                     blockLongDistNode.getAttribute("userSet"));
      }

      // feature group
      setFeatureGroup(userData, CALL_BLOCK_GROUP, clblNode);

      // admin enabled
      userData.set(UserTableModel.CALL_BLOCK_ADMIN_ENABLED,
                   clblNode.getAttribute("enabled"));

      //prefix
      nodeList = clblNode.getElementsByTagName("prefix");
      if (nodeList.getLength() > 0)
      {
        Node node = nodeList.item(0);
        if (node.hasChildNodes())
        {
          String data = (String)node.getFirstChild().getNodeValue();
          userData.set(UserTableModel.CALL_BLOCK_PREFIX,data);
        }
        else
        {
          userData.set(UserTableModel.CALL_BLOCK_PREFIX,"");
        }
      }
    }


    // Call screening
    Element csNode = null;

    nodeList = user.getElementsByTagName("cs");

    if (nodeList.getLength() > 0)
    {
      csNode = (Element) nodeList.item(0);
    }

    if (csNode != null)
    {

      // feature group
      setFeatureGroup(userData, CALL_SCREEN_GROUP, csNode);
      userData.set(UserTableModel.CALL_SCREEN_ADMIN_ENABLED,
                   csNode.getAttribute("enabled"));

      // get the list of name,numbers for the call screening feature
      NodeList csList = csNode.getElementsByTagName("screenfrom");
      String screenFrom = "";

      for (int i = 0; i < csList.getLength(); i++)
      {
        Element screen = (Element) csList.item(i);

        screenFrom = screenFrom + screen.getAttribute("name");
        screenFrom = screenFrom + ",";
        screenFrom = screenFrom + screen.getAttribute("number");
        screenFrom = screenFrom + ";";
      }

      userData.set(UserTableModel.CALL_SCREEN_NUMBERS, screenFrom);
    }


    // forward all calls
    Element cfaNode = null;

    nodeList = user.getElementsByTagName("cfa");

    if (nodeList.getLength() > 0)
    {
      cfaNode = (Element) nodeList.item(0);
    }

    if (cfaNode != null)
    {
      setFeatureGroup(userData, FORWARD_ALL_GROUP, cfaNode);
      setSetfeat(userData, FORWARD_ALL_USER_SET, cfaNode);
      userData.set(UserTableModel.FORWARD_ALL_ADMIN_ENABLED,
                   cfaNode.getAttribute("enabled"));

      // destination
      nodeList = cfaNode.getElementsByTagName("forwardto");

      if (nodeList.getLength() > 0)
      {
        Node node = nodeList.item(0);

        if (node.hasChildNodes())
        {
          userData.set(FORWARD_ALL_DESTINATION,
                       node.getFirstChild().getNodeValue());
        }
      }
    }


    // forward no-answer, busy calls
    Element fnabNode = null;

    nodeList = user.getElementsByTagName("fnab");

    if (nodeList.getLength() > 0)
    {
      fnabNode = (Element) nodeList.item(0);
    }

    if (fnabNode != null)
    {
      setFeatureGroup(userData, FORWARD_BUSY_GROUP, fnabNode);
      userData.set(UserTableModel.FORWARD_BUSY_ADMIN_ENABLED,
                   fnabNode.getAttribute("enabled"));

      // call forward busy destination
      nodeList = fnabNode.getElementsByTagName("cfb");

      if (nodeList.getLength() > 0)
      {
        Element cfbNode = (Element) nodeList.item(0);

        nodeList = cfbNode.getElementsByTagName("forwardto");

        if (nodeList.getLength() > 0)
        {
          Node node = nodeList.item(0);

          if (node.hasChildNodes())
          {
            userData.set(FORWARD_BUSY_DESTINATION,
                         node.getFirstChild().getNodeValue());
          }
        }

        // call forward busy set
        userData.set(UserTableModel.FORWARD_BUSY_USER_SET,
                     cfbNode.getAttribute("set"));
      }


      // call forward no answer destination
      nodeList = fnabNode.getElementsByTagName("fna");

      if (nodeList.getLength() > 0)
      {
        Element fnaNode = (Element) nodeList.item(0);

        nodeList = fnaNode.getElementsByTagName("forwardto");

        if (nodeList.getLength() > 0)
        {
          Node node = nodeList.item(0);

          if (node.hasChildNodes())
          {
            userData.set(FORWARD_UNANSWERED_DESTINATION,
                         node.getFirstChild().getNodeValue());
          }
        }

        // call forwared no answer set
        userData.set(UserTableModel.FORWARD_UNANSWERED_USER_SET,
                     fnaNode.getAttribute("set"));
      }
    }


    // load the list of aliases, if present
    String aliasesString = "";
    NodeList aliases = user.getElementsByTagName("aliases");

    if (aliases != null)
    {
      if (aliases.getLength() > 0)
      {
        Element aliasesElement = (Element) aliases.item(0);
        NodeList aliasList = aliasesElement.getElementsByTagName("alias");

        if (aliasList != null)
        {
          if (aliasList.getLength() > 0)
          {
            for (int i = 0; i < aliasList.getLength(); i++)
            {
              Element aliasElement = (Element) aliasList.item(i);
              String alias = aliasElement.getAttribute("value");

              aliasesString += alias + ";";
            }
          }
        }
      }
    }

    userData.set(UserTableModel.ALIASES, aliasesString);

    NodeList fcs = user.getElementsByTagName("failurecase");

    if (fcs.getLength() > 0)
    {
      Node fc = fcs.item(0);

      if (fc.hasChildNodes())
      {
        userData.set(FAILURE_CASE, fc.getFirstChild().getNodeValue());
      }
    }

    // Authentication type and password
    NodeList authenticationTypes =
      user.getElementsByTagName("authenticationType");

    if (authenticationTypes.getLength() > 0)
    {
      Element authenticationType = (Element) authenticationTypes.item(0);
      String type = authenticationType.getAttribute("value");

      userData.set(AUTHENTICATION_TYPE, type);

      NodeList authPasswords =
        authenticationType.getElementsByTagName("authenticationPassword");

      if (authPasswords.getLength() > 0)
      {
        Element authPassword = (Element) authPasswords.item(0);
        String pw = authPassword.getAttribute("value");

        userData.set(AUTHENTICATION_PASSWORD, pw);
      }
    }

    // static registration
    NodeList staticRegs = user.getElementsByTagName("staticRegistration");

    if (staticRegs.getLength() > 0)
    {
      Element staticReg = (Element) staticRegs.item(0);
      String enabled = staticReg.getAttribute("value");

      userData.set(STATIC_REGISTRATION_ENABLED, enabled);

      NodeList contactLists =
        staticReg.getElementsByTagName("terminatingContactList");

      if (contactLists.getLength() > 0)
      {
        Element contactList = (Element) contactLists.item(0);

        NodeList contacts = contactList.getElementsByTagName("contact");

        // need to find the terminating host/port:
        for (int i = 0; i < contacts.getLength(); i++)
        {
          Element contact = (Element) contacts.item(i);
          String contactStr = contact.getAttribute("value");

          // hopefully, there is only one of these that specifies a host/port
          // combination or this is not going to work
          if (contactStr.indexOf(":") != -1)
          {
            String host = contactStr.substring(0, contactStr.indexOf(":"));
            String port = contactStr.substring(contactStr.indexOf(":") + 1);

            userData.set(UserTableModel.TERMINATING_HOST, host);
            userData.set(UserTableModel.TERMINATING_PORT, port);
          }
        }
      }
    }

    // call return
    NodeList callReturns = user.getElementsByTagName("callReturn");

    if (callReturns.getLength() > 0)
    {
      Element callReturn = (Element) callReturns.item(0);
      String enabled = callReturn.getAttribute("enabled");

      userData.set(UserTableModel.CALL_RETURN_ADMIN_ENABLED, enabled);

      NodeList groups = callReturn.getElementsByTagName("featuregroup");

      if (groups.getLength() > 0)
      {
        Element group = (Element) groups.item(0);

        if (group.hasChildNodes())
        {
          String groupName = group.getFirstChild().getNodeValue();

          userData.set(UserTableModel.CALL_RETURN_GROUP, groupName);
        }
      }
    }

    // caller id blocking
    NodeList idBlockings = user.getElementsByTagName("callerIdBlocking");

    if (idBlockings.getLength() > 0)
    {
      Element idBlocking = (Element) idBlockings.item(0);
      String enabled = idBlocking.getAttribute("enabled");

      userData.set(UserTableModel.CALLER_ID_BLOCKING_ADMIN_ENABLED, enabled);

      NodeList groups = idBlocking.getElementsByTagName("featuregroup");

      if (groups.getLength() > 0)
      {
        Element group = (Element) groups.item(0);

        if (group.hasChildNodes())
        {
          String groupName = group.getFirstChild().getNodeValue();

          userData.set(UserTableModel.CALLER_ID_BLOCKING_GROUP, groupName);
        }
      }

      NodeList setFeats = idBlocking.getElementsByTagName("setfeat");

      if (setFeats.getLength() > 0)
      {
        Element setFeat = (Element) setFeats.item(0);

        if (setFeat.hasChildNodes())
        {
          String set = setFeat.getFirstChild().getNodeValue();

          userData.set(UserTableModel.CALLER_ID_BLOCKING_USER_SET, set);
        }
      }
    }

    return userData;
  }

  public void loadUserNames() throws VPPNoSuchFileException
  {
    String[] names = psInterface.getAllUserNames();

    data = new Vector(names.length);

    Logger.swingLog("Adding users ...");

    for (int i = 0; i < names.length; i++)
    {
      Vector user = new Vector();

      user.addElement(names[i]);
      data.addElement(user);

      if (i % 1000 == 0)
      {
        Logger.swingLog("Adding user #" + i + " - " + names[i]);
      }
    }

    Logger.swingLog("Done adding users");

    fireTableDataChanged();
  }

  public void deleteUserAt(int row)
    throws VPPNoSuchFileException, IOException, SAXException,
           NoSuchNodeException, NotTextNodeException, VPPException
  {
    Vector user = (Vector) data.elementAt(row);
    String userName = (String)user.elementAt(this.USER_NAME);

    if (user.size() <= 1)
    {

      // if the data for this user was not loaded, need to load it first to
      // find out what aliases are defined for this user so the alias xml files
      // may be deleted as well
      user = loadUserAt(row);
    }

    boolean deleted;

    if (user.elementAt(this.IS_ALIAS).equals("true"))
    {
      deleted = deleteAlias(user);
    }
    else
    {
      deleted = deleteUser(user);
    }

    // remove the user from the table
    if (deleted)
    {
      //some aliases may have already been removed from the data vector, which
      //may cause the position of this user to change -> need to find it again

      int index = this.findMasterIndex(userName);
      data.removeElementAt(index);
      this.fireTableDataChanged();
    }
  }

  private boolean deleteAlias(Vector alias)
    throws IOException, SAXException, NotTextNodeException,
           NoSuchNodeException, VPPNoSuchFileException, VPPException
  {

    // find the master user for this alias
    String masterIndexString = (String) alias.elementAt(this.MASTER_INDEX);
    int masterIndex;

    if (masterIndexString != null)
    {
      masterIndex = Integer.parseInt(masterIndexString);
    }
    else
    {
      masterIndex =
        findMasterIndex((String) alias.elementAt(this.MASTER_NAME));

      alias.setElementAt(masterIndex + "", this.MASTER_INDEX);
    }

    // This is a bug. If the master index ends up -1, it means that the master
    // user's vector could not be found. This situation should never arise if
    // if the data is consistent. But since it has come up, this bit of code
    // is put in specifically to allow the alias to be deleted even if the master
    // user cannot be found.
    if (masterIndex == -1)
    {
      if (JOptionPane.showConfirmDialog(null, "The master user for this alias was not found.\nYour user data is likely corrupted.\n\nDo you still want to delete this alias?", "ERROR", JOptionPane.ERROR_MESSAGE)
          == JOptionPane.NO_OPTION)
      {
        return false;
      }
    }
    else
    {

      // If the master user was found, modify it to remove the alias
      Vector masterUser = (Vector) data.elementAt(masterIndex);

      // modify the list of aliases for the master user to remove the alias being
      // deleted
      // if the user has not been loaded yet, need to do that to access the
      // aliases list
      if (masterUser.size() <= 1)
      {
        masterUser = loadUserAt(masterIndex);
      }

      // get the list of all aliases
      String aliasesString = (String) masterUser.elementAt(this.ALIASES);

      // remove the selected alias
      String newAliases =
        removeAliasFromList((String) alias.elementAt(this.USER_NAME),
              aliasesString);

      // save the new list of aliases
      masterUser.setElementAt(newAliases, this.ALIASES);

      // write back the the user file with the new alias list
      writeBackUserAt(masterIndex, false,
                      false);   // aliases should not affect cpl and cl
    }

    // always delete the alias from the server, even if the master was not found

    // delete the alias file from the server
    psInterface.deleteAliasFile((String) alias.elementAt(this.USER_NAME));

    // now update the vector of all user and all aliases stored in this table
    // to not include the deleted alias
    aliases.remove(alias);

    return true;
  }

  private boolean deleteUser(Vector user) throws VPPNoSuchFileException
  {

    // delete the user xml file from the server
    psInterface.deleteUser((String) user.elementAt(this.USER_NAME));

    // now delete all the alias files as well
    String aliasStr = (String) user.elementAt(this.ALIASES);
    StringTokenizer tokenizer = new StringTokenizer(aliasStr, ";");
    String alias;

    while (tokenizer.hasMoreTokens())
    {
      alias = tokenizer.nextToken();

      try
      {
        psInterface.deleteAliasFile(alias);
      }
      catch (VPPNoSuchFileException ex)
      {

        // This should never really happen. It means the data on the server is
        // corrupted or inconsistent. But just in case it happens ....
        ex.printStackTrace();
        JOptionPane.showMessageDialog(null,
                "The alias " + alias
                + " for this user file does not exist.\n\nYour user data is probably corrupted.",
                "ERROR", JOptionPane.ERROR_MESSAGE);
      }

      // Even if the alias file did not really exist on the server, for some
      // strange reason, it may still be in the vector of all aliases.
      // It should be removed in that case (and in the regular case).

      // delete the alias from the vector of all aliases
      for (int i = 0; i < aliases.size(); i++)
      {
        Vector aliasVector = (Vector) aliases.elementAt(i);

        if (aliasVector.elementAt(USER_NAME).equals(alias))
        {
          aliases.remove(aliasVector);

          if (data.contains(aliasVector))
          {
            data.remove(aliasVector);
          }

          break;
        }
      }
    }

    return true;
  }

  public boolean aliasExists(String alias)
  {
    try
    {
      psInterface.getAliasNamed(alias);
    }
    catch (VPPNoSuchFileException e)
    {
      return false;
    }

    return true;
  }

  public boolean userExists(String userName)
  {
    Document doc;

    try
    {
      doc = psInterface.getUserNamed(userName);
    }
    catch (VPPNoSuchFileException e)
    {
      return false;
    }
    catch (IOException e)
    {

      // the file could not be read but probably does not necessarily mean it
      // does not exist. Ummmm ... say it doesn't exist anyway
      return false;
    }
    catch (SAXException e)
    {

      // the file exists but could not be parsed. If its corrupted, say it
      // doesn't exist
      return false;
    }
    catch (VPPException ex)
    {
      return false;
    }

    if (doc == null)
    {
      return false;
    }

    return true;
  }


  public static String removeAliasFromList(String alias, String list)
  {
    int index = list.indexOf(alias);    // position where alias starts

    if (index != -1)
    {
      String newList;

      newList = list.substring(0, index)
                + list.substring(index + alias.length() + 1);
      list = newList;
    }

    return list;
  }


  private void removeAliasFromVectors(String alias, int row)
  {
    String userName = (String) getValueAt(row, USER_NAME);

    // could not save this alias because it is not uniqe
    // remove it from the list of aliases in the vector for this user
    // since it is not a valid alias
    String aliasList = removeAliasFromList(alias,
            (String) getValueAt(row, this.ALIASES));

    ((Vector) data.elementAt(row)).setElementAt(aliasList, this.ALIASES);

    for (int i = 0; i < aliases.size(); i++)
    {
      Vector aliasVector = (Vector) aliases.elementAt(i);
      String name = (String) aliasVector.elementAt(this.USER_NAME);
      String masterName = (String) aliasVector.elementAt(this.MASTER_NAME);

      if (name.equals(alias) && masterName.equals(userName))
      {
        aliases.remove(aliasVector);

        if (data.contains(aliasVector))
        {
          data.remove(aliasVector);
        }
      }
    }

    fireTableDataChanged();

    // complain
    JOptionPane.showMessageDialog(null,
            "Could not create alias " + alias + " for user " + userName
            + " because the alias is not unique.");
  }

  public void writeBackUserAt(int row, boolean generateCpl,
          boolean generateCl) throws NoSuchNodeException,
          NotTextNodeException, SAXException, IOException
  {
    Exception error = null;

    // If the user data loaded from the server did not contain an entry for the
    // caller id blocking group, this contacts the provisioning server,
    // determines the list of all known caller id blocking groups and
    // defaults the value for this user to the first group on the list.
    try
    {
      if (getValueAt(row,
                     this.CALLER_ID_BLOCKING_GROUP).equals("not loaded from xml"))
      {
        String[] groups = psInterface.getFeatureGroups("CallerIdBlocking");

        if (groups != null)
        {
          if (groups.length > 0)
          {
            setValueAt(groups[0], row, CALLER_ID_BLOCKING_GROUP);
          }
        }
      }
    }
    catch (Exception e)
    {

      // quietly suppress this exception
      e.printStackTrace();
    }

    String userName = (String) getValueAt(row, USER_NAME);

    // generate the alias files in the Aliases directory
    String aliasesString = (String) getValueAt(row, ALIASES);

    while (aliasesString.length() > 0)
    {
      String alias = aliasesString.substring(0, aliasesString.indexOf(";"));

      aliasesString = aliasesString.substring(aliasesString.indexOf(";") + 1);

      if (alias.equals(userName))
      {
        removeAliasFromVectors(alias, row);
      }
      else if (!psInterface.saveAlias(alias,
              userName))        // try saving the alias file
      {
        removeAliasFromVectors(alias, row);
      }                         // end if the alias was not unique
    }                           // end for all aliases


    Document doc = convertUserAtToDocument(row);


    if (generateCpl)
    {
      CplGenerator cplGen = new CplGenerator(psInterface.getConnection());
      String cpl = "";

      if (getValueAt(row, this.FORWARD_ALL_USER_SET).equals("ON"))
      {
        System.out.println("---TRACE---cfa set by user");
        System.out.println("---TRACE---Building cpl for cfa with destination="
                           + getValueAt(row, FORWARD_ALL_DESTINATION));

        try
        {
          cpl = cplGen.buildCfaXML((String) getValueAt(row,
                  FORWARD_ALL_DESTINATION));

          writeCpl((String) getValueAt(row, FORWARD_ALL_GROUP),
                   (String) getValueAt(row, USER_NAME), cpl);
        }
        catch (Exception ex)
        {

          // catch the exception, save it and throw it only once an attempt
          // has been made to generate each of the cpl scripts
          error = ex;
        }
      }


      // call return
      System.out.println("---TRACE--- generating call return cpl");

      // the setfeat sould have been set to on in the convertUserToDocument function
      if (getValueAt(row, this.CALL_RETURN_ADMIN_ENABLED).equals("true"))
      {
        cpl = cplGen.buildCallReturnUserXML();
      }
      else
      {

        // write cpl even if feature is set off
        cpl = cplGen.buildFeatureDisabledXML();
      }

      writeCpl((String) getValueAt(row, this.CALL_RETURN_GROUP),
               (String) getValueAt(row, this.USER_NAME), cpl);


      // caller id blocking
      System.out.println("---TRACE--- generating caller id blocking cpl");

      if (getValueAt(row, this.CALLER_ID_BLOCKING_USER_SET).equals("ON"))
      {

        // only write cpl if feature is set on
        cpl = cplGen.buildCallerIdBlockingXML();

        writeCpl((String) getValueAt(row, this.CALLER_ID_BLOCKING_GROUP),
                 (String) getValueAt(row, this.USER_NAME), cpl);
      }


      // generate call blocking cpl
      boolean block900 = false;
      boolean blockLongDist = false;

      if (getValueAt(row, BLOCK_900_ADMIN_ENABLED).equals("true"))
      {
        System.out.println("---TRACE---900 calls blocked by admin");

        block900 = true;
      }
      else
      {
        if (getValueAt(row, BLOCK_900_USER_SET).equals("true"))
        {
          System.out.println("---TRACE---900 calls blocked by user");

          block900 = true;
        }
      }


      if (getValueAt(row, BLOCK_LONG_DISTANCE_ADMIN_ENABLED).equals("true"))
      {
        System.out.println("---TRACE---long distance calls blocked by admin");

        blockLongDist = true;
      }
      else
      {
        if (getValueAt(row, BLOCK_LONG_DISTANCE_USER_SET).equals("true"))
        {
          System.out.println("---TRACE---long distance calls blocked by user");

          blockLongDist = true;
        }
      }


      if (block900 || blockLongDist)
      {
        System.out.println("Building cpl for clbl with block900=" + block900
                           + " and blockLongDist=" + blockLongDist);

        try
        {
          cpl = cplGen.buildClblXML(block900, blockLongDist, (String)getValueAt(row, UserTableModel.CALL_BLOCK_PREFIX));

          writeCpl((String) getValueAt(row, CALL_BLOCK_GROUP),
                   (String) getValueAt(row, USER_NAME), cpl);
        }
        catch (Exception ex)
        {
          error = ex;
        }
      }


      // generate call screening cpl
      // The setfeat field for this feature is not actually in the vector used
      // to display the table. So we need to get it from the dom which was
      // created

      NodeList css = dom.getElementsByTagName("cs");

      if (css.getLength() > 0)
      {
        Element cs = (Element) css.item(0);
        NodeList setfeats = cs.getElementsByTagName("setfeat");

        if (setfeats.getLength() > 0)
        {
          Element setfeat = (Element) setfeats.item(0);

          if (setfeat.hasChildNodes())
          {
            if (setfeat.getFirstChild().getNodeValue().equals("ON"))
            {
              System.out.println("Building cpl for cs");

              try
              {
                cpl = cplGen.buildCsXML(convertUserAtToDocument(row));

                writeCpl((String) getValueAt(row, CALL_SCREEN_GROUP),
                         (String) getValueAt(row, USER_NAME), cpl);
              }
              catch (Exception ex)
              {
                error = ex;
              }
            }
          }
        }
      }

      boolean fna = false;

      if (getValueAt(row, this.FORWARD_UNANSWERED_USER_SET).equals("true"))
      {
        System.out.println("---TRACE---forward no answer set by user");

        fna = true;
      }

      boolean fb = false;

      if (getValueAt(row, this.FORWARD_BUSY_USER_SET).equals("true"))
      {
        System.out.println("---TRACE---forward busy set by user");

        fb = true;
      }

      if (fna || fb)
      {

        // write forward no answer, blocking cpl
        String fnaDestination = (String) getValueAt(row,
                this.FORWARD_UNANSWERED_DESTINATION);
        String fbDestination = (String) getValueAt(row,
                this.FORWARD_BUSY_DESTINATION);
        String failureCause = (String) getValueAt(row, this.FAILURE_CASE);

        System.out.println("Building cpl for fnab with username=" + userName
                           + ", fna=" + fna + ", fnaDestination="
                           + fnaDestination + ", fb=" + fb
                           + ", fbDestination= " + fbDestination
                           + "failureCause=" + failureCause);

        try
        {
          cpl = cplGen.buildFnabXML(userName, fna, fnaDestination, fb,
                  fbDestination, failureCause);

          writeCpl((String) getValueAt(row, this.FORWARD_BUSY_GROUP),
                   (String) getValueAt(row, USER_NAME), cpl);
        }
        catch (Exception ex)
        {
          error = ex;
        }
      }
    }   // end if generate cpl

    if (generateCl)
    {
      ContactListGenerator clGen = new ContactListGenerator();
      String called = clGen.buildCalledContactList(doc);
      String calling = clGen.buildCallingContactList(doc);

      psInterface.put("Contact_Lists_Called",
                      (String) getValueAt(row, this.USER_NAME), called);
      psInterface.put("Contact_Lists_Calling",
                      (String) getValueAt(row, this.USER_NAME), calling);
    }

    // write back the user xml
    psInterface.saveUser(userName, doc);

    if (error != null)
    {
      if (NoSuchNodeException.class.isInstance(error))
      {
        throw (NoSuchNodeException) error;
      }
      else if (NotTextNodeException.class.isInstance(error))
      {
        throw (NotTextNodeException) error;
      }
      else if (SAXException.class.isInstance(error))
      {
        throw (SAXException) error;
      }
      else if (IOException.class.isInstance(error))
      {
        throw (IOException) error;
      }
    }
  }

  private void writeCpl(String group, String userName, String cpl)
  {
    String[] paths = cplGen.getAddressArray(group);

    if (paths == null)
    {
      System.out.println("---ERROR--- Could not get cpl destination paths for group "
                         + group
                         + "\n---ERROR--- Probably, the group does not exist in the ListOfFeatureServers"
                         + "\n---ERROR--- Or there is no server configured for this feature.");

      return;
    }

    String filename = userName + ".cpl";
    String filegroup;

    for (int i = 1; i < paths.length; i++)
    {
      filegroup = paths[i];

      psInterface.put(filegroup, filename, cpl);
    }
  }

  public Vector loadUserAt(int row)
    throws SAXException, IOException, VPPException
  {
    Vector userData = (Vector) data.elementAt(row);

    // if there is one element in the data vector for the selected user, this
    // means that only the user's name has been loaded, not his xml file
    if (userData.size() <= 1)
    {

      // find the name of the user in the selected row
      String userName = (String) userData.elementAt(UserTableModel.USER_NAME);

      // load the user's xml file fron the pserver
      Document doc = psInterface.getUserNamed(userName);

      // convert the xml to a vector
      userData = convertUserToVector(doc);

      // save the vector in the appropriate row
      data.set(row, userData);
      fireTableDataChanged();
    }

    return userData;
  }

  /**
   * Return a new vector which contains the "default" configuration for a
   * user. <p> This is used when adding a new user to the system
   */
  public Vector getDummyUser()
  {
    return (Vector) dummyUser.clone();
  }


  public Vector getUserAt(int row)
  {
    return (Vector) data.elementAt(row);
  }


  private Vector createAliasVector(String name, String masterName)
  {
    Vector alias = new Vector(this.MAX_COLUMN_ID);

    for (int j = 0; j <= this.MAX_COLUMN_ID; j++)
    {
      alias.addElement("");
    }

    alias.set(this.USER_NAME, name);
    alias.set(this.IS_ALIAS, "true");
    alias.set(this.MASTER_NAME, masterName);
    alias.set(this.MASTER_INDEX, null);

    return alias;
  }

  public void showAliases(boolean show)
  {
    isShowingAliases = show;

    if (show)
    {
      if (aliasesLoadedFromServer)
      {

        // if the aliases have been loaded from the server
        // add them to the main data vector
        for (int i = 0; i < aliases.size(); i++)
        {
          data.addElement(aliases.elementAt(i));
        }

        this.fireTableDataChanged();
      }
      else
      {

        // if the aliases have not been loaded from the server
        String[] aliasNames;

        try
        {
          aliasNames = psInterface.getAllAliasNames();
        }
        catch (VPPNoSuchFileException e)
        {

          // there are no aliases defined - that's ok
          return;
        }

        for (int i = 0; i < aliasNames.length; i++)
        {
          try
          {

            // get the file for this alias
            String aliasXml = psInterface.getAliasNamed(aliasNames[i]);

            // get the master user name this alias belongs to
            String masterName =
              psInterface.getMasterUserNameFromXml(aliasXml);

            // create a new vector for this alias
            Vector alias = createAliasVector(aliasNames[i], masterName);

            // add the vector to the list of aliases and to the main data vector

            // If a new user was created along with some aliases, before others
            // were loaded from the server, these may have already been added to
            // the aliases vector
            boolean found = false;

            for (int j = 0; j < aliases.size(); j++)
            {
              Vector aliasVector = (Vector) aliases.elementAt(j);
              String aliasName =
                (String) aliasVector.elementAt(this.USER_NAME);

              if (aliasName.equals(aliasNames[i]))
              {
                found = true;

                break;
              }
            }

            if (!found)
            {
              aliases.addElement(alias);
            }

            data.addElement(alias);
          }
          catch (VPPNoSuchFileException e)
          {
            e.printStackTrace();

            // this shouldn't have happenend. It was just found out above that
            // a file with this name exists ... so we'll just quietly suppress
            // this expception
          }

          this.fireTableDataChanged();

          aliasesLoadedFromServer = true;
        }
      }

      sortByUserName();
    }
    else
    {

      // if the aliases have been loaded, remove them from the main data list
      if (aliases.size() != 0)
      {
        for (int i = 0; i < aliases.size(); i++)
        {
          Object alias = aliases.elementAt(i);

          data.remove(alias);
        }

        fireTableDataChanged();
      }
    }
  }

  public int findMasterIndex(String masterName)
  {
    for (int i = 0; i < data.size(); i++)
    {
      String name = (String) getValueAt(i, this.USER_NAME);

      if (name.equals(masterName))
      {
        return i;
      }
    }

    return -1;  // let's hope it doesn't get here
  }

  /**
   * Returns the array which stores the information about column width and title
   */
  public ColumnInfo[] getColumns()
  {
    return columns;
  }

  public String getColumnName(int column)
  {
    return columns[column].getName();
  }


  /**
   * Performs a binary search and relies on the fact that the list of users
   * in the table will be sorted from lowest to highest.
   */
  public int findRowStartingWithName(String substring)
  {
    int low = 0;
    int high = data.size() - 1;         // index, not count

    int mid = 0;

    while (low <= high)
    {
      mid = (low + high) / 2;

      Vector user = (Vector) data.elementAt(mid);
      String userName = (String) user.elementAt(this.USER_NAME);

      if (substring.compareTo(userName) > 0)
      {
        low = mid + 1;
      }
      else if (substring.compareTo(userName) < 0)
      {

        // because, for example, 4 < 4000. Otherwise, the closest match ends up
        // being 3999
        if (userName.startsWith(substring))
        {
          if (mid > 0)
          {
            Vector previousUser = (Vector) data.elementAt(mid - 1);
            String previousName =
              (String) previousUser.elementAt(this.USER_NAME);

            if (substring.compareTo(previousName) > 0)
            {
              return mid;
            }
          }
        }

        high = mid - 1;
      }
      else
      {
        return mid;
      }
    }

    return mid;
  }


  public void sortByUserName()
  {
    java.util.Collections.sort(data, new NameComparator());
  }

  private class NameComparator implements java.util.Comparator
  {
    public int compare(Object object1, Object object2)
    {
      String name1 =
        (String) ((Vector) object1).elementAt(UserTableModel.USER_NAME);
      String name2 =
        (String) ((Vector) object2).elementAt(UserTableModel.USER_NAME);

      return name1.compareTo(name2);
    }

  }

  public void setValueAt(String value, int row,
          int column) throws VPPException, IOException, SAXException
  {
    Vector user = (Vector) data.elementAt(row);

    // if the data being set is the list of aliases, need to do some extra work:
    // need to determine whether any aliases for this user were created or
    // removed so that the overall list of aliases in the table is also
    // appropriately updated
    if (column == this.ALIASES)
    {
      Vector newAliases = new Vector();
      Vector oldAliases = new Vector();

      String currentAliases = (String) user.elementAt(this.ALIASES);
      StringTokenizer tokenizer = new StringTokenizer(currentAliases, ";");

      while (tokenizer.hasMoreTokens())
      {
        oldAliases.addElement(tokenizer.nextToken());
      }

      tokenizer = new StringTokenizer(value, ";");

      while (tokenizer.hasMoreElements())
      {
        newAliases.addElement(tokenizer.nextToken());
      }

      // find out if any aliases have been removed (old aliases contain an element the new ones don't)
      for (int j = 0; j < oldAliases.size(); j++)
      {
        String alias = (String) oldAliases.elementAt(j);

        if (!newAliases.contains(alias))        // then the alias has been removed
        {

          // find the vector for this alias
          for (int i = 0; i < aliases.size(); i++)
          {
            Vector aliasVector = (Vector) aliases.elementAt(i);
            String aliasName = (String) aliasVector.elementAt(this.USER_NAME);

            if (aliasName.equals(alias))        // if this is the alias that has been deleted
            {

              // remove it from the list of aliases and the master list
              aliases.remove(aliasVector);

              if (data.contains(aliasVector))
              {
                data.remove(aliasVector);
              }
            }                                   // end if names match
          }                                     // end for all alias vectors
        }                                       // end if alias has been removed
      }                                         // end for all old aliases


      // now see if there have been any aliases added -> are in the new list but not the old one
      for (int i = 0; i < newAliases.size(); i++)
      {
        String alias = (String) newAliases.elementAt(i);

        if (!oldAliases.contains(alias))        // the alias has been added
        {
          Vector newAlias = this.createAliasVector(alias,
                  (String) user.elementAt(this.USER_NAME));

          aliases.add(newAlias);

          if (isShowingAliases)
          {
            data.addElement(newAlias);
          }
        }
      }
    }

    // Load the data for this user. If the data is being set, it has changed
    // and it is assumed it will be written back. Since the entire data set has
    // to be written at once, it must be completely loaded before it is written
    // back. It is loaded here, before the new values are set, so that they are
    // not overwritten later by data from the server
    if (user.size() <= 1)       // if data for this user has not been loaded yet
    {
      user = loadUserAt(row);
    }

    user.set(column, value);

    // can't sort ailases into table here because the user data may not have
    // been completely saved yet and sorting may change the row which the user
    // being saved is at which will produce the wrong result.
  }

  public Object getValueAt(int row, int column)
  {
    Vector user = (Vector) data.elementAt(row);

    if (column == USER_NAME)    // name should always be available
    {
      return (String) user.elementAt(USER_NAME);
    }

    // if the user's data vector contains only one element, the xml has not been
    // loaded yet
    if (user.size() <= 1)
    {
      return "";
    }

    String value = (String) user.elementAt(column);

    return value;
  }

  public Class getColumnClass(int columnIndex)
  {
    if (Boolean.class.isInstance(getValueAt(0, columnIndex)))
    {
      return Boolean.class;
    }

    return String.class;
  }

  public int getRowCount()
  {
    return data.size();
  }

  public int getColumnCount()
  {
    return columns.length;
  }

}

