/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.userEditor;


import vocal.comm.VPPTransactionWrapper;
import vocal.comm.VPPException;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.*;
import java.awt.*;

import java.applet.Applet;

public class Usred extends Applet
{
  private String hostName;
  private int portNumber;
  private static Editor theEditor;

  public Usred() {}

  // Get a parameter value
  public String getParameter(String key, String def)
  {
    return (getParameter(key) != null ? getParameter(key) : def);
  }


  // Initialize the applet
  public void init()
  {
    try
    {
      hostName = this.getParameter("host", "localhost");
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

    try
    {
      portNumber = Integer.parseInt(this.getParameter("port", "6005"));
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

    createEditor(hostName, portNumber);
    theEditor.setSize(700, 500);
    theEditor.validate();

    JFrame frame = new JFrame("User Editor");

    frame.setSize(700, 500);
    frame.getContentPane().add(theEditor);
    frame.setVisible(true);
  }

  private static void createEditor(String host, int port)
  {
    System.out.println("Connecting to: " + host + ":" + port);

    try
    {
      VPPTransactionWrapper conn = new VPPTransactionWrapper(host, port);

      theEditor = new Editor(conn);

      theEditor.init();
    }
    catch (VPPException e)
    {
      VPPTransactionWrapper.showVPPException(e, 
              "Failed to connect to " + host + ":" + Integer.toString(port));

      return;
    }
  }

  public static void main(String[] args)
  {
    final JFrame frame = new JFrame("Test");

    if (args.length < 2)
    {
      System.out.println("Usage: Generate host port");

      return;
    }

    createEditor(args[0], Integer.parseInt(args[1]));   // host,port

    frame.setSize(700, 500);
    frame.getContentPane().add(theEditor);

    // event handler which exits the vm when the user exits the application
    frame.addWindowListener(new WindowAdapter()
    {
      public void windowClosing(WindowEvent e)
      {
        frame.setVisible(false);
        System.exit(0);
      }

    });

    frame.setVisible(true);
  }

}

