/*
 * ====================================================================
 * The Vovida Software License, Version 1.0
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 * and "Vovida Open Communication Application Library (VOCAL)" must
 * not be used to endorse or promote products derived from this
 * software without prior written permission. For written
 * permission, please contact vocal@vovida.org.
 * 
 * 4. Products derived from this software may not be called "VOCAL", nor
 * may "VOCAL" appear in their name, without prior written
 * permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 * 
 */

package vocal.util;


import javax.swing.SwingUtilities;
import java.lang.reflect.InvocationTargetException;

/**
 * A task which runs in its own thread but has begin and finish methods which
 * are executed in the swing event dispatching thread.
 */
public abstract class Task implements Runnable
{
  private InvocationTargetException exception = null;

  /**
   * Sub classes can override this method if they want any code to be executed
   * in the Swing event processing thread before doWork is called. <p>
   * Implementation is empty.
   */
  public void begin()
  {

  }

  /**
   * Sub clsses can override this method if they want any code the be executed
   * in the Swing event processing thread after doWrok has started. <p>
   * Implementaition is empty.
   */
  public void finish()
  {

  }

  public abstract void doWork();

  public void run()
  {
    Runnable begin = new Runnable()
    {
      public void run()
      {
        begin();
      }

    };

    // does not necessarily mean that this will happen before the doWork is
    // called. In fact, most likely it won't. Still, if you want to put a message
    // up on the screen, it could be useful.
    // Uncomment this to allow any code written in begin to be executed.
    // SwingUtilities.invokeLater(begin);


    try
    {
      doWork();
    }
    catch (Throwable e)
    {
      saveException(e);
    }

    Runnable finish = new Runnable()
    {
      public void run()
      {
        finish();
      }

    };

    // Uncomment this to allow any code written in begin to be executed.
    // SwingUtilities.invokeLater(finish);
  }

  public void start()
  {
    new Thread(this, "Task thread " + this.getClass().getName()).start();
  }

  private void saveException(Throwable e)
  {
    exception = new InvocationTargetException(e, "Task failed");

    e.printStackTrace();
  }

  public InvocationTargetException getException()
  {
    return exception;
  }

}

