#ifndef HostInterface_hxx
#define HostInterface_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const HostInterface_hxx_Version =
    "$Id: HostInterface.hxx,v 1.9 2001/01/26 23:05:44 dabryan Exp $";

#include <list>
#include <map>
#include <string>
#include "Sptr.hxx"
#include "PSReader.hxx"
#include "PSWriter.hxx"
#include "NetworkAddress.h"
#include "Data.hxx"

//User defined

class VNetworkException;
class VMissingDataException;
class VBadDataException;
class NetworkAddress;

class HostInterface
{

    public:

        /// Constructor
        HostInterface(const char* host, const int port);

        /// Destructor
        ~HostInterface()
        {}

        /// Reads an item from provision server

        string readItem(const string &group, const string &name)
        throw (VException&);

        /// Reads a list of items in given group from provision server
        void getList(const string& group, vector < Data > & retList)
        throw (VNetworkException &);

        /// Write a data item to provision server
        void writeItem(const string &group, const string &name,
                       const string& contents)
        throw (VNetworkException &);

        /// Delete a subscriber from the server
        void deleteSubscriber(const string& subscriberName)
        throw (VNetworkException &);

    ///
        const NetworkAddress& getNetAddress()
        {
            return (*_networkAddress);
        }

        /// used to let PSInterface class keep track of this guy
        bool isAlive()
        {
            return _isAlive;
        };

    ///
        void setAlive(bool val)
        {
            _isAlive = val;
        };

        /// used to let PSInterface class keep track of this guy
        bool inUse()
        {
            return _inUse;
        };

    ///
    void setUse(bool val)
        {
            _inUse = val;
        };

        /// register and unregister for items and entire groups
        void registerForItem(const string &group, const string &name,
                             const string &hostName) throw (VNetworkException &);

        /// register to receive updates when the members of a group change
        void registerForGroup(const string &group, const string &hostName)
        throw (VNetworkException &);

        /// unregister for an item
        void unregisterForItem(const string &group, const string &name,
                               const string &hostName) throw (VNetworkException &);

        /// unregister for a group
        void unregisterForGroup(const string &group, const string &hostName)
        throw (VNetworkException &);


    private:


        /// bool to tell if the server is alive 
        bool _isAlive;

    /// bool to tell if this server is in use (active)
    bool _inUse;

        /// given a message, parses it and returns the XML
        string getXmlContents(const string& dRead) throw (VException &);

        /// Reader object to serve read request
        Sptr <PSReader> _reader;

        /// Write object to serve write requests
        Sptr <PSWriter> _writer;

        /// Network address of the Provision Server
        Sptr <NetworkAddress> _networkAddress;


};

#endif
