/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const PSListener_cxx_Version =
    "$Id: PSListener.cxx,v 1.11 2001/09/26 21:39:53 bko Exp $";

#include "PSListener.hxx"

#include <sys/time.h>
#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "Sptr.hxx"
#include "Tcp_ServerSocket.hxx"
#include "Connection.hxx"
#include "cpLog.h"
#include "VNetworkException.hxx"
#include "Vpp_def.h"
#include "VEnvVar.hxx"
#include "ProvisionInterface.hxx"


void*
PSListener::runThr(void* args)
{

    PSListener* self = (PSListener*)args;
    self->run();
    return NULL;

}


void
PSListener::run()
{

    bool socketSuccessful = false;
    Sptr <TcpServerSocket> serverSocket;

    while ( ! socketSuccessful )
    {
        socketSuccessful = true;
        try
        {
            serverSocket = new TcpServerSocket(_listenerPort);
            _init = true;
        }
        catch ( VNetworkException )
        {
            // could not listen here, so retry
            sleep(1);
            _listenerPort++;
            socketSuccessful = false;  // redo

            cpLog(LOG_DEBUG, "Trying port %d", _listenerPort);
        }
    }

    if (socketSuccessful)
    {
        cpLog(LOG_ALERT, "Successfully opened listner port %d", _listenerPort);
    }
    else
    {
	// this never gets called, but i'm putting it in for future use
	cpLog(LOG_ALERT, "Could not open a listener port");
    }

    //Loop through for Incoming messages
    for (; ; )
    {
        Connection conn;
        serverSocket->accept(conn);
        processUpdateRequest(conn);
        conn.close();
        if (_exitFlg) break;
    }

}


PSListener::~PSListener()
{
    _exitFlg = true;
}


PSListener::PSListener()
        : _exitFlg(false),
        _init(false)
{
    _listenerPort = VEnvVar::VLISTENER_PORT;
    pthread_create( &_ptid, 0, PSListener::runThr, (void*)this);
}


PSListener::PSListener(int port)
        : _exitFlg(false),
        _listenerPort(port),
        _init(false)
{
    pthread_create( &_ptid, 0, PSListener::runThr, (void*)this);
}


void
PSListener::processUpdateRequest(Connection& conn)
{

    char buf[MAXLINE];
    string dRead;
    buf[0] = '\0';
    int nread = 0;

    while (conn.readLine(buf, sizeof(buf), nread) > 0)
    {
        buf[nread] = 0;
        dRead = buf;
        char req[VPPREQ_BUF_LEN], group[URL_LENGTH], name[URL_LENGTH],
        rest[MAXLINE];
        sscanf(buf, "%s%s%s%s", req, group, name, rest);
        string vppReq;
        vppReq = req;

        cpLog(LOG_DEBUG, "Processing update request: (%s)",
              buf);

        if (vppReq == string(UPDATEITEM_REQ))
        {
            ProvisionInterface::instance().
            sendUpdateNotification(group, name);
        }

        else if (vppReq == string(DELITEM_REQ))
        {
            ProvisionInterface::instance().
            sendDeleteNotification(group, name);
        }

        else if (vppReq == string(UPDATEGROUP_REQ))
        {
            ProvisionInterface::instance().
            sendUpdateNotification(group, name, false);
        }

        else if (vppReq == string(DELGROUP_REQ))
        {
            ProvisionInterface::instance().
            sendUpdateNotification(group, name, true);
        }

        buf[0] = '\0';
        nread = 0;
        dRead = "";

    }

}


int
PSListener::getListenerPort() const
{

    while (!_init)
    {
        cpLog(LOG_INFO, "PSLIB: PSListener is not initialized waiting..");
        sleep(1);
    }

    return _listenerPort;

}
