#ifndef  VFEATURESERVERDATA_HXX
#define  VFEATURESERVERDATA_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const VFeatureServerData_hxx_Version =
    "$Id: VFeatureServerData.hxx,v 1.13 2001/01/25 22:52:21 dabryan Exp $";

#include <stdio.h>
#include <string>
#include <vector>

#include "VXmlParser.hxx"
#include "VSuavmServerData.hxx"
#include "VException.hxx"


/// Feature server data container
class VFeatureServerData
{
    public:

    ///
        VFeatureServerData()
        { }
        ;


    /// Constructor from XML input stream
        VFeatureServerData(const string& xmlData) throw (VException&)
        {
            parse(xmlData);
        }

    /// Copy constructor
        VFeatureServerData(const VFeatureServerData &copy)
        {
            deepCopy(copy);
        }

    /// assignment operator
        VFeatureServerData& operator=(const VFeatureServerData& other)
        {
            if (this != &other)
            {
                deepCopy(other);
            }
            return (*this);
        }

    /// parse the input data
        void parse(const string& xmlData) throw (VException&)
        {

            _firstPort = -1;
            _lastPort = -1;
            _vmHost = "none";

            VXmlParser parser(xmlData);

            xmlNode *theNode = parser.findNode(NULL, "featureServer");
            _host = parser.getAttribute(theNode, "host");
            _port = atoi(parser.getAttribute(theNode, "port").c_str());
            _name = _host + ":" + (parser.getAttribute(theNode, "port"));

            theNode = parser.findNode(NULL, "type");
            _type = parser.getAttribute(theNode, "value");

            theNode = parser.findNode(NULL, "group");
            _group = parser.getAttribute(theNode, "value");

            theNode = parser.findNode(NULL, "uaVMServers");
            if (theNode)
            {
                _firstPort = atoi(parser.getAttribute(theNode,
                                                      "firstPort").c_str());
                _lastPort = atoi(parser.getAttribute(theNode,
                                                     "lastPort").c_str());
                _vmHost = parser.getAttribute(theNode, "host");
            }

        }

    ///
        friend ostream& operator<<(ostream& os, const VFeatureServerData& elem);

    ///
        const string& getName() const
        {
            return _name;
        };

    ///
        const string& getHost() const
        {
            return _host;
        };

    ///
        int getPort() const
        {
            return _port;
        };

    ///
        const string& getType() const
        {
            return _type;
        };

    ///
        const string& getGroup() const
        {
            return _group;
        };

    ///
        void getVMList(vector < VSuavmServerData > &retVal) const
        {
            retVal.clear();
            VSuavmServerData item(_vmHost, _firstPort, _lastPort);
            retVal.push_back(item);
            return ;
        }

    private:

/// copy operation to copy all elements (used by copy constructor and assign)
    void deepCopy(const VFeatureServerData &copy)
        {

            _name = copy._name;
            _host = copy._host;
            _port = copy._port;
            _type = copy._type;
            _group = copy._group;
            _firstPort = copy._firstPort;
            _vmHost = copy._vmHost;

        };

    string _name;
        string _host;
        int _port;
        string _type;
        string _group;
        int _firstPort;
        int _lastPort;
        string _vmHost;


};

inline ostream& operator<<(ostream& os, const VFeatureServerData& elem)
{

    os << "Name:              " << elem._name << endl;
    os << "Host:              " << elem._host << endl;
    os << "Port:              " << elem._port << endl;
    os << "Server type:       " << elem._type << endl;
    os << "Server group:      " << elem._group << endl;
    if (elem._firstPort != -1)
    {
        os << "VM First port:     " << elem._firstPort << endl;
        os << "VM Last port:      " << elem._lastPort << endl;
        os << "VM Host:           " << elem._vmHost << endl;
    }
    return os;

}

#endif
