#ifndef  VLOCATIONROUTEDATA_HXX
#define  VLOCATIONROUTEDATA_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const VLocationRouteData_hxx_Version =
    "$Id: VLocationRouteData.hxx,v 1.1 2001/04/10 00:38:25 kevinmc Exp $";

#include <stdio.h>
#include <string>

#include "VRouteElement.hxx"
#include "VXmlParser.hxx"
#include "VException.hxx"


/// Location server data container class
class VLocationRouteData
{

    public:

    /// 
        VLocationRouteData()
        { }
        ;

    /// constructor from XML data string
        VLocationRouteData(const string& xmlData) throw (VException&)
        {
            parse(xmlData);
        }


    /// Copy constructor
        VLocationRouteData(const VLocationRouteData &copy)
        {

            deepCopy(copy);

        }

    /// assignment operator
        VLocationRouteData& operator=(const VLocationRouteData& other)
        {

            if (this != &other)
            {
                deepCopy(other);
            }

            return (*this);

        }

    /// parse the input XML stream
        void parse(const string& xmlData) throw (VException&)
        {

            _routes.clear();

            VXmlParser parser(xmlData);

            xmlNode *theNode = parser.findNode(NULL, "locationRoutes");
            _host = parser.getAttribute(theNode, "host");
            _port = atoi(parser.getAttribute(theNode, "port").c_str());

            theNode = parser.findNode(NULL, "routeList");
            if (theNode)
            {

                list < xmlNodePtr > listOfNodes;
                parser.findNode(theNode, "route", listOfNodes);

                for (list < xmlNodePtr > ::iterator itr =
                            listOfNodes.begin();
                        itr != listOfNodes.end(); itr++)
                {
                    xmlNodePtr itemNode = (*itr);
                    // get addr attribute from node
                    string addr = parser.getAttribute(itemNode, "addr");
                    string proto = parser.getAttribute(itemNode, "proto");
                    string prefix = parser.getAttribute(itemNode, "prefix");
                    string next = parser.getAttribute(itemNode, "next");
                    VRouteElement routeElem(addr, proto, prefix, next);
                    _routes.push_back(routeElem);
                }

            }
        }

    ///
    friend ostream& operator<<(ostream &os, const VLocationRouteData &elem);

///
    const string& getHost() const
        {
            return _host;
        };

///
    int getPort() const
        {
            return _port;
        };

    ///
    int getRouteListSize() const
        {
            return _routes.size();
        };

    ///
    void getRouteList(vector < VRouteElement > &retVal) const
        {
            retVal.clear();
            for (vector < VRouteElement > ::const_iterator itr = _routes.begin();
                    itr != _routes.end(); itr++)
            {
                retVal.push_back(*itr);
            }
        };

    private:

   /// copy operation to copy all elements (used by copy constructor and assign)
    void deepCopy(const VLocationRouteData &copy)
        {
            _host = copy._host;
            _port = copy._port;
            _routes = copy._routes;

        };


    
        string _host;
        int _port;
        vector < VRouteElement > _routes;

};

///
inline ostream& operator<<(ostream &os, const VLocationRouteData &elem)
{

    os << "Host             : " << elem._host << endl;
    os << "Port             : " << elem._port << endl;
    if (elem.getRouteListSize())
    {
        os << "Route List: " << endl;
        vector < VRouteElement > routeList;
        elem.getRouteList(routeList);
        for (vector < VRouteElement > ::const_iterator itr = routeList.begin();
                itr != routeList.end(); itr++)
        {
            os << *itr << endl;
        }
    }

    return os;
}

#endif
