#ifndef  VLOCATIONSERVERDATA_HXX
#define  VLOCATIONSERVERDATA_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const VLocationServerData_hxx_Version =
    "$Id: VLocationServerData.hxx,v 1.1 2001/04/10 00:38:25 kevinmc Exp $";

#include <stdio.h>
#include <string>

#include "VXmlParser.hxx"
#include "VException.hxx"


/// Location server data container class
class VLocationServerData
{

    public:

    /// 
        VLocationServerData()
        { }
        ;

    /// constructor from XML data string
        VLocationServerData(const string& xmlData) throw (VException&)
        {
            parse(xmlData);
        }


    /// Copy constructor
        VLocationServerData(const VLocationServerData &copy)
        {

            deepCopy(copy);

        }

    /// assignment operator
        VLocationServerData& operator=(const VLocationServerData& other)
        {

            if (this != &other)
            {
                deepCopy(other);
            }

            return (*this);

        }

    /// parse the input XML stream
        void parse(const string& xmlData) throw (VException&)
        {

            _protocols.clear();
            _addressFamilies.clear();
            _lsPeers.clear();

            VXmlParser parser(xmlData);

            xmlNode *theNode = parser.findNode(NULL, "locationServer");
            _host = parser.getAttribute(theNode, "host");
            _port = atoi(parser.getAttribute(theNode, "port").c_str());
            _name = _host + ":" + (parser.getAttribute(theNode, "port"));

            theNode = parser.findNode(NULL, "type");
            _capability = parser.getAttribute(theNode, "value");

            theNode = parser.findNode(NULL, "itad");
            _itad = atoi(parser.getAttribute(theNode, "value").c_str());

            theNode = parser.findNode(NULL, "holdTimer");
            _holdTimer = atoi(parser.getAttribute(theNode, "value").c_str());

            theNode = parser.findNode(NULL, "restartTimer");
            _restartTimer = atoi(parser.getAttribute(theNode, "value").c_str());

            theNode = parser.findNode(NULL, "maxRestartTimer");
            _maxRestartTimer = atoi(parser.getAttribute(theNode, "value").c_str());

            theNode = parser.findNode(NULL, "connectTimer");
            _connectTimer = atoi(parser.getAttribute(theNode, "value").c_str());

            theNode = parser.findNode(NULL, "maxConnectTimer");
            _maxConnectTimer = atoi(parser.getAttribute(theNode, "value").c_str());

            theNode = parser.findNode(NULL, "advertiseTimer");
            _advertiseTimer = atoi(parser.getAttribute(theNode, "value").c_str());

            theNode = parser.findNode(NULL, "protocolList");
            if (theNode)
            {

                list < xmlNodePtr > listOfNodes;
                parser.findNode(theNode, "protocolType", listOfNodes);

                for (list < xmlNodePtr > ::iterator itr =
                            listOfNodes.begin();
                        itr != listOfNodes.end(); itr++)
                {
                    xmlNodePtr itemNode = (*itr);
                    // get value attribute from node
                    string val = parser.getAttribute(itemNode, "value");
                    _protocols.push_back(val);
                }

            }

            theNode = parser.findNode(NULL, "addressFamilyList");
            if (theNode)
            {

                list < xmlNodePtr > listOfNodes;
                parser.findNode(theNode, "addressFamilyType", listOfNodes);

                for (list < xmlNodePtr > ::iterator itr =
                            listOfNodes.begin();
                        itr != listOfNodes.end(); itr++)
                {
                    xmlNodePtr itemNode = (*itr);
                    // get value attribute from node
                    string val = parser.getAttribute(itemNode, "value");
                    _addressFamilies.push_back(val);
                }

            }

            theNode = parser.findNode(NULL, "lsPeerList");
            if (theNode)
            {

                list < xmlNodePtr > listOfNodes;
                parser.findNode(theNode, "lsPeerNode", listOfNodes);

                for (list < xmlNodePtr > ::iterator itr =
                            listOfNodes.begin();
                        itr != listOfNodes.end(); itr++)
                {
                    xmlNodePtr itemNode = (*itr);
                    // get value attribute from node
                    string host = parser.getAttribute(itemNode, "host");
                    string port = parser.getAttribute(itemNode, "port");
                    string val = host + ":" + port;
                    _lsPeers.push_back(val);
                }

            }

        }

    ///
    friend ostream& operator<<(ostream &os, const VLocationServerData &elem);

    ///
    const string& getName() const
        {
            return _name;
        };

///
    const string& getHost() const
        {
            return _host;
        };

///
    int getPort() const
        {
            return _port;
        };

///
    const string& getCapability() const
        {
            return _capability;
        };

    ///
    int getItad() const
        {
            return _itad;
        };

    ///
    int getHoldTimer() const
        {
            return _holdTimer;
        };

    ///
    int getRestartTimer() const
        {
            return _restartTimer;
        };

    ///
    int getMaxRestartTimer() const
        {
            return _maxRestartTimer;
        };

    ///
    int getConnectTimer() const
        {
            return _connectTimer;
        };

    ///
    int getMaxConnectTimer() const
        {
            return _maxConnectTimer;
        };

    ///
    int getAdvertiseTimer() const
        {
            return _advertiseTimer;
        };

    ///
    int getProtoListSize() const
        {
            return _protocols.size();
        };

    ///
    void getProtoList(vector < string > &retVal) const
        {
            retVal.clear();
            for (list < string > ::const_iterator itr = _protocols.begin();
                    itr != _protocols.end(); itr++)
            {
                retVal.push_back(*itr);
            }
        };

    ///
    int getAddrListSize() const
        {
            return _addressFamilies.size();
        };

    ///
    void getAddrList(vector < string > &retVal) const
        {
            retVal.clear();
            for (list < string > ::const_iterator itr = _addressFamilies.begin();
                    itr != _addressFamilies.end(); itr++)
            {
                retVal.push_back(*itr);
            }
        };

    ///
    int getPeerListSize() const
        {
            return _lsPeers.size();
        };

    ///
    void getPeerList(vector < string > &retVal) const
        {
            retVal.clear();
            for (list < string > ::const_iterator itr = _lsPeers.begin();
                    itr != _lsPeers.end(); itr++)
            {
                retVal.push_back(*itr);
            }
        };

    private:

   /// copy operation to copy all elements (used by copy constructor and assign)
    void deepCopy(const VLocationServerData &copy)
        {
            _name = copy._name;
            _host = copy._host;
            _port = copy._port;
            _capability = copy._capability;
            _itad = copy._itad;
            _holdTimer = copy._holdTimer;
            _restartTimer = copy._restartTimer;
            _maxRestartTimer = copy._maxRestartTimer;
            _connectTimer = copy._connectTimer;
            _maxConnectTimer = copy._maxConnectTimer;
            _advertiseTimer = copy._advertiseTimer;
            _addressFamilies = copy._addressFamilies;
            _protocols = copy._protocols;
            _lsPeers = copy._lsPeers;

        };


    
        string _name;
        string _host;
        int _port;
        string _capability;
        u_int16_t _itad;
        int _holdTimer;
        int _restartTimer;
        int _maxRestartTimer;
        int _connectTimer;
        int _maxConnectTimer;
        int _advertiseTimer;
        list < string > _addressFamilies;
        list < string > _protocols;
        list < string > _lsPeers;

};

///
inline ostream& operator<<(ostream &os, const VLocationServerData &elem)
{

    os << "Name             : " << elem._name << endl;
    os << "Host             : " << elem._host << endl;
    os << "Port             : " << elem._port << endl;
    os << "Capability       : " << elem._capability << endl;
    os << "Itad             : " << elem._itad << endl;
    os << "HoldTimer        : " << elem._holdTimer << endl;
    os << "RestartTimer     : " << elem._restartTimer << endl;
    os << "MaxRestartTimer  : " << elem._maxRestartTimer << endl;
    os << "ConnectTimer     : " << elem._connectTimer << endl;
    os << "MaxConnectTimer  : " << elem._maxConnectTimer << endl;
    os << "AdvertiseTimer   : " << elem._advertiseTimer << endl;
    if (elem.getProtoListSize())
    {
        os << "Protocol List: " << endl;
        vector < string > protoList;
        elem.getProtoList(protoList);
        for (vector < string > ::const_iterator itr = protoList.begin();
                itr != protoList.end(); itr++)
        {
            os << "  " << *itr << endl;
        }
    }
    if (elem.getAddrListSize())
    {
        os << "Address Family List: " << endl;
        vector < string > addrList;
        elem.getAddrList(addrList);
        for (vector < string > ::const_iterator itr = addrList.begin();
                itr != addrList.end(); itr++)
        {
            os << "  " << *itr << endl;
        }
    }
    if (elem.getPeerListSize())
    {
        os << "LS Peer List: " << endl;
        vector < string > peerList;
        elem.getPeerList(peerList);
        for (vector < string > ::const_iterator itr = peerList.begin();
                itr != peerList.end(); itr++)
        {
            os << "  " << *itr << endl;
        }
    }

    return os;
}

#endif
