#ifndef  VMARSHALSERVERDATA_HXX
#define  VMARSHALSERVERDATA_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const VMarshalServerData_hxx_Version =
    "$Id: VMarshalServerData.hxx,v 1.19 2001/01/25 22:52:21 dabryan Exp $";

#include <stdio.h>
#include <string>

#include "VXmlParser.hxx"
#include "VException.hxx"


/// Marshal server data container class
class VMarshalServerData
{

    public:

    /// 
        VMarshalServerData()
        { }
        ;

    /// constructor from XML data string
        VMarshalServerData(const string& xmlData) throw (VException&)
        {
            parse(xmlData);
        }


    /// Copy constructor
        VMarshalServerData(const VMarshalServerData &copy)
        {

            deepCopy(copy);

        }

    /// assignment operator
        VMarshalServerData& operator=(const VMarshalServerData& other)
        {

            if (this != &other)
            {
                deepCopy(other);
            }

            return (*this);

        }

    /// parse the input XML stream
        void parse(const string& xmlData) throw (VException&)
        {

            _accessNumbers.clear();
            _bridgeNumber = "none";
            _allowUnbill = "true";

            VXmlParser parser(xmlData);

            xmlNode *theNode = parser.findNode(NULL, "marshalServer");
            _host = parser.getAttribute(theNode, "host");
            _port = atoi(parser.getAttribute(theNode, "port").c_str());
            _name = _host + ":" + (parser.getAttribute(theNode, "port"));

            theNode = parser.findNode(NULL, "type");
            _type = parser.getAttribute(theNode, "value");

            theNode = parser.findNode(NULL, "group");
            _group = parser.getAttribute(theNode, "value");

            theNode = parser.findNode(NULL, "gateway");
            _gatewayHost = parser.getAttribute(theNode, "host");
            _gatewayPort = atoi(parser.getAttribute(theNode, "port").c_str());

            theNode = parser.findNode(NULL, "allowUnbillableCalls");
            if (theNode)
            {
                _allowUnbill = parser.getAttribute(theNode, "value");
            }



            theNode = parser.findNode(NULL, "accessList");
            if (theNode)
            {

                list < xmlNodePtr > listOfAccessNumbers;
                parser.findNode(theNode, "accessNumber", listOfAccessNumbers);

                for (list < xmlNodePtr > ::iterator itr =
                            listOfAccessNumbers.begin();
                        itr != listOfAccessNumbers.end(); itr++)
                {
                    xmlNodePtr numberNode = (*itr);
                    // get value attribute from node
                    string val = parser.getAttribute(numberNode, "value");
                    _accessNumbers.push_back(val);
                }

            }

            theNode = parser.findNode(NULL, "noResponseTimer");
            _noResponse = atoi(parser.getAttribute(theNode, "value").c_str());

            theNode = parser.findNode(NULL, "bridgeNumber");
            if (theNode)
            {
                _bridgeNumber = parser.getAttribute(theNode, "value");
            }

        }

    ///
    friend ostream& operator<<(ostream &os, const VMarshalServerData &elem);

    ///
    const string& getName() const
        {
            return _name;
        };

///
    const string& getHost() const
        {
            return _host;
        };

///
    int getPort() const
        {
            return _port;
        };

///
    const string& getType() const
        {
            return _type;
        };

    ///
    const string& getGroup() const
        {
            return _group;
        };

    ///
    const string& getGatewayHost() const
        {
            return _gatewayHost;
        };

    ///
    int getGatewayPort() const
        {
            return _gatewayPort;
        };

    ///
    int getNumberListSize() const
        {
            return _accessNumbers.size();
        };

    ///
    void getNumberList(list < string > &retVal) const
        {
            retVal.clear();
            for (list < string > ::const_iterator itr = _accessNumbers.begin();
                    itr != _accessNumbers.end(); itr++)
            {
                retVal.push_back(*itr);
            }
        };

    ///
    int getNoResponse() const
        {
            return _noResponse;
        };

    ///
    const string& getBridgeNumber() const
        {
            return _bridgeNumber;
        };

    ///
    const string& getAllowUnbillable() const
        {
            return _allowUnbill;
        };

    private:

   /// copy operation to copy all elements (used by copy constructor and assign)
    void deepCopy(const VMarshalServerData &copy)
        {
            _name = copy._name;
            _host = copy._host;
            _port = copy._port;
            _type = copy._type;
            _group = copy._group;
            _gatewayHost = copy._gatewayHost;
            _gatewayPort = copy._gatewayPort;
            _accessNumbers = copy._accessNumbers;
            _noResponse = copy._noResponse;
            _bridgeNumber = copy._bridgeNumber;
            _allowUnbill = copy._allowUnbill;

        };


    
        string _name;
        string _host;
        int _port;
        string _type;
        string _group;
        string _gatewayHost;
        int _gatewayPort;
        list < string > _accessNumbers;
        int _noResponse;
        string _bridgeNumber;
        string _allowUnbill;

};

///
inline ostream& operator<<(ostream &os, const VMarshalServerData &elem)
{

    os << "Name             : " << elem._name << endl;
    os << "Host             : " << elem._host << endl;
    os << "Port             : " << elem._port << endl;
    os << "Server type      : " << elem._type << endl;
    os << "Server group     : " << elem._group << endl;
    os << "Gateway Host     : " << elem._gatewayHost << endl;
    os << "Gateway Port     : " << elem._gatewayPort << endl;
    if (elem.getNumberListSize())
    {
        os << "Access List: " << endl;
        list < string > numList;
        elem.getNumberList(numList);
        for (list < string > ::const_iterator itr = numList.begin();
                itr != numList.end(); itr++)
        {
            os << "  " << *itr << endl;
        }
    }
    os << "Bridge Num       : " << elem._bridgeNumber << endl;
    os << "No Response      : " << elem._noResponse << endl;
    os << "Allow unbillable : " << elem._allowUnbill << endl;
    return os;
}

#endif
