#ifndef  VREDIRECTSERVERDATA_HXX
#define  VREDIRECTSERVERDATA_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const VRedirectServerData_hxx_Version =
    "$Id: VRedirectServerData.hxx,v 1.11 2001/01/25 22:52:21 dabryan Exp $";

#include <stdio.h>
#include <string>

#include "VXmlParser.hxx"
#include "cpLog.h"
#include "VException.hxx"


class VRedirectServerData
{

public:
    
    ///
    VRedirectServerData(void)
        {}
    
    /// Constructor taking an XML stream as input
    VRedirectServerData(const string &xmlData) throw (VException&)
        {
            parse(xmlData);
        }

    /// Copy constructor
    VRedirectServerData(const VRedirectServerData &copy)
        {
            deepCopy(copy);
        }

    /// Assignment operator
    VRedirectServerData& operator=(const VRedirectServerData &other)
        {
            
            if (this != &other)
            {
                deepCopy(other);
            }
            
            return (*this);
            
        }

    /// Parse the XML input
    void parse(const string &xmlData) throw (VException&)
        {
            
            VXmlParser parser(xmlData);

            xmlNode *theNode = parser.findNode(NULL, "redirectServer");
            _host = parser.getAttribute(theNode, "host");
            _port = atoi(parser.getAttribute(theNode, "port").c_str());
            _name = _host + ":" + (parser.getAttribute(theNode, "port"));

            theNode = parser.findNode(NULL, "group");
            _group = parser.getAttribute(theNode, "value");

            theNode = parser.findNode(NULL, "syncPort");
            _syncPort = atoi(parser.getAttribute(theNode, "value").c_str());
            
        }

    ///
    friend ostream& operator<<(ostream& os, const VRedirectServerData& elem);

    ///
    const string& getName() const
        {
            return _name;
        };

    ///
    const string& getHost() const
        {
            return _host;
        };

    ///
    int getPort() const
        {
            return _port;
        };

    ///
    int getSyncPort() const
        {
            return _syncPort;
        };

    ///
    const string& getGroup() const
        {
            return _group;
        };
    
private:
    
    /// copy operation to copy all elements (used by copy constructor and assign)
    void deepCopy(const VRedirectServerData &copy)
        {
            
            _name = copy._name;
            _host = copy._host;
            _port = copy._port;
            _group = copy._group;
            _syncPort = copy._syncPort;
            
        };
    
        string _name;
        string _host;
        int _port;
        string _group;
        int _syncPort;

};

inline ostream& operator<<(ostream& os, const VRedirectServerData& elem)
{
    os << "Name:      " << elem._name << endl;
    os << "Host:      " << elem._host << endl;
    os << "Port:      " << elem._port << endl;
    os << "Group:     " << elem._group << endl;
    os << "Sync Port: " << elem._syncPort << endl;
    return os;
}

#endif
