#ifndef  VROUTEELEMENT_HXX
#define  VROUTEELEMENT_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const VRouteElement_hxx_Version =
    "$Id: VRouteElement.hxx,v 1.1 2001/04/10 00:38:25 kevinmc Exp $";

    
/// Dial plan element container
class VRouteElement
{

public:
    
    /// Constructor
    VRouteElement(const string& addr, const string& proto, const string& prefix, const string& nexthop)
        {
            _addr = addr;
            _proto = proto;
            _prefix = prefix;
            _nexthop = nexthop;
        }
    
    /// Assignment operator
    VRouteElement& operator=(const VRouteElement& other)
        {
	    
            if (this != &other)
            {
                _addr = other._addr;
                _proto = other._proto;
                _prefix = other._prefix;
                _nexthop = other._nexthop;
            }
            return (*this);
        }
    
    /// 
    const string& getAddr() const
        {
            return _addr;
        };

    ///
    const string& getProto() const
        {
            return _proto;
        };

    ///
    const string& getPrefix() const
        {
            return _prefix;
        };

    ///
    const string& getNextHop() const
        {
            return _nexthop;
        };
    
    friend ostream& operator<<(ostream& os, const VRouteElement& elem);
    
private:
    
    string _addr;
    string _proto;
    string _prefix;
    string _nexthop;
};

///
inline ostream& operator<<(ostream& os, const VRouteElement& elem)
{
    
    os << "Address       : " << elem.getAddr() << endl;
    os << "Protocol      : " << elem.getProto() << endl;
    os << "Prefix        : " << elem.getPrefix() << endl;
    os << "NextHop       : " << elem.getNextHop() << endl;
    return os;
}

#endif
