#ifndef  VSuavmServerData_hxx
#define  VSuavmServerData_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const VSuavmServerData_hxx_Version =
    "$Id: VSuavmServerData.hxx,v 1.8 2001/01/25 22:52:21 dabryan Exp $";

#include <stdio.h>
#include <string>

#include "VXmlParser.hxx"
#include "cpLog.h"

/// SUAVM server container class
class VSuavmServerData
{

public:

    /// constructor
    VSuavmServerData(const string hostName, int firstPort, int lastPort)
        {
            _host = hostName;
            _firstPort = firstPort;
            _lastPort = lastPort;
        }

    /// Assignment operator
    VSuavmServerData& operator=(const VSuavmServerData& other)
        {
            if (this != &other)
            {
                deepCopy(other);
            }
            return (*this);
        }

    /// Copy constructor
    VSuavmServerData(const VSuavmServerData &copy)
        {
            deepCopy(copy);
        }
    
    
    ///
    friend ostream& operator<<(ostream& os, const VSuavmServerData& elem);

    ///
    const string& getHost() const
        {
            return _host;
        };

    ///
    int getFirstPort() const
        {
            return _firstPort;
        };

    ///
    int getLastPort() const
        {
            return _lastPort;
        };

    private:

    /// copy operation to copy all elements (used by copy constructor and assign)
    void deepCopy(const VSuavmServerData &copy)
        {
            _host = copy._host;
            _firstPort = copy._firstPort;
            _lastPort = copy._lastPort;
        }   
    
        string _host;
        int _firstPort;
        int _lastPort;

};

inline ostream& operator<<(ostream& os, const VSuavmServerData& elem)
{

    os << "Name       : " << elem._host << endl;
    os << "First port : " << elem._firstPort << endl;
    os << "Last port  : " << elem._lastPort << endl;
    return os;

}

#endif
