
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

// blindly copied over and modified from the heartbeat directory -- look
// there for more detail

static const char* const PServerContainer_cxx_Version =
    "$Id: PServerContainer.cxx,v 1.14 2001/06/29 20:26:52 bko Exp $";

#include "ReadLockHelper.hxx"
#include "WriteLockHelper.hxx"
#include "PServerContainer.hxx"

Sptr < PServerContainer > PServerContainer::_thePSrvContainer = 0;


/// Return the one Server container or create it
/// if it doesn't already exist.
PServerContainer&
PServerContainer::instance()
{
    if ( _thePSrvContainer == 0 )
    {
        _thePSrvContainer = new PServerContainer;
    }
    return *_thePSrvContainer;
}


PServerContainer::PServerContainer()
        : BaseServerContainer()
{}



void
PServerContainer::AddServer (const Data& key)
{
    WriteLockHelper tableHelper(tableLock);

    Sptr < PServerObject > server = new PServerObject();
    serverList[key] = server;
}


PServerContainer::~PServerContainer( )
{}



bool
PServerContainer::processHeartbeat( const Data& key)
{

    Sptr < Server > serverPtr;

    if (search( key, serverPtr) )
    {
        ReadLockHelper tableHelper(tableLock);

        Sptr < PServerObject > psPtr;

        /// print data contained in the server object
        switch (serverPtr->getType() )
        {
            case SERVER_PS:
            {
                psPtr.dynamicCast(serverPtr);
                psPtr->processHeartbeat(key);

                break;
            }
            default:
            {
                break;
            }
        }

        return true;

    }

    else
    {
        return false;
    }

}


void
PServerContainer::doHouseKeeping()
{
    Data serverId;
    Sptr < Server > serverPtr;

    ReadLockHelper tableHelper(tableLock);

    if ( !serverList.empty() )
    {
        Sptr < PServerObject > psPtr;

        for ( TableIter i = serverList.begin(); i != serverList.end(); ++i)
        {

            serverId = i->first;
            serverPtr = i->second;

            switch (serverPtr->getType() )
            {
                case SERVER_PS:
                {
                    psPtr.dynamicCast(serverPtr);
                    psPtr->doHouseKeeping(serverId);
                    break;
                }
                default:
                break;
            }

        } // end for

    }  // end if
}



void
PServerContainer::getListofActive( const ServerType type, Sptr < vector < Data > > & listofServers )
{
    // function not used in the PS at this time.
}



void
PServerContainer::displayLists()
{
    // function not used in the PS at this time.
    return ;
}


// End of File
