
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

// part of the heartbeat -- see the heartbeat directory for more detail

static const char* const PServerObject_cxx_Version =
    "$Id: PServerObject.cxx,v 1.15 2002/02/25 05:50:21 sprajpat Exp $";

#include "cpLog.h"
#include "HeartbeatUtil.h"
#include "ProvisionServer.hxx"
#include "PServerObject.hxx"

PServerObject::PServerObject() :
        Server(SERVER_PS, 4)
{ }


///

PServerObject::PServerObject( const PServerObject& rhs ) : Server(rhs)
{}


///

PServerObject&
PServerObject::operator=( const PServerObject& rhs )
{
    if ( this != &rhs )
    {
        Server::operator= (rhs);
    }
    return *this;
}


PServerObject::~PServerObject()
{ }



void
PServerObject::processHeartbeat( const Data& key )
{

    incrHeartbeatCountByOne();

    if (itsStatus == Inactive)
        gotFirstHeartbeat(key);
    else
        if (itsStatus == ActivePending )
            gotHiccup();

    ProvisionServer::instance().gotBeat();

}


void
PServerObject::gotFirstHeartbeat( const Data& key )
{
    Server::gotFirstHeartbeat(key);

    // notify snmp server up
    Data serverAddress( convertToString(itsType) );
    serverAddress += ":";
    serverAddress += key;

    // hook here

    cpLog (LOG_DEBUG, "Got First Heartbeat!");

}


void
PServerObject::doHouseKeeping( const Data& serverId)
{

    // cpLog(LOG_DEBUG, "Local do housekeeping");

    if ( itsStatus != Inactive )
    {
        if ( heartbeatCount == 0 )
        {
            incrNumOfHeartbeatMissed();

            // *** keep this code around in case we decide
            // *** to reinstate the ActivePending state.
            //if ( itsStatus == Active)
            //   setStatus(ActivePending);

            //cpLog( LOG_DEBUG,"missed %d HBs", numOfHeartbeatMissed);

            if ( serverIsDead() )
            {

                cpLog( LOG_ALERT, "\n");
                cpLog( LOG_ALERT, "** %s Down: %s **",
                       convertToString(itsType),
                       serverId.logData() );

                // hook to PS server
                ProvisionServer::instance().lostBeat();
                setStatus(Inactive);

                // reset heartbeat counters
                clearHeartbeatCount();
                clearNumOfHeartbeatMissed();

            }

        }

        else
        {

            // reset heartbeat counters
            clearHeartbeatCount();
            clearNumOfHeartbeatMissed();

        }

    }
    // else do nothing
}
