#ifndef REGISTERMANAGER_HXX
#define REGISTERMANAGER_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const RegisterManager_hxx_Version =
    "$Id: RegisterManager.hxx,v 1.11 2001/03/23 13:43:14 icahoon Exp $";

#include <string>
#include <map>
#include <list>
#include "LockHelper.hxx"
#include "VMutex.h"
// #include <pthread.h>

class RegisterManager
{

    public:

        /// constructor
        RegisterManager()
        {
// Use LockHelper instead
//             pthread_mutex_init(&_lock, 0);
        }

        /// register for updates on a particular item
        void registerItem(const string &group, const string &name,
                          const string &hostWithPort);

        /// register for updates on a particular item
        void registerItemCat(const string &groupAndName,
                             const string &hostWithPort);

        /// unregister for updates on a particular item
        void unregisterItem(const string &group, const string &name,
                            const string &hostWithPort);

        /// register for updates on a particular itme
        void registerGroup(const string &group, const string &hostWithPort);

        /// unregister for updates on a particular item
        void unregisterGroup(const string &group, const string &hostWithPort);

        /// we modified or added item
        void modifyItem(const string &group, const string &name,
                        bool delFlag = false);

        /// we deleted an item
        void deleteItem(const string &group, const string &name)
        {
            modifyItem(group, name, true);
        }

        /// Get list formated map of registrations for sync
        void getList(string &list);

    private:

        typedef list < string > ClientHostList;
        typedef map < string, ClientHostList > ClientHostMap;

        /// Holds the list of clients registered for items
        ClientHostMap _itemMap;

        /// Holds the list of clients registered for group
        ClientHostMap _groupMap;

        /// Send item modify message to registered client
        void sendItemModify (const string &group, const string &name,
                             const string &host);

        /// Send item delete message to registered client
        void sendItemDelete (const string &group, const string &name,
                             const string &host);

        /// Send group modify message to registered client
        void sendGroupModify (const string &group, const string &name,
                              const string &host);

        /// Send group delete message to registered client
        void sendGroupDelete (const string &group, const string &name,
                              const string &host);

        // pthread_mutex_t _lock;
        VMutex _aMutex;

};

#endif
