/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



#include "ProvisionInterface.hxx"
#include "VException.hxx"
#include "VFeatureServerData.hxx"
#include "ConnectPserver.h"
#include "Feature.h"


void
getFeatureServers()
{
    list < string > retData;
    try
    {
        ProvisionInterface& pif = ProvisionInterface::instance();
        pif.getFeatureServers(retData);

        for (list < string > ::iterator itr = retData.begin();
                itr != retData.end(); itr++)
        {
            cout << "=====> Feature server: " << (*itr) << endl;
        }
    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
    }

}

void
getAllFeatureData()
{
    list < string > retData;
    VFeatureServerData myFeature;
    try
    {
        ProvisionInterface& pif = ProvisionInterface::instance();
        pif.getFeatureServers(retData);

        for (list < string > ::iterator itr = retData.begin();
                itr != retData.end(); itr++)
        {
            // cout << "=====> Feature server: " << (*itr) << endl;
            pif.getFeatureData(*itr, myFeature);
            // cout << "********************** " << endl;
            // cout << myFeature << endl;
            // cout << "********************** " << endl;
        }
    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
    }
}

void
getFeatureData()
{
    string feature;
    cout << "Enter Feature Server <ip:port> : ";
    cin >> feature;
    cout << endl;

    try
    {
        ProvisionInterface& pif = ProvisionInterface::instance();

        VFeatureServerData myFeature;
        pif.getFeatureData(feature, myFeature);
        cout << "**************\n";
        cout << myFeature << endl;
        cout << "**************\n";
    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
    }

}

void
getFeatureGroups()
{
    list < string > retData;
    try
    {
        ProvisionInterface& pif = ProvisionInterface::instance();
        pif.getFeatureGroups(retData);

        for (list < string > ::iterator itr = retData.begin();
                itr != retData.end(); itr++)
        {
            cout << "=====> Feature Groups: " << (*itr) << endl;
        }
    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
    }
}


void
getDialPlan(int ch)
{
    if (ch == 0)
    {
        cout << "Get Default Dial plan for (1)phone or (2)ip ??? ";
        cin >> ch;
        cout << endl;
    }

    try
    {
        ProvisionInterface& pif = ProvisionInterface::instance();
        if (ch == 1)
        {
            cout << "*** Get Phone dial plan ***" << endl;
            vector < VPlanElement > retData;
            pif.getDialPlan("defaultPhonePlan.xml", retData);

            for (vector < VPlanElement > ::iterator itr = retData.begin();
                    itr != retData.end(); itr++)
            {
                cout << (*itr) << endl;
            }
        }
        else if (ch == 2)
        {
            cout << "*** Get IP dial plan ***" << endl;
            vector < VPlanElement > retData;
            pif.getDialPlan("defaultIpPlan.xml", retData);

            for (vector < VPlanElement > ::iterator itr = retData.begin();
                    itr != retData.end(); itr++)
            {
                cout << (*itr) << endl;
            }
        }
    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
    }

}


void
featureServers()
{
    cout << "\n===> Feature Servers <===\n";

    int ch;
    while (true)
    {
        cout << "\n(1) Connect to pServer" << endl;
        cout << "(2) Get List of Feature Servers" << endl;
        cout << "(3) Get Feature Data" << endl;
        cout << "(4) Get Feature Groups" << endl;
        cout << "(5) Get Dial Plan" << endl;
        cout << "(999) Exit" << endl;
        cout << "Enter your choice: ";
        cin >> ch;
        cout << endl;

        switch (ch)
        {
            case 1:
            connectPserver();
            break;
            case 2: getFeatureServers();
            break;
            case 3: getFeatureData();
            break;
            case 4: getFeatureGroups();
            break;
            case 5: getDialPlan();
            break;
            case 999: return ;
            break;
            default:
            cout << "Don't know the option.\n\n";
            break;
        }

    }
}


