/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


#include "ProvisionInterface.hxx"
#include "VException.hxx"

#include "ConnectPserver.h"
#include "Heartbeat.h"


void
getHeartbeatServers()
{
    list < string > retData;
    try
    {
        ProvisionInterface& pif = ProvisionInterface::instance();
        pif.getHeartbeatServers(retData);

        for (list < string > ::iterator itr = retData.begin();
                itr != retData.end(); itr++)
        {
            cout << "=====> Heartbeat server: " << (*itr) << endl;
        }
    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
    }

}

void
getHeartbeatGroups()
{
    list < string > retData;
    try
    {
        ProvisionInterface& pif = ProvisionInterface::instance();
        pif.getHeartbeatGroups(retData);

        for (list < string > ::iterator itr = retData.begin();
                itr != retData.end(); itr++)
        {
            cout << "=====> Heartbeat Groups: " << (*itr) << endl;
        }
    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
    }
}


void
heartbeatServers()
{
    cout << "\n===> Heartbeat Servers <===\n";

    int ch;
    while (true)
    {
        cout << "\n(1) Connect to pServer" << endl;
        cout << "(2) Get List of Heartbeat Servers" << endl;
        cout << "(3) Get Heartbeat Server Group" << endl;
        cout << "(999) Exit" << endl;
        cout << "Enter your choice: ";
        cin >> ch;
        cout << endl;

        switch (ch)
        {
            case 1:
            connectPserver();
            break;
            case 2: getHeartbeatServers();
            break;
            case 3: getHeartbeatGroups();
            break;
            case 999: return ;
            break;
            default:
            cout << "Don't know the option.\n\n";
            break;
        }

    }
}

