/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "ProvisionInterface.hxx"
#include "VException.hxx"
#include "VJtapiServerData.hxx"
#include "ConnectPserver.h"
#include "Jtapi.h"


void
getJtapiServers()
{
    list < string > retData;
    try
    {
        ProvisionInterface& pif = ProvisionInterface::instance();
        pif.getJtapiServers(retData);

        for (list < string > ::iterator itr = retData.begin();
                itr != retData.end(); itr++)
        {
            cout << "=====> Jtapi server: " << (*itr) << endl;
        }
    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
    }

}

void
getAllJtapiData()
{
    list < string > retData;
    VJtapiServerData myJtapi;
    try
    {
        ProvisionInterface& pif = ProvisionInterface::instance();
        pif.getJtapiServers(retData);

        for (list < string > ::iterator itr = retData.begin();
                itr != retData.end(); itr++)
        {
            // cout << "=====> Jtapi server: " << (*itr) << endl;
            // pif.getJtapiData(*itr, myJtapi);
            // cout << "********************" << endl;
            // cout << myJtapi << endl;
            // cout << "********************" << endl;
        }
    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
    }
}

void
getJtapiData()
{
    string jtapi;
    cout << "Enter jtapi Server <ip:port> : ";
    cin >> jtapi;
    cout << endl;

    try
    {
        ProvisionInterface& pif = ProvisionInterface::instance();

        VJtapiServerData myJtapi;
        pif.getJtapiData(jtapi, myJtapi);
        cout << "**************\n";
        cout << myJtapi << endl;
        cout << "**************\n";
    }
    catch (VException& e)
    {
        printf("Got exception here...\n");
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
    }

}

void
getJtapiGroups()
{
    list < string > retData;
    try
    {
        ProvisionInterface& pif = ProvisionInterface::instance();
        pif.getJtapiGroups(retData);

        for (list < string > ::iterator itr = retData.begin();
                itr != retData.end(); itr++)
        {
            cout << "=====> Jtapi Groups: " << (*itr) << endl;
        }
    }
    catch (VException& e)
    {
        cout << "Throw exception: " << e.getDescription().c_str() << endl;
    }

}


void
jtapiServers()
{
    cout << "\n===> Jtapi Servers <===\n";

    int ch;
    while (true)
    {
        cout << "\n(1) Connect to pServer" << endl;
        cout << "(2) Get List of Jtapi Servers" << endl;
        cout << "(3) Get Jtapi Data" << endl;
        cout << "(4) Get Jtapi Groups" << endl;
        cout << "(999) Exit" << endl;
        cout << "Enter your choice: ";
        cin >> ch;
        cout << endl;

        switch (ch)
        {
            case 1:
            connectPserver();
            break;
            case 2: getJtapiServers();
            break;
            case 3: getJtapiData();
            break;
            case 4: getJtapiGroups();
            break;
            case 999: return ;
            break;
            default:
            cout << "Don't know the option.\n\n";
            break;
        }

    }
}

