
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const VServerGroupContainer_cxx_Version =
    "$Id: VServerGroupContainer.cxx,v 1.21 2001/04/21 08:38:39 icahoon Exp $";
#include "global.h"
#include <stdio.h>
#include <iostream.h>
#include <cassert>
#include <algorithm>
#include "VXmlParser.hxx"
#include "VPs_def.h"
#include "VServerGroupContainer.hxx"
#include "VException.hxx"
#include "cpLog.h"
#include "VBadDataException.hxx"

VServerGroupContainer::VServerGroupContainer()
        : _updateServerGrpMap(true)
{}



VServerGroupContainer::~VServerGroupContainer()
{}



bool
VServerGroupContainer::needsUpdate()
{

    return _updateServerGrpMap;

}


void
VServerGroupContainer::markForUpdate()
{

    _updateServerGrpMap = true;

}


const MapOfServerGroups&
VServerGroupContainer::getServerMap()
throw (VException&)
{

    assert(_data.size());
    initializeMap(_data);
    return (_serverGrpMap);

}


ListOfServers
VServerGroupContainer::getServerGroup(const string& server)
throw (VException&)
{

    ListOfServers retList;
    assert(_data.size());
    initializeMap(_data);
    MapOfServerGroups::iterator tItr;

    for (tItr = _serverGrpMap.begin(); tItr != _serverGrpMap.end();
            tItr++)
    {

        GroupMap& gMap = (*tItr).second;
        GroupMap::iterator gItr;

        for (gItr = gMap.begin(); gItr != gMap.end(); gItr++)
        {

            ListOfServers& sList = (*gItr).second;
            ListOfServers::iterator itr = find(sList.begin(), sList.end(),
                                               server);
            if (itr != sList.end())
            {

                retList.push_back((*gItr).first);
                break;

            }

        }

    }

    return retList;

}


ListOfServers
VServerGroupContainer::getServerList(const string& type)
throw (VException&)
{

    assert(_data.size());
    initializeMap(_data);
    ListOfServers retList;

    if (_serverGrpMap.count(type))
    {

        GroupMap& gMap = _serverGrpMap[type];
        GroupMap::iterator gItr;

        for (gItr = gMap.begin(); gItr != gMap.end(); gItr++)
        {
            ListOfServers& sList = (*gItr).second;
            copy(sList.begin(), sList.end(),
                 inserter(retList, retList.begin()));
        }

    }

    return retList;

}


ListOfServers
VServerGroupContainer::getServerList()
throw (VException&)
{

    assert(_data.size());
    initializeMap(_data);
    ListOfServers retList;
    MapOfServerGroups::iterator tItr;

    for (tItr = _serverGrpMap.begin(); tItr != _serverGrpMap.end();
            tItr++)
    {
        GroupMap& gMap = (*tItr).second;
        GroupMap::iterator gItr;

        for (gItr = gMap.begin(); gItr != gMap.end(); gItr++)
        {
            ListOfServers& sList = (*gItr).second;
            copy(sList.begin(), sList.end(),
                 inserter(retList, retList.begin()));
        }

    }

    return retList;

}


void
VServerGroupContainer::getGroupList(list < string > &retList)
throw (VException&)
{

    assert(_data.size());
    initializeMap(_data);
    MapOfServerGroups::iterator tItr;

    for (tItr = _serverGrpMap.begin(); tItr != _serverGrpMap.end();
            tItr++)
    {

        GroupMap& gMap = (*tItr).second;

        GroupMap::iterator gItr;
        for (gItr = gMap.begin(); gItr != gMap.end(); gItr++)
        {

            retList.push_back((*gItr).first);

        }

    }

}


void
VServerGroupContainer::getHostToGroupMap(HostToGroupMap &retmap)
throw (VException&)
{

    assert(_data.size());
    initializeMap(_data);
    MapOfServerGroups::iterator tItr;
    retmap.clear();

    for (tItr = _serverGrpMap.begin(); tItr != _serverGrpMap.end();
            tItr++)
    {

        GroupMap& gMap = (*tItr).second;

        GroupMap::iterator gItr;
        for (gItr = gMap.begin(); gItr != gMap.end(); gItr++)
        {
            string group = ((*gItr).first);
            list < string > &servers = ((*gItr).second);

            for (list < string > ::iterator itr = servers.begin();
                    itr != servers.end(); itr++)
            {
                retmap[*itr] = group;
            }

        }

    }

}


void
VServerGroupContainer::initializeMap(const string& dataInput)
throw (VException&)
{
    if (!_updateServerGrpMap)
    {
        return ;
    }

    char buf[256];
    sprintf(buf, "Calling initializeMap for %s", getContainerName().c_str());
    cpLog(LOG_DEBUG, buf);
    _data = dataInput;
    _serverGrpMap.erase(_serverGrpMap.begin(), _serverGrpMap.end());
    
    // ensure they passed us some data (the XML to parse) and setup a parser
    assert(dataInput.size());
    
    try
    {
        
        VXmlParser parser(dataInput);
        
        if (usesTypes()) {
        
            // generate a list, one entry for every serverType
            list < xmlNodePtr > listOfServerTypes;
            parser.findNode(NULL, "serverType", listOfServerTypes);

            // iterate throught the serverType list
            for (list < xmlNodePtr > ::iterator itr = listOfServerTypes.begin();
                 itr != listOfServerTypes.end(); itr++)
            {
                
                xmlNodePtr serverNode = (*itr);
                // get value attribute from node
                string fType = parser.getAttribute(serverNode, "value");
                cpLog(LOG_DEBUG, "Parsing for serverType (%s)", fType.c_str());
                
                // get a list of the serverGroups within a particular serverType
                list < xmlNodePtr > listOfServerGroups;
                parser.findNode(serverNode, "serverGroup", listOfServerGroups);
                cpLog(LOG_DEBUG, "Number of server groups : %d",
                      listOfServerGroups.size());
                
                for (list < xmlNodePtr > ::iterator itr2 = listOfServerGroups.begin();
                     itr2 != listOfServerGroups.end(); itr2++)
                {
                    
                    xmlNodePtr groupNode = (*itr2);
                    string groupName = parser.getAttribute(groupNode, "value");
                    cpLog(LOG_DEBUG, "Parsing for server group (%s)",
                          groupName.c_str());
                    list < xmlNodePtr > listOfServers;
                    parser.findNode(groupNode, "server", listOfServers);
                    
                    for (list < xmlNodePtr > ::iterator itr3 = listOfServers.begin();
                         itr3 != listOfServers.end(); itr3++)
                    {
                        
                        xmlNodePtr sNode = (*itr3);
                        string hostName = parser.getAttribute(sNode, "host");
                        string port = parser.getAttribute(sNode, "port");
                        
                        //Save data as host/port
                        hostName += ":";
                        hostName += port;
                        cpLog(LOG_DEBUG,
                              "Adding (%s) for type (%s) and group (%s)",
                              hostName.c_str(), fType.c_str(), groupName.c_str());
                        
                        // create a list of concatanated host/port names
                        // the serverType indexes to ServerGroups maps, and index
                        // it to list of host/port nams
                        GroupMap& gMap = _serverGrpMap[fType];
                        gMap[groupName].push_back(hostName);
                        
                    }

                }
                
            }
            
        }
        else  // does not have types (use a static string for type)
        {
            

            // generate a list, one entry for every serverType
            list < xmlNodePtr > listOfServerGroups;
            parser.findNode(NULL, "serverGroup", listOfServerGroups);
            
            // iterate throught the serverType list
            for (list < xmlNodePtr > ::iterator itr = listOfServerGroups.begin();
                 itr != listOfServerGroups.end(); itr++)
            {
                
                xmlNodePtr groupNode = (*itr);
                string groupName = parser.getAttribute(groupNode, "value");
                
                list < xmlNodePtr > listOfServers;
                parser.findNode(groupNode, "server", listOfServers);
                
                for (list < xmlNodePtr > ::iterator itr3 = listOfServers.begin();
                     itr3 != listOfServers.end(); itr3++)
                {
                    
                    xmlNodePtr sNode = (*itr3);
                    string hostName = parser.getAttribute(sNode, "host");
                    string port = parser.getAttribute(sNode, "port");

                    //Save data as host/port
                    hostName += ":";
                    hostName += port;
                    cpLog(LOG_DEBUG, "Adding (%s) for group (%s)",
                          hostName.c_str(), groupName.c_str());
                    
                    // create a list of concatanated host/port names
                    // the serverType indexes to ServerGroups maps, and index
                    // in it to list of host/port nams
                    // Use "Heartbeat" for type, which is just ignored
                    GroupMap& gMap = _serverGrpMap[getStaticTypeString()];
                    gMap[groupName].push_back(hostName);
                    
                }
                
            }

        } // end else (if usesTypes())

        
    } // end try block
    catch (VBadDataException& e)
    {
        cpLog(LOG_DEBUG, "Failed to get details for feature %s",
              e.getDescription().c_str());
    }

    _updateServerGrpMap = false;

}

