#ifndef VServerGroupContainer_hxx
#define VServerGroupContainer_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "global.h"

static const char* const VServerGroupContainer_hxx_Version =
    "$Id: VServerGroupContainer.hxx,v 1.17 2001/03/21 03:41:36 icahoon Exp $";

#include <string>
#include <list>
#include <map>
#include "VException.hxx"

typedef list < string > ListOfServers;
typedef map < string, ListOfServers > GroupMap;
typedef map < string, GroupMap > MapOfServerGroups;

typedef map < string, string > HostToGroupMap;


class VServerGroupContainer
{

    public:

        /// Indicate if the map in memory needs to be updated
        bool needsUpdate();

        /// Returns list of groups , a given server belongs to
        ListOfServers getServerGroup(const string& server) throw (VException&);

        /** Returns list of servers for a given type, for example
          * for a feature type, returns a list of servers serving the
          * feature.
          */
        ListOfServers getServerList(const string& type) throw (VException&);

        /** Returns list of all servers including all types
          */
        ListOfServers getServerList() throw (VException&);

        /** Returns list of all groups for all servers
         */
        void getGroupList(list < string > &retList) throw (VException&);

        /// Sets the flag indicating we need to reread this (it is old)
        void markForUpdate();

        /// Returns server group hirearchy from the cache
        const MapOfServerGroups& getServerMap() throw (VException&);

        /// Get a map of hosts to group they belong to
        void getHostToGroupMap(HostToGroupMap &retmap) throw (VException&);

    /// Overload to parse input to initialize list of servers
    virtual void initializeMap(const string& data) throw (VException&);

    protected:

        /// Constructor
        VServerGroupContainer();

    /// destructor
        virtual ~VServerGroupContainer();

    /// Get the name of the container
    virtual string getContainerName() {return "VServerGroupContainer";}

    /// Is serverType used in XML? If so, use file data, else use static type 
    virtual bool usesTypes() = 0;

    /// Name to use if using static type -- dummy string if stored in XML
    virtual string getStaticTypeString() = 0;

    /// Map of the server types -> server groups -> servers
        MapOfServerGroups _serverGrpMap;

    /// Local variable to indicate that the server map needs to be updated (old data)
    bool _updateServerGrpMap;

    /// store the data
    string _data;

};

#endif
