#!/usr/bin/perl

# ====================================================================
# The Vovida Software License, Version 1.0 
# 
# Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. The names "VOCAL", "Vovida Open Communication Application Library",
#    and "Vovida Open Communication Application Library (VOCAL)" must
#    not be used to endorse or promote products derived from this
#    software without prior written permission. For written
#    permission, please contact vocal@vovida.org.
# 
# 4. Products derived from this software may not be called "VOCAL", nor
#    may "VOCAL" appear in their name, without prior written
#    permission of Vovida Networks, Inc.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
# NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
# NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
# IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
# 
# ====================================================================
# 
# This software consists of voluntary contributions made by Vovida
# Networks, Inc. and many individuals on behalf of Vovida Networks,
# Inc.  For more information on Vovida Networks, Inc., please see
# <http://www.vovida.org/>.

# $Id: provactions.pm,v 1.11.2.1 2002/04/24 16:54:05 bsamson Exp $

package provactions;

use psinslib;
use provtemplates;

BEGIN {
#    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

    # if using RCS/CVS, this may be preferred
    $VERSION = do { my @r = (q$Revision: 1.11.2.1 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; # must be all one line, for MakeMaker
    @ISA         = qw(Exporter);
    @EXPORT      = qw(
		      &get_account_list
		      &get_vm_box
		      &get_data_one_user
		      &put_data_one_user
		      &create_new_user
		      &validate_input
		      &print_input
		      &update_one_user_entry
		      &update_one_user
		      &get_gw_data
		      &update_gw
		      &update_vm_box
		      &create_vm_box
		      &delete_vm_box
		      &delete_one_user
		      &set_vm_dialplan
		      &verify_active
		     );


    %EXPORT_TAGS = ( );     # eg: TAG => [ qw!name1 name2! ],
    
    # your exported package globals go here,
    # as well as any optionally exported functions
    @EXPORT_OK   = qw();
}
#our @EXPORT_OK;

$VOCAL_BASE = "/usr/local/vocal";  # VOCAL_BASE
$VOCAL_BASE_CONFIGURED = '@prefix@';
if($VOCAL_BASE_CONFIGURED ne '@'.'prefix@') {
    $VOCAL_BASE = $VOCAL_BASE_CONFIGURED;
}


$vmhome_dir = "$VOCAL_BASE/var/vmhome";

chomp($hostname = `hostname`);
($junk1,$junk2,$junk3,$junk4,@addr) = gethostbyname($hostname);
($a,$b,$c,$d) = unpack('C4', $addr[0]);
$local_ip = "$a.$b.$c.$d";


sub verify_active {
    # verify that provisioning system is active
    my $data = &get_data("localhost", "SystemConfiguration", "GlobalConfigData");
    if(defined($data)) {
	return 1;
    } else {
	return 0;
    }
}

sub get_account_list {
    my($users) = [];
    @$users = split(/,/, &list_data("localhost", "Accounts"));
    pop(@$users);

    return $users;
}


sub get_vm_box {
    my($params) = @_;

    open(CONFIG, "$vmhome_dir/$$params{user}/config") || return -1;
    while(<CONFIG>) {
	chomp;
	if(s/^SendTo\s+string\s+//) {
	    $$params{voicemail_addr} = $_;
	}
	if(s/^Greeting\s+string\s+//) {
	    if(/custom\.wav/ && -f "$vmhome_dir/$$params{user}/custom.wav") {
		$$params{use_custom} = 1;
	    }
	}
    }
    close(CONFIG) || return -1;

    return;
}


sub get_data_one_user {
    my($userid) = @_;
    my($data) = &get_data("localhost", "Accounts", $userid);
    my($parameters) = {};

    if($data) {
	
	$$parameters{user} = $userid;
	
	# scan for xml
	if($data =~ /\<authenticationType value\=\"(\w*)\"/) {
	    $auth_type = $1;
	    if($auth_type eq "None") {
		$$parameters{use_passwd} = 0;
	    } elsif ($auth_type eq "Digest") {
		$$parameters{use_passwd} = 1;
	    } 
	}
	if($data =~ /\<authenticationPassword value\=\"(\w*)\"/) {
	    $$parameters{passwd} = $1;
	}
	$$parameters{enable_vm} = 0;
	if($data =~ /\<fnab enabled\=\"(\w*)\"/) {
	    if($1 eq "true") {
		$$parameters{enable_vm} = 1;
	    } 
	}
	
	# get vm data
	&get_vm_box($parameters);
	
	return $parameters;
    } else {
	return undef;
    }
}


sub put_data_one_user {
    my($userid, $account, $called, $calling, $fna) = @_;
    my $ret;
    $ret = &put_data("localhost", "Accounts", $userid, $account);
    $ret = $ret 
      || &put_data("localhost", "Contact_Lists_Called", $userid, $called);
    $ret = $ret 
      || &put_data("localhost", "Contact_Lists_Calling", $userid, $calling);
    $ret = $ret
      || &put_data("localhost", "Features_${local_ip}_5105_Called", "$userid.cpl", $fna);
    return $ret;
}




sub create_new_user {
    my($params) = @_;
    my($output);
    my($params_clean) = {};

#    print "xxx: $$params{user} xxx\n\n";
    # need to verify that user does not already exist
    # need to verify that user has "safe" userid
    
    if($$params{user} =~ /^[0-9]{4,4}$/ && $$params{user} ne "7000") {
	$$params_clean{user} = $$params{user};
	$$params_clean{passwd} = "password";
	my($user) = &update_one_user_entry($params_clean);
	my($vm);

	if(!$user) {
	    $vm = &create_vm_box($params_clean);
	}

	if(!$vm && !$user) {
	    $output .= "Created user \"$$params{user}\" successfully";
	} else {
	    $output .= "Failed to create user \"$$params{user}\": $!";
	    if($vm) {
		$output .= "<br>voice mailbox creation failed<p>";
	    }
	    if ($user) {
		$output .= "<br>user creation failed<P>";
	    }
	}
    } else {
	if($$params{user} ne "7000") {
	    $output = "Failed: Users may have 4 digit user IDs only<p>";
	} else {
	    $output = "Failed: User 7000 is reserved for voice mail<p>";
	}
    }

    return $output;
}


sub validate_input {
    my($params) = @_;

    # validate input here, return 1 if ok.

    if( 

       # validate that the user can only contain 4 digits and is not
       # 7000

       $$params{user} =~ /^[0-9]{4,4}$/ && 
       $$params{user} ne "7000" &&

       # validate that passwords can only contain "reasonable" characters

       $$params{passwd} =~ /^([0-9A-Za-z\!\@\#\$\%\^\&\*\(\)\[\]\{\}\|\\\,\.\<\>\/\?]*)$/
      ) {
	return 1;
    } else {
	return 0;
    }
}


sub print_input {
    my($params) = @_;
    my($output, $tmp);

    foreach $tmp (sort keys %$params) {
        $output .= $tmp . " : " . $$params{$tmp} . "<p>";
    }
    print $output;
}


# returns -1 on error or 0 if ok
sub update_one_user_entry {
    my($params) = @_;

    if(&validate_input($params)) { 
        my($new_user_xml) = &create_user_xml($params);
	my($new_called_xml) = &create_contact_called_xml($params);
	my($new_calling_xml) = &create_contact_calling_xml($params);
	my($new_fna_xml) = &create_fna_feature_xml($params);

        my($ret) = &put_data_one_user($$params{user}, 
				      $new_user_xml, 
				      $new_called_xml,
				      $new_calling_xml,
				      $new_fna_xml);

	return $ret;
    } else {
	return -1;
    }
}


# returns -1 on error or 0 if ok
sub update_one_user {
    my($params) = @_;
    my($output);

    if(!&update_one_user_entry($params) && !&update_vm_box($params)) { 
        $output = "Saved user \"$$params{user}\" successfully";
    } else {
        $output = "Failed to save user \"$$params{user}\" successfully: $!";
    }

    return $output;
}


sub get_gw_data {
    my($data) = &get_data("localhost", "Marshals", "${local_ip}:5065");
    my($param) = {};

    if($data =~ /gateway host=\"((\w|\d|[\.\-])*)\" port=\"([0-9]*)\"/) {
	$$param{gw_location} = "$1:$3";
    }
    return $param;
}


sub update_gw {
    my($params) = @_;
    my($output);

    my($retcode) = 0;
    my($reason);
    # validate here

    if($$params{gw_location} =~ /^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}.[0-9]{1,3}(:[0-9]{1,5})?$/) {
	$retcode = 0;
    } else {
	$retcode = "Illegal gateway location (must be in the form host:port, where host is an IP address)";
    }

    if(!$retcode) {
	my($gw_xml) = &create_gw_marshal_xml($params);
	$retcode = &put_data("localhost", "Marshals", "${local_ip}:5065", $gw_xml);
	if($retcode) {
	    $retcode = "Unable to write data to provisioning server"; 
	}
    }

    if(!$retcode) {
        $output = "Wrote new gateway successfully";
    } else {
        $output = "Failed to write new gateway successfully.<p>Reason: $retcode";
    }

    return $output;
}


sub update_vm_box {
    my($params) = @_;
    my($greeting) = "greeting.wav";

    if(!$$params{enable_vm}) {
	# nothing to do if there's no VM enabled
#	print "no vm enabled\n";
	return 0;
    }

    if($$params{use_custom}) {
	# use a custom greeting
	$greeting = "custom.wav";
    }

    # modify the config file with the appropriate bits
    
    open(CONFIG, ">$vmhome_dir/$$params{user}/config") || return -1;
    print CONFIG "Name      string  User\n";
    print CONFIG "SendTo    string  $$params{voicemail_addr}\n";
    print CONFIG "Greeting  string  $greeting\n";
    close(CONFIG) || return -1;

    # if the wav file has been uploaded, upload it.

    if($$params{uploaded_greeting}) {
	# print uploaded_greeting
	my($fh) = upload('uploaded_greeting');
#	print "I received an uploaded_greeting!\n";
	if($fh) {
#	    print "I received an uploaded_greeting!\n";
	    open(CUSTOM, ">$vmhome_dir/$$params{user}/custom.wav") || return -1;
	    while(<$fh>) {
		print CUSTOM;
	    }
	    close(CUSTOM);
	}
    }

    return 0;
}


# returns -1 on error or 0 if ok
sub create_vm_box {
    my($params) = @_;

    # test to make sure that I can write in vmhome
    if(!-w $vmhome_dir) {
	return -1;
    }

    mkdir "$vmhome_dir/$$params{user}", 0755 || return -1;

    return 0;
}


sub delete_vm_box {
    my($user) = @_;

    if($user !~ /\.\./ && $user !~ /\//) {
	my($userdir) = "$vmhome_dir/$user";
	my($x);
	# allow deletions if things don't have 2 . in them
	# simultaneously or a / 
	opendir(DIR, $userdir) || return -1;
	@files = grep(!/^\.\.?$/, readdir(DIR));
	closedir(DIR);

	foreach $x (@files) {
	    $x = "$userdir/$x";
#	    print "deleting $x...<br>\n";
	    unlink $x || return -1;
	}

#	unlink @files || return -1;
	rmdir $userdir || return -1;
    } 
    return 0;
}


sub delete_one_user {
    my($params) = @_;
    my($output);

    my $retcode = !defined(&delete_user("localhost", $$params{user}));
    $retcode = &delete_vm_box($$params{user}) || $retcode;

    if(!$retcode) { 
        $output = "Deleted user \"$$params{user}\" successfully";
    } else {
        $output = "Failed to delete user \"$$params{user}\" successfully: $!";
    }

    return $output;
}


sub set_vm_dialplan {
    my($output);
    my($retcode) = &put_data("localhost",
			     "SystemConfiguration", 
			     "defaultPhonePlan", 
			     &create_digitalplan_xml());
    $retcode = &put_data("localhost",
			 "SystemConfiguration",
			 "defaultIpPlan", 
			 &create_ipplan_xml()) || $retcode;

    if(!$retcode) {
	$output = "Created VM dialplan successfully";
    } else {
	$output = "Failed to create VM dialplan successfully: $!";
    }

    return $output;
}

return 1;


