#!/usr/bin/perl

# ====================================================================
# The Vovida Software License, Version 1.0 
# 
# Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. The names "VOCAL", "Vovida Open Communication Application Library",
#    and "Vovida Open Communication Application Library (VOCAL)" must
#    not be used to endorse or promote products derived from this
#    software without prior written permission. For written
#    permission, please contact vocal@vovida.org.
# 
# 4. Products derived from this software may not be called "VOCAL", nor
#    may "VOCAL" appear in their name, without prior written
#    permission of Vovida Networks, Inc.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
# NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
# NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
# IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
# 
# ====================================================================
# 
# This software consists of voluntary contributions made by Vovida
# Networks, Inc. and many individuals on behalf of Vovida Networks,
# Inc.  For more information on Vovida Networks, Inc., please see
# <http://www.vovida.org/>.

# $Id: provtemplates.pm,v 1.4 2002/02/06 22:32:43 bko Exp $

package provtemplates;

BEGIN {
#    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

    # if using RCS/CVS, this may be preferred
    $VERSION = do { my @r = (q$Revision: 1.4 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; # must be all one line, for MakeMaker
    @ISA         = qw(Exporter);
    @EXPORT      = qw(&create_contact_called_xml 
		      &create_contact_calling_xml
		      &create_user_xml
		      &create_fna_feature_xml
		      &create_ipplan_xml
		      &create_digitalplan_xml
		      &create_gw_marshal_xml
		      );
    %EXPORT_TAGS = ( );     # eg: TAG => [ qw!name1 name2! ],
    
    # your exported package globals go here,
    # as well as any optionally exported functions
    @EXPORT_OK   = qw();
}
#our @EXPORT_OK;

chomp($hostname = `hostname`);
($junk1,$junk2,$junk3,$junk4,@addr) = gethostbyname($hostname);
($a,$b,$c,$d) = unpack('C4', $addr[0]);
$local_ip = "$a.$b.$c.$d";


sub create_contact_called_xml {
    my($params) = @_;
    my($output);

    if($$params{enable_vm}) {
	$output = <<EOL
<?xml version="1.0" ?>
<calledContactList>
  <contact>
    <host>ForwardNoAnswerBusyGroup</host>
    <port>any</port>
  </contact>
</calledContactList>
EOL
  ;
    } else {
	$output = <<EOL
<?xml version="1.0" ?>
<calledContactList>
</calledContactList>
EOL
  ;
    }
    return $output;
}

sub create_contact_calling_xml {
    my($params) = @_;
    my($output) = <<EOL
<?xml version="1.0" ?>
<callingContactList>
</callingContactList>
EOL
  ;
    return $output;
}

sub create_user_xml {
    my($params) = @_;

    my($authentication_type) = "None";
    if($$params{use_passwd}) {
	$authentication_type = "Digest";
    }

    my($callforward_block) = <<EOL
<fnab enabled="false"><setfeat>OFF</setfeat>
<featuregroup>ForwardNoAnswerBusyGroup</featuregroup>
<fna set="false"><forwardto>none</forwardto>
</fna>
<cfb set="false"><forwardto>none</forwardto>
</cfb>
</fnab>
EOL
    ;
    if($$params{enable_vm}) {
    $callforward_block = <<EOL
<fnab enabled="true"><setfeat>ON</setfeat>
<featuregroup>ForwardNoAnswerBusyGroup</featuregroup>
<fna set="true"><forwardto>sip:7000</forwardto>
</fna>
<cfb set="true"><forwardto>sip:7000</forwardto>
</cfb>
</fnab>
EOL
    ;
    }

    my($output) = <<EOL
<user><name>$$params{user}</name>
<password>6f7d5634f65192a3b9c8479cee3b655</password>
<ipaddr>0.0.0.0</ipaddr>
<marshalgroup>UserAgentGroup</marshalgroup>
<staticRegistration value="false"><terminatingContactList><contact value="UserAgentGroup"></contact>
<contact value="000.000.000.000:0000"></contact>
</terminatingContactList>
</staticRegistration>
<authenticationType value="$authentication_type"><authenticationPassword value="$$params{passwd}"></authenticationPassword>
</authenticationType>
<failurecase>unknown</failurecase>
<cfa enabled="false"><setfeat>OFF</setfeat>
<featuregroup>ForwardAllCallsGroup</featuregroup>
<forwardto>none</forwardto>
</cfa>
$callforward_block<cs enabled="false"><setfeat>OFF</setfeat>
<featuregroup>CallScreeningGroup</featuregroup>
</cs>
<clbl enabled="false"><setfeat>OFF</setfeat>
<featuregroup>CallBlockingGroup</featuregroup>
<nineHundredBlocked adminSet="false" userSet="false"></nineHundredBlocked>
<longDistanceBlocked adminSet="false" userSet="false"></longDistanceBlocked>
<prefix>not loaded from xml</prefix>
</clbl>
<jtapi enabled="false"><setfeat>OFF</setfeat>
<featuregroup>JtapiGroup</featuregroup>
</jtapi>
<callReturn enabled="false"><setfeat>OFF</setfeat>
<featuregroup>CallReturnGroup</featuregroup>
</callReturn>
<callerIdBlocking enabled="false"><setfeat>OFF</setfeat>
<featuregroup>CallerIdBlockingGroup</featuregroup>
</callerIdBlocking>
<voicemail enabled="false"><setfeat>OFF</setfeat>
<featuregroup>VoiceMailGroup</featuregroup>
</voicemail>
<userGroup>Kazoo</userGroup>
<aliases></aliases>
</user>
EOL
    ;
    return $output;
}

# this isn't updated
sub create_fna_feature_xml {
    my($params) = @_;

    my($output) = <<EOL
<?xml version="1.0" ?>
<!DOCTYPE cpl SYSTEM "cpl.dtd">
<cpl><incoming><lookup clear="yes" source="sip:$$params{user};user=phone"><success><proxy ordering="first-only" timeout="15"><busy><location clear="yes" url="sip:7000;user=phone"><redirect></redirect>
</location>
</busy>
<noanswer><location clear="yes" url="sip:7000;user=phone"><redirect></redirect>
</location>
</noanswer>
<failure><location clear="yes" url="sip:unknown"><redirect></redirect>
</location>
</failure>
</proxy>
</success>
<notfound><reject status="reject"></reject>
</notfound>
</lookup>
</incoming>
</cpl>
EOL
  ;
    return $output;
}


sub create_ipplan_xml {
    return <<EOL
<ipplan><key index="0" value="^sip:[0-9][0-9][0-9][0-9][0-9][0-9]*"><contact value="sip:\$USER\@${local_ip}:5065;user=phone"></contact>
</key>
<key index="1" value="^sip:9.*"><contact value="sip:\$USER\@${local_ip}:5065;user=phone"></contact>
</key>
</ipplan>
EOL
    ;
}

sub create_digitalplan_xml {
    return <<EOL
<digitalplan><key index="0" value="^sip:7000"><contact value="sip:7000\@${local_ip}:5110;user=phone"></contact>
</key>
<key index="1" value="^sip:9.*"><contact value="sip:\$USER\@${local_ip}:5065;user=phone"></contact>
</key>
<key index="2" value="^sip:[0-9][0-9][0-9][0-9][0-9][0-9]*"><contact value="sip:\$USER\@${local_ip}:5065;user=phone"></contact>
</key>
</digitalplan>
EOL
  ;
}


sub create_gw_marshal_xml {
    my($params) = @_;

    my($gateway) = $$params{gw_location};
    my($gw_host);
    my($gw_port) = 5060;

    if($gateway =~ s/\:([0-9]+)//) {
	$gw_port = $1;
    }
    $gw_host = $gateway;

    return <<EOL
<marshalServer host="${local_ip}" port="5065"><type value="Gateway"></type>
<group value="GatewayGroup"></group>
<gateway host="${gw_host}" port="${gw_port}"></gateway>
<noResponseTimer value="100"></noResponseTimer>
<allowUnbillableCalls value="true"></allowUnbillableCalls>
<allowUnknownCallers value="false"></allowUnknownCallers>
<conferenceBridge><bridgeNumber value="none"></bridgeNumber>
<accessList></accessList>
</conferenceBridge>
</marshalServer>
EOL
  ;
}

return 1;
