#!/usr/bin/perl

# ====================================================================
# The Vovida Software License, Version 1.0 
# 
# Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. The names "VOCAL", "Vovida Open Communication Application Library",
#    and "Vovida Open Communication Application Library (VOCAL)" must
#    not be used to endorse or promote products derived from this
#    software without prior written permission. For written
#    permission, please contact vocal@vovida.org.
# 
# 4. Products derived from this software may not be called "VOCAL", nor
#    may "VOCAL" appear in their name, without prior written
#    permission of Vovida Networks, Inc.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
# NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
# NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
# IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
# 
# ====================================================================
# 
# This software consists of voluntary contributions made by Vovida
# Networks, Inc. and many individuals on behalf of Vovida Networks,
# Inc.  For more information on Vovida Networks, Inc., please see
# <http://www.vovida.org/>.

# $Id: psinslib.pm,v 1.10 2002/05/09 19:49:06 bko Exp $

package psinslib;
use IO::Socket;
use Socket;

BEGIN {
#    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

    # if using RCS/CVS, this may be preferred
    $VERSION = do { my @r = (q$Revision: 1.10 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; # must be all one line, for MakeMaker
    @ISA         = qw(Exporter);
    @EXPORT      = qw(&get_data &put_data &list_data &delete_user);
    %EXPORT_TAGS = ( );     # eg: TAG => [ qw!name1 name2! ],
    
    # your exported package globals go here,
    # as well as any optionally exported functions
    @EXPORT_OK   = qw();
}
#our @EXPORT_OK;


sub create_connection {
    my($addr, $port) = @_;

    my($remote);
    $remote = IO::Socket::INET->new( Proto     => "tcp",
                                     PeerAddr  => $addr,
                                     PeerPort  => $port,
                                   );
    if(defined($remote)) {
	$remote->autoflush(1);
    }
    return ($remote);
}


sub create_sock {
    my($hostport) = @_;
    my($port) = 6005;
    my($sock);

    if($hostport =~ s/\:([0-9]+)$//) {
	$port = $1;
    }
    $sock = &create_connection($hostport, $port);
    return $sock;
}


sub put_data {
    my($hostport, $dirname, $filename, $data) = @_;
    my($socket) = &create_sock($hostport);
    my($len);
    my($retval);
    my($flag) = 0;
    my($count);
    my($buf);
    my($tmpdata);
    my($oldfh);
    my($rin);

    $oldfh = select($socket); $| = 1; select($oldfh);


    if(!defined($socket)) {
	return -1;
    }

    $len = length($data);
    if (! ((print $socket "PUT $dirname $filename\n") && 
	   (print $socket "Content-Length: $len\n") &&
	   (print $socket $data) &&
	   (print $socket "\n"))) {
	# error
	return -1;
    }

    while(!$flag) {
	my($rin);
	$rin = '';

	vec($rin, fileno($socket), 1) = 1;
	$count = select($rin, undef, undef, undef);
	if($count > 0) {
	    @bits = split(//, unpack("b*", $rin));
	    if($bits[fileno($socket)]) {
		$l = sysread($socket, $tmpdata, 1);
		$buf .= $tmpdata;
		if($buf =~ /\n/) {
		    $flag = 1;
		}
	    } else {
		# this is a problem
		close($socket);
		return -1;
	    }
	} elsif ($count == 0) {
	    # this is a timeout
	    close($socket);
	    return -1;
	} else {
	    close($socket);
	    return -1;
	}
    }

    if($buf !~ /200 OK/) {
	# no good
	close($socket);
	return -1;
    }


    if(!close($socket)) {
	# error
	return -1;
    }

    return 0;
}


sub list_data {
    my($hostport, $listname) = @_;
    my($socket) = &create_sock($hostport);
    my($flag, $len, $buf, $data, $rin);
    my($gotheader);

    print $socket "LIST $listname VPP/1.0\n\n";
    while(!$flag) {
	my($rin);
	$rin = '';

	vec($rin, fileno($socket), 1) = 1;
	if(select($rin, undef, undef, undef)) {
	    @bits = split(//, unpack("b*", $rin));
	    if($bits[fileno($socket)]) {
		$l = read($socket, $data, 1);
		$buf .= $data;
		if($buf =~ /\n/) {
		    $flag = 1;
		}
	    }
	}
    }

    close($socket);

    return $buf;
}


sub get_data {
    my($hostport, $dirname, $filename) = @_;
    my($socket) = &create_sock($hostport);
    my($flag, $len, $buf, $data);
    my($gotheader, $l);

    if (!defined($socket)) {
	return undef;
    }

    print $socket "GET $dirname $filename VPP/1.0\n\n";
    while(!$flag) {
	my($rin, $ein);
	$rin = '';
	$ein = '';

	vec($rin, fileno($socket), 1) = 1;
	vec($ein, fileno($socket), 1) = 1;
	if(select($rin, undef, $ein, undef) > 0) {
	    @bits = split(//, unpack("b*", $rin));
	    @errbits = split(//, unpack("b*", $ein));
            if($errbits[fileno($socket)]) {
		return undef;
	    }
	    if($bits[fileno($socket)]) {
		$l = read($socket, $data, 1);
		if(!defined($l)) {
		    # some sort of error
			return undef;
		}
		$buf .= $data;
		if($gotheader) {
		    if($len == length($buf)) {
			$flag = 1;
		    }
		}
		if($buf =~ s/^200 OK\nContent-Length: ([0-9]*)\n//) {
		    $len = $1;
		    $gotheader = 1;
		} elsif ($buf =~ s/^400 FAILED\n//) {
		    $flag = 2;
		}
	    } else {
		# some sort of error 
		return undef;
	    }
	} else {
	    # some sort of error
	    return undef;
	}
    }
    close($socket);
    if($flag == 2) {
	return undef;
    }
    return $buf;
}


sub delete_user {
    my($hostport, $username) = @_;
    my($socket) = &create_sock($hostport);
    my($flag, $len, $buf, $data);
    my($gotheader);

    if (!defined($socket)) {
	return undef;
    }

    print $socket "DELETEUSER $username \n\n";
    while(!$flag) {
	my($rin);
	$rin = '';
	vec($rin, fileno($socket), 1) = 1;
	if(select($rin, undef, undef, undef)) {
	    @bits = split(//, unpack("b*", $rin));
	    if($bits[fileno($socket)]) {
		$l = read($socket, $data, 1);
#		print $data, "\n";
		$buf .= $data;
		if($gotheader) {
		    if($len <= length($buf)) {
			$flag = 1;
		    }
		}
		if($buf =~ s/^200 OK\nContent-Length: ([0-9]*)\n//) {
		    $len = $1;
		    $gotheader = 1;
		} elsif ($buf =~ s/^400 FAILED\n//) {
		    $flag = 2;
		}
	    }
	}
    }

    close($socket);

    if($flag == 1) {
	return 0;
    } else {
	return undef;
    }
}

END {
}


1;
