#ifndef AAATransceiver_hxx
#define AAATransceiver_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const AAATransceiver_hxx_Version =
    "$Id: AAATransceiver.hxx,v 1.6.2.1 2002/05/24 23:50:10 sprajpat Exp $";

#include "global.h"
#include "vtypes.h"
#include "B2bDef.hxx"
#include "VThread.hxx"
#include "AAAEvent.hxx"
#include "RadiusMessage.hxx"

class UdpStack;

using Vocal::Fifo;

#define RADIUS_MAX_RETRANS_COUNT 4

///Radius payload
struct RadiusPayload
{
    u_int16_t retransCount;
    TimerEntryId myRetransId;
    Sptr<RadiusMessage> rMsg;
    Sptr<AAAEvent> aEvent;
    RadiusPayload() : retransCount(RADIUS_MAX_RETRANS_COUNT) { }
};

/// Transceiver to send/receive AAA requests
class AAATransceiver 
{
   public:
      /// Create one with default values
      AAATransceiver(int recvPort , Sptr<Fifo<Sptr<SipProxyEvent> > > outputFifo); 

      ///
      ~AAATransceiver();

      ///
      AAAEventFifo&  getSendFifo() { return(mySendFifo); }

      ///
      void sendRequest(Sptr<AAAEvent> aaaEvent);

      ///
      void setResponse(Sptr<AAAEvent> event);

      ///
      void sendAuthRequest(Sptr<AAAEvent> aaaEvent);
      ///
      void sendAcctStart(Sptr<AAAEvent> aaaEvent);
      ///
      void sendAcctInterim(Sptr<AAAEvent> aaaEvent);
      ///
      void sendAcctStop(Sptr<AAAEvent> aaaEvent);

   private:
      ///
      static void* sendThreadWrapper(void *p);
      ///
      static void* rcvThreadWrapper(void *p);
      ///
      void* receiveMain();
      ///
      void* sendMain();
      ///
      u_int8_t getNextId() { return(myRadiusIdCounter++); };
      ///
      void udpSend(Sptr<AAAEvent> aaaEvent);

      ///
      void buildThePayload(Sptr<AAAEvent> aaaEvent,
                                u_int8_t type, u_int8_t subType);
      ///
      void fillAAAData(Sptr<AAAEvent> aEvent,
                            Sptr<RadiusMessage> rMsg) const;
      ///
      void fillAccountStartData(Sptr<AAAEvent> aEvent,
                                     Sptr<RadiusMessage> rMsg) const;
      ///
      void fillInterimAccountData(Sptr<AAAEvent> aEvent,
                                  Sptr<RadiusMessage> rMsg) const;
      ///
      void fillAccountStopData(Sptr<AAAEvent> aEvent,
                                     Sptr<RadiusMessage> rMsg) const;


      ///
      void cleanUp(u_int8_t rId);
      ///
      void handleRetransmission(Sptr<AAAEvent> aaaEvent);

      ///Not defined intentionally
      AAATransceiver( const AAATransceiver& src ) { };
      ///Not defined intentionally
      const AAATransceiver& operator =( const AAATransceiver& src );

      ///
      Sptr<UdpStack> myUdpStack;
      ///
      AAAEventFifo mySendFifo;
      ///
      Sptr<Fifo<Sptr<SipProxyEvent> > > myOutputFifo;
      ///
      VThread myRecvThread;
      ///
      VThread mySendThread;
      ///
      Sptr<RadiusPayload> myRetransMap[256];
      ///
      bool myShutdownFlg;
      ///
      u_int8_t   myRadiusIdCounter;
      ///
      VMutex     myMutex;
};


#endif
