#ifndef AuthData_hxx
#define AuthData_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const AuthDara_hxx_Version =
    "$Id: AuthData.hxx,v 1.5 2001/09/10 22:49:19 sprajpat Exp $";

#include "global.h"
#include "BasicAAAData.hxx" 
#include "B2bDef.hxx"


/// data container for Auth Data 
class AuthData : public BasicAAAData
{
   public:
      /// Create one with default values
      AuthData(const string& uName, const string& uPasswd, 
               const string& selfIp, const string& callingId, 
               const string& calledId)
         : BasicAAAData(uName, selfIp, callingId, calledId), 
           myUserPasswd(uPasswd),
           mySessionTimeOut(0),
           myStatus(A_NONE)  
      { };

      ///
      string className() { return "AuthData"; };

      ///
      AuthData( const AuthData& src ): BasicAAAData(src)
      {
          copyObj(src);
      }

      /// 
      const AuthData& operator =( const AuthData& src )
      {
          if(this != &src)
          {
              BasicAAAData::copyObj(src);
              copyObj(src);
          }
          return *this;
      }

      ///
      void copyObj(const AuthData& src )
      {
          mySessionTimeOut = src.mySessionTimeOut;
          myUserPasswd = src.myUserPasswd;
          myStatus = src.myStatus;
      }

      ///
      void setUserPasswd(const string& uPasswd) { myUserPasswd = uPasswd; }
      ///
      const string&  getUserPasswd()const { return(myUserPasswd); }

      ///
      void setStatus(AAAStatus s) { myStatus = s; }
      ///
      AAAStatus getStatus()const { return(myStatus); }

      ///
      int getSessionTimeOut() const { return (mySessionTimeOut); };
      ///
      void setSessionTimeOut(int sOut) { mySessionTimeOut = sOut; };

   private:
       ///
       string myUserPasswd;
       ///
       int mySessionTimeOut;
       ///
       AAAStatus myStatus;
};


#endif
