/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const B2bUa_cxx_Version = 
    "$Id: B2bUa.cxx,v 1.11 2002/04/09 19:27:01 sprajpat Exp $";


#include "Fifo.h"
#include "B2bConfig.hxx"
#include "B2bUa.hxx"
#include "MultiLegCallControl.hxx"
#include "CallDB.hxx"

extern "C" {
    void B2bUa_destroy(void)
    {
        B2bUa::destroy();
    }
};

B2bUa* B2bUa::myInstance = 0;

void
B2bUa::destroy()
{
    cpLog(LOG_DEBUG, "B2bUa::destroy()");
    MultiLegCallControl::destroy();
    CallDB::destroy();

    delete B2bUa::myInstance;
    B2bUa::myInstance = 0;
}

B2bUa&
B2bUa::instance()
{
    assert(myInstance != 0);
    return *myInstance;
}

B2bUa&
B2bUa::instance(const Data& applName, unsigned short  defaultSipPort, bool filteron, bool nat)
{
    assert(myInstance == 0);
    if(myInstance == 0)
    {
        myInstance = new B2bUa(applName,defaultSipPort,filteron,nat);
    }
    return *myInstance;
}

B2bUa::B2bUa( const Data& applName, unsigned short  defaultSipPort, bool filteron, bool nat)
{
    cpLog(LOG_DEBUG, "B2bUa::B2bUa");
    if (atexit(B2bUa_destroy) < 0)
    {
        cpLog(LOG_ALERT, "Failed to register with atexit()");
    };

    myInputEventFifo = new Fifo < Sptr < SipProxyEvent > >;
    assert( myInputEventFifo != 0 );
    myWorkerThread = new B2bWorkerThread(myInputEventFifo);

    mySipStack = new SipTransceiver(applName, defaultSipPort, nat);

    assert( mySipStack != 0 );

    mySipThread = new SipThread(mySipStack, myInputEventFifo, false);
    assert( myWorkerThread != 0 );
    assert( mySipThread != 0 );
    myWorkerThread->run();
    mySipThread->run();
}

void
B2bUa::run()
{
    mySipThread->join();
    myWorkerThread->join();
}

B2bUa::~B2bUa()
{
}

Sptr<AAATransceiver> 
B2bUa::getAAATransceiver()
{
    if(myAAATransceiver == 0)
    {
        ///Create AAA Transceiver, let the UDP stack select a local port
//        int port = B2bConfig::instance().getInt(B2bConfig::RAD_LOCAL_AUTH_PORT)+8000;
        myAAATransceiver = new AAATransceiver(-1, myInputEventFifo);
    }
    return myAAATransceiver;
}
