#ifndef B2bUa_hxx
#define B2bUa_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const B2bUa_hxx_Version = 
    "$Id: B2bUa.hxx,v 1.6 2002/04/03 07:42:57 sprajpat Exp $";


#include "Sptr.hxx"
#include "Fifo.h"
#include "B2bWorkerThread.hxx"
#include "SipThread.hxx"
#include "AAATransceiver.hxx"
#include "SipTransceiver.hxx"

using Vocal::SipTransceiver;
using Vocal::Fifo;
using Vocal::SipThread;

/** B2bUa is the Main wrapper, that wraps all the components needed
    for B2b UA.
 */
class B2bUa
{
    public:
      ///
      static B2bUa& instance();

      ///Constructor to initialze B2bUa global instance.Should be called only once
      static B2bUa& instance(const Data& applName, unsigned short  defaultSipPort, 
                             bool filteron, bool nat);


      ///
      string className() { return "B2bUa"; }

      /**
         Frees the memory associated with singelton instance.
         gets register to atexit() function at the time of creation.
      */
      static void destroy(void);

      /** Virtual destructor
       */
      virtual ~B2bUa();

      ///
      void run();

      ///
      Sptr<SipTransceiver> getSipTransceiver() { return mySipStack; };

      ///
      Sptr<AAATransceiver> getAAATransceiver();

      ///
      Sptr< Fifo < Sptr < SipProxyEvent > > > getEventFifo() 
      { 
          return myInputEventFifo; 
      };

    private:
        //To avoid annoying warning from compiler
        friend class dummy;
        /** Create the Instance of B2bUa, to wrap up the application wide
         *  sipStack .
         */
        B2bUa( const Data& applName, unsigned short  defaultSipPort, bool filteron, bool nat);

        ///
        static B2bUa* myInstance;
        ///
        Sptr<SipThread> mySipThread;
        ///
        Sptr<B2bWorkerThread> myWorkerThread;

        ///
        Sptr<SipTransceiver> mySipStack;

        ///
        Sptr<AAATransceiver> myAAATransceiver;

        ///
        Sptr< Fifo < Sptr < SipProxyEvent > > > myInputEventFifo;

        /** Suppress copying
         */
        B2bUa(const B2bUa &);
        
        /** Suppress copying
         */
        const B2bUa & operator=(const B2bUa &);
};

#endif
