/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const B2bWorkerThread_cxx_Version =
    "$Id: B2bWorkerThread.cxx,v 1.9 2001/12/04 18:30:48 sprajpat Exp $";


#include "SipEvent.hxx"
#include "UaBase.hxx"
#include "AAAEvent.hxx"
#include "CallTimerEvent.hxx"
#include "CallDB.hxx"
#include "MultiLegCallControl.hxx"
#include "B2bWorkerThread.hxx"

void
B2bWorkerThread::shutdown()
{
    assert( myFifo != 0 );

    ThreadIf::shutdown();

    myFifo->add(0);
}


void
B2bWorkerThread::thread()
{
    cpLogSetLabelThread (VThread::selfId(), "bcWorker");

    while ( true )
    {
        assert( myFifo != 0 );

        Sptr < SipProxyEvent > nextEvent = myFifo->getNext();

        cpLog(LOG_DEBUG, "B2bWorkerThread received event");

        if ( isShutdown() == true )
        {
            return;
        }

        assert( nextEvent != 0 );
        //TODO add processing for each type of events
        //SIP_EVENT, TIMER_EVENT, HTTP_EVENT, RADIUS_EVENT
        Sptr<SipEvent> sipEvent;
        sipEvent.dynamicCast(nextEvent);
        if(sipEvent != 0)
        {
            handleSipEvent(sipEvent);
            continue;
        }

        Sptr<AAAEvent> authEvent;
        authEvent.dynamicCast(nextEvent);
        if(authEvent != 0)
        {
            cpLog(LOG_DEBUG, "Handling AAA event");
            MultiLegCallControl::instance().processEvent(nextEvent);
            continue;
        }
        Sptr<CallTimerEvent> timerEvent;
        timerEvent.dynamicCast(nextEvent);
        if(timerEvent != 0)
        {
            cpLog(LOG_DEBUG, "Handling Timer event");
            MultiLegCallControl::instance().processEvent(nextEvent);
            continue;
        }
    }
    cpLog(LOG_DEBUG, "Exiting worker thread...");
}

void
B2bWorkerThread::handleSipEvent(Sptr<SipEvent> sipEvent)
{
    cpLog(LOG_DEBUG, "Handling SIP event");
    //get the callleg and lookup in the Call database to
    //see if callleg already exists.If so get the UserAgent and
    //invoke the action on it.
    Sptr<SipMsg> sipMsg = sipEvent->getSipMsg();
    assert(sipMsg != 0);
    SipCallLeg cLeg(sipMsg->getFrom(), sipMsg->getTo(), sipMsg->getCallId());
    Sptr<MultiLegCallData> mData = CallDB::instance().getMultiLegCallData(cLeg);
    if(mData != 0)
    {
        //Call Data found
        Sptr<UaBase> agent;
        agent.dynamicCast(mData->getCallLeg(cLeg));
        if(agent != 0)
            agent->receivedMsg(sipMsg);
    }
    else
    {
        MultiLegCallControl::instance().processEvent(sipEvent);
    } 
}

