#ifndef BillingData_hxx
#define BillingData_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const BillingDara_hxx_Version =
    "$Id: BillingData.hxx,v 1.2 2001/09/10 22:49:37 sprajpat Exp $";

#include "global.h"
#include <string>
#include "BasicAAAData.hxx" 
#include "B2bDef.hxx"


/// data container for Billing Data 
class BillingData : public BasicAAAData
{
   public:
      /// Create one with default values
      BillingData(const string& sessionId, 
               const string& uName, 
               const string& selfIp, const string& callingId, 
               const string& calledId)
         : BasicAAAData(uName, selfIp, callingId, calledId), 
           mySessionId(sessionId),
           myDuration(0),
           myTerminationCause(BT_NONE) ,
           mySetupDelay(0)
      { };

      ///
      string className() { return "BillingData"; };

      ///
      BillingData( const BillingData& src ): BasicAAAData(src)
      {
          copyObj(src);
      }

      /// 
      const BillingData& operator =( const BillingData& src )
      {
          if(this != &src)
          {
              BasicAAAData::copyObj(src);
              copyObj(src);
          }
          return *this;
      }

      ///
      void copyObj(const BillingData& src )
      {
           mySessionId = src.mySessionId;
           myDuration = src.myDuration;
           myTerminationCause = src.myTerminationCause;
           mySetupDelay = src.mySetupDelay;
      }

      ///
      const string& getSessionId() const { return (mySessionId); };
      ///
      int getDuration() const { return (myDuration); };
      ///
      void setDuration(int duration) { myDuration = duration; };
      ///
      BillingTermCause getTermCause() const { return (myTerminationCause); };
      ///
      void setTermCause(BillingTermCause cs) { myTerminationCause=cs; };
      ///
      int getSetupDelay() const { return (mySetupDelay); };
      ///
      void setSetupDelay(int delay) { mySetupDelay = delay; };

   private:
       ///
       string mySessionId;
       ///
       int myDuration;
       ///
       BillingTermCause myTerminationCause;
       ///
       int mySetupDelay;
};


#endif
