/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const MultiLegCallControl_cxx_Version =
    "$Id: MultiLegCallControl.cxx,v 1.16.2.1 2002/05/24 23:50:10 sprajpat Exp $";


#include "SipEvent.hxx" 
#include "SystemInfo.hxx" 
#include "AAAEvent.hxx" 
#include "AuthAgent.hxx" 
#include "CallTimerEvent.hxx" 
#include "CallDB.hxx" 
#include "B2bUa.hxx" 
#include "MultiLegCallControl.hxx" 

using namespace Vocal;

MultiLegCallControl* MultiLegCallControl::myInstance = 0;

void 
MultiLegCallControl::receivedRequest(UaBase& agent, const Sptr<SipMsg>& msg)
{
    //Find the agent's peer and send status to all peers current
    Sptr<ContactData> cData;
    switch(msg->getType())
    {
        case SIP_BYE:
        {
            //Done with the contact, remove from the contact list
            cData = agent.popContact();
        }
        break;
        case SIP_ACK:
        {
            //Check if the message has SDP data, if so update the SDP data in agent
            Sptr<SipSdp> sdp;
            sdp.dynamicCast(msg->getContentData(0));
            if(sdp != 0) agent.setSdp(sdp);
            //Follow through to do the rest
        }
        default:
        {
            cData = agent.getContactData();
        }
        break;
    }
    if(cData != 0)
    {
        const ContactData::CallPeerList& pList = cData->getPeerList();
        for(ContactData::CallPeerList::const_iterator itr = pList.begin();
                                    itr != pList.end(); itr++)
        {
            Sptr<UaBase> pAgent;
            pAgent.dynamicCast(*itr);
            assert(pAgent != 0);
            //Remove the peer from the peer list
            if(msg->getType() == SIP_BYE)
            {
                int nPeers = CallDB::instance().removePeer(*pAgent, agent);
                //If this was the last peer, send bye msg 
                if(nPeers == 0)
                {
                    //Send the call-leg
                    pAgent->sendMsg(msg);
                }
            }
            else
            {
                //Send the call-leg
                pAgent->sendMsg(msg);
            }
        }
    }


    if(msg->getType() == SIP_BYE)
    {
        CallDB::instance().removeCallData(agent);
    }
}

void 
MultiLegCallControl::receivedStatus(UaBase& agent, const Sptr<SipMsg>& msg)
{
    Sptr<StatusMsg> statusMsg;
    statusMsg.dynamicCast(msg);
    assert(statusMsg != 0);
    if(msg->getCSeq().getMethod() == CANCEL_METHOD)
    {
        //do Nothing
        return;
    }
    
    //Find the agent's peer and send status to all peers
    Sptr<ContactData> cData = agent.getContactData();
    assert(cData != 0);
    const ContactData::CallPeerList& pList = cData->getPeerList();
    for(ContactData::CallPeerList::const_iterator itr = pList.begin();
                                itr != pList.end(); itr++)
    {
        Sptr<UaBase> pAgent;
        pAgent.dynamicCast(*itr);
        assert(pAgent != 0);
        pAgent->sendMsg(msg);
    }

    if(msg->getCSeq().getMethod() == BYE_METHOD)
    {
        CallDB::instance().removePeer(agent);
    }
}

void 
MultiLegCallControl::processEvent(const Sptr<SipProxyEvent>& event)
{
    Sptr<SipEvent> sipEvent;
    sipEvent.dynamicCast(event);
    if(sipEvent != 0)
    {
        Sptr<SipMsg> sipMsg = sipEvent->getSipMsg();
        cpLog(LOG_DEBUG, "Received event with message:%s", sipMsg->encode().logData());
        assert(sipMsg != 0);
        if(sipMsg->getType() != SIP_INVITE)
        {
            //Handle Cancel and BYE messages and return.
            if((sipMsg->getType() == SIP_CANCEL) ||
               (sipMsg->getType() == SIP_BYE))
            {
                Sptr<SipCommand> sipCmd;
                sipCmd.dynamicCast(sipMsg);
                if(sipCmd != 0)
                {
                    Sptr<StatusMsg> sMsg = new StatusMsg(*sipCmd, 200);
                    cpLog(LOG_DEBUG, "Sending reply :%s", sMsg->encode().logData()); 
                    B2bUa::instance().getSipTransceiver()->sendReply(sMsg);
                }
            }
            return;
        }
        //Send 100 first
        Sptr<SipCommand> sCmd;
        sCmd.dynamicCast(sipMsg);
        Sptr<StatusMsg> statusMsg = new StatusMsg(*sCmd, 100);
        B2bUa::instance().getSipTransceiver()->sendReply(statusMsg);

        //Do authentication
        Sptr<AuthAgent> authAgent = new AuthAgent(getNextSessionId(), sipMsg);
        myMultiLegCallAuthMap[authAgent->getId()] = authAgent; 
        authAgent->doAuth();

        return;
    }

    Sptr<AAAEvent> authEvent;
    authEvent.dynamicCast(event);
    if(authEvent != 0)
    {
        AuthAgent* authAgent = authEvent->getAuthAgent();
        cpLog(LOG_DEBUG, "Received AAA response for id: %ld", authAgent->getId()); 
        authAgent->authDone(authEvent);
        return;
    }

    Sptr<CallTimerEvent> timerEvent;
    timerEvent.dynamicCast(event);
    if(timerEvent != 0)
    {
        (timerEvent->getAgent())->timerExpired();
    }
}


MultiLegCallControl& 
MultiLegCallControl::instance()
{
    if(myInstance == 0)
    {
        myInstance = new MultiLegCallControl();
    }
    return *myInstance;
}

void
MultiLegCallControl::destroy(void)
{
    cpLog(LOG_DEBUG, "MultiLegCallControl::destroy");
    delete MultiLegCallControl::myInstance;
    MultiLegCallControl::myInstance = 0;
}


MultiLegCallControl::~MultiLegCallControl()
{
    myShutdownFlag = true;
    myFriedFifo.add(0);
    myCleanupThread.join();
}

MultiLegCallControl::MultiLegCallControl()
    : myShutdownFlag(false)
{
    //Initialze the value to be the unix seconds at the time of creation
    time_t t;
    mySessionId = time(&t);
    myCleanupThread.spawn(MultiLegCallControl::cleanupThread, this);
}

void*
MultiLegCallControl::cleanupThread(void* args)
{
    MultiLegCallControl* self = static_cast<MultiLegCallControl*>(args);
    while(1)
    {
        unsigned long id = self->myFriedFifo.getNext();
        if(self->myShutdownFlag) break;
        MultiLegCallAuthMap::iterator itr = self->myMultiLegCallAuthMap.find(id);
        if(itr != self->myMultiLegCallAuthMap.end())
        {
            cpLog(LOG_DEBUG,  "Removing id:%d ",id);
            self->myMultiLegCallAuthMap.erase(id);
        } 
    }
    return 0;
}


void 
MultiLegCallControl::removeAgent(unsigned long id)
{
    myFriedFifo.addDelayMs(id, 5000);
}
