#ifndef SipCallLegData_hxx
#define SipCallLegData_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const SipCallLegData_hxx_Version =
    "$Id: SipCallLegData.hxx,v 1.4 2002/04/03 07:42:57 sprajpat Exp $";

#include "global.h"
#include <vector>

#include "SipCallLeg.hxx" 
#include "ContactData.hxx" 
#include "B2bDef.hxx"
#include "SipMsg.hxx"
#include "SipSdp.hxx"

using Vocal::SipMsg;
using Vocal::SipSdp;
using Vocal::SipCallLeg;

/// data container for a callLeg
class SipCallLegData 
{
   public:
      ///
      typedef vector<Sptr<ContactData> > ContactVector;

      ///
      SipCallLegData( const Sptr<SipMsg>& reqMsg );

      ///
      SipCallLegData( const SipCallLegData& src )
      {
          copyObj(src);
      }

      /// 
      const SipCallLegData& operator =( const SipCallLegData& src )
      {
          if(this != &src)
          {
              copyObj(src);
          }
          return *this;
      }

      ///
      void copyObj(const SipCallLegData& src) 
      {
          mySipCallLeg = src.mySipCallLeg;
          mySdpData = src.mySdpData;
          myRequestMsg = src.myRequestMsg;
          myContactVector.erase(myContactVector.begin(), myContactVector.end());
          for(ContactVector::const_iterator itr = src.myContactVector.begin(); 
                                itr != src.myContactVector.end(); itr++)
          {
              myContactVector.push_back(*itr);
          }
      }

      /**Insert an element in vector, insert in the begining 
      * if begining == true.
      */
      void addContact(const Sptr<ContactData>& contact, bool begining);

      ///
      void pushContact(const Sptr<ContactData>& contact);

      ///
      Sptr<ContactData> popContact();

      ///
      int getNumContacts() const { return myContactVector.size(); };

      ///
      Sptr<ContactData> getContactData(int index=-1) const;

      ///
      void setRequest(const Sptr<SipMsg>& sipMsg);
      ///
      void setResponse(const Sptr<SipMsg>& sipMsg);

      ///
      void setAck(const Sptr<SipMsg>& sipMsg);

      ///
      void setSdp(const Sptr<SipSdp>& sdpData) { mySdpData = sdpData; } ;
      ///
      void setCallLegState(CallLegState cState) { myCallLegState = cState; };

      ///
      const Sptr<SipSdp> getSdp() const { return mySdpData; };
      ///
      CallLegState getCallLegState() const { return myCallLegState; };

      ///
      const Sptr<SipMsg> getRequest() const { return myRequestMsg; };

      ///
      const Sptr<SipMsg> getResponse() const { return myResponseMsg; };

      ///
      const Sptr<SipMsg> getAck() const { return myAckMsg; };

      ///
      const SipCallLeg& getCallLeg() const { return *mySipCallLeg; };

      ///
      virtual ~SipCallLegData() { };

   protected:
      ///
      Sptr<SipSdp> mySdpData;
      ///
      Sptr<SipMsg> myRequestMsg;
      ///
      Sptr<SipMsg> myResponseMsg;
      ///
      Sptr<SipMsg> myAckMsg;
      ///
      CallLegState myCallLegState;
      /// 
      Sptr<SipCallLeg> mySipCallLeg;
      ///
      ContactVector myContactVector; 
};


#endif
