/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const UasStateTrying_cxx_Version =
    "$Id: UasStateTrying.cxx,v 1.8.2.2 2002/06/11 18:17:28 sprajpat Exp $";

#include "UasStateTrying.hxx"
#include "UaStateFactory.hxx"
#include "SystemInfo.hxx"
#include "B2bConfig.hxx"
#include "B2bUa.hxx"

using namespace Vocal;

void 
UasStateTrying::recvRequest(UaBase& agent, Sptr<SipMsg> msg)
                 throw (CInvalidStateException&)
{
    cpLog(LOG_DEBUG, "UasStateTrying::recvRequest");
    if((msg->getType() == SIP_CANCEL) ||
       (msg->getType() == SIP_BYE))
    {
        //Send 200 
        agent.sendReplyForRequest(msg, 200);
        if(msg->getType() == SIP_CANCEL)
        {
            agent.sendReplyForRequest(agent.getRequest(), 487);
            if(agent.getControllerAgent()) 
                agent.getControllerAgent()->doCancel();
        }
        //Notify CC
        if(agent.getControllerAgent()) 
            agent.getControllerAgent()->receivedRequest(agent, msg);
        changeState(agent, UaStateFactory::instance().getState(U_STATE_FAILURE));
    }
    else if(msg->getType() == SIP_INVITE)
    {
        changeState(agent, UaStateFactory::instance().getState(U_STATE_S_TRYING));
    }
}

void 
UasStateTrying::sendStatus(UaBase& agent, Sptr<SipMsg> msg)
                 throw (CInvalidStateException&)
{
    Sptr<StatusMsg> statusMsg;
    statusMsg.dynamicCast(msg);
    assert(statusMsg != 0);
    int statusCode = statusMsg->getStatusLine().getStatusCode();
    //Only handle status code 18x, 2xx and above
    if(statusCode == 100)
    {
        //Do nothing
        return;
    }


    if((statusCode >= 180) && (statusCode < 200))
    {
        //Send status message
        agent.sendReplyForRequest(agent.getRequest(), statusCode);
        //Transit to ringing
        changeState(agent, UaStateFactory::instance().getState(U_STATE_RINGING));
    }
    else if(statusCode > 200)
    {
        //Send status message
        agent.sendReplyForRequest(agent.getRequest(), statusCode);
        //Transit to Failure
        changeState(agent, UaStateFactory::instance().getState(U_STATE_FAILURE));
    }
    else if(statusCode == 200)
    {
        //Send status message
        ///Manipulate SDP data
        Sptr<SipContentData> sdpData = msg->getContentData(0);

        Sptr<SipCommand> sipCmd;
        sipCmd.dynamicCast(agent.getRequest());
        assert(sipCmd != 0);
        Sptr<StatusMsg> sendSMsg = new StatusMsg(*sipCmd, statusCode);
        sendSMsg->setNumContentData(0);
        if(sdpData != 0)
        {
            sendSMsg->setContentData(sdpData.getPtr());
        }
        //// Find a nice way to do it


        Sptr<BaseUrl> bUrl = sipCmd->getTo().getUrl();
        Sptr<SipUrl> mUrl;
        mUrl.dynamicCast(bUrl);
        assert(mUrl != 0);

        mUrl->setHost(theSystem.gethostAddress());
        int localSipPort = B2bConfig::instance().getLocalSipPort();
        mUrl->setPort(localSipPort);

        SipContact me;
        me.setUrl(mUrl);
        sendSMsg->setNumContact( 0 );
        sendSMsg->setContact( me );

        cpLog(LOG_DEBUG, "(%s) sending status %s", className().c_str(), sendSMsg->encode().logData());
        B2bUa::instance().getSipTransceiver()->sendReply(sendSMsg);
        if(agent.getControllerAgent()) agent.getControllerAgent()->inCall();
        agent.setCallLegState(C_LIVE);
        //Transit to ringing
        changeState(agent, UaStateFactory::instance().getState(U_STATE_INCALL));
    }
}


