#ifndef HEART_LESS_PROXY_HXX
#define HEART_LESS_PROXY_HXX

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const HeartLessProxy_hxx_Version =
    "$Id$";


#include "Fifo.h"
#include "CallContainer.hxx"
#include "SipTransceiver.hxx"

namespace Vocal
{

class Builder;
class WorkerThread;
class SipThread;


/** This class creates a  call container, callprocessing fifo, sipstack, 
 *  worker and sip threads. HeartLessProxy::run method starts the worker 
 *  and sip threads. A builder object and SIP port number is given on 
 *  instantiation.
 */
class HeartLessProxy
{
    public:


        /** Create the proxy given the builder, the sip port (which defaults
         *  to 5060), the application name (which defaults to unknown).
         *  The boolean filteron indicates whether to createg a filtering sip 
         *  transceiver or not (defaults to true). The boolean nat indicates
         *  whether the sip transceiver should be created to handle network
         *  address translation (defaults to false).
	 */
        HeartLessProxy(const Sptr < Builder >   builder,
                        unsigned short          defaultSipPort = 5060,
                        Data                    applName = "unknown", 
                        bool                    filteron = true, 
                        bool                    nat = false,
                        SipAppContext           aContext=APP_CONTEXT_GENERIC);


        /** Virtual destructor
         */
        virtual ~HeartLessProxy();


        /** Runs the underlying sip thread and worker thread.
         */
        virtual void run();


        /** Shutdown the underlying sip thread and worker thread.
         */
        virtual void shutdown();


        /** Joins the underlying sip thread and worker thread.
         */
        virtual void join();


    protected:


        /** Call container associated with the builder.
         */
        Sptr < CallContainer >  myCallContainer;


        /** Builder supplied on construction.
         */
        Sptr < Builder >        myBuilder;


        /** Shared call processing queue between the sip thread and the
         *  worker thread. The sip thread writes to the queue and the
         *  sip thread reads from the queue.
         */
        Sptr < Fifo < Sptr < SipProxyEvent > > > myCallProcessingQueue;


        /** Sip transceiver that receives the incoming sip message, and
         *  that is used to send outgoing sip messages. The sip thread
         *  uses the sip transceiver to read incoming sip messages.
         */
        Sptr < SipTransceiver > mySipStack;


        /** The worker thread reads incoming sip events and processes them,
         *  via the supplied builder.
         */
        Sptr < WorkerThread > myWorkerThread;


        /** Ther sip thread reads the incoming sip messages from the 
         *  sip transceiver and posts them to the worker thread.
         */
        Sptr < SipThread > mySipThread;


    private:


        /** Suppress copying
         */
        HeartLessProxy(const HeartLessProxy &);
        

        /** Suppress copying
         */
        const HeartLessProxy & operator=(const HeartLessProxy &);
};
 
}

#endif // HEART_LESS_PROXY_HXX
