#ifndef INVITEMSG_H
#define INVITEMSG_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "SipCommand.hxx"
#include "StatusMsg.hxx"
#include "symbols.hxx"
#include "BaseUrl.hxx"
#include "SipSessionExpires.hxx"
#include "SipMinSE.hxx"

static const char* const InviteMsg_hxx_Version =
    "$Id: InviteMsg.hxx,v 1.37 2002/03/19 21:39:54 jason Exp $";


namespace Vocal
{

/// class for INVITE messages
class InviteMsg : public SipCommand
{
    public:
        ///
        enum InviteMsgForward { inviteMsgForward };
        ///
        InviteMsg();

        /// Create a new invite based on a diversion of a prev invite
        InviteMsg( const InviteMsg& msg, Sptr <BaseUrl> uri );
        /// The listenPort is required to build the INVITE msg, in the Via field.
        InviteMsg(Sptr <BaseUrl> url, int listenPort = 5060 , int rtpPort = 3456);  
        /// for forwarding
        InviteMsg(const InviteMsg& msg, enum InviteMsgForward);
        /// for re-invite based on 200 OK
        InviteMsg(const StatusMsg& statusMsg);
        /** construct a new InviteMsg based on the text of an INVITE msg. 
            this should not be called by anyone but the SipMsg class
         *  @param data  message to be parsed
         *  @exception   thrown if message fails basic parsing
         */
        InviteMsg( const Data& data ) throw (SipParserException&);
        ///
        InviteMsg& operator =(const InviteMsg&);
        ///
        bool operator ==(const InviteMsg& src) const;
        ///
        InviteMsg(const InviteMsg&);
        ///
        virtual ~InviteMsg(){}
        ///
        virtual Method getType() const;

        //--- SessionTimer headers //
        ///
        const SipSessionExpires& getSessionExpires() const;        
        ///
        const SipMinSE& getMinSE() const ;
        ///
        void setSessionExpires(const SipSessionExpires& sess);
        ///
        void setMinSE(const SipMinSE& minSe);

    private:
        void setInviteDetails(Sptr <BaseUrl> url, int listenPort, int rtpPort);
};
 
}// namespace Vocal

/* Local Variables: */
/* c-file-style:"stroustrup" */
/* c-basic-offset:4 */
/* c-file-offsets:((inclass . ++)) */
/* indent-tabs-mode:nil */
/* End: */

#endif
