/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const SipWwwAuthenticate_cxx_version =
    "$Id: SipWwwAuthenticate.cxx,v 1.45 2002/03/19 21:39:56 jason Exp $";

#include "global.h"
#include <iostream>

#include "SipWwwAuthenticate.hxx"
#include "symbols.hxx"
#include "SipParserMode.hxx"
#include "cpLog.h"

using namespace Vocal;


string
SipWwwAuthenticateParserException::getName( void ) const
{
    return "SipWwwAuthenticateParserException";
}


SipWwwAuthenticate::SipWwwAuthenticate()
{
}


SipWwwAuthenticate& SipWwwAuthenticate::operator=(const SipWwwAuthenticate& src)
{
    if (this != &src)
    {
        authScheme = src.authScheme;
        tokenMap = src.tokenMap;   //contains tokens, and tokenValues.

        realmValue = src.realmValue;
    }
    return (*this);
}


bool SipWwwAuthenticate::operator ==(const SipWwwAuthenticate& src) const
{
  return
    (
      ( authScheme == src.authScheme ) &&
      ( tokenMap == src.tokenMap ) &&
      ( realmValue == src.realmValue )
      );
}
SipWwwAuthenticate::SipWwwAuthenticate( const Data& srcData )
        : authScheme(),
        tokenMap(),
        tokenMutex()
{
    Data fdata = srcData;
    try
    {
        decode(fdata);
    }
    catch (SipWwwAuthenticateParserException&)
    {
        if (SipParserMode::sipParserMode())
        {

            throw SipWwwAuthenticateParserException(
                "failed to decode the WwwAuthenticate string",
                __FILE__,
                __LINE__,
                DECODE_FAILED_WWWAUTHENTICATE);
        }
    }
}


void SipWwwAuthenticate::decode(const Data& data)
{

    Data nData = data;

    try
    {

        scanSipWwwauthorization(nData);
    }
    catch (SipWwwAuthenticateParserException& exception)
    {
        if (SipParserMode::sipParserMode())
        {

            throw SipWwwAuthenticateParserException(
                exception.getDescription(),
                __FILE__,
                __LINE__,
                DECODE_FAILED_WWWAUTHENTICATE
            );
        }
    }
}

void
SipWwwAuthenticate::scanSipWwwauthorization(const Data& tmpdata)
{
    Data authdetails = tmpdata ;
    Data authType;
    int ret = authdetails.match(" ", &authType, true);
    if (ret == FIRST)
    {
        if (SipParserMode::sipParserMode())
        {

            throw SipWwwAuthenticateParserException(
                "failed to decode the WwwAuthenticate string",
                __FILE__,
                __LINE__, DECODE_FAILED_WWWAUTHENTICATE);
        }

    }
    if (ret == FOUND)
    {
        setAuthScheme(authType);

        if ( ( authType == AUTH_BASIC ) ||
                ( authType == AUTH_DIGEST ) ||
                ( authType == AUTH_PGP )
           )
        {

            //scanRealmValue.

            Data realm;

            int ret = authdetails.match("=", &realm, false);
            if (ret == FOUND)
            {
                if (realm == REALM)
                {
                    ret = authdetails.match("=", &realm, true);
                    ret = authdetails.match(",", &realmValue, true);
                    string tokenstr = realmValue.convertString();
                    int pos;
                    pos = tokenstr.find("\"");
                    if (pos != (int)(string::npos))
                    {
                        tokenstr = tokenstr.substr(pos + 1, tokenstr.length() - 2);
                    }
                    realmValue = tokenstr;
                }
            }
            else
            {
                if (SipParserMode::sipParserMode())
                {

                    throw SipWwwAuthenticateParserException(
                        "realm not present.",
                        __FILE__,
                        __LINE__,
                        DECODE_FAILED_WWWAUTHENTICATE);
                }
            }

        }
    
        //scanAuthTokens.
        try
        {
            scanAuthTokens(authdetails);
            }
        catch (SipWwwAuthenticateParserException&)
        {
            if (SipParserMode::sipParserMode())
            {
                throw (SipWwwAuthenticateParserException(
                    "failed in parsing auth tokens",
                    __FILE__,
                    __LINE__,
                    DECODE_FAILED_WWWAUTHENTICATE));
            }
        }
        
    }
    if (ret == NOT_FOUND)
    {
        throw SipWwwAuthenticateParserException(
            "No AuthScheme",
            __FILE__,
            __LINE__,
            DECODE_FAILED_WWWAUTHENTICATE);
    }

}

/*
      challenge        =  "Digest" digest-challenge

      digest-challenge  = 1#( realm | [ domain ] | nonce |
                          [ opaque ] |[ stale ] | [ algorithm ] |
                          [ qop-options ] | [auth-param] )


      domain            = "domain" "=" <"> URI ( 1*SP URI ) <">
      URI               = absoluteURI | abs_path
      nonce             = "nonce" "=" nonce-value
      nonce-value       = quoted-string
      opaque            = "opaque" "=" quoted-string
      stale             = "stale" "=" ( "true" | "false" )
      algorithm         = "algorithm" "=" ( "MD5" | "MD5-sess" |
                           token )
      qop-options       = "qop" "=" <"> 1#qop-value <">
      qop-value         = "auth" | "auth-int" | token
*/



void
SipWwwAuthenticate::scanAuthTokens(const Data& data)
{
    Data authTokens = data;
    Data nextToken;

    bool firstToken = true;
    while (authTokens.length())
    {

        Data token, tokenValue;

        token = nextToken;
	token.removeSpaces();

        Data value;

        int ret = authTokens.match("=", &value, true);

        if  (ret == NOT_FOUND)
        {
            if (firstToken)
            {

                if (SipParserMode::sipParserMode())
                {
                    throw SipWwwAuthenticateParserException(
                        "no tokens or challenge",
                        __FILE__,
                        __LINE__,
                        DECODE_FAILED_WWWAUTHENTICATE);
                }
                authTokens = "";
            }
            else
            {
                value = authTokens;
                authTokens = "";
                //insert this into the tokenMap.
                nextToken.removeSpaces();
                string tokenstr = value.convertString();
                int pos;
                pos = tokenstr.find("\"");
                    
                if (pos != (int)(string::npos))
                {
                    tokenstr = tokenstr.substr(pos + 1, tokenstr.length() - 2);
                }
                tokenMutex.lock();
                tokenMap[nextToken] = tokenstr;
                tokenMutex.unlock();
            }
        }
        else if (ret == FIRST)
        {
            if (SipParserMode::sipParserMode())
            {
                throw SipWwwAuthenticateParserException(
                    "no tokens or challenge",
                    __FILE__,
                    __LINE__,
                    DECODE_FAILED_WWWAUTHENTICATE);
            }
        }
        else if (ret == FOUND)
        {
            firstToken = false;
            
            //check if this has any commas.
            bool atleastoneSubParam = false;

            bool secondSubParam = false;
            while (value.length())
            {
                Data subParms;
                ret = value.match(",", &subParms, true);


                if (ret == FOUND)
                {
                    if (secondSubParam)
                    {
                        tokenValue+= ",";
                    }
                    tokenValue+= subParms;
                    secondSubParam = true;
                       
                    atleastoneSubParam = true;
                }
                else if (ret == NOT_FOUND)
                {
                    if (atleastoneSubParam)
                    {
                        //add it to map
                        //strip "" off this.
                        string tokenstr = tokenValue.convertString();
                        int pos;
                        pos = tokenstr.find("\"");
                        
                        if (pos != (int)(string::npos))
                        {
                            tokenstr = tokenstr.substr(pos + 1, tokenstr.length() - 2);
                        }
                        token.removeSpaces();
                        tokenMutex.lock();
                        tokenMap[token] = tokenstr;
                        tokenMutex.unlock();
                    }

                    nextToken = value;
                    break;
                }
            }//end while value.length
        }// if = is found.
        
    }//end while
}

   
///

SipWwwAuthenticate::SipWwwAuthenticate( const SipWwwAuthenticate& src )
        : authScheme(src.authScheme),
        tokenMap(src.tokenMap),
        realmValue(src.realmValue)
{}


/*** return the encoded string version of this. This call should only be
     used inside the stack and is not part of the API */
Data SipWwwAuthenticate::encode() const
{
    Data data;



    if ( authScheme.length() )
    {
        data += SIP_WWWAUTHENTICATE;
        data += SP;
        data += authScheme;
        data += SP;
        if (realmValue.length())
        {
            data += "realm";
            data += "=";
            data += "\"";
            data += realmValue;
            data += "\"";
            if(tokenMap.size())
                data+= ",";
        }
        if (authScheme != AUTH_BASIC)
        {
            tokenMutex.lock();
            TokenMapWwwAuth::const_iterator i = tokenMap.begin();
            while (i != tokenMap.end())
            {
                
                Data token = i->first;
                Data tokenValue = i->second;
    
        
		data += token;
                data += "=";

		// these three tokens should not have dbl quotes
		// around the values
                if ( isEqualNoCase( token, "algorithm") ||
		     isEqualNoCase( token, "stale") ||
		     isEqualNoCase( token, "id") )
		{
		    data += tokenValue;
		}
		else
		{

		    data += "\"";
		    data += tokenValue;
		    data += "\"";
		}

                ++i;
                if ( i != tokenMap.end())
                {
                    data += ",";
                }
            }  //end while
            tokenMutex.unlock();
            //data+= CRLF;
        }//other than basic.
        data += CRLF;
    }

    return data;
}


void SipWwwAuthenticate::setAuthTokenDetails(const Data& token,
        const Data& tokenValue)
{

    Data changeToken = token;
    
    tokenMutex.lock();
    changeToken.removeSpaces();
    tokenMap[token] = tokenValue;
    tokenMutex.unlock();
}

Sptr < SipWwwAuthenticate::TokenMapWwwAuth >
SipWwwAuthenticate::getTokenDetails()
{
    tokenMutex.lock();
    Sptr < SipWwwAuthenticate::TokenMapWwwAuth > dupMap = new TokenMapWwwAuth(tokenMap);
    tokenMutex.unlock();
    return dupMap;
}


void SipWwwAuthenticate::setRealmValue(const Data& data)
{
    realmValue = data;
}


Data SipWwwAuthenticate::getRealmValue()
{
    return realmValue;
}


Data SipWwwAuthenticate::getTokenValue(const Data& token) const
{
   
    
    
    Data tokenValue;

    string tokenstr;
    string newtokenstr;
    
    tokenMutex.lock();
    TokenMapWwwAuth::const_iterator check = tokenMap.find(token);

    if (check != tokenMap.end())
    {
        tokenValue = check->second;

	//strip the quotes off.
        LocalScopeAllocator lo;
	tokenstr = tokenValue.getData(lo);
        newtokenstr = tokenstr;
	unsigned int pos;
	pos = tokenstr.find("\"");
	
        
	if (pos != string::npos)
	{
	    newtokenstr = tokenstr.substr(pos + 1, tokenstr.length() - 2);
	}
    }
    tokenMutex.unlock();
    
    
    return Data(newtokenstr);
}


SipHeader*
SipWwwAuthenticate::duplicate() const
{
    return new SipWwwAuthenticate(*this);
}


bool
SipWwwAuthenticate::compareSipHeader(SipHeader* msg) const
{
    SipWwwAuthenticate* otherMsg = dynamic_cast<SipWwwAuthenticate*>(msg);
    if(otherMsg != 0)
    {
	return (*this == *otherMsg);
    }
    else
    {
	return false;
    }
}


/* Local Variables: */
/* c-file-style: "stroustrup" */
/* indent-tabs-mode: nil */
/* c-file-offsets: ((access-label . -) (inclass . ++)) */
/* c-basic-offset: 4 */
/* End: */
